define(['GenericLightBox', 'SceneObjectQuizQuestion'], function (GenericLightBox) {
    return function (rawQuizQuestionModel, pageModel) {
        var that = this;

        this.defineClasses = function () {

            that.QuizQuestionView = ElementBaseView.extend({
                tagName: "div",
                className: "sceneObject",
                id: rawQuizQuestionModel.properties.id,
                autoMediaTemplate: templates.getTemplate("image"),
                parentElementModel: pageModel,
                isIcon: rawQuizQuestionModel.customProperties["questionModel"]["displayType"] == DisplayType.ICON,

                customInitialize: function () {
                    this.model.set("allObjectTypesInPage", rawQuizQuestionModel.customProperties["allObjectTypesInPage"]);
                    this.listenTo(this.model, "change:contentHeight", this.updateQuestionHeight);
                    this.listenTo(this.model, "change:computeScore", this.computeScoreForQuestion);
                    this.listenTo(this.model, "change:resetQuestion", this.resetQuestion);
                },

                events: function () {
                    var _events = {};
                    if (this.isIcon) {
                        _events[AppUtils.EventTypes.mouseDown] = "touchstart";
                        _events[AppUtils.EventTypes.click] = "clickHandler";
                        _events[AppUtils.EventTypes.mouseUp] = "mouseUpHandler";
                    }
                    return _events;
                },
                removeView: function () {
                    if (this.model.get("removed")) {
                        this.remove();
                        this.unbind();
                    }
                },
                activateObject: function () {
                    if (this.model.get("activateObject")) {
                        this.model.set("objectActivated", true);
                        this.model.set("activateObject", false);
                    }
                },
                objectActivated: function () {
                    if (this.model.get("objectActivated")) {
                        this.model.set("objectActivated", false);

                        if (this.isIcon) {
                            var lightBoxWindowSettings = {
//                                width: rawQuizQuestionModel.properties.originalWidth,
//                                height: rawQuizQuestionModel.properties.originalHeight
                                width: "90%",
                                height: "90%",
                                horizontalMargin: '5%',
                                verticalMargin: '5%',
                                color: '#FFFFFF'
                            };

                            this.lightBox = new GenericLightBox(null, lightBoxWindowSettings, {color: '#FFFFFF'});
                            this.lightBox.init();
                            App.viewPortView.$el.append(this.lightBox.getView().render().el);
                            this.printQuestionObject(this.lightBox.getContentView());
                        }
                    }
                },

                resetQuestion: function () {
                    if (this.model.get('resetQuestion')) {
                        this.quizQuestionObject.resetQuestion(false);
                        this.model.set('resetQuestion', false);
                    }

                },
                updateQuestionHeight: function () {
                    console.log(this.model.get('index'), "  ", this.model.get('contentHeight'));
                    pageModel.set('contentHeight', this.model.get('contentHeight'))
                },
                printQuestionObject: function (targetDOMElView) {
                    var questionJSON = rawQuizQuestionModel.customProperties["questionModel"];
                    questionJSON["useNumbering"] = false;
                    this.quizQuestionObject = new SceneObjectQuizQuestion(questionJSON, targetDOMElView);
                },
                computeScoreForQuestion: function () {
                    if (this.model.get('computeScore')) {
                        this.quizQuestionObject.computeQuestionScoreAndOptionLevelFeedback(true);
                        this.model.set('computeScore', false);
                    }
                },
                render: function () {
                    var thisView = this;
                    if (this.shouldRenderObject()) {
                        this.rotateObject();
                        if (this.isIcon) {
                            var quizIconInfo = Chapter.getSettings()["quizIconInfo"];
                            this.model.set("imageURL", quizIconInfo['quizIconURL']);
                            this.$el.html(this.autoMediaTemplate(this.model.attributes));
                            this.model.set("width", quizIconInfo['quizIconWidth']);
                            this.model.set("height", quizIconInfo['quizIconHeight']);
                        } else {
                            this.pushUnloadHandler(function () {
                                thisView.quizQuestionObject.resetQuestion(false);
                            });
//                            this.setBackgroundColor("#F4F4F4");
//                            this.setBackgroundColor(this.model.get('bgColor'), this.model.get('bgOpacity'));
                            this.startZoomEffectsIfNeeded();
                            _(function () {
                                thisView.printQuestionObject(thisView);
                            }).defer();
                        }
                        this.bypassChromeBugs(rawQuizQuestionModel.customProperties["allObjectTypesInPage"]);
                        this.setLeft(this.model.get("left"));
                        this.setWidth(this.model.get("width"));
                        this.setHeight(this.model.get("height"));
                        this.setTop(this.model.get("top"));
                        this.setZIndex(this.model.get("ZIndex"));
                        this.setInitialVisibility();
                        return this;
                    }
                    return null;
                }
            });
        };

        this.instantiateObjects = function () {
            this.quizQuestionModel = this.getQuizQuestionModel();
            this.quizQuestionView = new this.QuizQuestionView({
                model: that.quizQuestionModel
            });
        };

        this.getQuizQuestionModel = function () {
            return new Backbone.Model(rawQuizQuestionModel.properties);
        };

        return {
            init: function () {
                that.defineClasses();
                that.instantiateObjects();
            },
            getView: function () {
                return that.quizQuestionView;
            },
            getModel: function () {
                return that.quizQuestionModel;
            },
            getQuestionObject: function () {
                return that.quizQuestionView.quizQuestionObject;
            }

        }
    }
});
