if (!window["App"]) window["App"] = {};
(function (App) {
    App.mql = null;
    App.model = {};
    App.menuModel = {};
    App.lastKnownOrientation = SupportedOrientation.LANDSCAPE;
    App.changedOrientation = false;
    App.dualLandscape = null;
    App.minimumSidePaddingForDesktopBrowsers = 200;
    /**
     * The period used for a timer in which the orientation handler became inactive
     * It is a hack that had to be done since orientation handler came multiple (3) times when orientation is changed.
     * @type {number}
     */
    App.ORIENTATION_TIMER_PERIOD = 300;

    App.shouldUpdateOrientation = true;

    App.defineClasses = function () {
        App.ViewportView = BaseView.extend({
                tagName: "div",
                className: function () {
                    var className = "";
//                    if (!ProjectGlobals.IS_TIP_OF_THE_DAY) {
//                        className = 'viewPortBackgroundColorClass';
//                    }
                    return className;
                },
                id: "viewPort",
                position: null,

                initialize: function () {
                    this.listenTo(this.model, "change:bindEvents", this.bindEvents);
                    this.listenTo(this.model, "change:orientation", this.updateOrientation);
                },

                bindEvents: function () {
                    this.delegateEvents(this.getEventsObj());
                },

                getEventsObj: function () {
                    var _events = {};
                    _events[AppUtils.EventTypes.mouseDown] = "mouseDownHandler";
                    _events[AppUtils.EventTypes.mouseUp] = "mouseUpHandler";
                    _events["touchcancel"] = "touchCancelHandler";
                    return _events;
                },

                mouseDownHandler: function (e) {
                    this.position = this.getPosition(e);
                },
                mouseUpHandler: function (e) {
                    if (this.tapEvent(e)) {
                        Chapter.getBottomMenu().getModel().set('show', !Chapter.getBottomMenu().getModel().get('show'));
                    }
                },

                adjustBodyBackground: function () {
                    var backgroundCSS = this.model.get("backgroundCSS"),
                        bodyEl = $("body");
                    if (!$.isEmptyObject(backgroundCSS)) {
                        if (backgroundCSS["type"] == "image") {
                            var bgImageURL = ResourceManager.getPathForImage(backgroundCSS["objectId"], backgroundCSS["extension"]);
                            bodyEl.css("background-image", "url(" + bgImageURL + ")")
                                .css("background-size", "cover");
                        } else if (backgroundCSS["type"] == "color") {
                            bodyEl.css("background", backgroundCSS["colorCode"]);
                        }
                    } else if (!ProjectGlobals.IS_TIP_OF_THE_DAY) {
                        bodyEl.css('background-color', '#404041');
                    }
                },

                getWidth: function () {
                    var width = this.model.get("width");
                    if (AppUtils.nativeApp && AppUtils.DeviceType.CHROME && ViewportPadding.isPaddingAreaPossible()) {
                        width -= 1; // due to a bug that's only visible on Android (both XWalk and standard web view), see MOB-2000
                    }
                    return width;
                },

                render: function () {
                    var leftValue, topValue,
                        viewportWidth = this.getWidth(),
                        viewportHeight = this.model.get("height");

                    this.adjustBodyBackground();
                    this.setWidth(viewportWidth);
                    this.setHeight(viewportHeight);
                    leftValue = Math.floor(App.settings.windowWidth / 2 - viewportWidth / 2);
                    topValue = Math.floor(App.settings.windowHeight / 2 - viewportHeight / 2);

                    this.setLeft(leftValue);
                    this.setTop(topValue);

                    setTimeout(function () {
                        // load some custom css files
                        var cssCustomStylesPath = Chapter.getSettings()["cssCustomStylesPath"];
                        if (cssCustomStylesPath != null) {
                            AppUtils.loadCssFiles([cssCustomStylesPath + "styles/custom_styles.css"]);
                        }
                    }, 0);

                    return this;
                },

                updateOrientation: function () {
                    var leftValue, topValue;

                    this.setWidth(App.settings.width);
                    this.setHeight(App.settings.height);

                    leftValue = Math.floor(App.settings.windowWidth / 2 - App.settings.width / 2);
                    topValue = Math.floor(App.settings.windowHeight / 2 - App.settings.height / 2);

                    this.setLeft(leftValue);
                    this.setTop(topValue);

//                var newOrientation = this.model.get("orientation");
//                if (newOrientation == SupportedOrientation.PORTRAIT) {
//                } else if (newOrientation == SupportedOrientation.LANDSCAPE) {
//                }
                }
            }
        )
        ;
    };

    App.instantiateObjects = function (viewPortModel) {

        App.viewPortModel = new Backbone.Model(viewPortModel);

        App.viewPortView = new App.ViewportView({
            model: App.viewPortModel
        });
    };

    function getScaleFactor(viewPortModelWidth, viewPortModelHeight) {
        var scaleWidth, scaleHeight;
        if (App.dualLandscapeActive()) {
            scaleWidth = App.settings.windowWidthWithoutSidePadding / (2 * viewPortModelWidth);
            scaleHeight = App.settings.windowHeight / viewPortModelHeight;
        } else {
            scaleWidth = App.settings.windowWidthWithoutSidePadding / viewPortModelWidth;
            scaleHeight = App.settings.windowHeight / viewPortModelHeight;
        }
        return parseFloat(Math.min(scaleWidth, scaleHeight).toFixed(5));
    }

    function getMenuScaleFactor(chapterOriginalWidth, chapterOriginalHeight) {
        var ret;
        if (App.dualLandscapeActive()) {
            chapterOriginalWidth *= 2;
        }
        if (chapterOriginalWidth > chapterOriginalHeight) {
            ret = parseFloat(Math.min(App.settings.windowWidthWithoutSidePadding / 1024, App.settings.windowHeight / 768).toFixed(5))
        } else {
            ret = parseFloat(Math.min(App.settings.windowWidthWithoutSidePadding / 768, App.settings.windowHeight / 1024).toFixed(5))
        }
        return ret;
    }

    App.updateSettings = function (isDualLandscape) {
        var widthScaleFactor;
        App.settings.windowWidth = window.innerWidth;
        App.settings.windowHeight = window.innerHeight;
        App.settings.windowWidthWithoutSidePadding = (AppUtils.DeviceType.MOBILE_DEVICE || ProjectGlobals.IS_TIP_OF_THE_DAY || ProjectGlobals.IS_SCENE_PREVIEW) ? App.settings.windowWidth : App.settings.windowWidth - App.minimumSidePaddingForDesktopBrowsers;
        App.setDualLandscapeActive(isDualLandscape);
        App.settings.scaleFactor = getScaleFactor(App.settings.originalWidth, App.settings.originalHeight);
        App.settings.menuScaleFactor = getMenuScaleFactor(App.settings.originalWidth, App.settings.originalHeight);
        widthScaleFactor = App.dualLandscapeActive() ? App.settings.scaleFactor * 2 : App.settings.scaleFactor;
        App.settings.width = Math.round(App.settings.originalWidth * widthScaleFactor);
        // TODO: this might be needed for dual pages but it has a bad impact on others
//        if (App.settings.width % 2 == 1) {
//            App.settings.width++; // make sure we don't have floating point width for page
//        }
        App.settings.height = Math.round(App.settings.originalHeight * App.settings.scaleFactor);
        App.settings.pageWidth = App.dualLandscapeActive() ? App.settings.width / 2 : App.settings.width;
        if(Chapter.getSettings().reserveMenuArea){
            App.settings.pageHeight = App.settings.height - 220;
            if(Chapter.getSettings().adjustWidthWhenMenuAreaIsReserved)
                App.settings.pageWidth = (App.settings.originalWidth/App.settings.originalHeight) * App.settings.pageHeight ;
        }
        else
            App.settings.pageHeight = App.settings.height;
        App.settings.left = Math.round((App.settings.windowWidth - App.settings.pageWidth) / 2);
        App.settings.top = Math.round((App.settings.windowHeight - App.settings.height) / 2);
    };

    App.init = function (viewPortModel) {

        App.settings = viewPortModel;

        App.updateSettings(window.innerWidth > window.innerHeight);

        App.addOrientationHandlers(viewPortModel.savedOrientation);

        App.defineClasses();
        App.instantiateObjects(App.settings);
    };

    App.addOrientationHandlers = function (savedOrientation) {
        App.lastKnownOrientation = savedOrientation; // default is landscape
        if (App.mql == null && !AppUtils.DeviceType.IE9) {
            App.mql = window.matchMedia("(orientation:portrait)");
        }
        if (!window.onResizeHandlerAdded) {
            window.onResizeHandlerAdded = true;
            window.onresize = function () {
                var newOrientation;
                if (AppUtils.DeviceType.iOS5) {
                    newOrientation = App.getNewOrientationiOS5();
                } else {
                    newOrientation = App.getNewOrientation(App.mql);
                }
                App.handleOrientationChange(newOrientation);
            }
        }
    };

    App.getNewOrientation = function (mql) {
        var orientation;
        if (mql.matches) {
            orientation = SupportedOrientation.PORTRAIT;
        } else {
            orientation = SupportedOrientation.LANDSCAPE;
        }
        return orientation;
    };

    App.getNewOrientationiOS5 = function () {
        var orientation;
        if (window.innerWidth < window.innerHeight) {
            orientation = SupportedOrientation.PORTRAIT;
        } else {
            orientation = SupportedOrientation.LANDSCAPE;
        }
        return orientation;
    };

    App.handleOrientationChange = function (orientation) {
        if (App.shouldUpdateOrientation) {
            if (orientation != App.lastKnownOrientation) {
                App.shouldUpdateOrientation = false;
                console.log("The device orientation changed to " + orientation);
                App.lastKnownOrientation = orientation;
                App.updateForNewOrientation(App.lastKnownOrientation);
                Chapter.getBottomMenu().getModel().set('orientation', App.lastKnownOrientation); // only for KPT Sabeltann titles
                App.enableChangeOrientationHandlerWithDelay();
            } else {
                console.log("Orientation remains the same " + orientation);
            }
        } else {
            App.enableChangeOrientationHandlerWithDelay();
        }
    };

    App.enableChangeOrientationHandlerWithDelay = function () {
        setTimeout(function () {
            App.shouldUpdateOrientation = true;
        }, App.ORIENTATION_TIMER_PERIOD);
    };

    App.updateForNewOrientation = function (orientation) {
        if (ProjectGlobals.ORIENTATION != SupportedOrientation.ALL) {
            // we should load related chapter
            App.loadChapterForOrientation(orientation);
        } else if (Chapter.getSettings()["useTwoInOneLandscapeMode"]) {
            ViewportPadding.handleOrientationChanged();
        }
    };

    App.loadChapterForOrientation = function (orientation) {
        var activePage = Chapter.getActivePage();
        ViewportPadding.removeAllPaddings();
        // now loading project for the new orientation
        if (typeof SoundPlayer != 'undefined') {
//            SoundPlayer.endAllPlayingSounds();
        }
        // set a flag here to know that we are loading a related chapter
        localStorage.setItem("relatedChapterWillLoad", true);
        require(['ChapterElement'], function (ChapterElement) {
            App.changedOrientation = true;
            if (null != Chapter.getScrollingElementsModel()) {
                Chapter.getScrollingElementsModel().set('remove', true);
            }

            var bottomMenuView = Chapter.getBottomMenu().getView();
            bottomMenuView.remove();
            bottomMenuView.unbind();

            App.viewPortView.remove();
            App.viewPortView.unbind();

            setupAppBasicFunctions(ChapterElement);
            App.changedOrientation = false;
            Chapter.loadRelatedProjectForOrientation(orientation, activePage);
        });
    };

    App.saveCurrentModel = function (chapterRawModel) {
        // general condition for such a scenario
        if (ProjectGlobals.ORIENTATION != SupportedOrientation.ALL) {
            if (ProjectSettings.shareUrl == ProjectGlobals.SHARE_URL) {
                App.model[ProjectGlobals.ORIENTATION] = JSON.stringify(chapterRawModel);
            } else {
                App.model[ProjectGlobals.RELATED_ORIENTATION] = JSON.stringify(chapterRawModel);
            }
        }
    };

    App.saveCurrentMenuModel = function (menuModel) {
        // general condition for such a scenario
        if (ProjectGlobals.ORIENTATION != SupportedOrientation.ALL) {
            if (ProjectSettings.shareUrl == ProjectGlobals.SHARE_URL) {
                App.menuModel[ProjectGlobals.ORIENTATION] = JSON.stringify(menuModel);
            } else {
                App.menuModel[ProjectGlobals.RELATED_ORIENTATION] = JSON.stringify(menuModel);
            }
        }
    };

    App.setDualLandscapeActive = function (value) {
        this.dualLandscape = value && Chapter.getSettings()["useTwoInOneLandscapeMode"];
    };

    App.dualLandscapeActive = function () {
        return this.dualLandscape;
    };

    App.maxActivePageNumberForDualLandscape = function () {
        var lastPageNumber = Chapter.getSettings().lastPageNumber,
            ret = lastPageNumber - App.segmentLength();
        if (App.dualLandscapeActive() && lastPageNumber % 2 == 1) {
            ret += 1;
        }
        return ret;
    };

    App.maxTranslateValueForLastPage = function () {
        var lastPageNumber = Chapter.getSettings().lastPageNumber,
            ret = -(lastPageNumber - App.segmentLength()) * App.settings.pageWidth;
        if (lastPageNumber % 2 == 1 && App.dualLandscapeActive()) { // chapter with odd number of pages in dual landscape active
            ret -= App.settings.pageWidth;
        }
        return ret;
    };

    App.segmentLength = function () {
        return App.dualLandscapeActive() ? 2 : 1;
    };

    App.getBufferPages = function () {
        return App.dualLandscapeActive() ? [0, 1, -1, -2, 2, 3] : [0, -1, 1];
    };

    App.setAutoMediaIconsURL = function () {

        function getAutoMediaIconURL(autoMediaIconInfo) {
            if ("defaultIconURL" in autoMediaIconInfo) {
                return autoMediaIconInfo["defaultIconURL"];
            } else {
                return ResourceManager.getPathForImage(autoMediaIconInfo["objectId"], autoMediaIconInfo["extension"]);
            }
        }

        App.soundIconURL = getAutoMediaIconURL(Chapter.getSettings()["soundIconInfo"]);
        App.videoIconURL = getAutoMediaIconURL(Chapter.getSettings()["videoIconInfo"]);
        App.quizIconURL = getAutoMediaIconURL(Chapter.getSettings()["quizIconInfo"]);
        App.cameraIconURL = getAutoMediaIconURL(Chapter.getSettings()["cameraIconInfo"]);
        App.drawingIconURL = getAutoMediaIconURL(Chapter.getSettings()["drawingIconInfo"]);
        App.downloadPDFIconURL = getAutoMediaIconURL(Chapter.getSettings()["downloadPDFIconInfo"]);
    };

})(window["App"]);
