var AppUtils = {

    EventTypes: {
        click: 'click',
        mouseDown: 'mousedown',
        mouseDownSafeIOS5: 'mousedown',
        mouseMove: 'mousemove',
        mouseUp: 'mouseup',
        scroll: 'scroll',
        mouseWheel: 'mousewheel',
        mouseOut: 'touchCancel',
        mouseEnter: 'mouseover',
        mouseLeave: 'mouseout',
        DOMAttrModified: 'DOMAttrModified'
    },

    DeviceType: {
        ANDROID: false,
        iOS: false,
        iOS5: false,
        W8: false,
        OTHER: false,
        MOBILE_DEVICE: false,
        CHROME: false,
        Firefox: false,
        XWalk: false,
        IE9: false
    },

    nativeApp: false,
    offlineExport: false,
    deviceReady: false,
    isSCORM: false,

    storageFolder: "",

    IOS_STORAGE_FOLDER: "../../../Library/Caches/",
    W8_STORAGE_FOLDER: "ms-appdata:///local/",

    loadedProjectStyle: null,
    navigationDivEl: null,
    playerWasLoaded: false,

    initDeviceType: function () {
        if (navigator.userAgent.match(/(iPad|iPhone|iPod)/g)) {
            AppUtils.DeviceType.iOS = true;
            var userAgentInfo = navigator.userAgent.match(/CPU OS .* like Mac OS X/g);
            if (userAgentInfo != null) {
                var iOSVersionString = userAgentInfo[0].split(" ")[2];
                if (iOSVersionString < "6") { // lower than iOS 6 (iPad 1)
                    AppUtils.DeviceType.iOS5 = true;
                }
            }
            AppUtils.DeviceType.MOBILE_DEVICE = true;
        } else if (navigator.userAgent.indexOf('Android') != -1) {
            AppUtils.DeviceType.ANDROID = true;
            AppUtils.DeviceType.MOBILE_DEVICE = true;
        } else if (navigator.userAgent.indexOf('MSIE 10.0') != -1) {
            // TODO: see how we can differentiate desktop IE 10 from tablet version
            AppUtils.DeviceType.W8 = true;
//            AppUtils.DeviceType.MOBILE_DEVICE = true;
        } else if (navigator.userAgent.indexOf('MSIE 9.0') != -1) {
            AppUtils.DeviceType.IE9 = true;
        } else {
            AppUtils.DeviceType.OTHER = true;
        }
        if (navigator.userAgent.indexOf('Chrome') != -1) {
            AppUtils.DeviceType.CHROME = true;
        } else if (navigator.userAgent.indexOf('Firefox') != -1) {
            AppUtils.DeviceType.Firefox = true;
        }
        if (navigator.userAgent.indexOf("Crosswalk") != -1) {
            AppUtils.DeviceType.XWalk = true;
        }
    },

    setStorageFolder: function () {
        if (AppUtils.nativeApp) {
            if (AppUtils.DeviceType.iOS) {
                AppUtils.storageFolder = AppUtils.IOS_STORAGE_FOLDER;
            } else if (AppUtils.DeviceType.W8) {
                AppUtils.storageFolder = AppUtils.W8_STORAGE_FOLDER;
            } else if (AppUtils.DeviceType.ANDROID) {
                try {
                    // all native Android projects should implement methods in DirectJSCalls
                    AppUtils.storageFolder = "file:///mnt/sdcard/" + DirectJSCalls.getAppPublicStorageDir();
                } catch (e) {
                    console.error("Android failed to set storageFolder");
                }
            }
        }
    },

    loadCordovaOnDemand: function () {
        if (AppUtils.nativeApp) {
            var loadScripts = [];
            if (window.hasOwnProperty("cordovaVersion")) {
                console.log("Version of corodva is " + window["cordovaVersion"]);
                if (window["cordovaVersion"].indexOf("2") == 0) { // versions 2.X of Cordova
                    if (AppUtils.DeviceType.iOS) {
                        loadScripts.push("Cordova2XiOS");
                    } else if (AppUtils.DeviceType.ANDROID) {
                        loadScripts.push("Cordova2XAndroid"); // not tested !!!
                    }
                }
            }
            if (loadScripts.length == 0) {
                loadScripts.push("fakeCordova");
                if (AppUtils.DeviceType.iOS) {
                    loadScripts.push("CordovaiOS");
                } else if (AppUtils.DeviceType.ANDROID) {
                    loadScripts.push("CordovaAndroid");
                }
            }
            loadScripts.push('CordovaWrapper');
            document.addEventListener("deviceready", AppUtils.onDeviceReady, false);
            require(loadScripts, function () {
                console.log("Corodva libs loaded via require JS");
                // we can only register other device specific handlers after cordova js file has been loaded !!!
                appEventsEngine.initNativeAppEvents();
            });
        }
    },

    onDeviceReady: function () {
        AppUtils.deviceReady = true;
        if (AppUtils.DeviceType.iOS) {
            require(["PhoneGapUtils"], function () {
                window.plugins.PhoneGapUtils.getCachesFolderPath(function (path) {
                    ProjectGlobals.DOWNLOAD_PATH = path;
                });
            });
        } else if (AppUtils.DeviceType.ANDROID) {
            ProjectGlobals.DOWNLOAD_PATH = "file:///mnt/" + DirectJSCalls.getAppPublicStorageDir(false);
        }
    },

    initEventTypes: function () {
        if (AppUtils.DeviceType.MOBILE_DEVICE) {
            AppUtils.EventTypes.mouseDown = "touchstart";
            AppUtils.EventTypes.mouseDownSafeIOS5 = "touchstart";
            if (AppUtils.DeviceType.iOS5) {
                AppUtils.EventTypes.mouseDownSafeIOS5 = "click";
            }
            AppUtils.EventTypes.mouseMove = "touchmove";
            AppUtils.EventTypes.mouseUp = "touchend";
            AppUtils.EventTypes.scroll = "touchmove";
        }
    },

    checkNativeAppAndOfflineMode: function () {
        // special trick for Android
        if (AppUtils.DeviceType.ANDROID && typeof DirectJSCalls != 'undefined' && typeof DirectJSCalls.getNativeAppInfo == 'function') {
            var nativeAppInfo = DirectJSCalls.getNativeAppInfo();
            nativeAppInfo = JSON.parse(nativeAppInfo);
            window.isNativeApp = nativeAppInfo.isNativeApp;
            window.cordovaVersion = nativeAppInfo.cordovaVersion;
        }

        AppUtils.useNativeScroll = false;
        if (window.hasOwnProperty("isNativeApp")) {
            AppUtils.nativeApp = true;
            if (AppUtils.DeviceType.iOS) {
                AppUtils.useNativeScroll = true;
            }
        }
        if (!AppUtils.nativeApp && window.location.href.indexOf(".jsp?") == -1) {
            AppUtils.offlineExport = true;
        }
    },

    initDeviceSettings: function () {
        this.initDeviceType();
        this.initEventTypes();
        this.checkNativeAppAndOfflineMode();
        this.loadCordovaOnDemand();
        this.setStorageFolder(); // have this after we check if native app or not
    },

    getMenuScaleFactor: function () {
        var ret = AppUtils.runCustomMenuMethod("getCustomMenuScaleFactor"), screenDiagonal;
        if (ret == null) {
            ret = App.settings.menuScaleFactor;
        }
        if (navigator.userAgent.match(/(iPhone|iPod)/g)) {
            ret *= 1.5;
        } else if (AppUtils.DeviceType.ANDROID && typeof DirectJSCalls != 'undefined' && typeof DirectJSCalls.getScreenDiagonal == 'function') {
            screenDiagonal = DirectJSCalls.getScreenDiagonal();
            if (screenDiagonal < 6.5) {
                ret *= 1.3;
            }
        }
        return ret;
    },

    scaleNonRetinaImage: function (imageEl) {
        imageEl.width = imageEl.width * AppUtils.getMenuScaleFactor();
    },

    scaleNonRetinaImageOutsideMenu: function (imageEl) {
        imageEl.width = imageEl.width * App.settings.menuScaleFactor;
    },

    scaleRetinaImage: function (imageEl) {
        imageEl.width = imageEl.naturalWidth * AppUtils.getMenuScaleFactor() / 2;
    },

    adjustRecIconGlow: function (imageEl) {
        var playIcon, marginTop, marginLeft;
        this.scaleNonRetinaImage(imageEl);
        imageEl = $(imageEl);
        playIcon = imageEl.siblings();
        marginTop = (playIcon.height() - imageEl.height()) / 2;
        marginLeft = (playIcon.width() - imageEl.width()) / 2;
        imageEl
            .css("margin-top", marginTop + "px")
            .css("margin-left", marginLeft + "px");
    },

    scaleRecordingWizardImage: function (imageEl) {
        this.scaleRetinaImage(imageEl);
        var $imgEl = $(imageEl);
        $imgEl.css("margin-top", Math.round(-imageEl.height / 2) + "px");
    },

    loadCssFiles: function (cssURLs) {
        var idx, url;
        for (idx = 0; idx < cssURLs.length; idx++) {
            url = cssURLs[idx];
            var link = document.createElement("link");
            link.type = "text/css";
            link.rel = "stylesheet";
            link.href = url;
            document.getElementsByTagName("head")[0].appendChild(link);
        }
    },

    loadPlatformSpecificCssFiles: function () {
        if (AppUtils.DeviceType.iOS) {
            console.log("No specific CSS files for iOS platform");
        } else if (AppUtils.DeviceType.W8) {
            console.log("No specific CSS files for W8 platform");
        } else if (AppUtils.DeviceType.ANDROID) {
            AppUtils.loadCssFiles(androidCssURLs);
        } else if (!AppUtils.DeviceType.MOBILE_DEVICE) {
            AppUtils.loadCssFiles(browsersCssURLs);
        }
    },

    unloadCssFiles: function (cssURLs) {
        var idx, element;
        for (idx = 0; idx < cssURLs.length; idx++) {
            element = document.querySelectorAll('[href="' + cssURLs[idx] + '"]');
            element[0].parentNode.removeChild(element[0]);
        }
    },

    updateCssProjectStyle: function (newCssURL) {
        if (AppUtils.loadedProjectStyle != null) {
            AppUtils.unloadCssFiles([AppUtils.loadedProjectStyle]);
        }
        AppUtils.loadedProjectStyle = newCssURL;
        AppUtils.loadCssFiles([newCssURL]);
    },

    getLocalStorageBooleanItem: function (key, defaultValue) {
        var itemValue = localStorage.getItem(key);
        if (itemValue === null) {
            return defaultValue;
        }
        return itemValue == "true";
    },

    getValueAsBoolean: function (value) {
        return value == "true";
    },

    runCustomMenuMethod: function () {
        var methodName = arguments[0];
        if (typeof CustomMenuScripts != "undefined" && typeof CustomMenuScripts[methodName] == "function") {
            return CustomMenuScripts[methodName].apply(this, [].splice.call(arguments, 1));
        }
        return null;
    },

    navigateToPage: function (nextPage, customTransitionProperty) {
        var currentPage = Chapter.getActivePage(),
            translateOffset;
        if (!Chapter.trySpecialNavigation(customTransitionProperty, nextPage)) {
            // if not having a special navigation type we go on and do regular slide transition
            if (nextPage + App.segmentLength() == currentPage || nextPage - App.segmentLength() == currentPage) {
                translateOffset = Chapter.computeTranslateOffset(currentPage, nextPage);
                Chapter.translateScenesTo(translateOffset, function () {
                    Chapter.removeScrollingElementTransitionEndHandler();
                    Chapter.setActivePage(nextPage);
                    // TODO: remove the bellow hack, that is no longer required when using XWalk web view on Android !!!
                    setTimeout(function () {
                        if (AppUtils.DeviceType.ANDROID) {
                            Chapter.getScrollingElementsWrapper().getView().el.scrollLeft = 1;
                            Chapter.getScrollingElementsWrapper().getView().el.scrollLeft = 0;
                        }
                    }, 0);
                });
            } else {
                Chapter.setActivePage(nextPage);
                if (AppUtils.useNativeScroll) {
                    var newScrollValue = nextPage * App.settings.pageWidth;
                    AppUtils.getNavigationDiv().css("left", newScrollValue + "px");
                    document.getElementById("slidingScenesWrapper").scrollLeft = newScrollValue;
                }
            }
        }

        if(AppUtils.DeviceType.IE9 == true) {
            Chapter.setActivePage(nextPage);
        }
    },

    getMouseEventCoordinates: function (e) {
        var touch = {};
        if ((typeof event != 'undefined') && (typeof event.touches != 'undefined')) {
            var touches = event.touches;
            if (touches.length == 0) {
                touch.x = event.changedTouches[0].pageX;
                touch.y = event.changedTouches[0].pageY;
                return touch;
            }
            touch.x = event.touches[0].pageX;
            touch.y = event.touches[0].pageY;
        } else if (typeof e.originalEvent != 'undefined') {
            touch.x = e.originalEvent.pageX;
            touch.y = e.originalEvent.pageY;
        } else {
            touch.x = e.pageX;
            touch.y = e.pageY;
        }
        return touch;
    },

    getParentElement: function () {
        var parentElementURL = document.referrer;
        if (parentElementURL.length > 0) {
            var urlElements = parentElementURL.split('/');
            parentElementURL = urlElements[0] + '//' + urlElements[2] + '/';
        } else {
            parentElementURL = null;
        }
        return parentElementURL;
    },
    notifyParentThatPlayerIsLoaded: function () {
        if (!AppUtils.playerWasLoaded) {
            AppUtils.playerWasLoaded = true;
            if (null != AppUtils.parentElement) {
                window.parent.postMessage("Loaded", AppUtils.parentElement);
            }
        }
    },

    getNavigationDiv: function () {
        if (AppUtils.navigationDivEl == null) {
            AppUtils.navigationDivEl = $("<div>").addClass("navigationHelper").width(App.settings.width);
            $("#slidingScenes").append(AppUtils.navigationDivEl);
        }
        return AppUtils.navigationDivEl;
    },

    runWithIScroll: function () {
        return (AppUtils.DeviceType.ANDROID && !AppUtils.DeviceType.XWalk) || AppUtils.DeviceType.iOS5;
    },

    openURL: function (url) {
        if (AppUtils.nativeApp) {
            if (url.indexOf("mailto") > -1 && AppUtils.DeviceType.iOS) {
                // Opening mail form on iOS doesn't trigger an app paused/resume, so we don't reset current page
                require(["PhoneGapUtils"], function () {
                    window.plugins.PhoneGapUtils.openURLInSafari(url);
                });
            } else {
                Chapter.setAppPaused(true);
                Chapter.getActivePageElement().model.set("deactivatePage", function () {
                    if (AppUtils.DeviceType.ANDROID) {
                        DirectJSCalls.goToWeb(url);
                    } else if (AppUtils.DeviceType.iOS) {
                        require(["PhoneGapUtils"], function () {
                            window.plugins.PhoneGapUtils.openURLInSafari(url);
                        });
                    }
                });
            }
        } else {
            window.open(url);
        }
    }

};

AppUtils.initDeviceSettings();

