var CSSHandler = {
        browserPrefix: {
            firefox: {
                transform: 'transform',
                transitionEnd: 'transitionend',
                transition: '-moz-transition',
                transitionTimingFunction: '-moz-transition-timing-function',
                backFaceVisibility: '-moz-backface-visibility',
                perspective: '-moz-perspective',
                transformOrigin: '-moz-transform-origin'
            },
            webkit: {
                transform: '-webkit-transform',
                transitionEnd: 'webkitTransitionEnd',
                transition: '-webkit-transition',
                transitionTimingFunction: '-webkit-transition-timing-function',
                backFaceVisibility: '-webkit-backface-visibility',
                perspective: '-webkit-perspective',
                transformOrigin: '-webkit-transform-origin'
            },
            opera: {
                transform: '-o-transform',
                transitionEnd: 'oTransitionEnd',
                transition: '-o-transition',
                transitionTimingFunction: '-o-transition-timing-function',
                backFaceVisibility: '-o-backface-visibility',
                perspective: '-o-perspective',
                transformOrigin: '-o-transform-origin'
            },
            IE: {
                transform: '-ms-transform',
                transitionEnd: 'transitionend',
                transition: 'transition',
                transitionTimingFunction: 'transition-timing-function',
                backFaceVisibility: 'backface-visibility',
                perspective: 'perspective',
                transformOrigin: 'transform-origin'
            }
        },
        currentBrowserPrefix: null,
        cssPrefix: "",

        setCurrentBrowserPrefix: function () {
            if (navigator.userAgent.indexOf('WebKit/') > -1) {
                this.cssPrefix = "-webkit-";
                this.currentBrowserPrefix = this.browserPrefix.webkit;
            } else if (navigator.userAgent.indexOf('Firefox/') > -1) {
                this.cssPrefix = "-moz-";
                this.currentBrowserPrefix = this.browserPrefix.firefox;
            } else if (navigator.userAgent.indexOf('Opera/') > -1) {
                this.cssPrefix = "-o-";
                this.currentBrowserPrefix = this.browserPrefix.opera;
            } else if (this.isInternetExplorer()) {
                this.currentBrowserPrefix = this.browserPrefix.IE;
            } else {
                this.cssPrefix = "-webkit-";
                this.currentBrowserPrefix = this.browserPrefix.webkit;
            }
        },
        setZoom: function ($element, scaleFactor) {
            $element.css('zoom', scaleFactor);
        },

        isInternetExplorer: function () {
            var rv = -1;
            if (navigator.appName == 'Microsoft Internet Explorer') {
                var re = new RegExp("MSIE ([0-9]{1,}[\.0-9]{0,})");
                if (re.exec(navigator.userAgent) != null)
                    rv = parseFloat(RegExp.$1);
            }
            else if (navigator.appName == 'Netscape') {
                var re = new RegExp("Trident/.*rv:([0-9]{1,}[\.0-9]{0,})");
                if (re.exec(navigator.userAgent) != null)
                    rv = parseFloat(RegExp.$1);
            }
            return rv >= 0;
        },

        getCurrentBrowserPrefix: function () {
            return this.currentBrowserPrefix;
        },

        setWidth: function ($element, value) {
            var validValue = this.getValidValue(value);
            if (this.validate($element) && this.validate(validValue))
                $element.css("width", validValue);
        },
        setHeight: function ($element, value) {
            var validValue = this.getValidValue(value);
            if (this.validate($element) && this.validate(validValue))
                $element.css("height", validValue);
        },
        setMinWidth: function ($element, value) {
            var validValue = this.getValidValue(value);
            if (this.validate($element) && this.validate(validValue))
                $element.css("min-width", validValue);
        },
        setBackgroundColor: function ($element, color, opacity) {
            if (this.validate($element)) {
                if (!this.validate(opacity))
                    opacity = 1;
                if(!this.validate(color) || color == "null") {
                    color="#FFFFFF";
                    opacity = 0;
                }
                $element.css("backgroundColor", this.hexToRGBA(color, opacity));
            }
        },
        setTitle: function($element, title) {
            if (this.validate($element) && this.validate(title)) {
                $element.attr("title", title);
            }
        },
        setColor: function ($element, color, opacity) {
            if (this.validate($element) && this.validate(color)) {
                if (!this.validate(opacity))
                    opacity = 1;
                $element.css("color", this.hexToRGBA(color, opacity));
            }
        },
        setLeft: function ($element, value) {
            var validValue = this.getValidValue(value);
            if (this.validate($element) && this.validate(validValue))
                $element.css("left", validValue);
        },
        setRight: function ($element, value) {
            var validValue = this.getValidValue(value);
            if (this.validate($element) && this.validate(validValue))
                $element.css("right", validValue);
        },
        setTop: function ($element, value) {
            var validValue = this.getValidValue(value);
            if (this.validate($element) && this.validate(validValue))
                $element.css("top", validValue);
        },
        setBottom: function ($element, value) {
            var validValue = this.getValidValue(value);
            if (this.validate($element) && this.validate(validValue))
                $element.css("bottom", validValue);
        },
        setPosition: function ($element, position) {
            $element.css('position', position);
        },
        setHorizontalMargin: function ($element, leftMargin, rightMargin) {
            var validLeftMargin = this.getValidValue(leftMargin),
                validRightMargin = this.getValidValue(rightMargin);
            if (this.validate($element) && this.validate(validLeftMargin) && this.validate(validRightMargin))
                $element.css("margin-left", validLeftMargin).css("margin-right", validRightMargin);
        },
        setVerticalMargin: function ($element, topMargin, bottomMargin) {
            var validTopMargin = this.getValidValue(topMargin),
                validBottomMargin = this.getValidValue(bottomMargin);
            if (this.validate($element) && this.validate(validTopMargin) && this.validate(validBottomMargin))
                $element.css("margin-top", validTopMargin).css("margin-bottom", validBottomMargin);
        },
        setHorizontalPadding: function ($element, leftPadding, rightPadding) {
            var validLeftPadding = this.getValidValue(leftPadding),
                validRightPadding = this.getValidValue(rightPadding);
            if (this.validate($element) && this.validate(validLeftPadding) && this.validate(validRightPadding))
                $element.css("padding-left", validLeftPadding).css("padding-right", validRightPadding);
        },

        setVerticalPadding: function ($element, topPadding, bottomPadding) {
            var validTopPadding = this.getValidValue(topPadding),
                validBottomPadding = this.getValidValue(bottomPadding);
            if (this.validate($element) && this.validate(validTopPadding) && this.validate(validBottomPadding))
                $element.css("padding-top", validTopPadding).css("padding-bottom", validBottomPadding);
        },
        setBackgroundSize: function ($element, width, height) {
            var validWidth = this.getValidValue(width),
                validHeight = this.getValidValue(height);
            if (this.validate($element) && this.validate(validWidth) && this.validate(validHeight))
                $element.css("background-size", validWidth + " " + validHeight);
        },

        setVisibility: function ($element, opacity) {
            if (this.validate($element) && this.validate(opacity)) {
                $element.css("visibility", opacity);
            }
        },
        setOpacity: function ($element, opacity) {
            if (this.validate($element) && this.validate(opacity)) {
                $element.css("opacity", opacity);
            }
        },
        setBackgroundImage: function ($element, url) {
            $element.css('background-image', 'url(' + url + ')');
        },
        setBackgroundPosition: function ($element, position) {
            $element.css('background-position', position);
        },
        setBackgroundRepeat: function ($element, repeat) {
            $element.css('background-repeat', repeat);
        },
        setFontSize: function ($element, fontSize) {
            var validFontSize = this.getValidValue(fontSize);
            if (this.validate($element) && this.validate(validFontSize))
                $element.css("fontSize", validFontSize);
        },
        setLineHeight: function ($element, lineHeight) {
            var validLineHeight = this.getValidValue(lineHeight);
            if (this.validate($element) && this.validate(validLineHeight))
                $element.css("line-height", lineHeight);
        },

        translate3D: function ($element, xOffset, yOffset, zOffset, transitionPeriod, timingFunction, existingTransitionPeriod, use3d) {
            var validXOffset = this.getValidValue(xOffset),
                validYOffset = this.getValidValue(yOffset),
                validZOffset = this.getValidValue(zOffset),
                timingPeriod;

            if (null == this.getValidValue(transitionPeriod)) {
                transitionPeriod = 0;
            }
            timingPeriod = this.currentBrowserPrefix.transform + " " + transitionPeriod + "ms";

            if (this.validate($element) && this.validate(validXOffset) && this.validate(validYOffset) && this.validate(validZOffset)) {
                if (null != timingFunction) {
                    timingPeriod += " " + "cubic-bezier(" + timingFunction + ")";
                }
                if (null != existingTransitionPeriod) {
                    timingPeriod += ", " + existingTransitionPeriod;
                }

                $element.css(this.currentBrowserPrefix.transition, timingPeriod);

                if(use3d == undefined || use3d == null)
                    use3d = false;

//                console.log("start move");
                if (use3d == false) {
                    $element.css(this.currentBrowserPrefix.transform, "translate(" + validXOffset + "," + validYOffset + ")");
                } else {
                    $element.css(this.currentBrowserPrefix.transform, "translate3d(" + validXOffset + "," + validYOffset + "," + validZOffset + ")")
                        .css(this.currentBrowserPrefix.backFaceVisibility, "hidden");
//                .css(this.currentBrowserPrefix.perspective, 1000); //prevent a white line bellow fading elements;
                }

            }
        },
        setBackfaceVisibility: function ($element, backFaceVisibility) {
            $element.css(this.currentBrowserPrefix.backFaceVisibility, backFaceVisibility);
        },
        scale: function ($element, scaleFactor) {
            var validScaleFactor = this.getValidNumber(scaleFactor);
            validScaleFactor = (null == validScaleFactor) ? '' : validScaleFactor;
            if (this.validate($element) && this.validate(validScaleFactor))
//                setTimeout(function () {
                $element.css(this.currentBrowserPrefix.transform, "scale(" + validScaleFactor + ")");
//                }, 0);

        },
        setPositionType: function ($element, position) {
            $element.css('position', position);
        },

        rotate: function ($element, rotateAngle) {
            $element.css(this.currentBrowserPrefix.transform, "rotate(" + rotateAngle + "deg)");
        },

        setTransitionEndHandler: function ($element, handler, suffix) {
            if (this.validate($element) && this.validate(handler)) {
                if (suffix != null) {
                    $element.bind(this.currentBrowserPrefix.transitionEnd + "." + suffix, handler);
                } else {
                    $element.bind(this.currentBrowserPrefix.transitionEnd, handler);
                }
            }
        },
        removeTransitionEndHandler: function ($element, suffix) {
            if (this.validate($element)) {
                if (suffix != null) {
                    $element.unbind(this.currentBrowserPrefix.transitionEnd + "." + suffix);
                } else {
                    $element.unbind(this.currentBrowserPrefix.transitionEnd);
                }
            }
        },

        restoreTransitionValues: function ($element, neededTransition) {
            console.log("restoreTransitionValues  " + neededTransition);
            var timingPeriod = null == neededTransition ? '' : neededTransition;
            $element.css(this.currentBrowserPrefix.transition, timingPeriod);
        },
        restoreTransformValues: function ($element) {
//            setTimeout(function () {
            $element.css(this.currentBrowserPrefix.transform, '');
//            }, 0);
        },

        setTransition: function ($element, transitionProperty, duration) {
            $element.css(this.currentBrowserPrefix.transition, transitionProperty + " " + duration + "ms");
        },

        setTransitionTimingFunction: function ($element, timingFunction) {
            if (this.validate($element) && this.validate(timingFunction)) {
                $element.css(this.currentBrowserPrefix.transitionTimingFunction, "cubic-bezier(" + timingFunction + ")");
            }
        },

        addTransition: function ($element, timingFunction, property, timing) {
            var newTransitionValue;
            if (this.validate($element) && this.validate(timingFunction)) {
                newTransitionValue = CommonActions.oldTimingFunction != null ? CommonActions.oldTimingFunction + ", " : "";
                newTransitionValue += property + " " + "cubic-bezier(" + timingFunction + ")" + " " + timing + "ms";
                $element.css(this.currentBrowserPrefix.transition, newTransitionValue);
            }
        },
        getMatrix: function ($element) {
            return $element.css(this.currentBrowserPrefix.transform);
        },
        getTranslationOffset: function (transformString) {
            var matchingMatrix = /matrix\(\s*-?\d+(?:\.\d+)?\s*,\s*-?\d+(?:\.\d+)?\s*,\s*-?\d+(?:\.\d+)?\s*,\s*-?\d+(?:\.\d+)?\s*\,\s*(-?\d+(?:\.\d+)?)\s*,\s*(-?\d+(?:\.\d+)?)\s*\)/;
            var retMatrix = {
                xOffset: 0,
                yOffset: 0
            };

            var match = transformString.match(matchingMatrix);
            if (match) {
                retMatrix.xOffset = match[1];
                retMatrix.yOffset = match[2];
            }
            return retMatrix;
        },

        hexToRGBA: function (hex, opacity) {
            var rgb = hex.replace('#', '').match(/(.{2})/g);
            var i = 3;
            while (i--) {
                rgb[i] = parseInt(rgb[i], 16);
            }
            if (typeof opacity == 'undefined') {
                return 'rgb(' + rgb.join(', ') + ')';
            }
            return 'rgba(' + rgb.join(', ') + ', ' + opacity + ')';
        },

        scrollTo: function ($element, scrollLeft, scrollTop, time, scrollEndHandler) {
            var validXOffset = this.getValidValue(scrollLeft),
                validYOffset = this.getValidValue(scrollTop),
                validTime = this.getValidValue(time);

            if (this.validate(validTime) && this.validate($element)) {
                if (this.validate(validXOffset)) {
                    if (this.validate(validYOffset)) {
                        $element.animate({
                            scrollLeft: validXOffset,
                            scrollTop: validYOffset
                        }, time, function () {
                            if (scrollEndHandler) {
                                scrollEndHandler();
                            }
                        });
                    } else {
                        $element.animate({
                            scrollLeft: validXOffset
                        }, time, function () {
                            if (scrollEndHandler) {
                                scrollEndHandler();
                            }
                        });
                    }
                } else {
                    if (this.validate(validYOffset)) {
                        $element.animate({
                            scrollTop: validYOffset
                        }, time, function () {
                            if (scrollEndHandler) {
                                scrollEndHandler();
                            }
                        });
                    }
                }
            }
        },
        stopAnimation: function ($element) {
            if (this.validate($element)) {
                $element.stop();
            }
        },

        validate: function (element) {
            var ret = false;
            if (element != null) {
                ret = true;
            }
            return ret;
        },

        getValidValue: function (value) {
            var ret = value;
            if (null != value) {
                if (typeof value == 'number') {
                    ret += 'px';
                } else {
                    if (value.indexOf("px") == -1) {
                        if (value.indexOf("%") == -1) {
                            ret = null;
                        }
                    }
                }
            }

            return ret;
        },
        getValidNumber: function (value) {
            var ret = null;
            if (null != value) {
                if (typeof value == 'number') {
                    ret = value;
                }
            }
            return ret;
        },

        removeBrowserSpecificProperty: function ($element, property, propertyValue) {
            // TODO: here we should make a difference between translate3d and scale from matrix values, a bit more complicate ...
//            var currentPropertyValues, idx, newPropertyValue;
//            if (propertyValue) {
//                newPropertyValue = "";
//                currentPropertyValues = $element.css(this.currentBrowserPrefix[property]).split(" ");
//                for (idx = 0; idx < currentPropertyValues.length; idx++) {
//                    if (currentPropertyValues[idx].indexOf(propertyValue) == -1) {
//                        if (newPropertyValue != "") {
//                            newPropertyValue += " "
//                        }
//                        newPropertyValue += currentPropertyValues[idx];
//                    }
//                }
//                $element.css(this.currentBrowserPrefix[property], newPropertyValue);
//            } else {
//                $element.css(this.currentBrowserPrefix[property], "");
//            }
            $element.css(this.currentBrowserPrefix[property], "");
        },

        getBrowserPrefixedProperty: function (property) {
            return this.currentBrowserPrefix[property];
        },

        getCssPrefix: function () {
            return this.cssPrefix;
        },

        setZIndex: function ($element, zIndex) {
            if (this.validate(zIndex) && this.validate($element)) {
                $element.css("z-index", zIndex);
            }
        },

        setTransformOrigin: function ($element, xOrigin, yOrigin) {
            $element.css(this.currentBrowserPrefix.transformOrigin, this.getValidValue(xOrigin) + " " + this.getValidValue(yOrigin));
        }
    }
    ;
CSSHandler.setCurrentBrowserPrefix();

