define(['DualPageElement', 'Templates', 'PersistentSceneObjectsUtils'], function (DualPageElement) {
        return function () {
            var that = this;

            // private fields
            this.globalModels = {};
            this.preventLeftUpdateAfterPageChange = false;
            this.pagesRawModel = null;
            this.settings = null;
            this.menuModel = null;
            //swipe period in milliseconds
            this.DEFAULT_TRANSITION_DURATION = 1200;
            this.persistentCategories = null;
            this.globalTimers = [];
            this.bottomMenuCustomActiveTabsCollection = new Backbone.Collection();
            this.appPaused = false;

            this.defineClasses = function () {
                that.ScrollingElementView = BaseView.extend({
                    tagName: "div",
                    id: "slidingScenes",
                    loadedPages: {},
                    activePageModel: null,
                    numberOfPagesToLoad: 0,
                    newPagesQueue: [],
                    dualPageElements: [],
                    allowedToCreateNewPage: true,
                    transitionEndHandler: null,

                    initialize: function (attr) {
                        this.listenTo(this.model, "change:remove", this.removeView);
                        this.listenTo(this.model, "change:activePage", this.updatePages);
                        this.listenTo(this.model, "change:resetCurrentPage", this.resetCurrentPage);
                        this.listenTo(this.model, "change:translate3DOffset", this.translateTo);
                        this.listenTo(this.model, "change:toggledCategories", this.toggledCategories);
                        this.listenTo(this.model, "change:orientation", this.updateOrientation);
                        this.listenTo(this.model, "change:updateZoomDualPage", this.updateZoomOnOrientationChange);
                    },

                    updatePagesModel: function () {
                        var models = that.pagesModelCollection.models, pageNo;
                        for (pageNo = 0; pageNo < models.length; pageNo++) {
                            models[pageNo].set("left", pageNo * App.settings.pageWidth);
                            models[pageNo].set("width", App.settings.pageWidth);
                            models[pageNo].set("height", App.settings.pageHeight);
                        }
                    },

                    updateOrientation: function () {
                        var thisView = this,
                            models = that.pagesModelCollection.models,
                            dualPageModel, idx;
                        // adjust pages (width, height, left)
                        this.updatePagesModel();
                        // now notify all pages about orientation change
                        for (idx = 0; idx < models.length; idx++) {
                            console.log("MOB-1718: Update page, send notification to page " + models[idx].get("index"));
                            // will actually be executed only by loaded pages, but updating model to all pages is required
                            models[idx].set("orientation", this.model.get("orientation"));
                        }
                        // adjust dualPageElements
                        for (idx in this.dualPageElements) {
                            if (this.dualPageElements.hasOwnProperty(idx)) {
                                dualPageModel = this.dualPageElements[idx].getModel();
                                dualPageModel.set("orientation", this.model.get("orientation"));
                                dualPageModel.set("height", App.settings.pageHeight);
                                dualPageModel.set("width", App.settings.pageWidth * 2);
                                dualPageModel.set("left", App.settings.pageWidth * 2 * idx);
                            }
                        }
                        require(['PageElement', 'TransitionPlayer'], function (PageElement) {
                            thisView.setupRequiredPages(thisView.getModelsForPagesToRender(thisView.model.get("activePage")), PageElement, true);
                        });
                    },

                    updateZoomOnOrientationChange: function () {
                        var pageNo, dualPageNo;
                        if (this.model.get("updateZoomDualPage")) {
                            for (dualPageNo in this.dualPageElements) {
                                if (this.dualPageElements.hasOwnProperty(dualPageNo)) {
                                    var dualPageView = this.dualPageElements[dualPageNo].getView(),
                                        dualPageModel = this.dualPageElements[dualPageNo].getModel(),
                                        zoomInstance = dualPageView.getZoomInstance();
                                    if (zoomInstance != null && !dualPageModel.get("needsRender")) {
                                        dualPageView.$el.css("transform", "");
                                        dualPageView.$el.css("transform-origin", "");
                                        CSSHandler.removeBrowserSpecificProperty(dualPageView.$el, 'transition', null);
                                        zoomInstance.destroy();
                                        delete dualPageView.zoomInstance;
                                        console.log("add/remove zoom: remove called for dual page wrapper " + dualPageNo);
                                    }
                                }
                            }
                        } else {
                            var models = that.pagesModelCollection.models;
                            for (pageNo in this.loadedPages) {
                                if (this.loadedPages.hasOwnProperty(pageNo)) {
                                    if (this.loadedPages[pageNo]) {
                                        console.log("add/remove zoom: removeZoom value for page " + models[pageNo].get("index") + " is " + models[pageNo].get("removeZoom"));
                                        models[pageNo].set("removeZoom", true);
                                    }
                                }
                            }
//                        for (pageNo = 0; pageNo < models.length; pageNo++) {
//                            console.log("add/remove zoom: removeZoom value for page " + models[pageNo].get("index") + " is " + models[pageNo].get("removeZoom"));
//                            models[pageNo].set("removeZoom", true);
//                        }
                        }
                    },

                    addTransitionEndHandler: function (transitionEndHandler) {
                        if (AppUtils.useNativeScroll) {
                            this.transitionEndHandler = transitionEndHandler;
                        } else {
                            that.scrollingElementsView.removeTransitionEndHandler();
                            if (null != transitionEndHandler) {
                                this.setTransitionEndHandler(transitionEndHandler);
                            }
                        }
                    },

                    runTransitionEndHandler: function () {
                        if (this.transitionEndHandler != null) {
                            this.transitionEndHandler();
                            this.transitionEndHandler = null; // consume this after run
                        }
                    },

                    translateTo: function () {
                        var translate3DOffset = this.model.get("translate3DOffset");
                        if (translate3DOffset == 0 && AppUtils.DeviceType.iOS) {
                            CSSHandler.removeBrowserSpecificProperty(this.$el, 'transform', null);
                            CSSHandler.removeBrowserSpecificProperty(this.$el, 'transition', null);
                        } else if (translate3DOffset >= App.maxTranslateValueForLastPage()) {
                            this.translate3D(translate3DOffset, 0, 0, this.model.get("transitionPeriod"), null, null, false);
                        } else {
                            console.log("translateTo: Skip navigating past last page " + translate3DOffset);
                        }
                    },
                    removeView: function () {
                        var models = that.pagesModelCollection.models, idx;
                        for (idx = 0; idx < models.length; idx++) {
                            models[idx].set('removed', true);
                        }
                        this.remove();
                        this.unbind();
                    },
                    getModelsForPagesToRender: function (activePage) {
                        var models = that.pagesModelCollection.models,
                            firstPage = 0,
                            lastPage = models.length - 1,
                            returnModels = [],
                            thisView = this,
                            bufferPages = App.getBufferPages();

                        var addPageForLoad = function (pageNo) {
                            if (!(pageNo < firstPage) && !(pageNo > lastPage) && !thisView.loadedPages[pageNo]) {
                                returnModels.push(models[pageNo]);
                                thisView.loadedPages[pageNo] = true;
                                console.log("MOB-1718: addPageForLoad " + pageNo + " page width " + models[pageNo].get("width"));
                            }
                        };

                        for (var idx = 0; idx < bufferPages.length; idx++) {
                            addPageForLoad(activePage + bufferPages[idx]);
                        }

                        return returnModels;

                    },
                    toggledCategories: function () {
                        var activePage = this.model.get("activePage"),
                            models = that.pagesModelCollection.models,
                            categories = this.model.get("toggledCategories"),
                            element;
                        if (null != categories) {
                            this.model.set("toggledCategories", null);
                            for (element in this.loadedPages) {
                                if (this.loadedPages.hasOwnProperty(element)) {
                                    if (this.loadedPages[element]) {
                                        models[element].set("toggledCategories", categories);
                                    }
                                }
                            }

                            var buttonsModels = that.bottomMenuCustomActiveTabsCollection.models, idx;
                            for (idx = 0; idx < buttonsModels.length; idx++) {
                                buttonsModels[idx].set('toggledCategories', categories);
                            }

                            this.updatePersistentCategoriesIfNeeded(categories.activatedCategories);
                            this.updatePersistentCategoriesIfNeeded(categories.deactivatedCategories);
                            this.saveGlobalActionsIfNeeded(categories);
                        }

                    },
                    updatePersistentCategoriesIfNeeded: function (categories) {
                        var categoryId, persistentCategoryChanged = false;

                        for (categoryId in categories) {
                            if (null != that.persistentCategories && that.persistentCategories.hasOwnProperty(categoryId)) {
                                persistentCategoryChanged = true;
                            }
                        }

                        if (persistentCategoryChanged)
                            that.savePersistentCategories();
                    },
                    saveGlobalActionsIfNeeded: function (categories) {
                        if (categories.globalCategoryAction) {
                            var projectCategories = Chapter.getObjectCategories(),
                                globalSavedCategories = localStorage.getItem('globalCategoryActions'),
                                categoryId, updatedCategory;
                            if (null != globalSavedCategories) {
                                globalSavedCategories = JSON.parse(globalSavedCategories);
                            } else {
                                globalSavedCategories = {};
                            }
                            for (categoryId in categories.activatedCategories) {
                                if (categories.activatedCategories.hasOwnProperty(categoryId)) {
                                    updatedCategory = JSON.parse(JSON.stringify(projectCategories[categoryId]));
                                    updatedCategory['updatedByProjectid'] = that.settings.localRootSequenceId;
                                    globalSavedCategories[categoryId] = updatedCategory;
                                }
                            }
                            for (categoryId in categories.deactivatedCategories) {
                                if (categories.deactivatedCategories.hasOwnProperty(categoryId)) {
                                    updatedCategory = JSON.parse(JSON.stringify(projectCategories[categoryId]));
                                    updatedCategory['updatedByProjectid'] = that.settings.localRootSequenceId;
                                    globalSavedCategories[categoryId] = updatedCategory;
                                }
                            }
                            localStorage.setItem("globalCategoryActions", JSON.stringify(globalSavedCategories));
                            console.log("sa");
                        }
                    },

                    getZoomElementForPage: function (pageNumber) {
                        if (App.dualLandscapeActive()) {
                            return this.dualPageElements[Math.floor(pageNumber / 2)].getView();
                        } else {
                            return Chapter.getActivePageElement();
                        }
                    },

                    appendPageInWrapper: function (page) {
                        var renderedPage = page.getView().render(),
                            dualPageNo, dualPageModel, pageModel, renderedDualPage, dualPageElement, leftValue,
                            thisView = this;
                        if (Chapter.getSettings()["useTwoInOneLandscapeMode"]) {
                            pageModel = page.getModel();
                            dualPageNo = Math.floor(pageModel.get("index") / 2);
                            if (typeof thisView.dualPageElements[dualPageNo] == "undefined") {
                                leftValue = pageModel.get("left");
                                if (pageModel.get("index") % 2 == 1) {
                                    leftValue -= App.settings.pageWidth;
                                }
                                dualPageModel = {
                                    wrapperNo: dualPageNo,
                                    width: (App.lastKnownOrientation == SupportedOrientation.LANDSCAPE ? App.settings.pageWidth : App.settings.pageWidth * 2),
                                    height: App.settings.pageHeight,
                                    left: leftValue,
                                    addedPage: null
                                };
                                console.log("MOB-1718: Create new dualPageModel with left: " + leftValue + " for page " + pageModel.get("index"));
                                dualPageElement = new DualPageElement(dualPageModel);
                                thisView.dualPageElements[dualPageNo] = dualPageElement;
                                dualPageElement.init();
                                renderedDualPage = dualPageElement.getView().render();
                                renderedDualPage.$el.append(renderedPage.el);
                                thisView.$el.append(renderedDualPage.el);
                            } else {
                                var dualPageView = thisView.dualPageElements[dualPageNo].getView();
                                if (this.dualPageElements[dualPageNo].getModel().get("needsRender")) {
                                    dualPageView.addListeners();
                                    renderedDualPage = dualPageView.render();
                                    renderedDualPage.$el.append(renderedPage.el);
                                    thisView.$el.append(renderedDualPage.el);
                                }
                                dualPageView.$el.append(renderedPage.el);
                            }
                            this.dualPageElements[dualPageNo].getModel().set("addedPage", pageModel.get("index"));
                        } else {
                            this.$el.append(renderedPage.el);
                        }
                    },
                    setActivePageCallback: function (page, totalPages) {
                        this.appendPageInWrapper(page);
//                    this.$el.append(page.getView().render().el); // replaced by the above logic
                        this.numberOfPagesToLoad++;
                        console.log("setActivePageCallback: pageCreated and added to DOM " + page.getModel().get("index"));
                        if (page.getModel().get("index") == this.model.get("activePage")) {
                            this.activePageModel = that.pagesModelCollection.models[this.model.get("activePage")];
                            that.removePageTransitionDuration();
                            console.log("Will scroll to " + (this.model.get("activePage") * App.settings.width) + " for active page " + this.model.get("activePage"));
                            if (AppUtils.DeviceType.iOS) {
                                if (!AppUtils.useNativeScroll) {
                                    this.model.set("translate3DOffset", 0);
                                }
                            } else {
                                that.translateScenesTo(-this.model.get("activePage") * App.settings.pageWidth);
                            }
                            console.log("MOB-1718: Update scrollLeft value to " + this.model.get("activePage") * App.settings.pageWidth);
                            console.log("Current scroll value is: " + that.scrollingElementsWrapper.getModel().get("scrollLeft"));
                            that.scrollingElementsWrapper.getModel().set("scrollLeft", this.model.get("activePage") * App.settings.pageWidth);
                            that.restorePageTransitionDuration();
//                        this.activePageModel.set('zIndex', zIndex.activePage);
                            this.activePageModel.set("shouldBeActive", true);
                        } else {
                            console.log("ZINDEX FOR INACTIVE PAGE");
//                        page.getModel().set('zIndex', zIndex.inactivePage);
                        }
                        if (this.numberOfPagesToLoad == totalPages) {
                            console.log("Now safe to scroll to current page");
                            this.numberOfPagesToLoad = 0;
                            this.removeOldLoadedPages();
                            this.updateZIndexToAllInactivePages();
                        }
                        this.allowedToCreateNewPage = true;
                        this.createAndAppendPage();

                    },

                    setupRequiredPages: function (models, PageElement, skipRemove) {
                        var idx, ret = [],
                            numberOfPages = models.length;

                        if (models.length == 0 && !skipRemove) {
                            this.removeOldLoadedPages();
                        } else {
                            for (idx = 0; idx < models.length; idx++) {
                                this.newPagesQueue.push(this.getPageSetupInformation(models[idx], numberOfPages, PageElement));
                                this.createAndAppendPage();
                            }
                        }
                        return ret;
                    },
                    createAndAppendPage: function () {
                        if (this.allowedToCreateNewPage && this.newPagesQueue.length > 0) {
                            this.allowedToCreateNewPage = false;
                            var page, pageInformation = this.newPagesQueue.shift();
                            console.log("pageCreated  " + pageInformation.pageModel.get("index"));
                            page = new pageInformation.PageElement(pageInformation.pageModel);
                            this.doPageInit(page, pageInformation.totalPages);
                        }
                    },
                    getPageSetupInformation: function (pageModel, totalPages, PageElement) {
                        var ret = {};
                        ret.pageModel = pageModel;
                        ret.totalPages = totalPages;
                        ret.PageElement = PageElement;
                        return ret;
                    },
                    doPageInit: function (page, totalNumberOfPages) {
                        var thisView = this;
                        page.init(function () {
                            thisView.setActivePageCallback(page, totalNumberOfPages);
                        });
                    },
                    updateZIndexToAllInactivePages: function () {
                        var page;
                        zIndex.activePage = zIndex.inactivePage;
                        zIndex.inactivePage--;
                        for (page in this.loadedPages) {
                            if (this.loadedPages.hasOwnProperty(page) && this.loadedPages[page]) {
                                if (this.model.get("activePage") != page) {
                                    that.pagesModelCollection.models[page].set('zIndex', zIndex.inactivePage);
                                }
                            }
                        }
                    },
                    resetCurrentPage: function () {
                        if (this.activePageModel != null) {
                            var willPlay = this.model.get("resetCurrentPage"),
                                wasPagePaused = this.activePageModel.get("paused");
                            console.log("GYL-414", "resetCurrentPage called with value " + willPlay + " and page paused " + wasPagePaused);
                            if (!willPlay) {
                                console.log("GYL-414", "Now deactivate page");
                                this.activePageModel.set("activatePage", false); // page deactivate is allowed in all cases
                                if (typeof SoundPlayer != 'undefined') {
                                    SoundPlayer.endAllPlayingSoundsForPage(this.activePageModel.get("index"), false);
                                }
                            } else if (!wasPagePaused) {
                                console.log("GYL-414", "Can activate page because it was not paused in previous session")
                                this.activePageModel.set("activatePage", true);
                            } else {
                                console.log("GYL-414", "Will not play because app was paused using menu buttons");
                            }
                        } else {
                            console.log("resetCurrentPage: there is no activePageModel set !!!");
                        }
                    },
                    updatePages: function () {
                        var activePage = this.model.get("activePage"),
                            thisView = this;

                        if (thisView.activePageModel != null) {
                            console.log("PAGE  " + this.activePageModel.get("index") + "DEACTIVATED");
                            this.activePageModel.set("activatePage", false);
                            this.activePageModel.set("shouldBeActive", false);
                        }

                        if (this.loadedPages[this.model.get("activePage")]) {
                            Chapter.cancelGlobalTimers();
                            if (activePage > 0) { // for Kpt Sabeltann
                                if (null != that.bottomMenu) {
                                    that.bottomMenu.getModel().set('show', false);
                                }
                            }
                            if (!that.preventLeftUpdateAfterPageChange) {
                                if (AppUtils.useNativeScroll) {
                                    // don't do anything here, where native scroll will handle everything
                                } else {
                                    that.removePageTransitionDuration();
                                    if (AppUtils.DeviceType.iOS) {
                                        this.model.set("translate3DOffset", 0);
                                    } else {
                                        // only relevant in portrait after changing from dual landscape and page is already loaded !!!
                                        this.model.set("translate3DOffset", this.model.get("activePage") * (-App.settings.pageWidth));
                                    }

                                    that.scrollingElementsWrapper.getModel().set("scrollLeft", this.model.get("activePage") * App.settings.pageWidth);
                                    that.restorePageTransitionDuration();
                                }
                            } else {
                                console.log("GYL-401: page transition was not using slide");
                            }
                            console.log("PAGE " + this.model.get("activePage") + " SHOULD BE ACTIVATED FROM UPDATE PAGES");
                            thisView.activePageModel = that.pagesModelCollection.models[thisView.model.get("activePage")];
//                        thisView.activePageModel.set('zIndex', zIndex.activePage);
                            thisView.activePageModel.set("shouldBeActive", true);

                        }
                        require(['PageElement', 'TransitionPlayer'], function (PageElement) {
                            thisView.setupRequiredPages(thisView.getModelsForPagesToRender(thisView.model.get("activePage")), PageElement, false);
                        });

                    },

                    removeOldLoadedPages: function () {
                        var activePage = this.model.get("activePage"),
                            element, idx,
                            models = that.pagesModelCollection.models,
                            presumablyLoadedPages = [activePage],
                            bufferPages = App.getBufferPages();

                        for (idx = 0; idx < bufferPages.length; idx++) {
                            presumablyLoadedPages.push(activePage + bufferPages[idx]);
                        }

                        for (element in this.loadedPages) {
                            if (this.loadedPages.hasOwnProperty(element)) {
                                if (this.loadedPages[element] && presumablyLoadedPages.indexOf(parseInt(element)) == -1) {
//                                models[element].set("zIndex", 0);
                                    models[element].set("removed", true);
                                    if (Chapter.getSettings()["useTwoInOneLandscapeMode"]) {
                                        this.dualPageElements[Math.floor(element / 2)].getModel().set("removedPage", element);
                                    }
                                    models[element].set("activateObjects", false);
                                    models[element].set("allObjectsLoaded", false);
                                    models[element].set("allResourcesLoaded", false);
                                    console.log("removeOldLoadedPages called for page " + element);
                                    this.loadedPages[element] = false;
                                    models[element].set("removed", false);
                                }
                            }
                        }
                    },
                    render: function () {
                        console.log("Now render sliding scenes");
                        var thisView = this;

                        if (thisView.model.get("activePage") != -1) { // TODO: check this scenario !!!
                            this.$el.html();
                            require(['PageElement', 'TransitionPlayer'], function (PageElement) {
                                thisView.setupRequiredPages(thisView.getModelsForPagesToRender(thisView.model.get("activePage")), PageElement, false);
                                thisView.activePageModel = that.pagesModelCollection.models[thisView.model.get("activePage")];
                            });
                        }

                        return this;
                    },
                    sendToggleVisibleTextBoxSignal: function (visible) {
                        for (var element in this.loadedPages) {
                            if (this.loadedPages[element]) {
                                var loadedPageModel = that.pagesModelCollection.models[element];
                                loadedPageModel.set("toggleTextBoxVisibility", visible);
                            }
                        }
                    }
                });


                that.NavigationArrowView = BaseView.extend({
                    tagName: "div",
                    template: templates.getTemplate('retinaImage'),
                    className: "chapterBottomNavigationArrow",
                    mouseDown: false,

                    initialize: function () {
                        this.listenTo(this.model, "change:remove", this.removeView);
                        this.listenTo(this.model, "change:pageNumber", this.updateForPage);
                    },
                    events: function () {
                        var _events = {};
                        _events[AppUtils.EventTypes.mouseUp] = "navigateToPage";
                        _events[AppUtils.EventTypes.mouseDown] = "mouseDownHandler";
                        _events[AppUtils.EventTypes.click] = "clickHandler";
                        return _events;
                    },

                    clickHandler: function (e) {
                        if (!this.mouseDown) {
                            e.stopPropagation();
                            e.preventDefault();
                            this.navigateToPage(e);
                        }
                    },
                    mouseDownHandler: function (e) {
                        this.mouseDown = true;
                        e.stopPropagation();
                        e.preventDefault();
                    },

                    navigateToPage: function (event) {
                        event.stopPropagation();
                        event.preventDefault();
                        var direction = this.model.get("direction");
                        if (!AppUtils.useNativeScroll) {
                            that.restorePageTransitionDuration();
                        }
                        that.scrollingElementsWrapper.getView().navigateToNeighborPage(direction);
                    },


                    removeView: function () {
                        this.remove();
                        this.unbind();
                    },
                    updateForPage: function () {
                        // TODO: we might want to reconsider this for multiple pages visible at a time
                        if (this.model.get("direction") == -1) { // left arrow
                            if (this.model.get("pageNumber") == 0) {
                                this.$el.hide();
                            } else {
                                this.$el.show();
                            }
                        } else if (this.model.get("direction") == 1) { // right arrow
                            if (this.model.get("pageNumber") == that.settings.lastPageNumber - App.segmentLength()) {
                                this.$el.hide();
                            } else if (this.model.get("pageNumber") == 0 && this.model.get("hideNavigationOnFirstPage")) {
                                this.$el.hide();
                            } else {
                                this.$el.show();
                            }
                        }
                    },
                    render: function () {
                        var left = this.model.get("left"),
                            right = this.model.get("right");
                        if (that.settings.useNavigationArrows && !ViewportPadding.isPaddingAreaPossible() && !ProjectGlobals.IS_TIP_OF_THE_DAY) {
                            this.$el.html(this.template(this.model.attributes));
                            this.setBottom(this.model.get("bottom"));
                            if (typeof left != "undefined") {
                                this.setLeft(left);
                            }
                            if (typeof right != "undefined") {
                                this.setRight(right);
                            }
                        }

                        return this;
                    }
                });

            };

            this.instantiateObjects = function (pagesRawModel, activePage, callback) {
                var idx, model;

                that.DEFAULT_TRANSITION_DURATION = parseInt(pagesRawModel.settings.transitionDuration);

                that.scrollingElementsModel = new Backbone.Model({
                    activePage: activePage,
                    nextActivePage: activePage,
                    scrollLeft: 0,
                    translate3DOffset: 0,
                    transitionPeriod: that.DEFAULT_TRANSITION_DURATION,
                    resetCurrentPage: true
                });
                that.scrollingElementsView = new that.ScrollingElementView({
                    model: that.scrollingElementsModel
                });

                that.navigationArrowLeftModel = new Backbone.Model({
                    bottom: 10,
                    left: 10,
                    direction: -1,
                    imageURL: that.settings.leftNavigationArrow
                });

                that.navigationArrowLeftView = new that.NavigationArrowView({
                    model: that.navigationArrowLeftModel
                });

                that.navigationArrowRightModel = new Backbone.Model({
                    bottom: 10,
                    right: 10,
                    direction: 1,
                    imageURL: that.settings.rightNavigationArrow,
                    hideNavigationOnFirstPage: that.settings.hideNavigationOnFirstPage
                });

                that.navigationArrowRightView = new that.NavigationArrowView({
                    model: that.navigationArrowRightModel
                });

                that.pagesModelCollection = new Backbone.Collection(
                );
                for (idx = 0; idx < pagesRawModel.pages.length; idx++) {
                    model = createPageModel(idx, pagesRawModel.pages[idx], idx == 0, idx == pagesRawModel.pages.length - 1);
                    that.pagesModelCollection.add(model);
                }
                that.scrollingElementsWrapper = new ScrollingElementsWrapper();
                that.scrollingElementsWrapper.init();
                that.scrollingElementsWrapper.getView().$el.append(that.scrollingElementsView.render().el);
                App.viewPortView.$el.append(that.navigationArrowLeftView.render().el);
                App.viewPortView.$el.append(that.navigationArrowRightView.render().el);
                if (null != callback) {
                    callback();
                }
            };

            function createPageModel(index, page, isFirstPage, isLastPage) {
                var settings = page.settings,
//                pageWidth = Math.round(settings.resolutionWidth * App.settings.scaleFactor),
                // to be in sync with App.settings
                    pageWidth = App.settings.pageWidth,
                    pageHeight = Math.round(settings.resolutionHeight * App.settings.scaleFactor),
                    model = new Backbone.Model({
                        width: pageWidth,
                        height: pageHeight,
                        absoluteWidth: pageWidth,
                        absoluteHeight: pageHeight,
                        top: 0,
                        left: index * pageWidth,
                        index: index,
                        removed: false,
                        objects: page.objects,
                        // we take these from page settings
                        karaokeModeEnabled: settings["karaokeModeEnabled"],
                        hasMovingLayers: settings["hasMovingLayers"],
                        hideLeftSideMenuArrow: settings.hideLeftSideMenuArrow,
                        hideRightSideMenuArrow: settings.hideRightSideMenuArrow,
                        id: settings['objectId'],
                        isFirstPage: isFirstPage,
                        isLastPage: isLastPage
                    });
                return model;
            }

            this.doFadeTransition = function (fadeTiming, nextPage) {
                console.log("GYL-401: Now do fade transition in " + fadeTiming + "ms");
                var currentPageLeft = this.pagesModelCollection.models[this.activePage].get("left"),
                    nextPageEl = $("#page-" + nextPage),
                    activePageEl = this.activePageElement.$el,
                    browserTransformProperty = CSSHandler.getBrowserPrefixedProperty("transform");
                this.preventLeftUpdateAfterPageChange = true;
                nextPageEl.css("left", currentPageLeft + "px")
                    .css(browserTransformProperty, "translateZ(0)"); // make page HW accelerated before page on top will fade out
                CSSHandler.setTransition(activePageEl, "opacity", fadeTiming);
                CSSHandler.setTransitionEndHandler(activePageEl, function () {
                    console.log("GYL-401: Now set active page after fade transition end " + nextPage);
                    Chapter.setActivePage(nextPage);
                    setTimeout(function () {
                        // this must be executed after z-index was updated for activePageEl !!!
                        activePageEl.css(CSSHandler.getBrowserPrefixedProperty("transition"), "");
                        activePageEl.css("opacity", "");
                        console.log("GYL-401: have set scroll left to " + nextPage * App.settings.pageWidth);
                        if (AppUtils.useNativeScroll) {
                            document.getElementById("slidingScenesWrapper").scrollLeft = nextPage * App.settings.pageWidth;
                            nextPageEl.css("left", (App.settings.pageWidth * nextPage) + "px");
                            setTimeout(function () {
                                nextPageEl.css(browserTransformProperty, ""); // remove HW acceleartion, tested and works fine on iPad 2, iOS 8
                            }, 0);
                        } else {
                            nextPageEl.css("left", (App.settings.pageWidth * nextPage) + "px");
                            that.removePageTransitionDuration();
                            that.scrollingElementsView.model.set("translate3DOffset", -nextPage * App.settings.pageWidth);
                            that.restorePageTransitionDuration();
                        }
                    }, 5000);  // TODO: this should be more precisely determined, but using a lower value like 1000, will cause flickering on EP2
                }, "opacity");
                activePageEl.css("opacity", 0);
            };

            this.doAppearTransition = function (nextPage) {
                console.log("Now do appear transition");
            };

            this.translateScenesTo = function (offset, transitionEndHandler) {
                console.log("translateScenesTo: " + offset);
                // setup what is to be done after page transition
                that.scrollingElementsView.addTransitionEndHandler(transitionEndHandler);
                if (AppUtils.useNativeScroll) {
                    if (this.scrollingElementsModel.get("transitionPeriod") == 0) {
                        console.log("MOB-1718: translateScenesTo with no transition");
                        document.getElementById("slidingScenesWrapper").scrollLeft = offset;
                    } else {
                        require(["PhoneGapUtils"], function () {
                            // some sort of hack here to make Frøet second page load correct !!!
                            //var slidingScenesWrapper = document.getElementById("slidingScenesWrapper");
                            //if (offset > slidingScenesWrapper.scrollLeft) {
                            //    slidingScenesWrapper.scrollLeft += 10;
                            //} else {
                            //    slidingScenesWrapper.scrollLeft -= 10;
                            //}
                            window.plugins.PhoneGapUtils.scrollToOffsetWithAnimation(offset);
                        });
                    }
                } else {
                    that.scrollingElementsModel.set("translate3DOffset", offset);
                }
            };


            this.setActivePage = function (activePage) {
                var oldPage = that.activePage;

                // There are cases when user swipes fast back and forward.
                // In that scenario new active page and old active page are the same
                // For now we just skip doing any changes in this case
                if (oldPage != activePage) {

                    if (activePage > 0 && pontusSound) {
                        console.log("Pontus sound start position for page " + activePage + " is " + parseInt(pontusSound.position));
                        oldPosition = parseInt(pontusSound.position);
                    }

                    that.activePage = activePage;

                    console.log("Will go to page " + activePage);

                    Chapter.stopPlayingMultimediaForPage(oldPage);
                    if (null != that.bottomMenu) {
                        that.bottomMenu.getModel().set('pageNumber', activePage);
                    }

                    var scrollingElementsModelActivePage = that.scrollingElementsModel.get("activePage");

                    if ((scrollingElementsModelActivePage == activePage) ||
                        (App.dualLandscapeActive() && scrollingElementsModelActivePage == activePage + 1)) {
                        /**
                         * Two cases covered here:
                         * 1) going from portrait to dual landscape, where in portrait the active page was the right page, so now we activate left page
                         * 2) going from dual landscape to portrait, and in landscape the active page was the right page, so now we activate left page
                         */
                        that.pagesModelCollection.models[oldPage].set("shouldBeActive", false);
                        that.pagesModelCollection.models[oldPage].set("activatePage", false);
                        that.pagesModelCollection.models[activePage].set("shouldBeActive", true);
                        that.pagesModelCollection.models[activePage].set("activatePage", true);
                        that.scrollingElementsView.activePageModel = that.pagesModelCollection.models[activePage];
                    }

                    // make sure we update pages content and navigate to new page
                    that.scrollingElementsModel.set("activePage", activePage);

                    // we want to update thumbnails and highlight the one for active page
                    if (null != Chapter.getGlobalModels().thumbnailWrapperModel) {
                        Chapter.getGlobalModels().thumbnailWrapperModel.set("activateThumbs", activePage);
                    }

                    // update navigation arrows
                    that.navigationArrowLeftModel.set("pageNumber", activePage);
                    that.navigationArrowRightModel.set("pageNumber", activePage);

                    if (typeof ViewportPadding.leftPaddingAreaNavigationArrowModel != "undefined") {
                        ViewportPadding.leftPaddingAreaNavigationArrowModel.set("pageNumber", activePage);
                        ViewportPadding.rightPaddingAreaNavigationArrowModel.set("pageNumber", activePage);
                    }

                    // update menu icon visibility
                    if (that.bottomMenu) {
                        that.bottomMenu.getMenuIconModel().set("updateVisibility", activePage);
                    }

                    // update recording play icon state
                    if (typeof SoundRecorder != "undefined") {
                        SoundRecorder.updatePlayIconStateForPage(activePage);
                    }
                }

            }
            ;

            this.restorePageTransitionDuration = function () {
                this.setPageTransitionDuration(this.DEFAULT_TRANSITION_DURATION);
            };
            this.setPageTransitionDuration = function (value) {
                this.scrollingElementsModel.set("transitionPeriod", value);
            };
            this.removePageTransitionDuration = function () {
                this.setPageTransitionDuration(0);
            };
            this.updatePersistentCategories = function () {
                var savedCategories = localStorage.getItem(that.settings.localRootSequenceId + "-persistentOC"),
                    categoryId;
                if (null != savedCategories) {
                    console.log("Update persistent categories");
                    savedCategories = JSON.parse(savedCategories);
                    for (categoryId in savedCategories) {
                        if (savedCategories.hasOwnProperty(categoryId) && null != that.objectCategories[categoryId] && that.objectCategories[categoryId].persistent) {
                            this.objectCategories[categoryId].active = savedCategories[categoryId].active;
                        }
                    }

                }
            };
            this.updateRelatedProjectsIds = function (chapterRawModel) {
                var relatedProjectsIds = localStorage.getItem('relatedProjectsIds');
                if (null != relatedProjectsIds) {
                    console.log("Update persistent categories");
                    relatedProjectsIds = JSON.parse(relatedProjectsIds);
                } else {
                    relatedProjectsIds = {};
                }
                if (null == this.getParentProjectIdFor(relatedProjectsIds, that.settings.localRootSequenceId) && null != chapterRawModel.settings.relatedProjectsIds) {
                    relatedProjectsIds[that.settings.localRootSequenceId] = chapterRawModel.settings.relatedProjectsIds[that.settings.localRootSequenceId];
                    localStorage.setItem('relatedProjectsIds', JSON.stringify(relatedProjectsIds));
                }
            };

            this.getParentProjectIdFor = function (relatedProjectsIds, searchedProjectId) {
                var projectId, ret = null;

                for (projectId in relatedProjectsIds) {
                    if (relatedProjectsIds.hasOwnProperty(projectId)) {
                        if (projectId == searchedProjectId) {
                            ret = projectId;
                        }
                        if (null != ret) {
                            return ret;
                        }
                        ret = this.getParentProjectIdFor(relatedProjectsIds[projectId], searchedProjectId);
                        if (null != ret) {
                            ret = projectId;
                            return ret;
                        }
                    }
                }
                return ret;
            };
            this.updateCategoriesFromGlobalCategories = function () {
                var projectCategories = Chapter.getObjectCategories(),
                    globalSavedCategories = localStorage.getItem('globalCategoryActions'),
                    categoryId, globalCategory, relatedProjectsIds, parentProjectObjectId;

                if (null != globalSavedCategories) {
                    relatedProjectsIds = localStorage.getItem('relatedProjectsIds');
                    if (null != relatedProjectsIds) {
                        console.log("Update persistent categories");
                        relatedProjectsIds = JSON.parse(relatedProjectsIds);
                    } else {
                        relatedProjectsIds = {};
                    }
                    parentProjectObjectId = this.getParentProjectIdFor(relatedProjectsIds, that.settings.localRootSequenceId);
                    globalSavedCategories = JSON.parse(globalSavedCategories);
                    for (categoryId in projectCategories) {
                        if (projectCategories.hasOwnProperty(categoryId)) {
                            globalCategory = this.getGlobalCategoryFor(projectCategories[categoryId].name, globalSavedCategories, relatedProjectsIds, parentProjectObjectId);
                            if (null != globalCategory) {
                                projectCategories[categoryId].active = globalCategory.active;
                            }
                        }
                    }
                }
            };

            this.getGlobalCategoryFor = function (currentCategoryName, globalCategories, relatedProjectsIds, currentParentObjectId) {
                var globalCategory, globalCategoryId, globalCategoryProjectId;
                for (globalCategoryId in globalCategories) {
                    if (globalCategories.hasOwnProperty(globalCategoryId)) {
                        globalCategory = globalCategories[globalCategoryId];
                        if (globalCategory.name == currentCategoryName) {
                            globalCategoryProjectId = this.getParentProjectIdFor(relatedProjectsIds, globalCategory.updatedByProjectid);
                            if (globalCategoryProjectId == currentParentObjectId) {
                                return globalCategory;
                            }
                        }
                    }
                }

            };

            this.extractPersistentCategories = function () {
                console.log("Save persistent categories");
                var categoryId, category, persistentCategories = {},
                    persistentCategoryExist = false;
                for (categoryId in this.objectCategories) {
                    if (this.objectCategories.hasOwnProperty(categoryId)) {
                        category = this.objectCategories[categoryId];
                        if (category.persistent) {
                            persistentCategories[categoryId] = category;
                            persistentCategoryExist = true;
                        }
                    }
                }
                return persistentCategoryExist ? persistentCategories : null;
            };

            this.savePersistentCategories = function () {
                if (null != this.persistentCategories) {
                    localStorage.setItem(that.settings.localRootSequenceId + "-persistentOC", JSON.stringify(this.persistentCategories));
                }
            };

            this.backTo = function () {
                App.mql.removeListener(App.handleOrientationChange); // unnecessary to trigger here another orientation change in title that will exit
                if (AppUtils.nativeApp) {

                    var parentOrientations = this.getParentOrientations();
                    localStorage.setItem("backFromBook", "true");
                    if (typeof SoundPlayer != 'undefined') {
                        SoundPlayer.endAllPlayingSounds();
                    }
                    if (AppUtils.DeviceType.ANDROID) {
                        if (typeof DirectJSCalls.backToBookshelf == 'function') {
                            if (typeof DirectJSCalls.setAvailableOrientation == 'function') {
                                if (parentOrientations.portraitModeForBookshelf) {
                                    if (parentOrientations.landscapeModeForBookshelf)
                                        window.DirectJSCalls.setAvailableOrientation('ALL');
                                    else
                                        window.DirectJSCalls.setAvailableOrientation('PORTRAIT');
                                } else {
                                    window.DirectJSCalls.setAvailableOrientation('LANDSCAPE');
                                }
                            }
                            DirectJSCalls.backToBookshelf();
                        } else {
                            history.go(-1);
                        }
                    } else if (AppUtils.DeviceType.iOS) {
                        if (window.backEnabled) {
                            localStorage.setItem("landscapeOrientation", parentOrientations.landscapeModeForBookshelf);
                            localStorage.setItem("portraitOrientation", parentOrientations.portraitModeForBookshelf);
                            require(["PluginUtils"], function () {
                                window.plugins.PluginUtils.recheckScreenOrientation();
                            });
                            setTimeout(function () {
                                window.location = "nativeCode:cmd=backToShelf";
                            }, 100);
                        } else {
                            history.go(-1);
                        }
                    }
                }
                else {
//                if (AppUtils.getLocalStorageBooleanItem("bookChapter", false)) {
                    history.go(-1);
//                } else {
//                    alert("Back to shelf only available in native app");
//                }
                }
            };
            this.getParentOrientations = function () {
                var ret = {
                    landscapeModeForBookshelf: true,
                    portraitModeForBookshelf: true
                };

                if (null == localStorage.getItem('BookLandscapeOrientation')) {
                    if (null != localStorage.getItem('BookShelfLandscapeOrientation')) {
                        ret.landscapeModeForBookshelf = localStorage.getItem('BookShelfLandscapeOrientation');
                    }
                } else {
                    ret.landscapeModeForBookshelf = localStorage.getItem('BookLandscapeOrientation');
                }
                if (null == localStorage.getItem('BookPortraitOrientation')) {
                    if (null != localStorage.getItem('BookShelfPortraitOrientation')) {
                        ret.portraitModeForBookshelf = localStorage.getItem('BookShelfPortraitOrientation');
                    }
                } else {
                    ret.portraitModeForBookshelf = localStorage.getItem('BookPortraitOrientation');
                }
                return ret;
            };

            this.closeBottomMenu = function () {
                if (this.bottomMenu != null)
                    this.bottomMenu.getModel().set('expanded', MenuState.CLOSED);
            };
            return {

                initSettings: function (chapterSettings) {
                    that.settings = JSON.parse(JSON.stringify(chapterSettings));
                },

                init: function (chapterRawModel) {
                    that.pagesRawModel = chapterRawModel.pages;
                    that.embeddedScenes = JSON.parse(JSON.stringify(chapterRawModel.embeddedScenes));
                    if (null != that.settings['moveBackgroundObjectBy']) {
                        that.settings['moveBackgroundObjectBy'] = JSON.parse(that.settings['moveBackgroundObjectBy']);
                    }
                    that.settings.resolutionWidth = Math.round(App.settings.scaleFactor * chapterRawModel.settings.resolutionWidth);
                    that.settings.resolutionHeight = Math.round(App.settings.scaleFactor * chapterRawModel.settings.resolutionHeight);
                    that.objectCategories = chapterRawModel.objectCategories;
                    console.log("On chapter load categories are: " + JSON.stringify(that.objectCategories));
                    that.objectCategoriesForPages = chapterRawModel.objectCategoriesForPages;
                    that.updatePersistentCategories();
                    that.updateRelatedProjectsIds(chapterRawModel);
                    that.updateCategoriesFromGlobalCategories();
                    that.persistentCategories = that.extractPersistentCategories();
                    that.savePersistentCategories();
                    ProjectGlobals.objectCategories = chapterRawModel.objectCategories;
                    AppUtils.updateCssProjectStyle([ProjectSettings.URLs.chapterStylesCssURL()]);

                    // save in settings how many pages we have in a chapter
                    // easier to do this here rather than server side
                    that.settings.lastPageNumber = chapterRawModel.pages.length;
                    that.defineClasses();
                },
                createPages: function (pagesRawModel, activePage, callback) {
                    var scrollingElementsWrapperPath = AppUtils.DeviceType.iOS ? "platforms/iOS/ScrollingElementsWrapper" : "ScrollingElementsWrapper";
                    require([scrollingElementsWrapperPath], function () {
                        that.instantiateObjects(pagesRawModel, activePage, callback);
                    });

                },
                createBottomMenu: function (menuRawModel, activePage, callback) {
                    menuRawModel.thumbs.originalMaxThumbHeight = menuRawModel.thumbs.maxThumbHeight;
                    menuRawModel.thumbs.maxThumbHeight *= AppUtils.getMenuScaleFactor();
                    if (null == that.bottomMenu) {
                        require(['ChapterBottomMenu', 'CSSConstants'], function (ChapterBottomMenu) {
                            that.bottomMenu = new ChapterBottomMenu(menuRawModel, activePage);
                            that.bottomMenu.init(callback);
                        });

                    }
                },
                getModel: function () {
                    return that.chapterModel;
                },
                getScrollingElementsWrapper: function () {
                    return that.scrollingElementsWrapper;
                },
                getScrollingElementsView: function () {
                    return that.scrollingElementsView;
                },
                getBottomMenu: function () {
                    return that.bottomMenu;
                },
                getPagesModelCollection: function () {
                    return that.pagesModelCollection;
                },
                getGlobalModels: function () {
                    return that.globalModels;
                },
                getSettings: function () {
                    return that.settings;
                },
                getEmbeddedScenes: function () {
                    return that.embeddedScenes;
                },
                getMenuModel: function () {
                    return that.menuModel;
                },
                setMenuModel: function (menuModel) {
                    that.menuModel = menuModel
                },
                getObjectCategories: function () {
                    return that.objectCategories;
                },
                getObjectCategoriesForPages: function () {
                    return that.objectCategoriesForPages;
                },
                setActivePage: function (activePage) {
                    that.setActivePage(activePage);
                },
                getActivePage: function () {
                    return that.activePage;
                },
                translateScenesTo: function (activePage, transitionEndHandler) {
                    that.translateScenesTo(activePage, transitionEndHandler);
                },
                trySpecialNavigation: function (customTransitionProperty, nextPage) {
                    var fadeTiming,
                        useCustomTransition = false;
                    if (customTransitionProperty) {
                        if (customTransitionProperty.indexOf("transitionFade-") > -1) {
                            useCustomTransition = true;
                            fadeTiming = customTransitionProperty.substring("transitionFade-".length);
                            that.doFadeTransition(fadeTiming, nextPage);
                        } else if (customTransitionProperty.indexOf("transitionAppear") > -1) {
                            useCustomTransition = true;
                            that.doAppearTransition(nextPage);
                        }
                    }
                    return useCustomTransition;
                },
                computeTranslateOffset: function (currentPage, nextPage) {
                    var translateOffset;
                    if (AppUtils.DeviceType.iOS) {
                        if (AppUtils.useNativeScroll) {
                            translateOffset = App.settings.pageWidth * nextPage;
                        } else {
                            translateOffset = App.settings.pageWidth * (currentPage - nextPage);
                        }
                    } else {
                        translateOffset = App.settings.pageWidth * (-nextPage);
                    }
                    return translateOffset;
                },
                getTranslateOffsetForSamePageOnOrientationChange: function (currentPage) {
                    var translateOffset;
                    if (AppUtils.DeviceType.iOS) {
                        if (AppUtils.useNativeScroll) {
                            translateOffset = App.settings.pageWidth * currentPage;
                        } else {
                            translateOffset = App.settings.pageWidth * currentPage;
                        }
                    } else {
                        translateOffset = App.settings.pageWidth * (-currentPage);
                    }
                    return translateOffset;
                },
                getScrollingElementsModel: function () {
                    return that.scrollingElementsModel;
                },
                getSoundEffects: function () {
                    return AppUtils.getLocalStorageBooleanItem(that.settings.rootProjectId + '-effectSoundsOn', true);
                },
                setSoundEffects: function (enabled) {
                    localStorage.setItem(that.settings.rootProjectId + '-effectSoundsOn', enabled);
                },
                getGlobalSoundsSetting: function () {
                    return AppUtils.getLocalStorageBooleanItem(that.settings.rootProjectId + '-globalSoundsOn', true);
                },
                setGlobalSoundsSetting: function (enabled) {
                    localStorage.setItem(that.settings.rootProjectId + '-globalSoundsOn', enabled);
                },
                getTextVisible: function () {
                    var ret = true,
                        itemValue = localStorage.getItem(that.settings.rootProjectId + '-showTextBoxes');
                    if (itemValue == "off") {
                        ret = false;
                    }
                    return ret;
                },
                setTextVisible: function (visible) {
                    var newValue = visible ? "on" : "off";
                    localStorage.setItem(that.settings.rootProjectId + '-showTextBoxes', newValue);
                    that.scrollingElementsView.sendToggleVisibleTextBoxSignal(visible);
                },
                getReadTextSettings: function () {
                    var ret = ReadSoundSettings.SOUNDS_ON,
                        readTextSettings = localStorage.getItem(that.settings.rootProjectId + '-readTextSettings');
                    if (readTextSettings != null) {
                        ret = readTextSettings;
                    } else if (that.menuModel != null) {
                        // TODO: here for the first page of the chapter menuModel is not available !!!
                        // not a very big problem since sound is only loaded and not played
                        if (that.menuModel.settings.recordingEnabled) {
                            ret = ReadSoundSettings.OWN_SOUNDS;
                        }
                    }
                    return ret;
                },
                setReadTextSettings: function (value) {
                    localStorage.setItem(that.settings.rootProjectId + '-readTextSettings', value);
                },
                removePageTransitionDuration: function () {
                    that.removePageTransitionDuration();
                },
                restorePageTransitionDuration: function () {
                    that.restorePageTransitionDuration();
                },
                setPageTransitionDuration: function (value) {
                    that.setPageTransitionDuration(value);
                },

                addGlobalTimer: function (timerObject) {
                    that.globalTimers.push(timerObject);
                },
                removeScrollingElementTransitionEndHandler: function () {
                    if (!AppUtils.useNativeScroll) {
                        that.scrollingElementsView.removeTransitionEndHandler();
                    }
                },
                cancelGlobalTimers: function () {
                    var idx;
                    console.log("Will cancel all global timers " + that.globalTimers.length);
                    for (idx = 0; idx < that.globalTimers.length; idx++) {
                        clearTimeout(that.globalTimers[idx]);
                    }
                    that.globalTimers = [];
                },
                getPageNumberForSceneId: function (sceneId) {
                    var idx;
                    console.log("getPageNumberForSceneId");
                    for (idx = 0; idx < that.pagesRawModel.length; idx++) {
                        if (that.pagesRawModel[idx].settings.objectId == sceneId) {
                            return idx;
                        }
                    }
                    return null;
                },
                getBottomMenuCustomActiveTabsCollection: function () {
                    return that.bottomMenuCustomActiveTabsCollection;
                },
                backTo: function () {
                    that.backTo();
                },
                closeBottomMenu: function () {
                    that.closeBottomMenu();
                },
                getScrollingElementXOffset: function () {
                    return CSSHandler.getTranslationOffset(CSSHandler.getMatrix(that.scrollingElementsView.$el)).xOffset;
                },
                setAppPaused: function (value) {
                    that.appPaused = value;
                },
                isAppPaused: function () {
                    return that.appPaused;
                },
                setActivePageElement: function (activePageElement) {
                    that.activePageElement = activePageElement;
                },
                getActivePageElement: function () {
                    return that.activePageElement;
                },
                activateNextPageOnSoundFinish: function () {
                    that.pagesModelCollection.models[that.activePage].set("activatePage", false);
                    that.pagesModelCollection.models[that.activePage].set("shouldBeActive", false);
                    that.pagesModelCollection.models[that.activePage + 1].set("activatePage", true);
                    that.scrollingElementsView.activePageModel = that.pagesModelCollection.models[that.activePage + 1];
                    that.activePage += 1;
                },
                stopPlayingMultimediaForPage: function (pageNumber) {
                    if (typeof pageNumber == "number") {
                        if (typeof SoundPlayer != "undefined") {
                            SoundPlayer.endAllPlayingSoundsForPage(pageNumber, false);
                            if (App.dualLandscapeActive()) {
                                SoundPlayer.endAllPlayingSoundsForPage(pageNumber + 1, false);
                            }
                        }
                        if (typeof VideoPlayer != "undefined") {
                            VideoPlayer.stopAllPlayingVideosForPage(that.pagesModelCollection.models[pageNumber]);
                            if (App.dualLandscapeActive()) {
                                VideoPlayer.stopAllPlayingVideosForPage(that.pagesModelCollection.models[pageNumber + 1]);
                            }
                        }
                    }
                }
            }
        }
    }
);