var HTTPRequestUtils = function () {
    var instance;

    function init() {
        var ret = {};
        ret.doRequest = doXmlHttpRequest;
        ret.getJSON = getJSON;
        ret.doPostRequest = doPostRequest;
        return ret;
    }

    return {
        getInstance: function () {
            if (null == instance) {
                instance = init();
            }
            return instance;
        }
    };


    /**
     * Make a server request
     * @param url Server URL where the request is made
     * @param verb Represent request type ("POST", "GET")
     * @param params An array of parameters that will be sent in the request
     * @param contentType
     * @param acceptContentType
     * @param successCallback
     * @param errorCallback
     * @param optional
     */
    function doXmlHttpRequest(url, verb, params, contentType, acceptContentType, successCallback, errorCallback, optional) {
        console.log("doXmlHttpRequest called on bookUtils.js");
        try {
            var httpRequest = new XMLHttpRequest();
            httpRequest.open(verb, url, true);
            httpRequest.withCredentials = true;
            httpRequest.setRequestHeader("Content-type", contentType);
            httpRequest.setRequestHeader("Cache-Control", "no-cache");
            httpRequest.setRequestHeader("Pragma", "no-cache");
            httpRequest.setRequestHeader("Accept", acceptContentType);
            httpRequest.onreadystatechange = function () {
                if (this.readyState == 4) {
                    if (this.status == 200) {
                        if (null != successCallback) {
                            successCallback(this.responseText);
                        }
                    } else {
                        if (null != errorCallback) {
                            errorCallback();
                        }
                    }
                }
            };

            httpRequest.send(params);
        }
        catch
            (e) {
            console.log("Exception caught in doXmlHttpRequest function in HTTPRequestUtils file" + e.message);
        }
    }

    function doPostRequest(url, data, successCallback, failCallback, dataType) {
        $.post(url, data, successCallback, dataType)
            .fail(function () {
                if (failCallback) {
                    failCallback();
                }
            });
    }

    /**
     * Used to read JSON file form server of from local file
     * @param url The URL where the JSON file is located
     * @param successCallback Callback executed if the JSON file was successfully read
     * @param errorCallback Callback executed if the JSON could not be read
     */
    function getJSON(url, successCallback, errorCallback) {
        $.getJSON(url, function () {
            console.log("success");
        })
            .done(function (data) {
                console.log("second success");
                successCallback(data);
            })
            .fail(function (error) {
                console.log("error");
                errorCallback(error);
            })
            .always(function () {
                console.log("complete");
            });
    }

}();