/**
 * Created by ionutmovila on 04/12/14.
 */

function MessagingAPI() {
    var iFrameActions = {
            "CHANGE_PAGE": "changePageHandler",
            "PLAY_SOUND": "playSoundHandler",
            "STOP_SOUND": "stopSoundHandler",
            "OC_UPDATE": "ocUpdateHandler",
            "OPEN_WEB_URL": "openWebURL"
        },
        soundObjects = {},
        privateMethods = {
            init: function () {
                console.log("init messaging");
                window.addEventListener("message", function (e) {
                    var messageData = e.data,
                        packageMessageContent,
                        iFrameAction;
                    if (!isNaN(messageData)) { // for now this is the only functionality that does not comply with the messaging API
                        Chapter.setActivePage(e.data)
                    } else if (typeof messageData == "object") { // this complies with messaging API
                        if ("type" in messageData) {
                            require(["CustomMessages"], function () {
                                if (messageData["type"] == "PACKAGE_MESSAGE") {
                                    packageMessageContent = messageData["message"];
                                    console.log("Will handle message from an internal package " + JSON.stringify(packageMessageContent));
                                    iFrameAction = packageMessageContent["action"];
                                    if (typeof customMessages == "object" && customMessages.hasCustomAction(iFrameAction)) {
                                        console.log("Action handled by custom handler " + iFrameAction);
                                        customMessages[customMessages.getCustomActionHandler(iFrameAction)](packageMessageContent["params"]);
                                    } else if (iFrameAction in iFrameActions) {
                                        console.log("Action handled by default handler " + iFrameAction);
                                        privateMethods[iFrameActions[iFrameAction]](packageMessageContent["params"]);
                                    } else {
                                        console.log("iFrame action not defined");
                                    }
                                }
                            });
                        } else {
                            console.log("Received message with unknown type")
                        }
                    } else {
                        console.log("No handler for received message " + messageData);
                    }
                }, false);
            },
            changePageHandler: function (args) {
                var pageNumber = args["pageNumber"];
                console.log("changePageHandler");
                AppUtils.navigateToPage(pageNumber);
            },
            playSoundHandler: function (args) {
                var soundURL = args["soundURL"],
                    soundId = args["soundId"];
                if (AppUtils.nativeApp) {
                    soundObjects[soundId] = new Media(soundURL, function () {
                        console.log("Finished play");
                    }, function (errorCode) {
                        console.log("Play error, code is " + errorCode);
                    });
                    soundObjects[soundId].play();
                } else {
                    pontusSound = soundManager.createSound({
                        id: soundId,
                        url: soundURL,
                        autoLoad: true,
                        autoPlay: true,
                        volume: 10
                    });
                }
            },
            stopSoundHandler: function (args) {
                var soundId = args["soundId"];
                if (AppUtils.nativeApp) {
                    if (soundId in soundObjects) {
                        soundObjects[soundId].stop();
                        soundObjects[soundId].release();
                        delete soundObjects[soundId];
                    }
                } else {
                    soundManager.destroySound(soundId);
                }
            },
            ocUpdateHandler: function (args) {
                var ocId, ocAction, idx;
                console.log("Now handle OC update " + JSON.stringify(args));
                var activatedCategories = {}, deactivatedCategories = {}, transportObject = {};
                for (idx = 0; idx < args.length; idx++) {
                    ocId = args[idx]["ocId"];
                    ocAction = args[idx]["ocAction"];
                    OCUtils.extractActivatedCategoryFrom([ocId], ocAction.toLowerCase(), activatedCategories, deactivatedCategories);
                }
                transportObject.activatedCategories = activatedCategories;
                transportObject.deactivatedCategories = deactivatedCategories;
                Chapter.getScrollingElementsModel().set("toggledCategories", transportObject);
            },
            openWebURL: function (args) {
                var url = args["webURL"];
                console.log("Now handle openWebURL " + JSON.stringify(args));
                AppUtils.openURL(url);
            }
        },
        publicMethods = {
            ocUpdateHandler: privateMethods.ocUpdateHandler,
            pauseSoundWithId: function (soundId) {
                if (AppUtils.nativeApp) {
                    if (soundId in soundObjects) {
                        soundObjects[soundId].pause();
                    }
                } else {
                    soundManager.getSoundById(soundId).pause();
                }
            },
            playSoundWithId: function (soundId) {
                if (AppUtils.nativeApp) {
                    if (soundId in soundObjects) {
                        soundObjects[soundId].play();
                    }
                } else {
                    soundManager.getSoundById(soundId).play();
                }
            },
            seekNewPositionForSoundWithId: function (soundId, newPosition) {
                if (AppUtils.nativeApp) {
                    if (soundId in soundObjects) {
                        console.log("Now seek to " + newPosition);
                        soundObjects[soundId].seekTo(newPosition);
                    }
                } else {
                    soundManager.setPosition(soundId, newPosition);
                }
            }
        };

    privateMethods.init();

    return publicMethods;
}

var messagingAPI = new MessagingAPI();

var pontusSound = null,
    oldPosition = 0;
