function ScrollingElementsWrapper() {
    var that = this;
    this.defineClasses = function () {
        that.ScrollingElementWrapperView = BaseView.extend({
            tagName: "div",
            id: "slidingScenesWrapper",
            touchStartTime: 0,
            leftValueOnMouseDown: 0,
            position: {},
            minDistanceForChanging: App.settings.width / 16,
            currentPosition: 0,
            swipeMaxPeriod: 300,
            swipeMinDistance: 50,
            direction: 0,
            lastValueToMoveOn: 0,
            lastMovedValue: 0,
            shouldMove: true,
            pageInTransition: false,
            className:"viewPortBackgroundColorClass",

            initialize: function (attr) {
                this.listenTo(this.model, "change:scrollLeft", this.updateLeftPosition);
                this.listenTo(this.model, "change:orientation", this.updateOrientation);
            },
            events: function () {
                var _events = {};
                _events[AppUtils.EventTypes.mouseDown] = "mouseDownHandler";
                _events[AppUtils.EventTypes.mouseUp] = "mouseUpHandler";
                _events[AppUtils.EventTypes.mouseMove] = "mouseMoveHandler";
                _events[AppUtils.EventTypes.mouseLeave] = "mouseUpHandler";
                if (AppUtils.DeviceType.MOBILE_DEVICE) {
                    _events["touchcancel"] = "touchCancelHandler";
                }
                return _events;
            },
            touchCancelHandler: function (e) {
                console.log("ScrollingElementsWrapper touchcancelHandler");
                this.mousePressed = false;
                this.revertToCurrentPage();
            },

            mouseDownHandler: function (e) {
                if (e.button != 2) { // discard right click, that we use for debugging
                    console.log("ScrollingElementsWrapper mouseDownHandler " + e.type);
                    this.stopAnimation();
                    Chapter.removePageTransitionDuration();
                    var sl = Chapter.getScrollingElementXOffset();
                    // initial translate
                    Chapter.getScrollingElementsModel().set("translate3DOffset", sl);
                    this.lastValueToMoveOn = sl;
                    this.lastMovedValue = sl;
                    this.leftValueOnMouseDown = this.lastValueToMoveOn;
                    console.log("mouseDownHandler:leftValueOnMouseDown " + this.leftValueOnMouseDown);
                    this.position = this.getPosition(e);
                    this.touchStartTime = new Date();
                    this.mousePressed = true;
                    Chapter.closeBottomMenu();
                }
            },
            mouseMoveHandler: function (e) {
                e.preventDefault();
                if (this.model.get("pinchInProgress")) {
                    this.touchCancelHandler(null);
                }
                if (this.mousePressed) {
                    console.log("ScrollingElementsWrapper mouseMoveHandler " + e.type);
                    var nextX = this.getPosition(e).x,
                        offset = this.position.x - nextX;
                    this.position.x = nextX;
                    this.lastValueToMoveOn = parseInt(this.lastValueToMoveOn) - offset;
                    if (this.lastValueToMoveOn <= 0 && this.lastValueToMoveOn >= App.maxTranslateValueForLastPage()) {
                        console.log("mouseMoveHandler " + this.lastValueToMoveOn);
                        var thisView = this;
                        if (this.shouldMove) {
                            this.shouldMove = false;
                            window.requestAnimationFrame(function (timeStamp) {
                                if (thisView.mousePressed) {
                                    Chapter.getScrollingElementsModel().set("translate3DOffset", thisView.lastValueToMoveOn);
                                    thisView.lastMovedValue = thisView.lastValueToMoveOn;
                                }
                                thisView.shouldMove = true;
                            }, this.el);
                        }
                    } else {
                        console.log("translateTo: Skip navigating past last page " + this.lastValueToMoveOn);
                    }
                }
            },
            mouseUpHandler: function (e) {
                if (this.mousePressed) {
                    console.log("ScrollingElementsWrapper mouseUpHandler " + e.type);
                    Chapter.restorePageTransitionDuration();
                    this.touchEndTime = new Date();
                    this.updatedLeftValueOnMouseUp = this.lastMovedValue;
                    if (this.tapEvent(e)) {
                        this.revertToCurrentPage();
                    } else if (this.isSwipeEvent() || this.shouldNavigateToNextPage()) {
                        this.navigateToNextPage();
                    } else if (this.updatedLeftValueOnMouseUp != this.leftValueOnMouseDown) {
                        this.revertToCurrentPage();
                    }

                    console.log("mouseUpHandler:leftValueOnMouseDown " + this.leftValueOnMouseDown);
                    console.log("mouseUpHandler:updatedLeftValueOnMouseUp " + this.updatedLeftValueOnMouseUp);
                    this.mousePressed = false;
                }
            },

            isSwipeEvent: function () {
                console.log("distance: " + Math.abs(this.leftValueOnMouseDown - this.updatedLeftValueOnMouseUp));
                console.log("time: " + (this.touchEndTime.getTime() - this.touchStartTime.getTime()));

                return (Math.abs(this.leftValueOnMouseDown - this.updatedLeftValueOnMouseUp) > this.swipeMinDistance) && ((this.touchEndTime.getTime() - this.touchStartTime.getTime()) < this.swipeMaxPeriod);
            },
            shouldNavigateToNextPage: function () {
                var navigateDistance = Math.abs(this.leftValueOnMouseDown - this.updatedLeftValueOnMouseUp);
                console.log("shouldNavigateToNextPage: navigate distance is " + navigateDistance);
                return navigateDistance > this.minDistanceForChanging;
            },
            navigateToNextPage: function () {
                var nextPos, thisView = this;
                if (this.leftValueOnMouseDown < this.updatedLeftValueOnMouseUp) {
                    nextPos = this.currentPosition + this.model.get("width");

                } else {
                    nextPos = this.currentPosition - this.model.get("width");
                }
                nextPos = Math.max(nextPos, App.maxTranslateValueForLastPage());
                nextPos = Math.min(nextPos, 0);
                this.currentPosition = nextPos;
                ZoomUtils.resetZoom();
                console.log("navigateToNextPage: start translating to nextPos " + nextPos);
                setTimeout(function () {
                    thisView.pageInTransition = true;
                    Chapter.translateScenesTo(nextPos, function (event) {
                        console.log("navigateToNextPage:transitionEnd");
                        event.stopPropagation();
                        Chapter.getScrollingElementsView().removeTransitionEndHandler();
                        Chapter.setActivePage(thisView.getActivePage(nextPos));
                        thisView.pageInTransition = false;
                        // TODO: remove the bellow hack, that is no longer required when using XWalk web view on Android !!!
                        setTimeout(function () {
                            thisView.el.scrollLeft = 1;
                            thisView.el.scrollLeft = 0;
                        }, 0);
                    });

                }, 0);

                if(AppUtils.DeviceType.IE9 == true) {
                    Chapter.setActivePage(thisView.getActivePage(nextPos));
                    thisView.pageInTransition = false;
                }
            },

            navigateToNeighborPage: function (direction) {
                this.leftValueOnMouseDown = Chapter.getScrollingElementXOffset();
                this.updatedLeftValueOnMouseUp = this.leftValueOnMouseDown - direction;
                this.direction = direction;
                that.scrollingElementsWrapperView.navigateToNextPage();
            },

            revertToCurrentPage: function () {
                var thisView = this;
                console.log("revertToCurrentPage: start translating to this.currentPosition" + this.currentPosition);
                if (Chapter.getScrollingElementXOffset() != this.currentPosition) {
                    thisView.pageInTransition = true;
                    Chapter.translateScenesTo(this.currentPosition, function () {
                        console.log("revertToCurrentPage:transitionEnd" + (-thisView.currentPosition));
                        thisView.updateLeftPosition();
                        thisView.pageInTransition = false;
                    });
                } else {
                    console.log("Nothing to translate");
                }
            },

            getActivePage: function (nextPosition) {
                var ret;
                if (App.dualLandscapeActive()) {
                    ret = Math.abs(Math.round(nextPosition * 2 / this.model.get("width")));
                } else {
                    ret = Math.abs(Math.round(nextPosition / this.model.get("width")));
                }
                return ret
            },

            removeView: function () {
                this.remove();
                this.unbind();
            },
            render: function () {
                this.$el.html();
                if (ProjectGlobals.IS_SCENE_PREVIEW) {
                    this.$el.css('overflow-x', 'hidden');
                }

                this.$el.css('width', App.settings.pageWidth);
                this.$el.css('height', App.settings.pageHeight);

                return this;
            },
            updateLeftPosition: function () {
                this.lastValueToMoveOn = -this.model.get("scrollLeft");
                this.lastMovedValue = this.lastValueToMoveOn;
                this.currentPosition = this.lastValueToMoveOn;
                console.log("updateLeftPosition" + this.currentPosition);
                if (this.currentPosition % App.settings.width != 0) {
                    console.log("Damn bug with page swipe");
                }
            },
            updateOrientation: function () {
                this.model.set("width", App.settings.pageWidth);
                this.model.set("height", App.settings.pageHeight);
            }
        });
    };

    this.instantiateObjects = function () {
        that.scrollingElementsWrapperModel = new Backbone.Model({
            width: App.settings.pageWidth,
            height: App.settings.pageHeight,
            scrollLeft: 0, // keep this for compatibility with alternative impl
            currentPosition: 0
        });
        that.scrollingElementsWrapperView = new that.ScrollingElementWrapperView({
            model: that.scrollingElementsWrapperModel
        });

    };

    return {
        init: function () {
            that.defineClasses();
            that.instantiateObjects();
            that.scrollingElementsWrapperView.render();
        },
        getModel: function () {
            return that.scrollingElementsWrapperModel;
        },
        getView: function () {
            return that.scrollingElementsWrapperView;
        }
    }
}