define(['BottomMenuOcHandler', 'Templates'], function (OCHandler) {
        return function (bottomMenuRawModel, actionAreaHeight, bottomMenuModel) {

            var MENU_ITEMS_NO_CONTENT = ["back_button", "game_tab", "mute_button", "text_toggle_button", "oc_button", "notes_tab", "go_to_previous_chapter", "go_to_next_chapter"],
                CUSTOM_MENU_TABS_VIEW = ['oc_button'];

            var that = this,
                activeTabDefaultHeight = 30,
                navigateToNeighbourChapterButtonDefaultWidth = 165,
                navigateTroughProjects = false,
                activeTabsModels = getUpdatedActiveTabs(bottomMenuRawModel.settings.activeTabs),
                totalActiveTabsNumber = activeTabsModels.length,
                createdTabsNumber = 0,
                pairsGameContainer = null;


            this.defineClasses = function () {

                that.TabContentView = BaseView.extend({
                    tagName: "div",
                    id: "bottomMenuTabContentArea",
                    template: templates.getTemplate('bottomMenuTabContentArea'),

                    initialize: function (attr) {
                    },

                    events: function () {
                        var _events = {};
                        _events[AppUtils.EventTypes.mouseDown] = "preventPageChange";
                        return _events;
                    },
                    preventPageChange: function (e) {
//                    e.preventDefault();
                    },

                    removeView: function () {
                        this.remove();
                        this.unbind();
                    },
                    expandCollapse: function () {
                    },
                    addDefaultMenuBackgroundIfNeeded: function () {
                        var customMenuProperties = bottomMenuRawModel['customMenuModel'],
                            customMenuType = customMenuProperties['CUSTOM_MENU_TYPE'],
                            mainMenuBackground = $("#mainMenuBackground", this.el),
                            mainMenuRepeatBackground = $("#mainMenuRepeatBackground", this.el);
                        if (!this.isEmpty(customMenuProperties) && customMenuType != "NONE") {
                            mainMenuBackground.removeClass("mainMenuBackgroundClass");
                            mainMenuRepeatBackground.removeClass("mainMenuRepeatBackgroundClass");
                        } else {
                            mainMenuBackground.addClass("mainMenuBackgroundClass");
                            mainMenuRepeatBackground.addClass("mainMenuRepeatBackgroundClass");
                        }
                    },
                    isEmpty: function (obj) {

                        // null and undefined are "empty"
                        if (obj == null) return true;

                        // Assume if it has a length property with a non-zero value
                        // that that property is correct.
                        if (obj.length > 0)    return false;
                        if (obj.length === 0)  return true;

                        // Otherwise, does it have any properties of its own?
                        // Note that this doesn't handle
                        // toString and toValue enumeration bugs in IE < 9
                        for (var key in obj) {
                            if (obj.hasOwnProperty(key)) return false;
                        }

                        return true;
                    },
                    render: function () {
                        this.$el.prepend(this.template());
                        this.addDefaultMenuBackgroundIfNeeded();
                        this.setHeight(this.model.get("height"));
                        return this;
                    }

                });

                that.ActiveTabView = BaseView.extend({
                    className: "chapterBottomMenuTabClass centerAlign",
                    template: templates.getTemplate('activeTab'),
                    tagName: "div",
                    initialize: function (attr) {
                        this.listenTo(this.model, "change:removed", this.removeView);
                        this.listenTo(this.model, "change:active", this.activateTab);
                    },
                    activateTab: function () {
                        if (this.model.get("active") == true) {
                            this.$el.find(".menuIconHighlight").show();
                        } else {
                            this.$el.find(".menuIconHighlight").hide();
                        }
                    },
                    removeView: function () {
                        this.remove();
                        this.unbind();
                    },
                    events: function () {
                        var _events = {};
                        _events[AppUtils.EventTypes.mouseDownSafeIOS5] = "setActiveTab";
                        return _events;
                    },

                    setActiveTab: function () {
                        var tabName = this.model.get("name");
                        if (MENU_ITEMS_NO_CONTENT.indexOf(tabName) == -1) {
                            displayActiveTabContent(tabName);
                            for (var idx = 0; idx < that.activeTabsCollection.models.length; idx++) {
                                that.activeTabsCollection.models[idx].set("active", false);
                            }
                            this.model.set("active", true);
                        } else {
                            if ('back_button' == tabName) {
                                Chapter.backTo();
                            } else if ('game_tab' == tabName) {
                                this.showPairsGame();
                            } else if ('notes_tab' == tabName) {
                                this.showNotes();
                            } else if ('mute_button' == tabName) {
                                this.toggleMuteSounds();
                            } else if ('text_toggle_button' == tabName) {
                                this.toggleTextShow();
                            }

                        }
                    },

                    toggleMuteSounds: function () {
                        var soundsEnabled = Chapter.getGlobalSoundsSetting();
                        console.log("toggleMuteSounds " + soundsEnabled);
                        if (soundsEnabled) { // sounds on, will turn off
                            SoundPlayer.endAllPlayingSoundsForPage(Chapter.getActivePage(), false);
                            this.$el.find(".images").attr("src", this.model.get("toggleIcon"));
                        } else { // sounds off will turn on
                            this.$el.find(".images").attr("src", this.model.get("imageURL"));
                        }
                        Chapter.setGlobalSoundsSetting(!soundsEnabled);
                    },

                    toggleTextShow: function () {
                        var textVisible = Chapter.getTextVisible();
                        Chapter.setTextVisible(!textVisible);
                        if (textVisible) {
                            console.log("toggleShowText disable");
                            this.$el.find(".images").attr("src", this.model.get("toggleIcon"));
                        } else {
                            console.log("toggleShowText enable");
                            this.$el.find(".images").attr("src", this.model.get("imageURL"));
                        }
                    },

                    showNotes: function () {
                        require(['moment', 'HashCode', 'notes/inputTextBox', 'notes/projectNotes', 'notes/projectNotesTemplate'], function () {
                            Chapter.getBottomMenu().getModel().set('expanded', MenuState.CLOSED);
                            projectNotes.show();
                        });
                    },

                    showPairsGame: function () {
                        if (!pairsGameContainer) {
                            pairsGameContainer = $("<div>").addClass("pairsGameContainer");
                            pairsGameContainer.appendTo("#viewPort");
                        }
                        pairsGameContainer.html("<iframe id='pairsGameContainer' width='100%' height='100%'  src='games/default/pairs-plugin-divcontent.html'>");
                        pairsGameContainer.show();
                    },

                    adjustMenuBarButtons: function () {
                        if (this.model.get("name") == "mute_button" && !Chapter.getGlobalSoundsSetting()) {
                            this.$el.find(".images").attr("src", this.model.get("toggleIcon"));
                        } else if (this.model.get("name") == "text_toggle_button" && !Chapter.getTextVisible()) {
                            this.$el.find(".images").attr("src", this.model.get("toggleIcon"));
                        }
                    },

                    render: function () {
                        var $imagesEl;
                        this.$el.html(this.template(this.model.attributes));
                        this.adjustMenuBarButtons();
                        this.setHeight(CSSConstants.ActiveTabViewHeight());
                        this.setWidth(100 / this.model.get("visibleTabsNumber") + "%");

                        $imagesEl = this.$el.find(".images");
                        CSSHandler.setHeight($imagesEl, CSSConstants.ActiveTabViewImageHeight());
                        return this;
                    }
                });
                that.CustomTabView = that.ActiveTabView.extend({
                    className: "chapterBottomMenuTabClass centerAlign",
                    previousChapterTextWidth: 235,
                    nextChapterTextWidth: 179,
                    defaultPercentageChapterTextWidth: 70,
                    neighbourChapterInfo: null,
                    itemDisabled: false,
                    $neighbourPageNavigationElement: null,
                    $neighbourChapterNavigationElement: null,

                    initialize: function (attr) {
                        this.listenTo(this.model, "change:removed", this.removeView);
                        this.listenTo(this.model, "change:active", this.activateTab);
                        this.listenTo(this.model, "change:moveToNeighbourChapter", this.activateMoveToNeighbourChapterArrow);
                        this.listenTo(this.model, "change:pageNumber", this.updatePageNumberPageNavigationArrows);
                    },

                    events: function () {
                        var _events = {};
                        _events[AppUtils.EventTypes.mouseDownSafeIOS5 + ' .goToNeighbourChapterArrowButton'] = "goToNeighbourChapter";
                        _events[AppUtils.EventTypes.mouseDownSafeIOS5 + ' .goToNeighbourChapterTextButtonWrapper'] = "goToNeighbourPage";

                        return _events;
                    },
                    goToNeighbourChapter: function () {
                        var tabName = this.model.get('name');
                        if ('go_to_previous_chapter' == tabName) {
                            this.navigateToNeighbourChapter(-1);
                        }
                        else if ('go_to_next_chapter' == tabName) {
                            this.navigateToNeighbourChapter(1);
                        }

                    },

                    goToNeighbourPage: function () {
                        var scrollingElementsWrapperView = Chapter.getScrollingElementsWrapper().getView(),
                            canNavigate = true,
                            tabName = this.model.get('name');

                        if (typeof SoundRecorder != "undefined" && SoundRecorder.operationMode == SoundRecorderOperationMode.MULTIPLE_PAGE) {
                            canNavigate = false;
                        }
                        if (scrollingElementsWrapperView.pageInTransition) {
                            canNavigate = false;
                        }

                        if (!this.pageNavigationItemDisabled || !canNavigate) {
                            if ('go_to_previous_chapter' == tabName) {
                                scrollingElementsWrapperView.navigateToNeighborPage(-1);
                            }
                            else if ('go_to_next_chapter' == tabName) {
                                scrollingElementsWrapperView.navigateToNeighborPage(1);
                            }
                        }

                    },

                    updatePageNumberPageNavigationArrows: function () {
                        var tabName = this.model.get('name');
                        var chapterSettings = Chapter.getSettings();
                        if (tabName == 'go_to_next_chapter') { // right arrow
                            if (this.model.get("pageNumber") == 0 && chapterSettings.hideNavigationOnFirstPage) {
                                this.$neighbourPageNavigationElement.addClass("disabled");
                                this.pageNavigationItemDisabled = true;
                            } else if (this.model.get("pageNumber") == App.maxActivePageNumberForDualLandscape()) {
                                this.pageNavigationItemDisabled = true;
                                this.$neighbourPageNavigationElement.addClass("disabled");
                            } else {
                                this.pageNavigationItemDisabled = false;
                                this.$neighbourPageNavigationElement.removeClass("disabled");
                            }
                        } else if (tabName == 'go_to_previous_chapter') { // left arrow
                            if (this.model.get("pageNumber") == 0) {
                                this.pageNavigationItemDisabled = true;
                                this.$neighbourPageNavigationElement.addClass("disabled");
                            } else {
                                this.pageNavigationItemDisabled = false;
                                this.$neighbourPageNavigationElement.removeClass("disabled");
                            }
                        }
                    },

                    activateMoveToNeighbourChapterArrow: function () {
                        var moveToNeighbourChapter = this.model.get('moveToNeighbourChapter'), tabName = this.model.get("name");
                        if (null != moveToNeighbourChapter) {
                            this.nextChapterInfo = moveToNeighbourChapter;
                            if (moveToNeighbourChapter.direction < 0 && tabName == 'go_to_previous_chapter') {
                                this.pulsateCustomTab();
                            } else if (moveToNeighbourChapter.direction > 0 && tabName == 'go_to_next_chapter') {
                                this.pulsateCustomTab();
                            }
                            this.model.set('moveToNeighbourChapter', null);
                        }

                    },
                    pulsateCustomTab: function () {
                        var thisView = this;
                        this.setOpacity(1);
                        this.$el.addClass('pulsateNavigationArrow');
                        setTimeout(function () {
                            thisView.$el.removeClass('pulsateNavigationArrow');
                            thisView.setOpacity(1);
                        }, 3000);
                    },

                    template: function (tabName) {
                        var template;
                        if ('go_to_previous_chapter' == tabName) {
                            template = templates.getTemplate('goToPreviousChapter')
                        }
                        else if ('go_to_next_chapter' == tabName) {
                            template = templates.getTemplate('goToNextChapter')
                        }
                        return template(this.model.attributes);
                    },
                    tagName: "div",

                    activateTab: function () {
                    },
                    removeView: function () {
                        this.remove();
                        this.unbind();
                    },
                    removeOldViewportAndLoadNewChapter: function (url, activePage) {
                        ViewportPadding.removeAllPaddings();
                        // now loading project for the new orientation
                        if (typeof SoundPlayer != 'undefined') {
//            SoundPlayer.endAllPlayingSounds();
                        }
                        require(['ChapterElement'], function (ChapterElement) {
                            if (null != Chapter.getScrollingElementsModel()) {
                                Chapter.getScrollingElementsModel().set('remove', true);
                            }

                            var bottomMenuView = Chapter.getBottomMenu().getView();
                            bottomMenuView.remove();
                            bottomMenuView.unbind();

                            App.viewPortView.remove();
                            App.viewPortView.unbind();

                            setupAppBasicFunctions(ChapterElement);
                            window.location.replace(url);
                        });
                    },

                    navigateToNeighbourChapter: function (direction) {
                        if (!this.neighbourChapterInfo.itemDisabled) {
                            if (direction < 0) {
                                var activePage = this.neighbourChapterInfo.previousChapter.lastPageNumber - this.neighbourChapterInfo.previousChapter.firstPageNumber;
                                localStorage.setItem('currentProjectIndex', this.neighbourChapterInfo.currentProjectIndex - 1);
                                if (activePage < 0) {
                                    activePage = 0;
                                }
                                this.removeOldViewportAndLoadNewChapter(this.neighbourChapterInfo.previousChapter.url, activePage);
                            } else {
                                localStorage.setItem('currentProjectIndex', this.neighbourChapterInfo.currentProjectIndex + 1);
                                this.removeOldViewportAndLoadNewChapter(this.neighbourChapterInfo.nextChapter.url, 0);
                            }
                        }
                    },
                    setNeighbourChapterInfo: function () {
                        var bookId = Chapter.getSettings().bookId,
                            bookInfo = localStorage.getItem('bookInfo_' + bookId),
                            currentProjectIndex = localStorage.getItem('currentProjectIndex'),
                            bookChapters, previousChapter, nextChapter,
                            tabName = this.model.get('name');

                        if (null != bookInfo) {
                            currentProjectIndex = parseInt(currentProjectIndex);
                            bookInfo = JSON.parse(bookInfo);
                            bookChapters = bookInfo.chapters;
                            previousChapter = bookChapters[currentProjectIndex - 1];
                            nextChapter = bookChapters[currentProjectIndex + 1];
                        }
                        this.neighbourChapterInfo = {};
                        this.neighbourChapterInfo.currentProjectIndex = currentProjectIndex;
                        this.neighbourChapterInfo.previousChapter = previousChapter;
                        this.neighbourChapterInfo.nextChapter = nextChapter;
                        if ('go_to_previous_chapter' == tabName) {
                            this.neighbourChapterInfo.itemDisabled = (null == previousChapter || !previousChapter.downloaded);
                        } else if ('go_to_next_chapter' == tabName) {
                            this.neighbourChapterInfo.itemDisabled = (null == nextChapter || !nextChapter.downloaded);
                        }
                        return true;
                    },

                    render: function () {
                        var $imagesEl,
                            horizontalMargins = this.model.get("horizontalMargins"),
                            tabName = this.model.get("name");
                        this.setNeighbourChapterInfo();
                        this.$el.html(this.template(tabName));
                        this.$neighbourPageNavigationElement = $('.goToNeighbourChapterTextButtonWrapper', this.el);
                        this.$neighbourChapterNavigationElement = $('.goToNeighbourChapterArrowButtonWrapper', this.el);
                        this.adjustMenuBarButtons();
                        this.setHeight(CSSConstants.ActiveTabViewHeight());
//                        if (tabName == 'go_to_next_chapter') {
//                            var scaleFactor = this.nextChapterTextWidth / this.previousChapterTextWidth;
//                            CSSHandler.setWidth($(".goToNeighbourChapterTextButtonWrapper", this.el), (scaleFactor * this.defaultPercentageChapterTextWidth) + '%');
//                        }
                        this.setWidth(100 / this.model.get("visibleTabsNumber") + "%");
                        //this.setWidth(CSSConstants.CustomActiveTabViewWidth(navigateToNeighbourChapterButtonDefaultWidth));
                        $imagesEl = this.$el.find(".images");
                        if (this.neighbourChapterInfo.itemDisabled) {
                            this.$neighbourChapterNavigationElement.addClass('disabled');
                        }
                        CSSHandler.setHorizontalMargin($imagesEl, -CSSConstants.ActiveTabViewImageHeight() / 2, 0);
                        CSSHandler.setVerticalMargin($imagesEl, -CSSConstants.ActiveTabViewImageHeight() / 2, 0);
                        CSSHandler.setHeight($imagesEl, CSSConstants.ActiveTabViewImageHeight());
                        this.model.set('pageNumber', 0);
                        //this.setHorizontalMargin(horizontalMargins, horizontalMargins);
                        return this;
                    }
                });

                that.ActiveTabsWrapperView = BaseView.extend({
                    id: 'bottomMenuActiveTabsWrapper',
                    className: 'chapterBottomMenuActiveTabsWrapperMenuClass',
                    tagName: 'div',
                    initialize: function (attr) {
                        this.listenTo(this.model, "change:remove", this.removeView);
                    },

                    removeView: function () {
                        this.remove();
                        this.unbind();
                    },
                    render: function () {
                        // TODO: should compute a max width for this element to have buttons more compact
                        var idx, view, models, size, model, activeTabIndex = 0;
                        models = that.activeTabsCollection.models;
                        size = models.length;
                        this.$el.html();
                        for (idx = 0; idx < size; idx++) {
                            model = models[idx];
                            if (model.get('name') == 'toc_tab') {
                                activeTabIndex = idx;
                            }
                            if (CUSTOM_MENU_TABS_VIEW.indexOf(model.get('name')) != -1) {
                                if (model.get("ocToggleInfo")["toggleInMenu"]) {
                                    var ocButton = new OCHandler(models[idx]);
                                    ocButton.init();
                                    view = ocButton.getView();
                                    Chapter.getBottomMenuCustomActiveTabsCollection().add(model);
                                }
                            } else if (model.get("imageURL") == null && model.get("leftImgSrc") == null) {
                                // do not show TOC icon when this has been disabled via project setting
                                continue;
                            } else {
                                if (model.get("name") == "back_button" && AppUtils.offlineExport) {
                                    continue; // don't show back button in offline exports, CD-285
                                }
                                if (model.get("name") == 'go_to_previous_chapter' || model.get("name") == 'go_to_next_chapter') {
                                    view = new that.CustomTabView({
                                        model: model
                                    });
                                } else {
                                    view = new that.ActiveTabView({
                                        model: model
                                    });
                                }

                            }
                            this.$el.append(view.render().el);
                        }
                        models[activeTabIndex].set("active", true);
                        return this;
                    }
                });

                that.ActiveTabsAreaView = BaseView.extend({
                    id: 'bottomMenuActiveTabsArea',
                    className: function () {
                        var customMenuProperties = bottomMenuRawModel['customMenuModel'],
                            customMenuType = customMenuProperties['CUSTOM_MENU_TYPE'];
                        if (!this.isEmpty(customMenuProperties) && customMenuType != "NONE") {
                            return 'chapterBottomMenuActiveTabsMenuClass';
                        } else {
                            return 'chapterBottomMenuActiveTabsMenuClass defaultMenuActiveTabsAreaPattern';
                        }

                    },
                    tagName: 'div',
                    initialize: function (attr) {
                        this.listenTo(this.model, "change:remove", this.removeView);
                        this.listenTo(this.model, "change:moveToNeighbourChapter", this.activateMoveToNeighbourChapterArrow);
                        this.listenTo(this.model, "change:pageNumber", this.updatePageNumber);
                    },

                    activateMoveToNeighbourChapterArrow: function () {
                        var models, idx, moveToNeighbourChapter = this.model.get('moveToNeighbourChapter');
                        if (null != moveToNeighbourChapter) {
                            models = that.activeTabsCollection.models;
                            for (idx = 0; idx < models.length; idx++) {
                                models[idx].set('moveToNeighbourChapter', moveToNeighbourChapter);
                            }
                            this.model.set('moveToNeighbourChapter', null);
                        }

                    },
                    updatePageNumber: function () {
                        var models, idx, pageNumber = this.model.get('pageNumber');
                        if (null != pageNumber) {
                            models = that.activeTabsCollection.models;
                            for (idx = 0; idx < models.length; idx++) {
                                models[idx].set('pageNumber', pageNumber);
                            }
                        }

                    },
                    events: function () {
                        var _events = {};
                        _events[AppUtils.EventTypes.mouseDown] = "preventPageChange";
                        return _events;
                    },

                    preventPageChange: function (e) {
//                    e.preventDefault();
                    },

                    removeView: function () {
                        this.remove();
                        this.unbind();
                    },

                    isEmpty: function (obj) {

                        // null and undefined are "empty"
                        if (obj == null) return true;

                        // Assume if it has a length property with a non-zero value
                        // that that property is correct.
                        if (obj.length > 0)    return false;
                        if (obj.length === 0)  return true;

                        // Otherwise, does it have any properties of its own?
                        // Note that this doesn't handle
                        // toString and toValue enumeration bugs in IE < 9
                        for (var key in obj) {
                            if (obj.hasOwnProperty(key)) return false;
                        }

                        return true;
                    },
                    render: function () {
                        this.$el.html();
                        this.setTop(this.model.get("top"));
                        this.setWidth(this.model.get("width"));
                        this.setHeight(this.model.get("height"));
                        this.$el.append(that.activeTabsWrapperView.render().el);
                        return this;
                    }
                });
            };

            this.instantiateObjects = function (callback) {
                that.activeTabContentWrapperModel = new Backbone.Model({
                    height: actionAreaHeight
                });

                that.activeTabContentWrapperView = new that.TabContentView({
                    model: that.activeTabContentWrapperModel
                });
                that.tabsContentViews = [];
                createActiveTabsElement(callback);
            };

            function getUpdatedActiveTabs(projectActiveTabs) {
                var bookId = Chapter.getSettings().bookId,
                    bookInfo = localStorage.getItem('bookInfo_' + bookId),
                    ret = projectActiveTabs;
                if (bookId != 0) {
                    if (null != bookInfo) {
                        bookInfo = JSON.parse(bookInfo);
                        if (bookInfo['navigateToNeighbourChapter']) {
                            ret.splice(0, 0, getGoToNeighbourChapter(-1));
                            ret.push(getGoToNeighbourChapter(1));
                            navigateTroughProjects = true;
                        }
                    }
                }
                return ret;
            }

            function getGoToNeighbourChapter(direction) {
                var ret = {};
                ret.highlightURL = 'gfx/highlight_overlay.png';
                ret.useIconURL = true;
                if (direction < 0) {
                    ret.name = 'go_to_previous_chapter';
                    ret.leftImgSrc = 'gfx/menu/PREVIOUS-CHAPTER-ICON.png';
                    ret.rightImgSrc = 'gfx/menu/PREVIOUS-CHAPTER-SINGLE.png';
                } else {
                    ret.name = 'go_to_next_chapter';
                    ret.leftImgSrc = 'gfx/menu/NEXT-CHAPTER-SINGLE.png';
                    ret.rightImgSrc = 'gfx/menu/NEXT-CHAPTER-ICON.png';
                }
                return ret;
            }

            function createActiveTabsElement(callback) {

                function computeImageURL(currentTab) {
                    var url;
                    if (currentTab["useIconURL"]) {
                        url = currentTab["iconSrc"];
                    } else {
                        url = ResourceManager.getPathForImage(currentTab["objectId"], currentTab["extension"]);
                    }
                    return url;
                }

                var visibleTabsNumber = function () {
                    var idx,
                        ret = activeTabsModels.length;
                    for (idx = 0; idx < activeTabsModels.length; idx++) {
                        if (activeTabsModels[idx]["name"] == "toc_tab" && activeTabsModels[idx]["iconSrc"] == null) {
                            ret--;
                        } else if (activeTabsModels[idx]["name"] == "back_button" && AppUtils.offlineExport) {
                            // don't show back button in offline exports, CD-285
                            ret--;
                        }
                    }
                    return ret;
                }();

                var idx, currentTab, tabsNumber = activeTabsModels.length, tabModel, tabMargin, activeTabIndex = 0;
                if (tabsNumber > 0) {

                    that.activeTabsAreaModel = new Backbone.Model({
                        height: CSSConstants.DefaultActiveTabWrapperHeight(),
                        top: actionAreaHeight,
                        width: '100%'
                    });

                    that.activeTabsAreaView = new that.ActiveTabsAreaView({
                        model: that.activeTabsAreaModel
                    });

                    that.activeTabsWrapperModel = new Backbone.Model({});
                    that.activeTabsWrapperView = new that.ActiveTabsWrapperView({
                        model: that.activeTabsWrapperModel
                    });

                    that.activeTabsCollection = new Backbone.Collection();
                    that.activeTabsContentCollection = new Backbone.Collection();
                    tabMargin = getActiveTabsMargins(visibleTabsNumber);

                    for (idx = 0; idx < tabsNumber; idx++) {
                        currentTab = activeTabsModels[idx];
                        if (currentTab.name == 'toc_tab') {
                            activeTabIndex = idx;
                        }
                        // TODO: take care of tabs with no content
                        if (CUSTOM_MENU_TABS_VIEW.indexOf(currentTab.name) != -1) {
                            tabModel = new Backbone.Model(currentTab);
                            tabModel.set('height', activeTabDefaultHeight);
                            tabModel.set('horizontalMargins', tabMargin);
                            tabModel.set('visibleTabsNumber', visibleTabsNumber);
                        } else {
                            tabModel = new Backbone.Model({
                                height: activeTabDefaultHeight,
                                horizontalMargins: tabMargin,
                                imageURL: computeImageURL(currentTab),
                                visibleTabsNumber: visibleTabsNumber,
                                toggleIcon: currentTab.toggleIconSrc,
                                highlightURL: currentTab.highlightURL,
                                name: currentTab.name,
                                active: false,
                                ocIds: currentTab.ocIds,
                                leftImgSrc: currentTab.leftImgSrc,
                                rightImgSrc: currentTab.rightImgSrc
                            });
                        }

                        createTabsContent(currentTab, function () {
                            displayActiveTabContent(activeTabsModels[activeTabIndex].name);
                        }, callback);
                        that.activeTabsCollection.add(tabModel);
                    }

                }
            }

            function getActiveTabsMargins(tabNumbers) {
                var tabWidth = CSSConstants.ActiveTabViewWidth(),
                    customTabWidth = CSSConstants.CustomActiveTabViewWidth(navigateToNeighbourChapterButtonDefaultWidth),
                    wrapperWidth = App.settings.width,
                    spaceLeft;
                if (navigateTroughProjects) {
                    spaceLeft = wrapperWidth - ((tabNumbers - 2) * tabWidth) - (2 * customTabWidth);
                } else {
                    spaceLeft = wrapperWidth - (tabNumbers * tabWidth);
                }
                return Math.floor(spaceLeft / (2 * tabNumbers));

            }

            function createTabsContent(currentTab, createTabCallback, addToDOMCallback) {
                var tabName = currentTab.name;
                if (tabName == 'toc_tab') {
                    setTocTabContent(tabName, createTabCallback, addToDOMCallback);
                } else if (tabName == 'settings_tab') {
                    setSettingsTabContent(currentTab, createTabCallback, addToDOMCallback);
                } else if (tabName == 'recording_tab') {
                    setRecordingTabContent(tabName, createTabCallback, addToDOMCallback);
                } else if (MENU_ITEMS_NO_CONTENT.indexOf(tabName) != -1) { // for buttons that don't have a tab content
                    createdTabsNumber++;
                } else { // for other possible cases
                    createdTabsNumber++;
                }
            }

            function displayActiveTabContent(tabName) {
                var models = that.activeTabsContentCollection.models,
                    activeTabsNumber = models.length,
                    model,
                    idx;

                if (MENU_ITEMS_NO_CONTENT.indexOf(tabName) == -1) {
                    for (idx = 0; idx < activeTabsNumber; idx++) {
                        model = models[idx];
                        if (model.get('name') == tabName) {
                            model.set("hidden", false);
                        } else {
                            model.set("hidden", true);
                        }

                    }
                    var renderResult = that.tabsContentViews[tabName].render();
                    if (renderResult != null) {
                        that.activeTabContentWrapperView.$el.append(renderResult.el);
                    }
                } else {
                    console.log("TODO: handle action for menu button " + tabName);
                }

            }

            function setTocTabContent(tabName, createTabCallback, addToDOMCallback) {
                require(['TocTabContentElement'], function (TocTabContentElement) {
                    var tocTabContent = new TocTabContentElement(bottomMenuRawModel.thumbs.array, bottomMenuRawModel.thumbs.maxThumbHeight, bottomMenuRawModel.settings.navigationArrows);
                    tocTabContent.init();
                    that.activeTabsContentCollection.add(tocTabContent.getModel());
                    that.tabsContentViews[tabName] = tocTabContent.getView();
                    that.tabsContentViews.length++;
                    createdTabsNumber++;
                    if (createdTabsNumber == totalActiveTabsNumber) {
                        if (null != createTabCallback) {
                            createTabCallback();
                        }
                        if (null != addToDOMCallback) {
                            addToDOMCallback();
                        }
                    }
                });
            }

            function setSettingsTabContent(currentTab, createTabCallback, addToDOMCallback) {
                var tabName = currentTab.name;
                require(['SettingsTabContentElement'], function (SettingsTabContentElement) {
                    var settingsTabContent = new SettingsTabContentElement(bottomMenuRawModel.thumbs.array,
                        bottomMenuRawModel.thumbs.maxThumbHeight,
                        bottomMenuRawModel.settings.navigationArrows,
                        bottomMenuRawModel.settings);
                    settingsTabContent.init();
                    that.activeTabsContentCollection.add(settingsTabContent.getModel());
                    that.tabsContentViews[tabName] = settingsTabContent.getView();
                    that.tabsContentViews.length++;
                    createdTabsNumber++;
                    if (createdTabsNumber == totalActiveTabsNumber) {
                        if (null != createTabCallback) {
                            createTabCallback();
                        }
                        if (null != addToDOMCallback) {
                            addToDOMCallback();
                        }
                    }
                });
            }

            function setRecordingTabContent(tabName, createTabCallback, addToDOMCallback) {
                require(['RecordingTabContent'], function (RecordingTabContent) {
                    var recordingTabContent = new RecordingTabContent(bottomMenuModel, bottomMenuRawModel.thumbs.array, bottomMenuRawModel.thumbs.maxThumbHeight, bottomMenuRawModel.settings.navigationArrows);
                    recordingTabContent.init();
                    that.activeTabsContentCollection.add(recordingTabContent.getModel());
                    that.tabsContentViews[tabName] = recordingTabContent.getView();
                    that.tabsContentViews.length++;
                    createdTabsNumber++;
                    if (createdTabsNumber == totalActiveTabsNumber) {
                        if (null != createTabCallback) {
                            createTabCallback();
                        }
                        if (null != addToDOMCallback) {
                            addToDOMCallback();
                        }
                    }
                });
            }

            return {
                init: function (callback) {
                    that.defineClasses();
                    that.instantiateObjects(callback);
                },
                getTabContentWrapperModel: function () {
                    return that.activeTabsAreaModel;
                },
                getActiveTabsWrapperModel: function () {
                    return that.activeTabContentWrapperModel;
                },
                getActiveTabsContentAreaView: function () {
                    return that.activeTabContentWrapperView;
                },
                getActiveTabsAreaView: function () {
                    return that.activeTabsAreaView;
                }

            }
        }
    }
)
;