define(['Templates'], function () {
    return function (model) {
        var that = this;

        this.defineClasses = function () {

            that.ToggleOCView = BaseView.extend({
                tagName: "div",
                className: "chapterBottomMenuTabClass centerAlign",
                thumbnailsCollection: [],

                initialize: function (attr) {
                    this.listenTo(this.model, "change:remove", this.removeView);
                    this.listenTo(this.model, "change:toggledCategories", this.toggleCategories);
                },

                removeView: function () {
                    this.remove();
                    this.unbind();
                },
                toggleCategories: function () {
                    var models = that.buttonsCollection.models,
                        idx, toggledCategories = this.model.get('toggledCategories');
                    if (null != toggledCategories) {
                        for (idx = 0; idx < models.length; idx++) {
                            models[idx].set('toggledCategories', toggledCategories)
                        }
                        this.model.set('toggledCategories', null);
                    }


                },

                addButtons: function () {
                    var buttons = this.model.get('ocToggleInfo')['buttons'],
                        idx, model, view,
                        thisView = this, buttonObject, ocInfoArray, firstOCInfo,
                        buttonWidth = 100 / buttons.length + "%",
                        iconActive, iconInactive;

                    for (idx = 0; idx < buttons.length; idx++) {
                        buttonObject = buttons[idx];
                        ocInfoArray = buttonObject["ocInfo"];
                        // for now we assume this array only has one element
                        if (ocInfoArray.length > 0) {
                            firstOCInfo = ocInfoArray[0];
                        } else {
                            continue;
                        }

                        iconActive = "";
                        iconInactive = "";

                        if (typeof buttonObject["iconActive"]["objectId"] != "undefined") {
                            iconActive = ResourceManager.getPathForImage(buttonObject["iconActive"]["objectId"], buttonObject["iconActive"]["extension"]);
                        }
                        if (typeof buttonObject["iconInactive"]["objectId"] != "undefined") {
                            iconInactive = ResourceManager.getPathForImage(buttonObject["iconInactive"]["objectId"], buttonObject["iconInactive"]["extension"]);
                        }

                        model = new Backbone.Model({
                            caption: buttonObject.caption,
                            effectAction: buttonObject["effectAction"],
                            height: thisView.model.get('height'),
                            ocId: firstOCInfo["categoryId"],
                            ocAction: firstOCInfo["action"],
                            width: buttonWidth,
                            iconActive: iconActive,
                            iconInactive: iconInactive,
                            active: null
                        });
                        view = new that.OCButtonView({
                            model: model
                        });
                        that.buttonsCollection.add(model);
                        this.$el.append(view.render().el);
                    }
                },

                render: function () {
                    this.$el.html();

                    this.setHeight(CSSConstants.ActiveTabViewHeight());

                    //this.setWidth(CSSConstants.ActiveTabViewWidth());
                    this.setWidth(100 / this.model.get("visibleTabsNumber") + "%");
                    //this.setHorizontalMargin(this.model.get("horizontalMargins"), this.model.get("horizontalMargins"));

                    this.addButtons();
                    return this;
                }
            });

            that.OCButtonView = BaseView.extend({
                tagName: "div",
                className: "ocToggleButton",
                template: templates.getTemplate('toggleOCMenuButton'),
                thumbnailsCollection: [],
                activeImage: null,
                inactiveImage: null,

                initialize: function (attr) {
                    this.listenTo(this.model, "change:remove", this.removeView);
                    this.listenTo(this.model, "change:active", this.toggleActivation);
                    this.listenTo(this.model, "change:toggledCategories", this.toggleCategories);
                },

                toggleCategories: function () {
                    var toggledCategories = this.model.get('toggledCategories'),
                        ocId = this.model.get('ocId');
                    if (null != toggledCategories) {

                        if ((null != toggledCategories.activatedCategories && toggledCategories.activatedCategories[ocId]) || (null != toggledCategories.deactivatedCategories && toggledCategories.deactivatedCategories[ocId])) {
                            this.model.set('active', this.isActive());
                        }
                        this.model.set('toggledCategories', null);
                    }


                },

                toggleActivation: function () {
                    if (this.model.get('active')) {
                        this.activeImage.show();
                        this.inactiveImage.hide();
                    } else {
                        this.activeImage.hide();
                        this.inactiveImage.show();
                    }
                },

                removeView: function () {
                    this.remove();
                    this.unbind();
                },
                expandCollapse: function () {
                },
                events: function () {
                    var _events = {};
                    _events[AppUtils.EventTypes.mouseDown] = "toggleOC";
                    return _events;
                },
                toggleOC: function (event) {
                    var action = this.model.get('ocAction');

                    this.processEffects(this.model.get('effectAction'));

                    if (action == 'toggle') {
                        var activatedCategories = {}, deactivatedCategories = {}, transportObject = {}, ids = [];

                        ids.push(this.model.get('ocId'));

                        OCUtils.extractActivatedCategoryFrom(ids, action, activatedCategories, deactivatedCategories);

                        transportObject.activatedCategories = activatedCategories;
                        transportObject.deactivatedCategories = deactivatedCategories;

                        Chapter.getScrollingElementsModel().set("toggledCategories", transportObject);


                    }
//                    this.model.set('active', !this.model.get('active'));
                },

                processEffects: function (effectAction) {
                    if (typeof  effectAction != "undefined") {
                        if (effectAction == "SCENE_OBJECT_EFFECTS_RECORDING_WIZARD_STOP") {
                            EffectsHandler.stopPlayRecordedSounds();
                        } else if (effectAction == "SCENE_OBJECT_EFFECTS_RECORDING_WIZARD_RECORD") {
                            EffectsHandler.startRecordingWizard();
                        } else if (effectAction == "SCENE_OBJECT_EFFECTS_RECORDING_WIZARD_PLAY") {
                            EffectsHandler.startPlayRecordedSounds();
                        }
                    }
                },

                isActive: function () {
                    var objectCategory = Chapter.getObjectCategories()[this.model.get('ocId')];
                    if (typeof objectCategory != "undefined") {
                        return objectCategory.active;
                    }
                    return false;
                },


                render: function () {

                    var $templateEl = $(this.template(this.model.attributes));
                    this.$el.html($templateEl);
                    this.setWidth(this.model.get('width'));
                    this.activeImage = this.$el.find(".activeImage");
                    this.inactiveImage = this.$el.find(".inactiveImage");

                    CSSHandler.setHeight(this.activeImage, CSSConstants.ActiveTabViewImageHeight());
                    CSSHandler.setHeight(this.inactiveImage, CSSConstants.ActiveTabViewImageHeight());
                    CSSHandler.setHeight(this.$el, "100%");
                    CSSHandler.setVerticalMargin(this.activeImage, -CSSConstants.ActiveTabViewImageHeight() / 2, 0);

                    CSSHandler.setVerticalMargin(this.inactiveImage, -CSSConstants.ActiveTabViewImageHeight() / 2, 0);
                    this.model.set('active', this.isActive());

                    return this;
                }
            });


        };
        this.instantiateObjects = function () {
            that.buttonsCollection = new Backbone.Collection();

            that.toggleOCView = new that.ToggleOCView({
                model: model
            });
        };

        return {
            init: function () {
                that.defineClasses();
                that.instantiateObjects();

            },
            getModel: function () {
                return that.model;
            },
            getView: function () {
                return that.toggleOCView;
            }
        }
    }
});