define(['Templates'], function () {
    return function (bottomMenuModel, activePage) {
        var that = this,
            maxThumbHeight = Math.round(bottomMenuModel.thumbs.maxThumbHeight),
            thumbMargin = 6,
            top = 10;


        this.defineClasses = function () {

            that.MenuIconView = BaseView.extend({
                id: 'bottomMenuIcon',
                className: 'chapterBottomMenuIconClass',
                template: templates.getTemplate("image"),
                tagName: 'div',
                initialize: function (attr) {
                    this.listenTo(this.model, "change:remove", this.removeView);
                    this.listenTo(this.model, "change:updateVisibility", this.updateVisibility);
                },
                events: function () {
                    var _events = {};
                    if (this.model.get("iconVisible")) {
                        _events[AppUtils.EventTypes.mouseDownSafeIOS5] = "toggleBottomMenu";
                    }
                    return _events;
                },

                toggleBottomMenu: function (e, direction) {
                    console.log("toggleBottomMenu");
                    if (e) {
                        e.stopPropagation();
                        e.preventDefault();
                    }
                    var expanded = that.chapterBottomMenuModel.get("expanded"),
                        canOpen = (direction == null) || (direction == "up"),
                        canClose = (direction == null) || (direction == "down");
                    if (expanded == MenuState.CLOSED && canOpen) {
                        that.chapterBottomMenuModel.set("expanded", MenuState.FULL_EXPANDED);
                    } else if (canClose) {
                        that.chapterBottomMenuModel.set("expanded", MenuState.CLOSED);
                    }
                },

                removeView: function () {
                    this.remove();
                    this.unbind();
                },

                renderTransparentActivationArea: function () {
                    var thisView = this;
                    this.$el.css("width", "100%")
                        .css("height", "100px")
                        .css("top", "-100px");
                    require(['hammerJS'], function (Hammer) {
                        Hammer(thisView.el).on("dragend", function (event) {
                            thisView.toggleBottomMenu(null, event.gesture.direction);
                        })
                    });
                    return this;
                },

                renderRegularMenuIcon: function () {
                    var menuIconMarginTop = AppUtils.runCustomMenuMethod("getMenuIconMarginTop", this.model.get("chapterBottomMenuHeight"));
                    this.$el.html(this.template(this.model.attributes));
                    this.setTop(this.model.get("top"));
                    this.setWidth(this.model.get("width"));
                    this.setHeight(this.model.get("height"));
                    this.setLeft("50%");
                    this.setHorizontalMargin(-this.model.get("width") / 2, 0);
                    this.setRight(this.model.get("right"));
                    if (menuIconMarginTop != null) {
                        this.$el.css("margin-top", Math.round(menuIconMarginTop) + "px");
                    }
                    if (activePage == 0 && !this.model.get("iconVisibleOnFirstPage")) {
                        this.$el.hide();
                    }
                    if (AppUtils.DeviceType.ANDROID && !AppUtils.DeviceType.XWalk) {
                        // only for android when running with default web browser
                        this.$el.addClass("menuIconDefaultWebViewAndroid");
                    }
                    return this;
                },

                render: function () {
                    if (this.model.get("iconVisible")) {
                        return this.renderRegularMenuIcon();
                    } else if (this.model.get("transparentActivationArea")) {
                        return this.renderTransparentActivationArea();
                    }
                    return null;
                },

                updateVisibility: function () {
                    var activePage = this.model.get("updateVisibility");
                    if (activePage != 0) {
                        this.$el.show();
                    } else {
                        if (!this.model.get("iconVisibleOnFirstPage")) {
                            this.$el.hide();
                        }
                    }
                }
            });

            that.ChapterBottomMenuView = BaseView.extend({
                tagName: "div",
                id: "chapterBottomMenu",
                className: 'chapterBottomMenuClass',
                actionAreaElement: {},
                activeTabsAreaElement: {},
                initialize: function (attr) {
                    this.listenTo(this.model, "change:remove", this.removeView);
                    this.listenTo(this.model, "change:expanded", this.toggleMenu);
                    this.listenTo(this.model, "change:moveToNeighbourChapter", this.activateMoveToNeighbourChapterArrow);
                    this.listenTo(this.model, "change:pageNumber", this.updatePageNumberForBottomMenu);
                },
                activateMoveToNeighbourChapterArrow: function () {
                    var moveToNeighbourChapter = this.model.get('moveToNeighbourChapter');
                    if (null != moveToNeighbourChapter) {
                        this.model.set('expanded', MenuState.FULL_EXPANDED);
                        this.activeTabsAreaElement.getTabContentWrapperModel().set('moveToNeighbourChapter', moveToNeighbourChapter);
                        this.model.set('moveToNeighbourChapter', null);
                    }
                },
                updatePageNumberForBottomMenu: function () {
                    this.activeTabsAreaElement.getTabContentWrapperModel().set('pageNumber', this.model.get('pageNumber'));
                },
                toggleMenu: function () {
                    var finalPosition = 0,
                        transitionTiming = 700;
                    if (this.model.get("expanded") == MenuState.FULL_EXPANDED) {
                        finalPosition = -this.model.get("height") + CSSConstants.OffsetForHiddenTabs();
                    } else if (this.model.get("expanded") == MenuState.DOWN_FOR_RECORDING) {
                        finalPosition = -CSSConstants.DefaultActiveTabWrapperHeight();
                        transitionTiming = 500;
                    } else if (this.model.get("expanded") == MenuState.CLOSED) {
                        finalPosition = 0;
                    }
                    this.translate3D(0, finalPosition, 0, transitionTiming);
                },
                removeView: function () {
                    this.remove();
                    this.unbind();
                },
                expandCollapse: function () {
                },
                setActiveTab: function () {

                },
                setupCustomMenuBackground: function () {
                    var customMenuProperties = bottomMenuModel['customMenuModel'],
                        customMenuType, backgroundImageURL, backgroundRepeat, backgroundColor;
                    if (!this.isEmpty(customMenuProperties)) {
                        customMenuType = customMenuProperties['CUSTOM_MENU_TYPE'];
                        backgroundImageURL = this.getImageUrl(customMenuProperties['CUSTOM_MENU_TYPE_' + customMenuType], customMenuType);
                        if (null != backgroundImageURL) {
                            this.setBackgroundImage(backgroundImageURL);
                        }
                        backgroundRepeat = this.getBackgroundRepeat(customMenuProperties['CUSTOM_MENU_TYPE_' + customMenuType], customMenuType);

                        this.setBackgroundRepeat(backgroundRepeat);

                        backgroundColor = this.getBackgroundColor(customMenuProperties['CUSTOM_MENU_TYPE_' + customMenuType], customMenuType);
                        if (null != backgroundColor) {
                            this.setBackgroundColor(backgroundColor);
                        }
                    }
                },
                getImageUrl: function (customMenuProperties, customMenuType) {
                    var imageURL = null;
                    if (customMenuType == 'IMAGE') {
                        imageURL = ResourceManager.getPathForImage(customMenuProperties['BACKGROUND_IMAGE']['ID'], customMenuProperties['BACKGROUND_IMAGE']['FILE_EXTENSION']);
                    } else if (customMenuType == 'PATTERN') {
                        var pattern = customMenuProperties['PATTERN'];
                        if (null != 'NO_PATTERN') {
                            imageURL = 'gfx/patterns/' + pattern + '.png';
                        }
                    }
                    return imageURL;
                },
                getBackgroundRepeat: function (customMenuProperties, customMenuType) {
                    var backgroundRepeat = 'repeat';
                    if (customMenuType == 'IMAGE') {
                        backgroundRepeat = AppUtils.getValueAsBoolean(customMenuProperties['BACKGROUND_REPEAT']) ? 'repeat' : 'no-repeat';
                    } else if (customMenuType = 'PATTERN') {
                        //do nothing
                    }
                    return backgroundRepeat;
                },
                getBackgroundColor: function (customMenuProperties, customMenuType) {
                    var backgroundColor = 'null';
                    if (customMenuType == 'PATTERN') {
                        backgroundColor = customMenuProperties['BACKGROUND_COLOR'] == 'none' ? null : customMenuProperties['BACKGROUND_COLOR'];
                    } else if (customMenuType = 'IMAGE') {
                        //do nothing. WILL ADD LOGIC LATER IF NEEDED
                    }
                    return backgroundColor;
                },
                isEmpty: function (obj) {

                    // null and undefined are "empty"
                    if (obj == null) return true;

                    // Assume if it has a length property with a non-zero value
                    // that that property is correct.
                    if (obj.length > 0)    return false;
                    if (obj.length === 0)  return true;

                    // Otherwise, does it have any properties of its own?
                    // Note that this doesn't handle
                    // toString and toValue enumeration bugs in IE < 9
                    for (var key in obj) {
                        if (obj.hasOwnProperty(key)) return false;
                    }

                    return true;
                },
                render: function () {
                    var thisView = this,
                        cssBottomValue = -this.model.get("height");
                    this.$el.html();
                    require(['ActiveTabs'], function (ActiveTabs) {
                        thisView.activeTabsAreaElement = new ActiveTabs(bottomMenuModel, thisView.model.get("tabContentHeight"), that.chapterBottomMenuModel);
                        thisView.activeTabsAreaElement.init(function () {
                            thisView.$el.append(thisView.activeTabsAreaElement.getActiveTabsContentAreaView().render().el)
                                .append(thisView.activeTabsAreaElement.getActiveTabsAreaView().render().el);
                        });
                    });

                    this.setWidth(this.model.get("width"));
                    this.setHeight(this.model.get("height"));
                    if (AppUtils.DeviceType.XWalk) {
                        // causing problems of 1px shift when running with XWalk 5, maybe no longer needed when we update to XWalk 8
                        cssBottomValue -= 1;
                    }
                    this.setBottom(cssBottomValue);
                    this.setupCustomMenuBackground();
                    var menuIconRenderResult = that.menuIconView.render();
                    if (menuIconRenderResult) {
                        this.$el.append(menuIconRenderResult.el);
                    }
                    return this;
                }
            });
        };
        this.instantiateObjects = function (callback) {
            var thumbnailWrapperHeight = maxThumbHeight + 2 * thumbMargin,
                actionAreaHeight = thumbnailWrapperHeight + 2 * top,
                chapterBottomMenuHeight = actionAreaHeight + CSSConstants.DefaultActiveTabWrapperHeight();

            var chapterBottomMenuWidth = function () {
                var width = "100%";
                if (AppUtils.nativeApp && AppUtils.DeviceType.CHROME) {
                    width = App.settings.width + 1; // due to a bug that's only visible on Android (both XWalk and standard web view)
                }
                return width;
            };

            that.chapterBottomMenuModel = new Backbone.Model({
                height: chapterBottomMenuHeight,
                top: Chapter.getSettings().resolutionHeight,
                width: chapterBottomMenuWidth(),
                expanded: MenuState.CLOSED,
                tabContentHeight: actionAreaHeight
            });

            that.chapterBottomMenuView = new that.ChapterBottomMenuView({
                model: that.chapterBottomMenuModel
            });
            createMenuIcon(chapterBottomMenuHeight);
            if (null != callback) {
                callback();
            }
        };

        function createMenuIcon(chapterBottomMenuHeight) {
            var menuIconURL;
            bottomMenuModel.settings.menuIcon.height = Math.round(bottomMenuModel.settings.menuIcon.height * AppUtils.getMenuScaleFactor());
            bottomMenuModel.settings.menuIcon.width = Math.round(bottomMenuModel.settings.menuIcon.width * AppUtils.getMenuScaleFactor());

            if (bottomMenuModel.settings.menuIcon.useCustomIcon) {
                menuIconURL = ResourceManager.getPathForImage(bottomMenuModel.settings.menuIcon.iconObjectId, bottomMenuModel.settings.menuIcon.iconFileExtension);
            } else {
                menuIconURL = bottomMenuModel.settings.menuIcon.iconSrc;
            }

            that.menuIconModel = new Backbone.Model({
                height: bottomMenuModel.settings.menuIcon.height,
                width: bottomMenuModel.settings.menuIcon.width,
                top: -bottomMenuModel.settings.menuIcon.height,
                imageURL: menuIconURL,
                activeTab: '',
                iconVisibleOnFirstPage: bottomMenuModel.settings.menuIcon.iconVisibleOnFirstPage,
                iconVisible: bottomMenuModel.settings.menuIcon.iconVisible,
                transparentActivationArea: bottomMenuModel.settings.menuIcon.transparentActivationArea,
                expanded: MenuState.CLOSED,
                chapterBottomMenuHeight: chapterBottomMenuHeight
            });

            that.menuIconView = new that.MenuIconView({
                model: that.menuIconModel
            });
        }

        return {
            init: function (callback) {
                that.defineClasses();
                that.instantiateObjects(callback);

            },
            getModel: function () {
                return that.chapterBottomMenuModel;
            },
            getMenuIconModel: function () {
                return that.menuIconModel;
            },
            getView: function () {
                return that.chapterBottomMenuView;
            }
        }
    }
});