define(['Templates'], function () {
    return function (bottomMenuModel, thumbs, maxThumbHeight, navigationArrows) {
        var that = this,
            thumbMargin = 6,
            top = 10,
            isRendered = false;

        this.defineClasses = function () {

            that.RecordingTabContentWrapperView = BaseView.extend({
                tagName: "div",
                id: "recordingTabContentWrapper",

                initialize: function (attr) {
                    this.listenTo(this.model, "change:hidden", this.toggleVisibillity);
                },
                removeView: function () {
                    // not called at all, can be removed ?
                    this.remove();
                    this.unbind();
                },
                expandCollapse: function () {
                },
                render: function () {
                    if (isRendered) {
                        this.model.set("hidden", false);
                        return null;
                    } else {
                        this.$el.html();
                        this.$el.append(that.soundInfoView.render().el)
                            .append(that.recordSoundView.render().el)
                            .append(that.playSoundView.render().el);
                        isRendered = true;
                        return this;
                    }
                }

            });

            that.SoundInfoView = BaseView.extend({
                className: "bottomMenuSoundInfoClass",
                id: 'bottomMenuSoundInfo',
                template: templates.getTemplate('soundsInfo'),
                tagName: "div",
                initialize: function (attr) {
                    this.listenTo(this.model, "change:remove", this.removeView);
                },
                removeView: function () {
                    this.remove();
                    this.unbind();
                },

                render: function () {
                    this.$el.html(this.template());
                    this.scale(AppUtils.getMenuScaleFactor());
                    this.$el.css(CSSHandler.currentBrowserPrefix.transform + "-origin", "0 0");
                    return this;
                }
            });

            that.RecordSoundView = BaseView.extend({
                tagName: "div",
                className: "recordSoundClass",
                template: templates.getTemplate('recordSound'),
                id: 'recordSound',

                initialize: function (attr) {
                    this.listenTo(this.model, "change:remove", this.removeView);
                    this.listenTo(this.model, "change:toggleGlow", this.toggleGlow);
                },
                events: function () {
                    var _events = {};
                    _events[AppUtils.EventTypes.mouseDown] = "toggleSoundRecording";
                    return _events;
                },
                toggleSoundRecording: function (event) {
                    var recordSoundModel = this.model;
                    event.stopPropagation();
                    require([ "RecordingLightBox", "SoundRecorder"], function (RecordingLightBox) {
                        var recordingLightBox = new RecordingLightBox(bottomMenuModel);
                        recordingLightBox.init();
                        App.viewPortView.$el.append(recordingLightBox.getView().render().el);
                        if (typeof SoundPlayer != "undefined") {
                            SoundPlayer.endAllPlayingSoundsForPage(Chapter.getActivePage(), false);
                        }
                        SoundRecorder.setOperationMode(SoundRecorderOperationMode.SINGLE_PAGE);
                        SoundRecorder.setRecordingLightBoxModel(recordingLightBox.getModel());
                        SoundRecorder.setRecordSoundModel(recordSoundModel);
                        SoundRecorder.toggleSoundRecordingForPage(Chapter.getActivePage());
                        bottomMenuModel.set("expanded", MenuState.DOWN_FOR_RECORDING);
                    });
                },
                toggleGlow: function () {
                    var glowImage = this.$el.find(".recordingGlow");
                    if (this.model.get("toggleGlow")) {
                        glowImage.show();
                    } else {
                        glowImage.hide();
                    }
                },
                removeView: function () {
                    this.remove();
                    this.unbind();
                },
                expandCollapse: function () {
                },
                render: function () {
                    this.$el.html(this.template());

                    return this;
                }
            });

            that.PlaySoundView = BaseView.extend({
                tagName: "div",
                className: "playSoundClass",
                template: templates.getTemplate('playSound'),
                id: 'playSound',

                initialize: function (attr) {

                    this.listenTo(this.model, "change:remove", this.removeView);
                    this.listenTo(this.model, "change:toggleGlow", this.toggleGlow);
                    this.listenTo(this.model, "change:disable", this.toggleDisable);
                },
                events: function () {
                    var _events = {};
                    _events[AppUtils.EventTypes.mouseDown] = "togglePlayRecordedSound";
                    return _events;
                },
                togglePlayRecordedSound: function () {
                    SoundRecorder.togglePlayRecordedSoundForPage(Chapter.getActivePage());
                },
                toggleGlow: function () {
                    var glowImage = this.$el.find(".playGlow");
                    if (this.model.get("toggleGlow")) {
                        glowImage.show();
                    } else {
                        glowImage.hide();
                    }
                },
                toggleDisable: function () {
                    if (this.model.get("disable")) {
                        this.$el.css("opacity", "0.2");
                    } else {
                        this.$el.css("opacity", "1");
                    }
                },
                removeView: function () {
                    this.remove();
                    this.unbind();
                },
                expandCollapse: function () {
                },
                render: function () {
                    var thisView = this;
                    this.$el.html(this.template());
                    require(["SoundRecorder"], function () {
                        SoundRecorder.setPlaySoundModel(thisView.model);
                        SoundRecorder.updatePlayIconStateForPage(Chapter.getActivePage());
                    });
                    return this;
                }
            });
        };
        this.instantiateObjects = function () {
            var thumbnailWrapperHeight = maxThumbHeight + 2 * thumbMargin,
                actionAreaHeight = thumbnailWrapperHeight + 2 * top;

            that.recordingTabContentWrapperModel = new Backbone.Model({
                height: actionAreaHeight,
                name: 'recording_tab',
                hidden: true
            });
            that.recordingTabContentWrapperView = new that.RecordingTabContentWrapperView({
                model: that.recordingTabContentWrapperModel
            });

            that.soundInfoModel = new Backbone.Model();

            that.soundInfoView = new that.SoundInfoView({
                model: that.soundInfoModel
            });

            that.recordSoundModel = new Backbone.Model({
                height: maxThumbHeight
            });
            that.recordSoundView = new that.RecordSoundView({
                model: that.recordSoundModel
            });

            that.playSoundModel = new Backbone.Model({
                height: maxThumbHeight
            });
            that.playSoundView = new that.PlaySoundView({
                model: that.playSoundModel
            });

        };

        return {
            init: function () {
                that.defineClasses();
                that.instantiateObjects();
            },

            getModel: function () {
                return that.recordingTabContentWrapperModel;
            },
            getView: function () {
                return that.recordingTabContentWrapperView;
            }

        }
    }
});