define(['BottomMenuOcHandler', 'Templates'], function (OCHandler) {
    return function (thumbs, maxThumbHeight, navigationArrows, settings) {
        var that = this,
            thumbMargin = 6,
            top = 10,
            isRendered = false, settingsItems = settings.settingsItems;

        this.defineClasses = function () {

            that.SettingsTabContentWrapperView = BaseView.extend({
                tagName: "div",
                id: "settingsTabContentWrapper",

                initialize: function (attr) {
                    this.listenTo(this.model, "change:hidden", this.toggleVisibillity);
                },
                removeView: function () {
                    this.remove();
                    this.unbind();
                },
                expandCollapse: function () {
                },
                render: function () {

                    if (isRendered) {
                        this.model.set("hidden", false);
                        return null;
                    } else {
                        this.$el.html();
                        if (settingsItems.indexOf("globalSoundsControl") > -1) {
                            this.$el.append(that.globalSoundsView.render().el);
                        }
                        if (settingsItems.indexOf("soundEffectsControl") > -1) {
                            this.$el.append(that.soundEffectsView.render().el);
                        }
                        if (settingsItems.indexOf("textOnOffControl") > -1) {
                            this.$el.append(that.showTextView.render().el);
                        }
                        if (settingsItems.indexOf("readTextControl") > -1) {
                            this.$el.append(that.readTextView.render().el);
                        }
                        if (null != settings['OCToToggle']) {
                            var o = new OCHandler(this.model.get("height"), settings['OCToToggle'][0]);
                            o.init();
                            this.$el.append(o.getView().render().el);
                        }

                        isRendered = true;
                        return this;
                    }

                }

            });

            that.GlobalSoundsView = BaseView.extend({
                className: "bottomMenuGlobalSoundsClass menuSettingsItem",
                id: 'bottomMenuGlobalSounds',
                template: templates.getTemplate('bottomMenuGlobalSounds'),
                tagName: "div",
                initialize: function (attr) {
                    this.listenTo(this.model, "change:remove", this.removeView);
                },
                removeView: function () {
                    this.remove();
                    this.unbind();
                },
                events: function () {
                    var _events = {};
                    _events[AppUtils.EventTypes.mouseDown] = "toggleGlobalSounds";
                    return _events;
                },
                toggleGlobalSounds: function (event) {
                    var buttonPressedEl = $(event.target);
                    if (buttonPressedEl.hasClass("globalSoundsDisabled")) {
                        console.log("toggle GlobalSounds disable");
                        Chapter.setGlobalSoundsSetting(false);
                        if (typeof SoundPlayer != 'undefined') {
                            SoundPlayer.endAllPlayingSoundsForPage(Chapter.getActivePage(), false);
                        }
                    } else if (buttonPressedEl.hasClass("globalSoundsEnabled")) {
                        console.log("toggle GlobalSounds enable");
                        Chapter.setGlobalSoundsSetting(true);
                    } else {
                        return;
                    }
                    buttonPressedEl.addClass("activeSettingButton").removeClass("inactiveSettingButton");
                    buttonPressedEl.siblings().addClass("inactiveSettingButton").removeClass("activeSettingButton");
                },

                render: function () {

                    var templateEl = $(this.template());
                    if (!Chapter.getGlobalSoundsSetting()) {
                        templateEl.find(".globalSoundsEnabled").toggleClass("activeSettingButton").toggleClass("inactiveSettingButton");
                        templateEl.find(".globalSoundsDisabled").toggleClass("inactiveSettingButton").toggleClass("activeSettingButton");
                    }
                    this.$el.html(templateEl);

                    this.$el.css("font-size", CSSConstants.MenuSettingsFontSize() + "px");
                    return this;
                }
            });

            that.SoundEffectsView = BaseView.extend({
                className: "bottomMenuSoundEffectsClass menuSettingsItem",
                id: 'bottomMenuSoundEffects',
                template: templates.getTemplate('bottomMenuSoundEffects'),
                tagName: "div",
                initialize: function (attr) {
                    this.listenTo(this.model, "change:remove", this.removeView);
                },
                removeView: function () {
                    this.remove();
                    this.unbind();
                },
                events: function () {
                    var _events = {};
                    _events[AppUtils.EventTypes.mouseDown] = "toggleSoundEffects";
                    return _events;
                },
                toggleSoundEffects: function (event) {
                    var buttonPressedEl = $(event.target);
                    if (buttonPressedEl.hasClass("soundEffectDisabled")) {
                        console.log("toggleSoundEffects disable");
                        Chapter.setSoundEffects(false);
                    } else if (buttonPressedEl.hasClass("soundEffectEnabled")) {
                        console.log("toggleSoundEffects enable");
                        Chapter.setSoundEffects(true);
                    } else {
                        return;
                    }
                    buttonPressedEl.addClass("activeSettingButton").removeClass("inactiveSettingButton");
                    buttonPressedEl.siblings().addClass("inactiveSettingButton").removeClass("activeSettingButton");
                },

                render: function () {

                    var templateEl = $(this.template());
                    if (!Chapter.getSoundEffects()) {
                        templateEl.find(".soundEffectEnabled").toggleClass("activeSettingButton").toggleClass("inactiveSettingButton");
                        templateEl.find(".soundEffectDisabled").toggleClass("inactiveSettingButton").toggleClass("activeSettingButton");
                    }
                    this.$el.html(templateEl);

                    this.$el.css("font-size", CSSConstants.MenuSettingsFontSize() + "px");
                    return this;
                }
            });

            that.ShowTextView = BaseView.extend({
                tagName: "div",
                id: "bottomMenuShowText",
                className: "bottomMenuShowTextClass menuSettingsItem",
                template: templates.getTemplate('bottomMenuShowText'),
                thumbnailsCollection: [],

                initialize: function (attr) {
                    this.listenTo(this.model, "change:remove", this.removeView);
                },

                removeView: function () {
                    this.remove();
                    this.unbind();
                },
                expandCollapse: function () {
                },
                events: function () {
                    var _events = {};
                    _events[AppUtils.EventTypes.mouseDown] = "toggleShowText";
                    return _events;
                },
                toggleShowText: function (event) {
                    var buttonPressedEl = $(event.target);
                    if (buttonPressedEl.hasClass("showTextDisabled")) {
                        if (Chapter.getTextVisible() != false) {
                            console.log("toggleShowText disable");
                            Chapter.setTextVisible(false);
                        }
                    } else if (buttonPressedEl.hasClass("showTextEnabled")) {
                        if (Chapter.getTextVisible() != true) {
                            console.log("toggleShowText enable");
                            Chapter.setTextVisible(true);
                        }
                    } else {
                        return;
                    }
                    buttonPressedEl.addClass("activeSettingButton").removeClass("inactiveSettingButton");
                    buttonPressedEl.siblings().addClass("inactiveSettingButton").removeClass("activeSettingButton");
                },

                render: function () {
                    var templateEl = $(this.template());
                    if (!Chapter.getTextVisible()) {
                        templateEl.find(".showTextEnabled").toggleClass("activeSettingButton").toggleClass("inactiveSettingButton");
                        templateEl.find(".showTextDisabled").toggleClass("inactiveSettingButton").toggleClass("activeSettingButton");
                    }
                    this.$el.html(templateEl);
                    this.$el.css("font-size", CSSConstants.MenuSettingsFontSize() + "px");
                    return this;
                }
            });

            that.ReadTextView = BaseView.extend({
                tagName: "div",
                className: "bottomMenuReadTextClass menuSettingsItem",
                template: templates.getTemplate('bottomMenuReadText'),
                id: 'bottomMenuReadText',

                initialize: function (attr) {
                    this.listenTo(this.model, "change:remove", this.removeView);
                },
                removeView: function () {
                    this.remove();
                    this.unbind();
                },
                events: function () {
                    var _events = {};
                    _events[AppUtils.EventTypes.mouseDown] = "changeReadSettingsText";
                    return _events;
                },
                changeReadSettingsText: function (event) {
                    var buttonPressedEl = $(event.target);
                    if (buttonPressedEl.hasClass("readTextEnabled") && Chapter.getReadTextSettings() != ReadSoundSettings.SOUNDS_ON) {
                        console.log("use default sounds");
                        Chapter.setReadTextSettings(ReadSoundSettings.SOUNDS_ON);
                    } else if (buttonPressedEl.hasClass("readTextOwnVoice") && Chapter.getReadTextSettings() != ReadSoundSettings.OWN_SOUNDS) {
                        console.log("use own sounds where available else default sounds");
                        Chapter.setReadTextSettings(ReadSoundSettings.OWN_SOUNDS);
                    } else if (buttonPressedEl.hasClass("readTextDisabled") && Chapter.getReadTextSettings() != ReadSoundSettings.SOUNDS_OFF) {
                        console.log("no read sounds");
                        Chapter.setReadTextSettings(ReadSoundSettings.SOUNDS_OFF);
                    } else {
                        return;
                    }
                    buttonPressedEl.addClass("activeSettingButton").removeClass("inactiveSettingButton");
                    buttonPressedEl.siblings().addClass("inactiveSettingButton").removeClass("activeSettingButton");
                },
                expandCollapse: function () {
                },
                render: function () {
                    var readTextTemplate = $(this.template()),
                        $readTextEnabled = readTextTemplate.find(".readTextEnabled"),
                        $readTextDisabled = readTextTemplate.find(".readTextDisabled"),
                        $readTextOwnVoice = readTextTemplate.find(".readTextOwnVoice");

                    if (!Chapter.getMenuModel().settings.recordingEnabled) {
                        $readTextOwnVoice.remove();
                        $readTextEnabled.css("width", "50%");
                        if (AppUtils.DeviceType.ANDROID) {
                            $readTextDisabled.css("width", "49%"); // for Android 4.3 and lower
                        }
                        $readTextDisabled.css("width", "calc(50% - 1px)");
                        this.$el.css("min-width", "13.67%");
                    }

                    var readTextSettings = Chapter.getReadTextSettings();
                    if (readTextSettings == ReadSoundSettings.SOUNDS_ON) {
                        $readTextEnabled.addClass("activeSettingButton");
                        $readTextDisabled.addClass("inactiveSettingButton");
                        $readTextOwnVoice.addClass("inactiveSettingButton");
                    } else if (readTextSettings == ReadSoundSettings.SOUNDS_OFF) {
                        $readTextEnabled.addClass("inactiveSettingButton");
                        $readTextDisabled.addClass("activeSettingButton");
                        $readTextOwnVoice.addClass("inactiveSettingButton");
                    } else if (readTextSettings == ReadSoundSettings.OWN_SOUNDS) {
                        $readTextEnabled.addClass("inactiveSettingButton");
                        $readTextDisabled.addClass("inactiveSettingButton");
                        $readTextOwnVoice.addClass("activeSettingButton");
                    }

                    this.$el.html(readTextTemplate);
                    this.$el.css("font-size", CSSConstants.MenuSettingsFontSize() + "px");
                    return this;
                }
            });
        };
        this.instantiateObjects = function () {
            var thumbnailWrapperHeight = maxThumbHeight + 2 * thumbMargin,
                actionAreaHeight = thumbnailWrapperHeight + 2 * top;

            that.settingsTabContentWrapperModel = new Backbone.Model({
                height: actionAreaHeight,
                name: 'settings_tab',
                hidden: true
            });
            that.settingsTabContentWrapperView = new that.SettingsTabContentWrapperView({
                model: that.settingsTabContentWrapperModel
            });

            that.globalSoundsModel = new Backbone.Model({
                height: actionAreaHeight
            });
            that.globalSoundsView = new that.GlobalSoundsView({
                model: that.globalSoundsModel
            });

            that.soundEffectsModel = new Backbone.Model({
                height: actionAreaHeight
            });
            that.soundEffectsView = new that.SoundEffectsView({
                model: that.soundEffectsModel
            });

            that.showTextModel = new Backbone.Model({
                height: actionAreaHeight
            });
            that.showTextView = new that.ShowTextView({
                model: that.showTextModel
            });

            that.readTextModel = new Backbone.Model({
                height: actionAreaHeight
            });
            that.readTextView = new that.ReadTextView({
                model: that.readTextModel
            });


        };

        return {
            init: function () {
                that.defineClasses();
                that.instantiateObjects();

            },
            getModel: function () {
                return that.settingsTabContentWrapperModel;
            },
            getView: function () {
                return that.settingsTabContentWrapperView;
            }
        }
    }
});