define([''], function () {
    return function TocTabContentElement(thumbs, maxThumbHeight, navigationArrows) {
        var that = this,
            thumbMargin = 6,
            top = function () {
                var customValue = AppUtils.runCustomMenuMethod("getTopValueForMenuThumbnailWrapperView");
                return (customValue != null) ? customValue : 10;
            }(),
            isRendered = false;

        this.defineClasses = function () {

            that.TocTabContentWrapper = BaseView.extend({
                tagName: "div",
                id: "tocTabContentWrapper",

                initialize: function (attr) {
                    this.listenTo(this.model, "change:hidden", this.toggleVisibillity);
                },

                removeView: function () {
                    this.$el.hide();
                },
                render: function () {
                    if (isRendered) {
                        this.model.set("hidden", false);
                        return null;
                    } else {
                        // do the render now
                        this.$el.html();
                        this.$el.append(that.leftArrowView.render().el);
                        this.$el.append(that.thumbnailWrapperView.render().el);
                        this.$el.append(that.rightArrowView.render().el);
                        isRendered = true;
                        return this;
                    }
                }

            });

            that.ArrowView = BaseView.extend({
                className: "chapterBottomMenuNavigationArrowsClass",
                template: templates.getTemplate("image"),
                tagName: "div",
                initialize: function (attr) {
                    this.listenTo(this.model, "change:remove", this.removeView);
                    this.listenTo(this.model, "change:disabled", this.toggleDisabled);
                },
                removeView: function () {
                    this.remove();
                    this.unbind();
                },
                events: function () {
                    var _events = {};
                    _events[AppUtils.EventTypes.mouseDownSafeIOS5] = "scrollTocTab";
                    return _events;
                },
                scrollTocTab: function () {
                    console.log("ArrowView: scrollTocTab with direction " + this.model.get("direction"));
                    Chapter.getGlobalModels().thumbnailWrapperModel.set("scrollDirection", this.model.get("direction"));
                },
                toggleDisabled: function () {
                    if (this.model.get("disabled")) {
                        this.$el.css("opacity", "0.3");
                    } else {
                        this.$el.css("opacity", "1");
                    }
                },
                render: function () {
                    var horizontalMargins = this.model.get("horizontalMargins");
                    this.$el.html(this.template(this.model.attributes));
                    this.setTop(this.model.get("top"));
                    this.setWidth(this.model.get("width"));
                    this.setHeight(this.model.get("height"));
                    this.setLeft(this.model.get("left"));
                    this.setRight(this.model.get("right"));
                    this.setHorizontalMargin(horizontalMargins, horizontalMargins);

                    return this;
                }
            });


            that.ThumbnailWrapperView = BaseView.extend({
                tagName: "div",
                id: "bottomMenuThumbnailWrapper",
                thumbnailsCollection: [],
                wrapperWidth: 0,
                iScrollWrapper: null,
                iScrollInstance: null,
                nextScrollPosition: 0,
                viewPortWidth: 0,
                maxScrollDelay: 300,
                scrollTimer: null,
                temp: 0,

                initialize: function (attr) {
                    this.viewPortWidth = App.settings.width - (this.model.get("left") + this.model.get("right"));
                    this.listenTo(this.model, "change:remove", this.removeView);
                    this.listenTo(this.model, "change:activateThumbs", this.activateThumbWithNumber);
                    this.listenTo(this.model, "change:scrollDirection", this.scrollForDirection);
                    this.initializeThumbnailCollection();
                },
                events: function () {
                    var _events = {};
                    if (!AppUtils.runWithIScroll()) {
                        _events[AppUtils.EventTypes.scroll] = "scrollHandler";
                    }
                    _events[AppUtils.EventTypes.mouseDown] = "allowThumbSwipe";
                    return _events;
                },

                scrollHandler: function (e) {
                    var thisView = this;
                    if (null != this.scrollTimer) {
                        window.clearTimeout(this.scrollTimer);
                    }
                    this.scrollTimer = window.setTimeout(function () {
                        thisView.updateArrowsStateForNewPosition();
                    }, this.maxScrollDelay);
                },
                removeView: function () {
                    this.remove();
                    this.unbind();
                },
                expandCollapse: function () {
                },

                allowThumbSwipe: function (e) {
                    e.stopPropagation();
                },
                activateThumbWithNumber: function () {
                    var idx, pages,
                        pageString = this.model.get("activateThumbs").toString(),
                        twinThumbIdx = -1;
                    if (pageString != "-1") {
                        pages = pageString.split(",")
                        for (idx = 0; idx < this.thumbnailsCollection.length; idx++) {
                            if (pages.indexOf(idx.toString()) > -1) {
                                this.thumbnailsCollection[idx].set("activeThumb", true);
                                if (App.dualLandscapeActive() && (idx + 1 < this.thumbnailsCollection.length)) {
                                    twinThumbIdx = idx + 1;
                                    this.thumbnailsCollection[twinThumbIdx].set("activeThumb", true)
                                }
                                this.scrollThumbnailWrapperForPage(idx);
                            } else if (idx != twinThumbIdx) {
                                this.thumbnailsCollection[idx].set("activeThumb", false);
                            }
                        }
                    }
                },
                scrollThumbnailWrapperForPage: function (pageNumber) {
                    var newScrollValue,
                        maxScrollValue = this.getScrollWidth() - this.viewPortWidth,
                        minScrollValue = 0,
                        thumbWidth = thumbs[0].width + thumbMargin * 2, // thumb width already has menu scale factor included
                        scrollTiming = 500;

                    newScrollValue = pageNumber * thumbWidth - this.viewPortWidth / 2 + thumbWidth / 2;

                    if (App.dualLandscapeActive()) {
                        newScrollValue += thumbWidth / 2;
                    }

                    newScrollValue = Math.max(newScrollValue, minScrollValue);
                    newScrollValue = Math.min(newScrollValue, maxScrollValue);

                    this.scrollTOC(newScrollValue, scrollTiming, function () {
                        if (null != that.leftArrowView && null != that.leftArrowView.model) {
                            if (newScrollValue == minScrollValue) { // disable left arrow
                                that.leftArrowView.model.set("disabled", true);
                            } else {
                                that.leftArrowView.model.set("disabled", false);
                            }
                        }
                        if (null != that.rightArrowView && null != that.rightArrowView.model) {
                            if (newScrollValue == maxScrollValue) { // enable right arrow
                                that.rightArrowView.model.set("disabled", true);
                            } else {
                                that.rightArrowView.model.set("disabled", false);
                            }
                        }

                    });
                },

                updateArrowsStateForNewPosition: function () {
                    var currentScrollValue = this.getCurrentScrollValue(),
                        maxScrollValue = this.getScrollWidth() - this.viewPortWidth,
                        minScrollValue = 0,
                        disableLeftArrow, disableRightArrow;
                    this.nextScrollPosition = currentScrollValue;
                    console.log("updateArrowsStateForNewPosition called for position " + currentScrollValue);

                    disableLeftArrow = (currentScrollValue - minScrollValue) < 5;
                    disableRightArrow = (maxScrollValue - currentScrollValue) < 5;

                    if (disableLeftArrow) {
                        console.log("updateArrowsStateForNewPosition LEFT arrow DISABLED");
                        that.leftArrowView.model.set("disabled", true);
                    } else {
                        console.log("updateArrowsStateForNewPosition LEFT arrow ENABLED");
                        that.leftArrowView.model.set("disabled", false);
                    }
                    if (disableRightArrow) {
                        console.log("updateArrowsStateForNewPosition RIGHT arrow DISABLED");
                        that.rightArrowView.model.set("disabled", true);
                    } else {
                        console.log("updateArrowsStateForNewPosition RIGHT arrow ENABLED");
                        that.rightArrowView.model.set("disabled", false);
                    }

                },
                getCurrentScrollValue: function () {
                    var ret = 0;
                    if (AppUtils.runWithIScroll()) {
                        ret = new WebKitCSSMatrix(window.getComputedStyle(this.iScrollWrapper[0]).webkitTransform).m41;
                        ret *= -1;
                    } else {
                        ret = this.el.scrollLeft;
                    }
                    return ret;
                },
                scrollTOC: function (newScrollValue, scrollTiming, callback) {
                    console.log("Will scroll thumbnails to " + newScrollValue);
                    if (AppUtils.runWithIScroll()) {
                        if (this.iScrollInstance) {
                            console.log("scrollTOC to position " + newScrollValue + " and timing " + scrollTiming);
                            this.iScrollInstance.scrollTo(-newScrollValue, 0, scrollTiming, IScroll.utils.ease.quadratic);
                        }
                    } else {
                        this.stopAnimation();
                        console.log("scrollTOC with animation");
                        this.scrollWithAnimationTo(newScrollValue, 0, scrollTiming, callback);
                    }
                    this.nextScrollPosition = newScrollValue;
                },
                getScrollWidth: function () {
                    return this.wrapperWidth;
                },
                getNewScrollValue: function (currentScrollValue, scrollStep, minScrollValue, maxScrollValue) {
                    if (minScrollValue == null) {
                        return Math.min(this.nextScrollPosition + scrollStep, maxScrollValue);
                    } else {
                        return Math.max(this.nextScrollPosition - scrollStep, minScrollValue);
                    }
                },
                scrollForDirection: function () {
                    var newScrollValue,
                        currentScrollValue = this.getCurrentScrollValue(),
                        maxScrollValue = this.getScrollWidth() - this.viewPortWidth,
                        minScrollValue = 0,
                        scrollStep = thumbs[0].width * 3, // thumb width already has menu scale factor included
                        scrollTiming = 500,
                        scrollDirection = this.model.get("scrollDirection");
                    if (scrollDirection == -1) {
                        console.log("Scroll toc left");
                        newScrollValue = this.getNewScrollValue(currentScrollValue, scrollStep, minScrollValue, null);
                        if (newScrollValue != currentScrollValue) {
                            this.scrollTOC(newScrollValue, scrollTiming,
                                function () {
                                    if (newScrollValue == minScrollValue) { // disable left arrow
                                        that.leftArrowView.model.set("disabled", true);
                                    }
                                    if (currentScrollValue == maxScrollValue) { // enable right arrow
                                        that.rightArrowView.model.set("disabled", false);
                                    }
                                });
                        }
                    } else if (scrollDirection == 1) {
                        console.log("Scroll toc right");
                        newScrollValue = this.getNewScrollValue(currentScrollValue, scrollStep, null, maxScrollValue);
                        if (newScrollValue != currentScrollValue) {
                            this.scrollTOC(newScrollValue, scrollTiming,
                                function () {
                                    if (newScrollValue == maxScrollValue) { // disable right arrow
                                        that.rightArrowView.model.set("disabled", true);
                                    }
                                    if (currentScrollValue == minScrollValue) { // enable left arrow
                                        that.leftArrowView.model.set("disabled", false);
                                    }
                                });
                        }
                    }
                    this.model.set("scrollDirection", 0);
                },

                initializeThumbnailCollection: function () {
                    var model, idx, currentLeft = -thumbMargin, thumb,
                        chapterSettings = Chapter.getSettings();

                    for (idx = 0; idx < thumbs.length; idx++) {
                        thumb = thumbs[idx];

                        thumb.width = Math.round(thumb.width * AppUtils.getMenuScaleFactor());
                        thumb.height = Math.round(thumb.height * AppUtils.getMenuScaleFactor());

                        currentLeft = currentLeft + 2 * thumbMargin;

                        model = new Backbone.Model(thumbs[idx]);
                        model.set("index", idx);
                        model.set("top", thumbMargin);
                        model.set("left", currentLeft);
                        model.set("sceneId", thumbs[idx].sceneId);
                        if (AppUtils.nativeApp || AppUtils.offlineExport) {
                            model.set("thumbURL", "thumbs/" + model.get("sceneId") + ".png");
                        } else {
                            // MOB-1898: we don't want thumb regeneration that often so on web browsers we do have fixed size for thumbs
                            model.set("thumbURL", "/thumb?action=getSceneThumbnail&sceneId=" +
                                model.get("sceneId") +
                                "&width=" + Math.round(App.settings.originalWidth / 5) +
                                "&height=" + Math.round(App.settings.originalHeight / 5) +
                                "&customerId=" + chapterSettings.customerId +
                                "&lastModified=" + thumbs[idx].lastModified +
                                "&sequenceId=" + chapterSettings.sequenceId);
                        }
                        model.set("activeThumb", false);
                        currentLeft += thumb.width;

                        this.thumbnailsCollection.push(model);
                    }

                    this.wrapperWidth = currentLeft;
                },

                renderNextThumbnail: function () {
                    if (this.nextThumbToRender < this.thumbnailsCollection.length) {
                        this.thumbnailsCollection[this.nextThumbToRender].set("loadThumbnail", true);
                        this.nextThumbToRender++;
                    } else {
                        console.log("MOB-1898: All thumbnails loaded");
                    }
                },

                createViewsForEachThumbnail: function (thumbnailsContainer) {
                    var idx, view;
                    for (idx = 0; idx < this.thumbnailsCollection.length; idx++) {
                        view = new that.ThumbnailView({
                            model: this.thumbnailsCollection[idx]
                        });
                        thumbnailsContainer.append(view.render().el);
                    }
                    // TODO: might be that we want thumbs loaded directly when in nativeApp !!!
                    this.nextThumbToRender = 0;
                    this.renderNextThumbnail();
                    // we want to update thumbnails and highlight the one for active page
                    if (null != Chapter.getGlobalModels().thumbnailWrapperModel) {
                        Chapter.getGlobalModels().thumbnailWrapperModel.set("activateThumbs", Chapter.getActivePage());
                    }
                },

                render: function () {
                    var thisView = this;

                    this.iScrollWrapper = $("<div>").width(this.wrapperWidth);

                    this.$el.html();
                    this.setTop(top);
                    this.setHeight(this.model.get("height"));
                    this.setLeft(this.model.get("left"));
                    this.setRight(this.model.get("right"));

                    if (AppUtils.runWithIScroll()) {
                        this.$el.append(this.iScrollWrapper);
                        this.createViewsForEachThumbnail(this.iScrollWrapper);
                        require(['iScroll'], function () {
                            thisView.iScrollInstance = new IScroll(thisView.el,
                                {
                                    disableMouse: true,
                                    disablePointer: true,
                                    preventDefault: true,
                                    scrollX: true,
                                    scrollY: false,
                                    stopPropagation: false,
                                    bindToWrapper: true
                                });
                            thisView.iScrollInstance.on('scrollEnd', function () {
                                console.log("iScrollInstance scrollEnd to " + this.x);
                                thisView.updateArrowsStateForNewPosition();
                            });
                        });
                    } else {
                        this.createViewsForEachThumbnail(this.$el);
                        this.$el.addClass("horizontalScroll");
                    }

                    this.updateArrowsStateForNewPosition();

                    return this;
                }
            });
            that.ThumbnailView = BaseView.extend({
                    tagName: "div",
                    className: "chapterBottomMenuThumbnailsClass",
                    template: templates.getTemplate('thumbnail'),
                    imageTemplate: templates.getTemplate('thumbnailImage'),
                    position: {},

                    initialize: function (attr) {
                        this.listenTo(this.model, "change:remove", this.removeView);
                        this.listenTo(this.model, "change:loadThumbnail", this.loadThumbnailHandler);
                        this.listenTo(this.model, "change:activeThumb", this.makeActive);
                    },
                    removeView: function () {
                        this.remove();
                        this.unbind();
                    },
                    expandCollapse: function () { // TODO: this method is not used at all can be removed in all places in player
                    },
                    events: function () {
                        var _events = {};
                        _events[AppUtils.EventTypes.mouseDown] = "stampCurrentPosition";
                        _events[AppUtils.EventTypes.mouseUp] = "navigateToPage";
                        return _events;
                    },
                    stampCurrentPosition: function (e) {
                        this.position = this.getPosition(e);
                    },
                    makeActive: function () {
                        if (this.model.get("activeThumb")) {
                            this.$el.find(".thumbShadow").addClass("chapterActivePageThumb");
                        } else {
                            this.$el.find(".thumbShadow").removeClass("chapterActivePageThumb");
                        }
                    },
                    getModel: function () {
                        return this.model;
                    },
                    navigateToPage: function (e) {
                        if (this.tapEvent(e)) {
                            console.log("navigateToPage " + e.type);
                            ZoomUtils.resetZoom();
                            var currentPage = Chapter.getActivePage(),
                                nextPage = this.model.get("pageNumber");
                            // update next page for dual landscape mode:
                            nextPage = App.dualLandscapeActive() ? parseInt(nextPage / 2) * 2 : nextPage;
                            console.log("Will go to page " + nextPage + " using bottom menu");
                            if (currentPage != nextPage) {
                                Chapter.getGlobalModels().thumbnailWrapperModel.set("activateThumbs", nextPage);
                                setTimeout(function () {
                                    AppUtils.navigateToPage(nextPage);
                                }, 5);
                            } else {
                                console.log("You are already on page " + nextPage);
                            }
                        }
                    },
                    loadThumbnailHandler: function () {
                        var thisView = this,
                            thumbImage = $(this.imageTemplate(this.model.attributes));
                        console.log("MOB-1898: Now render thumb for page " + this.model.get("index"));
                        thisView.$el.find(".thumbShadow").after(thumbImage);
                        thumbImage.bind("error", function () {
                            thumbImage.unbind("error").unbind("load");
                            console.log("MOB-1898: Failed to load image with url " + thumbImage.attr("src"));
                            thumbImage.attr("src", "gfx/menu/default_thumb.png");
                            thisView.$el.find(".thumbLoading").remove();
                            that.thumbnailWrapperView.renderNextThumbnail();
                        });
                        thumbImage.bind("load", function () {
                            thumbImage.unbind("load");
                            console.log("MOB-1898: Loaded image with url " + thumbImage.attr("src"));
                            thisView.$el.find(".thumbLoading").remove();
                            that.thumbnailWrapperView.renderNextThumbnail();
                        });
                    },
                    render: function () {
                        this.$el.html(this.template(this.model.attributes));
                        this.setWidth(this.model.get("width"));
                        this.setHeight(this.model.get("height"));
                        this.setTop(this.model.get("top"));
                        this.setLeft(this.model.get("left"));
                        return this;
                    }
                }
            )
            ;
        };
        this.instantiateObjects = function () {
            var thumbnailWrapperHeight = maxThumbHeight + 2 * thumbMargin,
                actionAreaHeight = thumbnailWrapperHeight + 2 * top;

            that.tocTabContentWrapperModel = new Backbone.Model({
                height: actionAreaHeight,
                name: 'toc_tab',
                removed: true
            });
            that.tocTabContentWrapperView = new that.TocTabContentWrapper({
                model: that.tocTabContentWrapperModel
            });

            adjustArrowModelForMenuScaleFactor(navigationArrows);

            that.leftArrowModel = getLeftArrowModel(navigationArrows.left);
            that.rightArrowModel = getRightArrowModel(navigationArrows.right);

            that.leftArrowView = new that.ArrowView({
                model: that.leftArrowModel
            });
            that.rightArrowView = new that.ArrowView({
                model: that.rightArrowModel
            });

            Chapter.getGlobalModels().thumbnailWrapperModel = new Backbone.Model({
                height: thumbnailWrapperHeight,
                width: App.settings.width - 2 * CSSConstants.MenuTocArrowWidth(),
                left: CSSConstants.MenuTocArrowWidth(),
                right: CSSConstants.MenuTocArrowWidth()
            });
            that.thumbnailWrapperView = new that.ThumbnailWrapperView({
                model: Chapter.getGlobalModels().thumbnailWrapperModel
            });

        };

        function adjustArrowModelForMenuScaleFactor(navigationArrows) {
            navigationArrows.left.height = Math.round(navigationArrows.left.height * AppUtils.getMenuScaleFactor());
            navigationArrows.left.width = Math.round(navigationArrows.left.width * AppUtils.getMenuScaleFactor());
            navigationArrows.right.height = Math.round(navigationArrows.right.height * AppUtils.getMenuScaleFactor());
            navigationArrows.right.width = Math.round(navigationArrows.right.width * AppUtils.getMenuScaleFactor());
        }

        function getLeftArrowModel(arrowRawModel) {
            var top = Math.round((that.tocTabContentWrapperModel.get("height") - arrowRawModel.height) / 2);
            return new Backbone.Model({
                direction: -1,
                height: arrowRawModel.height,
                width: arrowRawModel.width,
                left: 0,
                top: top,
                horizontalMargins: Math.round((CSSConstants.MenuTocArrowWidth() - arrowRawModel.width) / 2),
                imageURL: arrowRawModel.src
            });
        }

        function getRightArrowModel(arrowRawModel) {
            var top = Math.round((that.tocTabContentWrapperModel.get("height") - arrowRawModel.height) / 2);
            return new Backbone.Model({
                direction: 1,
                height: arrowRawModel.height,
                width: arrowRawModel.width,
                right: 0,
                top: top,
                horizontalMargins: Math.round((CSSConstants.MenuTocArrowWidth() - arrowRawModel.width) / 2),
                imageURL: arrowRawModel.src
            });
        }

        return {
            init: function () {
                that.defineClasses();
                that.instantiateObjects();

            },
            getModel: function () {
                return that.tocTabContentWrapperModel;
            },
            getView: function () {
                return that.tocTabContentWrapperView;
            }

        }
    }
});