var CommonActions = {
    browserPrefix: {
        firefox: {
            transform: 'transform',
            transitionEnd: 'transitionend',
            transition: '-moz-transition',
            transitionTimingFunction: '-moz-transition-timing-function',
            backFaceVisibility: '-moz-backface-visibility',
            perspective: '-moz-perspective'
        },
        webkit: {
            transform: '-webkit-transform',
            transitionEnd: 'webkitTransitionEnd',
            transition: '-webkit-transition',
            transitionTimingFunction: '-webkit-transition-timing-function',
            backFaceVisibility: '-webkit-backface-visibility',
            perspective: '-webkit-perspective'
        },
        opera: {
            transitionEnd: 'oTransitionEnd',
            transition: '-o-transition',
            transitionTimingFunction: '-o-transition-timing-function',
            backFaceVisibility: '-o-backface-visibility',
            perspective: '-o-perspective'
        },
        IE: {
            transitionEnd: 'transitionend',
            transition: 'transition',
            transitionTimingFunction: 'transition-timing-function',
            backFaceVisibility: 'backface-visibility',
            perspective: 'perspective'
        }
    },
    constants: {
        FADE_IN_TRANSITION_MS: 1000,
        FADE_OUT_TRANSITION_MS: 1000
    },
    currentBrowserPrefix: null,
    transitionsToBeRemoved: [],
    canRemoveTransition: true,
    oldTimingFunction: null,


    show: function ($element, opacity) {
        $element.css('opacity', opacity);
        $element.css('visibility', 'visible');
    },
    hide: function ($element, preventHide) {
        $element.css('opacity', 0);
        if (!preventHide) {
            $element.css('visibility', 'hidden');
        }
    },

    fadeIn1: function ($element, model, transitionEnd) {
        var opacity = model.get("opacity");
        $element.fadeTo(this.constants.FADE_IN_TRANSITION_MS, opacity, function () {
            if (!model.get("shouldNotRun")) {
                model.set('displayed', true);
                if (null != transitionEnd) {
                    transitionEnd();
                }
            } else {
                console.log("fadeIn: Canceled by page change");
                model.set("shouldNotRun", false);
            }
        });
    },

    bypassChromeFadeBug: function ($element) {
        if (AppUtils.DeviceType.CHROME) {
            if ($element.siblings(".chromeFadeBug").length == 0) {
                $element.before($("<div>").addClass("chromeFadeBug"));
            }
        }
    },

    fadeIn: function ($element, model, period, transitionEnd) {
        var opacity = model.get("opacity"),
            that = this;

        if (null == period) {
            period = this.constants.FADE_OUT_TRANSITION_MS;
        } else {
            period = parseFloat(period) / 2;
        }
        this.bypassChromeFadeBug($element);

        CSSHandler.removeTransitionEndHandler($element, "fadeIn");
        CSSHandler.setTransitionEndHandler($element, function (event) {
            //console.log("fadeIn transitionend changed property was " + event.originalEvent.propertyName);
            event.stopPropagation();
            if (event.originalEvent.propertyName == "opacity") {
                //console.log("fadeIn transitionend correct property !");
                CSSHandler.removeTransitionEndHandler($element, "fadeIn");
                if (null != transitionEnd) {
                    transitionEnd();
                }
            }
        }, "fadeIn");
        $element.css("visibility", "visible"); // make sure we have object displayed even if it has opacity 0
        this.processFadingAction($element, model, period, function () {
            if (model.get("id") == "SceneObject_30D6DC6F_28AD_4373_9842_1FAA42F05A0E") {
                //console.log("FADING_THE_PROBLEM_OBJECT");
                //console.log("FADING_THE_PROBLEM_OBJECT_OPACITY   " + opacity);
            }

            setTimeout(function () {
//                if (model.get("id") == "SceneObject_FDFD6161_2A25_499B_B431_AD146758AB97")
                //console.log("SHOWING_THE_PROBLEM_OBJECT");
                model.set('displayed', true);
                that.show($element, opacity);
            }, 0);

        });
    },

    fadeOut: function ($element, model, period, transitionEnd) {
        var that = this;
        if (null == period) {
            period = this.constants.FADE_OUT_TRANSITION_MS;
        }
        else {
            period = parseFloat(period) / 2;
        }
        this.bypassChromeFadeBug($element);

        CSSHandler.removeTransitionEndHandler($element, "fadeOut");
        CSSHandler.setTransitionEndHandler($element, function (event) {
            //console.log("fadeOut transitionend changed property was " + event.originalEvent.propertyName);
            event.stopPropagation();
            if (event.originalEvent.propertyName == "opacity") {
                //console.log("fadeOut transitionend correct property !");
                if (!ModelAccessFunctions.shouldNotHideBecauseOfEffects(model)) {
                    model.set('displayed', false);
                    $element.css("visibility", "hidden");
                }
                CSSHandler.removeTransitionEndHandler($element, "fadeOut");
                if (null != transitionEnd) {
                    transitionEnd();
                }
            }
        }, "fadeOut");
        this.processFadingAction($element, model, period, function () {
            setTimeout(function () {
                model.set('displayed', false);
                that.hide($element, true)
            }, 0);
        });
    },

    onePulseOnceOnClick: function ($element, model, period) {

        var that = this;
        model.set("effectInProgress", true);
        if (model.get('displayed')) {
            this.fadeOut($element, model, period, function () {
                that.fadeIn($element, model, function () {
                    model.set("effectInProgress", false);
                });
            });
        } else {
            this.fadeIn($element, model, period, function () {
                that.fadeOut($element, model, function () {
                    model.set("effectInProgress", false);
                });
            });
        }
    },
    onePulseOnClick: function ($element, model, period) {
        this.onePulseOnceOnClick($element, model, period);
        model.set("activateEffect", false);
    },
    multiplePulsesOnceOnClick: function ($element, model, period) {
        var that = this;

        if (model.get('displayed')) {
            console.log("multiplePulsesOnceOnClick now fade out");
            this.fadeOut($element, model, period, function () {
                console.log("multiplePulsesOnceOnClick callback now fade in");
                that.fadeIn($element, model, period, function () {
                    that.multiplePulsesOnceOnClick($element, model, period);
                });
            });
        } else {
            console.log("multiplePulsesOnceOnClick now fade in");
            this.fadeIn($element, model, period, function () {
                console.log("multiplePulsesOnceOnClick callback now fade out");
                that.fadeOut($element, model, period, function () {
                    that.multiplePulsesOnceOnClick($element, model, period);
                });
            });
        }
    },
    displayHideOnceOnClick: function ($element, model) {
        var visibility = model.get('displayed'),
            opacity = visibility ? 0 : model.get("opacity");
        this.show($element, opacity);
        model.set('displayed', !visibility);
    },

    fadeInOutOnceOnClick: function ($element, model) {
        var visibility = model.get('displayed'),
            opacity = visibility ? 0 : model.get("opacity"),
            that = this,
            callback = function () {
                model.set('displayed', !visibility);
                if (visibility) {
                    that.hide($element, true);
                } else {
                    that.show($element, opacity);
                }
            };
        this.processFadingAction($element, model, null, callback);
    },

    fadeInOutOnClick: function ($element, model) {
        this.fadeInOutOnceOnClick($element, model);
        model.set("activateEffect", false);
    },

    zoomInOCObject: function ($element, zoomPortInfo, zoomSettings) {
        console.log("zoomInOCObject called for element " + $element.attr("id"));
        var zoomSettingsForOC = this.getZoomSettingsForOC(zoomPortInfo),
            zoomDelay = Math.max(zoomSettings["zoomStartingPointDelayBeforeZooming"], 100),
            timingFunction = zoomSettings["useCustomZoomStartingPointTransitionTiming"] == "true" ? zoomSettings["zoomStartingPointTimingCustomFunction"] : zoomSettings["zoomStartingPointTimingFunction"],
            that = this;
        CSSHandler.setTransformOrigin($element, zoomSettingsForOC.Fx, zoomSettingsForOC.Fy);

        //Give time to apply transform origin property
        return setTimeout(function () {
            $element.css(that.currentBrowserPrefix.transition, zoomSettings["zoomStartingPointDuration"] + "ms");

            CSSHandler.setTransitionTimingFunction($element, timingFunction);

            CSSHandler.scale($element, zoomSettingsForOC.scaleFactor);
        }, zoomDelay);
    },

    zoomInStartingPoint: function ($element, model, zoomedInObjectInformation) {
        var zoomSettings = this.getZoomSettings(zoomedInObjectInformation),
            zoomDelay = Math.max(zoomedInObjectInformation.delay, 100),
            that = this;
        CSSHandler.setTransformOrigin($element, zoomSettings.Fx, zoomSettings.Fy);

        //Give time to apply transform origin property
        setTimeout(function () {
            $element.css(that.currentBrowserPrefix.transition, zoomedInObjectInformation.zoomStartingPointDuration + "ms");

            CSSHandler.setTransitionTimingFunction($element, zoomedInObjectInformation.timingFunction);
            model.set("transformStartTime", new Date().getTime());
            CSSHandler.scale($element, zoomSettings.scaleFactor);
        }, zoomDelay);
    },

    zoomOutStartingPoint: function ($element, model, zoomedOutObjectInformation) {
        var that = this,
            zoomDelay = Math.max(zoomedOutObjectInformation.delay, 100);

        //Give time to apply transform origin property
        setTimeout(function () {
            $element.css(that.currentBrowserPrefix.transition, zoomedOutObjectInformation.zoomStartingPointDuration + "ms");
            CSSHandler.setTransitionTimingFunction($element, zoomedOutObjectInformation.timingFunction);
            model.set("transformStartTime", new Date().getTime());
            CSSHandler.restoreTransformValues($element);
        }, zoomDelay);
    },

    getZoomSettings: function (model) {
        var ret = {};
        ret.Fx = model.width * model.left / (App.settings.width - model.width) + model.left;
        ret.Fy = model.height * model.top / (App.settings.height - model.height) + model.top;
        ret.scaleFactor = Math.min(App.settings.width / model.width, App.settings.height / model.height);
        return ret;
    },

    getZoomSettingsForOC: function (model) {
        var ret = {};
        ret.Fx = (model.width * model.left / (App.settings.width - model.width) + model.left) - model.objectLeft;
        ret.Fy = (model.height * model.top / (App.settings.height - model.height) + model.top) - model.objectTop;
        if (model.zoomFactor != null) {
            ret.scaleFactor = parseFloat(model.zoomFactor);
        } else {
            ret.scaleFactor = Math.min(App.settings.width / model.width, App.settings.height / model.height);
        }
        return ret;
    },

    getFadeInOutDuration: function (model, period) {
        if (null == period) {

            var visibility = model.get('displayed');
            return visibility ? this.constants.FADE_IN_TRANSITION_MS : this.constants.FADE_OUT_TRANSITION_MS;
        }
        return period;
    },
    processFadingAction: function ($element, model, period, callback) {
        var transitionsToBeRemoved = [], that = this,
            currentTimingFunction = "opacity " + this.getFadeInOutDuration(model, period) + "ms linear",
            transition = {};
        transitionsToBeRemoved.push('all');
        transitionsToBeRemoved.push('opacity');
        //console.log($element.css(this.currentBrowserPrefix.transition));
        //console.log("remove transition for fade");
        transition['transitionsToBeRemoved'] = transitionsToBeRemoved;
        transition['callback'] = function () {
            if (null != that.oldTimingFunction) {
                currentTimingFunction += ', ' + that.oldTimingFunction;
            }
            $element.css(that.currentBrowserPrefix.transition, currentTimingFunction);
            // On Android elements with backface visibility hidden will cause other elements not to show.
            // Happens only when resuming the app.
            if (!AppUtils.DeviceType.ANDROID) {
                $element.css(that.currentBrowserPrefix.backFaceVisibility, "hidden");
            }
//            $element.css(that.currentBrowserPrefix.perspective, 1000); //prevent a white line bellow fading elements
            //console.log($element.css(that.currentBrowserPrefix.transition));
            //console.log("fading effect started");
            if (null != callback)
                callback();
            that.canRemoveTransition = true;
            that.removeTransitionTiming();
        };
        transition['$element'] = $element;
        this.addActionToBeRemoved(transition);
    },

    setCurrentBrowserPrefix: function () {
        if (navigator.userAgent.indexOf('WebKit/') > -1) {
            this.currentBrowserPrefix = this.browserPrefix.webkit;
        } else if (navigator.userAgent.indexOf('Firefox/') > -1) {
            this.currentBrowserPrefix = this.browserPrefix.firefox;
        } else if (navigator.userAgent.indexOf('Opera/') > -1) {
            this.currentBrowserPrefix = this.browserPrefix.opera;
        } else if (navigator.userAgent.indexOf(' MSIE') > -1) {
            this.currentBrowserPrefix = this.browserPrefix.IE;
        } else {
            this.currentBrowserPrefix = this.browserPrefix.webkit;
        }
    },

    /**
     * Remove transition property for a specific effect
     * Should be called from each effect callback
     */
    removeTransitionTiming: function () {
        if (this.canRemoveTransition) {
            if (this.transitionsToBeRemoved.length > 0) {

                this.canRemoveTransition = false;
                var transition = this.transitionsToBeRemoved.shift(),
                    timingTypes = transition['transitionsToBeRemoved'], $element = transition['$element'],
                    currentTransitionTiming = $element.css(this.currentBrowserPrefix.transition);

                if (currentTransitionTiming == undefined || currentTransitionTiming == null)
                    currentTransitionTiming = "";

                var regex = 's, ',
                    transitionTimings = currentTransitionTiming.split(regex),
                    idx, count, shouldRemain;

                LinksHandler.logWithTimeStampFromSceneActivation("removeTransitionTiming start", $element.attr("id"));

                //console.log("removeTransitionTiming start for " + $element.attr("id"));

                this.oldTimingFunction = '';
                for (idx = 0; idx < transitionTimings.length; idx++) {
                    shouldRemain = true;
                    for (count = 0; count < timingTypes.length; count++) {
                        //console.log("transition to remove:  " + timingTypes[count]);
                        if (transitionTimings[idx].indexOf(timingTypes[count]) != -1) {
                            shouldRemain = false;
                        }
                    }

                    if (shouldRemain && this.oldTimingFunction.indexOf(transitionTimings[idx]) == -1) {
                        this.oldTimingFunction += transitionTimings[idx] + regex;
                    }
                }
                this.oldTimingFunction = this.oldTimingFunction.substring(0, this.oldTimingFunction.length - regex.length);
                this.oldTimingFunction = (this.oldTimingFunction.length == 0) ? null : this.oldTimingFunction;
                LinksHandler.logWithTimeStampFromSceneActivation("removeTransitionTiming before running callback", $element.attr("id"));
                if (null != transition['callback']) {
                    transition['callback']();
                }
                //console.log("removeTransitionTiming end for " + $element.attr("id"));
            }
        }
    },

    addActionToBeRemoved: function (transitions) {
        this.transitionsToBeRemoved.push(transitions);
        if (!this.canRemoveTransition) {
            LinksHandler.logWithTimeStampFromSceneActivation("Can't remove transition before moveTo", transitions["$element"].attr("id"))
        }
        this.removeTransitionTiming();
    }
};
CommonActions.setCurrentBrowserPrefix();

