var EffectsHandler = {
    browserPrefix: {
        firefox: {
            transform: 'transform',
            transitionEnd: 'transitionend',
            transition: '-moz-transition',
            transitionTimingFunction: '-moz-transition-timing-function',
            backFaceVisibility: '-moz-backface-visibility',
            perspective: '-moz-perspective'
        },
        webkit: {
            transform: '-webkit-transform',
            transitionEnd: 'webkitTransitionEnd',
            transition: '-webkit-transition',
            transitionTimingFunction: '-webkit-transition-timing-function',
            backFaceVisibility: '-webkit-backface-visibility',
            perspective: '-webkit-perspective'
        },
        opera: {
            transitionEnd: 'oTransitionEnd',
            transition: '-o-transition',
            transitionTimingFunction: '-o-transition-timing-function',
            backFaceVisibility: '-o-backface-visibility',
            perspective: '-o-perspective'
        },
        IE: {
            transitionEnd: 'transitionend',
            transition: 'transition',
            transitionTimingFunction: 'transition-timing-function',
            backFaceVisibility: 'backface-visibility',
            perspective: 'perspective'
        }
    },
    constants: {
        FADE_IN_TRANSITION_MS: 1000,
        FADE_OUT_TRANSITION_MS: 1000
    },
    currentBrowserPrefix: null,

    displayHideOnceOnClick: function ($element, model) {
        var visibility = model.get('displayed'),
            opacity = visibility ? 0 : model.get("opacity");
        console.log("toggleVisibility called will show " + visibility, model.get("id"), (new Date().getTime()) - window.sceneActivationTime);
        if (visibility) {
            CommonActions.hide($element, ModelAccessFunctions.shouldNotHideBecauseOfEffects(model));
        } else {
            CommonActions.show($element, opacity);
        }
        model.set('displayed', !visibility);

    },
    displayHideOnClick: function ($element, model) {
        this.displayHideOnceOnClick($element, model);
        model.set("activateEffect", false);
    },
    fadeInOutOnceOnClick: function ($element, model) {
        CommonActions.fadeInOutOnceOnClick($element, model);

    },
    toggleVisibility: function ($element, model) {
        this.displayHideOnceOnClick($element, model);
    },

    zoomInStartingPoint: function ($element, model, zoomedInObjectInformation) {
        CommonActions.zoomInStartingPoint($element, model, zoomedInObjectInformation);
    },

    zoomOutStartingPoint: function ($element, model, zoomedOutObjectInformation) {
        CommonActions.zoomOutStartingPoint($element, model, zoomedOutObjectInformation);
    },

    fadeInOutOnClick: function ($element, model) {
        this.fadeInOutOnceOnClick($element, model);
        model.set("activateEffect", false);
    },
    onePulseOnceOnClick: function ($element, model) {
        CommonActions.onePulseOnceOnClick($element, model, this.getPulsatePeriod(model));
    },
    onePulseOnClick: function ($element, model) {
        CommonActions.onePulseOnClick($element, model, this.getPulsatePeriod(model));
    },
    multiplePulsesOnceOnClick: function ($element, model, period) {
        if (null == period) {
            period = this.getPulsatePeriod(model);
        }
        CommonActions.multiplePulsesOnceOnClick($element, model, period);
    },
    multiplePulsesOnClick: function ($element, model) {
        this.multiplePulsesOnceOnClick($element, model, this.getPulsatePeriod(model));
        model.set("activateEffect", false);
    },
    setCurrentBrowserPrefix: function () {
        if (navigator.userAgent.indexOf('WebKit/') > -1) {
            this.currentBrowserPrefix = this.browserPrefix.webkit;
        } else if (navigator.userAgent.indexOf('Firefox/') > -1) {
            this.currentBrowserPrefix = this.browserPrefix.firefox;
        } else if (navigator.userAgent.indexOf('Opera/') > -1) {
            this.currentBrowserPrefix = this.browserPrefix.opera;
        } else if (navigator.userAgent.indexOf(' MSIE') > -1) {
            this.currentBrowserPrefix = this.browserPrefix.IE;
        } else {
            this.currentBrowserPrefix = this.browserPrefix.webkit;
        }
    },
    startRecordingWizard: function () {
        Chapter.getSettings()["playRecordedWizard"] = false;
        require([ "RecordingWizardLightBox", "SoundRecorder"], function (RecordingWizard) {
            var recordingWizardLightBox = new RecordingWizard();
            recordingWizardLightBox.init();
            SoundRecorder.setOperationMode(SoundRecorderOperationMode.MULTIPLE_PAGE);
            App.viewPortView.$el.append(recordingWizardLightBox.getView().render().el);
        });
    },
    startPlayRecordedSounds: function () {
        Chapter.getSettings()["playRecordedWizard"] = true;
    },
    stopPlayRecordedSounds: function () {
        Chapter.getSettings()["playRecordedWizard"] = false;
    },
    getPulsatePeriod: function (model) {
        var pulsatePeriod = 2000;
        if (null != model.get('pulsatePeriod')) {
            pulsatePeriod = model.get('pulsatePeriod');
            return pulsatePeriod;
        }
    }
};
EffectsHandler.setCurrentBrowserPrefix();

