var LinksHandler = {
    browserPrefix: {
        firefox: {
            transform: 'transform',
            transitionEnd: 'transitionend',

            transition: '-moz-transition'
        },
        webkit: {
            transform: '-webkit-transform',
            transitionEnd: 'webkitTransitionEnd',
            transition: '-webkit-transition'
        },
        opera: {
            transitionEnd: 'oTransitionEnd',
            transition: '-o-transition'
        },
        IE: {
            transitionEnd: 'transitionend',
            transition: 'transition'
        }
    },
    currentBrowserPrefix: null,

    moveBy: function (model, $element, activatedLinkObject) {
        if (null != activatedLinkObject) {
            var moveByXPixels = parseFloat(activatedLinkObject.moveByXPixels),
                moveByYPixels = parseFloat(activatedLinkObject.moveByYPixels),
                realXOffset = App.settings.width * moveByXPixels / App.settings.originalWidth,
                realYOffset = App.settings.height * moveByYPixels / App.settings.originalHeight;

            if (moveByXPixels || moveByYPixels) {
                setTimeout(function () {
                    CSSHandler.translate3D($element, realXOffset, realYOffset, 0, parseFloat(activatedLinkObject.moveByDuration), activatedLinkObject.moveByTimingFunction, null);
                }, 100);
            }
        }
    },

    logWithTimeStampFromSceneActivation: function (message, id) {
        console.log(message, id, (new Date().getTime()) - window.sceneActivationTime);
    },

    moveTo: function (model, $element, linkProperties) {
        var thisLinksHandler = this,
            elementId = $element.attr("id");
        thisLinksHandler.logWithTimeStampFromSceneActivation("MoveTo - called", elementId);
        var xOffset, yOffset, transitionsToBeRemoved = [], transition = {};

        transitionsToBeRemoved.push('all');


        if (linkProperties['onMoveEndEffect'] == "FADE_OUT") {
            transitionsToBeRemoved.push('opacity');

            // reset opacity to make object visible
            $element.css("display", "");
            $element.css("opacity", parseFloat(model.get("opacity")));
            CSSHandler.setVisibility($element, 'visible');
        }

        transitionsToBeRemoved.push(thisLinksHandler.currentBrowserPrefix.transform);
        thisLinksHandler.logWithTimeStampFromSceneActivation("MoveTo - remove transition for move", elementId);
        transition['transitionsToBeRemoved'] = transitionsToBeRemoved;
        transition['callback'] = function () {
            xOffset = Math.round(App.settings.width * (parseFloat(linkProperties['destinationOnXAxis']) - parseFloat(model.get("left"))) / 100);
            yOffset = Math.round(App.settings.height * (parseFloat(linkProperties['destinationOnYAxis']) - parseFloat(model.get("top"))) / 100);

            setTimeout(function () {
                thisLinksHandler.logWithTimeStampFromSceneActivation("MoveTo - transitionend event remove handler for before starting moveBack", elementId);
                if (linkProperties['onMoveEndEffect'] == "FADE_OUT") {
                    model.set('displayed', true);
                    LinksHandler.fadeOut(model, $element);
                } else if (linkProperties['onMoveEndEffect'] == "MOVE_BACK") {
                    thisLinksHandler.logWithTimeStampFromSceneActivation("MoveTo - prepare to execute moveBack with delay " + linkProperties['moveBackEffectDelay'], elementId);
                    setTimeout(function () {
                        thisLinksHandler.logWithTimeStampFromSceneActivation("MoveTo - run moveBack", elementId);
                        var moveBackTransition = {};
                        moveBackTransition['$element'] = $element;
                        moveBackTransition['transitionsToBeRemoved'] = [CSSHandler.getCurrentBrowserPrefix()["transform"]];
                        moveBackTransition['callback'] = function () {
                            CommonActions.canRemoveTransition = true;
                        };
                        CommonActions.addActionToBeRemoved(moveBackTransition);
                        CSSHandler.addTransition($element, linkProperties['moveBackTimingFunction'], CSSHandler.getCurrentBrowserPrefix()["transform"], linkProperties['moveBackEffectDuration']);
                        CSSHandler.removeBrowserSpecificProperty($element, "transform", "-webkit-transform");
                    }, linkProperties['moveBackEffectDelay']);
                }
            }, parseInt(linkProperties['moveToEffectDuration']));

            CSSHandler.restoreTransitionValues($element, CommonActions.oldTimingFunction);
            CSSHandler.restoreTransformValues($element);
            thisLinksHandler.logWithTimeStampFromSceneActivation("MoveTo - after restoreTransformValues", elementId);
            //Set a 0s timeout in order to allow the UI to be updated
            setTimeout(function () {
                thisLinksHandler.logWithTimeStampFromSceneActivation("MoveTo - now start move", elementId);
                console.log("GYL-414", "Move started at " + new Date().toISOString());
                model.set("transformStartTime", new Date().getTime());
                CSSHandler.translate3D($element, xOffset, yOffset, 0, parseFloat(linkProperties['moveToEffectDuration']), linkProperties['moveToTimingFunction'], CommonActions.oldTimingFunction, false);
                CommonActions.canRemoveTransition = true;
                CommonActions.removeTransitionTiming();
            }, 0);

        };
        transition['$element'] = $element;
        CommonActions.addActionToBeRemoved(transition);


    },

    fadeIn: function (model, $element) {
        console.log("fadeIn link action called");
        CommonActions.fadeIn($element, model);
    },

    fadeOut: function (model, $element) {
        CommonActions.fadeOut($element, model);
        console.log("fadeOut link action called");
    },

    printElement: function (model, $element) {
        $element.printElement();
        console.log("printElement link action called");
    },

    pulsate: function (model, $element) {

        CommonActions.multiplePulsesOnceOnClick($element, model, this.getPulsatePeriod(model));
        console.log("pulsate link action called");
    },

    show: function (model, $element) {
        CommonActions.show($element, model.get("opacity"));
        model.set('displayed', true);
        console.log("show link action called");
    },
    hide: function (model, $element) {
        CommonActions.hide($element, ModelAccessFunctions.shouldNotHideBecauseOfEffects(model));
        model.set('displayed', false);
        console.log("hide link action called");
    },
    setCurrentBrowserPrefix: function () {
        if (navigator.userAgent.indexOf('WebKit/') > -1) {
            this.currentBrowserPrefix = this.browserPrefix.webkit;
        } else if (navigator.userAgent.indexOf('Firefox/') > -1) {
            this.currentBrowserPrefix = this.browserPrefix.firefox;
        } else if (navigator.userAgent.indexOf('Opera/') > -1) {
            this.currentBrowserPrefix = this.browserPrefix.opera;
        } else if (navigator.userAgent.indexOf(' MSIE') > -1) {
            this.currentBrowserPrefix = this.browserPrefix.IE;
        } else {
            this.currentBrowserPrefix = this.browserPrefix.webkit;
        }
    },

    getPulsatePeriod: function (model) {
        var pulsatePeriod = 2000,
            links = model.get('links'), link;
        if (null != links.from) {
            link = this.getPulsateLink(links.from);
            if (null != link) {
                if (null != link.properties) {
                    pulsatePeriod = link.properties['pulsatePeriod'];
                }
            }
        }
        return pulsatePeriod;
    },
    getPulsateLink: function (links) {
        var index, link;
        for (index = 0; index < links.length; index++) {
            link = links[index];
            if (links[index]['type'] == 'pulsate') {
                link = links[index];
                break;
            }
        }
        return link;
    }
};
LinksHandler.setCurrentBrowserPrefix();

