/*
 syncGroupName: this should be a class name for all text related text boxes
 syncPointsForGroup: this is an array representing time at in a sound when each word should be highlighted
 leftRightThreshold: optional, you can set an integer value meaning all text boxes with left greater then this value will be considered on right column when doing smart arranging
 highlightColor: optional, color for highlight
 smartOrder: optional, either to use or no smart order
 soundURL: to be added
 */

(function ($) {

    var methods = {

        init: function (options) {

            function addSpanElementsToGroup() {
                var groupIndex = 0;
                for (var index = 0; index < $this.length; index++) {
                    var groupItem = $this.filter('[orderInGroup=' + index + ']');
                    if (!groupItem.data("hasSyncSpans")) {
                        groupItem.find("span:not(span:has(strong)), em, b, i, strong, p").each(function () {
                            var innerHTML = this.innerHTML;
                            var isEmptyElement = (innerHTML.replace(/&nbsp;|\s/g, "").length == 0);
                            if (innerHTML.indexOf("<span") == -1 && !isEmptyElement) {
                                //var textContent = this.innerText;
                                var textContent = $(this).html();
                                textContent = textContent.replace(/<br.*?>/g, "<br>");
                                console.log("SPAN TEXT : " + textContent);
                                var divTextSplitBySpaces = textContent.split(/[\s]+|(&nbsp;)+/gi);
                                var theHtml = "";
                                for (var word in divTextSplitBySpaces) {
                                    if (divTextSplitBySpaces[word] != ""
                                        && typeof(divTextSplitBySpaces[word]) != "undefined"
                                        && divTextSplitBySpaces[word] != "undefined"
                                        && divTextSplitBySpaces[word] != "&nbsp;") {
                                        theHtml += "<span class='usedForSync' id='" + options.syncGroupName + "-w" + groupIndex + "'>" + divTextSplitBySpaces[word] + " </span>";
                                        groupIndex++;
                                    }
                                }
                                this.innerHTML = theHtml;
                            }
                        });
                        groupItem.data("hasSyncSpans", true);
                    }
                }
            }

            function generateAutoSyncPoints() {
                options.syncPointsForGroup = [];
                for (var idx = 0; idx < $this.find(".usedForSync").length; idx++) {
                    options.syncPointsForGroup.push(idx * options.generateSyncPointsAtInterval);
                }
            }

            function playSoundWithHighlight() {
                var elementToBeHighlighted = $("#" + options.syncGroupName + "-w0");

                console.log("Will highlight word no. 0");
                if (options.useCustomHighlight) {
                    elementToBeHighlighted.css(options.cssBrowserPrefix + "transition-duration", options.syncPointsForGroup[0] + "ms")
                        .css(options.cssBrowserPrefix + "transition-property", "all")
                        .css(options.cssBrowserPrefix + "transition-timing-function", "linear");
                    elementToBeHighlighted.addClass("highlightingText");
                } else {
                    elementToBeHighlighted.css(options.cssBrowserPrefix + "transition", "background-color " + options.syncPointsForGroup[0] + "ms linear");
                    elementToBeHighlighted.css("background-color", options.highlightColor);
                }
                var T1 = setTimeout(function () {
                    highlightNextIfMore(1);
                }, options.syncPointsForGroup[0]);
                $this.data("T1", T1);
                console.log("Timer with delay " + options.syncPointsForGroup[0] + "scheduled with id " + T1);
            }

            function highlightNextIfMore(currentSyncPoint) {
                if (currentSyncPoint < options.syncPointsForGroup.length) {
                    console.log("Will highlight word no. " + currentSyncPoint);
                    var syncInterval = options.syncPointsForGroup[currentSyncPoint] - options.syncPointsForGroup[currentSyncPoint - 1];
                    var elementToBeHighlighted = $("#" + options.syncGroupName + "-w" + currentSyncPoint);
                    if (options.useCustomHighlight) {
                        elementToBeHighlighted.css(options.cssBrowserPrefix + "transition-duration", syncInterval + "ms")
                            .css(options.cssBrowserPrefix + "transition-property", "all")
                            .css(options.cssBrowserPrefix + "transition-timing-function", "linear");
                        elementToBeHighlighted.addClass("highlightingText");
                    } else {
                        elementToBeHighlighted.css(options.cssBrowserPrefix + "transition", "background-color " + syncInterval + "ms linear");
                        elementToBeHighlighted.css("background-color", options.highlightColor);
                    }
                    var T2 = setTimeout(function () {
                        currentSyncPoint++;
                        highlightNextIfMore(currentSyncPoint);
                    }, syncInterval);
                    $this.data("T2", T2);
                    console.log("Timer with delay " + syncInterval + "scheduled with id " + T2);
                }
            }

            function orderLeftRightTopBottom() {
                var arrayToBeSorted = [], i;
                $this.each(function () {
                    var parentEl = $(this).parent();
                    arrayToBeSorted.push({
                        "$element": $(this),
                        "elemTop": parentEl.position().top,
                        "elemLeft": parentEl.position().left
                    });
                });
                for (i = 0; i < arrayToBeSorted.length - 1; i++) {
                    for (var j = i + 1; j < arrayToBeSorted.length; j++) {
                        if (arrayToBeSorted[i]["elemTop"] > arrayToBeSorted[j]["elemTop"]) {
                            var temp = arrayToBeSorted[i];
                            arrayToBeSorted[i] = arrayToBeSorted[j];
                            arrayToBeSorted[j] = temp;
                        }
                    }
                }
                if (options.leftRightThreshold != 0) {
                    var rightArray = [];
                    var leftArray = [];
                    for (i = 0; i < arrayToBeSorted.length; i++) {
                        if (arrayToBeSorted[i]["elemLeft"] < options.leftRightThreshold) {
                            leftArray.push(arrayToBeSorted[i]);
                        } else {
                            rightArray.push(arrayToBeSorted[i]);
                        }
                    }
                    arrayToBeSorted = leftArray.concat(rightArray);
                }
                for (i = 0; i < arrayToBeSorted.length; i++) {
                    arrayToBeSorted[i]["$element"].attr("orderInGroup", i);
                }
            }

            var $this = $(this);
            // here we could save all options to be used in other methods than init

            if ($this.length > 0) {
                if (options.smartOrder) {
                    orderLeftRightTopBottom();
                }
                addSpanElementsToGroup();
                if (options.generateSyncPointsAtInterval != 0) {
                    generateAutoSyncPoints();
                }
                // have a small delay here so that first highlight will go with animation,
                // on iPad also works without delay 0
                setTimeout(playSoundWithHighlight, 0);
            }

            return this;
        },

        stopSyncAndRemoveHighlight: function (options) {
            console.log("stopSyncAndRemoveHighlight");
            // stop the sync process
            clearTimeout($(this).data("T1"));
            clearTimeout($(this).data("T2"));
            console.log("Timer cleared with id " + $(this).data("T1"));
            console.log("Timer cleared with id " + $(this).data("T2"));
            // now go and remove existing highlights but keep spans for further use
            // anyway they will be deleted if not in active segment
            var syncSpans = $(this).find(".usedForSync");
            syncSpans.css(options.cssBrowserPrefix + "transition-duration", "1000ms");
            if (options.useCustomHighlight) {
                syncSpans.removeClass("highlightingText");
            } else {
                syncSpans.css("background-color", "");
            }

            return this;
        }
    };

    $.fn.syncTextWithAudio = function (method) {

        // set up default options
        var defaults = {
            smartOrder: false,
            leftRightThreshold: 0,
            highlightColor: 'lime',
            generateSyncPointsAtInterval: 0,
            useCustomHighlight: false,
            cssBrowserPrefix: CSSHandler.getCssPrefix()
        };

        var options;

        // Method calling logic
        if (methods[method]) {
            options = $.extend({}, defaults, arguments[1]);
            return methods[ method ].call(this, options);
        } else if (typeof method === 'object' || !method) {
            options = $.extend({}, defaults, arguments[0]);
            return methods.init.call(this, options);
        } else {
            $.error('Method ' + method + ' does not exist on jQuery.syncTextWithAudio');
        }
    };

})(jQuery);