define(function () {
    return function (lightBoxSettings, windowSettings, lightBoxContentSettings) {
        var that = this;

        this.windowSettings = {
            width: '80%',
            height: '80%',
            opacity: '1',
            closeButton: 'on',
            horizontalMargin: '0%',
            verticalMargin: '0%',
            content: null
        };
        this.lightBoxSettings = {
            width: '100%',
            height: '100%',
            opacity: '0.5',
            closeAction: null
        };

        this.contentSettings = {
            width: '100%',
            height: '100%',
            opacity: '1',
            content: null
        };
        this.defineClasses = function () {

            that.GenericLightBoxView = BaseView.extend({
                tagName: "div",
                id: 'GenericLightBoxView',
                $overlay: null,
                $lightBoxOverlay: null,
                initialize: function (attr) {
                    this.listenTo(this.model, "change:remove", this.removeView);
                },
                events: function () {
                    var _events = {};
                    _events[AppUtils.EventTypes.click] = "clickHandler";
                    return _events;
                },
                mouseDownHandler: function (event) {
                    event.stopPropagation();
                },
                mouseMoveHandler: function (event) {
                    event.stopPropagation();
                },
                mouseUpHandler: function (event) {
                    event.stopPropagation();
                },
                clickHandler: function (event) {
                    event.stopPropagation();
                    this.model.set("remove", true);
                },
                removeView: function () {
                    var closeAction;
                    closeAction = this.model.get('closeAction');
                    if (null != closeAction) {
                        closeAction();
                    }
                    this.$overlay.unbind();
                    this.$overlay.remove();
                    this.$lightBoxOverlay.unbind();
                    this.$lightBoxOverlay.remove();
                    this.$el.append(that.genericLightBoxWindowView.render().el);
                    that.genericLightBoxWindowModel.set("remove,true");
                    this.remove();
                    this.unbind();
                },
                appendOverlayInBody: function () {
                    var thisView = this;
                    this.$overlay = $('<div>').attr('id', 'GenericLightBoxViewportCloseOverlay');
                    $('#viewPort').before(this.$overlay);
                    this.$overlay.bind(AppUtils.EventTypes.mouseDown, function (e) {
                        e.stopPropagation();
                    });
                    this.$overlay.bind(AppUtils.EventTypes.mouseUp, function (e) {
                        e.stopPropagation();
                        thisView.removeView();
                    });

                },
                appendOverlayInLightBox: function () {
                    var thisView = this;
                    this.$lightBoxOverlay = $('<div>').attr('id', 'GenericLightBoxCloseOverlay');
                    this.$el.append(this.$lightBoxOverlay);
                    this.$lightBoxOverlay.bind(AppUtils.EventTypes.mouseDown, function (e) {
                        e.stopPropagation();
                    });
                    this.$lightBoxOverlay.bind(AppUtils.EventTypes.mouseUp, function (e) {
                        e.stopPropagation();
                        thisView.removeView();
                    });

                },
                render: function () {
                    this.appendOverlayInBody();
                    this.appendOverlayInLightBox();
                    this.setWidth(this.model.get("width"));
                    this.setHeight(this.model.get("height"));
                    this.setBackgroundColor(this.model.get("color"), this.model.get("opacity"));
                    this.$el.append(that.genericLightBoxWindowView.render().el);
                    return this;
                }
            });

            that.GenericLightBoxWindowView = BaseView.extend({
                tagName: "div",
                id: 'GenericLightBoxWindowView',


                initialize: function (attr) {
                    this.listenTo(this.model, "change:remove", this.removeView);
                },
                removeView: function () {
                    this.$overlay.unbind();
                    this.$overlay.remove();
                    that.genericLightBoxCloseModel.set("remove,true");
                    this.remove();
                    this.unbind();
                },
                appendCloseButtonIfNeeded: function () {
                    if (this.model.get('closeButton') == 'on') {
                        this.$el.append(that.genericLightBoxCloseView.render().el);
                    }
                },

                render: function () {
                    this.setWidth(this.model.get("width"));
                    this.setHeight(this.model.get("height"));
                    this.setLeft(this.model.get("left"));
                    this.setTop(this.model.get("top"));
                    this.setHorizontalMargin(this.model.get("marginLeft"), this.model.get("marginRight"));
                    this.setVerticalMargin(this.model.get("marginTop"), this.model.get("marginBottom"));
                    this.setBackgroundColor(this.model.get("color"), this.model.get("opacity"));

                    this.$el.append(that.genericLightBoxContentView.render().el);
                    this.appendCloseButtonIfNeeded();
                    return this;
                }
            });

            that.GenericLightBoxContentView = BaseView.extend({
                tagName: "div",
                id: 'GenericLightBoxContentView',


                initialize: function (attr) {
                    this.listenTo(this.model, "change:remove", this.removeView);
                },
                events: function () {
                    var _events = {};
//                    _events[AppUtils.EventTypes.mouseDown] = "mouseDownHandler";
                    _events[AppUtils.EventTypes.click] = "clickHandler";
//                    _events[AppUtils.EventTypes.mouseUp] = "mouseUpHandler";
                    return _events;
                },
                mouseDownHandler: function (event) {
                    event.stopPropagation();
                },
                mouseUpHandler: function (event) {
                    event.stopPropagation();
                },
                clickHandler: function (event) {
                    event.stopPropagation();
                },
                removeView: function () {
                    that.genericLightBoxCloseModel.set("remove,true");
                    this.remove();
                    this.unbind();
                },
                getContentFrom: function (originalContent) {
                    if (originalContent instanceof jQuery) {
                        return originalContent[0];
                    }
                    return originalContent;
                },
                render: function () {
                    var originalContent;
                    this.setWidth(this.model.get("width"));
                    this.setHeight(this.model.get("height"));
                    this.setLeft('50%');
                    this.setTop('50%');
                    this.setHorizontalMargin(this.model.get("horizontalMargin"), this.model.get("horizontalMargin"));
                    this.setVerticalMargin(this.model.get("verticalMargin"), this.model.get("verticalMargin"));
                    this.setBackgroundColor(this.model.get("color"), this.model.get("opacity"));
                    originalContent = this.model.get('content');
                    if (null != originalContent) {
                        this.$el.append(this.getContentFrom(originalContent))
                    }
                    return this;
                }
            });

            that.GenericLightBoxCloseView = BaseView.extend({
                tagName: "div",
                id: 'GenericLightBoxCloseView',


                initialize: function (attr) {
                    this.listenTo(this.model, "change:remove", this.removeView);
                },
                events: function () {
                    var _events = {};
                    _events[AppUtils.EventTypes.click] = "clickHandler";
                    return _events;
                },
                removeView: function () {
                    this.remove();
                    this.unbind();
                },
                clickHandler: function (event) {
                    event.stopPropagation();
                    event.preventDefault();
                    that.genericLightBoxModel.set("remove", true);

                },
                render: function () {
                    return this;
                }
            });
        };
        this.instantiateObjects = function () {
            this.setGenericLightBoxWindowSettings();
            this.setGenericLightBoxSettings();
            this.setGenericLightBoxContentSettings();

            this.genericLightBoxModel = new Backbone.Model(this.lightBoxSettings);
            this.genericLightBoxWindowModel = new Backbone.Model(this.windowSettings);
            this.genericLightBoxContentModel = new Backbone.Model(this.contentSettings);
            this.genericLightBoxCloseModel = new Backbone.Model({
                remove: false
            });

            this.genericLightBoxView = new this.GenericLightBoxView({
                model: that.genericLightBoxModel
            });
            this.genericLightBoxWindowView = new this.GenericLightBoxWindowView({
                model: that.genericLightBoxWindowModel
            });
            this.genericLightBoxContentView = new this.GenericLightBoxContentView({
                model: that.genericLightBoxContentModel
            });
            this.genericLightBoxCloseView = new this.GenericLightBoxCloseView({
                model: that.genericLightBoxCloseModel
            });
        };

        this.setGenericLightBoxContentSettings = function () {
            var leftMargin, topMargin, wrapperWidth, wrapperHeight, width, height, property;

            for (property in lightBoxContentSettings) {
                if (lightBoxContentSettings.hasOwnProperty(property)) {
                    this.contentSettings[property] = lightBoxContentSettings[property]
                }
            }

            leftMargin = parseFloat(this.windowSettings.horizontalMargin);
            topMargin = parseFloat(this.windowSettings.verticalMargin);
            wrapperWidth = this.windowSettings.width;
            wrapperHeight = this.windowSettings.height;

            leftMargin = (leftMargin * wrapperWidth / 100);
            width = wrapperWidth - 2 * leftMargin;
            topMargin = (topMargin * wrapperHeight / 100);
            height = wrapperHeight - 2 * topMargin;

            this.contentSettings.width = (100 * width / wrapperWidth) + '%';
            this.contentSettings.height = (100 * height / wrapperHeight) + '%';
            this.contentSettings.horizontalMargin = ((-100) * (width / 2) / wrapperWidth) + '%';
            this.contentSettings.verticalMargin = ((-100) * (height / 2) / wrapperWidth) + '%';

        };
        this.setGenericLightBoxSettings = function () {
            var property;
            for (property in lightBoxSettings) {
                if (lightBoxSettings.hasOwnProperty(property)) {
                    this.lightBoxSettings[property] = lightBoxSettings[property]
                }
            }
            this.lightBoxSettings.left = 0;
            this.lightBoxSettings.top = 0;
            this.lightBoxSettings.remove = false;
        };

        this.setGenericLightBoxWindowSettings = function () {
            var property;
            for (property in windowSettings) {
                if (windowSettings.hasOwnProperty(property)) {
                    this.windowSettings[property] = windowSettings[property]
                }
            }
            this.windowSettings.width = this.getValidValue(App.settings.width, windowSettings.width, this.windowSettings.width);
            this.windowSettings.height = this.getValidValue(App.settings.height, windowSettings.height, this.windowSettings.height);

            this.windowSettings.left = '50%';
            this.windowSettings.top = '50%';
            this.windowSettings.marginLeft = this.getMarginLeft(this.windowSettings.width);
            this.windowSettings.marginTop = this.getMarginTop(this.windowSettings.height);
            this.windowSettings.marginRight = this.windowSettings.marginLeft;
            this.windowSettings.marginBottom = this.windowSettings.marginTop;
            this.windowSettings.remove = false;
        };

        this.getMarginLeft = function (width) {
//            var validWidth = this.getValidValue(App.settings.width, this.windowSettings.width);
            return Math.round((-100) * (width / 2) / App.settings.width) + '%';
        };

        this.getMarginTop = function (height) {
//            var validHeight = this.getValidValue(App.settings.height, this.windowSettings.height);
            return Math.round((-100) * (height / 2) / App.settings.width) + '%';
        };


        this.getValidValue = function (viewPortValue, value, defaultValue) {
            var ret = defaultValue;
            if (null != value) {
                if (typeof value == 'number') {
                    //do nothing. This is the value we are looking for
                } else {
                    if (value.indexOf("px") == -1) {
                        if (value.indexOf("%") == -1) {
                            ret = null;
                        } else {
                            ret = parseFloat(value) * viewPortValue / 100;
                        }
                    } else {
                        ret = parseFloat(value);
                    }
                }
            }
            return ret;
        };

        return {
            init: function () {
                that.defineClasses();
                that.instantiateObjects();
            },
            getView: function () {
                return that.genericLightBoxView;
            },
            getModel: function () {
                return that.genericLightBoxModel;
            },
            getContentView: function () {
                return that.genericLightBoxContentView;
            }
        }
    }
})
;