define(function () {
    return function (bottomMenuModel) {
        var that = this;


        this.defineClasses = function () {

            that.RecordingLightBoxView = Backbone.View.extend({
                tagName: "div",
                id: 'RecordingLightBoxView',


                initialize: function (attr) {
                    this.listenTo(this.model, "change:displayedMessage", this.updateWizard);
                    this.listenTo(this.model, "change:RecordingCountDownViewClassName", this.updateRecordingCountDownViewClassName);
                    this.listenTo(this.model, "change:recordingFinished", this.recordingFinished);
                    this.listenTo(this.model, "change:interruptCountDown", this.abortRecordingWithoutSaving);
                },
                events: function () {
                    var _events = {};
                    _events[AppUtils.EventTypes.mouseDown] = "mouseDownHandler";
                    return _events;
                },
                abortRecordingWithoutSaving: function () {
                    this.removeView();
                },
                removeView: function () {
                    this.remove();
                    this.unbind();
                    bottomMenuModel.set("expanded", MenuState.FULL_EXPANDED);
                },
                mouseDownHandler: function () {
                    SoundRecorder.toggleSoundRecordingForPage(Chapter.getActivePage());
                },
                recordingFinished: function () {
                    if (this.model.get('recordingFinished')) {
                        this.model.set('recordingFinished', false);
                        this.removeView();
                    }
                },
                updateWizard: function () {
                    that.recordingCountDownModel.set("displayedMessage", this.model.get("displayedMessage"));
                    that.recordingBottomMenuInfoMessageViewModel.set("displayedMessage", this.model.get("displayedMessage"));
                    that.recordingBottomMenuMicrofonViewModel.set("startPulsate", this.model.get("displayedMessage") == "");
                    that.recordingBottomMenuMicrofonViewModel.set("endPulsate", this.model.get("displayedMessage") == CAPTIONS.SAVING_SOUND);
                },
                updateRecordingCountDownViewClassName: function () {
                    that.recordingCountDownModel.set("RecordingCountDownViewClassName", this.model.get("RecordingCountDownViewClassName"));
                },
                render: function () {
                    this.$el.append(that.recordingCountDownView.render().el)
                        .append(that.recordingBottomMenuView.render().el);
                    return this;
                }
            });
            that.RecordingCountDownView = Backbone.View.extend({
                tagName: "div",
                id: 'RecordingCountDownView',
                template: templates.getTemplate('recordingLightBoxWizardMessage'),
                $messageWrapper: null,
                initialize: function (attr) {
                    this.listenTo(this.model, "change:displayedMessage", this.updateDisplayedMessage);
                    this.listenTo(this.model, "change:RecordingCountDownViewClassName", this.updateRecordingCountDownViewClassName);
                },
                removeView: function () {
                    if (this.model.get("removed")) {
                        this.remove();
                        this.unbind();
                    }
                },
                getMessageWrapper: function(){
                    if (null == this.$messageWrapper) {
                        this.$messageWrapper = $('#recordingLightBoxWizardMessageWrapper', this.el);
                    }
                    return this.$messageWrapper;
                },
                updateDisplayedMessage: function () {
                    this.getMessageWrapper().html(this.model.get("displayedMessage"));
                },
                updateRecordingCountDownViewClassName: function () {
                    this.getMessageWrapper().attr("class", this.model.get("RecordingCountDownViewClassName"));
                },
                render: function () {
                    this.$el.html(this.template(this.model.attributes));
                    return this;
                }
            });
            that.RecordingBottomMenuView = Backbone.View.extend({
                tagName: "div",
                id: 'RecordingBottomMenuView',

                removeView: function () {
                    if (this.model.get("removed")) {
                        this.remove();
                        this.unbind();
                    }
                },
                render: function () {
                    this.$el.append(that.recordingBottomMenuMicrophoneView.render().el)
                        .append(that.recordingBottomMenuInfoMessageView.render().el);
                    this.$el.height(CSSConstants.DefaultActiveTabWrapperHeight());
                    return this;
                }
            });
            that.RecordingBottomMenuInfoMessageView = Backbone.View.extend({
                tagName: "div",
                id: 'RecordingBottomMenuInfoMessageView',
                template: templates.getTemplate('recordingLightBoxWizardBottomMenuMessage'),
                $messageWrapper: null,

                initialize: function (attr) {
                    this.listenTo(this.model, "change:displayedMessage", this.updateDisplayedMessage);
//                    this.listenTo(this.model, "change:RecordingCountDownViewClassName", this.updateRecordingCountDownViewClassName);
                },
                updateDisplayedMessage: function () {
                    if (null == this.$messageWrapper) {
                        this.$messageWrapper = $('#recordingLightBoxWizardBottomMenuMessageWrapper', this.el);
                    }
                    var displayedMessage = this.model.get('displayedMessage');
                    if ('' == displayedMessage || CAPTIONS.SAVING_SOUND == displayedMessage) {
                        this.$messageWrapper.html(CAPTIONS.END_RECORDING_MESSAGE);
                    } else {
                        this.$messageWrapper.html(CAPTIONS.RECORDING_ABOUT_TO_START_PART_1 + ' ' + displayedMessage + ' ' + CAPTIONS.RECORDING_ABOUT_TO_START_PART_2);
                    }
                },
                removeView: function () {
                    if (this.model.get("removed")) {
                        this.remove();
                        this.unbind();
                    }
                },
                render: function () {
                    this.$el.html(this.template(this.model.attributes));
                    return this;
                }
            });
            that.RecordingBottomMenuMicrophoneView = Backbone.View.extend({
                tagName: "div",
                id: 'RecordingBottomMenuMicrophoneView',
                template: templates.getTemplate('recordingLightBoxMicrophoneImage'),
//                className: 'pulsateEffect',
                initialize: function (attr) {
                    this.listenTo(this.model, "change:startPulsate", this.addPulsateEffect);
                    this.listenTo(this.model, "change:endPulsate", this.removePulsateEffect);
                },
                addPulsateEffect: function () {
                    if (this.model.get("startPulsate")) {
                        this.model.set("startPulsate", false);
                        this.$el.addClass("pulsateEffect");
                    }
                },
                removePulsateEffect: function () {
                    if (this.model.get("endPulsate")) {
                        this.model.set("endPulsate", false);
                        this.$el.removeClass("pulsateEffect");
                    }
                },
                removeView: function () {
                    if (this.model.get("removed")) {
                        this.remove();
                        this.unbind();
                    }
                },
                render: function () {
                    this.$el.html(this.template(this.model.attributes));
                    return this;
                }
            });
        };
        this.instantiateObjects = function () {
            this.recordingLightBoxModel = new Backbone.Model({
                displayedMessage: null
            });

            this.recordingCountDownModel = new Backbone.Model();

            this.recordingBottomMenuModel = new Backbone.Model();

            this.recordingBottomMenuMicrofonViewModel = new Backbone.Model();

            this.recordingBottomMenuInfoMessageViewModel = new Backbone.Model();


            this.recordingLightBoxView = new this.RecordingLightBoxView({
                model: that.recordingLightBoxModel
            });
            this.recordingCountDownView = new this.RecordingCountDownView({
                model: that.recordingCountDownModel
            });
            this.recordingBottomMenuView = new this.RecordingBottomMenuView({
                model: that.recordingBottomMenuModel
            });
            this.recordingBottomMenuMicrophoneView = new this.RecordingBottomMenuMicrophoneView({
                model: that.recordingBottomMenuMicrofonViewModel
            });
            this.recordingBottomMenuInfoMessageView = new this.RecordingBottomMenuInfoMessageView({
                model: that.recordingBottomMenuInfoMessageViewModel
            });

        };


        return {
            init: function () {
                that.defineClasses();
                that.instantiateObjects();
            },
            getView: function () {
                return that.recordingLightBoxView;
            },
            getModel: function () {
                return that.recordingLightBoxModel;
            },
            getRecordingCountDownModel: function () {
                return that.recordingCountDownModel;
            }
        }
    }
});