define(function () {
    return function () {
        var that = this;


        this.defineClasses = function () {

            that.RecordingLightBoxView = Backbone.View.extend({
                tagName: "div",
                id: 'RecordingLightBoxView',
                lastPageToRecordOn: 0,
                shouldAbortRecording: false,


                initialize: function (attr) {
                    this.listenTo(this.model, "change:displayedMessage", this.updateWizard);
                    this.listenTo(this.model, "change:RecordingCountDownViewClassName", this.updateRecordingCountDownViewClassName);
                    this.listenTo(this.model, "change:recordingFinished", this.recordingFinished);
                    this.listenTo(this.model, "change:interruptCountDown", this.abortRecordingWithoutSaving);
                    this.listenTo(this.model, "change:userAbortRecording", this.abortRecordingWithSaving);
                },
                events: function () {
                    var _events = {};
                    _events[AppUtils.EventTypes.mouseDown] = "mouseDownHandler";
                    return _events;
                },

                abortRecordingWithSaving: function () {
                    this.shouldAbortRecording = this.model.get('userAbortRecording');
                },
                abortRecordingWithoutSaving: function () {
                    SoundRecorder.setOperationMode(SoundRecorderOperationMode.MULTIPLE_PAGE_FINISHED);
                    this.removeView();
                },
                removeView: function () {
                    this.remove();
                    this.unbind();
                },
                mouseDownHandler: function (e) {
                    e.preventDefault();
                    e.stopPropagation();
                    if (this.model.get("displayedMessage") == "")
                        SoundRecorder.toggleSoundRecordingForPage(Chapter.getActivePage());
                },
                recordingFinished: function () {
                    if (this.model.get('recordingFinished')) {
                        this.model.set('recordingFinished', false);
                        if (this.shouldAbortRecording) {
                            this.abortRecordingWithoutSaving();
                            Chapter.setActivePage(0);
                            document.getElementById("slidingScenesWrapper").scrollLeft = 0;
                        } else {
                            this.navigateToNextPageAndStartRecording();
                        }
                    }
                },
                updateWizard: function () {
                    that.recordingCountDownModel.set("displayedMessage", this.model.get("displayedMessage"));
                    that.recordingBottomMenuInfoMessageViewModel.set("displayedMessage", this.model.get("displayedMessage"));
                    that.recordingBottomMenuModel.set('visible', this.model.get("displayedMessage") == "");
                    that.recordingBottomMenuMicrofonViewModel.set("startPulsate", this.model.get("displayedMessage") == "");
                    that.recordingBottomMenuMicrofonViewModel.set("endPulsate", this.model.get("displayedMessage") == CAPTIONS.SAVING_SOUND);
                },
                updateRecordingCountDownViewClassName: function () {
                    that.recordingCountDownModel.set("RecordingCountDownViewClassName", this.model.get("RecordingCountDownViewClassName"));
                },

                navigateToNextPageAndStartRecording: function () {
                    var thisView = this;
                    console.log("navigateToPage tapEvent");
                    var currentPage = Chapter.getActivePage(),
                        nextPage = currentPage + 1,
                        translateOffset;
                    this.$el.hide();
                    that.recordingBottomMenuModel.set('visible', false);
                    if (this.lastPageToRecordOn > nextPage) {
                        console.log("Will go to page " + nextPage + " using bottom menu");
                        translateOffset = Chapter.computeTranslateOffset(currentPage, nextPage);
                        Chapter.translateScenesTo(translateOffset, function () {
                            Chapter.removeScrollingElementTransitionEndHandler();
                            Chapter.setActivePage(nextPage);
                            if (typeof SoundPlayer != 'undefined') {
                                SoundPlayer.endAllPlayingSoundsForPage(nextPage, false);
                            }

                            thisView.$el.show();
                            SoundRecorder.toggleSoundRecordingForPage(Chapter.getActivePage());
                        });
                    } else {
                        this.abortRecordingWithoutSaving();
                        Chapter.setActivePage(0);
                        document.getElementById("slidingScenesWrapper").scrollLeft = 0;
                        console.log("We are already on last page" + nextPage);
                    }
                },

                render: function () {
                    var chapterSettings = Chapter.getSettings();
                    this.$el.append(that.recordingCountDownView.render().el)
                        .append(that.recordingBottomMenuView.render().el);
                    SoundRecorder.setRecordingLightBoxModel(this.model);

                    this.lastPageToRecordOn = chapterSettings['preventRecordingOnLastPage'] ? chapterSettings.lastPageNumber - App.segmentLength() : chapterSettings.lastPageNumber;
                    this.$el.hide();
                    this.navigateToNextPageAndStartRecording();
                    return this;
                }
            });
            that.RecordingCountDownView = Backbone.View.extend({
                tagName: "div",
                id: 'RecordingCountDownView',
                template: templates.getTemplate('recordingLightBoxWizardMessage'),
                $messageWrapper: null,
                initialize: function (attr) {
                    this.listenTo(this.model, "change:displayedMessage", this.updateDisplayedMessage);
                    this.listenTo(this.model, "change:RecordingCountDownViewClassName", this.updateRecordingCountDownViewClassName);
                },
                removeView: function () {
                    if (this.model.get("removed")) {
                        this.remove();
                        this.unbind();
                    }
                },
                updateDisplayedMessage: function () {
                    if (null == this.$messageWrapper) {
                        this.$messageWrapper = $('#recordingLightBoxWizardMessageWrapper', this.el);
                    }
                    this.$messageWrapper.text(this.model.get("displayedMessage"));
                },
                updateRecordingCountDownViewClassName: function () {
                    if (null == this.$messageWrapper) {
                        this.$messageWrapper = $('#recordingLightBoxWizardMessageWrapper', this.el);
                    }
                    this.$messageWrapper.attr("class", this.model.get("RecordingCountDownViewClassName"));
                },

                render: function () {
                    this.$el.html(this.template(this.model.attributes));
                    return this;
                }
            });
            that.RecordingBottomMenuView = Backbone.View.extend({
                tagName: "div",
                id: 'RecordingBottomMenuView',

                initialize: function (attr) {
                    this.listenTo(this.model, "change:visible", this.showHide);
                },

                showHide: function () {
                    this.model.get('visible') ? this.$el.show() : this.$el.hide();
                },
                removeView: function () {
                    if (this.model.get("removed")) {
                        this.remove();
                        this.unbind();
                    }
                },
                render: function () {
                    this.$el.append(that.recordingBottomMenuAbortRecordingButtonView.render().el)
                        .append(that.recordingBottomMenuMicrophoneView.render().el)
                        .append(that.recordingBottomMenuInfoMessageView.render().el);
                    this.$el.height(CSSConstants.DefaultActiveTabWrapperHeight());
                    this.$el.hide();
                    return this;
                }
            });
            that.RecordingBottomMenuAbortRecordingButtonView = Backbone.View.extend({
                tagName: "div",
                id: 'RecordingBottomMenuAbortRecordingButtonView',
                template: templates.getTemplate('recordingLightBoxAbortRecording'),

                removeView: function () {
                    if (this.model.get("removed")) {
                        this.remove();
                        this.unbind();
                    }
                },

                events: function () {
                    var _events = {};
                    _events[AppUtils.EventTypes.mouseDown] = "mouseDownHandler";
                    return _events;
                },
                mouseDownHandler: function (e) {
                    e.stopPropagation();
                    e.preventDefault();
                    that.recordingLightBoxModel.set('userAbortRecording', true);
                    SoundRecorder.toggleSoundRecordingForPage(Chapter.getActivePage());
                },
                render: function () {
                    this.$el.html(this.template(this.model.attributes));
                    return this;
                }
            });
            that.RecordingBottomMenuInfoMessageView = Backbone.View.extend({
                tagName: "div",
                id: 'RecordingBottomMenuInfoMessageView',
                template: templates.getTemplate('recordingLightBoxWizardBottomMenuMessage'),
                $messageWrapper: null,

                initialize: function (attr) {
                    this.listenTo(this.model, "change:displayedMessage", this.updateDisplayedMessage);
//                    this.listenTo(this.model, "change:RecordingCountDownViewClassName", this.updateRecordingCountDownViewClassName);
                },
                updateDisplayedMessage: function () {
                    if (null == this.$messageWrapper) {
                        this.$messageWrapper = $('#recordingLightBoxWizardBottomMenuMessageWrapper', this.el);
                    }
                    this.$messageWrapper.text(CAPTIONS.RECORDING_WIZARD_MESSAGE);
                },
                removeView: function () {
                    if (this.model.get("removed")) {
                        this.remove();
                        this.unbind();
                    }
                },
                render: function () {
                    this.$el.html(this.template(this.model.attributes));
                    return this;
                }
            });
            that.RecordingBottomMenuMicrophoneView = Backbone.View.extend({
                tagName: "div",
                id: 'RecordingBottomMenuMicrophoneView',
                template: templates.getTemplate('recordingLightBoxMicrophoneImage'),
//                className: 'pulsateEffect',
                initialize: function (attr) {
                    this.listenTo(this.model, "change:startPulsate", this.addPulsateEffect);
                    this.listenTo(this.model, "change:endPulsate", this.removePulsateEffect);
                },
                addPulsateEffect: function () {
                    if (this.model.get("startPulsate")) {
                        this.model.set("startPulsate", false);
                        this.$el.addClass("pulsateEffect");
                    }
                },
                removePulsateEffect: function () {
                    if (this.model.get("endPulsate")) {
                        this.model.set("endPulsate", false);
                        this.$el.removeClass("pulsateEffect");
                    }
                },
                removeView: function () {
                    if (this.model.get("removed")) {
                        this.remove();
                        this.unbind();
                    }
                },
                render: function () {
                    this.$el.html(this.template(this.model.attributes));
                    return this;
                }
            });
        };
        this.instantiateObjects = function () {
            this.recordingLightBoxModel = new Backbone.Model({
                displayedMessage: null
            });

            this.recordingCountDownModel = new Backbone.Model();

            this.recordingBottomMenuModel = new Backbone.Model({
                visible: false
            });

            this.recordingBottomMenuAbortRecordingButtonModel = new Backbone.Model();

            this.recordingBottomMenuMicrofonViewModel = new Backbone.Model();

            this.recordingBottomMenuInfoMessageViewModel = new Backbone.Model();


            this.recordingLightBoxView = new this.RecordingLightBoxView({
                model: that.recordingLightBoxModel
            });
            this.recordingCountDownView = new this.RecordingCountDownView({
                model: that.recordingCountDownModel
            });
            this.recordingBottomMenuView = new this.RecordingBottomMenuView({
                model: that.recordingBottomMenuModel
            });
            this.recordingBottomMenuAbortRecordingButtonView = new this.RecordingBottomMenuAbortRecordingButtonView({
                model: that.recordingBottomMenuAbortRecordingButtonModel
            });
            this.recordingBottomMenuMicrophoneView = new this.RecordingBottomMenuMicrophoneView({
                model: that.recordingBottomMenuMicrofonViewModel
            });
            this.recordingBottomMenuInfoMessageView = new this.RecordingBottomMenuInfoMessageView({
                model: that.recordingBottomMenuInfoMessageViewModel
            });

        };


        return {
            init: function () {
                that.defineClasses();
                that.instantiateObjects();
            },
            getView: function () {
                return that.recordingLightBoxView;
            },
            getModel: function () {
                return that.recordingLightBoxModel;
            },
            getRecordingCountDownModel: function () {
                return that.recordingCountDownModel;
            }
        }
    }
});