define(['GenericLightBox'], function (GenericLightBox) {
    return function (unlockContentSettings, environmentId, objectView) {
        var that = this,
            CONTENT_UNLOCK_SUCCESS_RESPONSE_CODE = 42,
            ERROR_SUCCESS_MESSAGE_TIMEOUT = 3000;

        this.defineClasses = function () {

            that.PromoCodeInputView = Backbone.View.extend({
                className: "inputPromoCodeLightBox",
                templateInputCodeScreen: templates.getTemplate("inputPromoCodeLightBox"),
                templateWrongCodeScreen: templates.getTemplate("promoCodeLightBoxWrongCode"),
                templateContentUnlockedScreen: templates.getTemplate("promoCodeLightBoxContentUnlocked"),

                initialize: function () {
                    this.delegateEvents(this.getEventsObj());
                },

                getEventsObj: function () {
                    var eventObj = {};
                    eventObj[AppUtils.EventTypes.mouseDown + " .promoCodeLightBoxUnlockButton"] = "doUnlockByPromoCode";
                    return eventObj;
                },

                doUnlockByPromoCode: function () {
                    var self = this;
                    var promoCode = this.$el.find("input").val();
                    var serverURL = unlockContentSettings["SERVICE_URL"],
                        productId = unlockContentSettings["PRODUCT_ID"],
                        requestData = {};

                    requestData["action"] = "useVoucher";
                    requestData["sessionId"] = ""; // nothing needed here, since this has public access
                    requestData["params"] = {};
                    requestData["params"]["promoCode"] = promoCode;
                    requestData["params"]["userEnvironmentId"] = environmentId;
                    if (productId) {
                        requestData["params"]["productId"] = productId;
                    }

                    HTTPRequestUtils.getInstance().doPostRequest(serverURL + "/partner/promoVoucher",
                            "msg=" + JSON.stringify(requestData),
                        function (successData) {
                            if (successData["responseStatus"]["code"] == CONTENT_UNLOCK_SUCCESS_RESPONSE_CODE) {
                                self.renderContentUnlockedView();
                                require(['IAPService'], function (IAPService) {
                                    var iapService = new IAPService(),
                                        productId, productIdx;
                                    for (productIdx in successData["data"]) {
                                        productId = successData["data"][productIdx]["productId"];
                                        console.log("Now unlock product with ID " + productId);
                                        iapService.setPurchasedProduct(productId);
                                    }
                                    objectView.doOnRestoreComplete(iapService);
                                    // this object must be part of a single persistent category that will be now deactivated
                                    objectView.model.set("removeObjectCategoryIds", objectView.model.get("objectCategoryIds"));
                                    // this will cause object categories to be processed
                                    objectView.parentElementModel.set("activatedObject", objectView.getActivatedObjectInformation());
                                });
                            } else {
                                self.renderWrongCodeView();
                            }
                        }, function () {
                            console.error("Failed to use promo code");
                            self.renderWrongCodeView();
                        },
                        "json");
                },

                renderWrongCodeView: function () {
                    var self = this;
                    this.$el.html(this.templateWrongCodeScreen);
                    setTimeout(function () {
                        self.renderInputCodeScreen();
                    }, ERROR_SUCCESS_MESSAGE_TIMEOUT);
                },

                renderContentUnlockedView: function () {
                    this.$el.html(this.templateContentUnlockedScreen);
                    setTimeout(function () {
                        that.lightBox.getModel().set("remove", true);
                    }, ERROR_SUCCESS_MESSAGE_TIMEOUT);
                },

                renderInputCodeScreen: function () {
                    this.$el.html(this.templateInputCodeScreen);
                    return this;
                }

            });
        };

        this.instantiateObjects = function () {
            this.promoCodeInputView = new this.PromoCodeInputView({});
        };

        this.showInLightBox = function () {
            var lightBoxWindowSettings = {
                width: '50%',
                height: '35%',
                closeButton: 'off'
            };
            this.lightBox = new GenericLightBox(null, lightBoxWindowSettings, {content: this.promoCodeInputView.renderInputCodeScreen().el});
            this.lightBox.init();
            App.viewPortView.$el.append(this.lightBox.getView().render().el);
        };

        return {
            init: function () {
                that.defineClasses();
                that.instantiateObjects();
                that.showInLightBox();
            },
            getView: function () {
                return that.promoCodeInputView;
            }
        }
    }
});