/**
 * Creates one single object in the global namespace.
 */
if (!window["projectNotes"]) window["projectNotes"] = {};

/**
 *  Code used to handle the notes feature
 */
(function (projectNotes) {
    projectNotes.selectedModel = {};
    projectNotes.pressingTimer = 0;

    projectNotes.drawElement = null;
    projectNotes.inputTextBoxElement = null;

    projectNotes.titleItemListEditMode = false;

    projectNotes.colorCollection = new Array("#870B80", "#549817", "#B51818", "#EA7B0E", "#1489C0", "#28CCA0");

    projectNotes.defineViews = function () {

        projectNotes.position = null;

        /**
         *  View used for showing notes view on screen
         */
        projectNotes.ViewPortView = Backbone.View.extend({
            id: "viewPortView",
            tagName: "div",
            className: 'ViewPortViewClass',
            top: 0,

            initialize: function (attributes, options) {
                this.listenTo(this.model, "change:displayed", this.showHide);
                this.listenTo(this.model, "change:shouldClose", this.close);
            },

            /**
             *  function used to raise/ lower down the viewport
             */
            showHide: function () {
                var that = this;
                if (this.model.get("displayed")) {
                    this.$el.unbind('transitionEnd');
                    this.$el.show();
                    this.$el.css("top", this.top);
                    //To wait for the ui rendering
                    setTimeout(function () {
                        that.$el.css("transform", "translate3d(0px, -" + that.top + ", 0px)");
                    }, 0);
                }
                else {
                    this.$el.css("transform", "");
                    this.$el.unbind('transitionEnd')
                        .bind('transitionEnd', function () {
                            that.$el.hide();
                        });
                }
            },

            /**
             * function used to close the viewport
             */
            close: function () {
                if (this.model.get("shouldClose")) {
                    var that = this;
                    this.$el.unbind('transitionend')
                        .bind('transitionend', function () {
                            that.$el.css(CSSHandler.currentBrowserPrefix.transition, '');
                            setTimeout(function () {
                                that.$el.css("transform", "translate3d( 0px, 0px, 0px)");
                                that.$el.css(CSSHandler.currentBrowserPrefix.transition, '700ms');
                                that.$el.unbind('transitionend');
                                projectNotes.notesContainerModel.set("shouldClose", true);
                            }, 0);
                        });
                    this.model.set("shouldClose", false);
                    this.model.set("displayed", false);
                }
            },

            /**
             * function used to render the viewport
             */
            render: function () {
                this.top = App.settings.height + "px";
                this.$el.css("width", App.settings.width + "px");
                this.$el.append(projectNotes.notesContainerView.render().el);
                this.model.set("displayed", true);
                return this;
            }
        });

        /**
         * View used as the container for notes view area
         */
        projectNotes.NotesContainerView = Backbone.View.extend({
            id: "notesContainerView",
            tagName: "div",
            className: 'notesContainerViewClass',
            top: 0,
            left: 0,
            initialize: function (attributes, options) {
                this.listenTo(this.model, "change:moveLeft", this.moveLeft);
                this.listenTo(this.model, "change:moveRight", this.moveRight);
                this.listenTo(this.model, "change:shouldClose", this.close);
            },

            /**
             *  function used to raise/ lower down the container
             */
            showHide: function () {
                var that = this;
                if (this.model.get("displayed")) {
                    this.$el.unbind('transitionEnd');
                    this.$el.show();
                    this.$el.css("top", this.top);
                    //To wait for the ui rendering
                    setTimeout(function () {
                        that.$el.css("transform", "translate3d(0px, -" + that.top + ", 0px)");
                    }, 0);
                }
                else {
                    this.$el.css("transform", "");
                    this.$el.unbind('transitionEnd')
                        .bind('transitionEnd', function () {
                            that.$el.hide();
                        });
                }
            },

            /**
             *  function used to move left the container
             */
            moveLeft: function () {
                var that = this;
                this.$el.unbind('transitionEnd');
                if (this.model.get("moveLeft")) {
                    this.$el
                        .unbind(CSSHandler.currentBrowserPrefix.transitionEnd)
                        .bind(CSSHandler.currentBrowserPrefix.transitionEnd, function () {
                            projectNotes.notesPrimaryModel.set("blockingGlassVisible", false);
                        });

                    setTimeout(function () {
                        //that.$el.css("transform", "translate3d(-" + that.left + ", -" + that.top + ", 0px)");
                        that.$el.css("transform", "translate3d(-" + that.left + ", 0px , 0px)");
                    }, 0);
                    this.model.set("moveLeft", false);
                }
            },

            /**
             *  function used to move right
             */
            moveRight: function () {
                var that = this;
                this.$el.unbind('transitionEnd');
                if (this.model.get("moveRight")) {
                    setTimeout(function () {
                        //that.$el.css(CSSHandler.currentBrowserPrefix.transform, "translate3d(0px, -" + that.top + ", 0px)");
                        that.$el.css("transform", "translate3d(0px, 0px, 0px)");
                    }, 0);
                    this.model.set("moveRight", false);
                }
            },

            /**
             *  function used to close the container
             */
            close: function () {
                if (this.model.get("shouldClose")) {
                    var that = this;

                    that.$el.css(CSSHandler.currentBrowserPrefix.transition, '');
                    setTimeout(function () {
                        that.$el.css("transform", "translate3d( 0px, 0px, 0px)");
                    }, 0);
                    this.model.set("shouldClose", false);
                }
            },

            /**
             *  function used to render the container
             */
            render: function () {
                var Input;
                this.top = App.settings.height + "px";
                this.left = App.settings.width + "px";
                ghostInput = $("<input>").addClass("ghostInput").attr("type", "text");
                this.$el.html(ghostInput);
                console.log("The width is:" + App.settings.width);
                this.$el.css("width", 2 * App.settings.width + "px");
                this.$el.append(projectNotes.notesListPrimaryView.render().el);
                this.$el.append(projectNotes.notesPrimaryView.render().el);
                this.model.set("displayed", true);
                return this;
            }
        });

        /**
         *  View used to manage the notes edit view
         */
        projectNotes.NotesPrimaryView = Backbone.View.extend({
            id: "notesPrimaryView",
            tagName: "div",
            className: 'notesPrimaryViewClass',
            top: 0,
            $glassEl: null,
            initialize: function (attributes, options) {
                this.listenTo(this.model, "change:blockingGlassVisible", this.toggleBlockingGlass);
            },

            /**
             * function used to bind the events the view is listening to
             * @returns _events - an array containing the events
             */
            events: function () {
                var _events = {};
                _events[AppUtils.EventTypes.mouseDownSafeIOS5] = 'doOnClick';
                return _events;
            },

            /**
             * function used to handle the mouse down event
             */
            doOnClick: function () {
                //console.log("NotesPrimaryView doOnClick blur");
                $("#notesTitleView").blur();
            },

            /**
             * function used for catching blur event
             */
            toggleBlockingGlass: function () {
                var isGlassVisible = this.model.get("blockingGlassVisible");
                if (this.$glassEl) {
                    if (isGlassVisible) {
                        this.$glassEl.show();
                    } else {
                        this.$glassEl.hide();
                    }
                }
            },

            /**
             * function used to render the view
             */
            render: function () {
                this.top = App.settings.height + "px";
                //projectNotes.drawingModel.set("height", top * 0.8);
                //projectNotes.drawingModel.set("width", App.settings.width);
                this.$el.html();
                this.$glassEl = $("<div>").addClass("blockEvents");
                this.$el.append(projectNotes.toolbarView.render().el);
                this.$el.append(projectNotes.titleView.render().el);
                this.$el.append(projectNotes.dateView.render().el);
                this.$el.append(projectNotes.contentView.render().el);
                this.$el.append(this.$glassEl);
                return this;
            }
        });

        /**
         * View used to hold the buttons in the notes edit mode view
         */
        projectNotes.ToolbarView = Backbone.View.extend({
            id: "notesToolbarView",
            tagName: "div",
            className: 'notesToolbarViewClass',
            initialize: function (attributes, options) {
                this.listenTo(this.model, "change:color", this.changeColor);
            },

            /**
             * function used to change the background color of the toolbar
             */
            changeColor: function () {
                if (this.model.get("color") != null) {
                    this.$el.css("background", this.model.get("color"));
                }
            },

            /**
             * function used to render the toolbar
             */
            render: function () {
                this.$el.html();
                this.$el.append(projectNotes.notesBackBtnView.render().el);
                //this.$el.append(projectNotes.closePrimaryViewBtnView.render().el);
                this.$el.append(projectNotes.deleteNoteBtnView.render().el);
                this.$el.append(projectNotes.clearBtnView.render().el);
                this.$el.append(projectNotes.drawingBtnView.render().el);
                this.$el.append(projectNotes.textBtnView.render().el);
                return this;
            }
        });

        /**
         * View used to manage the title of the note in note edit mode
         */
        projectNotes.TitleView = Backbone.View.extend({
            id: "notesTitleView",
            tagName: "input",
            className: 'notesTitleViewClass',

            initialize: function (attributes, options) {
                this.listenTo(this.model, "change:titleValue", this.updateTitle);
                this.listenTo(this.model, "change:color", this.changeColor);
            },

            /**
             * function used to change the color of the title
             */
            changeColor: function () {
                if (this.model.get("color") != null) {
                    this.$el.css("color", this.model.get("color"));
                }
            },

            /**
             * function used to bind the events the view is listening to
             * @returns _events - an array containing the events
             */
            events: function () {
                var _events = {};
                _events[AppUtils.EventTypes.mouseDownSafeIOS5] = 'doOnClick';
                return _events;
            },

            /**
             * function used to handle the mouse down event
             * @param e - the mouse down event
             */
            doOnClick: function (e) {
                e.stopPropagation();
                console.log("TitleView mouseDownHandler");
                projectNotes.selectedModel.set("oldTitle", this.$el.val());
            },

            /**
             * function used to update the title view with the new value of the selected note title
             */
            updateTitle: function () {
                this.$el.val(this.model.get("titleValue"));
            },

            /**
             * function used to render the title view
             */
            render: function () {
                var that = this;
                this.$el.bind("blur", function () {
                    var titleValue = that.$el.val();
                    //console.log("Blur event caught in TitleView");
                    projectNotes.titleModel.set("titleValue", titleValue);
                    that.model.get("selectedItemModel").set("titleValue", titleValue);
                    projectNotes.selectedModel.set("shouldUpdateTitle", true);
                });
                return this;
            }
        });

        /**
         * view used to manage the date of a selected note in note edit mode
         */
        projectNotes.DateView = Backbone.View.extend({
            id: "notesDateView",
            tagName: "div",
            className: 'notesDateViewClass',
            initialize: function (attributes, options) {
                this.listenTo(this.model, "change:dateValue", this.updateDate);
                this.listenTo(this.model, "change:update", this.update);
            },

            /**
             * function used to update the date from time to time
             */
            updateDate: function () {
                this.$el.html(moment(this.model.get("dateValue")).fromNow());
            },

            /**
             * function used to update the date with the date of the selected note
             */
            update: function () {
                if (this.model.get("update")) {
                    this.$el.html(moment(this.model.get("dateValue")).fromNow());
                    this.model.set("update", false);
                }
            },

            /**
             * function used to render the date view
             */
            render: function () {
                this.$el.html();
                return this;
            }
        });

        /**
         * view used as a container for the note content (both drawing and text input)
         */
        projectNotes.ContentView = Backbone.View.extend({
            id: "notesContentView",
            tagName: "div",
            className: 'notesContentViewClass',

            initialize: function (attributes, options) {
                this.listenTo(this.model, "change:updateContent", this.updateContent);
                this.listenTo(this.model, "change:saveContent", this.saveContent);
            },

            /**
             * function used to load the drawing and/or the text saved in the local storage when selecting a note from the list
             */
            updateContent: function () {
                if (this.model.get("updateContent")) {
                    if (this.model.get("selectedItemModel") != null) {
                        var id = HashCode.value(projectNotes.titleModel.get("titleValue") + projectNotes.dateModel.get("itemDateMilliseconds"));
                        projectNotes.drawElement.getModel().set("shouldClear", true);
                        projectNotes.drawElement.getModel().set("canvasId", id + "_canvas");

                        projectNotes.inputTextBoxElement.getModel().set("textInputId", id + "_text");
                        projectNotes.inputTextBoxElement.getModel().set("loadText", true);
                        this.model.set("selectedItemModel", null);
                    }
                    this.model.set("updateContent", false);
                }
            },

            /**
             * function used to save the drawing and text into the local storage when the back btn is pressed
             */
            saveContent: function () {
                if (this.model.get("saveContent")) {
                    projectNotes.drawElement.getModel().set("drawing", false);
                    var notesList = JSON.parse(localStorage.getItem(Chapter.getSettings().sequenceId + "notesList"));
                    if (notesList != null) {
                        for (var i = 0; i < notesList.length; i++) {
                            if (notesList[i].titleModel.itemTitle == projectNotes.selectedModel.get("titleValue") && notesList[i].dateModel.itemDateMilliseconds == projectNotes.selectedModel.get("itemDateMilliseconds")) {
                                if (notesList[i].isCanvasDirty != projectNotes.drawElement.getModel().get("canvasDirty")) {
                                    notesList[i].isCanvasDirty = projectNotes.drawElement.isCanvasDirty;
                                    localStorage.removeItem(Chapter.getSettings().sequenceId + "notesList");
                                    localStorage.setItem(Chapter.getSettings().sequenceId + "notesList", JSON.stringify(notesList));
                                }
                            }
                        }
                    }
                    projectNotes.inputTextBoxElement.getModel().set("saveText", true);
                    this.model.set("saveContent", false);
                }
            },

            /**
             * function used to render the content view
             */
            render: function () {
                this.$el.html();
                this.$el.append(projectNotes.inputTextBoxElement.getView().render().el);
                this.$el.append(projectNotes.drawElement.getView().render().el);
                return this;
            }
        });

        /**
         * view used to handle the back btn logic
         */
        projectNotes.NotesBackBtnView = Backbone.View.extend({
            id: "notesBackBtnView",
            tagName: "div",
            className: 'notesBackBtnView',
            template: projectNotesTemplates.getTemplate("backBtnImage"),
            initialize: function (attributes, options) {
            },

            /**
             * function used to bind the events the view is listening to
             * @returns _events - an array containing the events
             */
            events: function () {
                var _events = {};
                _events[AppUtils.EventTypes.mouseDownSafeIOS5] = 'doOnClick';
                return _events;
            },

            /**
             * function used to handle the mouse down event
             */
            doOnClick: function () {
                projectNotes.notesContainerModel.set("moveRight", true);
                projectNotes.contentModel.set("saveContent", true);
                if (projectNotes.drawElement.getModel().get("canvasDirty") == false) {
                    projectNotes.drawElement.getModel().set("removeContent", true);
                }
                projectNotes.drawElement.getModel().set("canvasId", null);
                this.$el.addClass("pressedButton");
                var that = this;
                setTimeout(function () {
                    that.$el.removeClass("pressedButton");
                }, 200);

            },

            /**
             * function used to render the back btn view
             */
            render: function () {
                this.$el.html(this.template(this.model.attributes));
                this.$el.addClass("CommonButtonClass");
                return this;
            }
        });

        /**
         * view used to handle the text btn logic
         */
        projectNotes.TextBtnView = Backbone.View.extend({
            id: "notesTextBtnView",
            tagName: "div",
            className: 'notesTextBtnViewClass',
            template: projectNotesTemplates.getTemplate("textBtnImage"),
            initialize: function (attributes, options) {
                this.listenTo(this.model, "change:displayed", this.showHide);
            },

            /**
             * function used to bind the events the view is listening to
             * @returns _events - an array containing the events
             */
            events: function () {
                var _events = {};
                _events[AppUtils.EventTypes.mouseDownSafeIOS5] = 'doOnClick';
                return _events;
            },

            /**
             * function used to handle the mouse down event
             */
            doOnClick: function () {
                console.log("TextBtnView doOnClick");
                projectNotes.drawElement.getModel().set("displayed", false);
                projectNotes.inputTextBoxElement.getModel().set("displayed", true);
                this.$el.addClass("pressedButton");
                projectNotes.drawingBtnView.$el.removeClass("pressedButton");
            },

            /**
             * function used to render the text btn view
             */
            render: function () {
                this.$el.html(this.template(this.model.attributes));
                this.$el.addClass("CommonButtonClass");
                return this;
            }
        });

        /**
         * view used to handle the drawing btn logic
         */
        projectNotes.DrawingBtnView = Backbone.View.extend({
            id: "notesDrawingBtnView",
            tagName: "div",
            className: 'notesDrawingBtnViewClass',
            template: projectNotesTemplates.getTemplate("drawingBtnImage"),
            initialize: function (attributes, options) {
                this.listenTo(this.model, "change:displayed", this.showHide);
            },

            /**
             * function used to bind the events the view is listening to
             * @returns _events - an array containing the events
             */
            events: function () {
                var _events = {};
                _events[AppUtils.EventTypes.mouseDownSafeIOS5] = 'doOnClick';
                return _events;
            },

            /**
             * function used to handle the mouse down event
             */
            doOnClick: function () {
                console.log("DrawingBtnView doOnClick");
                projectNotes.drawElement.getModel().set("displayed", true);
                projectNotes.inputTextBoxElement.getModel().set("displayed", false);
                this.$el.addClass("pressedButton");
                projectNotes.textBtnView.$el.removeClass("pressedButton");
            },

            /**
             * function used to render the drawing btn view
             */
            render: function () {
                this.$el.html(this.template(this.model.attributes));
                this.$el.addClass("CommonButtonClass");
                return this;
            }
        });

        /**
         * view used to handle the clear btn logic
         */
        projectNotes.ClearBtnView = Backbone.View.extend({
            id: "notesClearBtnView",
            tagName: "div",
            className: 'notesClearBtnViewClass',
            template: projectNotesTemplates.getTemplate("clearBtnImage"),
            initialize: function (attributes, options) {
            },

            /**
             * function used to bind the events the view is listening to
             * @returns _events - an array containing the events
             */
            events: function () {
                var _events = {};
                _events[AppUtils.EventTypes.mouseDownSafeIOS5] = 'doOnClick';
                return _events;
            },

            /**
             * function used to handle the mouse down event
             */
            doOnClick: function () {
                this.$el.addClass("pressedButton");
                var that = this;
                setTimeout(function () {
                    that.$el.removeClass("pressedButton");
                }, 200);
                if (projectNotes.drawElement.getModel().get("displayed")) {
                    require(['UserDialogs'], function () {
                        UserDialogs.showConfirmation("Clear drawing?", "Warning", CAPTIONS.YES + "," + CAPTIONS.NO, function (buttonIndex) {
                            if (buttonIndex == 1 || buttonIndex == true) {
                                projectNotes.drawElement.getModel().set("shouldClear", true);
                            }
                        });
                    });
                } else if (projectNotes.textModel.get("displayed")) {
                    require(['UserDialogs'], function () {
                        UserDialogs.showConfirmation("Clear text?", "Warning", CAPTIONS.YES + "," + CAPTIONS.NO, function (buttonIndex) {
                            if (buttonIndex == 1 || buttonIndex == true) {
                                projectNotes.inputTextBoxElement.getModel().set("shouldClear", true);
                            }
                        });
                    });
                }
            },

            /**
             * function used to render the clear btn view
             */
            render: function () {
                this.$el.html(this.template(this.model.attributes));
                this.$el.addClass("CommonButtonClass");
                return this;
            }
        });

        /**
         * view used to handle the delete btn logic
         */
        projectNotes.DeleteNoteBtnView = Backbone.View.extend({
            id: "deleteNoteBtnView",
            tagName: "div",
            className: 'deleteNoteBtnViewClass',
            template: projectNotesTemplates.getTemplate("deleteBtnImage"),
            initialize: function (attributes, options) {
            },

            /**
             * function used to bind the events the view is listening to
             * @returns _events - an array containing the events
             */
            events: function () {
                var _events = {};
                _events[AppUtils.EventTypes.mouseDownSafeIOS5] = 'doOnClick';
                return _events;
            },

            /**
             * function used to handle the mouse down event
             */
            doOnClick: function () {
                this.$el.addClass("pressedButton");
                var that = this;
                setTimeout(function () {
                    that.$el.removeClass("pressedButton");
                }, 200);
                require(['UserDialogs'], function () {
                    UserDialogs.showConfirmation("Are you sure do you want to delete this note?", "Warning", CAPTIONS.YES + "," + CAPTIONS.NO, function (buttonIndex) {
                        if (buttonIndex == 1 || buttonIndex == true) {
                            projectNotes.notesListModel.set("removeItem", true);
                            projectNotes.notesContainerModel.set("moveRight", true);
                            projectNotes.textModel.set("displayed", true);
                            projectNotes.drawElement.getModel().set("removeContent", true);
                            projectNotes.inputTextBoxElement.getModel().set("removeText", true);

                        }
                    });
                });
            },

            /**
             * function used to render the delete btn view
             */
            render: function () {
                this.$el.html(this.template(this.model.attributes));
                this.$el.addClass("CommonButtonClass");
                return this;
            }
        });

        /**
         * view used to handle the note list view
         */
        projectNotes.NotesListPrimaryView = Backbone.View.extend({
            id: "notesListPrimaryView",
            tagName: "div",
            className: 'notesListPrimaryViewClass',
            initialize: function (attributes, options) {
                this.listenTo(this.model, "change:displayed", this.showHide);
            },

            /**
             * function used to render the note list view
             */
            render: function () {
                this.top = App.settings.width + "px";
                this.$el.html();
                this.$el.append(projectNotes.notesHeaderView.render().el);
                this.$el.append(projectNotes.notesListView.render().el);
                //this.model.set("displayed", true);
                return this;
            }
        });

        /**
         * view used as a toolbar containing buttons from list view
         */
        projectNotes.NotesHeaderView = Backbone.View.extend({
            id: "notesHeaderView",
            tagName: "div",
            className: 'notesHeaderViewClass',

            initialize: function (attributes, options) {
                this.listenTo(this.model, "change:color", this.changeColor);
            },
            /**
             * function used to change the background color of the header
             */
            changeColor: function () {
                this.$el.css("background", this.model.get("color"));
            },

            /**
             * function used to render the header view
             */
            render: function () {
                this.$el.html();
                this.$el.append(projectNotes.newNoteBtnView.render().el);
                this.$el.append(projectNotes.closeListViewBtnView.render().el);
                this.$el.css("background", projectNotes.colorCollection[0]);
                return this;
            }
        });

        /**
         * view used to handle the new note btn logic
         */
        projectNotes.NewNoteBtnView = Backbone.View.extend({
            id: "newNoteBtnView",
            tagName: "div",
            className: 'newNoteBtnViewClass',
            template: projectNotesTemplates.getTemplate("newNoteBtnImage"),
            initialize: function (attributes, options) {
            },

            /**
             * function used to bind the events the view is listening to
             * @returns _events - an array containing the events
             */
            events: function () {
                var _events = {};
                _events[AppUtils.EventTypes.mouseDownSafeIOS5] = 'doOnClick';
                return _events;
            },

            /**
             * function used to handle the mouse down event
             */
            doOnClick: function () {
                this.$el.addClass("pressedButton");
                var that = this;
                setTimeout(function () {
                    that.$el.removeClass("pressedButton");
                }, 200);
                projectNotes.notesListModel.set("addItem", true);
            },

            /**
             * function used to render the new note btn view
             */
            render: function () {
                this.$el.html(this.template(this.model.attributes));
                this.$el.addClass("CommonButtonClass");
                return this;
            }
        });

        /**
         * view used to handle the title of every note item in the list
         */
        projectNotes.NoteItemTitleView = Backbone.View.extend({
            id: "noteItemTitleView",
            tagName: "span",
            className: 'noteItemTitleViewClass',
            template: projectNotesTemplates.getTemplate("itemTitleView"),
            isScroll: false,

            initialize: function (attributes, options) {
                this.listenTo(this.model, "change:getTitle", this.getTitle);
                this.listenTo(this.model, "change:updateTitleValue", this.updateTitleValue);
                //this.listenTo(this.model, "change:title", this.updateTitleValue);
            },

            /**
             * function used to bind the events the view is listening to
             * @returns _events - an array containing the events
             */
            events: function () {
                var _events = {};
                /*
                 _events[AppUtils.EventTypes.mouseDown] = 'mouseDownHandler';
                 _events[AppUtils.EventTypes.mouseUp] = 'mouseUpHandler';
                 _events[AppUtils.EventTypes.click] = 'clickHandler';
                 _events[AppUtils.EventTypes.mouseMove] = 'moveHandler';
                 */
                return _events;
            },

            /**
             * function used to return the title of the note item
             */
            getTitle: function () {
                this.model.get("noteItemModel").set("titleViewValue", this.el.innerHTML);
            },

            /**
             * function used to handle the click event
             * @param e- the click event
             */
            clickHandler: function (e) {
                if (this.$el.attr("contenteditable") == "true") {
                    e.stopPropagation();
                }
            },

            /**
             * function used to handle the move event
             * @param e - move event
             */
            moveHandler: function (e) {
                this.isScroll = true;
            },

            /**
             * function used to handle the mouse down event
             * @param e - mouse down event
             */
            mouseDownHandler: function (e) {
                this.isScroll = false;
                console.log("NoteItemTitleView : mouse down handler");
                e.stopPropagation();
                var that = this,
                    isEditable = this.$el.attr("contenteditable") == "true";
                if (!isEditable) {
                    $('span[id^="noteItemTitleView"]').attr("contenteditable", false);
                    projectNotes.pressingTimer = window.setTimeout(function () {
                        projectNotes.titleItemListEditMode = true;
                        that.$el.attr("contenteditable", true);
                        $('div[id^="noteItemView"]').removeClass("selectedItem");
                        that.model.get("noteItemModel").set("shouldSelect", true);
                        setTimeout(function () {
                            console.log("long press");
                            that.model.get("noteItemModel").set("oldTitle", that.el.textContent);
                            placeCaretAtEnd(that.el);
                            that.model.get("noteItemModel").set("titleWasEdited", true);
                        }, 100);
                    }, 750);
                }
            },

            /**
             * function used to handle the mouse up event
             * @param e - mouse up event
             */
            mouseUpHandler: function (e) {
                console.log("NoteItemTitleView: mouse up handler");
                var isEditable = this.$el.attr("contenteditable") == "true";
                e.stopPropagation();
                if (this.isScroll) {
                    window.clearTimeout(projectNotes.pressingTimer);
                } else if (!isEditable) {
                    this.model.get("noteItemModel").set("executeClickHandler", true);
                    window.clearTimeout(projectNotes.pressingTimer);
                }

            },

            /**
             * function used to change the title of the note item
             */
            updateTitleValue: function () {
                this.$el.html(this.model.get("updateTitleValue"));
            },

            /**
             * function used to render the note item title view
             */
            render: function () {
                this.$el.html(this.template(this.model.attributes));
                if (this.model.get("color")) {
                    this.$el.css("color", this.model.get("color"));
                }
                this.$el.keypress(function (e) {
                    if (e.keyCode == 13) {
                        e.preventDefault();
                        console.log("you pressed enter");
                    }
                });
                return this;
            }
        });

        /**
         * view used to handle the date of every note item in the list
         */
        projectNotes.NoteItemDateView = Backbone.View.extend({
            id: "noteItemDateView",
            tagName: "div",
            className: 'noteItemDateViewClass',
            initialize: function (attributes, options) {
                this.listenTo(this.model, "change:getDate", this.getDate);
                this.listenTo(this.model, "change:update", this.update);
            },

            /**
             * function used to get the date value
             */
            getDate: function () {
                this.model.get("noteItemModel").set("itemDateMilliseconds", this.model.get("itemDateMilliseconds"));
                this.model.get("noteItemModel").set("dateViewValue", this.model.get("itemDate"));
            },

            /**
             * function used to update the date from time to time
             */
            update: function () {
                if (this.model.get("update")) {
                    this.$el.html(moment(this.model.get("itemDate")).fromNow());
                    this.model.set("update", false);
                }
            },

            /**
             * function used to render the note item date view
             */
            render: function () {
                this.$el.html(moment(this.model.get("itemDate")).fromNow());
                return this;
            }
        });

        /**
         * view used to handle the note item from the notes list
         */
        projectNotes.NoteItemView = Backbone.View.extend({
            id: "noteItemView",
            tagName: "div",
            className: 'noteItemViewClass',
            itemIndex: 0,
            titleModel: null,
            dateModel: null,
            isMove: false,
            wasMouseDown: false,
            initialize: function (attributes, options) {
                this.listenTo(this.model, "change:remove", this.removeView);
                this.listenTo(this.model, "change:shouldSelect", this.shouldSelect);
                this.listenTo(this.model, "change:titleViewValue", this.updateNoteView);
                this.listenTo(this.model, "change:dateViewValue", this.updateNote);
                this.listenTo(this.model, "change:shouldUpdateTitle", this.shouldUpdateTitle);
                this.listenTo(this.model, "change:titleValue", this.titleValueChange);
                this.listenTo(this.model, "change:executeClickHandler", this.executeClickHandler);
                this.listenTo(this.model, "change:updateDate", this.updateDate);
            },

            /**
             * function used to execute the click handler
             */
            executeClickHandler: function () {
                if (this.model.get("executeClickHandler")) {
                    this.isMove = false;
                    this.actionOnClick();
                    this.model.set("executeClickHandler", false);
                }
            },

            /**
             * function used to change the value of the item's title
             */
            titleValueChange: function () {
                this.titleModel.set("updateTitleValue", this.model.get("titleValue"));
            },

            /**
             * function used to bind the events the view is listening to
             * @returns _events - an array containing the events
             */
            events: function () {
                var _events = {};
                _events[AppUtils.EventTypes.mouseDownSafeIOS5] = 'doOnClick';
                _events[AppUtils.EventTypes.mouseMove] = 'doOnMouseMove';
                return _events;
            },

            /**
             * function used to update the note view according to selected note item
             */
            updateNoteView: function () {
                if (this.model.get("titleViewValue") != null) {
                    projectNotes.titleModel.set("titleValue", this.model.get("titleViewValue"));
                    this.model.set("titleViewValue", null);
                }
            },

            /**
             * function used to update the note according to selected note item
             */
            updateNote: function () {
                if (this.model.get("dateViewValue") != null) {
                    projectNotes.dateModel.set("dateValue", this.model.get("dateViewValue"));
                    if (this.model.get("itemDateMilliseconds") != null) {
                        projectNotes.dateModel.set("itemDateMilliseconds", this.model.get("itemDateMilliseconds"));
                        if (this.model.get("titleWasEdited")) {
                            if (this.model.get("oldTitle") != null) {
                                this.updateTitle();
                            }
                            this.model.set("titleWasEdited", false);
                        }
                    }
                    this.model.set("dateViewValue", null);
                    projectNotes.notesContainerModel.set("moveLeft", true);
                    projectNotes.notesPrimaryModel.set("blockingGlassVisible", true);
                }
            },

            /**
             * function used to update the title of the selected note in the note edit mode
             */
            shouldUpdateTitle: function () {
                if (this.model.get("shouldUpdateTitle")) {
                    this.updateTitle();
                    this.model.set("shouldUpdateTitle", false);
                }

            },

            /**
             * method used to change the keys in the local storage if the title changed
             */
            updateTitle: function () {
                if (this.model.get("oldTitle") != projectNotes.titleModel.get("titleValue")) {
                    var notesList = JSON.parse(localStorage.getItem(Chapter.getSettings().sequenceId + "notesList"));
                    if (notesList != null) {
                        for (var i = 0; i < notesList.length; i++) {
                            if (notesList[i].titleModel.itemTitle == this.model.get("oldTitle") && notesList[i].dateModel.itemDateMilliseconds == this.model.get("itemDateMilliseconds")) {
                                var key = HashCode.value(this.model.get("oldTitle") + this.model.get("itemDateMilliseconds"));
                                var drawing = localStorage.getItem(key + "_canvas");
                                var text = localStorage.getItem(key + "_text");
                                var drawingIsNotNull = false;
                                var textIsNotNull = false;
                                var newTitle = projectNotes.titleModel.get("titleValue");
                                var newKey = HashCode.value(newTitle + this.model.get("itemDateMilliseconds"));
                                if (drawing != null) {
                                    localStorage.removeItem(key + "_canvas");
                                    drawingIsNotNull = true;
                                }
                                if (text != null) {
                                    localStorage.removeItem(key + "_text");
                                    textIsNotNull = true;
                                }

                                if (drawingIsNotNull || textIsNotNull) {
                                    if (drawingIsNotNull) {
                                        localStorage.setItem(newKey + "_canvas", drawing);
                                    }
                                    if (textIsNotNull) {
                                        localStorage.setItem(newKey + "_text", text);
                                    }
                                }
                                projectNotes.drawElement.getModel().set("canvasId", newKey + "_canvas");
                                projectNotes.inputTextBoxElement.getModel().set("textInputId", newKey + "_text");
                                this.model.set("oldTitle", newTitle);
                                notesList[i].titleModel.itemTitle = newTitle;
                            }
                        }
                        localStorage.removeItem(Chapter.getSettings().sequenceId + "notesList");
                        localStorage.setItem(Chapter.getSettings().sequenceId + "notesList", JSON.stringify(notesList));
                    }
                }
            },

            /**
             * function used to handle the mouse move event
             * @param e - the mouse move event
             */
            doOnMouseMove: function (e) {
                if (this.wasMouseDown) {
                    //console.log("NoteItemView doOnMouseMove");
                    this.isMove = true;
                }
            },

            /**
             * function used to handle the mouse down event
             * @param e - mouse down event
             */
            doOnClick: function (e) {
                //console.log("NoteItemView doOnClick");
                var that = this;
                this.wasMouseDown = true;
                this.isMove = false;
                setTimeout(function () {
                    that.wasMouseDown = false;
                    that.actionOnClick(e);
                }, 50);
            },

            /**
             * function used to handle the logic for selecting a note item
             * @param e - mouse down event
             */
            actionOnClick: function (e) {
                //console.log("NoteItemView actionOnClick");
                if (!projectNotes.titleItemListEditMode && !this.isMove) {
                    if (e) {
                        e.stopPropagation();
                    }
                    $('div[id^="noteItemView"]').removeClass("selectedItem");
                    this.$el.addClass("selectedItem");
                    // transition to primary view
                    projectNotes.selectedModel = this.model;
                    projectNotes.inputTextBoxElement.getModel().set("displayed", true);
                    projectNotes.drawElement.getModel().set("displayed", false);
                    this.titleModel.set("getTitle", !this.titleModel.get("getTitle"));
                    this.dateModel.set("getDate", !this.dateModel.get("getDate"));
                    projectNotes.contentModel.set("selectedItemModel", this.model);
                    projectNotes.titleModel.set("selectedItemModel", this.model);
                    projectNotes.contentModel.set("updateContent", true);
                    projectNotes.notesHeaderModel.set("color", this.titleModel.get("color"));
                    projectNotes.toolbarModel.set("color", this.titleModel.get("color"));
                    projectNotes.titleModel.set("color", this.titleModel.get("color"));
                    projectNotes.textBtnView.$el.addClass("pressedButton");
                    projectNotes.drawingBtnView.$el.removeClass("pressedButton");

                }
            },

            /**
             * function used for selecting an note item
             */
            shouldSelect: function () {
                if (this.model.get("shouldSelect")) {
                    this.$el.addClass("selectedItem");
                    this.model.set("shouldSelect", false);
                }
            },

            /**
             * function used to remove a note item from list
             */
            removeView: function () {
                var notesList = JSON.parse(localStorage.getItem(Chapter.getSettings().sequenceId + "notesList"));
                if (notesList != null) {
                    for (var i = 0; i < notesList.length; i++) {
                        if (notesList[i].titleModel.itemTitle == this.model.get("titleValue") && notesList[i].dateModel.itemDateMilliseconds == this.model.get("itemDateMilliseconds")) {
                            notesList.splice(i, 1);
                        }
                    }
                    localStorage.removeItem(Chapter.getSettings().sequenceId + "notesList");
                    localStorage.setItem(Chapter.getSettings().sequenceId + "notesList", JSON.stringify(notesList));
                }
                this.remove();
                this.unbind();
            },

            /**
             * function used to load items from local storage
             */
            loadItemFromList: function () {
                if (this.model.get("titleValue") != null && this.model.get("dateValue") != null) {
                    this.$el.html();
                    this.titleModel = new Backbone.Model({
                        itemTitle: this.model.get("titleValue"),
                        id: "noteItemTitleView" + this.model.get("itemIndex"),
                        color: projectNotes.colorCollection[this.model.get("itemIndex") % projectNotes.colorCollection.length],
                        noteItemModel: this.model,
                        getTitle: false
                    });

                    this.dateModel = new Backbone.Model({
                        itemDate: this.model.get("dateValue"),
                        id: "noteItemDateView" + this.model.get("itemIndex"),
                        itemDateMilliseconds: this.model.get("itemDateMilliseconds"),
                        noteItemModel: this.model,
                        getDate: false
                    });

                    var titleItem = new projectNotes.NoteItemTitleView({id: "noteItemTitleView" + this.model.get("itemIndex"), model: this.titleModel});
                    var dateItem = new projectNotes.NoteItemDateView({id: "noteItemDateView" + this.model.get("itemIndex"), model: this.dateModel});
                    this.$el.append(titleItem.render().el);
                    this.$el.append(dateItem.render().el);
                    return this;
                }
            },

            /**
             * function used to update the date of the note edit mode according to the selected note item date
             */
            updateDate: function () {
                if (this.model.get("updateDate")) {
                    this.dateModel.set("update", true);
                    this.model.set("updateDate", false);
                }
            },

            /**
             * function used to render the note item view
             */
            render: function () {
                this.$el.html();
                var that = this;
                this.titleModel = new Backbone.Model({
                    itemTitle: CAPTIONS.NOTE_ITEM_TITLE,
                    id: "noteItemTitleView" + this.model.get("itemIndex"),
                    noteItemModel: that.model,
                    color: projectNotes.colorCollection[this.model.get("itemIndex") % projectNotes.colorCollection.length],
                    getTitle: false
                });
                var currentdate = new Date();

                var datetime = currentdate;
                this.dateModel = new Backbone.Model({
                    itemDate: datetime,
                    itemDateMilliseconds: datetime + ":" + currentdate.getMilliseconds(),
                    id: "noteItemDateView" + this.model.get("itemIndex"),
                    noteItemModel: that.model,
                    getDate: false
                });
                var titleItem = new projectNotes.NoteItemTitleView({id: "noteItemTitleView" + this.model.get("itemIndex"), model: this.titleModel});
                var dateIem = new projectNotes.NoteItemDateView({id: "noteItemDateView" + this.model.get("itemIndex"), model: this.dateModel});
                this.$el.append(titleItem.render().el);
                this.$el.append(dateIem.render().el);

                var notesList = JSON.parse(localStorage.getItem(Chapter.getSettings().sequenceId + "notesList"));
                if (notesList != null) {
                    notesList.push(new storageElement(this.titleModel, this.dateModel));
                    localStorage.removeItem(Chapter.getSettings().sequenceId + "notesList");
                    localStorage.setItem(Chapter.getSettings().sequenceId + "notesList", JSON.stringify(notesList));
                }
                else {
                    notesList = new Array(new storageElement(this.titleModel, this.dateModel));
                    localStorage.setItem(Chapter.getSettings().sequenceId + "notesList", JSON.stringify(notesList));
                }
                return this;
            }
        });

        /**
         * view for managing the list of note items
         * @type {*}
         */
        projectNotes.NotesListView = Backbone.View.extend({
            id: "notesListView",
            tagName: "div",
            className: 'notesListViewClass scrollable',
            itemIndex: 0,
            $welcomeText: null,
            notesList: null,
            modelsCollection: null,
            timer: 0,
            initialize: function (attributes, options) {
                this.listenTo(this.model, "change:addItem", this.addItem);
                this.listenTo(this.model, "change:removeItem", this.removeItem);

                this.modelsCollection = new Backbone.Collection();
                var that = this;
                this.timer = setInterval(function () {
                    console.log("it's time to update")
                    for (var i = 0; i < that.modelsCollection.models.length; i++) {
                        that.modelsCollection.models[i].set("updateDate", true);
                    }
                    projectNotes.dateModel.set("update", true);
                }, 60000);
            },

            /**
             * function used to bind the events the view is listening to
             * @returns _events - an array containing the events
             */
            events: function () {
                var _events = {};
                _events[AppUtils.EventTypes.click] = "exitTitleEditMode";
                return _events;
            },

            /**
             * function used to handle the click event
             */
            exitTitleEditMode: function () {
                if (projectNotes.titleItemListEditMode) {
                    // only exit edit mode
                    $('span[id^="noteItemTitleView"]').attr("contenteditable", false);

                    if (AppUtils.DeviceType.iOS) {
                        $(".ghostInput").focus().blur();
                    }

                    projectNotes.titleItemListEditMode = false;
                }
            },

            /**
             * function used to add a new item in the list
             */
            addItem: function () {
                if (this.model.get("addItem")) {
                    var itemModel = new Backbone.Model({
                        itemIndex: this.itemIndex,
                        id: "noteItemView" + this.itemIndex
                    });
                    if (this.itemIndex == 0) {
                        clearInterval(this.timer);
                        this.$welcomeText.css("display", "none");
                        var that = this;
                        this.timer = setInterval(function () {
                            console.log("it's time to update")
                            for (var i = 0; i < that.modelsCollection.models.length; i++) {
                                that.modelsCollection.models[i].set("updateDate", true);
                            }
                            projectNotes.dateModel.set("update", true);
                        }, 60000);
                    }
                    var noteItem = new projectNotes.NoteItemView({id: "noteItemView" + this.itemIndex, model: itemModel});
                    this.$el.append(noteItem.render().el);
                    this.itemIndex++;
                    this.modelsCollection.add(itemModel);
                    this.model.set("addItem", false);
                }
            },

            /**
             * function used to add a new note item in the list from local storage
             * @param title - the note item title
             * @param date  - the note item date
             * @param itemDateMilliseconds - the note item date with miliseconds
             */
            addItemFromList: function (title, date, itemDateMilliseconds) {
                var itemModel = new Backbone.Model({
                    itemIndex: this.itemIndex,
                    id: "noteItemView" + this.itemIndex,
                    titleValue: title,
                    dateValue: date,
                    itemDateToShow: moment(date).fromNow(),
                    itemDateMilliseconds: itemDateMilliseconds
                });
                var noteItem = new projectNotes.NoteItemView({id: "noteItemView" + this.itemIndex, model: itemModel});
                this.$el.append(noteItem.loadItemFromList().el);
                this.modelsCollection.add(itemModel);
                this.itemIndex++;
            },

            /**
             * function used to remove a note item from the list
             */
            removeItem: function () {
                if (this.model.get("removeItem")) {
                    projectNotes.selectedModel.set("remove", true);
                    this.itemIndex--;
                    if (this.itemIndex == 0) {
                        clearInterval(this.timer);
                        this.$welcomeText.css("display", "block");
                    }
                    this.modelsCollection.remove(projectNotes.selectedModel);
                    this.model.set("removeItem", false);
                }
            },

            /**
             * function used to render the note item list view
             */
            render: function () {
                this.$el.html();
                var notesList = JSON.parse(localStorage.getItem(Chapter.getSettings().sequenceId + "notesList"));
                if (notesList != null && notesList.length > 0) {
                    for (var i = 0; i < notesList.length; i++) {
                        this.addItemFromList(notesList[i].titleModel.itemTitle, notesList[i].dateModel.itemDate, notesList[i].dateModel.itemDateMilliseconds);
                    }
                }
                else {
                    this.$welcomeText = $(("<span id = 'welcomeText' class = 'welcomeTextClass' > Tap the icon in the top right corner to add a note </span>"));
                    this.$welcomeText.css("display", "block");
                    this.$el.append(this.$welcomeText);
                }
                return this;
            }
        });

        /**
         * view used to handle the close btn logic
         */
        projectNotes.CloseListViewBtnView = Backbone.View.extend({
            id: "closeListViewBtnView",
            tagName: "div",
            className: 'closeListViewBtnViewClass',
            template: projectNotesTemplates.getTemplate("closeListViewBtnViewImage"),
            initialize: function (attributes, options) {
            },

            /**
             * function used to bind the events the view is listening to
             * @returns _events - an array containing the events
             */
            events: function () {
                var _events = {};
                _events[AppUtils.EventTypes.mouseDownSafeIOS5] = 'doOnClick';
                return _events;
            },

            /**
             * function used to handle the mouse down event
             */
            doOnClick: function () {
                this.$el.addClass("pressedButton");
                var that = this;
                setTimeout(function () {
                    that.$el.removeClass("pressedButton");
                }, 200);
                projectNotes.viewPortModel.set("displayed", false);
            },

            /**
             * function used to render the close btn view
             */
            render: function () {
                this.$el.html(this.template(this.model.attributes));
                this.$el.addClass("CommonButtonClass");
                return this;
            }
        });

        /**
         * view used to handle the close primary view btn logic
         */
        projectNotes.ClosePrimaryViewBtnView = Backbone.View.extend({
            id: "closePrimaryViewBtnView",
            tagName: "div",
            className: 'notesClosePrimaryViewBtnViewClass',
            template: projectNotesTemplates.getTemplate("closePrimaryViewBtnImage"),
            initialize: function (attributes, options) {
            },

            /**
             * function used to bind the events the view is listening to
             * @returns _events - an array containing the events
             */
            events: function () {
                var _events = {};
                _events[AppUtils.EventTypes.mouseDownSafeIOS5] = 'doOnClick';
                return _events;
            },

            /**
             * function used to handle the mouse down event
             */
            doOnClick: function () {
                this.$el.addClass("pressedButton");
                var that = this;
                setTimeout(function () {
                    that.$el.removeClass("pressedButton");
                }, 200);
                projectNotes.viewPortModel.set("shouldClose", true);
            },

            /**
             * function used to render the close primary view btn view
             */
            render: function () {
                this.$el.html(this.template(this.model.attributes));
                this.$el.addClass("CommonButtonClass");
                return this;
            }
        });

    };

    /**
     * function used to initialize the notes feature
     */
    projectNotes.show = function () {

        if (null != projectNotes.viewPortModel) {
            projectNotes.viewPortModel.set("displayed", true);
        } else {
            projectNotes.defineViews();
            projectNotes.createModels();
            projectNotes.createViews();

        }
    };

    /**
     * function used for creating backbone models
     */
    projectNotes.createModels = function () {

        projectNotes.viewPortModel = new Backbone.Model({
            displayed: false
        });
        projectNotes.notesContainerModel = new Backbone.Model({
        });
        projectNotes.notesPrimaryModel = new Backbone.Model({
        });
        projectNotes.toolbarModel = new Backbone.Model({
        });
        projectNotes.titleModel = new Backbone.Model({
            titleTextValue: "title"
        });
        projectNotes.dateModel = new Backbone.Model({
            dateValue: "date"
        });
        projectNotes.contentModel = new Backbone.Model({
        });
        projectNotes.textModel = new Backbone.Model({
            displayed: true,
            shouldClear: false
        });
        projectNotes.drawingModel = new Backbone.Model({
            displayed: false,
            shouldClear: false
        });
        projectNotes.notesBackBtnModel = new Backbone.Model({
        });
        projectNotes.drawingBtnModel = new Backbone.Model({
        });
        projectNotes.textBtnModel = new Backbone.Model({
        });
        projectNotes.clearBtnModel = new Backbone.Model({
        });
        projectNotes.notesListPrimaryModel = new Backbone.Model({
            displayed: false
        });
        projectNotes.notesHeaderModel = new Backbone.Model({
        });
        projectNotes.newNoteBtnModel = new Backbone.Model({
        });
        projectNotes.deleteNoteBtnModel = new Backbone.Model({
        });
        projectNotes.notesListModel = new Backbone.Model({
            addItem: false,
            removeItem: false
        });
        projectNotes.closeListViewBtnModel = new Backbone.Model({
        });
        projectNotes.closePrimaryViewBtnModel = new Backbone.Model({
        });

    };

    /**
     * function used for creating backbone views
     */
    projectNotes.createViews = function () {

        var drawingProperties = {};
        drawingProperties.id = "notes_drawing_element";
        drawingProperties.lineColor = '#000000';
        drawingProperties.markerSize = '3px';
        drawingProperties.borderStyle = "Solid";
        drawingProperties.areColorsEnabled = "true";
        drawingProperties.left = '8%';
        drawingProperties.top = '24%';
        drawingProperties.width = '84%';
        drawingProperties.height = '69%';
        drawingProperties.displayed = true;
        drawingProperties.menuLeft = -45;
        drawingProperties.drawingIconURL = 'gfx/notes/PAINT_input.png';

        var textInputProperties = {},
            customProperties = {};
        textInputProperties.id = "notes_text_element";
        textInputProperties.left = '8%';
        textInputProperties.top = '24%';
        textInputProperties.width = '84%';
        textInputProperties.height = '69%';
        textInputProperties.displayed = true;
        textInputProperties.isEnabled = true;
        textInputProperties.isUsedInNotes = true;
        textInputProperties.textStyleClass = "textAreaClass";
        customProperties.placeholder = CAPTIONS.EDIT_NOTE_PALCEHOLDER;

        if (null != device && typeof device != "") {
            if (device.version < "7") {
                customProperties.placeholder = CAPTIONS.EDIT_NOTE_PALCEHOLDER_SHORT;
            }
        }

        require(['TextInputElement', 'DrawingElement', 'DrawingElementMenu'], function (TextInputElement, DrawingElement) {
            projectNotes.drawElement = new DrawingElement({'properties': drawingProperties}, new Backbone.Model());
            projectNotes.drawElement.init();

            projectNotes.inputTextBoxElement = new TextInputElement({'properties': textInputProperties, 'customProperties': customProperties}, new Backbone.Model());
            projectNotes.inputTextBoxElement.init();

            projectNotes.viewPortView = new projectNotes.ViewPortView({
                model: projectNotes.viewPortModel
            });

            projectNotes.notesContainerView = new projectNotes.NotesContainerView({
                model: projectNotes.notesContainerModel
            });
            projectNotes.notesPrimaryView = new projectNotes.NotesPrimaryView({
                model: projectNotes.notesPrimaryModel
            });
            projectNotes.toolbarView = new projectNotes.ToolbarView({
                model: projectNotes.toolbarModel
            });
            projectNotes.titleView = new projectNotes.TitleView({
                model: projectNotes.titleModel
            });
            projectNotes.dateView = new projectNotes.DateView({
                model: projectNotes.dateModel
            });
            projectNotes.contentView = new projectNotes.ContentView({
                model: projectNotes.contentModel
            });
            projectNotes.notesBackBtnView = new projectNotes.NotesBackBtnView({
                model: projectNotes.notesBackBtnModel
            });
            projectNotes.drawingBtnView = new projectNotes.DrawingBtnView({
                model: projectNotes.drawingBtnModel
            });
            projectNotes.textBtnView = new projectNotes.TextBtnView({
                model: projectNotes.textBtnModel
            });
            projectNotes.clearBtnView = new projectNotes.ClearBtnView({
                model: projectNotes.clearBtnModel
            });
            projectNotes.notesListPrimaryView = new projectNotes.NotesListPrimaryView({
                model: projectNotes.notesListPrimaryModel
            });
            projectNotes.notesHeaderView = new projectNotes.NotesHeaderView({
                model: projectNotes.notesHeaderModel
            });
            projectNotes.newNoteBtnView = new projectNotes.NewNoteBtnView({
                model: projectNotes.newNoteBtnModel
            });
            projectNotes.deleteNoteBtnView = new projectNotes.DeleteNoteBtnView({
                model: projectNotes.deleteNoteBtnModel
            });
            projectNotes.notesListView = new projectNotes.NotesListView({
                model: projectNotes.notesListModel
            });
            projectNotes.closeListViewBtnView = new projectNotes.CloseListViewBtnView({
                model: projectNotes.closeListViewBtnModel
            });
            projectNotes.closePrimaryViewBtnView = new projectNotes.ClosePrimaryViewBtnView({
                model: projectNotes.closePrimaryViewBtnModel
            });

            projectNotes.addViewsToDOM();
        });


    };

    /**
     * function used add views in DOM
     */
    projectNotes.addViewsToDOM = function () {
        App.viewPortView.$el.append(projectNotes.viewPortView.render().el);
    };


})(window["projectNotes"]);

/**
 * data structure used to store a note in the local storage
 * @param titleModel - the title model of the note
 * @param dateModel - the date model of the note
 */
function storageElement(titleModel, dateModel) {
    this.titleModel = titleModel;
    this.dateModel = dateModel;
    this.isCanvasDirty = false;
}

/**
 * utility function used to place the carret at the end of an editing text box/input
 * @param el - the text box/input
 */
function placeCaretAtEnd(el) {
    el.focus();
    if (typeof window.getSelection != "undefined"
        && typeof document.createRange != "undefined") {
        var range = document.createRange();
        range.selectNodeContents(el);
        range.collapse(false);
        var sel = window.getSelection();
        sel.removeAllRanges();
        sel.addRange(range);
    } else if (typeof document.body.createTextRange != "undefined") {
        var textRange = document.body.createTextRange();
        textRange.moveToElementText(el);
        textRange.collapse(false);
        textRange.select();
    }
}
