define(['GenericLightBox'], function (GenericLightBox) {
    return function (rawCameraElementModel, pageModel) {
        var that = this,
            destinationType = 1;     //Camera.DestinationType.FILE_URI = 1
        this.defineClasses = function () {

            that.CameraElementView = ElementBaseView.extend({
                tagName: "div",
                className: "sceneObject cameraDefaultPictureWrapper",
                template: templates.getTemplate("image"),
                elementsViews: [],
                id: rawCameraElementModel.properties.id,
                parentElementModel: pageModel,
                lightBox: null,
                $storedImage: null,
                events: function () {
                    var _events = {};
                    if (AppUtils.nativeApp) {
                        _events[AppUtils.EventTypes.click] = "clickHandler";
                    }
                    return _events;
                },
                addCustomListeners: function () {
                    this.listenTo(this.model, "change:removeLightBox", this.removeLightBox);
                    this.listenTo(this.model, "change:successfulCameraPicture", this.successfulCameraPicture);
                    this.listenTo(this.model, "change:failedCameraPicture", this.failedCameraPicture);
                },
                successfulCameraPicture: function () {
                    var imageURI = this.model.get('successfulCameraPicture'),
                        img, $img;
                    if (null != imageURI) {
                        img = new Image();
                        img.src = imageURI;
                        $img = $(img);


                        if (!that.cameraElementModel.get('isStreched')) {   //if is not stretched then the image will be scaled
                            img.onload = function () {
                                var widthRatio = img.naturalWidth / that.cameraElementView.$el.width();
                                var heightRatio = img.naturalHeight / that.cameraElementView.$el.height();

                                if (widthRatio < heightRatio) {
                                    $img.addClass("cameraHeightImg");
                                    that.cameraElementView.$el.addClass("cameraHeightWrapper");
                                }
                                else {
                                    $img.addClass("cameraWidthImg");
                                    that.cameraElementView.$el.addClass("cameraWidthWrapper");
                                    that.cameraElementView.$el.css("line-height", that.cameraElementView.$el.height() + "px");
                                }
                            }
                        }
                        else {
                            $img.addClass("cameraLoadedPicture");
                            that.cameraElementView.$el.addClass("cameraLoadedPictureWrapper");
                        }
                        $img.attr('id', that.cameraElementView.model.get('id') + "_img");
                        localStorage.setItem(pageModel.get('id') + "_" + that.cameraElementView.model.get('id'), imageURI);
                        that.cameraElementView.$el.html($img);
                        that.cameraElementModel.set('removeLightBox', true);
                        this.model.set('imageURI', null);
                    }

                },
                failedCameraPicture: function () {
                    var message = this.model.get('failedCameraPicture');
                    if (null != message) {
                        if (message != "no image selected") {
                            UserDialogs.showAlert('Failed because: ' + message);
                        }
                        that.cameraElementModel.set('removeLightBox', true);
                        this.model.set('failedCameraPicture', null);
                    }

                },
                removeLightBox: function () {
                    if (this.model.get('removeLightBox')) {
                        this.model.set('removeLightBox', false);
                        this.lightBox.getModel().set('remove', true);
                        this.lightBox = null;
                    }

                },
                removeView: function () {
                    if (this.model.get("removed")) {
                        this.remove();
                        this.unbind();
                    }
                },
                activateObject: function () {
                    if (this.model.get("activateObject")) {
                        this.model.set("objectActivated", true);
                        this.model.set("activateObject", false);
                    }
                },

                objectActivated: function () {
                    if (this.model.get('objectActivated')) {

                        var model = new Backbone.Model(),
                            btnWrapperView = new that.BtnWrapperView({
                                model: model
                            }),
                            lightBoxWindowSettings = {
                                width: '40%',
                                height: '40%'
                            };

                        this.lightBox = new GenericLightBox(null, lightBoxWindowSettings, {content: btnWrapperView.render().el});
                        this.lightBox.init();
                        App.viewPortView.$el.append(this.lightBox.getView().render().el);
                        this.model.set('objectActivated', false);
                    }
                },


                getInitialContent: function () {
                    if (AppUtils.nativeApp) {
                        return this.getNativeElement();
                    }
                    else return this.getBrowserElement();
                },
                getNativeElement: function () {
                    var $ret = $(this.template(this.model.attributes)),
                        src = localStorage.getItem(pageModel.get('id') + "_" + this.model.get('id')),
                        widthRatio, heightRatio,
                        thisView = this, img = $ret[0];
                    if (null == src) {
                        $ret.removeClass('images').addClass('cameraDefaultPicture');
                    } else {
                        $ret.attr('src', src);
                        if (!this.model.get('isStreched')) {
                            $ret.bind("load", function () {
                                widthRatio = img.naturalWidth / thisView.$el.width();
                                heightRatio = img.naturalHeight / thisView.$el.height();

                                if (widthRatio <= heightRatio) {
                                    $ret.addClass("cameraHeightImg");
                                    thisView.$el.addClass("cameraHeightWrapper");
                                }
                                else {
                                    $ret.addClass("cameraWidthImg");
                                    thisView.$el.addClass("cameraWidthWrapper");
                                    thisView.$el.css("line-height", thisView.$el.height() + "px");
                                }
                            });
                        } else {
                            $ret.addClass("cameraLoadedPicture");
                            this.$el.addClass("cameraLoadedPictureWrapper");
                        }
                        $ret.attr('id', this.model.get('id') + "_img");
                    }
                    return $ret;
                },
                getBrowserElement: function () {
                    return $("<span>").html(CAPTIONS.CAMERA_NOT_SUPPORTED).addClass("cameraNotSupported");
                },
                render: function () {
                    if (this.shouldRenderObject()) {
                        this.renderStartHandler();
                        this.setLeft(this.model.get("left"));
                        this.setWidth(this.model.get("width"));
                        this.setHeight(this.model.get("height"));
                        this.setTop(this.model.get("top"));
                        this.$el.css("z-index", this.model.get("ZIndex"));
                        this.setInitialVisibility();
                        this.$el.html();
                        this.$el.append(this.getInitialContent());
                        return this;
                    }
                    return null;
                }
            });
            that.BtnWrapperView = Backbone.View.extend({
                tagName: "div",
                className: "btnContainer",
                id: 'cameraBtnContainer',

                removeView: function () {
                    if (this.model.get("removed")) {
                        this.remove();
                        this.unbind();
                    }
                },

                appendChildren: function () {
                    var newPictureBtnModel = new Backbone.Model({
                            imageURL: 'gfx/camera_icon.png'
                        }),

                        galleryBtnModel = new Backbone.Model({
                            imageURL: 'gfx/browse_files.png'
                        }),
                        newPictureBtnView = new that.NewPictureBtnView({
                            model: newPictureBtnModel
                        }),
                        galleryBtnView = new that.GalleryBtnView({
                            model: galleryBtnModel
                        });
                    this.$el.append(newPictureBtnView.render().el);
                    this.$el.append(galleryBtnView.render().el);
                },


                render: function () {
                    this.appendChildren();
                    return this;
                }
            });
            that.NewPictureBtnView = Backbone.View.extend({
                tagName: "div",
                className: "commonClass btnCamera",
                id: 'newPictureBtn',
                template: templates.getTemplate("cameraButtonImage"),
                removeView: function () {
                    if (this.model.get("removed")) {
                        this.remove();
                        this.unbind();
                    }
                },
                events: function () {
                    var _events = {};
                    _events[AppUtils.EventTypes.click + " .cameraButtonWrapper"] = "clickHandler";
                    return _events;
                },
                clickHandler: function (e) {
                    e.stopPropagation();
                    this.takePicture();
                },
                takePicture: function () {
                    var options =
                    {
                        quality: 100,
                        destinationType: destinationType,
                        sourceType: Camera.PictureSourceType.CAMERA
                    };

                    if (that.cameraElementModel.get('isStreched')) {
                        options['targetWidth'] = that.cameraElementView.$el.width();
                        options['targetHeight'] = that.cameraElementView.$el.height();
                    }

                    if (!Camera.Direction) {
                        Camera.Direction = {
                            FRONT: 1,
                            BACK: 0
                        }
                    }

                    if (that.cameraElementModel.get('selectedCamera') == "Front Camera") {
                        options['cameraDirection'] = Camera.Direction.FRONT;
                    } else if (that.cameraElementModel.get('selectedCamera') == "Back Camera") {
                        options['cameraDirection'] = Camera.Direction.BACK;
                    }
                    options['correctOrientation'] = true;
                    navigator.camera.getPicture(this.onSuccessCameraPicture, this.onFailCameraPicture, options);
                },

                onSuccessCameraPicture: function (imageURI) {
                    that.cameraElementModel.set('successfulCameraPicture', imageURI);
                },
                onFailCameraPicture: function (message) {
                    that.cameraElementModel.set('failedCameraPicture', message);
                },
                render: function () {
                    var $img ,
                        $template = $(this.template(this.model.attributes)),
                        cssProperty;
                    this.$el.html($template);
                    $img = $($template.children()[0]);
                    cssProperty = App.settings.width > App.settings.height ? 'height' : 'width';
                    $img.css(cssProperty, '100%');
                    return this;
                }
            });
            that.GalleryBtnView = BaseView.extend({
                tagName: "div",
                className: "commonClass btnGallery",
                id: 'galleryPictureBtn',
                template: templates.getTemplate("cameraButtonImage"),
                removeView: function () {
                    if (this.model.get("removed")) {
                        this.remove();
                        this.unbind();
                    }
                },

                events: function () {
                    var _events = {};
                    _events[AppUtils.EventTypes.click] = "clickHandler";
                    return _events;
                },
                clickHandler: function (e) {
                    e.stopPropagation();
                    this.takePicture();
                },
                takePicture: function () {
                    var options =
                    {
                        quality: 100,
                        destinationType: destinationType,
                        sourceType: Camera.PictureSourceType.PHOTOLIBRARY
                    };

                    if (that.cameraElementModel.get('isStreched')) {
                        options['targetWidth'] = that.cameraElementView.$el.width();
                        options['targetHeight'] = that.cameraElementView.$el.height();
                    }

                    navigator.camera.getPicture(this.onSuccessCameraPicture, this.onFailCameraPicture, options);
                },

                onSuccessCameraPicture: function (imageURI) {
                    that.cameraElementModel.set('successfulCameraPicture', imageURI);
                },
                onFailCameraPicture: function (message) {
                    that.cameraElementModel.set('failedCameraPicture', message);
                },

                render: function () {
                    var $img ,
                        $template = $(this.template(this.model.attributes)),
                        cssProperty;
                    this.$el.html($template);
                    $img = $($template.children()[0]);
                    cssProperty = App.settings.width > App.settings.height ? 'height' : 'width';
                    $img.css(cssProperty, '100%');
                    return this;
                }
            });

        };
        this.instantiateObjects = function () {
            this.cameraElementModel = this.getCameraElementModel();
            this.cameraElementView = new this.CameraElementView({
                model: that.cameraElementModel
            });

        };

        this.getCameraElementModel = function () {
            var model = new Backbone.Model(rawCameraElementModel.properties);
            model.set("imageURL", App.cameraIconURL);
            model.set('removeLightBox', false);
            return model;
        };


        return {
            init: function () {
                that.defineClasses();
                that.instantiateObjects();
            },
            getView: function () {
                return that.cameraElementView;
            },
            getModel: function () {
                return that.cameraElementModel;
            }

        }
    }
});