define(function () {
    return function (rawDownloadLinkModel, pageModel) {
        var that = this,
            mappedBorderStyleCSSClasses = {
                'Solid': 'divSolidBorder',
                'No border': 'divNoBorder',
                'Dotted': 'divDottedBorder',
                'Dashed': 'divDashedBorder',
                'Double': 'divDoubleBorder',
                'Groove': 'divGrooveBorder',
                'Ridge': 'divRidgeBorder'
            };

        this.defineClasses = function () {

            that.DownloadLinkView = ElementBaseView.extend({
                tagName: "div",
                className: "sceneObject",
                elementsViews: [],
                id: rawDownloadLinkModel.properties.id,
                parentElementModel: pageModel,
                template: templates.getTemplate("downloadLink"),

                customInitialize: function () {
                    this.model.set("allObjectTypesInPage", rawDownloadLinkModel.customProperties["allObjectTypesInPage"]);
                },
                events: function () {
                    var _events = {};
                    _events[AppUtils.EventTypes.mouseDown] = "touchstart";
//                    _events[AppUtils.EventTypes.mouseEnter] = "onMouseOverHandler";
//                    _events[AppUtils.EventTypes.mouseLeave] = "onMouseOutHandler";
                    _events[AppUtils.EventTypes.click] = "clickHandler";
                    _events["touchcancel"] = "touchcancelHandler";
                    _events["touchcend"] = "touchcendHandler";
                    return _events;
                },
                onMouseOverHandler: function () {
                },
                onMouseOutHandler: function () {
                },
                touchcendHandler: function () {
                    console.log("touchcendHandler on " + this.$el.attr("id"));
                },
                touchcancelHandler: function () {
                    console.log("touchcancelHandler on " + this.$el.attr("id"));
                },
                removeView: function () {
                    if (this.model.get("removed")) {
                        this.remove();
                        this.unbind();
                    }
                },
                activateObject: function () {
                    if (this.model.get("activateObject")) {
                        this.model.set("objectActivated", true);
                        this.model.set("activateObject", false);
                    }
                },
                activateToggle: function () {
                    this.displayHideOnceOnClick();
                },
                setBorder: function () {
                    this.$el.addClass(mappedBorderStyleCSSClasses[this.model.get('borderStyle')]);
                },

                render: function () {
                    if (this.shouldRenderObject()) {
                        var downloadPDFIconInfo = Chapter.getSettings()["downloadPDFIconInfo"];
                        this.model.set("imageURL", App.downloadPDFIconURL);
                        this.$el.html(this.template(this.model.attributes));
                        this.setLeft(this.model.get("left"));
                        this.setWidth(Math.round(downloadPDFIconInfo["downloadPDFIconWidth"] * App.settings.scaleFactor));
                        this.setHeight(Math.round(downloadPDFIconInfo["downloadPDFIconHeight"] * App.settings.scaleFactor));
                        this.setTop(this.model.get("top"));
                        this.$el.css("z-index", this.model.get("ZIndex"));
                        return this;
                    }
                    return null;
                }
            });
        };
        this.instantiateObjects = function () {
            this.downloadLinkModel = this.getDownloadLinkModel();
            this.downloadLinkView = new this.DownloadLinkView({
                model: that.downloadLinkModel
            });

        };

        this.getDownloadLinkModel = function () {
            return  new Backbone.Model(rawDownloadLinkModel.properties);

        };


        return {
            init: function () {
                that.defineClasses();
                that.instantiateObjects();
            },
            getView: function () {
                return that.downloadLinkView;
            },
            getModel: function () {
                return that.downloadLinkModel;
            }

        }
    }
});