define(function () {
    return function (dualPageModel) {
        var that = this;

        this.defineClasses = function () {

            that.DualPageView = BaseView.extend({
                tagName: "div",
                childPages: 0,
                className: "dualPageWrapper",
                id: "dualPageWrapper-" + dualPageModel.wrapperNo,
                elementsViews: [],
                zoomInstance: null,

                initialize: function () {
                    this.addListeners();
                },

                addListeners: function () {
//                    this.model.on("change:removedPage", this.updatedChildPages, this);
//                    this.model.on("change:addedPage", this.updatedChildPages, this);
//                    this.model.on("change:orientation", this.updateOrientation, this);
                    this.listenTo(this.model, "change:removedPage", this.updatedChildPages);
                    this.listenTo(this.model, "change:addedPage", this.updatedChildPages);
                    this.listenTo(this.model, "change:orientation", this.updateOrientation);
                },

                updateOrientation: function () {
                    console.log("Update dual page with index " + this.model.get("wrapperNo"));
                    this.setWidth(App.settings.pageWidth * 2);
                    this.setHeight(App.settings.height);
                    this.setLeft(App.settings.pageWidth * 2 * this.model.get("wrapperNo"))
                },

                updatedChildPages: function () {
                    var removedPage = this.model.get("removedPage");
                    var addedPage = this.model.get("addedPage");
                    if (removedPage != null) {
                        this.model.set("removedPage", null);
                        this.childPages--;
                        if (this.childPages == 0) {
                            this.removeView();
                        }
                    } else if (addedPage != null) {
                        this.model.set("addedPage", null);
                        this.childPages++
                    }
                },

                setZoomInstance: function (zoomInstance) {
                    this.zoomInstance = zoomInstance;
                },

                getZoomInstance: function () {
                    return this.zoomInstance;
                },

                removeView: function () {
                    if (this.zoomInstance) {
                        this.zoomInstance.destroy();
                        this.zoomInstance = null;
                    }
                    this.remove();
                    this.unbind();
                    // TODO: see how we can do a better remove of view !!!
//                    this.model.unbind('change', this.render, this); // should remove all change listeners
//                    delete this.$el; // Delete the jQuery wrapped object variable
//                    delete this.el; // Delete the variable reference to this node
                    this.model.set("needsRender", true);
                },
                render: function () {
                    this.setLeft(this.model.get("left"));
                    this.setWidth(this.model.get("width"));
                    this.setHeight(this.model.get("height"));
                    this.model.set("needsRender", false);
                    return this;
                }
            });
        };

        this.instantiateObjects = function () {
            that.dualPageModel = this.getDualPageModel();
            that.DualPageView = new this.DualPageView({
                model: that.dualPageModel
            });

        };

        this.getDualPageModel = function () {
            return new Backbone.Model(dualPageModel);
        };


        return {
            init: function () {
                that.defineClasses();
                that.instantiateObjects();
            },
            getView: function () {
                return that.DualPageView;
            },
            getModel: function () {
                return that.dualPageModel;
            }
        }
    }
});