var ElementBaseView = function (options) {
    BaseView.apply(this, [options]);
};

_.extend(ElementBaseView.prototype, BaseView.prototype, {

    lastZoomValueForAutomaticZoom: null,

    activateEffect: function () {
        if (this.model.get("activateEffect")) {
            var currentEffect = this.getActiveEffect();
            if (null != this.model.get(currentEffect))
                this[currentEffect]();
        }
    },

    displayHideOnceOnClick: function () {
        var thisView = this;
        EffectsHandler.displayHideOnceOnClick(thisView.$el, thisView.model);
    },
    displayHideOnClick: function () {
        var thisView = this;
        EffectsHandler.displayHideOnClick(thisView.$el, thisView.model);
    },
    fadeInOutOnceOnClick: function () {
        var thisView = this;
        EffectsHandler.fadeInOutOnceOnClick(thisView.$el, thisView.model);
    },
    fadeInOutOnClick: function () {
        var thisView = this;
        EffectsHandler.fadeInOutOnClick(thisView.$el, thisView.model);
    },
    onePulseOnceOnClick: function () {
        var thisView = this;
        EffectsHandler.onePulseOnceOnClick(thisView.$el, thisView.model);
    },
    onePulseOnClick: function () {
        var thisView = this,
            effectInprogress = this.model.get("effectInProgress");
        if (null == effectInprogress || !effectInprogress) {
            EffectsHandler.onePulseOnClick(thisView.$el, thisView.model);
        }
    },
    multiplePulsesOnceOnClick: function () {
        var thisView = this;
        EffectsHandler.multiplePulsesOnceOnClick(thisView.$el, thisView.model);
    },
    multiplePulsesOnClick: function () {
        var thisView = this;
        EffectsHandler.multiplePulsesOnceOnClick(thisView.$el, thisView.model);
    },
    rotateObject: function () {
        var rotateAngle = this.model.get("rotateDegree");
        if (rotateAngle != null && !isNaN(rotateAngle) && rotateAngle != 0) {
            CSSHandler.rotate(this.$el, -rotateAngle);
        }
    },
    isGPUAccelerated: function () {
        var ret = false;
        if (AppUtils.DeviceType.CHROME) {
            var advancedSettings = this.model.get("advancedSettings"),
                prerenderObjectInGPU;
            if (advancedSettings) {
                prerenderObjectInGPU = advancedSettings["prerenderObjectInGPU"];
                if (prerenderObjectInGPU) {
                    ret = true;
                }
            }
        }
        if (AppUtils.DeviceType.ANDROID) { // actions to mitigate flickering on android devices
            var toLinks = this.model.get("links")["to"]; // an array
            for (var idx = 0; idx < toLinks.length; idx++) {
                if (toLinks[idx]["type"] == "moveTo") {
                    ret = true;
                    break;
                }
            }
        }
        return ret;
    },
    addHWAccelerationIfRequired: function () {
        if (this.isGPUAccelerated()) {
            this.makeElementRenderInGPU();
        }
    },
    shouldAdjustForMovingLayers: function () {
        return this.parentElementModel.get("hasMovingLayers") && AppUtils.DeviceType.CHROME;
    },
    makeElementRenderInGPU: function () {
        CSSHandler.setBackfaceVisibility(this.$el, "hidden");
        this.parentElementModel.set("pageModelUpdate", {
            "key": "updateForHardwareAcceleration",
            "value": true,
            "salt": new Date().getTime()
        });
    },
    renderStartHandler: function () {
        this.rotateObject();
        this.bypassChromeBugs();
    },

    /**
     * This effect must be done at page level
     * We notify the page and send the necessary information to the page
     */
    zoomInStartingPoint: function (zoomProperties) {
        var zoomedIn = this.model.get('zoomedIn');
        if (null == zoomedIn || !zoomedIn) {
            this.model.set('zoomToInAction', true);
            this.model.set('zoomedIn', true);
            this.parentElementModel.set("zoomedInObject", this.getZoomedInObjectInformation(zoomProperties));
            this.model.set("activateEffect", false);
        } else {
            if ((null == this.model.get('zoomedOut') || this.model.get('zoomedOut')) && (this.model.get("disallowTapAbortZoom") == null || this.model.get("disallowTapAbortZoom") == false)) {
                this.model.set('zoomedOut', false);
                this.model.set('zoomedIn', false);
                this.zoomOutStartingPoint();
            }
        }
    },
    zoomOutStartingPoint: function () {
        var zoomedOut = this.model.get('zoomedOut');
        if (null == zoomedOut) {
            this.model.set("zoomedOut", false);
            this.parentElementModel.set("zoomedInObject", this.getZoomedInObjectInformation());
            this.model.set("activateEffect", false);
        } else if (!zoomedOut) {
            this.model.set("zoomedOut", true);
            this.parentElementModel.set("zoomedOutObject", this.getZoomedOutObjectInformation());
        }
    },
    backTo: function () {
        Chapter.backTo();
    },
    startRecordingWizard: function () {
        console.log("startRecordingWizard");
        EffectsHandler.startRecordingWizard();
        this.model.set("activateEffect", false);
    },
    startPlayRecordedSounds: function () {
        console.log("startPlayRecordedSounds");
        EffectsHandler.startPlayRecordedSounds();
        this.model.set("activateEffect", false);
    },
    stopPlayRecordedSounds: function () {
        console.log("stopPlayRecordedSounds");
        EffectsHandler.stopPlayRecordedSounds();
        this.model.set("activateEffect", false);
    },
    print: function () {
//      window.print();
    },
    close: function () {
        this.parentElementModel.set('closeLightBoxes', true);
    },
    unlockContentForEnvironment: function (environmentId) {
        console.log("Now unlockContentForEnvironment");
        var unlockContentSettings = this.model.get("UNLOCK_CONTENT_VIA_SS"),
            thisView = this;
        require(["UnlockContentLightBox"], function (UnlockContentLightBox) {
            var unlockContentLightBox = new UnlockContentLightBox(unlockContentSettings, environmentId, thisView);
            unlockContentLightBox.init();
        });
    },
    unlockContentViaSS: function () {
        console.log("Now unlockContentViaSS");
        var that = this;
        if (AppUtils.nativeApp) {
            window.plugins.PhoneGapUtils.getDeviceIdentifierForVendor(function (deviceVendorId) {
                that.unlockContentForEnvironment(deviceVendorId + "_" + Chapter.getSettings()["rootProjectId"]);
            });
        } else {
            // in this case we use as environmentId only rootProjectId // good for testing in web browsers
            this.unlockContentForEnvironment(Chapter.getSettings()["rootProjectId"]);
        }
        this.model.set("activateEffect", false);
    },

    getZoomedInObjectInformation: function (properties) {
        var ret = {};

        ret.left = parseFloat(this.model.get("left")) * App.settings.width / 100;
        ret.top = parseFloat(this.model.get("top")) * App.settings.height / 100;
        ret.width = parseFloat(this.model.get("width")) * App.settings.width / 100;
        ret.height = parseFloat(this.model.get("height")) * App.settings.height / 100;


        if (this.model.get('zoomToStarted')) {
            ret.zoomStartingPointDuration = properties == null ? 0 : properties['linkedZoomToEffectDuration'];
            this.model.set('zoomToStarted', false);
        } else {
            ret.zoomStartingPointDuration = this.model.get("zoomOutStartingPoint") ? 0 : this.model.get("zoomStartingPointDuration");
        }

        ret.delay = properties == null ? this.model.get("zoomStartingPointDelayBeforeZooming") : properties['linkedZoomToEffectDelayBeforeZooming'];
        ret.timingFunction = this.getZoomTimingFunction(properties);

        return ret;
    },
    getZoomedOutObjectInformation: function () {
        var ret = {};
        ret.zoomStartingPointDuration = this.model.get("zoomStartingPointDuration");
        ret.delay = this.model.get("zoomInStartingPoint") ? 0 : this.model.get("zoomStartingPointDelayBeforeZooming");
        ret.timingFunction = this.model.get("zoomStartingPointTimingFunction");
        return ret;
    },
    startZoomEffectsIfNeeded: function () {
        if (this.model.get("zoomOutStartingPoint")) {
            this.model.set("activateEffect", true);
        }
    },

    setObjectSizeAndPositionWithAbsoluteValues: function (size, position) {
        if (position) {
            var leftValuePX = Math.round(parseFloat(this.model.get("left")) * this.parentElementModel.get('absoluteWidth') / 100);
            var topValuePX = Math.round(parseFloat(this.model.get("top")) * this.parentElementModel.get('absoluteHeight') / 100);
            this.setLeft(leftValuePX);
            this.setTop(topValuePX);
        }
        if (size) {
            var widthValuePX = Math.round(parseFloat(this.model.get("width")) * this.parentElementModel.get('absoluteWidth') / 100);
            var heightValuePX = Math.round(parseFloat(this.model.get("height")) * this.parentElementModel.get('absoluteHeight') / 100);
            this.setWidth(widthValuePX);
            this.setHeight(heightValuePX);
        }
    },

    goToScene: function () {
        console.log("goToScene with id : " + this.model.get('goToObjectId'));
        var nextPage = Chapter.getPageNumberForSceneId(this.model.get('goToObjectId')),
            customTransition = this.model.get("CUSTOM_PROPERTIES") ? this.model.get("CUSTOM_PROPERTIES")[0] : null;
        console.log("getPageNumberForSceneId : " + this.model.get('goToObjectId'));
        AppUtils.navigateToPage(nextPage, customTransition);
    },

    goToProject: function () {
        if (AppUtils.nativeApp) {
            this.goToNativeProject();
        } else if (AppUtils.isSCORM) {
            this.goToBrowserProjectSCORM();
        } else {
            this.goToBrowserProject();
        }

    },
    goToNativeProject: function () {
        var thisView = this;
        require(['IAPService'], function (IAPService) {
            var iapService = new IAPService();
            iapService.makePurchase(iapService.getProductId(thisView.model), function () {
                thisView.parentElementModel.set('downloadComplete', thisView.model.get('objectCategoryIds'));
                require(['DownloadService'], function (DownloadService) {
                    var downloadService = new DownloadService();
                    downloadService.downloadProjectArchive(thisView.model.get('goToObjectId'), function (response) {
                        console.log('Download successful' + response);
                        require(['NavigationService'], function (NavigationService) {
                            var navigationService = new NavigationService();
                            navigationService.goToProject(thisView.model.get('goToObjectId'));
                        });
                    }, function (error) {
                        console.log('Project archive download process failed ' + error);
                    });

                });
            }, function (error) {
                console.log('Project purchase process failed ' + error);
            });
        });
    },
    goToBrowserProject: function () {
        var relativeURL = 'action=getShareURL&projectId=' + this.model.get('goToObjectId');
        require(['NavigationService'], function (NavigationService) {
            var navigationService = new NavigationService();
            navigationService.goToProject(relativeURL);
        });
    },

    goToBrowserProjectSCORM: function () {
        var projectId = this.model.get('goToObjectId');
        require(['NavigationService'], function (NavigationService) {
            var navigationService = new NavigationService();
            navigationService.goToProject(projectId);
        });
    },

    goToWebURL: function () {
        var url = this.model.get('externalURL');
        AppUtils.openURL(url);
    },

    getGoToObjectType: function () {
        var goToObjectType = this.model.get("goToObjectType"),
            ret = null;
        if (goToObjectType == GoToObjectType.GoToScene) {
            ret = 'goToScene';
        } else if (goToObjectType == GoToObjectType.GoToProject) {
            ret = 'goToProject';
        } else if (goToObjectType == GoToObjectType.GoToWebURL || goToObjectType == GoToObjectType.GoToAppStoreURL) {
            ret = 'goToWebURL';
        }
        return ret;
    },

    getActiveEffect: function () {
        var currentEffect;
        if (this.model.get("displayHideOnceOnClick")) {
            currentEffect = 'displayHideOnceOnClick';
        } else if (this.model.get("displayHideOnClick")) {
            currentEffect = 'displayHideOnClick';
        } else if (this.model.get("fadeInOutOnceOnClick")) {
            currentEffect = 'fadeInOutOnceOnClick';
        } else if (this.model.get("fadeInOutOnClick")) {
            currentEffect = 'fadeInOutOnClick';
        } else if (this.model.get("onePulseOnceOnClick")) {
            currentEffect = 'onePulseOnceOnClick';
        } else if (this.model.get("onePulseOnClick")) {
            currentEffect = 'onePulseOnClick';
        } else if (this.model.get("multiplePulsesOnceOnClick")) {
            currentEffect = 'multiplePulsesOnceOnClick';
        } else if (this.model.get("multiplePulsesOnClick")) {
            currentEffect = 'multiplePulsesOnClick';
        } else if (this.model.get("zoomInStartingPoint")) {
            currentEffect = 'zoomInStartingPoint';
        } else if (this.model.get("zoomOutStartingPoint")) {
            currentEffect = 'zoomOutStartingPoint';
        } else if (this.model.get("backTo")) {
            currentEffect = 'backTo';
        } else if (this.model.get("startRecordingWizard")) {
            currentEffect = 'startRecordingWizard';
        } else if (this.model.get("startPlayRecordedSounds")) {
            currentEffect = 'startPlayRecordedSounds';
        } else if (this.model.get("stopPlayRecordedSounds")) {
            currentEffect = 'stopPlayRecordedSounds';
        } else if (this.model.get("close")) {
            currentEffect = 'close';
        } else if (this.model.get("unlockContentViaSS")) {
            currentEffect = "unlockContentViaSS";
        } else if (this.model.get('print') == true) {
            currentEffect = 'print';
        }
        return currentEffect;
    },
    hasOtherAction: function () {
        var ret = false;

        if (this.model.get("addObjectCategoryIds") != null && this.model.get("addObjectCategoryIds") != "" && this.model.get("addObjectCategoryIds") != "null" && this.model.get("addObjectCategoryIds") != undefined) {
            ret = true;
        } else if (this.model.get("animateText")) {
            ret = true;
        } else if (this.model.get("goToObjectId") != null && this.model.get("goToObjectId") != "" && this.model.get("goToObjectId") != "null" && this.model.get("goToObjectId") != undefined) {
            ret = true;
        } else if (this.model.get("goToObjectType") != null && this.model.get("goToObjectType") != "" && this.model.get("goToObjectType") != "null" && this.model.get("goToObjectType") != undefined) {
            ret = true;
        } else if (this.model.get("moveObjectCategoryIds") != null && this.model.get("moveObjectCategoryIds") != "" && this.model.get("moveObjectCategoryIds") != "null" && this.model.get("moveObjectCategoryIds") != undefined) {
            ret = true;
        } else if (this.model.get("removeObjectCategoryIds") != null && this.model.get("removeObjectCategoryIds") != "" && this.model.get("removeObjectCategoryIds") != "null" && this.model.get("removeObjectCategoryIds") != undefined) {
            ret = true;
        } else if (this.model.get("toggleObjectCategoryIds") != null && this.model.get("toggleObjectCategoryIds") != "" && this.model.get("toggleObjectCategoryIds") != "null" && this.model.get("toggleObjectCategoryIds") != undefined) {
            ret = true;
        } else if (this.model.get("setObjectCategoryIds") != null && this.model.get("setObjectCategoryIds") != "" && this.model.get("setObjectCategoryIds") != "null" && this.model.get("setObjectCategoryIds") != undefined) {
            ret = true;
        } else if (this.model.get("zoomObjectCategoryIds") != null && this.model.get("zoomObjectCategoryIds") != "" && this.model.get("zoomObjectCategoryIds") != "null" && this.model.get("zoomObjectCategoryIds") != undefined) {
            ret = true;
        }
//       else if (this.model.get("moveByPropertiesValues") != undefined && this.model.get("moveByPropertiesValues") != null) {
//           ret = true;
//       }

        var toLinks = this.model.get("links")["to"];

        if (toLinks.length > 0)
            for (var idx = 0; idx < toLinks.length; idx++) {
                if (toLinks[idx]["type"] != "moveTo") {
                    ret = true;
                    break;
                }
            }

        var fromLinks = this.model.get("links")["from"];

        if (fromLinks.length > 0)
            for (var idx = 0; idx < fromLinks.length; idx++) {
                if (fromLinks[idx]["type"] == "moveTo") {
                    ret = true;
                    break;
                }
            }

        return ret;
    },
    clickHandler: function (e) {
        console.log("ElementBaseView clickHandler intercepted by " + this.$el.attr("id"));
        if (this.model.get("preventClickEvent")) {
            this.model.set("preventClickEvent", false);
        } else {
            this.model.set("activateObject", true);
        }
        this.customClickHandler(e);
    },
    customClickHandler: function (e) {
        // empty here, to be implemented in actual page objects
    },
    touchstart: function (e) {
        console.log("ElementBaseView touchstart intercepted by " + this.$el.attr("id") + e.type);
        if (this.firstActionsOnTouchStart()) {
            // can now continue other action
        } else {
            console.log("ElementBaseView touchstart was canceled by firstActionsOnTouch")
        }
        this.customTouchStartHandler(e);
    },
    customTouchStartHandler: function (e) {
        // empty here, to be implemented in actual page objects
    },
    firstActionsOnTouchStart: function () {
        if (Chapter.getBottomMenu() == null)
            return true;

        // check if menu is opened and close if so
        var menuState = Chapter.getBottomMenu().getModel().get("expanded");
        if (menuState == MenuState.FULL_EXPANDED ||
            menuState == MenuState.ONLY_TOC) {
            Chapter.getBottomMenu().getModel().set("expanded", MenuState.CLOSED);
            this.model.set("preventClickEvent", true);
            return false; // prevent other touchstart actions
        }
        return true;
    },
    setInitialVisibility: function () {
        var opacity = this.model.get('displayed') ? this.model.get("opacity") : 0;
        CSSHandler.setOpacity(this.$el, opacity);
        if (this.isMoveToLinkTarget() || !(this.model.get('displayed') || ModelAccessFunctions.shouldNotHideBecauseOfEffects(this.model))) {
            this.$el.css("visibility", "hidden");
        }
    },
    isMoveToLinkTarget: function () {
        var links = this.model.get('links');
        if (null != links) {
            var link, index, incomingLinks = links['from'];
            for (index = 0; index < incomingLinks.length; index++) {
                link = incomingLinks[index];
                if (link['type'] == 'moveTo') {
                    return true;
                }
            }
        }
        return false;
    },

    currentObjectIsPartOfAnActiveCategory: function () {
        var idx, ocIDs = this.model.get("objectCategoryIds"),
            categories = Chapter.getObjectCategories();
        if (null != ocIDs) {
            for (idx = 0; idx < ocIDs.length; idx++) {
                if (categories.hasOwnProperty(ocIDs[idx])) {
                    if (categories[ocIDs[idx]].active) {
                        return true;
                    }
                }
            }
        } else {
            return true; // object is not part of any category, so like it is part of a category initially active
        }
        return false;
    },

    objectActivated: function () {
        if (this.model.get("objectActivated")) {
            this.model.set("objectActivated", false);
            this.applyEffectIfNeeded();
            this.applyGoToObjectIfNeeded();
            this.restoreLinkEffectsIfNeeded();
            this.notifyParentThatObjectIsActivated();
            this.notifyLinkedObjectsThatObjectIsActivated();
            this.startZoomAndPan();
            this.doIAPProductRestore();
            this.customActivationHandler();
        }
    },

    customActivationHandler: function () {
        // override in page objects as needed, keep here empty body
    },

    doIAPProductRestore: function () {
        var thisView = this;
        if (this.model.get("iOSRestoreTransactions")) {
            require(['IAPService'], function (IAPService) {
                var iapService = new IAPService();
                iapService.restoreProducts(function () {
                    thisView.doOnRestoreComplete(iapService);
                });
            });
        }
    },

    doOnRestoreComplete: function (iapService) {
        var models = this.parentElementModel.get('workingObjectsCollection').models,
            model, modelCategories,
            ownedProducts = iapService.getOwnedProducts(), categories;

        categories = {};
        categories.activatedCategories = this.getObjectCategoriesForDownloadCompleteEventForEmbeddedScenes(ownedProducts, models, iapService);
        categories.globalCategoryAction = false;
        this.parentElementModel.set("activatedObject", categories);

        for (var i = 0; i < models.length; i++) {
            model = models[i];
            modelCategories = model.get('objectCategoryIds');
            if (ownedProducts.indexOf(iapService.getProductId(model)) > -1) {
                this.parentElementModel.set('downloadComplete', modelCategories);
            }
        }
    },

    getObjectCategoriesForDownloadCompleteEventForEmbeddedScenes: function (ownedProducts, models, iapService) {
        var model, index, embeddedScenesCategories = {}, embeddedSceneCategories = {};
        embeddedScenesCategories.toggleObjectCategoryIds = [];
        embeddedScenesCategories.setObjectCategoryIds = [];
        embeddedScenesCategories.addObjectCategoryIds = [];
        embeddedScenesCategories.removeObjectCategoryIds = [];
        embeddedScenesCategories.moveObjectCategoryIds = [];
        embeddedScenesCategories.zoomObjectCategoryIds = [];
        for (index = 0; index < models.length; index++) {
            model = models[index];
            if (model.get('type') == 'embeddedScene') {
                embeddedSceneCategories = this.getObjectCategoriesForDownloadCompleteEvent(ownedProducts, Chapter.getEmbeddedScenes()[model.get('id')].objects, iapService);
                embeddedScenesCategories.toggleObjectCategoryIds = embeddedScenesCategories.toggleObjectCategoryIds.concat(embeddedSceneCategories.toggleObjectCategoryIds);
                embeddedScenesCategories.setObjectCategoryIds = embeddedScenesCategories.setObjectCategoryIds.concat(embeddedSceneCategories.setObjectCategoryIds);
                embeddedScenesCategories.addObjectCategoryIds = embeddedScenesCategories.addObjectCategoryIds.concat(embeddedSceneCategories.addObjectCategoryIds);
                embeddedScenesCategories.removeObjectCategoryIds = embeddedScenesCategories.removeObjectCategoryIds.concat(embeddedSceneCategories.removeObjectCategoryIds);
                embeddedScenesCategories.moveObjectCategoryIds = embeddedScenesCategories.moveObjectCategoryIds.concat(embeddedSceneCategories.moveObjectCategoryIds);
                embeddedScenesCategories.zoomObjectCategoryIds = embeddedScenesCategories.zoomObjectCategoryIds.concat(embeddedSceneCategories.zoomObjectCategoryIds);
            }
        }
        return embeddedScenesCategories;
    },

    getObjectCategoriesForDownloadCompleteEvent: function (ownedProducts, models, iapService) {
        var model,
            index,
            modelRelatedCategories,
            categoriesThatNeedToBeUpdated = {},
            modelObjectCategoriesIds,
            categoryIdIndex,
            productId,
            objectCategoriesIds = [],
            owned;


        categoriesThatNeedToBeUpdated.toggleObjectCategoryIds = [];
        categoriesThatNeedToBeUpdated.setObjectCategoryIds = [];
        categoriesThatNeedToBeUpdated.addObjectCategoryIds = [];
        categoriesThatNeedToBeUpdated.removeObjectCategoryIds = [];
        categoriesThatNeedToBeUpdated.moveObjectCategoryIds = [];
        categoriesThatNeedToBeUpdated.zoomObjectCategoryIds = [];

        for (index = 0; index < models.length; index++) {
            model = models[index];
            productId = iapService.getProductIdFromObject(model.properties);
            if (ownedProducts.indexOf(productId) > -1) {
                modelObjectCategoriesIds = this.extractOCIdsFrom(model.properties['objectCategoryIds']);
                objectCategoriesIds = objectCategoriesIds.concat(modelObjectCategoriesIds);
            }
        }


        for (index = 0; index < models.length; index++) {
            model = models[index];
            modelRelatedCategories = this.extractOCIdsFrom(model.properties['relatedCategoryId']);

            if (null != modelRelatedCategories) {
                owned = false;
                for (categoryIdIndex = 0; categoryIdIndex < modelRelatedCategories; categoryIdIndex++) {
                    if (objectCategoriesIds.indexOf(modelRelatedCategories[categoryIdIndex]) > -1) {
                        owned = true;
                        break;
                    }
                }
                if (owned) {
                    categoriesThatNeedToBeUpdated.setObjectCategoryIds = categoriesThatNeedToBeUpdated.setObjectCategoryIds.concat(this.extractOCIdsFrom(model.properties['setObjectCategoryIds']));
                    categoriesThatNeedToBeUpdated.addObjectCategoryIds = categoriesThatNeedToBeUpdated.addObjectCategoryIds.concat(this.extractOCIdsFrom(model.properties['addObjectCategoryIds']));
                    categoriesThatNeedToBeUpdated.removeObjectCategoryIds = categoriesThatNeedToBeUpdated.removeObjectCategoryIds.concat(this.extractOCIdsFrom(model.properties['removeObjectCategoryIds']));
                    categoriesThatNeedToBeUpdated.moveObjectCategoryIds = categoriesThatNeedToBeUpdated.moveObjectCategoryIds.concat(this.extractOCIdsFrom(model.properties['moveObjectCategoryIds']));
                    categoriesThatNeedToBeUpdated.zoomObjectCategoryIds = categoriesThatNeedToBeUpdated.zoomObjectCategoryIds.concat(this.extractOCIdsFrom(model.properties['zoomObjectCategoryIds']));
                    categoriesThatNeedToBeUpdated.toggleObjectCategoryIds = categoriesThatNeedToBeUpdated.zoomObjectCategoryIds.concat(this.extractOCIdsFrom(model.properties['toggleObjectCategoryIds']));
                }

            }
        }
        categoriesThatNeedToBeUpdated.setObjectCategoryIds = this.getCleanArrayFrom(categoriesThatNeedToBeUpdated.setObjectCategoryIds);
        categoriesThatNeedToBeUpdated.addObjectCategoryIds = this.getCleanArrayFrom(categoriesThatNeedToBeUpdated.addObjectCategoryIds);
        categoriesThatNeedToBeUpdated.removeObjectCategoryIds = this.getCleanArrayFrom(categoriesThatNeedToBeUpdated.removeObjectCategoryIds);
        categoriesThatNeedToBeUpdated.moveObjectCategoryIds = this.getCleanArrayFrom(categoriesThatNeedToBeUpdated.moveObjectCategoryIds);
        categoriesThatNeedToBeUpdated.zoomObjectCategoryIds = this.getCleanArrayFrom(categoriesThatNeedToBeUpdated.zoomObjectCategoryIds);
        categoriesThatNeedToBeUpdated.toggleObjectCategoryIds = this.getCleanArrayFrom(categoriesThatNeedToBeUpdated.toggleObjectCategoryIds);


        return categoriesThatNeedToBeUpdated;

    },
    getCleanArrayFrom: function (array) {
        var idx, result = [];
        for (idx = 0; idx < array.length; idx++) {
            if (array[idx] != null) {
                result.push(array[idx]);
            }
        }
        return result;
    },
    shouldBypassLink: function () {
        var linksFrom = this.model.get('links').from, idx;
        for (idx = 0; idx < linksFrom.length; idx++) {
            if (linksFrom[idx].type == 'activateOnce') {
                return true;
            }
        }
        return false;
    },
    startZoomAndPan: function () {
        // for now will only be implemented for image for now
    },
    restoreLinkEffectsIfNeeded: function () {
        this.restoreZoomToLinkIfNeeded();
    },
    restoreZoomToLinkIfNeeded: function () {
        var zoomedOut = this.model.get('zoomedOut');
        if ((null == zoomedOut || zoomedOut) && this.model.get("zoomOutStartingPoint")) {
            this.model.set("zoomedOut", false);
            this.model.set('zoomedIn', null);
            this.zoomOutStartingPoint();
        }
    },
    applyEffectIfNeeded: function () {
        if (this.currentObjectIsPartOfAnActiveCategory()) {
            this.model.set("activateEffect", true);
        }
    },
    applyGoToObjectIfNeeded: function () {
        if (this.currentObjectIsPartOfAnActiveCategory()) {
            var goToObjectType = this.getGoToObjectType();
            if (null != goToObjectType) {
                this[goToObjectType]();
            }
        }
    },
    notifyParentThatObjectIsActivated: function () {
        if (this.willTriggerOCChanges())
            this.parentElementModel.set("activatedObject", this.getActivatedObjectInformation());
    },
    notifyLinkedObjectsThatObjectIsActivated: function () {
        var pageModels = this.parentElementModel.get('workingObjectsCollection').models,
            index, linkIndex, model,
            links = this.model.get('links'),
            link,
            linksToBeProcessed = this.cloneOutgoingLinks(links.to);
        this.addIncomingMoveToLinksTo(linksToBeProcessed, links.from);

        for (index = 0; index < pageModels.length; index++) {
            model = pageModels[index];
            for (linkIndex = 0; linkIndex < linksToBeProcessed.length; linkIndex++) {
                link = linksToBeProcessed[linkIndex];
                if (model.get('id') == link['relatedObjectId']) {
                    model.set('linkedObjectActivatedLink', link);
                }
            }
        }
    },
    cloneOutgoingLinks: function (outgoingLinks) {
        var retLinks = [], index;
        for (index = 0; index < outgoingLinks.length; index++) {
            retLinks.push(outgoingLinks[index]);
        }
        return retLinks;
    },
    addIncomingMoveToLinksTo: function (linksToBeProcessed, incomingLinks) {
        var link, index;
        for (index = 0; index < incomingLinks.length; index++) {
            link = incomingLinks[index];
            if (link['type'] == 'moveTo') {
                link.properties['destinationOnXAxis'] = this.model.get('left');
                link.properties['destinationOnYAxis'] = this.model.get('top');
                linksToBeProcessed.push(link);
            }
        }
    },
    notifyParentThatObjectIsLoaded: function () {
        console.log("Notify loaded for object of type " + this.model.get("type"));
        this.parentElementModel.set("objectLoaded", this.getLoadedInfo(true));
    },
    getLoadedInfo: function (loaded) {
        return {
            type: this.model.get("type"),
            id: this.model.get("id"),
            loaded: loaded,
            objectCategoryIds: this.model.get('objectCategoryIds')
        };
    },
    willTriggerOCChanges: function () {
        return (!this.isEmpty(this.model.get('toggleObjectCategoryIds')) || !this.isEmpty(this.model.get('setObjectCategoryIds')) || !this.isEmpty(this.model.get('addObjectCategoryIds')) || !this.isEmpty(this.model.get('removeObjectCategoryIds')) || !this.isEmpty(this.model.get('zoomObjectCategoryIds')) || !this.isEmpty(this.model.get('moveObjectCategoryIds')) || !this.isEmpty(this.model.get('objectCategoryIds')));
    },
    getActivatedObjectInformation: function () {
        var ret = {}, globalCategoriesActions,
            updatedZoomPropertiesValues = {},
            category, categoryId;
        ret.activatedCategories = {};
        ret.activatedCategories.toggleObjectCategoryIds = this.model.get('toggleObjectCategoryIds');
        ret.activatedCategories.setObjectCategoryIds = this.model.get('setObjectCategoryIds');
        ret.activatedCategories.addObjectCategoryIds = this.model.get('addObjectCategoryIds');
        ret.activatedCategories.removeObjectCategoryIds = this.model.get('removeObjectCategoryIds');
        ret.activatedCategories.moveObjectCategoryIds = this.model.get('moveObjectCategoryIds');
        ret.activatedCategories.zoomObjectCategoryIds = this.model.get('zoomObjectCategoryIds');

        ret.activatedCategories.objectCategoryIds = this.model.get('objectCategoryIds');
        ret.moveByPropertiesValues = this.model.get("moveByPropertiesValues");

        updatedZoomPropertiesValues = this.getCategoriesZoomProperties();
        ret.zoomPropertiesValues = this.model.get("zoomPropertiesValues");
        for (categoryId in updatedZoomPropertiesValues) {
            if (updatedZoomPropertiesValues.hasOwnProperty(categoryId)) {
                category = updatedZoomPropertiesValues[categoryId];
                if (null != ret.zoomPropertiesValues[categoryId]) {
                    ret.zoomPropertiesValues[categoryId].left = category.left;
                    ret.zoomPropertiesValues[categoryId].top = category.top;
                    ret.zoomPropertiesValues[categoryId].width = category.right - category.left;
                    ret.zoomPropertiesValues[categoryId].height = category.bottom - category.top;
//                    ret.zoomPropertiesValues[categoryId].zoomPropertiesValues = 2;
                }

            }
        }

        ret.id = this.model.get("id");
        globalCategoriesActions = this.model.get('globalCategoryAction');
        ret.globalCategoryAction = (null == globalCategoriesActions) ? false : globalCategoriesActions;
        return ret;
    },
    getCategoriesZoomProperties: function () {
        var zoomedCategoriesIds = this.model.get('zoomObjectCategoryIds'),
            categoriesIndex,
            updatedCategories = {},
            workingModels = this.parentElementModel.get('workingObjectsCollection').models,
            model, modelsIndex, ocIds,
            left, top, bottom, right, category;
        if (null != zoomedCategoriesIds) {
            for (categoriesIndex = 0; categoriesIndex < zoomedCategoriesIds.length; categoriesIndex++) {
                category = {};
                category.left = Number.MAX_VALUE;
                category.top = Number.MAX_VALUE;
                category.right = Number.MIN_VALUE;
                category.bottom = Number.MIN_VALUE;
                updatedCategories[zoomedCategoriesIds[categoriesIndex]] = category;
            }
            for (modelsIndex = 0; modelsIndex < workingModels.length; modelsIndex++) {
                model = workingModels[modelsIndex];
                ocIds = model.get('objectCategoryIds');
                if (null != ocIds) {
                    for (categoriesIndex = 0; categoriesIndex < ocIds.length; categoriesIndex++) {
                        if (null != updatedCategories[ocIds]) {
                            category = updatedCategories[ocIds];
                            left = this.getAbsoluteValue(model.get('left'), App.settings.width);
                            top = this.getAbsoluteValue(model.get('top'), App.settings.height);
                            right = this.getAbsoluteValue(model.get('width'), App.settings.width) + left;
                            bottom = this.getAbsoluteValue(model.get('height'), App.settings.height) + top;
                            if (left < category.left) {
                                category.left = left;
                            }
                            if (top < category.top) {
                                category.top = top;
                            }
                            if (right > category.right) {
                                category.right = right;
                            }
                            if (bottom > category.bottom) {
                                category.bottom = bottom;
                            }
                        }

                    }
                }

            }
            return updatedCategories;
        } else {
            return null;
        }
    },
    getAbsoluteValue: function (percentageValue, wrapperValue) {
        return Math.round(parseFloat(percentageValue) * wrapperValue / 100);
    },
    processLink: function () {
        var link = this.model.get('linkedObjectActivatedLink');
        if (null != link) {
            if (!this.model.get('shouldBypassLink')) {
                this.model.set('linkedObjectActivatedLink', null);
                this[link['type']](link['properties']);
                this.model.set('shouldBypassLink', this.shouldBypassLink());
            }
        }
    },
    removeView: function () {
        this.remove();
        this.unbind();
    },
    activate: function (linkProperties) {
        this.model.set("activateObject", true);
        this.model.set("activateObject", false);
    },
    activateOnce: function (activatedLinkedObject) {
        this.model.set("activateObject", true);
    },
    activateToggle: function (activatedLinkedObject) {
        //do nothing. Will be implemented for each element if needed
    },
    pulsate: function (activatedLinkedObject) {
        var thisView = this;
        LinksHandler.pulsate(thisView.model, thisView.$el, activatedLinkedObject);
    },
    show: function (activatedLinkedObject) {
        var thisView = this;
        LinksHandler.show(thisView.model, thisView.$el);
    },
    hide: function (activatedLinkedObject) {
        var thisView = this;
        LinksHandler.hide(thisView.model, thisView.$el);
    },
    toggleVisibility: function (activatedLinkedObject) {
        console.log("toggleVisibility link called");
        var thisView = this;
        EffectsHandler.toggleVisibility(thisView.$el, thisView.model);
    },
    fadeIn: function (activatedLinkedObject) {
        var thisView = this;
        LinksHandler.fadeIn(thisView.model, thisView.$el);
    },
    fadeOut: function (activatedLinkedObject) {
        var thisView = this;
        LinksHandler.fadeOut(thisView.model, thisView.$el);
    },
    zoomTo: function (linkProperties) {
        console.log("ZoomTo link was activated for " + this.model.get("id"));
        this.model.set('zoomToStarted', true);
        this.zoomInStartingPoint(linkProperties);
        this.model.set("activateEffect", false);
    },
    moveTo: function (linkProperties) {
        var thisView = this;
        if (!this.model.get("userTookControl")) {
            LinksHandler.moveTo(thisView.model, thisView.$el, linkProperties);
        } else {
            console.log("GYL-156: moveTo aborted due to user scroll control");
        }
    },
    printElement: function (activatedLinkedObject) {
        var thisView = this;
        LinksHandler.printElement(thisView.model, thisView.$el);
    },

    processObjectModelBeforeInit: function () {
        var moveByPropertiesValues = this.model.get("moveByPropertiesValues");
        if (typeof moveByPropertiesValues == "string" || typeof moveByPropertiesValues == "undefined") {
            this.model.set("moveByPropertiesValues", {});
        }
    },

    initialize: function (attr) {
        if (this.model.get("links") == null) {
            this.model.set("links", {
                "from": [],
                "to": []
            });
        }
        this.processObjectModelBeforeInit();
        this.updateOCForModel();
        this.addListeners();
        this.customInitialize();
        this.notifyParentThatObjectIsLoaded();
        this.unloadHandlers = [];

        if ((this.getActiveEffect() != undefined && this.getActiveEffect() != null && this.getActiveEffect() != "") || this.hasOtherAction())
            this.$el[0].style.cursor = 'pointer';
    },

    customInitialize: function () {
        // override in page objects as needed, keep here empty body
    },

    addListeners: function () {
        this.stopListening();
        this.listenTo(this.model, "change:objectModelUpdate", this.processAttributeUpdate);
        this.listenTo(this.model, "change:removed", this.removeView);
        this.listenTo(this.model, "change:activateEffect", this.activateEffect);
        this.listenTo(this.model, "change:activateObject", this.activateObject);
        this.listenTo(this.model, "change:objectActivated", this.objectActivated);
        this.listenTo(this.model, "change:restoreState", this.restoreState);
        this.listenTo(this.model, "change:loadObject", this.loadObject);
        this.listenTo(this.model, "change:movedCategory", this.moveObjectIfNeeded);
        this.listenTo(this.model, "change:zoomedCategory", this.zoomObjectForOC);
        this.listenTo(this.model, "change:moveByProjectOffset", this.moveObjectsWithoutCategoriesIfNeeded);
        this.listenTo(this.model, "change:startAutosStartEffects", this.startAutosStartEffects);
        this.listenTo(this.model, "change:linkedObjectActivatedLink", this.processLink);
        this.addCustomListeners();
        if (typeof this.events == "function") {
            this.delegateEvents(this.events());
        } else {
            this.delegateEvents(this.events);
        }
    },

    actions: {
        "updateForScrollableWithinPage": "renderObjectInGPU",
        "updateForHardwareAcceleration": "renderObjectInGPU"
    },

    renderObjectInGPU: function (keyValue) {
        if (AppUtils.DeviceType.ANDROID && keyValue) {
            this.$el.css("-webkit-backface-visibility", "hidden");
        }
    },

    processAttributeUpdate: function () {
        var newValues = this.model.get("objectModelUpdate"),
            updateKey = newValues["key"],
            keyValue = newValues["value"],
            handlerFunction;
        this.model.set(updateKey, keyValue);  // we also set value in object model, for easy access later
        handlerFunction = this[this.actions[updateKey]];
        if (typeof handlerFunction == "function") {
            handlerFunction.call(this, keyValue);
        }
    },

    startAutosStartEffects: function () {
        if (null != this.model.get('startAutosStartEffects')) {
            this.model.set('startAutosStartEffects', null);
            if (this.shouldRenderObject()) {
                this.startZoomEffectsIfNeeded();
            }
        }
    },
    addCustomListeners: function () {
        // leave it empty here but implement in each object as needed
    },

    loadObject: function () {
        console.log('LOADED OBJECT: ' + this.model.get('id'));
    },

    doBeforeViewRemove: function () {
    },

// only push once when object is created
    pushUnloadHandler: function (handler) {
        if (!Array.isArray(this.unloadHandlers)) {
            this.unloadHandlers = [];
        }
        this.unloadHandlers.push(handler);
    },

// executed when navigating to a neighbour page
    runUnloadHandlers: function () {
        if (Array.isArray(this.unloadHandlers)) {
            for (var idx = 0; idx < this.unloadHandlers.length; idx++) {
                this.unloadHandlers[idx]();
            }
        }
    },

    restoreState: function () {
        if (this.model.get("restoreState")) {
            this.model.set("restoreState", false);
            CSSHandler.removeTransitionEndHandler(this.$el, null);
            this.model.set("shouldNotRun", true);
            this.restoreObjectCSSProperties();
            this.restoreImageScrollableWithinPage();
            this.restoreDisplayValue();
            if (this.model.get("zoomOutStartingPoint")) {
                this.model.set('zoomedOut', null);
                this.zoomOutStartingPoint();
            }
            if (this.model.get("zoomToInAction")) {
                this.model.set("zoomToInAction", false);
                this.model.set('zoomedIn', false);
                this.parentElementModel.set("resetZoom", true);
            }
            if (this.objectInNeighbourPageToCurrent()) {
                this.runUnloadHandlers();
            }
        }
        this.customStateRestore();
    },

    objectInNeighbourPageToCurrent: function () {
        return Math.abs(this.parentElementModel.get("index") - Chapter.getActivePage()) in App.getBufferPages();
    },

    restoreDisplayValue: function () {
        // objects that have been hidden due to effects or links should return to their state
        var displayValue = this.model.get('displayed') ? "visible" : "hidden";
        this.$el.css("visibility", displayValue);
    },

    customStateRestore: function () {
        // this should be leaved empty here
    },

    updateOCForModel: function () {
        this.model.set('objectCategoryIds', this.extractOCIdsFrom(this.model.get('objectCategoryIds')));
        this.model.set('toggleObjectCategoryIds', this.extractOCIdsFrom(this.model.get('toggleObjectCategoryIds')));
        this.model.set('setObjectCategoryIds', this.extractOCIdsFrom(this.model.get('setObjectCategoryIds')));
        this.model.set('addObjectCategoryIds', this.extractOCIdsFrom(this.model.get('addObjectCategoryIds')));
        this.model.set('removeObjectCategoryIds', this.extractOCIdsFrom(this.model.get('removeObjectCategoryIds')));
        this.model.set('moveObjectCategoryIds', this.extractOCIdsFrom(this.model.get('moveObjectCategoryIds')));
        this.model.set('zoomObjectCategoryIds', this.extractOCIdsFrom(this.model.get('zoomObjectCategoryIds')));
        this.model.set('relatedObjectCategoryId', this.extractOCIdsFrom(this.model.get('relatedObjectCategoryId')));

    },

    extractOCIdsFrom: function (ocIdsAsCommaSeparatedString) {
        var ret = [], idx;
        if (null != ocIdsAsCommaSeparatedString) {
            ret = ocIdsAsCommaSeparatedString.split(',');
            for (idx = 0; idx < ret.length; idx++) {
                if (ret[idx] == '')
                    ret.splice(idx, 1);
            }
        }
        return ret.length == 0 ? null : ret;
    },

    objectAffectedByChromeBugs: function () {
        var allObjectTypesInPage = this.model.get("allObjectTypesInPage");
        return (AppUtils.DeviceType.CHROME && allObjectTypesInPage &&
            (allObjectTypesInPage.indexOf("video-yt-inline") > -1
                || allObjectTypesInPage.indexOf("video-inline") > -1
                || allObjectTypesInPage.indexOf("quizTest-inline") > -1
                || allObjectTypesInPage.indexOf("quizQuestion-inline") > -1
                || allObjectTypesInPage.indexOf("animation") > -1
                || allObjectTypesInPage.indexOf("drawing") > -1))
            || this.shouldAdjustForMovingLayers();
    },

    restoreObjectCSSProperties: function () {
        this.setInitialVisibility();
        CSSHandler.removeBrowserSpecificProperty(this.$el, 'transform');
        CSSHandler.removeBrowserSpecificProperty(this.$el, 'transition');
        CSSHandler.removeBrowserSpecificProperty(this.$el, 'transitionTimingFunction');
        if (!this.model.get("updateForScrollableWithinPage")
            && !this.model.get("updateForHardwareAcceleration")
            && !this.objectAffectedByChromeBugs()
            && !this.isGPUAccelerated()) {
            CSSHandler.removeBrowserSpecificProperty(this.$el, 'backFaceVisibility');
        }
//        CSSHandler.removeBrowserSpecificProperty(this.$el, 'perspective');
    },

    restoreImageScrollableWithinPage: function () {
        var scrollWrapper, thisView = this;
        if (this.model.get("scrollableWithinPage")) {
            scrollWrapper = this.$el.parent()[0];
            if (scrollWrapper) {
                var scrollLeftValue = Math.round(-parseInt(this.model.get("left")) * App.settings.width / 100),
                    scrollTopValue = Math.round(-parseInt(this.model.get("top")) * App.settings.height / 100);
                console.log("Now restore scroll values");
                scrollWrapper.scrollLeft = scrollLeftValue;
                scrollWrapper.scrollTop = scrollTopValue;
            }
            setTimeout(function () {
                thisView.model.set("userTookControl", false);
            }, 100);
        }
    },

    getZoomTimingFunction: function (zoomProperties) {
        if (null != zoomProperties) {
            return zoomProperties['zoomToTimingFunction'];
        } else {
            var useCustomZoomStartingPointTransitionTiming = this.model.get("useCustomZoomStartingPointTransitionTiming"),
                zoomStartingPointTimingFunction = this.model.get("zoomStartingPointTimingFunction"),
                zoomStartingPointTimingCustomFunction = this.model.get("zoomStartingPointTimingCustomFunction");

            zoomStartingPointTimingFunction = (null == zoomStartingPointTimingFunction) ? '0, 0, 1, 1' : zoomStartingPointTimingFunction;
            zoomStartingPointTimingCustomFunction = (null == zoomStartingPointTimingCustomFunction) ? '0, 0, 0.5, .5' : zoomStartingPointTimingCustomFunction;

            return (null == useCustomZoomStartingPointTransitionTiming || !useCustomZoomStartingPointTransitionTiming) ? zoomStartingPointTimingFunction : zoomStartingPointTimingCustomFunction;
        }
    },

    updateZoomHandler: function () {
        if (!this.preventZoomChanges) {
            console.log("updateZoomHandler");
            var zoomFactor = this.model.get("zoomFactor");
            if (this.model.get("zoomGestureShouldStartFromBeginning")) {
                this.model.set("zoomGestureShouldStartFromBeginning", false);
                var transformMatrix = window.getComputedStyle(this.el).getPropertyValue(CSSHandler.getCurrentBrowserPrefix().transform);
                if (transformMatrix != "none") {
                    var transformMatrixValues = transformMatrix.match(/-?[0-9\.]+/g);
                    this.lastZoomValueForAutomaticZoom = parseFloat(transformMatrixValues[0]);
                    CSSHandler.removeBrowserSpecificProperty(this.$el, "transition");
                }
            }
            var scaleFactor = Math.max(this.lastZoomValueForAutomaticZoom * zoomFactor, 1);
            CSSHandler.scale(this.$el, scaleFactor);
        }
    },

    zoomObjectForOC: function () {
        var zoomedCategories = this.model.get('zoomedCategory'),
            idx, objectCategories = this.model.get('objectCategoryIds'),
            zoomPortInfo, thisView = this;
        this.model.set('zoomedCategory', null);
        var doZoom = function ($el, zoomInfo, zoomedCategory) {
            var zoomTimer = CommonActions.zoomInOCObject($el, zoomInfo, zoomedCategory),
                zoomDelay = Math.max(zoomedCategory["zoomStartingPointDelayBeforeZooming"], 100),
                zoomStartDelayTimer = setTimeout(function () {
                    console.log("START listen to change:zoomFactor");
                    thisView.preventZoomChanges = false;
                    thisView.listenTo(thisView.model, "change:zoomFactor", thisView.updateZoomHandler);
                }, zoomDelay);
            thisView.pushUnloadHandler(function () {
                thisView.lastZoomValueForAutomaticZoom = null;
                clearTimeout(zoomTimer);
                clearTimeout(zoomStartDelayTimer);
                thisView.preventZoomChanges = true;
                // TODO: see how we can unbind from "change:zoomFactor", none bellow worked !!!
//                console.log("STOP listen to change:zoomFactor");
//                thisView.stopListening("change:zoomFactor");
//                thisView.off("change:zoomFactor", thisView.updateZoomHandler);
//                thisView.off('change', function () {
//                    console.log("STOP listen to change:zoomFactor");
//                });
                CSSHandler.removeBrowserSpecificProperty(thisView.$el, 'transform');
                CSSHandler.removeBrowserSpecificProperty(thisView.$el, 'transition');
                CSSHandler.removeBrowserSpecificProperty(thisView.$el, 'transformOrigin');
            });
        };
        if (null != objectCategories && null != zoomedCategories) {
            for (idx = 0; idx < objectCategories.length; idx++) {
                if (zoomedCategories.hasOwnProperty(objectCategories[idx])) {
                    var zoomedCategory = zoomedCategories[objectCategories[idx]];
                    if (zoomedCategory["activateZoom"] == "true") {
                        zoomPortInfo = {
                            left: zoomedCategory.left,
                            top: zoomedCategory.top,
                            width: zoomedCategory.width,
                            height: zoomedCategory.height,
                            zoomFactor: zoomedCategory.zoomFactor,
                            objectLeft: parseInt(this.$el.css("left")),
                            objectTop: parseInt(this.$el.css("top"))
                        };
                        doZoom(this.$el, zoomPortInfo, zoomedCategory)
                    }
                }
            }
        }
    },

    moveObjectIfNeeded: function () {
        var movedCategories = this.model.get('movedCategory'),
            idx, objectCategories = this.model.get('objectCategoryIds');
        this.model.set('movedCategory', null);
        if (null != objectCategories && null != movedCategories) {
            for (idx = 0; idx < objectCategories.length; idx++) {
                if (movedCategories.hasOwnProperty(objectCategories[idx])) {
                    LinksHandler.moveBy(this.model, this.$el, movedCategories[objectCategories[idx]]);
                }
            }
        }
    },
    moveObjectsWithoutCategoriesIfNeeded: function () {
        var moveByProperties = this.model.get('moveByProjectOffset'),
            moveByXPixels, moveByYPixels;

        if (null != moveByProperties) {
            this.model.set('moveByProjectOffset', null);
            moveByXPixels = parseFloat(moveByProperties.moveByXPixels);
            moveByYPixels = parseFloat(moveByProperties.moveByYPixels);
            if (moveByXPixels || moveByYPixels) {
                LinksHandler.moveBy(this.model, this.$el, moveByProperties);
            }
        }
    },


    categoryActivated: function () {
        //do nothing. Will be implemented where needed
    },
    shouldRenderObject: function () {
        return !this.isMoveToLinkTarget() && this.currentObjectIsPartOfAnActiveCategory();
    },

    bypassChromeBugs: function () {
        if (this.objectAffectedByChromeBugs()) {
            CSSHandler.setBackfaceVisibility(this.$el, "hidden");
        }
    },

    isEmpty: function (obj) {

        // null and undefined are "empty"
        if (obj == null) return true;

        // Assume if it has a length property with a non-zero value
        // that that property is correct.
        if (obj.length > 0)    return false;
        if (obj.length === 0)  return true;

        // Otherwise, does it have any properties of its own?
        // Note that this doesn't handle
        // toString and toValue enumeration bugs in IE < 9
        for (var key in obj) {
            if (obj.hasOwnProperty(key)) return false;
        }

        return true;
    }
})
;

ElementBaseView.extend = BaseView.extend;