define(['GenericLightBox', 'PageElement'], function (GenericLightBox, PageElement) {
    return function (rawEmptyBoxModel, pageModel) {
        var that = this;

        this.defineClasses = function () {

            that.EmptyBoxView = ElementBaseView.extend({
                tagName: "div",
                className: "sceneObject",
                elementsViews: [],
                id: rawEmptyBoxModel.properties.id,
                parentElementModel: pageModel,
                sourceLinksObjectsIds: {},
                template: null,
                embeddedPage: null,
                lightBox: null,
                addCustomListeners: function () {
                    this.listenTo(this.model, "change:toggledCategories", this.toggledCategories);
                    this.listenTo(this.model, "change:closeLightBox", this.closeLightBox);
                    this.listenTo(this.model, "change:downloadComplete", this.downloadComplete);
                },
                downloadComplete: function () {
                    var downloadComplete = this.model.get('downloadComplete');
                    if (null != downloadComplete) {
                        this.model.set('downloadComplete', null);
                        this.model.set('closeLightBox', true);
                    }
                },

                events: function () {
                    var _events = {};
                    if (this.model.get("isIconScene")) {
                        _events[AppUtils.EventTypes.mouseDown] = "touchstart";
                        _events[AppUtils.EventTypes.click] = "clickHandler";
                        _events[AppUtils.EventTypes.mouseUp] = "mouseUpHandler";
                    }
                    return _events;
                },

                mouseUpHandler: function () {
                    console.log("mouseUpHandler on " + this.$el.attr("id"));
                },
                removeView: function () {
                    if (this.model.get("removed")) {
                        this.remove();
                        this.unbind();
                    }
                },
                activateObject: function () {
                    if (this.model.get("activateObject")) {
                        this.model.set("objectActivated", true);
                        this.model.set("activateObject", false);
                    }
                },
                closeLightBox: function () {
                    if (this.model.get('closeLightBox')) {
                        this.model.set('closeLightBox', false);
                        if (null != this.lightBox) {
                            this.lightBox.getModel().set('remove', true);
                        }
                    }
                },
                objectActivated: function () {
                    if (this.model.get('objectActivated')) {
                        this.getSceneContent();
                        var thisView = this,
                            lightBoxWindowSettings = {
                                width: thisView.model.get('absoluteWidth'),
                                height: thisView.model.get('absoluteHeight'),
                                color: '#FFFFFF',
                                closeButton: 'off'
                            };
                        this.embeddedPage.init(function () {
                            thisView.lightBox = new GenericLightBox(null, lightBoxWindowSettings, {content: thisView.embeddedPage.getView().render().el, color: '#FFFFFF'});
                            thisView.lightBox.init();

                            App.viewPortView.$el.append(thisView.lightBox.getView().render().el);
                            thisView.embeddedPage.getModel().set('shouldBeActive', true);
                        });


                        this.model.set('objectActivated', false);
                    }
                },
                activateToggle: function () {
                    this.displayHideOnceOnClick();
                },

                setupIconScene: function () {
                    this.updateModelAbsoluteSize(App.settings.width, App.settings.height);
                    var embeddedSceneIconInfo = Chapter.getSettings()["embeddedSceneIconInfo"];
                    this.model.set("imageURL", embeddedSceneIconInfo['embeddedSceneIconURL']);
                    this.template = templates.getTemplate("embeddedScene");
                    this.$el.html(this.template(this.model.attributes));
                    this.setWidth(embeddedSceneIconInfo['embeddedSceneIconWidth']);
                    this.setHeight(embeddedSceneIconInfo['embeddedSceneIconHeight']);
                },
                setupInlineScene: function () {
                    var thisView = this;
                    this.setWidth(this.model.get("width"));
                    this.setHeight(this.model.get("height"));
                    this.updateModelAbsoluteSize(this.model.get("width"), this.model.get("height"));
                    this.getSceneContent();
                    this.embeddedPage.init(function () {
                        thisView.$el.append(thisView.embeddedPage.getView().render().el);
                        thisView.embeddedPage.getModel().set('shouldBeActive', true);
                    });
                },
                getSceneContent: function () {
                    if (null == this.embeddedPage) {
                        var embeddedSceneRawModel = Chapter.getEmbeddedScenes()[this.model.get('id')],
                            model = this.createPageModel(embeddedSceneRawModel);
                        model.set('parentModel', this.model);
                        model.set('absoluteWidth', this.model.get('absoluteWidth'));
                        model.set('absoluteHeight', this.model.get('absoluteHeight'));
                        this.embeddedPage = new PageElement(model);
                    }
                },

                createPageModel: function (page) {
                    var settings = page.settings,
                        pageWidth = Math.round(settings.resolutionWidth * App.settings.scaleFactor);

                    return new Backbone.Model({
                        height: Math.round(settings.resolutionHeight * App.settings.scaleFactor),
                        width: pageWidth,
                        top: 0,
                        left: 0,
                        index: 1,
                        absoluteWidth: 300,
                        absoluteHeight: 300,
                        removed: false,
                        hideLeftSideMenuArrow: true,
                        hideRightSideMenuArrow: true,
                        objects: page.objects,
                        id: settings['objectId']
                    });
                },
                toggledCategories: function () {
                    var toggledCategories = this.model.get('toggledCategories');
                    if (null != toggledCategories) {
                        this.model.set('toggledCategories', null);
                        if (null != this.embeddedPage) {
                            this.embeddedPage.getModel().set('toggledCategories', toggledCategories);
                        }
                    }
                },

                updateModelAbsoluteSize: function (containerWidth, containerHeight) {
                    var originalWidth = parseFloat(this.model.get('originalWidth')),
                        originalHeight = parseFloat(this.model.get('originalHeight')),
                        hScaleFactor = 100 / originalHeight,
                        vScaleFactor = 100 / originalWidth ,
                        scaleFactor = Math.min(hScaleFactor, vScaleFactor),
                        absoluteWidth, absoluteHeight;

                    if (scaleFactor < 1) {
                        absoluteWidth = Math.round(originalWidth * scaleFactor * containerWidth / 100);
                        absoluteHeight = Math.round(originalHeight * scaleFactor * containerHeight / 100);
                    } else {
                        absoluteWidth = Math.round(originalWidth * containerWidth / 100);
                        absoluteHeight = Math.round(originalHeight * containerHeight / 100);
                    }
                    this.model.set('absoluteWidth', absoluteWidth);
                    this.model.set('absoluteHeight', absoluteHeight);
                },
                render: function () {
                    if (this.shouldRenderObject()) {
                        this.setLeft(this.model.get("left"));
                        this.setTop(this.model.get("top"));

                        if (this.model.get("isIconScene")) {
                            this.setupIconScene();
                        } else {
                            this.setupInlineScene();
                        }

                        this.$el.css("z-index", this.model.get("ZIndex"));
                        this.setInitialVisibility();
                        this.startZoomEffectsIfNeeded();
                        return this;
                    }
                    return null;
                }
            });
        };
        this.instantiateObjects = function () {
            this.emptyBoxModel = this.getEmptyBoxModel();
            this.emptyBoxView = new this.EmptyBoxView({
                model: that.emptyBoxModel
            });

        };

        this.getEmptyBoxModel = function () {
            return  new Backbone.Model(rawEmptyBoxModel.properties);

        };


        return {
            init: function () {
                that.defineClasses();
                that.instantiateObjects();
            },
            getView: function () {
                return that.emptyBoxView;
            },
            getModel: function () {
                return that.emptyBoxModel;
            }

        }
    }
});