define(function () {
    return function (rawEventModel, pageModel) {
        var that = this;

        this.defineClasses = function () {

            that.EventView = ElementBaseView.extend({
                tagName: "div",
                className: "sceneObject",
                elementsViews: [],
                id: rawEventModel.properties.id,
                parentElementModel: pageModel,
                executedIterations: 0,

                removeView: function () {
                    if (this.model.get("removed")) {
                        if (this.parentElementModel.get('removed')) {
                            this.customStateRestore();
                            this.remove();
                            this.unbind();
                        }
                        this.model.set("activateObject", false);
                    }
                },
                addCustomListeners: function () {
                    this.listenTo(this.model, "change:downloadComplete", this.downloadComplete);
                    this.listenTo(this.model, "change:externalObjectActivated", this.externalObjectActivated);
                },
                downloadComplete: function () {
                    var downloadComplete = this.model.get('downloadComplete'), relatedCategoryId;
                    if (null != downloadComplete) {
                        relatedCategoryId = this.model.get('relatedCategoryId');
                        if (downloadComplete.indexOf(relatedCategoryId) != -1) {
                            this.model.set('objectActivated', true);
                            this.model.set('downloadComplete', null);
                        }
                    }
                },
                externalObjectActivated: function () {
                    var externalObjectActivated = this.model.get("externalObjectActivated");
                    if (null != externalObjectActivated) {

                        if (externalObjectActivated.activatedCategories.objectCategoryIds && externalObjectActivated.activatedCategories.objectCategoryIds.length > 0) {
                            var index;

                            for (index = 0; index < externalObjectActivated.activatedCategories.objectCategoryIds.length; ++index) {
                                var category = this.parentElementModel.get('activatedObjectsInCategoriesForCurrentPage')[externalObjectActivated.activatedCategories.objectCategoryIds[index]];

                                // if this event object should be activated only on download complete it won't be activated from here. It will be activated from downloadComplete function
                                if (this.model.get("eventType") == "ON_DOWNLOAD_COMPLETE")
                                    return;

                                if (this.model.get("eventType") == "ON_ACTIVATION_OF_FIRST_OBJECT" && category["numberOfActivatedObjects"] != null && category["numberOfActivatedObjects"] > 0)
                                    return;

                                // there are no restrictions to be added for ON_ACTIVATION_OF_ANY_OBJECT

                                if (this.model.get("eventType") == "ON_ACTIVATION_OF_LAST_OBJECT") {
                                    //If this is still null and there are several objects in the category, the first object is being activated. If totalNumberOfObjects = 1 then the first object is also the last object, so it's fine to go on an notify the parents.
                                    if (category["numberOfActivatedObjects"] == null && category["totalNumberOfObjects"] > 1)
                                        return;

                                    //If the difference is larger than 1 then there are several objects left to be activated. If the difference is 0, all the objects have already been activated.
                                    if (category["numberOfActivatedObjects"] != null && category["totalNumberOfObjects"] - category["numberOfActivatedObjects"] != 1)
                                        return;

                                    //If this is already true, then the current object has already been activated, it's probably been clicked again
                                    if (category[externalObjectActivated.id] == true)
                                        return;
                                }

                                if (this.model.get("relatedCategoryId") && this.model.get("relatedCategoryId") == externalObjectActivated.activatedCategories.objectCategoryIds[index])
                                    this.notifyParentThatObjectIsActivated();
                            }
                        }

                        this.processLink(externalObjectActivated);
                    }
                },

                shouldRenderObject: function () {
                    return this.currentObjectIsPartOfAnActiveCategory();
                },
                activateObject: function () {
                    if (this.model.get("activateObject")) {
                        this.model.set("activateObject", false);
                        console.log("EVENT DEACTIVATED: " + this.model.get("activateObject"));
                    }
                }

            });
        };
        this.instantiateObjects = function () {
            this.eventModel = this.getEventModel();
            this.eventView = new this.EventView({
                model: that.eventModel
            });
        };

        this.getEventModel = function () {
            return  new Backbone.Model(rawEventModel.properties);
        };


        return {
            init: function () {
                that.defineClasses();
                that.instantiateObjects();
            },
            getView: function () {
                return that.eventView;
            },
            getModel: function () {
                return that.eventModel;
            }

        }
    }
});