define(function () {
    return function (rawImageModel, pageModel) {
        var that = this,
            posX = 0, posY = 0,
            lastPosX = 0, lastPosY = 0,
            scale = 1, lastScale = 1,
            maxX = 0, maxY = 0, minX = 0, minY = 0,
            visibleImageWidth, visibleImageHeight,
            imageWidthPercent, imageHeightPercent,
            imageLeft, imageTop,
            scrolledToLeftBorder = false,
            scrolledToRightBorder = false,
            transitionToRightStared = false,
            transitionToLeftStared = false,
            preventPageChange = true,
            userIsScrolling = false,
            userIsZooming = false,
            scheduledMoveEffect = null,
            scheduledZoomEffect = null,
            TAG = "GYL-414";

        this.defineClasses = function () {

            that.ImageView = ElementBaseView.extend({
                tagName: "div",
                className: "sceneObject",
                elementsViews: [],
                template: rawImageModel.properties.fadeGradient == "1" ? templates.getTemplate("imageWithGradient") : templates.getTemplate("image"),
                gifTemplate: templates.getTemplate("gifImage"), // does not support gradient
                id: rawImageModel.properties.id,
                parentElementModel: pageModel,
                $imageEl: null,
                transitionPlayer: null,

                actions: {
                    "toggleObjectStatePause": "pauseImageAnimations",
                    "toggleObjectStateResume": "resumeImageAnimations"
                },

                getTransitionPlayer: function () {
                    if (!this.transitionPlayer) {
                        this.transitionPlayer = new TransitionPlayer(this.model, this.$el, TransitionPlayerType.MOVE);
                    }
                    return this.transitionPlayer;
                },

                getTemplate: function (attributes) {
                    return (rawImageModel.customProperties.extension == "gif") ? this.gifTemplate(attributes) : this.template(attributes);
                },

                pauseImageAnimations: function () {
                    console.log(TAG, "Now pause image with ID " + this.model.get("id") + " in page " + pageModel.get("index"));
                    // TODO: GYL-414, due to bugs in iOS 8 this feature can't be enabled at this point. The problem was solved in iOS 8.2
                    //this.getTransitionPlayer().handleCSSTransformPause();
                },

                resumeImageAnimations: function () {
                    console.log(TAG, "Now resume image with ID " + this.model.get("id") + " in page " + pageModel.get("index"));
                    // TODO: GYL-414, due to bugs in iOS 8 this feature can't be enabled at this point. The problem was solved in iOS 8.2
                    //this.getTransitionPlayer().handleCSSTransformResume();
                },

                customInitialize: function () {
                    this.listenTo(this.model, "change:pageDeactivated", this.pageDeactivatedHandler);
                    this.model.set("allObjectTypesInPage", rawImageModel.customProperties["allObjectTypesInPage"]);
                },

                events: function () {
                    var _events = {};
                    if ("zoomAndPanSettings" in rawImageModel.properties) {
                        if (!(rawImageModel.properties.zoomAndPanSettings.manualControlSettings.scrolling || rawImageModel.properties.zoomAndPanSettings.manualControlSettings.pinchZoom)) {
                            // can't enable something else on click since it used for zoom and pan
                            _events[AppUtils.EventTypes.click] = "clickHandler";
                        }
                    } else {
                        _events[AppUtils.EventTypes.click] = "clickHandler";
                    }
                    return _events;
                },

                customTouchStartHandler: function (e) {
                    if (this.model.get("print") == true)
                        window.print();
                },

                customClickHandler: function (e) {
                    if (this.model.get("print") == true)
                        window.print();
                },


                addCustomListeners: function () {
                    this.listenTo(this.model, "change:orientation", this.updateOrientation);
                },

                updateOrientation: function () {
                    this.updateSizeAndPosition();
                },

                removeView: function () {
                    if (this.model.get("removed")) {
                        if (null != this.$imageEl) {
                            this.$imageEl.unbind('load error');
                            this.$imageEl = null;
                        }
                        this.unbind();
                        this.remove();
                        this.notifyParentThatObjectIsLoaded();
                    }
                },
                activateObject: function () {
                    if (this.model.get("activateObject")) {
                        this.model.set("objectActivated", true);
                        this.model.set("activateObject", false);
                    }
                },
                activateToggle: function () {
                    this.displayHideOnceOnClick();
                },
                shouldRenderObject: function () {
                    return this.currentObjectIsPartOfAnActiveCategory();
                },
                notifyParentThatObjectIsLoaded: function () {
                    this.parentElementModel.set("objectLoaded", this.getLoadedInfo(false));
                },
                bindImageSpecificEvents: function () {
                    var thisView = this;
                    this.$imageEl.bind("load error", function () {
                        console.log(LOG_TAGS.SK_3, "Image loaded " + thisView.model.get("imageURL") + " for page " + thisView.parentElementModel.get("index"));
                        visibleImageWidth = thisView.$el.width();
                        visibleImageHeight = thisView.$el.height();
                        thisView.updateMaxXYForScaleFactor(1);
                        thisView.parentElementModel.set("objectLoaded", thisView.getLoadedInfo(true));
                    });
                },
                loadObject: function () {
                    if (this.model.get('loadObject')) {
                        if (null == this.$imageEl) {
                            this.$imageEl = $(this.getTemplate(this.model.attributes));
                            this.bindImageSpecificEvents();
                        }
                    }
                    console.log('LOADED OBJECT: ' + this.model.get('id'));
                },
                setupScrollOnAndroid: function () {
                    var scrollableImageWrapper,
                        transformMatrix, transformMatrixValues, currentOffsetX, currentOffsetY,
                        thisView = this,
                        imageOverflowWidth = thisView.$el.width() - App.settings.width,
                        previousMoveValue = null,
                        scrollLeftValue = Math.round(-parseInt(this.model.get("left")) * App.settings.width / 100),
                        scrollTopValue = Math.round(-parseInt(this.model.get("top")) * App.settings.height / 100);

                    console.log("GYL-156: imageOverflowWidth " + imageOverflowWidth);
                    scrollableImageWrapper = $("<div>").addClass("scrollableWithinPage");
                    scrollableImageWrapper.css("z-index", this.model.get("ZIndex"));
                    thisView.$el.wrap(scrollableImageWrapper);
                    scrollableImageWrapper = thisView.$el.parent();
                    thisView.$el.css("position", "relative");
                    thisView.model.set("userTookControl", false);

                    setTimeout(function () {
                        console.log("Now set scroll property");
                        scrollableImageWrapper.scrollLeft(scrollLeftValue);
                        scrollableImageWrapper.scrollTop(scrollTopValue);
                    }, 0);

                    scrollableImageWrapper.bind(AppUtils.EventTypes.mouseDown, function (e) {
                        console.log("GYL-156:scrollableImageWrapper:mouseDown: event");
                        previousMoveValue = thisView.getPosition(e).x;
                    });

                    setTimeout(function () {
                        scrollableImageWrapper.bind(AppUtils.EventTypes.scroll, function (e) {
                            // TODO: see how we can manage up and down scroll
                            var currentMoveValue = thisView.getPosition(e).x,
                                moveDirection;
                            if (currentMoveValue - previousMoveValue > 0) {
                                moveDirection = "right";
                            } else {
                                moveDirection = "left";
                            }
                            console.log("GYL-156:scrollableImageWrapper:mouseMove: move direction is " + moveDirection);
                            previousMoveValue = currentMoveValue;

                            if (!thisView.model.get("userTookControl")) {
                                e.stopPropagation();
                                console.log("GYL-156:scrollableImageWrapper:mouseMove: event first time -> move image manually");
                                thisView.model.set("userTookControl", true);
                                transformMatrix = window.getComputedStyle(thisView.$el[0]).getPropertyValue(CSSHandler.getCurrentBrowserPrefix().transform);
                                if (transformMatrix != "none") {
                                    transformMatrixValues = transformMatrix.match(/-?[0-9\.]+/g);
                                    currentOffsetX = Math.round(-transformMatrixValues[4]);
                                    currentOffsetY = Math.round(-transformMatrixValues[5]);
                                    CSSHandler.removeBrowserSpecificProperty(thisView.$el, "transition");
                                    CSSHandler.removeBrowserSpecificProperty(thisView.$el, "transform");
                                    console.log("GYL-156:scrollableImageWrapper:mouseMove: will scroll to X: " + currentOffsetX + " Y: " + currentOffsetY);
                                    scrollableImageWrapper[0].scrollLeft += currentOffsetX;
                                    scrollableImageWrapper[0].scrollTop += currentOffsetY;
                                }
                            } else {
                                console.log("GYL-156:scrollableImageWrapper:mouseMove: event controlled by user");

                                var updatedScrollLeft = scrollableImageWrapper[0].scrollLeft;
                                var navigateRightCondition = Math.abs(updatedScrollLeft - imageOverflowWidth) < 10;
                                var navigateLeftCondition = Math.abs(updatedScrollLeft) < 10;

                                if ((navigateLeftCondition && moveDirection == "right") || (navigateRightCondition && moveDirection == "left")) {
                                    console.log("GYL-156:scrollableImageWrapper:mouseMove: now scroll pages");
                                } else {
                                    e.stopPropagation();
                                    console.log("GYL-156:scrollableImageWrapper:mouseMove: now scroll image");
                                }
                            }
                        });
                    }, 100);
                },
                setupScrollOniOS: function () {
                    var scrollableImageWrapper = $("<div>").addClass("scrollableWithinPage"),
                        thisView = this,
                        transformMatrix, transformMatrixValues, currentOffsetX, currentOffsetY,
                        scrollLeftValue = Math.round(-parseInt(this.model.get("left")) * App.settings.width / 100),
                        scrollTopValue = Math.round(-parseInt(this.model.get("top")) * App.settings.height / 100);
                    scrollableImageWrapper.css("z-index", this.model.get("ZIndex"));
                    this.$el.wrap(scrollableImageWrapper);
                    scrollableImageWrapper = this.$el.parent();
                    this.$el.css("position", "relative");
                    this.model.set("userTookControl", false);
                    setTimeout(function () {
                        console.log("Now set scroll property");
                        scrollableImageWrapper.scrollLeft(scrollLeftValue);
                        scrollableImageWrapper.scrollTop(scrollTopValue);
                    }, 0);
                    setTimeout(function () {
                        console.log("Now bind to scroll event");
                        scrollableImageWrapper.bind(AppUtils.EventTypes.scroll, function () {
                            console.log("scrollableImageWrapper: scroll left on image is " + scrollableImageWrapper[0].scrollLeft);
                            if (!thisView.model.get("userTookControl")) {
                                thisView.model.set("userTookControl", true);
                                console.log("Now move image manually");
                                transformMatrix = window.getComputedStyle(thisView.$el[0]).getPropertyValue(CSSHandler.getCurrentBrowserPrefix().transform);
                                if (transformMatrix != "none") {
                                    transformMatrixValues = transformMatrix.match(/-?[0-9\.]+/g);
                                    currentOffsetX = -transformMatrixValues[4];
                                    currentOffsetY = -transformMatrixValues[5];
                                    CSSHandler.removeBrowserSpecificProperty(thisView.$el, "transition");
                                    CSSHandler.removeBrowserSpecificProperty(thisView.$el, "transform");
                                    console.log("Will scroll to X: " + currentOffsetX + " Y: " + currentOffsetY);
                                    scrollableImageWrapper[0].scrollLeft += currentOffsetX;
                                    scrollableImageWrapper[0].scrollTop += currentOffsetY;
                                }
                            }
                        });
                    }, 100);

                },
                setupScrollable: function () {
                    if (this.model.get("scrollableWithinPage")) {
                        var thisView = this;
                        // here we use underscore defer method in order to let element be added in DOM before we can wrap it.
                        // defer documentation is: "Defers invoking the function until the current call stack has cleared, similar to using setTimeout with a delay of 0."
                        _(function () {
                            if (AppUtils.DeviceType.ANDROID) {
                                if (thisView.scrollableWithinPageEnabled) {
                                    pageModel.set("pageModelUpdate", {
                                        "key": "usingScrollableWithinPage",
                                        "value": true,
                                        "salt": new Date().getTime()
                                    });
                                    thisView.setupScrollOnAndroid();
                                }
                            } else {
                                thisView.setupScrollOniOS(); // also used for web browsers
                            }
                        }).defer();
                    }
                },
                updateSizeAndPosition: function () {
                    if (this.scrollableWithinPageEnabled) {
                        this.setObjectSizeAndPositionWithAbsoluteValues(true, false);
                    } else {
                        this.setObjectSizeAndPositionWithAbsoluteValues(true, true);
                    }
                },
                render: function () {
                    if (this.shouldRenderObject()) {
                        this.renderStartHandler();
                        this.addHWAccelerationIfRequired();
                        if (null == this.$imageEl) {
                            this.$imageEl = $(this.getTemplate(this.model.attributes));
                            this.bindImageSpecificEvents();
                        }

                        if (this.model.get("scrollable")) {
                            this.$imageEl.removeAttr("width").removeAttr("height");
                            this.$el.addClass("overflowScroll");
                        }

                        this.$el.html(this.$imageEl);

                        // for image objects we use absolute size because of a bug in chrome causing images to jump when fade or move effects are started
                        // the bug is visible for XWalk web view on up-down move effects and also reported in GYL-329

                        this.scrollableWithinPageEnabled = this.model.get("scrollableWithinPage");

                        if (this.scrollableWithinPageEnabled && AppUtils.DeviceType.ANDROID) {
                            this.scrollableWithinPageEnabled = Chapter.getSettings()['enableScrollableWithinPageOnAndroid'];
                        }

                        this.updateSizeAndPosition();

                        this.$el.css("z-index", this.model.get("ZIndex"));
                        this.setInitialVisibility();
                        this.startZoomEffectsIfNeeded();
                        this.setupScrollable();
                        this.setupZoomAndPanOptions();
                        this.setupGIFImageAnimationControl();
                        return this;
                    } else {
                        return null;
                    }
                },

                setupGIFImageAnimationControl: function () {
                    var thisView = this;
                    if (rawImageModel.customProperties.extension == "gif") {
                        setTimeout(function () {
                            thisView.gifffer = new Gifffer(thisView.$el[0], {userInteraction: false});
                        }, 0); // wait for the element to be added to DOM
                    }
                },

                customActivationHandler: function () {
                    if (this.gifffer) {
                        this.gifffer.playGif();
                    }
                },

                pageDeactivatedHandler: function () {
                    if (this.model.get("pageDeactivated")) {
                        if (this.gifffer) {
                            this.gifffer.stopGif();
                        }
                        this.model.set("pageDeactivated", false);
                    }
                },

                setupZoomAndPanOptions: function () {
                    var zoomAndPanSettings;
                    if ("zoomAndPanSettings" in rawImageModel.properties) {
                        zoomAndPanSettings = rawImageModel.properties.zoomAndPanSettings;
                        this.imageSettings = zoomAndPanSettings.imageSettings;
                        this.zoomSettings = zoomAndPanSettings.zoomSettings;
                        this.moveSettings = zoomAndPanSettings.moveSettings;
                        this.manualControlSettings = zoomAndPanSettings.manualControlSettings;
                        if (this.zoomSettings.zoomEnabled || this.moveSettings.moveEnabled || this.manualControlSettings["scrolling"] || this.manualControlSettings["pinchZoom"]) {
                            console.log("Now init image for zoom and pan");
//                            this.$el.addClass("zeroTransform");
                            this.$imageEl.attr("width", this.imageSettings.imageWidth + "%").attr("height", this.imageSettings.imageHeight + "%");
                            this.$imageEl.css("position", "absolute");

                            imageWidthPercent = this.imageSettings.imageWidth;
                            imageHeightPercent = this.imageSettings.imageHeight;
                            imageLeft = this.imageSettings.imageLeft;
                            imageTop = this.imageSettings.imageTop;

                            CSSHandler.setTop(this.$imageEl, this.imageSettings.imageTop + "%");
                            CSSHandler.setLeft(this.$imageEl, this.imageSettings.imageLeft + "%");

                            if (this.zoomSettings.zoomEnabled || this.manualControlSettings.pinchZoom) {
                                this.zoomViewport = $("<div>").addClass("zoomViewport");
                                this.$imageEl.wrap(this.zoomViewport);
                                this.zoomViewport = this.$imageEl.parent();
                            }
                            if (this.moveSettings.moveEnabled || this.manualControlSettings.scrolling) {
                                this.moveViewport = $("<div>").addClass("moveViewport");
                                this.$imageEl.wrap(this.moveViewport);
                                this.moveViewport = this.$imageEl.parent();
                            }

                            this.setupHammerControl(this.manualControlSettings["scrolling"] || this.manualControlSettings["pinchZoom"]);
                            this.setupEffectReset();
                        }
                    }
                },

                customStateRestore: function () { // executed each time we leave the page
                    if (scheduledMoveEffect != null) {
                        clearTimeout(scheduledMoveEffect);
                        scheduledMoveEffect = null;
                    }
                    if (scheduledZoomEffect != null) {
                        clearTimeout(scheduledZoomEffect);
                        scheduledZoomEffect = null;
                    }
                    this.transitionPlayer = null;
                },

                setupEffectReset: function () {  // restore operations that need to be done only if going to a neighbour page
                    var thisView = this;
                    thisView.pushUnloadHandler(function () {
                        // start by reseting variables
                        posX = 0;
                        posY = 0;
                        lastPosX = 0;
                        lastPosY = 0;
                        scale = 1;
                        lastScale = 1;
                        scrolledToRightBorder = false;
                        scrolledToLeftBorder = false;
                        preventPageChange = true;
                        userIsScrolling = false;
                        userIsZooming = false;
                        thisView.updateMaxXYForScaleFactor(1);
                        if (typeof thisView.moveViewport != "undefined") {
                            CSSHandler.removeBrowserSpecificProperty(thisView.moveViewport, "transition");
                            CSSHandler.removeBrowserSpecificProperty(thisView.moveViewport, "transform");
                        }
                        if (typeof thisView.zoomViewport != "undefined") {
                            CSSHandler.removeBrowserSpecificProperty(thisView.zoomViewport, "transition");
                            CSSHandler.removeBrowserSpecificProperty(thisView.zoomViewport, "transform");
                        }
                    });
                },

                setupHammerControl: function (manualControl) {
                    var thisView = this;
                    require(['hammerJS'], function (Hammer) {
                        window.Hammer = Hammer;
                        require(['hammerFakeMultiTouch', 'hammerShowTouches'], function () {

                            if (!Hammer.HAS_TOUCHEVENTS && !Hammer.HAS_POINTEREVENTS) {
                                Hammer.plugins.showTouches();
                            }

                            if (!Hammer.HAS_TOUCHEVENTS && !Hammer.HAS_POINTEREVENTS) {
                                Hammer.plugins.fakeMultitouch();
                            }

                            var hammerTime = Hammer(thisView.el, {
                                transform_always_block: true,
                                transform_min_scale: 1,
                                drag_block_horizontal: false,
                                drag_block_vertical: true,
                                drag_min_distance: 0
                            });
                            var eventsToRegister = "transformstart transform transformend";
                            if (manualControl) {
                                eventsToRegister += " touch drag dragend";
                            }
                            hammerTime.on(eventsToRegister, function (ev) {
                                thisView.manageMultiTouch(ev);
                            });

                        });
                    });
                },

                updateMaxXYForScaleFactor: function (scale1) {
                    var baseX = (visibleImageWidth * (scale1 - 1)) / (2 * scale1); // relevant only for zoomed state
                    minX = baseX - imageLeft * visibleImageWidth / 100; // positive value if scale1 > 1
                    maxX = -(baseX + imageLeft * visibleImageWidth / 100 + (imageWidthPercent / 100 - 1) * visibleImageWidth); // negative value
                    var baseY = (visibleImageHeight * (scale1 - 1)) / (2 * scale1);
                    minY = baseY - imageTop * visibleImageHeight / 100; // positive value if scale1 > 1
                    maxY = -(baseY + imageTop * visibleImageHeight / 100 + (imageHeightPercent / 100 - 1) * visibleImageHeight); // negative value
                },

                updateValuesOnMoveEnd: function () {
                    var transformMatrix = window.getComputedStyle(this.moveViewport[0]).getPropertyValue(CSSHandler.getCurrentBrowserPrefix().transform);
                    if (transformMatrix != "none") {
                        var transformMatrixValues = transformMatrix.match(/-?[0-9\.]+/g);
                        posX = lastPosX = Math.round(transformMatrixValues[4]);
                        posY = lastPosY = Math.round(transformMatrixValues[5]);
                    }
                },

                updateValuesOnZoomEnd: function () {
                    var transformMatrix = window.getComputedStyle(this.zoomViewport[0]).getPropertyValue(CSSHandler.getCurrentBrowserPrefix().transform);
                    if (transformMatrix != "none") {
                        var transformMatrixValues = transformMatrix.match(/-?[0-9\.]+/g);
                        scale = lastScale = parseFloat(transformMatrixValues[0]);
                        this.processScaleZoomLevelChange(lastScale);
                    }
                },

                takeControlOverAutomaticEffects: function () {
                    if (this.manualControlSettings.scrolling && !userIsScrolling) {
                        console.log("Now take control for move");
                        userIsScrolling = true;
                        this.updateValuesOnMoveEnd();
                        CSSHandler.removeBrowserSpecificProperty(this.moveViewport, "transition");
                        CSSHandler.translate3D(this.moveViewport, lastPosX, lastPosY, 0, 0, null, null, false);
                    }
                    if (this.manualControlSettings.pinchZoom && !userIsZooming) {
                        console.log("Now take control for zoom");
                        userIsZooming = true;
                        this.updateValuesOnZoomEnd();
                        CSSHandler.removeBrowserSpecificProperty(this.zoomViewport, "transition");
                        CSSHandler.scale(this.zoomViewport, lastScale);
                    }
                },

                processScaleZoomLevelChange: function (zoomLevel) {
                    this.updateMaxXYForScaleFactor(zoomLevel);
                    posX = Math.max(maxX, posX);
                    posX = Math.min(posX, minX);
                    posY = Math.max(maxY, posY);
                    posY = Math.min(posY, minY);
                    lastPosX = posX;
                    lastPosY = posY;
                },

                cancelEvent: function (ev) {
                    console.log("Canceled event is: " + ev.gesture.srcEvent.type);
                    ev.gesture.srcEvent.stopPropagation();
                    ev.gesture.srcEvent.preventDefault();
                },

                getTouchPoint: function (ev) {
                    if ("touches" in ev.gesture.srcEvent) {
                        return {
                            pageX: parseInt(ev.gesture.srcEvent.touches[0].pageX),
                            pageY: parseInt(ev.gesture.srcEvent.touches[0].pageY)
                        };
                    } else {
                        return {pageX: ev.gesture.srcEvent.pageX, pageY: ev.gesture.srcEvent.pageY};
                    }
                },

                manageMultiTouch: function (ev) {
                    switch (ev.type) {
                        case 'touch':
                            lastScale = scale;
                            this.takeControlOverAutomaticEffects();
                            if (!AppUtils.useNativeScroll) {
                                this.cancelEvent(ev);
                            }
                            break;

                        case 'drag':
                            var possibleNextValue = ev.gesture.deltaX + lastPosX;

                            preventPageChange = true;

                            if (possibleNextValue < maxX) {
                                scrolledToRightBorder = true;
                                if (ev.gesture.direction == "left") {
                                    console.log("scrolling past right border");
                                    if (!transitionToRightStared) {
                                        transitionToRightStared = true;
                                        var touchPoint = this.getTouchPoint(ev);
                                        console.log("Simulate " + AppUtils.EventTypes.mouseDown + " on slidingScenesWrapper at point (" + touchPoint.pageX + ", " + touchPoint.pageY + ")");
                                        $('#slidingScenesWrapper').simulate(AppUtils.EventTypes.mouseDown, {
                                            clientX: touchPoint.pageX,
                                            clientY: touchPoint.pageY
                                        });
                                    }
                                    preventPageChange = false;
                                }
                            }
                            if (possibleNextValue > minX) {
                                scrolledToLeftBorder = true;
                                if (ev.gesture.direction == "right") {
                                    console.log("scrolling past left border");
                                    if (!transitionToLeftStared) {
                                        transitionToLeftStared = true;
                                        var touchPoint = this.getTouchPoint(ev);
                                        console.log("Simulate " + AppUtils.EventTypes.mouseDown + " on slidingScenesWrapper at point (" + touchPoint.pageX + ", " + touchPoint.pageY + ")");
                                        $('#slidingScenesWrapper').simulate(AppUtils.EventTypes.mouseDown, {
                                            clientX: touchPoint.pageX,
                                            clientY: touchPoint.pageY
                                        });
                                    }
                                    preventPageChange = false;
                                }
                            }

                            if (preventPageChange) {
                                console.log("Now prevent page change");
                                this.cancelEvent(ev);
                            } else {
                                console.log("Now allow page change");
                            }

                            posX = Math.max(maxX, ev.gesture.deltaX / scale + lastPosX);
                            posX = Math.min(posX, minX);
                            posY = Math.max(maxY, ev.gesture.deltaY / scale + lastPosY);
                            posY = Math.min(posY, minY);
                            break;

                        case 'transform':
                            this.cancelEvent(ev);
                            if (this.manualControlSettings.pinchZoom) {
                                scale = Math.max(1, Math.min(lastScale * ev.gesture.scale, this.zoomSettings.zoomFactor));
                            } else {
                                scale = 1;
                            }
                            this.processScaleZoomLevelChange(scale);
                            break;

                        case 'transformstart':
                            console.log("Event is " + ev.type);
                            Chapter.getScrollingElementsWrapper().getModel().set("pinchInProgress", true);
                            this.cancelEvent(ev);
                            break;

                        case 'transformend':
                            console.log("Event is " + ev.type);
                            Chapter.getScrollingElementsWrapper().getModel().set("pinchInProgress", false);
                            this.cancelEvent(ev);
                            break;

                        case 'dragend':
                            transitionToLeftStared = false;
                            transitionToRightStared = false;
                            lastPosX = posX;
                            lastPosY = posY;
                            break;
                    }

                    if (preventPageChange) {
                        if (this.manualControlSettings.scrolling && ev.type == "drag") {
                            CSSHandler.translate3D(this.moveViewport, posX, posY, 0, 0, null, null, false);
                        }

                        if (this.manualControlSettings.pinchZoom && ev.type == "transform") {
                            CSSHandler.scale(this.zoomViewport, scale);
                        }
                    }
                },

                startZoomAndPan: function () {
                    var moveOffsetX, moveOffsetY,
                        thisView = this;
                    if ("zoomAndPanSettings" in rawImageModel.properties) {
                        console.log("Now startZoomAndPan effects");

                        scheduledMoveEffect = setTimeout(function () {
                            if (thisView.moveSettings.moveEnabled && !userIsScrolling) {
                                CSSHandler.setTransitionEndHandler(thisView.moveViewport, function (event) {
                                    event.stopPropagation();
                                    console.log("move TransitionEnd event");
                                    CSSHandler.removeBrowserSpecificProperty(thisView.moveViewport, "transition", null);
                                    thisView.updateValuesOnMoveEnd();
                                }, null);
                                moveOffsetX = Math.round(thisView.moveSettings.moveOffsetX * visibleImageWidth / 100);
                                moveOffsetY = Math.round(thisView.moveSettings.moveOffsetY * visibleImageHeight / 100);
                                CSSHandler.translate3D(thisView.moveViewport, moveOffsetX, moveOffsetY, 0, thisView.moveSettings.moveDuration, "0, 0, 1, 1", null, false);
                            }
                        }, thisView.moveSettings.moveDelay);

                        scheduledZoomEffect = setTimeout(function () {
                            if (thisView.zoomSettings.zoomEnabled && !userIsZooming) {

//                            var zoomOriginX = Math.round(thisView.zoomSettings.originX * 100 / thisView.imageSettings.imageWidth);
//                            var zoomOriginY = Math.round(thisView.zoomSettings.originY * 100 / thisView.imageSettings.imageHeight);
//                            CSSHandler.setTransformOrigin(thisView.$imageEl, zoomOriginX + "%", zoomOriginY + "%");

                                var zoomOriginX = Math.round(thisView.zoomSettings.originX);
                                var zoomOriginY = Math.round(thisView.zoomSettings.originY);
                                CSSHandler.setTransformOrigin(thisView.zoomViewport, zoomOriginX + "%", zoomOriginY + "%");

                                CSSHandler.setTransition(thisView.zoomViewport, CSSHandler.getBrowserPrefixedProperty("transform"), thisView.zoomSettings.zoomDuration);
                                CSSHandler.setTransitionEndHandler(thisView.zoomViewport, function (event) {
                                    event.stopPropagation();
                                    console.log("zoom TransitionEnd event");
                                    CSSHandler.removeBrowserSpecificProperty(thisView.zoomViewport, "transition", null);
                                    thisView.updateValuesOnZoomEnd();
                                });
                                CSSHandler.scale(thisView.zoomViewport, thisView.zoomSettings.zoomFactor);
                            }
                        }, thisView.zoomSettings.zoomDelay);
                    }
                }
            });
        };
        this.instantiateObjects = function () {
            this.imageModel = this.getImageModel();
            this.imageView = new this.ImageView({
                model: that.imageModel
            });
        };

        this.getImageModel = function () {
            var model = new Backbone.Model(rawImageModel.properties);
            model.set("imageURL", ResourceManager.getPathForImage(rawImageModel.properties.objectId, rawImageModel.customProperties.extension));
            model.set("gradientColor", rawImageModel.properties.gradientColorRGB);
            model.set("value1", rawImageModel.properties.gradientValue1);
            model.set("value2", rawImageModel.properties.gradientValue2);
            model.set('print', rawImageModel.properties["print"]);
            return model;
        };

        return {
            init: function () {
                that.defineClasses();
                that.instantiateObjects();
            },
            getView: function () {
                return that.imageView;
            },
            getModel: function () {
                return that.imageModel;
            }

        }
    }
});