define(function () {
    return function (rawZipPackageModel, pageModel) {
        var that = this;

        this.defineClasses = function () {

            that.ZipPackageView = ElementBaseView.extend({
                tagName: "div",
                className: "sceneObject",
                elementsViews: [],
                template: templates.getTemplate("externalURL"),
                id: rawZipPackageModel.properties.id,
                parentElementModel: pageModel,
                events: function () {
                    var _events = {};
                    _events[AppUtils.EventTypes.mouseDown] = "touchstart";
                    _events[AppUtils.EventTypes.click] = "clickHandler";
                    _events["touchcancel"] = "touchcancelHandler";
                    _events["touchcend"] = "touchcendHandler";
                    return _events;
                },
                notifyParentThatObjectIsLoaded: function () {
                    this.parentElementModel.set("objectLoaded", this.getLoadedInfo(false));
                },
                customInitialize: function () {
                    this.listenTo(this.model, "change:pageDeactivated", this.pageDeactivatedHandler);
                },
                touchcendHandler: function () {
                    console.log("touchcendHandler on " + this.$el.attr("id"));
                },
                touchcancelHandler: function () {
                    console.log("touchcancelHandler on " + this.$el.attr("id"));
                },
                removeView: function () {
                    if (this.model.get("removed")) {
                        this.remove();
                        this.unbind();
                    }
                },
                activateObject: function () {
                    if (this.model.get("activateObject")) {
                        this.startIFrame();
                        this.model.set("objectActivated", true);
                        this.model.set("activateObject", false);
                    }
                },
                startIFrame: function () {
                    var iFrameContentWindow = this.getIFrameContentWindow();
                    iFrameContentWindow.postMessage("START_PLAY", iFrameContentWindow.location.href);
                },
                activateToggle: function () {
                    this.displayHideOnceOnClick();
                },
                pageDeactivatedHandler: function () {
                    var iFrameContentWindow;
                    if (this.model.get("pageDeactivated")) {
                        iFrameContentWindow = this.getIFrameContentWindow();
                        iFrameContentWindow.postMessage("STOP_PLAY", iFrameContentWindow.location.href);
                        this.model.set("pageDeactivated", false);
                    }
                },
                getIFrameContentWindow: function () {
                    return this.$el.find("iframe")[0].contentWindow;
                },
                bindIFrameLoadHandler: function () {
                    var thisView = this;
                    this.$el.find("iframe").load(function () {
                        thisView.parentElementModel.set("objectLoaded", thisView.getLoadedInfo(true));
                        console.log("iFrame content loaded");
                    });
                },
                render: function () {
                    if (this.shouldRenderObject()) {
                        this.renderStartHandler();
                        if (null == this.$loadedTemplate)
                            this.$loadedTemplate = $(this.template(this.model.attributes));

                        this.$el.html(this.$loadedTemplate);
                        this.bindIFrameLoadHandler();
                        this.setLeft(this.model.get("left"));
                        this.setWidth(this.model.get("width"));
                        this.setHeight(this.model.get("height"));
                        this.setTop(this.model.get("top"));
                        this.$el.css("z-index", this.model.get("ZIndex"));
                        this.setInitialVisibility();
                        this.startZoomEffectsIfNeeded();
                        return this;
                    }
                    return null;
                }
            });
        };
        this.instantiateObjects = function () {
            this.zipPackageModel = this.getZipPackageModel();
            this.zipPackageView = new this.ZipPackageView({
                model: that.zipPackageModel
            });

        };

        this.getZipPackageModel = function () {
            var model = new Backbone.Model(rawZipPackageModel.properties);
            if (AppUtils.nativeApp || AppUtils.offlineExport) {
                model.set("externalURL", "files/" + rawZipPackageModel.properties.objectId + "/" + rawZipPackageModel.properties['startFileName']);
            } else {
                model.set("externalURL", "../zip/" + rawZipPackageModel.properties.objectId + "/" + rawZipPackageModel.properties['startFileName']);
            }
            return model;
        };

        return {
            init: function () {
                that.defineClasses();
                that.instantiateObjects();
            },
            getView: function () {
                return that.zipPackageView;
            },
            getModel: function () {
                return that.zipPackageModel;
            }
        }
    }
});