define(['Templates', 'PersistentSceneObjectsUtils'], function () {
    return function (pageModel) {
        var that = this,
            totalNumberOfObjects = 0,
            numberOfCreatedObjects = 0,
            numberOfLoadedObjects = 0,
            initialActiveObjects = 0;
        this.defineClasses = function () {
            that.PageView = BaseView.extend({
                    tagName: "div",
                    className: "page",
                    id: function () {
                        return "page-" + this.model.get("index")
                    },
                    elementsViews: [],
                    workingObjectsModels: {},
                    categoriesForCurrentPage: null, // only categories where scene objects are part of, in page JSON "objectCategoryIds"
                    categoriesToBeProcessedQueue: [],
                    originalCategoriesForCurrentPage: null,
                    zoomInstance: null,
                    zoomTransitionPlayer: null,

                    events: function () {
                        var _events = {};
                        if (!Chapter.getSettings()["zoomEnabled"]) {
                            _events["gesturestart"] = "gestureStart";
                            _events["gesturechange"] = "gestureChange";
                            _events["gestureend"] = "gestureEnd";
                        }
                        // when using global zoom we don't support zoom on page
                        return _events;
                    },

                    gestureStart: function (event) {
                        event.preventDefault();
                        event.stopPropagation();
                        console.log("gestureStart");
                        var idx, model;
                        for (idx = 0; idx < that.workingObjectsCollection.length; idx++) {
                            model = that.workingObjectsCollection.models[idx];
                            model.set("zoomGestureShouldStartFromBeginning", true);
                        }
                    },

                    gestureChange: function (event) {
                        event.preventDefault();
                        event.stopPropagation();
                        var angle = event.originalEvent.rotation;
                        var scale = event.originalEvent.scale;
                        console.log("gestureChange angle is " + angle + ", scale is " + scale);
                        this.notifyAllPageObjectOfZoomChange(scale);
                    },

                    gestureEnd: function (event) {
                        event.preventDefault();
                        event.stopPropagation();
                        console.log("gestureEnd");
                    },

                    actions: {
                        "usingScrollableWithinPage": "processScrollableWithinPage",
                        "updateForHardwareAcceleration": "processMovingImage",
                        "pauseAllObjectsInPage": "processPauseObjectsInPage",
                        "resumeAllObjectsInPage": "processResumeObjectsInPage"
                    },

                    initialize: function (attr) {
                        this.listenTo(this.model, "change:pageModelUpdate", this.processPageModelUpdate);
                        this.listenTo(this.model, "change:removed", this.removeView);
                        this.listenTo(this.model, "change:activatePage", this.activatePage);
                        this.listenTo(this.model, "change:shouldBeActive", this.allObjectsLoaded);
                        this.listenTo(this.model, "change:allObjectsLoaded", this.allObjectsLoaded);
                        this.listenTo(this.model, "change:allResourcesLoaded", this.allObjectsLoaded);
                        this.listenTo(this.model, "change:activatedObject", this.activatedObject);
                        this.listenTo(this.model, "change:toggleTextBoxVisibility", this.toggleTextBoxVisibility);
                        this.listenTo(this.model, "change:toggledCategories", this.toggledCategories);
                        this.listenTo(this.model, "change:resetZoom", this.resetZoomHandler);
                        this.listenTo(this.model, "change:zoomedInObject", this.pageObjectZoomedIn);
                        this.listenTo(this.model, "change:zoomedOutObject", this.pageObjectZoomedOut);
                        this.listenTo(this.model, "change:objectLoaded", this.objectLoadedHandler);
                        this.listenTo(this.model, "change:zIndex", this.updateZIndex);
                        this.listenTo(this.model, "change:translateOffset", this.translatePageHandler);
                        this.listenTo(this.model, "change:orientation", this.updateOrientation);
                        this.listenTo(this.model, "change:zoomValue", this.updateZoomValue);
                        this.listenTo(this.model, "change:removeZoom", this.removeZoomHandler);
                        this.listenTo(this.model, "change:deactivatePage", this.deactivatePage);
                        this.listenTo(this.model, "change:closeLightBoxes", this.closeLightBoxes);
                        this.listenTo(this.model, "change:downloadComplete", this.downloadComplete);
                        this.setCategoriesForCurrentPage();
                    },
                    processPageModelUpdate: function () {
                        var newValues = this.model.get("pageModelUpdate"),
                            updateKey = newValues["key"],
                            keyValue = newValues["value"];
                        this.model.set(updateKey, keyValue);
                        this[this.actions[updateKey]](keyValue);
                    },
                    setZoomInstance: function (zoomInstance) {
                        this.zoomInstance = zoomInstance;
                    },
                    getZoomInstance: function () {
                        return this.zoomInstance;
                    },
                    updateOrientation: function () {
                        var idx, objectInPageModel, leftValueForUI;
                        console.log("MOB-1718: Update page " + this.model.get("index") + " for orientation " + this.model.get("orientation"));
                        this.setWidth(App.settings.pageWidth);
                        this.setHeight(App.settings.pageHeight);
//                        console.log("MOB-1718: Update page dimensions " + this.model.get("index") + " set width " + App.settings.pageWidth + " height " + App.settings.height);
                        leftValueForUI = this.model.get("left");
                        if (Chapter.getSettings()["useTwoInOneLandscapeMode"]) {
                            // now value that we use to update UI can be different here !!!
                            leftValueForUI = this.model.get("index") % 2 == 1 ? App.settings.pageWidth : 0;
                        }
                        // update model value and UI for left
                        console.log("MOB-1718: new left value is " + leftValueForUI);
                        this.setLeft(leftValueForUI);
                        // now announce all page objects about orientation change
                        for (idx = 0; idx < that.workingObjectsCollection.length; idx++) {
                            objectInPageModel = that.workingObjectsCollection.models[idx];
                            objectInPageModel.set("orientation", this.model.get("orientation"));
                        }
                    },
                    updateZIndex: function () {
                        console.log("updateZINDEX for " + this.model.get("index"));
                        this.setZIndex(this.model.get('zIndex'));
                    },
                    translatePageHandler: function () {
                        var translateOffset = this.model.get("translateOffset");
                        this.$el.css("transform", "translate(" + translateOffset + "px, 0)");
                    },
                    setCategoriesForCurrentPage: function () {
                        var categories = Chapter.getObjectCategoriesForPages()[this.model.get('id')], categoriesAsString;
                        if (null != categories && !this.isEmpty(categories)) {
                            categoriesAsString = JSON.stringify(categories);
                            this.categoriesForCurrentPage = JSON.parse(categoriesAsString);
                            this.activatedObjectsInCategoriesForCurrentPage = JSON.parse(categoriesAsString);
                            this.model.set("activatedObjectsInCategoriesForCurrentPage", this.activatedObjectsInCategoriesForCurrentPage);
                            this.chapterObjectCategories = JSON.parse(JSON.stringify(Chapter.getObjectCategories()));
                        }
                    },
                    objectLoadedHandler: function () {
                        var loadedObject = this.model.get('objectLoaded');
                        if (null != loadedObject) {
                            this.model.set('objectLoaded', null);
                            this.adjustObjectCategoriesForPage(loadedObject);
                            if (loadedObject["loaded"]) {
                                numberOfLoadedObjects++;
                                console.log("Number of loaded objects for page " + this.model.get("index") + " is now " + numberOfLoadedObjects + " with a total of " + totalNumberOfObjects);
                                if (numberOfLoadedObjects == initialActiveObjects) {
                                    this.model.set("allResourcesLoaded", true);
                                }
                            }
                        }
                    },
                    adjustObjectCategoriesForPage: function (loadedObject) {
                        var categoriesIdsForLoadedObject = loadedObject.objectCategoryIds,
                            idx;
                        if (null != categoriesIdsForLoadedObject) {
                            for (idx = 0; idx < categoriesIdsForLoadedObject.length; idx++) {

                                var category = this.categoriesForCurrentPage[categoriesIdsForLoadedObject[idx]];

                                if (category[loadedObject['id']] != loadedObject.loaded) {
                                    category[loadedObject['id']] = loadedObject.loaded;
                                    if (loadedObject.loaded) {
                                        category['numberOfLoadedObjects']++;
                                        this.activateCategoriesFromQueueIfNeeded();
                                    } else {
                                        category['numberOfLoadedObjects']--;
                                    }
                                }
                            }
                        }
                    },
                    activateCategoriesFromQueueIfNeeded: function () {
                        if (this.categoriesToBeProcessedQueue.length > 0) {
                            var categories = this.categoriesToBeProcessedQueue[0],
                                activatedCategories = categories.activatedCategories,
                                category, activatedCategory, shouldActivateCategories = true;
                            for (category in activatedCategories) {
                                if (activatedCategories.hasOwnProperty(category)) {
                                    activatedCategory = this.categoriesForCurrentPage[category];
                                    if (null != activatedCategory && activatedCategory['numberOfLoadedObjects'] != activatedCategory['totalNumberOfObjects']) {
                                        shouldActivateCategories = false;
                                        break;
                                    }
                                }
                            }
                            if (shouldActivateCategories) {
                                this.categoriesToBeProcessedQueue.shift();
                                Chapter.getScrollingElementsModel().set("toggledCategories", categories);
                            }
                        }
                    },
                    removeView: function () {
                        var pageNumber = this.model.get("index");
                        if (this.model.get("removed")) {
                            if (typeof SoundPlayer != 'undefined') {
                                SoundPlayer.endAllPlayingSoundsForPage(pageNumber, true);
                            }
                            if (typeof SoundRecorder != 'undefined') {
                                SoundRecorder.endAllRecordedSounds(pageNumber);
                            }
                            this.removeAllChildren();
                            this.remove();
                            this.unbind();

                        }
                    },
                    removeAllChildren: function () {
                        var models = that.workingObjectsCollection.models, idx;
                        for (idx = 0; idx < models.length; idx++) {
                            models[idx].set('removed', true);
                        }
                    },

                    render: function () {
                        var objects = that.pageObjects,
                            currentObject, renderedObject,
                            pageNumber = this.model.get("index");
                        this.pageContentHolder = $("<div>").addClass("pageContentHolder");
                        this.$el.append(this.pageContentHolder);
                        console.log("that.pageObjects.size render = " + Object.keys(that.pageObjects).length);
                        console.log("pageNumber = " + pageNumber);
                        for (currentObject in objects) {
                            if (objects.hasOwnProperty(currentObject) && !this.shouldBeExcludedFromDom(objects[currentObject])) {
                                renderedObject = objects[currentObject].getView().render();
                                if (null != renderedObject) {
                                    this.pageContentHolder.append(renderedObject.el);
                                }
                            }
                        }
                        console.log("renderPageElement");
                        var leftValue = this.model.get("left");

                        if (Chapter.getSettings()["useTwoInOneLandscapeMode"]) {
                            leftValue = pageNumber % 2 == 1 ? App.settings.pageWidth : 0;
                        }
                        this.setLeft(leftValue);
                        if (Chapter.getSettings().reserveMenuArea) {
                            this.setWidth(App.settings.pageWidth);
                        }
                        else {
                            this.setWidth(this.model.get("width"));
                        }
                        if (Chapter.getSettings().reserveMenuArea) {
                            this.setHeight(App.settings.pageHeight);
                        }
                        else {
                            this.setHeight(this.model.get("height"));
                        }
                        this.setZIndex(zIndex.inactivePage);
                        return this;
                    },
                    toggleTextBoxVisibility: function () {
                        console.log("Should signal all text boxes to toggle visibility to " + this.model.get("toggleTextBoxVisibility"));
                        var pageObject, pageObjectModel;
                        for (pageObject in that.pageObjects) {
                            pageObjectModel = that.pageObjects[pageObject].getModel();
                            if (pageObjectModel.get("type") == "textBox") {
                                if (this.model.get("toggleTextBoxVisibility")) {
                                    pageObjectModel.set("hiddenDueToTextSettings", false);
//                                    this.addNewObjectInPage(pageObject, pageObjectModel);
                                } else {
//                                    pageObjectModel.set("removed", true);
                                    pageObjectModel.set("hiddenDueToTextSettings", true);
                                }
                            }
                        }

                    },
                    processScrollableWithinPage: function (value) {
                        if (value) {
                            var pageObjectModel, idx;
                            for (idx = 0; idx < that.workingObjectsCollection.models.length; idx++) {
                                pageObjectModel = that.workingObjectsCollection.models[idx];
                                if (pageObjectModel.get("type") == "textBox") {
                                    console.log("Now signal text box in page " + this.model.get("index") + " to adjust for scrollable within page");
                                    pageObjectModel.set("objectModelUpdate", {
                                        "key": "updateForScrollableWithinPage",
                                        "value": true,
                                        "salt": new Date().getTime()
                                    });
                                }
                            }
                        }
                    },
                    processMovingImage: function (value) {
                        if (value) {
                            var pageObject, pageObjectModel;
                            for (pageObject in that.pageObjects) {
                                pageObjectModel = that.pageObjects[pageObject].getModel();
                                if (pageObjectModel.get("type") == "textBox") {
                                    console.log("Now signal text box in page " + this.model.get("index") + " to adjust for a moving image");
                                    pageObjectModel.set("objectModelUpdate", {
                                        "key": "updateForHardwareAcceleration",
                                        "value": true,
                                        "salt": new Date().getTime()
                                    });
                                }
                            }
                        }
                    },
                    processPauseObjectsInPage: function (value) {
                        var pageObject, pageObjectModel;
                        if (value) {
                            this.model.set("paused", true);
                            for (pageObject in that.pageObjects) {
                                pageObjectModel = that.pageObjects[pageObject].getModel();
                                console.log("Now signal all objects in page " + this.model.get("index") + " to pause");
                                pageObjectModel.set("objectModelUpdate", {
                                    "key": "toggleObjectStatePause",
                                    "value": true,
                                    "salt": new Date().getTime()
                                });
                            }
                            this.pausePageZoomAnimations();
                        }
                    },
                    processResumeObjectsInPage: function (value) {
                        var pageObject, pageObjectModel;
                        if (value) {
                            this.model.set("paused", false);
                            for (pageObject in that.pageObjects) {
                                pageObjectModel = that.pageObjects[pageObject].getModel();
                                console.log("GYL-414: Now signal all objects in page " + this.model.get("index") + " to resume");
                                pageObjectModel.set("objectModelUpdate", {
                                    "key": "toggleObjectStateResume",
                                    "value": true,
                                    "salt": new Date().getTime()
                                });
                            }
                            this.resumePageZoomAnimations();
                        }
                    },
                    notifyEventObjectsThatExternalObjectIsActivated: function (activatedObject) {
                        var pageObject, pageObjectModel;
                        for (pageObject in that.pageObjects) {
                            pageObjectModel = that.pageObjects[pageObject].getModel();
                            if (pageObjectModel.get("type") == "event") {
                                pageObjectModel.set("externalObjectActivated", activatedObject);
                            }
                        }
                    },
                    activatedObject: function () {
                        var activatedObject = this.model.get("activatedObject");
                        this.pageObjectActivated(activatedObject);

                        if (null == activatedObject)
                            return;

                        var categoriesIdsForActivatedObject = activatedObject.activatedCategories.objectCategoryIds,
                            idx;
                        if (null != categoriesIdsForActivatedObject) {
                            for (idx = 0; idx < categoriesIdsForActivatedObject.length; idx++) {
                                var category = this.activatedObjectsInCategoriesForCurrentPage[categoriesIdsForActivatedObject[idx]];

                                if (category['numberOfActivatedObjects'] == null)
                                    category['numberOfActivatedObjects'] = 0;

                                if (category[activatedObject['id']] == false) {
                                    category[activatedObject['id']] = true;
                                    category['numberOfActivatedObjects']++;
                                }
                            }
                        }

                    },
                    pageObjectActivated: function (activatedObject) {
                        if (null != activatedObject) {
                            this.model.set("activatedObject", null);
                            this.processCategoriesForActivatedObject(activatedObject);
                            this.notifyEventObjectsThatExternalObjectIsActivated(activatedObject);
                        }
                    },

                    resetZoomHandler: function () {
                        if (this.model.get("resetZoom")) {
                            CSSHandler.removeBrowserSpecificProperty(this.pageContentHolder, 'transform');
                            CSSHandler.removeBrowserSpecificProperty(this.pageContentHolder, 'transition');
                            CSSHandler.removeBrowserSpecificProperty(this.pageContentHolder, 'transformOrigin');
                            this.model.set("resetZoom", false);
                        }
                    },

                    getZoomTransitionPlayer: function () {
                        if (!this.zoomTransitionPlayer) {
                            this.zoomTransitionPlayer = new TransitionPlayer(this.model, this.pageContentHolder, TransitionPlayerType.ZOOM);
                        }
                        return this.zoomTransitionPlayer;
                    },

                    pausePageZoomAnimations: function () {
                        // TODO: GYL-414, due to bugs in iOS 8 this feature can't be enabled at this point
                        //this.getZoomTransitionPlayer().handleCSSTransformPause();
                    },

                    resumePageZoomAnimations: function () {
                        // TODO: GYL-414, due to bugs in iOS 8 this feature can't be enabled at this point
                        //this.getZoomTransitionPlayer().handleCSSTransformResume();
                    },

                    pageObjectZoomedIn: function () {
                        var zoomedInObjectInformation = this.model.get("zoomedInObject"),
                            thisView;
                        if (null != zoomedInObjectInformation) {
                            this.model.set("zoomedInObject", null);
                            thisView = this;
                            console.log("GYL-414", "Now start zoom in at " + new Date().toISOString());
                            EffectsHandler.zoomInStartingPoint(this.pageContentHolder, thisView.model, zoomedInObjectInformation);
                        }

                    },

                    pageObjectZoomedOut: function () {

                        var zoomedOutObjectInformation = this.model.get("zoomedOutObject"),
                            thisView;
                        if (null != zoomedOutObjectInformation) {
                            this.model.set("zoomedOutObject", null);
                            thisView = this;
                            console.log("GYL-414", "Now start zoom out at " + new Date().toISOString());
                            EffectsHandler.zoomOutStartingPoint(this.pageContentHolder, thisView.model, zoomedOutObjectInformation);
                        }

                    },

                    processCategoriesForActivatedObject: function (activatedObject) {
                        var activatedCategories = {},
                            deactivatedCategories = {},
                            movedCategories = {},
                            zoomedCategories = {},
                            categories = {};

                        console.log("processCategoriesForActivatedObject");

                        this.extractActivatedCategoryFrom(activatedObject.activatedCategories.moveObjectCategoryIds, 'move', movedCategories, null);
                        this.extractActivatedCategoryFrom(activatedObject.activatedCategories.zoomObjectCategoryIds, 'zoom', zoomedCategories, null);

                        this.extractActivatedCategoryFrom(activatedObject.activatedCategories.addObjectCategoryIds, 'add', activatedCategories, deactivatedCategories);
                        this.extractActivatedCategoryFrom(activatedObject.activatedCategories.removeObjectCategoryIds, 'remove', activatedCategories, deactivatedCategories);
                        this.extractActivatedCategoryFrom(activatedObject.activatedCategories.toggleObjectCategoryIds, 'toggle', activatedCategories, deactivatedCategories);
                        this.extractActivatedCategoryFrom(activatedObject.activatedCategories.setObjectCategoryIds, 'set', activatedCategories, deactivatedCategories);

                        console.log("Will activate categories for page " + this.model.get("index") + " : " + JSON.stringify(activatedCategories));
                        console.log("Will deactivate categories for page " + this.model.get("index") + " : " + JSON.stringify(deactivatedCategories));

                        if (!this.isEmpty(activatedCategories) || !this.isEmpty(deactivatedCategories) || !this.isEmpty(movedCategories) || !this.isEmpty(zoomedCategories)) {
                            categories.activatedCategories = activatedCategories;
                            categories.deactivatedCategories = deactivatedCategories;
                            if (!this.isEmpty(movedCategories)) {
                                categories.movedCategories = movedCategories;
                                categories.moveByPropertiesValues = activatedObject.moveByPropertiesValues;
                            }
                            if (!this.isEmpty(zoomedCategories)) {
                                categories.zoomedCategories = zoomedCategories;
                                categories.zoomPropertiesValues = activatedObject.zoomPropertiesValues;
                            }
                            categories.callback = activatedObject.callback;
                            categories.globalCategoryAction = activatedObject.globalCategoryAction;
                            if (this.allObjectForActivatedCategoriesAreLoaded(activatedCategories)) {
                                Chapter.getScrollingElementsModel().set("toggledCategories", categories);
                            } else {
                                this.categoriesToBeProcessedQueue.push(categories);
                                if (this.categoriesToBeProcessedQueue.length == 1) {
                                    this.startLoadingObjectsForCategories(categories.activatedCategories);
                                }
                            }
                        }
                    },
                    startLoadingObjectsForCategories: function (categories) {
                        var category, pageCategory,
                            models = that.workingObjectsCollection.models,
                            model, idx, objId;
                        for (idx = 0; idx < models.length; idx++) {
                            model = models[idx];

                            for (category in categories) {
                                if (categories.hasOwnProperty(category)) {
                                    if (category in this.categoriesForCurrentPage) {
                                        pageCategory = this.categoriesForCurrentPage[category];
                                        objId = model.get('id');
                                        if (null != pageCategory[objId] && !pageCategory[objId]) {
                                            var objectView = that.pageObjects[model.get('id')].getView();
                                            if (null != objectView) {
                                                objectView.unbind();
                                                objectView.addListeners();
                                                model.set('loadObject', true);
                                            }
                                        }
                                    } else {
                                        console.log("Category not used in page " + this.model.get("id"));
                                    }
                                }
                            }
                        }
                    },
                    allObjectForActivatedCategoriesAreLoaded: function (categories) {
                        var category, loaded = true, pageCategory;

                        if (null != categories) {
                            for (category in categories) {
                                if (categories.hasOwnProperty(category) && this.categoriesForCurrentPage != null) {
                                    pageCategory = this.categoriesForCurrentPage[category];
                                    if (typeof pageCategory != "undefined" && pageCategory['numberOfLoadedObjects'] != pageCategory['totalNumberOfObjects']) {
                                        loaded = false;
                                        break;
                                    }
                                }
                            }
                        }
                        return loaded;
                    },
                    toggledCategories: function () {
                        var categories = this.model.get("toggledCategories"),
                            pageObject;
                        if (null != categories) {
                            this.updateObjectsByCategories(categories);
                            this.model.set("toggledCategories", null);
                            for (pageObject in that.pageObjects) {
                                if (that.pageObjects[pageObject].getModel().get('type') == 'embeddedScene') {
                                    that.pageObjects[pageObject].getModel().set('toggledCategories', categories);
                                }
                            }
                        }
                    },
                    closeLightBoxes: function () {
                        var closeLightBoxes = this.model.get("closeLightBoxes");
                        if (closeLightBoxes) {
                            if (null != this.model.get('parentModel')) {
                                this.model.get('parentModel').set('closeLightBox', true);
                            }
                            this.model.set('closeLightBoxes', false);
                        }
                    },
                    downloadComplete: function () {
                        var downloadComplete = this.model.get("downloadComplete"),
                            pageObject;
                        if (null != downloadComplete) {
                            if (null != this.model.get('parentModel')) {
                                this.model.get('parentModel').set('downloadComplete', downloadComplete);
                            }
                            for (pageObject in that.pageObjects) {
                                var type = that.pageObjects[pageObject].getModel().get('type');
                                if (type == 'event' || type == 'embeddedScene') {
                                    that.pageObjects[pageObject].getModel().set('downloadComplete', downloadComplete);
                                }
                            }
                            this.model.set('downloadComplete', null);
                        }
                    },
                    updateObjectsByCategories: function (categories) {
                        var pageObject, pageObjectModel, objectCategories, objectCategoryIds, shouldRemove, shouldAdd, idx, jdx;
                        for (pageObject in that.pageObjects) {
                            pageObjectModel = that.pageObjects[pageObject].getModel();
                            objectCategoryIds = pageObjectModel.get("objectCategoryIds");
                            objectCategories = pageObjectModel.get("objectCategoryIds");
                            if (null != objectCategories) {
                                for (idx = 0; idx < objectCategories.length; idx++) {
                                    if (categories.deactivatedCategories[objectCategories[idx]]) {
                                        shouldRemove = true;
                                        if (objectCategoryIds instanceof Array) {
                                            for (jdx = 0; jdx < objectCategoryIds.length; jdx++) {
                                                if (categories.activatedCategories[objectCategoryIds[jdx]]) {
                                                    shouldRemove = false;
                                                    break;
                                                }
                                            }
                                        }
                                        if (shouldRemove) {
                                            pageObjectModel.set("removed", true);
                                            pageObjectModel.set('loadObject', false);
                                        }
                                    }
                                }
                                for (idx = 0; idx < objectCategories.length; idx++) {
                                    if (categories.activatedCategories[objectCategories[idx]]) {
                                        shouldAdd = true;
                                        if (objectCategoryIds instanceof Array) {
                                            for (jdx = 0; jdx < objectCategoryIds.length; jdx++) {
                                                if (categories.deactivatedCategories[objectCategoryIds[jdx]]) {
                                                    shouldAdd = false;
                                                    break;
                                                }
                                            }
                                        }
                                        if (shouldAdd) {
                                            this.addNewObjectInPage(pageObject, pageObjectModel);
                                        }
                                    } else if (this.model.get('index') == Chapter.getActivePage() &&
                                        typeof categories.movedCategories != "undefined" &&
                                        categories.movedCategories.hasOwnProperty(objectCategories[idx]) &&
                                        categories.movedCategories[objectCategories[idx]]) {
                                        pageObjectModel.set("movedCategory", categories.moveByPropertiesValues)
                                    } else if (this.model.get('index') == Chapter.getActivePage() &&
                                        typeof categories.zoomedCategories != "undefined" &&
                                        categories.zoomedCategories.hasOwnProperty(objectCategories[idx]) &&
                                        categories.zoomedCategories[objectCategories[idx]]) {
                                        pageObjectModel.set("zoomedCategory", categories.zoomPropertiesValues)
                                    }
                                }
                            }
                        }
                    },
                    addNewObjectInPage: function (objectId, pageObjectModel) {
                        var objectView = that.pageObjects[objectId].getView(), renderedObject;
                        pageObjectModel.set("removed", false); // make sure we are prepared for a new signal change
                        objectView.unbind();
                        objectView.addListeners();
                        renderedObject = objectView.render();
                        if (renderedObject != null) {
                            this.pageContentHolder.append(renderedObject.el);
                        }
                        if (this.model.get('index') == Chapter.getActivePage()) {
                            objectView.categoryActivated();
                        }
                    },
                    extractActivatedCategoryFrom: function (OCIds, action, activatedCategories, deactivatedCategories) {
                        var ocIdsIndex, categories = Chapter.getObjectCategories(),
                            changedCategory;
                        if (null != OCIds) {
                            if (action == 'set') {
                                for (ocIdsIndex = 0; ocIdsIndex < OCIds.length; ocIdsIndex++) {
                                    if (categories.hasOwnProperty(OCIds[ocIdsIndex])) {
                                        this.setCurrentCategoryAndUnsetTheRest(OCIds[ocIdsIndex], activatedCategories, deactivatedCategories);
                                    }
                                }
                            } else if (action == 'add') {
                                for (ocIdsIndex = 0; ocIdsIndex < OCIds.length; ocIdsIndex++) {
                                    changedCategory = OCIds[ocIdsIndex];
                                    if (!categories[changedCategory].active) {
                                        categories[changedCategory].active = true;
                                        activatedCategories[changedCategory] = true;
                                    }
                                }
                            } else if (action == 'remove') {
                                for (ocIdsIndex = 0; ocIdsIndex < OCIds.length; ocIdsIndex++) {
                                    changedCategory = OCIds[ocIdsIndex];
                                    if (categories[changedCategory].active) {
                                        categories[changedCategory].active = false;
                                        deactivatedCategories[changedCategory] = true;
                                    }
                                }
                            } else if (action == 'toggle') {
                                for (ocIdsIndex = 0; ocIdsIndex < OCIds.length; ocIdsIndex++) {
                                    changedCategory = OCIds[ocIdsIndex];
                                    if (!categories[changedCategory].active) {
                                        categories[changedCategory].active = true;
                                        activatedCategories[changedCategory] = true;
                                    } else {
                                        categories[changedCategory].active = false;
                                        deactivatedCategories[changedCategory] = true;
                                    }
                                }
                            } else if (action == 'move' || action == "zoom") {
                                for (ocIdsIndex = 0; ocIdsIndex < OCIds.length; ocIdsIndex++) {
                                    activatedCategories[OCIds[ocIdsIndex]] = true;
                                }
                            }
                        }
                    },

                    playRecordedSounds: function () {
                        var activePage = this.model.get('index');
                        require(["SoundRecorder"], function () {
                            var globalSoundsSettings = Chapter.getGlobalSoundsSetting();
                            if (Chapter.getSettings()["playRecordedWizard"] && SoundRecorder.getRecordedWizardSoundAvailableForPage(activePage)) {
                                console.log("Will play recorded wizard sound");
                                SoundRecorder.startPlayRecordedWizardSoundForPage(activePage);
                            } else if (Chapter.getSettings()["alwaysPlayRecordedSoundsWithAutoStart"] &&
                                SoundRecorder.getRecordedSoundAvailableForPage(activePage) &&
                                globalSoundsSettings) {  // this is SCH-121, might need to be changed if not using globalSoundsSettings
                                SoundRecorder.playRecordedSoundForPage(activePage);
                            }
                        });
                    },

                    stopRecordedWizardSound: function () {
                        var activePage = this.model.get('index');
                        if (Chapter.getSettings()["playRecordedWizard"] && SoundRecorder.getRecordedWizardSoundAvailableForPage(activePage)) {
                            console.log("Will stop recorded wizard sound");
                            SoundRecorder.stopPlayRecordedWizardSoundForPage(activePage);
                        }
                    },

                    notifyAllPageObjectOfZoomChange: function (zoomFactor) {
                        console.log("notifyAllPageObjectOfZoomChange " + zoomFactor);
                        var idx, model;
                        for (idx = 0; idx < that.workingObjectsCollection.length; idx++) {
                            model = that.workingObjectsCollection.models[idx];
                            model.set("zoomFactor", zoomFactor);
                        }
                    },

                    getZoomView: function () {
                        var zoomView = Chapter.getScrollingElementsView().getZoomElementForPage(this.model.get("index"));
                        var wrapperId = zoomView.model.get("wrapperNo"),
                            wrapperMessage = typeof wrapperId != "undefined" ? " and dual page wrapper " + wrapperId : "";
                        if (!zoomView.getZoomInstance()) {
                            console.log("MOB-1601: add new IScroll for page " + this.model.get("index") + wrapperMessage);
                            return zoomView;
                        }
                        console.log("MOB-1601: skip new IScroll for page " + +this.model.get("index") + wrapperMessage);
                        return null;
                    },

                    addZoomOnDemand: function () {
                        if (Chapter.getSettings()["zoomEnabled"]) {
                            var zoomView = this.getZoomView();
                            ZoomUtils.updateDummyDivForActivePageOnOrientationChange(this.model.get("index"));
                            if (zoomView) {
                                // will add zoom for zoomView.$el.attr("id")
                                zoomView.setZoomInstance(true); // set this to true so that further calls won't add two instances
                                require(["iScrollZoomLib", "IScrollZoom"], function (undefined, IScrollZoom) {
                                    console.log("add/remove zoom: add called for " + zoomView.$el.attr("id") + " left is " + zoomView.model.get("left"));
                                    var zoomDOMElement = zoomView.el,
                                        zoomOptions = {
                                            zoomMax: 3,
                                            zoomMin: 1,
                                            translateOffset: zoomView.model.get("left")
                                        },
                                        zoomInstance = new IScrollZoom(zoomOptions);
                                    zoomView.setZoomInstance(zoomInstance);
                                    zoomInstance.initScroll(zoomDOMElement);
                                    zoomInstance.initDoubleTapZoom(zoomView.$el);
                                });
                            }
                        }
                    },

                    updateZoomValue: function () {
                        var zoomView = Chapter.getScrollingElementsView().getZoomElementForPage(this.model.get("index")),
                            zoomInstance = zoomView.getZoomInstance();
                        zoomInstance.doZoom(this.model.get("zoomValue"), undefined, undefined, 0);
                    },

                    removeZoomHandler: function () {
                        if (this.model.get("removeZoom")) {
                            if (this.zoomInstance != null) {
                                console.log("add/remove zoom: remove called for page " + this.model.get("index"));
                                this.$el.css("transform", "");
                                this.$el.css("transform-origin", "");
                                CSSHandler.removeBrowserSpecificProperty(this.$el, 'transition', null);
                                this.zoomInstance.destroy();
                                delete this.zoomInstance;
                            } else {
                                console.log("add/remove zoom: zoom instance is null for page " + this.model.get("index"));
                            }
                            this.model.set("removeZoom", false);
                        }
                    },

                    activatePage: function () {
                        var idx,
                            activatePage = this.model.get("activatePage"),
                            model,
                            moveByOffsetProperty = Chapter.getSettings()['moveBackgroundObjectBy'],
                            pageNumber = this.model.get("index");
                        if (this.isEmpty(moveByOffsetProperty)) {
                            moveByOffsetProperty = null;
                        }

                        if (activatePage) {
                            console.log("Now activate page " + pageNumber);
                            if (pageNumber == ProjectGlobals.START_PAGE && AppUtils.nativeApp && typeof navigator.splashscreen != "undefined") {
                                navigator.splashscreen.hide();
                            }

                            if (this.model.get('hideLeftSideMenuArrow') || (this.model.get('index') == 0 && Chapter.getSettings().hideNavigationOnFirstPage)) {
                                ViewportPadding.hideLeftPadding();
                            } else {
                                ViewportPadding.showLeftPadding();
                            }
                            if (this.model.get('hideRightSideMenuArrow') || (this.model.get('index') == 0 && Chapter.getSettings().hideNavigationOnFirstPage)) {
                                ViewportPadding.hideRightPadding();
                            } else {
                                ViewportPadding.showRightPadding();
                            }
                            Chapter.setActivePageElement(this);
                            this.addZoomOnDemand();
                            this.playRecordedSounds();
                            console.log("Should activate all objects within page . We have " + that.originalObjectCollection.length + " objects");
                            window.sceneActivationTime = new Date().getTime();
                            for (idx = 0; idx < that.originalObjectCollection.length; idx++) {
                                model = that.workingObjectsCollection.models[idx];
                                model.set("startAutosStartEffects", true);
                                model.set("enableObject", true);
                                if (this.objectShouldBeActivatedFromStart(model)) {
                                    model.set("activateObject", activatePage);
                                }
                                if (model.get("isFullSizeBackground")) {
                                    model.set("moveByOffset")
                                }
                                if (null != moveByOffsetProperty && null == model.get("objectCategoryIds")) {
                                    model.set("moveByProjectOffset", moveByOffsetProperty);
                                }
                            }
                            AppUtils.notifyParentThatPlayerIsLoaded();
                        } else {
                            this.model.set('deactivatePage', true);
//                            ZoomUtils.resetZoom(); // better to reset zoom here but will cause some problems -> current page that is zoomed overlapping neighbour page
                        }
                    },
                    deactivatePage: function () {
                        var callback = this.model.get('deactivatePage');
                        if (null != callback) {
                            this.model.attributes['activatePage'] = false;
                            this.model.set('deactivatePage', null);
                            console.log("Now de-activate page " + this.model.get("index"));
                            this.zoomTransitionPlayer = null;
                            this.stopRecordedWizardSound();
                            this.restoreObjectsModels();
                            this.restorePage();
                            if (typeof callback == 'function') {
                                callback();
                            }
                        }
                    },
                    allObjectsLoaded: function () {
                        if (this.model.get("shouldBeActive") && this.model.get("allObjectsLoaded") && this.model.get("allResourcesLoaded")) {
                            this.model.set("activatePage", true);
                        }
                    },

                    objectShouldBeActivatedFromStart: function (model) {
                        return (
                            this.isPartOfAnActiveLayer(model.get("objectCategoryIds")) &&
                            (model.get("autoStart") || model.get("zoomInStartingPoint") || model.get("zoomOutStartingPoint"))
                            ) || model.get("globalTimedEventItem");
                    },
                    isPartOfAnActiveLayer: function (objectsLayersIds) {
                        var layers = Chapter.getObjectCategories(),
                            index, ret = (objectsLayersIds == null), // if object is not in any category is like it is active !!!
                            layer, id;
                        if (null != objectsLayersIds) {
                            for (index = 0; index < objectsLayersIds.length; index++) {
                                id = objectsLayersIds[index];
                                layer = layers[id];
                                if (null != layer) {
                                    ret = layer.active || ret;
                                }
                            }
                        }
                        return ret;
                    },
                    setCurrentCategoryAndUnsetTheRest: function (setCategoryId, activatedCategories, deactivatedCategories) {
                        var categories = Chapter.getObjectCategories(),
                            setCategory = categories[setCategoryId],
                            complementaryCategories = this.getCategoryFromTheSameClass(setCategory),
                            category, idx;
                        if (complementaryCategories != null) {
                            for (idx = 0; idx < complementaryCategories.length; idx++) {
                                category = complementaryCategories [idx];
                                if (categories.hasOwnProperty(category)) {
                                    if (categories[category].active) {
                                        categories[category].active = false;
                                    }
                                    deactivatedCategories[category] = true;
                                }
                            }
                        }
                        if (!setCategory.active) {
                            setCategory.active = true;
                            activatedCategories[setCategoryId] = true;
                        }
                    },
                    getCategoryFromTheSameClass: function (setCategory) {
                        var ret = setCategory.categoriesInClass.split(','), idx;
                        for (idx = 0; idx < ret.length; idx++) {
                            if (ret[idx] == '')
                                ret.splice(idx, 1);
                        }
                        return ret.length == 0 ? null : ret;
                    },
                    shouldBeExcludedFromDom: function (domObject) {
                        return (null == domExcludedElements[domObject.getModel().get('type')]) ? false : domExcludedElements[domObject.getModel().get('type')];
                    },
                    isEmpty: function (obj) {

                        // null and undefined are "empty"
                        if (obj == null) return true;

                        // Assume if it has a length property with a non-zero value
                        // that that property is correct.
                        if (obj.length > 0)    return false;
                        if (obj.length === 0)  return true;

                        // Otherwise, does it have any properties of its own?
                        // Note that this doesn't handle
                        // toString and toValue enumeration bugs in IE < 9
                        for (var key in obj) {
                            if (obj.hasOwnProperty(key)) return false;
                        }

                        return true;
                    },
                    restorePage: function () {
                        if (AppUtils.DeviceType.CHROME) {
                            this.$el.find(".chromeFadeBug").remove();
                        }
                        this.model.set("usingScrollableWithinPage", false);
                    },
                    restoreObjectsModels: function () {
                        var models = that.originalObjectCollection.models,
                            idx,
                            originalModelAttributes,
                            workingModel,
                            originalModel,
                            attribute;
                        that.shouldBeActive = false;
                        for (pageObject in that.pageObjects) {
                            that.pageObjects[pageObject].getModel().set('pageDeactivated', true);
                        }
                        for (idx = 0; idx < models.length; idx++) {
                            originalModel = models[idx];
                            originalModelAttributes = originalModel.attributes;
                            workingModel = that.workingObjectsCollection.models[idx];
                            for (attribute in originalModelAttributes) {
                                if (originalModelAttributes.hasOwnProperty(attribute)) {
                                    workingModel.set(attribute, originalModel.get(attribute));
                                }
                            }
                            this.restoreCustomAttributesForObjectModel(workingModel);
                        }

                        this.restoreObjectCategories();

                        for (var pageObject in that.pageObjects) {
                            var pageObjectModel = that.pageObjects[pageObject].getModel();
                            if (this.isPartOfAnActiveLayer(pageObjectModel.get("objectCategoryIds"))) {
                                pageObjectModel.set("restoreState", true);
                            }
                        }
                    },
                    restoreCustomAttributesForObjectModel: function (objectModel) {
                        objectModel.set("activateEffect", false);
                        objectModel.set("disableObject", true);
                    },
                    restoreObjectCategories: function () {
                        var categories = Chapter.getObjectCategories(),
                            category, changedCategoriesObject = {},
                            activatedCategories = {};
                        activatedCategories.addObjectCategoryIds = [];
                        activatedCategories.removeObjectCategoryIds = [];

                        //remove all categories that needed to load for current page
                        this.categoriesToBeProcessedQueue = [];

                        for (category in this.chapterObjectCategories) {
                            if (this.chapterObjectCategories.hasOwnProperty(category) && categories[category].sceneLevel) {
                                if (categories[category].active != this.chapterObjectCategories[category].active) {
                                    if (categories[category].active) {
                                        activatedCategories.removeObjectCategoryIds.push(category);
                                    } else {
                                        activatedCategories.addObjectCategoryIds.push(category);
                                    }
                                }
                            }
                        }
                        if ((Math.abs(this.model.get('index') - Chapter.getActivePage()) != 1) && !Chapter.isAppPaused()) {
                            activatedCategories.addObjectCategoryIds = null;
                        }
                        changedCategoriesObject.activatedCategories = activatedCategories;
                        this.processCategoriesForActivatedObject(changedCategoriesObject);
                        for (category in this.chapterObjectCategories) {
                            if (this.chapterObjectCategories.hasOwnProperty(category) && categories[category].sceneLevel) {
                                categories[category].active = this.chapterObjectCategories[category].active;
                            }
                        }
                    }
                }
            );
        };
        this.setTotalNumberOfObjects = function (objectsNumber, pageObjects) {
            for (var idx = 0; idx < objectsNumber; idx++) {
                pageObjects[idx].properties["objectActivate"] = false;
                if (typeof elementsTypeMapping[pageObjects[idx].properties.type] != "undefined" &&
                    elementsTypeMapping[pageObjects[idx].properties.type]) {
                    totalNumberOfObjects++;
                }
            }
        };
        this.instantiateObjects = function (callback) {
            var pageObjects = pageModel.get("objects"),
                objectsNumber = pageObjects.length, idx;
            that.pageView = new that.PageView({
                model: pageModel
            });
            that.pageObjects = {};
            that.originalObjectCollection = new Backbone.Collection();
            that.workingObjectsCollection = new Backbone.Collection();
            pageModel.set('workingObjectsCollection', that.workingObjectsCollection);
            if (objectsNumber == 0) {
                if (null != callback)
                    callback();
            } else {
                this.setTotalNumberOfObjects(objectsNumber, pageObjects);
                this.setInitialActiveObjectsNumber(objectsNumber, pageObjects);
                for (idx = 0; idx < objectsNumber; idx++) {
                    pageObjects[idx].properties["objectActivate"] = false;
                    if (typeof elementsTypeMapping[pageObjects[idx].properties.type] != "undefined") {
                        var pageObject = PersistentSceneObjectsUtils.getPersistentObject(pageObjects[idx].properties.id);
                        if (null != pageObject) {
                            that.workingObjectsCollection.add(pageObject.getModel());
                            var temp = new Backbone.Model(pageObject.getModel().attributes);
                            temp.set("activateObject", false);
                            that.originalObjectCollection.add(temp);
                            that.pageObjects[pageObject.getModel().get("id")] = pageObject;
                            numberOfCreatedObjects++;
                            if (totalNumberOfObjects == numberOfCreatedObjects) {
                                console.log("that.pageObjects.size = " + Object.keys(that.pageObjects).length);
                                console.log("executeCallback for page " + that.pageView.model.get("index"));
                                pageModel.set("allObjectsLoaded", true);
                                if (null != callback) {
                                    that.addDragHandlers();
                                    callback();
                                }
                            }
                            pageModel.set("objectLoaded", pageObject.getView().getLoadedInfo(true));
                        } else {
                            this.createObject(elementsTypeMapping[pageObjects[idx].properties.type],
                                getAdditionalLibrariesForObjectType(pageObjects[idx].properties.type),
                                pageObjects[idx], callback);
                        }
                    } else {
                        console.log("Object of type " + pageObjects[idx].properties.type + " is not supported in new player yet");
                    }
                }
            }

        };

        this.setInitialActiveObjectsNumber = function (objectsNumber, pageObjects) {
            var categoriesForObject,
                objIdx, catIdx, allCategories = Chapter.getObjectCategories(),
                isActive = true, ret = [];
            for (objIdx = 0; objIdx < objectsNumber; objIdx++) {
                categoriesForObject = this.getCategoriesFor(pageObjects[objIdx].properties['objectCategoryIds']);
                initialActiveObjects++;
                for (catIdx = 0; catIdx < categoriesForObject.length; catIdx++) {
                    if (!allCategories[categoriesForObject[catIdx]].active) {
                        isActive = false;
                        initialActiveObjects--;
                        break;
                    }
                }
            }
            return ret;
        };

        this.getCategoriesFor = function (ocIdsAsCommaSeparatedString) {
            var ret = ocIdsAsCommaSeparatedString.split(','), idx;
            for (idx = 0; idx < ret.length; idx++) {
                if ((ret[idx] == '') || (null == that.pageView.categoriesForCurrentPage) || (!that.pageView.categoriesForCurrentPage[ret]))
                    ret.splice(idx, 1);
            }
            return ret;

        };
        this.updateVersionedChapters = function () {
            if (AppUtils.nativeApp) {
                require(['UpdateService'], function (UpdateService) {
                    var updateService = new UpdateService();
                    updateService.updatePricesForIAPPricePlaceHolders(that.workingObjectsCollection.models);
                    updateService.updateVersionedProjects(that.workingObjectsCollection.models);
                });
            }
        };
        this.createObject = function (objectType, additionalLibrariesToLoad, objectModel, callback) {
            var temp;

            require(additionalLibrariesToLoad, function () {
                require([objectType, 'ElementsBaseView', 'Templates'], function (pageElement) {
                    if (that.pageView.model.get("karaokeModeEnabled")) {
                        objectModel.properties["karaokeModeEnabled"] = true;
                    }
                    var pageObject = new pageElement(objectModel, that.pageView.model);
                    if (null != pageObject) {
                        pageObject.init();
                        that.workingObjectsCollection.add(pageObject.getModel());
                        temp = new Backbone.Model(pageObject.getModel().attributes);
                        temp.set("activateObject", false);
                        that.originalObjectCollection.add(temp);
                        that.pageObjects[pageObject.getModel().get("id")] = pageObject;
                        PersistentSceneObjectsUtils.addNewPersistentObject(pageObject);
                    }
                    numberOfCreatedObjects++;
                    if (totalNumberOfObjects == numberOfCreatedObjects) {
                        console.log("that.pageObjects.size = " + Object.keys(that.pageObjects).length);
                        console.log("executeCallback for page " + that.pageView.model.get("index"));
                        pageModel.set("allObjectsLoaded", true);
                        that.updateVersionedChapters();
                        if (null != callback) {
                            that.addDragHandlers();
                            callback();
                        }
                    }
                })
            });
        };
        this.addDragHandlers = function () {
            var model = this.pageView.model;
            if (model.get('isFirstPage') || model.get('isLastPage')) {
                require(['hammerJS'], function (Hammer) {
                    Hammer(that.pageView.el).on("dragend", function (event) {
                        if (model.get('isFirstPage') && event.gesture.direction == 'right') {
                            that.openMenuAndActivateChapterNavigationArrow(model);
                        } else if (model.get('isLastPage') && event.gesture.direction == 'left') {
                            that.openMenuAndActivateChapterNavigationArrow(model);
                        }
                    })
                });
            }
        };

        this.openMenuAndActivateChapterNavigationArrow = function (model) {
            var bookId = Chapter.getSettings().bookId,
                bookInfo = localStorage.getItem('bookInfo_' + bookId),
                currentProjectIndex = localStorage.getItem('currentProjectIndex'),
                bookChapters, previousChapter, nextChapter, neighbourChapterInfo = {};

            if (null != bookInfo) {
                currentProjectIndex = parseInt(currentProjectIndex);
                bookInfo = JSON.parse(bookInfo);
                bookChapters = bookInfo.chapters;
                previousChapter = bookChapters[currentProjectIndex - 1];
                nextChapter = bookChapters[currentProjectIndex + 1];
            }

            if (model.get('isFirstPage')) {
                if (null != previousChapter) {
                    if (previousChapter.downloaded) {
                        var activePage = previousChapter.lastPageNumber - previousChapter.firstPageNumber;
                        if (activePage < 0) {
                            activePage = 0;
                        }
                        neighbourChapterInfo.direction = -1;
                        Chapter.getBottomMenu().getModel().set('moveToNeighbourChapter', neighbourChapterInfo);
                    } else {
                    }
                } else {
                }
            } else {
                if (null != nextChapter) {
                    if (nextChapter.downloaded) {
                        neighbourChapterInfo.direction = 1;
                        Chapter.getBottomMenu().getModel().set('moveToNeighbourChapter', neighbourChapterInfo);
                    } else {
                    }
                } else {
                }
            }
        };

        return {
            init: function (callback) {
                that.defineClasses();
                that.instantiateObjects(callback);
            },
            getView: function () {
                return that.pageView;
            },
            getModel: function () {
                return that.pageView.model;
            }
        }
    }
})
;