define(function () {
    return function (rawPdfPackageModel, pageModel) {
        var that = this;

        this.defineClasses = function () {

            that.PdfPackageView = ElementBaseView.extend({
                    tagName: "div",
                    className: "sceneObject",
                    elementsViews: [],
                    template: templates.getTemplate("pdfPackage"),
                    id: rawPdfPackageModel.properties.id,
                    parentElementModel: pageModel,
                    maxAttemptsNumber: 30,
                    reorderedText: false,
                    attemptsNumber: 0,

                    customInitialize: function () {
                        this.model.set("allObjectTypesInPage", rawPdfPackageModel.customProperties["allObjectTypesInPage"]);
                    },
                    events: function () {
                        var _events = {};
                        // likely we don't want to activate any other object or do a custom action when we interact with a PDF page ???
//                    _events[AppUtils.EventTypes.mouseDown] = "touchstart";
//                    _events[AppUtils.EventTypes.click] = "clickHandler";
                        return _events;
                    },
                    addCustomListeners: function () {
                        this.listenTo(this.model, "change:orientation", this.updateOrientation);
                    },
                    updateOrientation: function () {
                        this.updateScaleFactor();
                    },
                    removeView: function () {
                        if (this.model.get("removed")) {
                            this.remove();
                            this.unbind();
                        }
                    },
                    activateObject: function () {
                        if (this.model.get("activateObject")) {
                            this.model.set("objectActivated", true);
                            this.model.set("activateObject", false);
                        }
                    },
                    activateToggle: function () {
                        this.displayHideOnceOnClick();
                    },
                    shouldRenderObject: function () {
                        return true;
                    },
                    loadPackageDivs: function () {
                        var startFileInfo = rawPdfPackageModel.customProperties["startFileInfo"],
                            loadHtmlSrc = ResourceManager.getPathForPDFPackage(startFileInfo["objectId"], startFileInfo["startFileName"]),
                            thisView = this, attemptsNumber = 0, index, idx, shouldAdjust;
                        $.get(loadHtmlSrc, function (htmlElement) {
                            idx = thisView.getJSFunctionForHTMLFile();
                            var $htmlElement = $(htmlElement);
                            thisView.$el.html($htmlElement);
                            var elements = [], $element, elementsProperties = window['f' + idx], repeatedCheckTimer, widths = [];
                            if (null == elementsProperties) {
                                elementsProperties = [];
                            }
                            for (index = 0; index < elementsProperties.length; index++) {
                                $element = $('#' + elementsProperties[index][0], $htmlElement);
                                if (null != $element) {
                                    widths.push($element.width());
                                    elements.push($element[0]);
                                } else {
                                    elements.push(null);
                                    widths.push(0);
                                }
                            }
                            thisView.reorderedText = true;
                            PDFUtils.adjustWordSpacing(elementsProperties, elements, function () {
                                thisView.reorderedText = false;
                            });
                            repeatedCheckTimer = window.setInterval(function () {
                                shouldAdjust = false;
                                if (!thisView.reorderedText) {
                                    if (attemptsNumber == thisView.maxAttemptsNumber) {
                                        window.clearInterval(repeatedCheckTimer);
                                    }
                                    for (index = 0; index < elements.length; index++) {
                                        if (null != elements[index]) {
                                            if (elements[index].offsetWidth != widths[index]) {
                                                shouldAdjust = true;
                                                widths[index] = elements[index].offsetWidth;
                                            }
                                        }
                                    }
                                    if (shouldAdjust) {
                                        PDFUtils.adjustWordSpacing(elementsProperties, elements, function () {
                                            thisView.reorderedText = false;
                                        });
                                    } else {
                                        attemptsNumber++;
                                    }
                                }
                            }, 50);
                            thisView.$jpedal = $(".jpedal", this.el);
                            thisView.updateScaleFactor();
                        });


                    },
                    getJSFunctionForHTMLFile: function () {
                        return rawPdfPackageModel['customProperties']['startFileInfo']['objectId'];
                    },
                    updateScaleFactor: function () {
                        console.log("MOB-1718: updateScaleFactor app height is " + App.settings.height);
                        var scaleFactor = parseFloat((App.settings.height / rawPdfPackageModel.customProperties["originalPdfPackageHeight"]).toFixed(3));
                        if (scaleFactor != 1) { // might happen to be 1 when running on device and project was created at the same resolution
                            CSSHandler.scale(this.$jpedal, scaleFactor);
                            this.$jpedal.css("transform", "translateY(-50%) translateX(-50%) scale(" + scaleFactor + ")");
                            //this.$jpedal.css("zoom", scaleFactor); // same as the above, to test UX on device
                        } else {
                            this.$jpedal.css("transform", "translateY(-50%) translateX(-50%)");
                        }
                    },
                    render: function () {
                        this.loadPackageDivs();
                        this.renderStartHandler();
                        this.setLeft(this.model.get("left"));
                        this.setWidth(this.model.get("width"));
                        this.setHeight(this.model.get("height"));
                        this.setTop(this.model.get("top"));
                        return this;
                    }
                }
            )
            ;
        };
        this.instantiateObjects = function () {
            this.pdfPackageModel = this.getPdfPackageModel();
            this.pdfPackageView = new this.PdfPackageView({
                model: that.pdfPackageModel
            });

        };

        this.getPdfPackageModel = function () {
            return new Backbone.Model(rawPdfPackageModel.properties);
        };

        return {
            init: function () {
                that.defineClasses();
                that.instantiateObjects();
            },
            getView: function () {
                return that.pdfPackageView;
            },
            getModel: function () {
                return that.pdfPackageModel;
            }
        }
    }
})
;