define(function () {
        return function (rawSoundModel, pageModel) {
            var that = this,
                COMPACT_PLAYER_SIZE = 128;

            this.defineClasses = function () {

                that.playingState = SoundState.NOT_LOADED;
                that.soundInfo = null;

                that.SoundView = ElementBaseView.extend({
                    tagName: "div",
                    className: "sceneObject",
                    elementsViews: [],
                    template: templates.getTemplate("image"),
                    compactPlayerTemplate: templates.getTemplate("compactPlayer"),
                    id: rawSoundModel.properties.id,
                    parentElementModel: pageModel,
                    soundUIHasBeenInitialized: false,
                    timeElapsedElement: null,
                    canUpdatePlayProgress: true,

                    customInitialize: function () {
                        this.model.set("allObjectTypesInPage", rawSoundModel.customProperties["allObjectTypesInPage"]);
                    },
                    removeView: function () {
                        if (this.model.get("removed") && (!this.model.get('availableInAllOrientations') || !App.changedOrientation)) {
                            this.doBeforeViewRemove();
                            this.unbind();
                            this.remove();
                        }
                        if (that.playingState == SoundState.PLAYING) {
                            that.endPlaySound();
                        }
                    },
                    doBeforeViewRemove: function () {
                        this.model.set("externalObjectActivated", null);
                        PersistentSceneObjectsUtils.removePersistentObject(this.model.get('id'));
                    },
                    events: function () {
                        var _events = {};
                        _events[AppUtils.EventTypes.mouseDown] = "touchstart";
//                        _events[AppUtils.EventTypes.click] = "clickHandler";
                        return _events;
                    },
                    touchStartAction: function () {
                        if (that.playingState == SoundState.ENDED || that.playingState == SoundState.PAUSED || that.playingState == SoundState.NOT_LOADED) {
                            that.startPlaySound(null, true);
                        } else if (that.playingState == SoundState.PLAYING) {
                            that.endPlaySound();
                        }
                    },
                    touchstart: function (event) {
                        // OBS: on touchstart on a visible sound element we don't close menu but play sound
                        if (typeof event != "undefined") {
                            event.stopPropagation();
                            if (event.button != 2) { // discard right click used for debugging
                                this.touchStartAction();
                            }
                        } else {
                            this.touchStartAction();
                        }

                    },
                    shouldRenderObject: function () {
                        return this.currentObjectIsPartOfAnActiveCategory();
                    },
                    notifyParentThatObjectIsLoaded: function () {
                        this.parentElementModel.set("objectLoaded", this.getLoadedInfo(false));
                    },
                    loadObject: function () {
                        console.log('LOADED OBJECT: ' + this.model.get('id'));
                        that.startPlaySound(null, false);
                    },
                    getSoundPlayerUIType: function () {
                        // TODO: check here how is this defined at sound level
                        // if default go take the value from project level
                        return Chapter.getSettings()["soundUI"];
                    },
                    initPlayProgress: function (duration) {
                        var uiType = this.getSoundPlayerUIType(),
                            thisView = this,
                            seekNewPositionTimeout;

                        this.timeElapsedElement = this.$el.find(".compactPlayer.timeElapsed");
                        if (uiType == SOUND_UI_SETTINGS.COMPACT_PLAYER) {
                            require(["JqueryKnob"], function () {
                                thisView.timeElapsedKnob = thisView.$el.find(".compactPlayer.timeElapsed").knob({
                                    "readOnly": false,
                                    "min": 0,
                                    "max": duration,
                                    "width": 128, // here the value needs to be in pixels
                                    "height": 128, // here the value needs to be in pixels
                                    "thickness": 0.25,
                                    "top": 0,
                                    "left": 0,
                                    "position": "absolute",
                                    "change": function (value) {
                                        thisView.canUpdatePlayProgress = false;
                                        clearTimeout(seekNewPositionTimeout);
                                        seekNewPositionTimeout = setTimeout(function () {
                                            thisView.seekToNewPositionInSound(value * 1000);
                                        }, 1000);
                                        console.log("Updated value to " + value);
                                    }
                                });
                            });
                        } else if (uiType == SOUND_UI_SETTINGS.FULL_PLAYER) {

                        }
                    },
                    seekToNewPositionInSound: function (position) {
                        console.log(LOG_TAGS.MOB_2206, "Now seek to new position " + position);
                        that.setPosition(position);
                        this.canUpdatePlayProgress = true;
                    },
                    updateUIPlayerState: function () {
                        console.log(LOG_TAGS.MOB_2206, "Now updateUIPlayerState for new state " + that.playingState);
                        var uiType = this.getSoundPlayerUIType();
                        if (uiType == SOUND_UI_SETTINGS.COMPACT_PLAYER) {
                            if (that.playingState == SoundState.PLAYING) {
                                this.$el.find(".playButton").hide();
                                this.$el.find(".pauseButton").show();
                            } else if (that.playingState == SoundState.PAUSED || that.playingState == SoundState.ENDED) {
                                this.$el.find(".playButton").show();
                                this.$el.find(".pauseButton").hide();
                                if (that.playingState == SoundState.PAUSED) {

                                } else if (that.playingState == SoundState.ENDED) {
                                    this.soundUIHasBeenInitialized = false;
                                    if (this.timeElapsedElement) { // can be null for players not started
                                        this.timeElapsedElement.val(0).trigger('change');
                                    }
                                }
                            } else if (that.playingState == SoundState.SUSPENDED) {
                                this.$el.find(".playButton").show();
                                this.$el.find(".pauseButton").hide();
                            }
                        } else if (uiType == SOUND_UI_SETTINGS.FULL_PLAYER) {
                            alert("Not implemented");
                        }
                    },
                    updateUIPlayerForPlayPosition: function (position, duration) {
                        //console.log(LOG_TAGS.MOB_2206, "Now update for new position " + position);
                        var animationDurationInSeconds = Math.floor(duration / 1000),
                            currentSoundPosition = Math.round(position / 1000);
                        if (!this.soundUIHasBeenInitialized) {
                            this.initPlayProgress(animationDurationInSeconds);
                            this.soundUIHasBeenInitialized = true;
                        }
                        if (currentSoundPosition <= animationDurationInSeconds && this.canUpdatePlayProgress) {
                            this.timeElapsedElement.val(currentSoundPosition).trigger('change');
                        }
                    },
                    getEventsForCompactPlayer: function () {
                        var eventObj = {};
                        eventObj["click .compactPlayerButtons.playButton"] = "resumeSoundPlay";
                        eventObj["click .compactPlayerButtons.pauseButton"] = "pauseSound";
                        return eventObj;
                    },
                    resumeSoundPlay: function () {
                        if (that.playingState == SoundState.PAUSED) {
                            console.log(LOG_TAGS.MOB_2206, "resume play sound");
                            that.resumeSound(null, true);
                        } else {
                            console.log(LOG_TAGS.MOB_2206, "start play sound");
                            that.startPlaySound(null, true);
                        }
                    },
                    pauseSound: function () {
                        console.log(LOG_TAGS.MOB_2206, "pauseSound");
                        that.pauseSound();
                    },
                    renderCompactPlayer: function () {
                        this.$el.width(COMPACT_PLAYER_SIZE).height(COMPACT_PLAYER_SIZE);
                        this.setLeft(this.model.get("left"));
                        this.setTop(this.model.get("top"));
                        this.$el.css("z-index", this.model.get("ZIndex"));
                        this.delegateEvents(this.getEventsForCompactPlayer());
                        return this.compactPlayerTemplate();
                    },
                    render: function () {
                        var soundIconInfo,
                            uiType = this.getSoundPlayerUIType();
                        if (this.shouldRenderObject()) {
                            this.renderStartHandler();
                            if (uiType != SOUND_UI_SETTINGS.NO_PLAYER) {
                                if (uiType == SOUND_UI_SETTINGS.COMPACT_PLAYER) {
                                    this.$el.html(this.renderCompactPlayer());
                                } else if (uiType == SOUND_UI_SETTINGS.FULL_PLAYER) {

                                }
                            } else if (Chapter.getSettings()["autoMediaActionIcons"]) {
                                soundIconInfo = Chapter.getSettings()["soundIconInfo"];
                                this.model.set("imageURL", App.soundIconURL);
                                this.$el.html(this.template(this.model.attributes));
                                this.setLeft(this.model.get("left"));
                                this.setWidth(Math.round(soundIconInfo["soundIconWidth"] * App.settings.scaleFactor));
                                this.setHeight(Math.round(soundIconInfo["soundIconHeight"] * App.settings.scaleFactor));
                                this.setTop(this.model.get("top"));
                                this.$el.css("z-index", this.model.get("ZIndex"));
                            }
                            that.startPlaySound(null, false);
                            return this;
                        } else {
                            this.parentElementModel.set("objectLoaded", this.getLoadedInfo(true));
                            return null;
                        }
                    },
                    categoryActivated: function () {
                        if (pageModel.get("index") == Chapter.getActivePage() && this.model.get("autoStart")) {
                            this.model.set("objectActivated", true);
                        }
                    },
                    activateToggle: function () {
                        // TODO: maybe use here the model instead ?
                        this.touchstart();
                    },
                    activateObject: function () {
                        if (this.model.get("activateObject")) {
                            this.model.set("objectActivated", true);
                            this.model.set("activateObject", false);
                        }
                    },
                    objectActivated: function () {
                        if (this.model.get("objectActivated")) {
                            if (this.shouldRenderObject()) {
                                this.toggleSoundPlay();
                            }
                            this.model.set("objectActivated", false);
                        }
                    },
                    toggleSoundPlay: function () {
                        if (this.model.get("objectActivated") == true) {
                            console.log("Now should play sound");
                            that.startPlaySound(null, true);
                        } else {
                            console.log("Now should stop sound");
                            if (that.playingState == SoundState.PLAYING) {
                                that.endPlaySound();
                            }
                        }
                    }

                });
            };

            this.instantiateObjects = function () {
                this.soundModel = this.getSoundModel();
                this.soundView = new this.SoundView({
                    model: that.soundModel
                });
            };

            this.getSoundModel = function () {
                var model = new Backbone.Model(rawSoundModel.properties);
                model.set("soundURL", ResourceManager.getPathForSound(rawSoundModel.customProperties.objectId, rawSoundModel.customProperties.extension));
                return model;
            };

            this.startAudioTextSync = function () {
                if (this.soundModel.get("loudReadingSound")) {
                    var leftRightThreshold = 0,
                        thisView = this;
                    if (this.soundModel.get("syncOrder") == "top-bottom-left-right") {
                        leftRightThreshold = App.settings.width / 2;
                    }
                    this.soundView.$el.parent().find(".useInAudioSync").syncTextWithAudio({
                        'useCustomHighlight': Chapter.getSettings()["useCustomHighlight"],
                        'syncPointsForGroup': thisView.soundModel.get("syncPoints").split(","),
                        'syncGroupName': 'useInAudioSync' + pageModel.get("index"), // must be something unique
                        'leftRightThreshold': leftRightThreshold,
                        'smartOrder': true
                    });

                }
            };

            this.stopAudioTextSync = function () {
                if (this.soundModel.get("loudReadingSound")) {
                    this.soundView.$el.parent().find(".useInAudioSync").syncTextWithAudio("stopSyncAndRemoveHighlight", {'useCustomHighlight': Chapter.getSettings()["useCustomHighlight"]});
                }
            };

            this.startPlaySound = function (onPlayEnd, autoPlay) {
                require(["SoundRecorder"], function () {
                    var chapterSettings = Chapter.getSettings();
                    var soundURL, isAutomedia = chapterSettings["autoMediaActionIcons"];
//                    if (this.model.get('availableInAllOrientations')){
//                        ret=false;
//                    }

                    if (that.getSoundPlayIsAllowed(autoPlay)) {
                        soundURL = ResourceManager.getPathForSound(rawSoundModel.customProperties.objectId, rawSoundModel.customProperties.extension);
                        that.soundInfo = {
                            soundId: rawSoundModel.properties.id,
                            soundURL: soundURL,
                            autoPlay: autoPlay,
                            pageNumber: pageModel.get("index"),
                            availableInAllOrientations: rawSoundModel.properties.availableInAllOrientations,
                            stopBeforePlay: !isAutomedia,
                            onPlayFinish: function () {
                                console.log("Sound play finished");
                                var activePage = Chapter.getActivePage();
                                if (chapterSettings["startSecondPageAfterAutoStartSoundOnFirstPageHasEnded"] &&
                                    App.dualLandscapeActive() &&
                                    rawSoundModel.properties.autoStart &&
                                    PageUtils.isPageOnTheLeftSide(activePage) &&
                                    activePage < chapterSettings["lastPageNumber"]) {
                                    console.log("Now activate next page when auto start sound has finished");
                                    Chapter.activateNextPageOnSoundFinish();
                                }
                            },
                            onPlayEnd: function () {
                                if (typeof onPlayEnd == "function") {
                                    onPlayEnd();
                                }
                                that.stopAudioTextSync();
                                that.removeAutoMediaPulsateEffect();
                                that.playingState = SoundState.ENDED;
                                that.soundView.updateUIPlayerState();
                            },
                            onPlayStart: function (duration) {
                                that.startAudioTextSync();
                                console.log(LOG_TAGS.MOB_2206, "onPlayStart");
                                that.playingState = SoundState.PLAYING;
                                that.soundView.updateUIPlayerState();
                                //that.soundView.initPlayProgress(duration); // at this point duration is not read correct on Safari iOS
                            },
                            onLoad: function () {
                                // using SM2 this happens after play start so just remove it for now
//                                that.playingState = SoundState.LOADED;
                                console.log("Sound object with id " + rawSoundModel.properties.id + " loaded for page " + pageModel.get("index"));
                                pageModel.set("objectLoaded", that.soundView.getLoadedInfo(true));
                            }
                        };
                        if (that.soundView.getSoundPlayerUIType() != SOUND_UI_SETTINGS.NO_PLAYER) {
                            // add these specific handlers only for player with UI
                            that.soundInfo.whilePlaying = function (position, duration) {
                                that.soundView.updateUIPlayerForPlayPosition(position, duration);
                            };
                            that.soundInfo.onPause = function () {
                                console.log(LOG_TAGS.MOB_2206, "onPause for sound with ID " + rawSoundModel.properties.id);
                                that.playingState = SoundState.PAUSED;
                                that.soundView.updateUIPlayerState();
                            };
                            that.soundInfo.onResume = function () {
                                console.log(LOG_TAGS.MOB_2206, "onResume for sound with ID " + rawSoundModel.properties.id);
                                that.playingState = SoundState.PLAYING;
                                that.soundView.updateUIPlayerState();
                            };
                            that.soundInfo.onSuspend = function () {
                                console.log(LOG_TAGS.MOB_2206, "onSuspend for sound with ID " + rawSoundModel.properties.id);
                                that.playingState = SoundState.SUSPENDED;
                                that.soundView.updateUIPlayerState();
                            }
                        }
                        if (!that.soundView.model.get("availableInAllOrientations") || (that.playingState != SoundState.PLAYING && !PersistentSceneObjectsUtils.anotherPersistentSoundIsPlaying(that.soundView.model.get("id")))) {
                            if (chapterSettings["onlyAllowOneSoundPlaying"] || chapterSettings["autoMediaActionIcons"]) {
                                SoundPlayer.endAllPlayingSoundsForPage(Chapter.getActivePage(), false);
                                if (App.dualLandscapeActive()) {
                                    SoundPlayer.endAllPlayingSoundsForPage(Chapter.getActivePage() + 1, false);
                                }
                            }
                            if (isAutomedia && autoPlay) {
                                that.soundView.$el.addClass("autoMediaPulsate");
                            }
                            SoundPlayer.playSoundWithURL(that.soundInfo);

                        } else {
                            pageModel.set("objectLoaded", that.soundView.getLoadedInfo(true));
                        }
                    } else {
                        pageModel.set("objectLoaded", that.soundView.getLoadedInfo(true));
                    }
//                    }else {
//                        // do nothing
//                    }
                });
            };

            this.endPlaySound = function () {
                SoundPlayer.endSound(that.soundInfo, false);
            };

            this.pauseSound = function () {
                SoundPlayer.pauseSound(that.soundInfo);
            };

            this.resumeSound = function () {
                SoundPlayer.resumeSound(that.soundInfo);
            };

            this.setPosition = function (position) {
                SoundPlayer.setPosition(that.soundInfo, position);
            };

            this.removeAutoMediaPulsateEffect = function () {
                if (null != Chapter.getSettings() && Chapter.getSettings()["autoMediaActionIcons"]) {
                    this.soundView.$el.removeClass("autoMediaPulsate");
                }
            };

            this.getSoundPlayIsAllowed = function (autoPlay) {
                var ret = true,
                    readTextSettings = Chapter.getReadTextSettings(),
                    soundEffectsSettings = Chapter.getSoundEffects(),
                    globalSoundsSettings = Chapter.getGlobalSoundsSetting(),
                    activePage = Chapter.getActivePage(),
                    settingsItems;

                if (null == Chapter.getMenuModel()) {
                    settingsItems = [];
                } else {
                    settingsItems = Chapter.getMenuModel().settings.settingsItems;
                }


                if (Chapter.getSettings()["alwaysPlayRecordedSoundsWithAutoStart"] &&
                    (rawSoundModel.customProperties["isReadText"] || rawSoundModel.properties.autoStart) &&
                    SoundRecorder.getRecordedSoundAvailableForPage(activePage) &&
                    globalSoundsSettings) {
                    // this is SCH-121, might need to be changed if not using globalSoundsSettings
                    return false;
                }
                if (settingsItems.indexOf("globalSoundsControl") > -1) {
                    ret = globalSoundsSettings;
                } else {
                    if ((rawSoundModel.customProperties["isReadText"] || rawSoundModel.properties.autoStart) && !Chapter.getSettings()["autoMediaActionIcons"]) {
                        if (readTextSettings == ReadSoundSettings.SOUNDS_OFF) {
                            if (settingsItems.indexOf("readTextControl") > -1) {
                                ret = false;
                            }
                            console.log("Sound playing not allowed due to menu settings");
                        } else if (readTextSettings == ReadSoundSettings.OWN_SOUNDS) {
                            if (SoundRecorder.getRecordedSoundAvailableForPage(activePage)) {
                                if (settingsItems.indexOf("readTextControl") > -1) {
                                    ret = false;
                                }
                                console.log("Won't play sound with URL " + ResourceManager.getPathForSound(rawSoundModel.customProperties.objectId, rawSoundModel.customProperties.extension) + " because we have a recorded sound to play.");
                                if (autoPlay) {
                                    SoundRecorder.playRecordedSoundForPage(activePage);
                                }
                            }
                        }
                    } else if (settingsItems.indexOf("soundEffectsControl") > -1) {
                        ret = soundEffectsSettings;
                    }
                }

                return ret;
            };

            return {
                init: function () {
                    that.defineClasses();
                    that.instantiateObjects();
                },
                getView: function () {
                    return that.soundView;
                },
                getModel: function () {
                    return that.soundModel;
                },
                getState: function () {
                    return that.playingState;
                }

            }
        }
    }
)
;