define(function () {
    return function (rawTextInputModel, pageModel) {
        var that = this,
            mappedBorderStyleCSSClasses = {
                'Solid': 'divSolidBorder',
                'No border': 'divNoBorder',
                'Dotted': 'divDottedBorder',
                'Dashed': 'divDashedBorder',
                'Double': 'divDoubleBorder',
                'Groove': 'divGrooveBorder',
                'Ridge': 'divRidgeBorder'
            },
            paddingValue = 0;
        this.defineClasses = function () {

            that.TextInputView = ElementBaseView.extend({
                    tagName: "div",
                    className: "sceneObject",
                    elementsViews: [],
                    id: rawTextInputModel.properties.id,
                    parentElementModel: pageModel,
                    displayMode: 'landscapeMode',
                    template: function (model) {
                        if (model.get('isSingleLined')) {
                            if (rawTextInputModel.properties.fadeGradient == "1")
                                return templates.getTemplate("textInputSingleLineWithGradient");
                            return  templates.getTemplate("textInputSingleLine");
                        } else {
                            if (rawTextInputModel.properties.fadeGradient == "1")
                                return templates.getTemplate("textInputMultipleLinesWithGradient");
                            return  templates.getTemplate("textInputMultipleLines");
                        }
                    },
                    $inputElement: null,
                    isFocused: false,


                    initialize: function (attr) {
                        this.addListeners();
                        this.updateOCForModel();
                        this.notifyParentThatObjectIsLoaded();
                    },
                    addCustomListeners: function () {
                        this.listenTo(this.model, "change:displayed", this.showHide);
                        this.listenTo(this.model, "change:loadText", this.loadText);
                        this.listenTo(this.model, "change:shouldClear", this.shouldClear);
                        this.listenTo(this.model, "change:saveText", this.saveText);
                        this.listenTo(this.model, "change:removeText", this.removeText);
                        this.listenTo(this.model, "change:enableObject", this.enableObject);
                        this.listenTo(this.model, "change:disableObject", this.disableObject);
                    },
                    enableObject: function () {
                        if (this.model.get('enableObject')) {
                            this.model.set('enableObject', false);
                            if (null != this.$inputElement) {
                                this.$inputElement.attr('disabled', false);
                            }
                        }


                    },
                    disableObject: function () {
                        if (this.model.get('disableObject')) {
                            this.model.set('disableObject', false);
                        }
                        if (null != this.$inputElement) {
                            this.$inputElement.attr('disabled', true);
                        }
                    },

                    events: function () {
                        var _events = {};
                        _events[AppUtils.EventTypes.mouseMove] = "mouseMoveHandler";
                        return _events;
                    },
                    mouseMoveHandler: function (e) {
                        if (this.isFocused) {
                            e.stopPropagation();
                            e.preventDefault();
                        }
                    },
                    removeView: function () {
                        if (this.model.get("removed")) {
                            this.remove();
                            this.unbind();
                        }
                    },
                    activateObject: function () {
                        if (this.model.get("activateObject")) {
                            this.model.set("objectActivated", true);
                            this.model.set("activateObject", false);
                        }
                    },
                    activateToggle: function () {
                        this.displayHideOnceOnClick();
                    },

                    setCustomCSSClasses: function () {
                        this.$inputElement.addClass(this.model.get('textStyleClass'));
                    },
                    bindCustomEvents: function () {
                        var thisView = this;
                        this.$inputElement.bind("blur", function () {
                            console.log('BLUR ARRIVED');
                            thisView.isFocused = false;
                            thisView.inputDeFocused();
                        });
                        this.$inputElement.bind("focus", function () {
                            console.log('FOCUS ARRIVED');
                            thisView.isFocused = true;
                            thisView.inputFocused();
                        });

                    },
                    inputFocused: function () {
                        this.$inputElement.attr('placeholder', '');
                    },
                    inputDeFocused: function () {
                        var inputValue = this.$inputElement.val();
                        this.model.set('inputTextValue', inputValue);
                        if ('' == inputValue) {
                            this.setInputValue();
//                            localStorage.removeItem(this.model.get("textInputId"));
                        } else {
                            localStorage.setItem(this.model.get("textInputId"), inputValue);
                        }

                        var pages = Chapter.getPagesModelCollection(),
                            i,
                            j,
                            currentPageObjects;

                        for (i = 0; i < pages.models.length; i++) {
                            currentPageObjects = pages.models[i].get('workingObjectsCollection');

                            if (currentPageObjects != null) {
                                for (j = 0; j < currentPageObjects.models.length; j++) {
                                    if (currentPageObjects.models[j].attributes.type == "textBox")
                                        currentPageObjects.models[j].set("textContent", "");
                                }
                            }
                        }
                    },
                    setInputValue: function () {
                        if (this.model.get('inputTextValue') != '') {
                            this.$inputElement.val(this.model.get('inputTextValue'));
                        }
                        else {
                            this.$inputElement.attr('placeholder', $("<div/>").html(this.model.get("placeHolder")).text());
                            localStorage.setItem(this.model.get("textInputId"), this.$inputElement.attr('placeholder'));
                        }
                    },
                    setBorder: function () {
                        this.$el.addClass(mappedBorderStyleCSSClasses[this.model.get('borderStyle')]);
                    },
                    updateInputElement: function () {
                        var size = this.model.get(this.displayMode + 'Size');
                        CSSHandler.setWidth(this.$inputElement, size.width);
                        CSSHandler.setHeight(this.$inputElement, size.height);

                        CSSHandler.setZoom(this.$inputElement, size.scaleFactor);
                    },
                    loadText: function () {
                        if (this.model.get("loadText")) {
                            if (this.model.get("textInputId") != null) {
                                if (localStorage.getItem(this.model.get("textInputId")) != null && localStorage.getItem(this.model.get("textInputId")) != '') {
                                    this.$inputElement.val(localStorage.getItem(this.model.get("textInputId")));
                                }
                                else {
                                    this.model.set("shouldClear", true);
                                }
                            }
                            this.model.set("loadText", false);
                        }
                    },
                    showHide: function () {
                        if (this.model.get("displayed")) {
                            this.$el.css("display", "block");
                        }
                        else {
                            this.$el.css("display", "none");
                        }
                        console.log("DrawingView showHide");
                    },
                    shouldClear: function () {
                        if (this.model.get("shouldClear")) {
                            this.$inputElement.val('');
                            this.$inputElement.attr('placeholder', $("<div/>").html(this.model.get("placeHolder")).text());
                            this.model.set("shouldClear", false);
                        }
                    },
                    saveText: function () {
                        if (this.model.get("saveText")) {
                            var inputValue = this.$inputElement.val();
                            this.model.set('inputTextValue', inputValue);
                            if ('' == inputValue) {
                                this.setInputValue();
                                localStorage.removeItem(this.model.get("textInputId"));
                            } else {
                                localStorage.setItem(this.model.get("textInputId"), inputValue);
                            }
                            this.model.set("saveText", false);
                        }
                    },
                    removeText: function () {
                        if (this.model.get("removeText")) {
                            localStorage.removeItem(this.model.get("textInputId"));
                            this.model.set("removeText", false);
                        }
                    },
                    render: function () {
                        if (this.shouldRenderObject()) {
                            this.renderStartHandler();
                            this.$inputElement = $(this.template(this.model)(this.model.attributes));
                            if (this.model.get("inputTextValue") != null && this.model.get("inputTextValue") != '') {
                                this.$inputElement.val(this.model.get('inputTextValue'));
                            }
                            else {
                                this.$inputElement.attr('placeholder', $("<div/>").html(this.model.get("placeHolder")).text());
                            }
                            this.$inputElement.attr('disabled', !this.model.get('isEnabled'));
                            this.$inputElement.css('-webkit-box-sizing', 'border-box');

                            this.$el.html(this.$inputElement);
                            this.setCustomCSSClasses();
                            this.updateInputElement();
                            this.setLeft(this.model.get("left"));
                            this.setWidth(this.model.get("width"));
                            this.setHeight(this.model.get("height"));
                            this.setTop(this.model.get("top"));
                            this.$el.css("z-index", this.model.get("ZIndex"));
                            this.setInitialVisibility();
                            this.startZoomEffectsIfNeeded();

                            this.bindCustomEvents();
                            this.setBorder();
                            this.model.set("enableObject", true);
                            return this;
                        }
                        return null;
                    }
                }
            )
            ;
        };
        this.instantiateObjects = function () {
            this.textInputModel = this.getTextInputModel();
            this.textInputView = new this.TextInputView({
                model: that.textInputModel
            });

        };

        this.getTextInputModel = function () {
            var model = new Backbone.Model(rawTextInputModel.properties);

            if (rawTextInputModel.properties["textInputIdForSummary"] && rawTextInputModel.properties["textInputIdForSummary"] != "")
                model.set('textInputId', rawTextInputModel.properties["textInputIdForSummary"]);
            else
                model.set('textInputId', 'textInputId_' + rawTextInputModel.properties.id);

            model.set('isEnabled', null == rawTextInputModel.properties.isEnabled ? false : rawTextInputModel.properties.isEnabled);
            model.set('inputTextValue', localStorage.getItem(model.get("textInputId")));
            model.set('placeHolder', rawTextInputModel.customProperties.placeholder != CAPTIONS.DEFAULT_MOB_PLACEHOLDER ? rawTextInputModel.customProperties.placeholder : CAPTIONS.INPUT_PLACEHOLDER);
            model.set('landscapeModeSize', this.getSize(rawTextInputModel.properties.width, rawTextInputModel.properties.height, App.settings.width, App.settings.height));
            model.set('portraitModeSize', this.getSize(rawTextInputModel.properties.width, rawTextInputModel.properties.height, App.settings.height, App.settings.width));
            model.set("gradientColor", rawTextInputModel.properties.gradientColorRGB);
            model.set("value1", rawTextInputModel.properties.gradientValue1);
            model.set("value2", rawTextInputModel.properties.gradientValue2);
            return model;
        };


        this.getSize = function (percentageWidth, percentageHeight, wrapperWidth, wrapperHeight) {
            var width, height;
            width = Math.round(parseFloat(percentageWidth) * wrapperWidth / 100) - 2 * paddingValue;
//            height = Math.round(parseFloat(percentageHeight) * wrapperHeight / 100) - 2 * paddingValue;
            return {
                width: rawTextInputModel.customProperties.originalWidth,
                height: rawTextInputModel.customProperties.originalHeight,
                scaleFactor: width / rawTextInputModel.customProperties.originalWidth
            }
        };

        return {
            init: function () {
                that.defineClasses();
                that.instantiateObjects();
            },
            getView: function () {
                return that.textInputView;
            },
            getModel: function () {
                return that.textInputModel;
            }

        }
    }
})
;