define(function () {
    return function (rawTimedEventModel, pageModel) {
        var that = this;

        this.defineClasses = function () {

            that.TimedEventView = ElementBaseView.extend({
                tagName: "div",
                className: "sceneObject",
                elementsViews: [],
                id: rawTimedEventModel.properties.id,
                parentElementModel: pageModel,
                executedIterations: 0,
                repeatedTimedEvent: null,
                singleTimedEvent: null,

                actions: {
                    "toggleObjectStatePause": "pauseTimer",
                    "toggleObjectStateResume": "resumeTimer"
                },

                removeView: function () {
                    if (this.model.get("removed")) {
                        //We will remove and unbind all events for this time event only if he is part of a category and is not acting as a global time event
                        if (null == this.model.get('globalTimedEventItem') || !this.model.get('globalTimedEventItem') || this.parentElementModel.get('removed')) {
                            this.customStateRestore();
                            this.remove();
                            this.unbind();
                        }
                        this.model.set("activateObject", false);
                    }
                },

                customStateRestore: function () {
                    if (null != this.singleTimedEvent) {
                        this.singleTimedEvent.clearTimeout();
                    }
                    if (null != this.repeatedTimedEvent) {
                        this.repeatedTimedEvent.clearTimeout();
                    }
                },

                shouldRenderObject: function () {
                    return this.currentObjectIsPartOfAnActiveCategory();
                },
                categoryActivated: function () {
                    console.log("TIMED_EVENT categoryActivated");
                    console.log("MODEL_ACTIVATED:  " + this.model.get("activateObject"));

                    //It seems that even if we set this model attribute "activateObject" on false, that value is ignored
                    //Fixed by unset and reset that property to true
                    //TODO: find why that value is ignored or where is set back to true
                    if (!this.model.get('globalTimedEventItem')) {
                        this.model.set("activateObject", false);
                        this.model.set("activateObject", true);
                    }
                },
                activateObject: function () {
                    if (this.model.get("activateObject")) {
                        var timerDelay = this.getIterationDelay(this.model.get("startDelayMin"), this.model.get("startDelayMax"));
                        console.log("TIMED_EVENT activated with a period of: " + timerDelay);
                        this.model.set("activateObject", false);
                        console.log("TIMED_EVENT deactivated: " + this.model.get("activateObject"));
                        if (this.shouldRenderObject() || this.model.get('globalTimedEventItem')) {
                            var thisView = this,
                                timerHandler,
                                repeatIntervalMin = this.model.get("repeatIntervalMin"),
                                repeatIntervalMax = this.model.get("repeatIntervalMax");
                            timerHandler = function () {
                                thisView.model.set("objectActivated", true);
                                console.log("TIMED_EVENT triggered for object " + thisView.model.get('id') + " with delay " + timerDelay);
                                thisView.executedIterations++;
                                if (thisView.executedIterations < thisView.model.get("iterations")) {
                                    timerDelay = thisView.getIterationDelay(repeatIntervalMin, repeatIntervalMax);
                                    thisView.repeatedTimedEvent = new PageUtils.runTimer(function () {
                                        thisView.repeatedTimedEvent = null;
                                        timerHandler();
                                    }, timerDelay);
//                                Chapter.addGlobalTimer();
                                } else {
                                    thisView.executedIterations = 0;
                                }
                            };
                            this.singleTimedEvent = new PageUtils.runTimer(function () {
                                thisView.singleTimedEvent = null;
                                timerHandler();
                            }, timerDelay);
                        }
                    }
                },

                pauseTimer: function () {
                    if (this.singleTimedEvent) {
                        this.singleTimedEvent.pause();
                    }
                },

                resumeTimer: function () {
                    if (this.singleTimedEvent) {
                        this.singleTimedEvent.resume();
                    }
                },

                getIterationDelay: function (min, max) {
                    return (0 == this.toNumber(max)) ? this.toNumber(min) : Math.round(this.toNumber(min) + Math.random() * (this.toNumber(max) - this.toNumber(min)));
                }
            });
        };
        this.instantiateObjects = function () {
            this.timedEventModel = this.getTimedEventModel();
            this.timedEventView = new this.TimedEventView({
                model: that.timedEventModel
            });
        };

        this.getTimedEventModel = function () {
            return new Backbone.Model(rawTimedEventModel.properties);
        };


        return {
            init: function () {
                that.defineClasses();
                that.instantiateObjects();
            },
            getView: function () {
                return that.timedEventView;
            },
            getModel: function () {
                return that.timedEventModel;
            }

        }
    }
});