define(['GenericLightBox', 'YoutubeHelpers'], function (GenericLightBox) {
        return function (rawVideoModel, pageModel) {
            var that = this;

            this.defineClasses = function () {

                that.VideoView = ElementBaseView.extend({
                    tagName: "div",
                    className: "sceneObject videoObject",
                    elementsViews: [],
                    inlineTemplate: templates.getTemplate("video"),
                    autoMediaTemplate: templates.getTemplate("image"),
                    externalTemplate: templates.getTemplate("externalVideo"),
                    youtubeTemplate: templates.getTemplate("youtubeVideo"),
                    id: rawVideoModel.properties.id,
                    parentElementModel: pageModel,
                    videoEl: null,
                    isPlaying: false,
                    YTPlayer: null,
                    YTPlayerLoaded: false,
                    customInitialize: function () {
                        this.model.set("allObjectTypesInPage", rawVideoModel.customProperties["allObjectTypesInPage"]);
                    },

                    events: function () {
                        var _events = {};
                        if (this.model.get("isIconVideo")) {
                            _events[AppUtils.EventTypes.mouseDown] = "touchstart";
                            _events[AppUtils.EventTypes.click] = "clickHandler";
                            _events[AppUtils.EventTypes.mouseUp] = "mouseUpHandler";
                        }
                        return _events;
                    },

                    addCustomListeners: function () {
                        this.listenTo(this.model, "change:videoState", this.changeVideoState);
                        this.listenTo(this.model, "change:YTPlayer", this.YTPlayerLoadComplete);
                    },
                    YTPlayerLoadComplete: function () {
                        this.YTPlayer = this.model.get('YTPlayer');
                        if (null != this.YTPlayer) {
                            this.YTPlayerLoaded = true;
                            this.objectActivated();
                        }

                    },
                    customStateRestore: function () {
                        if (this.isPlaying) {
                            this.model.set("videoState", VideoState.STOP);
                        }
                    },

                    changeVideoState: function () {
                        var newState = this.model.get("videoState"),
                            ytPlayerState;
                        if (newState != VideoState.WAITING) {
                            if (this.videoEl) {
                                if (newState == VideoState.STOP) {
                                    this.isPlaying = false;
                                    if (this.YTPlayer) {
                                        ytPlayerState = this.YTPlayer.getPlayerState();
                                        console.log("On page change YT player state is " + ytPlayerState);
                                        if (ytPlayerState == YT.PlayerState.PLAYING || ytPlayerState == YT.PlayerState.BUFFERING) {
                                            this.YTPlayer.pauseVideo();
                                            this.YTPlayer.seekTo(0);
                                        }
                                    } else {
                                        this.videoEl[0].pause();
                                        this.videoEl[0].currentTime = 0;
                                    }
                                } else if (newState == VideoState.PLAY) {
                                    this.isPlaying = true;
                                    if (this.YTPlayer) {
                                        this.YTPlayer.playVideo();
                                    } else {
                                        this.videoEl[0].play();
                                    }
                                } else if (newState == VideoState.PAUSE) {
                                    this.isPlaying = false;
                                    if (this.YTPlayer) {
                                        this.YTPlayer.pauseVideo();
                                    } else {
                                        this.videoEl[0].pause();
                                    }
                                }
                            }
                            this.model.set("videoState", VideoState.WAITING);
                        }
                    },

                    removeView: function () {
                        if (this.model.get("removed")) {
                            this.remove();
                            this.unbind();
                        }
                        this.isPlaying = false;
                        this.model.set("objectActivated", false);
                    },
                    shouldRenderObject: function () {
                        return this.currentObjectIsPartOfAnActiveCategory();
                    },
                    getVideoElement: function (autoPlay) {
                        var videoElement = null,
                            sourceVideos, idx, videoExtension,
                            thisView = this;
                        if (rawVideoModel.customProperties.videoType == VideoType.REGULAR) {
                            videoElement = $(this.inlineTemplate(this.model.attributes));
                            if (autoPlay) {
                                videoElement.prop("autoplay", true);
                            }
                        } else if (rawVideoModel.customProperties.videoType == VideoType.EXTERNAL) {
                            videoElement = $(this.externalTemplate(this.model.attributes));
                            if (autoPlay) {
                                videoElement.prop("autoplay", true);
                            }
                            sourceVideos = this.model.get("videoURL").split(";");
                            for (idx = 0; idx < sourceVideos.length; idx++) {
                                videoExtension = sourceVideos[idx].substring(sourceVideos[idx].lastIndexOf(".") + 1);
                                videoElement.prepend("<source src='" + sourceVideos[idx] + "' type='video/" + videoExtension + "'>");
                            }
                        } else if (rawVideoModel.customProperties.videoType == VideoType.YOUTUBE) {
                            videoElement = $(this.youtubeTemplate(this.model.attributes));
                            var propertiesId = rawVideoModel.properties.id,
                                customPropertiesId = rawVideoModel.customProperties.objectId,
                                youtubeVideoId = "yt-video-" + propertiesId + "-" + customPropertiesId + "-" + Math.floor(Math.random() * 999999);

                            videoElement.attr("id", youtubeVideoId).addClass("chapterTOCVideo");

                            if (autoPlay) {
                                this.model.set("autoPlay", true); // when YT is ready this will be checked and video started
                            }

                            setTimeout(function () {
                                YouTubeHelpers.startLoadingYoutubeVideo(youtubeVideoId, rawVideoModel.customProperties.youtubeID, thisView.model);
                            }, 100);
                        }
                        return videoElement;
                    },
                    render: function () {
                        var thisView = this;
                        if (this.shouldRenderObject()) {
                            this.renderStartHandler();
                            if (this.model.get("isIconVideo")) {
                                var videoIconInfo = Chapter.getSettings()["videoIconInfo"];
                                this.model.set("imageURL", App.videoIconURL);
                                this.$el.html(this.autoMediaTemplate(this.model.attributes));
                                this.model.set("width", videoIconInfo['videoIconWidth']);
                                this.model.set("height", videoIconInfo['videoIconHeight']);
                            } else { // inline video
                                this.videoEl = this.getVideoElement(false);
                                if (this.videoEl) {
                                    this.videoEl.bind("play", function () {
                                        console.log("Now video is playing");
                                        thisView.isPlaying = true;
                                    });
                                    this.videoEl.bind("pause stop", function () {
                                        console.log("Now video was paused/stopped");
                                        thisView.isPlaying = false;
                                    });
                                    this.$el.html(this.videoEl);
                                }
                            }
                            this.setLeft(this.model.get("left"));
                            this.setWidth(this.model.get("width"));
                            this.setHeight(this.model.get("height"));
                            this.setTop(this.model.get("top"));
                            this.setInitialVisibility();
                            this.$el.css("z-index", this.model.get("ZIndex"));
                            return this;
                        } else {
                            return null;
                        }
                    },
                    categoryActivated: function () {
                        if (pageModel.get("index") == Chapter.getActivePage() && this.model.get("autoStart")) {
                            this.model.set("objectActivated", true);
                        }
                    },
                    activateToggle: function () {
                        console.log("Video activate toggle");
                        this.toggleVideoPlay();
                    },
                    activateObject: function () {
                        if (this.model.get("activateObject")) {
                            this.model.set("activateObject", false);
                            if (!this.isPlaying) {
                                this.model.set("objectActivated", true);
                            }
                        }
                    },
                    objectActivated: function () {
                        if (this.model.get("objectActivated")) {

                            if (this.model.get("isIconVideo")) {

                                if (rawVideoModel.customProperties.videoType == VideoType.YOUTUBE) {
                                    this.model.set('YTPlayer', null);
                                }

                                var videoWidth = rawVideoModel.properties.defaultWidth,
                                    videoHeight = rawVideoModel.properties.defaultHeight,
                                    videoAspectRatio = videoWidth / videoHeight;

                                // might be some small issues with centering in case of videos with resolution that is bigger than screen resolution

                                if (videoWidth > App.settings.width) {
                                    videoWidth = 0.95 * App.settings.width;
                                    videoHeight = videoWidth / videoAspectRatio;
                                } else if (videoHeight > App.settings.height) {
                                    videoHeight = 0.95 * App.settings.height;
                                    videoWidth = videoHeight / videoAspectRatio;
                                } else if (videoWidth == 0) { // for YT videos
                                    this.model.set("autoStart", true);
                                    // assume a 16:9 aspect ratio
                                    videoWidth = 0.95 * App.settings.width;
                                    videoHeight = Math.min(videoWidth * 9 / 16, 0.95 * App.settings.height);
                                }

                                var lightBoxWindowSettings = {
                                    width: videoWidth,
                                    height: videoHeight
                                };

                                this.videoEl = this.getVideoElement(true);

                                this.lightBox = new GenericLightBox(null, lightBoxWindowSettings, {content: this.videoEl});
                                this.lightBox.init();
                                App.viewPortView.$el.append(this.lightBox.getView().render().el);
                                this.model.set("objectActivated", false);

                            } else {
                                if (rawVideoModel.customProperties.videoType == VideoType.YOUTUBE) {
                                    if (null != this.YTPlayer && this.YTPlayerLoaded) {
                                        if (this.shouldRenderObject()) {
                                            this.toggleVideoPlay();
                                        }
                                        this.model.set("objectActivated", false);
                                    }
                                } else {
                                    if (this.shouldRenderObject()) {
                                        this.toggleVideoPlay();
                                    }
                                    this.model.set("objectActivated", false);
                                }
                            }
                        }
                    },
                    toggleVideoPlay: function () {
                        if (this.isPlaying) {
                            console.log("Now should stop video");
                            this.model.set("videoState", VideoState.STOP);
                        } else {
                            console.log("Now should play video");
                            this.model.set("videoState", VideoState.PLAY);
                        }
                    }

                });
            };

            this.instantiateObjects = function () {
                this.videoModel = this.getVideoModel();
                this.videoView = new this.VideoView({
                    model: that.videoModel
                });
            };

            this.getVideoModel = function () {
                var model = new Backbone.Model(rawVideoModel.properties);
                model.set("videoURL", this.getVideoURL()); // not used for YT videos
                model.set("posterURL", this.getVideoPosterURL());
                model.set("videoType", rawVideoModel.customProperties.videoType);
                return model;
            };

            this.getVideoPosterURL = function () {
                var videoPosterURL = "";
                if (rawVideoModel.customProperties.videoType == VideoType.REGULAR) {
                    if (AppUtils.nativeApp) {
                        // TODO: make here a better logic for when running in native app
                        videoPosterURL = "thumbs/" + rawVideoModel.properties.objectId + "_video_thumb.png";
                    } else {
                        videoPosterURL = "/thumb?objectId=" + rawVideoModel.properties.objectId + "&action=getMediaThumbnail&originalSize=1";
                    }
                }
                return videoPosterURL;
            };

            this.getVideoURL = function () {
                var videoURL;
                if (rawVideoModel.customProperties.videoType == VideoType.REGULAR) {
                    videoURL = ResourceManager.getPathForVideo(rawVideoModel.customProperties.objectId, rawVideoModel.customProperties.extension);
                } else if (rawVideoModel.customProperties.videoType == VideoType.EXTERNAL) {
                    videoURL = rawVideoModel.customProperties.videoURL;
                }
                return videoURL;
            };

            return {
                init: function () {
                    that.defineClasses();
                    that.instantiateObjects();
                },
                getView: function () {
                    return that.videoView;
                },
                getModel: function () {
                    return that.videoModel;
                }

            }
        }
    }
);