define(['SceneObjectAnimation', 'EventsHandlers'], function (SceneObjectAnimation) {
    return function (rawAnimationModel, pageModel) {
        var that = this;

        this.defineClasses = function () {

            that.EmptyBoxView = ElementBaseView.extend({
                    tagName: "div",
                    className: "sceneObject",
                    elementsViews: [],
                    id: rawAnimationModel.properties.id,
                    parentElementModel: pageModel,
                    animationObjectInstance: null,
                    events: function () {
                        var _events = {};
                        _events[AppUtils.EventTypes.mouseDown] = "touchstart";
                        _events[AppUtils.EventTypes.click] = "clickHandler";
                        _events["touchcancel"] = "touchcancelHandler";
                        _events["touchcend"] = "touchcendHandler";
                        return _events;
                    },
                    touchcendHandler: function () {
                        console.log("touchcendHandler on " + this.$el.attr("id"));
                    },
                    touchcancelHandler: function () {
                        console.log("touchcancelHandler on " + this.$el.attr("id"));
                    },
                    removeView: function () {
                        if (this.model.get("removed")) {
                            this.remove();
                            this.unbind();
                        }
                    },
                    activateObject: function () {
                        if (this.model.get("activateObject")) {
                            if (this.model.get('animationSettings')['userInteractionRequired'] == 'false')
                                this.animationObjectInstance.startAnimationIfNeeded(true);
                            this.model.set("objectActivated", true);
                            this.model.set("activateObject", false);
                        }
                    },
                    addCustomListeners: function () {
                        this.listenTo(this.model, "change:allItemsLoaded", this.allItemsLoaded);
                        this.listenTo(this.model, "change:fullScreenRequired", this.fullScreenRequired);
                    },
                    allItemsLoaded: function () {
                        this.animationObjectInstance.setActiveItem(0);
                    },
                    activateToggle: function () {
                        this.displayHideOnceOnClick();
                    },

                    setBackgroundColorIfNeeded: function (color) {
                        if (color != 'transparent') {
                            this.setBackgroundColor(color, 1);
                        }
                    },
                    addUserInteractionElementsIfNeeded: function (userInteractionRequired) {
                        if (userInteractionRequired == 'true') {
                            var models, i;
                            this.animationObjectInstance.getDotsBarModel().set('parent', this.$el);
                            this.animationObjectInstance.getFullScreenButtonModel().set('parent', this.$el);
                            models = this.animationObjectInstance.getArrowsCollection().models;
                            for (i = 0; i < models.length; i++) {
                                models[i].set('parent', this.$el);
                            }
                        }
                    },
                    fullScreenRequired: function () {
                        var fullScreenProperties = this.model.get('fullScreenProperties');
                        this.setWidth(fullScreenProperties.width + '%');
                        this.setHeight(fullScreenProperties.height + '%');
                        this.setTop(fullScreenProperties.top + '%');
                        this.setLeft(fullScreenProperties.left + '%');
                    },
                    cancelFullScreen: function () {
                        this.setLeft(this.model.get("left"));
                        this.setWidth(this.model.get("width"));
                        this.setHeight(this.model.get("height"));
                        this.setTop(this.model.get("top"));
                    },
                    toggleFullScreen: function (isFullScreen) {
                        if (isFullScreen)
                            this.fullScreenRequired();
                        else
                            this.cancelFullScreen();
                    },
                    setFullScreenProperties: function () {
                        var percentageWidth = parseFloat(this.model.get('width')),
                            percentageHeight = parseFloat(this.model.get('height')),
                            scaleFactor, fullScreenProperties = {};

                        if (percentageWidth / percentageHeight < 1) {
                            scaleFactor = 100 / percentageHeight;
                        } else {
                            scaleFactor = 100 / percentageWidth;
                        }

                        fullScreenProperties.width = Math.min(scaleFactor * percentageWidth, 100);
                        fullScreenProperties.height = Math.min(scaleFactor * percentageHeight, 100);
                        fullScreenProperties.left = (100 - fullScreenProperties.width) / 2;
                        fullScreenProperties.top = (100 - fullScreenProperties.height) / 2;

                        this.model.set('fullScreenProperties', fullScreenProperties);
                    },
                    render: function () {
                        if (this.shouldRenderObject()) {
                            var thisView = this;
                            var animationSettings = rawAnimationModel.customProperties["animationJSON"]["animationSettings"],
                                animationItems = rawAnimationModel.customProperties["animationJSON"]["animationItems"];
                            this.setFullScreenProperties();
                            this.model.set('animationSettings', animationSettings);
                            this.animationObjectInstance = new SceneObjectAnimation(animationSettings, animationItems, this.model);
                            this.animationObjectInstance.init();

                            this.animationObjectInstance.getItemsWrapperModel().set('parent', this.$el);
                            this.setBackgroundColorIfNeeded(animationSettings['paddedAreaColor']);
                            this.addUserInteractionElementsIfNeeded(animationSettings['userInteractionRequired']);
                            this.renderStartHandler();
                            this.setLeft(this.model.get("left"));
                            this.setWidth(this.model.get("width"));
                            this.setHeight(this.model.get("height"));
                            this.setTop(this.model.get("top"));
                            this.setZIndex(this.model.get("ZIndex"));
                            this.setInitialVisibility();
                            this.startZoomEffectsIfNeeded();
                            this.setBackfaceVisibility('hidden');
                            this.animationObjectInstance.setActiveItem(0);
                            this.parentElementModel.set("objectLoaded", this.getLoadedInfo(true));
                            ListenerHandler.addFUllScreenStateChangeListener(this.el, function () {
                                thisView.fullScreenRequired()
                            }, function () {
                                thisView.cancelFullScreen()
                            });
                            return this;
                        } else {
                            this.parentElementModel.set("objectLoaded", this.getLoadedInfo(true));
                            return null;
                        }
                    }
                }
            )
            ;
        };
        this.instantiateObjects = function () {
            this.emptyBoxModel = this.getEmptyBoxModel();
            this.emptyBoxView = new this.EmptyBoxView({
                model: that.emptyBoxModel
            });

        };

        this.getEmptyBoxModel = function () {
            return  new Backbone.Model(rawAnimationModel.properties);

        };


        return {
            init: function () {
                that.defineClasses();
                that.instantiateObjects();
            },
            getView: function () {
                return that.emptyBoxView;
            },
            getModel: function () {
                return that.emptyBoxModel;
            }

        }
    }
})
;