define(['Templates'], function () {
    return function (animationSettings, animationItems, sceneAnimationModel) {
        var that = this,
            animationConstants = {
                FADE_TO_BLACK_TRANSITION_DURATION_MS: 600,
                TRIGGER_FULL_SCREEN_MODE_ZOOM_SCALE_LEVEL: 1.2,
                TRIGGER_NORMAL_MODE_ZOOM_SCALE_LEVEL: 0.9,
                PLAY_PAUSE_ICON_EFFECT_DURATION_MS: 500,
                PLAY_PAUSE_ICON_MAX_SCALE_FACTOR: 1.5,
                MAX_TIME_TO_WAIT_FOR_EXTERNAL_VIDEO_JS_API: 10000,
                TIME_INTERVAL_TO_CHECK_FOR_EXTERNAL_VIDEO_JS_API: 500,
                DEFAULT_DISPLAY_DURATION: 2000,
                DEFAULT_TRANSITION_DURATION: 1500
            },
            animationElementsConstants = {
                ARROW_WIDTH: 50,
                ARROW_HEIGHT: 50,
                LEFT_ARROW_URL: 'gfx/nav_small_arrow_left.png',
                RIGHT_ARROW_URL: 'gfx/nav_small_arrow_right.png',
                TOP_ARROW_URL: 'gfx/nav_small_arrow_top.png',
                BOTTOM_ARROW_URL: 'gfx/nav_small_arrow_bottom.png',

                MAX_DOT_SIZE: 15,
                DOT_MARGIN: 3,
                DOT__ON_URL: 'gfx/circle_on.png',
                DOT__OFF_URL: 'gfx/circle_off.png',

                FULl_SCREEN_BTN_URL: 'gfx/icon_resize.png',
                NORMAL_SIZE_SCREEN_BTN_URL: 'gfx/icon_close.png'
            },
            animationEffects = {
                HORIZONTAL_SLIDE: 'HORIZONTALSLIDE',
                VERTICAL_SLIDE: 'VERTICALSLIDE',
                SLIDELEFT: 'SLIDELEFT',
                SLIDERIGHT: 'SLIDERIGHT',
                SLIDEUP: 'SLIDEUP',
                SLIDEDOWN: 'SLIDEDOWN',
                APPEAR: 'APPEAR',
                FADE: 'FADE'
            },
            sceneAnimationWidth, sceneAnimationHeight;
        this.displayingTransitionInProgress = false;
        this.removingTransitionInProgress = false;


        this.defineClasses = function () {
            that.ItemWrapperView = BaseView.extend({
                tagName: 'div',
                className: 'animationItemWrapper',
                activeIndex: 0,
                initialize: function () {
                    this.listenTo(this.model, "change:parent", this.addViewToParent);
                },
                addViewToParent: function () {
                    var parent = this.model.get('parent');
                    if (null != parent)
                        parent.append(this.render().el);
                },
                render: function () {
                    var models, i;
                    this.setWidth(this.model.get('width'));
                    this.setHeight(this.model.get('height'));
                    this.setLeft(this.model.get('left'));
                    this.setTop(this.model.get('top'));
                    models = that.itemsCollection.models;
                    for (i = 0; i < models.length; i++) {
                        models[i].set('parent', this.$el);
                    }
                    return this;
                }

            });

            that.ItemView = BaseView.extend({
                tagName: 'div',
                className: 'animationItem',
                template: templates.getTemplate("animationItem"),
                activeIndex: 0,
                $textEl: null,
                $imageEl: null,
                $template: null,
                initialize: function () {
                    this.listenTo(this.model, "change:parent", this.addViewToParent);
                    this.listenTo(this.model, "change:activeItem", this.activeItemChanged);
                    this.listenTo(this.model, "change:translate", this.translateView);
                    this.listenTo(this.model, "change:left", this.adjustLeftProperty);
                    this.listenTo(this.model, "change:top", this.adjustTopProperty);
                    this.listenTo(this.model, "change:zIndex", this.adjustZIndexProperty);
                    this.listenTo(this.model, "change:transitionEndHandler", this.addTransitionEndHandler);
                    this.listenTo(this.model, "change:show", this.showHide);
                    this.listenTo(this.model, "change:addClassName", this.addClassName);
                    this.listenTo(this.model, "change:removeClassName", this.removeClassName);
                },
                activeItemChanged: function () {
                    var activeItem = this.model.get('activeItem');
                    if (activeItem != this.activeIndex) {
                        if (this.model.get('index') != activeItem) {
                            that.removeItemFromViewPort(this);
                        } else {
                            that.displayItemInViewPort(this);

                        }
                    }
                    this.activeIndex = activeItem;
                },
                addViewToParent: function () {
                    var parent = this.model.get('parent');
                    if (null != parent)
                        parent.append(this.render().el);
                },
                resizeItemImage: function (wrapperWidth, wrapperHeight, absoluteWidth, absoluteHeight) {
                    var thisView = this;
                    if (this.model.get('stretch')) {
                        CSSHandler.setWidth(thisView.$imageEl, '100%');
                        CSSHandler.setHeight(thisView.$imageEl, '100%');
                        this.$el.html(this.$template);
                        this.setBackgroundColorIfNeeded(this.model.get('gapColor'));
                    } else {
                        this.$imageEl.bind('load', function () {
                            var image = thisView.$imageEl[0],
                                imageWidth = image.naturalWidth,
                                imageHeight = image.naturalHeight,
                                wrapperAspectRatio = absoluteWidth / absoluteHeight,
                                imageAspectRatio = imageWidth / imageHeight,
                                newWidth, newHeight, left, top;
                            if (wrapperAspectRatio > imageAspectRatio) {
                                newHeight = Math.min(imageHeight, absoluteHeight);
                                newWidth = Math.round(newHeight * imageAspectRatio);
                            } else {
                                newWidth = Math.min(absoluteWidth, imageWidth);
                                newHeight = Math.round(newWidth / imageAspectRatio);
                            }
                            left = 100 * Math.round((absoluteWidth - newWidth) / 2) / absoluteWidth;
                            top = 100 * Math.round((absoluteHeight - newHeight) / 2) / absoluteHeight;
                            newWidth = 100 - 2 * left + '%';
                            newHeight = 100 - 2 * top + '%';

                            CSSHandler.setLeft(thisView.$imageEl, left + '%');
                            CSSHandler.setTop(thisView.$imageEl, top + '%');
                            CSSHandler.setWidth(thisView.$imageEl, newWidth);
                            CSSHandler.setHeight(thisView.$imageEl, newHeight);
                            thisView.$el.html(thisView.$template);
                            thisView.setBackgroundColorIfNeeded(thisView.model.get('gapColor'));
                        });
                    }
                    console.log("resizeItemImage called in SceneObjectAnimation.js");
                },
                setBackgroundColorIfNeeded: function (color) {
                    if (color != 'transparent') {
                        this.setBackgroundColor(color, 1);

                    }
                },
                setInitialVisibility: function () {
                    this.showHide();
                },
                translateView: function () {
                    var translateObject = this.model.get('translate');
                    if (null != translateObject)
                        this.translate3D(translateObject.x, translateObject.y, 0, this.transitionDuration);
                },
                adjustLeftProperty: function () {
                    if (null != this.model.get('left')) {
                        this.setLeft(this.model.get('left'));
                        that.removingTransitionInProgress = false;
                        CSSHandler.removeBrowserSpecificProperty(this.$el, 'transition', null);
                        CSSHandler.removeBrowserSpecificProperty(this.$el, 'transform', null);
                        this.model.set('translate', null);
                        this.model.set('left', null);
                    }
                },
                adjustZIndexProperty: function () {
                    this.setZIndex(this.model.get('zIndex'));
                },
                adjustTopProperty: function () {
                    if (null != this.model.get('top')) {
                        this.setTop(this.model.get('top'));
                        that.removingTransitionInProgress = false;
                        CSSHandler.removeBrowserSpecificProperty(this.$el, 'transition', null);
                        CSSHandler.removeBrowserSpecificProperty(this.$el, 'transform', null);
                        this.model.set('translate', null);
                        this.model.set('top', null);
                    }
                },
                addClassName: function () {
                    if (null != this.model.get('addClassName')) {
                        this.$el.addClass(this.model.get('addClassName'));
                        this.model.set('addClassName', null);
                    }
                },
                removeClassName: function () {
                    if (null != this.model.get('removeClassName')) {
                        this.$el.removeClass(this.model.get('removeClassName'))
                        this.model.set('removeClassName', null);
                    }
                },
                showHide: function () {
                    var opacity = this.model.get('show') ? this.model.get("opacity") : 0;
                    CSSHandler.setOpacity(this.$el, opacity);
                },
                addTransitionEndHandler: function () {
                    if (null != this.model.get('transitionEndHandler')) {
                        this.setTransitionEndHandler(this.model.get('transitionEndHandler'));
                    } else {
                        this.removeTransitionEndHandler();
                    }
                },
                setTextProperties: function () {
                    if (this.model.get('useCaptions')) {
                        this.$textEl.html(this.model.get('caption'));
                        this.$textEl.addClass(this.model.get('captionPosition'));
                        CSSHandler.setOpacity(this.$textEl, this.model.get('captionOpacity'));
                        CSSHandler.setFontSize(this.$textEl, this.model.get('captionFontSize'));
                        CSSHandler.setColor(this.$textEl, this.model.get('captionTextColor'));
                        CSSHandler.setVerticalPadding(this.$textEl, this.model.get('captionPadding'), this.model.get('captionPadding'));
                        if (this.model.get('captionBackgroundColor') == 'transparent') {
                            CSSHandler.setBackgroundColor(this.$textEl, '#FFFFFF', 0);
                        }
                    }
                },
                render: function () {
                    var transitionDuration = (null == animationSettings['transitionDuration']) ? animationConstants.DEFAULT_TRANSITION_DURATION : parseFloat(animationSettings['transitionDuration']);

                    this.transitionDuration = (transitionDuration <= 0) ? animationConstants.DEFAULT_TRANSITION_DURATION : transitionDuration;
                    this.setWidth(this.model.get('width'));
                    this.setHeight(this.model.get('height'));
                    this.setZIndex(this.model.get('zIndex'));
                    this.$template = $(this.template(this.model.attributes));
                    this.$imageEl = this.$template.filter('img');
                    this.$textEl = this.$template.filter('div');
                    this.resizeItemImage(this.model.get('width'), this.model.get('height'), this.model.get('absoluteWidth'), this.model.get('absoluteHeight'));
                    this.setTextProperties();
                    this.setInitialVisibility();
                    return this;
                }

            });
            that.ArrowView = BaseView.extend({
                tagName: 'div',
                arrowElement: null,
                className: 'animationArrow',
                initialize: function () {
                    this.listenTo(this.model, "change:parent", this.addViewToParent);
                    this.listenTo(this.model, "change:show", this.showHide);
                    this.listenTo(this.model, "change:adjustCSSProperties", this.adjustArrowsCSSProperties);
                },
                addViewToParent: function () {
                    var parent = this.model.get('parent');
                    if (null != parent)
                        parent.append(this.render().el);
                },
                events: function () {
                    var _events = {};
                    _events[AppUtils.EventTypes.click] = "clickHandler";
                    return _events;
                },
                clickHandler: function () {
                    var models, i,
                        thisView = this, direction;
                    if (!that.displayingTransitionInProgress && !that.removingTransitionInProgress) {
                        direction = (thisView.model.get('action') == 'increase') ? 1 : -1;
                        that.itemsCollection.prepareNeighborsItems(direction);
                        setTimeout(function () {
                            if (direction == 1)
                                that.itemsCollection.setNextActiveItem();
                            else
                                that.itemsCollection.setPreviousActiveItem();
                            models = that.arrowsCollection.models;
                            for (i = 0; i < models.length; i++)
                                models[i].set('adjustCSSProperties', true);
                            thisView.adjustArrowsCSSProperties();
                        }, 1);
                    }
                },
                adjustArrowsCSSProperties: function () {
                    if (this.model.get('adjustCSSProperties')) {
                        if ((this.model.get('action') == 'increase'))
                            if (that.itemsCollection.lastItemActive())
                                this.model.set('show', false);
                            else
                                this.model.set('show', true);
                        else if (that.itemsCollection.firstItemActive())
                            this.model.set('show', false);
                        else
                            this.model.set('show', true);
                        this.model.set('adjustCSSProperties', false);
                    }
                },
                showHide: function () {
                    if (this.model.get('show'))
                        this.$el.show();
                    else
                        this.$el.hide();
                },
                render: function () {

                    if (this.model.get('width') != -1) {
                        this.setWidth(this.model.get('width'));
                    }
                    if (this.model.get('height') != -1) {
                        this.setHeight(this.model.get('height'));
                    }
                    if (this.model.get('left') != -1) {
                        this.setLeft(this.model.get('left'));
                    }
                    if (this.model.get('right') != -1) {
                        this.setRight(this.model.get('right'));
                    }
                    if (this.model.get('top') != -1) {
                        this.setTop(this.model.get('top'));
                    }
                    if (this.model.get('bottom') != -1) {
                        this.setBottom(this.model.get('bottom'));
                    }
                    this.setBackgroundImage(this.model.get('imageURL'));
                    this.model.set('adjustCSSProperties', true);
                    return this;

                }
            });
            that.DotView = BaseView.extend({
                tagName: 'div',
                arrowElement: null,
                className: 'animationDot',
                initialize: function () {
                    this.listenTo(this.model, "change:parent", this.addViewToParent);
                    this.listenTo(this.model, "change:parent", this.addViewToParent);
                    this.listenTo(this.model, "change:activeItem", this.changeActiveItem);
                },
                addViewToParent: function () {
                    var parent = this.model.get('parent');
                    if (null != parent)
                        parent.append(this.render().el);
                },
                changeActiveItem: function () {
                    if (this.model.get('activeItem') == this.model.get('index'))
                        this.setBackgroundImage(animationElementsConstants.DOT__ON_URL);
                    else
                        this.setBackgroundImage(animationElementsConstants.DOT__OFF_URL);
                },
                render: function () {

                    if (null != this.model.get('width')) {
                        this.setWidth(this.model.get('width'));
                    }
                    if (null != this.model.get('height')) {
                        this.setHeight(this.model.get('height'));
                    }

                    this.setVerticalMargin(this.model.get('verticalMargin'), this.model.get('verticalMargin'));
                    this.setHorizontalMargin(this.model.get('horizontalMargin'), this.model.get('horizontalMargin'));
                    this.setBackgroundImage(animationElementsConstants.DOT__OFF_URL);
                    return this;

                }
            });
            that.DotBarView = BaseView.extend({
                tagName: 'div',
                arrowElement: null,
                className: 'dotsBar',
                initialize: function () {
                    this.listenTo(this.model, "change:parent", this.addViewToParent);
                },
                addViewToParent: function () {
                    var parent = this.model.get('parent');
                    if (null != parent)
                        parent.append(this.render().el);
                },
                render: function () {
                    var i, models;
                    this.setWidth(this.model.get('width'));
                    this.setHeight(this.model.get('height'));

                    if (this.model.get('left') != -1)
                        this.setLeft(this.model.get('left'));
                    if (this.model.get('top') != -1)
                        this.setTop(this.model.get('top'));
                    models = that.dotsCollection.models;
                    for (i = 0; i < models.length; i++) {
                        models[i].set('parent', this.$el);
                    }
                    return this;

                }
            });
            that.FullScreenButtonView = BaseView.extend({
                tagName: 'div',
                arrowElement: null,
                className: 'fullScreenToggleBtn',
                initialize: function () {
                    this.listenTo(this.model, "change:parent", this.addViewToParent);
                },
                addViewToParent: function () {
                    var parent = this.model.get('parent');
                    if (null != parent)
                        parent.append(this.render().el);
                },
                events: function () {
                    var _events = {};
                    _events[AppUtils.EventTypes.click] = "clickHandler";
                    return _events;
                },
                clickHandler: function () {
                    this.requestFullScreen();
                },
                requestFullScreen: function () {
                    var parent = this.model.get('parent')[0];
                    if (ListenerHandler.isElementFullScreen(parent))
                        ListenerHandler.cancelBrowserFullScreen(parent);
                    else
                        ListenerHandler.requestBrowserFullScreen(parent);
                },
                render: function () {
                    this.setBackgroundImage(this.model.get('imageURL'));
                    return this;

                }
            });
            that.ItemsCollection = Backbone.Collection.extend({
                activeItem: 0,
                currentZIndex: 0,
                looping: animationSettings['looping'] == 'true',
                firstTime: true,
                setCurrentZIndex: function (zIndex) {
                    this.currentZIndex = zIndex;
                },
                setActiveItem: function (itemIndex) {
                    this.models[this.activeItem].set('activeItem', itemIndex);
                    that.dotsCollection.models[this.activeItem].set('activeItem', itemIndex);
                    this.activeItem = itemIndex;
                    this.models[this.activeItem].set('activeItem', itemIndex);
                    that.dotsCollection.models[this.activeItem].set('activeItem', itemIndex);
                    if (this.firstTime) {
                        this.firstTime = false;
                        this.prepareNeighborsItems();
                    }
                },
                setPreviousActiveItem: function () {
                    var transitionEffect = animationSettings['transitionEffect'],
                        nextItem, previousElement, direction;
                    if (transitionEffect == animationEffects.SLIDERIGHT || transitionEffect == animationEffects.SLIDEDOWN) {
                        direction = -1;
                    } else {
                        direction = 1
                    }
                    previousElement = this.getPreviousItem();
                    nextItem = this.getNextItem();
                    if (null != previousElement) {
                        this.updateActiveItem(previousElement, nextItem, direction);
                    }

                },
                setNextActiveItem: function () {
                    var transitionEffect = animationSettings['transitionEffect'],
                        nextItem, previousElement, direction;
                    if (transitionEffect == animationEffects.SLIDERIGHT || transitionEffect == animationEffects.SLIDEDOWN) {
                        direction = 1;
                    } else {
                        direction = -1;
                    }
                    previousElement = this.getPreviousItem();
                    nextItem = this.getNextItem();
                    if (null != nextItem) {
                        this.updateActiveItem(nextItem, previousElement, direction);
                    }
                },
                updateActiveItem: function (element, previousElement, direction) {
                    var thisView = this;
                    this.setDirection(this.models[this.activeItem], direction);
                    this.setDirection(previousElement, direction);
                    this.setDirection(element, direction);
                    that.itemsCollection.prepareNeighborsItems(1);
                    setTimeout(function () {
                        thisView.setActiveItem(element.get('index'));
                    }, 1);

                },
                setDirection: function (model, direction) {
                    if (null != model) {
                        model.set('direction', direction);
                    }
                },
                prepareNeighborsItems: function (direction) {

                    var transitionEffect = animationSettings['transitionEffect'], previousItem, nextItem;

                    previousItem = this.getPreviousItem();
                    nextItem = this.getNextItem();
                    if (previousItem === nextItem) {
                        if (direction == 1) {
                            previousItem = null;
                        } else {
                            nextItem = null;
                        }
                    }

                    if (transitionEffect == animationEffects.VERTICAL_SLIDE || transitionEffect == animationEffects.SLIDEUP) {
                        if (null != previousItem)
                            previousItem.set('top', '-' + previousItem.get('height'));
                        if (null != nextItem)
                            nextItem.set('top', nextItem.get('height'));
                    } else if (transitionEffect == animationEffects.HORIZONTAL_SLIDE || transitionEffect == animationEffects.SLIDELEFT) {
                        if (null != previousItem)
                            previousItem.set('left', '-' + previousItem.get('width'));
                        if (null != nextItem)
                            nextItem.set('left', nextItem.get('width'));
                    } else if (transitionEffect == animationEffects.SLIDERIGHT) {
                        if (null != previousItem)
                            previousItem.set('left', previousItem.get('width'));
                        if (null != nextItem)
                            nextItem.set('left', '-' + nextItem.get('width'));
                    } else if (transitionEffect == animationEffects.SLIDEDOWN) {
                        if (null != nextItem)
                            nextItem.set('top', '-' + nextItem.get('height'));
                        if (null != previousItem)
                            previousItem.set('top', previousItem.get('height'));
                    } else if (transitionEffect == animationEffects.APPEAR || transitionEffect == animationEffects.FADE) {
                        if (null != previousItem) {
                            previousItem.set('show', false);
                            previousItem.set('zIndex', ++this.currentZIndex);
                        }
                        if (null != nextItem) {
                            nextItem.set('show', false);
                            nextItem.set('zIndex', ++this.currentZIndex);
                        }
                    }
                }, getPreviousItem: function () {
                    if (this.looping) {
                        if ((this.activeItem - 1) >= 0)
                            return this.models[ this.activeItem - 1];
                        else return this.models[this.models.length - 1];
                    } else {
                        if ((this.activeItem - 1) >= 0)
                            return this.models[ this.activeItem - 1];
                    }
                    return null;
                }, getNextItem: function () {
                    if (this.looping) {
                        if ((this.activeItem + 1) < this.models.length)
                            return this.models[ this.activeItem + 1];
                        else
                            return this.models[0];
                    } else {
                        if ((this.activeItem + 1) < this.models.length)
                            return this.models[ this.activeItem + 1];
                    }

                    return null;
                },
                lastItemActive: function () {
                    return (null == this.getNextItem());
                },
                firstItemActive: function () {
                    return (null == this.getPreviousItem());
                }
            });
        };
        this.instantiateObjects = function () {
            var imageWrapperSize = this.getImageWrapperSize(), arrowsModels, i, model;
            that.itemsWrapperModel = this.getItemModel(-1, imageWrapperSize, true);
            new this.ItemWrapperView({
                model: that.itemsWrapperModel
            });

            this.itemsCollection = new this.ItemsCollection();


            arrowsModels = this.getArrowsModels();
            that.arrowsCollection = new Backbone.Collection();
            for (i = 0; i < arrowsModels.length; i++) {
                new that.ArrowView({
                    model: arrowsModels[i]
                });
                this.arrowsCollection.add(arrowsModels[i]);
            }
            this.dotsBarModel = this.getDotsBarModel();
            new this.DotBarView({
                model: that.dotsBarModel
            });
            this.dotsCollection = new Backbone.Collection();
            for (i = 0; i < animationItems.length; i++) {
                model = this.getItemModel(i, imageWrapperSize, false);
                this.itemsCollection.add(model);
                this.itemsCollection.setCurrentZIndex(model.get('zIndex'));
                new this.ItemView({
                    model: model
                });
                model = this.getDotModel(i);
                this.dotsCollection.add(model);
                new this.DotView({
                    model: model
                });
            }
            this.fullScreenButtonModel = new Backbone.Model({
                imageURL: animationElementsConstants.FULl_SCREEN_BTN_URL
            });
            new this.FullScreenButtonView({
                model: that.fullScreenButtonModel
            });
        };

        this.getImageWrapperSize = function () {
            var size = {};
            sceneAnimationWidth = Math.round(App.settings.width * parseFloat(sceneAnimationModel.get('width')) / 100);
            sceneAnimationHeight = Math.round(App.settings.height * parseFloat(sceneAnimationModel.get('height')) / 100);
            size.absoluteWidth = Math.round(sceneAnimationWidth - 2 * animationSettings['horizontalPadding']);
            size.absoluteHeight = Math.round(sceneAnimationHeight - 2 * animationSettings['verticalPadding']);
            size.width = (100 * size.absoluteWidth / sceneAnimationWidth) + '%';
            size.height = (100 * size.absoluteHeight / sceneAnimationHeight) + '%';
            return size;
        };
        this.getArrowsModels = function () {
            var firstArrowModel = new Backbone.Model(),
                secondArrowModel = new Backbone.Model(),
                ret = [];
            this.setDefaultArrowProperties(firstArrowModel);
            this.setDefaultArrowProperties(secondArrowModel);
            firstArrowModel.set('action', 'decrease');
            secondArrowModel.set('action', 'increase');
            if (animationSettings['transitionEffect'] != animationEffects.VERTICAL_SLIDE) {
                firstArrowModel.set('imageURL', animationElementsConstants.LEFT_ARROW_URL);
                firstArrowModel.set('top', 50 * (sceneAnimationHeight - animationElementsConstants.ARROW_HEIGHT) / sceneAnimationHeight + '%');
                firstArrowModel.set('left', 0);
                secondArrowModel.set('imageURL', animationElementsConstants.RIGHT_ARROW_URL);
                secondArrowModel.set('top', 50 * (sceneAnimationHeight - animationElementsConstants.ARROW_HEIGHT) / sceneAnimationHeight + '%');
                secondArrowModel.set('right', 0);
            } else {
                firstArrowModel.set('imageURL', animationElementsConstants.TOP_ARROW_URL);
                firstArrowModel.set('left', 50 * (sceneAnimationWidth - animationElementsConstants.ARROW_WIDTH) / sceneAnimationWidth + '%');
                firstArrowModel.set('top', 0);
                secondArrowModel.set('imageURL', animationElementsConstants.BOTTOM_ARROW_URL);
                secondArrowModel.set('left', 50 * (sceneAnimationWidth - animationElementsConstants.ARROW_WIDTH) / sceneAnimationWidth + '%');
                secondArrowModel.set('bottom', 0);
            }
            ret.push(firstArrowModel);
            ret.push(secondArrowModel);
            return ret;
        };
        this.setDefaultArrowProperties = function (model) {
            model.set('width', animationElementsConstants.ARROW_WIDTH);
            model.set('height', animationElementsConstants.ARROW_HEIGHT);
            model.set('left', -1);
            model.set('right', -1);
            model.set('top', -1);
            model.set('bottom', -1);
            model.set('verticalMargin', 0);
            model.set('horizontalMargin', 0);
        };

        this.getDotsBarModel = function () {
            var dotsNumber = animationItems.length,
                dotSize,
                maxSize = dotsNumber * (animationElementsConstants.MAX_DOT_SIZE),
                animationSize, model, barDefaultSize;
            if (animationSettings['transitionEffect'] == animationEffects.VERTICAL_SLIDE) {
                animationSize = sceneAnimationHeight;
            } else {
                animationSize = sceneAnimationWidth;
            }
            if (animationSize > maxSize)
                animationSize = maxSize;

            dotSize = animationSize / dotsNumber;

            barDefaultSize = dotSize;
            dotSize = 100 * (dotSize / animationSize);

            model = new Backbone.Model({
                dotSize: dotSize + '%',
                dotMargin: 0,
                dotsNumber: dotsNumber,
                width: dotSize + '%',
                height: dotSize + '%',
                left: -1,
                top: -1
            });
            if (animationSettings['transitionEffect'] == animationEffects.VERTICAL_SLIDE) {
                var height = 100 * animationSize / sceneAnimationHeight;
                model.set('height', height + '%');
                model.set('width', 100 * barDefaultSize / sceneAnimationWidth + '%');
                model.set('top', ((100 - height) / 2) + '%');

            } else {
                var width = 100 * animationSize / sceneAnimationWidth;
                model.set('width', width + '%');
                model.set('height', 100 * barDefaultSize / sceneAnimationHeight + '%');
                model.set('left', ((100 - width) / 2) + '%');
            }
            return model;
        };
        this.getDotModel = function (index) {
            var model = new Backbone.Model({
                index: index,
                verticalMargin: 0,
                horizontalMargin: 0,
                imageURL: animationElementsConstants.DOT__OFF_URL
            });

            if (animationSettings['transitionEffect'] == animationEffects.VERTICAL_SLIDE) {
                model.set('width', '100%');
                model.set('height', this.dotsBarModel.get('dotSize'));
                model.set('verticalMargin', this.dotsBarModel.get('dotMargin'));
            } else {
                model.set('height', '100%');
                model.set('width', this.dotsBarModel.get('dotSize'));
                model.set('horizontalMargin', this.dotsBarModel.get('dotMargin'));
            }
            return model;
        };
        this.getItemModel = function (index, imageWrapperSize, isItemWrapper) {
            var model = new Backbone.Model({
                width: isItemWrapper ? imageWrapperSize.width : '100%',
                height: isItemWrapper ? imageWrapperSize.height : '100%',
                absoluteWidth: imageWrapperSize.absoluteWidth,
                absoluteHeight: imageWrapperSize.absoluteHeight,
                left: isItemWrapper ? (100 - parseFloat(imageWrapperSize.width)) / 2 + '%' : 0,
                top: isItemWrapper ? (100 - parseFloat(imageWrapperSize.height)) / 2 + '%' : 0,
                stretch: animationSettings['stretchItems'] == 'true',
                gapColor: animationSettings['gapColorAroundItems'],
                show: index == 0,
                index: index,
                opacity: 1,
                translate: {x: 0, y: 0},
                zIndex: index + 1
            });
            if (null != animationItems[index]) {
                var useCaption = (null != animationItems[index]['caption'] && animationItems[index]['caption'].length > 0 );
                model.set('imageURL', ResourceManager.getPathForImage(animationItems[index]['objectId'], animationItems[index]['fileExtension']));
                model.set('useCaptions', useCaption);
                if (useCaption) {
                    model.set('caption', animationItems[index].caption);
                    model.set('captionBackgroundColor', animationItems[index].captionBackgroundColor);
                    model.set('captionFontSize', parseFloat(animationItems[index].captionFontSize));
                    model.set('captionOpacity', parseFloat(animationItems[index].captionOpacity));
                    model.set('captionPosition', animationItems[index].captionPosition);
                    model.set('captionTextColor', animationItems[index].captionTextColor);
                    model.set('captionPadding', parseFloat(animationItems[index].captionPadding));
                }
            }
            return model;
        };
        this.removeItemFromViewPort = function (view) {
            var transitionEffect = animationSettings['transitionEffect'], translateOffset, translateObject;
            if (transitionEffect == animationEffects.VERTICAL_SLIDE || transitionEffect == animationEffects.SLIDEDOWN || transitionEffect == animationEffects.SLIDEUP) {
                translateOffset = parseFloat(view.model.get('translate').y) + parseFloat(view.model.get('height')) * view.model.get('direction');
                translateObject = {};
                translateObject.x = 0;
                translateObject.y = translateOffset + '%';
                view.model.set('translate', translateObject);
                that.removingTransitionInProgress = true;
            } else if (transitionEffect == animationEffects.HORIZONTAL_SLIDE || transitionEffect == animationEffects.SLIDERIGHT || transitionEffect == animationEffects.SLIDELEFT) {
                translateOffset = parseFloat(view.model.get('translate').x) + parseFloat(view.model.get('width')) * view.model.get('direction');
                translateObject = {};
                translateObject.x = translateOffset + '%';
                translateObject.y = 0;
                view.model.set('translate', translateObject);
                that.removingTransitionInProgress = true;
            } else if (transitionEffect == animationEffects.FADE) {

            } else if (transitionEffect == animationEffects.APPEAR) {

            }
        };
        this.displayItemInViewPort = function (view) {

            var transitionEffect = animationSettings['transitionEffect'], translateOffset, translateObject;
            if (transitionEffect == animationEffects.VERTICAL_SLIDE || transitionEffect == animationEffects.SLIDEDOWN || transitionEffect == animationEffects.SLIDEUP) {
                translateOffset = parseFloat(view.model.get('width')) * view.model.get('direction');
                translateObject = {};
                translateObject.x = 0;
                translateObject.y = translateOffset + '%';
                view.model.set('show', true);
                view.model.set('transitionEndHandler', function () {
                    view.model.set('transitionEndHandler', null);
                    that.displayingTransitionInProgress = false;
                    that.removingTransitionInProgress = false;
                });
                view.model.set('translate', translateObject);
                this.displayingTransitionInProgress = true;
            } else if (transitionEffect == animationEffects.HORIZONTAL_SLIDE || transitionEffect == animationEffects.SLIDELEFT || transitionEffect == animationEffects.SLIDERIGHT) {
                translateOffset = parseFloat(view.model.get('height')) * view.model.get('direction');
                translateObject = {};
                translateObject.x = translateOffset + '%';
                translateObject.y = 0;
                view.model.set('show', true);
                view.model.set('transitionEndHandler', function () {
                    view.model.set('transitionEndHandler', null);
                    that.displayingTransitionInProgress = false;
                    that.removingTransitionInProgress = false;
                });
                view.model.set('translate', translateObject);
                this.displayingTransitionInProgress = true;
            } else if (transitionEffect == animationEffects.FADE) {
                view.model.set('addClassName', 'fadeTransition');
                view.model.set('transitionEndHandler', function () {
                    view.model.set('transitionEndHandler', null);
                    that.displayingTransitionInProgress = false;
                    that.removingTransitionInProgress = false;
                    view.model.set('removeClassName', 'fadeTransition');
                });
                view.model.set('show', true);
            } else if (transitionEffect == animationEffects.APPEAR) {
                view.model.set('show', true);
            }
        };
        this.startAnimationIfNeeded = function (activatedObject) {
            var displayDuration = (null == animationSettings['displayDuration']) ? animationConstants.DEFAULT_DISPLAY_DURATION : parseFloat(animationSettings['displayDuration']),
                transitionDuration = (null == animationSettings['transitionDuration']) ? animationConstants.DEFAULT_TRANSITION_DURATION : parseFloat(animationSettings['transitionDuration']);
            displayDuration = (displayDuration <= 0) ? animationConstants.DEFAULT_DISPLAY_DURATION : displayDuration;
            transitionDuration = (transitionDuration <= 0) ? animationConstants.DEFAULT_TRANSITION_DURATION : transitionDuration;
            displayDuration += transitionDuration;
            if (activatedObject)
                that.autoAnimationTimer = setInterval(function () {
                    that.itemsCollection.setNextActiveItem();
                }, displayDuration);
        };
        return {
            init: function () {
                that.defineClasses();
                that.instantiateObjects();
            },
            getItemsWrapperModel: function () {
                return that.itemsWrapperModel;
            },
            getDotsBarModel: function () {
                return that.dotsBarModel;
            },
            getArrowsCollection: function () {
                return that.arrowsCollection;
            },
            getFullScreenButtonModel: function () {
                return that.fullScreenButtonModel;
            },
            setActiveItem: function (itemIndex) {
                that.itemsCollection.setActiveItem(itemIndex);
            },
            startAnimationIfNeeded: function (activatedObject) {
                that.startAnimationIfNeeded(activatedObject);
            }
        }
    }
});