define(['DrawingElementMenu'], function (DrawingElementMenu) {
    return function (rawEmptyBoxModel, pageModel) {
        var that = this ,
            INACTIVE_DRAWING_MODE_PERIOD = 5000,
            ERASER_SIZE = 20,
            mappedBorderStyleCSSClasses = {
                'Solid': 'divSolidBorder',
                'No border': 'divNoBorder',
                'Dotted': 'divDottedBorder',
                'Dashed': 'divDashedBorder',
                'Double': 'divDoubleBorder',
                'Groove': 'divGrooveBorder',
                'Ridge': 'divRidgeBorder'
            };
        this.defineClasses = function () {

            that.DrawingElementView = ElementBaseView.extend({
                    tagName: "div",
                    className: "sceneObject",
                    elementsViews: [],
                    id: rawEmptyBoxModel.properties.id,
                    template: templates.getTemplate("drawingElementCanvas"),
                    parentElementModel: pageModel,
                    $canvas: null,
                    canvas: null,
                    context: null,
                    displayMode: 'landscapeMode',
                    mousePressed: false,
                    previousPosition: null,
                    position: null,
                    inactiveDrawingModeTimer: null,
                    menu: null,
                    initialize: function (attr) {
                        this.listenTo(this.model, "change:tool", this.activateTool);
                        this.listenTo(this.model, "change:lineColor", this.setColor);
                        this.listenTo(this.model, "change:removeContent", this.removeContent);
                        this.listenTo(this.model, "change:drawing", this.changeState);
                        this.listenTo(this.model, "change:cancelInactiveDrawingModeTimer", this.clearInactiveDrawingModeTimer);
                        this.listenTo(this.model, "change:canvasId", this.loadStoredDrawing);
                        this.listenTo(this.model, "change:shouldClear", this.shouldClear);
                        this.listenTo(this.model, "change:displayed", this.showHide);
                        this.updateOCForModel();
                        this.notifyParentThatObjectIsLoaded();
                    },
                    events: function () {
                        var _events = {};
                        _events[AppUtils.EventTypes.mouseMove] = "mouseMoveHandler";
                        _events[AppUtils.EventTypes.mouseDown] = "mouseDownHandler";
                        _events[AppUtils.EventTypes.mouseUp] = "mouseUpHandler";

                        return _events;
                    },

                    mouseDownHandler: function (e) {
                        if (this.model.get('drawing')) {
                            e.stopPropagation();
                            e.preventDefault();
                        }
                        if (e.button != 2) { // discard right click, that we use for debugging
                            this.mousePressed = true;
                            this.previousPosition = this.getPosition(e);
                            this.previousPosition.x -= (App.settings.left + this.model.get(this.displayMode + 'Position').left);
                            this.previousPosition.y -= (App.settings.top + this.model.get(this.displayMode + 'Position').top);
                            this.position = this.getPosition(e);
                            this.model.set('cancelInactiveDrawingModeTimer', true);
                        }
                    },
                    clearInactiveDrawingModeTimer: function () {
                        if (this.model.get('cancelInactiveDrawingModeTimer')) {
                            if (null != this.inactiveDrawingModeTimer) {
                                window.clearTimeout(this.inactiveDrawingModeTimer);
                            }
                        } else {
                            this.startInactiveDrawingModeTimer();
                        }
                    },
                    mouseUpHandler: function (e) {
                        if (this.model.get('drawing')) {
                            e.stopPropagation();
                            e.preventDefault();
                        }
                        if (e.button != 2) { // discard right click, that we use for debugging
                            this.mousePressed = false;
                            if (!this.model.get('drawing')) {
                                if (this.tapEvent(e)) {
                                    this.model.set('drawing', true);
                                }
                            }
                            this.model.set('cancelInactiveDrawingModeTimer', false);
                        }
                    },
                    startInactiveDrawingModeTimer: function () {
                        var thisView = this;
                        this.inactiveDrawingModeTimer = window.setTimeout(function () {
                            thisView.model.set('drawing', false);
                        }, INACTIVE_DRAWING_MODE_PERIOD);
                    },
                    changeState: function () {
                        if (this.model.get('drawing')) {
                            this.menu.getModel().set('visibility', true);
                            this.$el.css('background-image', '');
                            this.$el.removeClass('drawingObject');
                        } else {
                            if (!this.model.get('canvasDirty')) {
                                this.$el.css('background-image', 'url(' + this.model.get("drawingIconURL") + ')');
                                this.$el.addClass('drawingObject');
                            }
                            else {
                                this.saveDrawing();
                            }
                            this.menu.getModel().set('visibility', false);
                        }
                    },
                    mouseMoveHandler: function (e) {

                        if (this.model.get('drawing')) {
                            e.stopPropagation();
                            e.preventDefault();
                        }
                        if (this.mousePressed) {

                            var currentPosition = this.getPosition(e);
                            //console.log("TUDOR MOVE" + currentPosition.x + "     " + currentPosition.y);
                            currentPosition.x -= (App.settings.left + this.model.get(this.displayMode + 'Position').left);
                            currentPosition.y -= (App.settings.top + this.model.get(this.displayMode + 'Position').top);
                            this.draw(currentPosition);

                        }
                    },
                    setColor: function () {
                        this.context.strokeStyle = this.model.get('lineColor');
                    },
                    activateTool: function () {

                        if (this.model.get('tool') == 'eraser') {
                            this.context.lineWidth = ERASER_SIZE;
                            this.context.globalCompositeOperation = 'destination-out';
                        }
                        else {
                            this.context.lineWidth = parseFloat(this.model.get('markerSize'));
                            this.context.globalCompositeOperation = 'source-over';
                            this.context.lineJoin = "round";
                        }
                        this.setColor();
                    },
                    removeContent: function () {
                        if (this.model.get('removeContent')) {
                            this.model.set('removeContent', false);
                            var size = this.model.get(this.displayMode + 'Size');
                            this.context.clearRect(0, 0, size.width, size.height);
                            localStorage.removeItem(this.model.get('canvasId'));
                        }
                    },
                    saveDrawing: function () {
                        localStorage.setItem(this.model.get('canvasId'), this.canvas.toDataURL());
                    },
                    loadStoredDrawing: function () {
                        var thisView = this,
                            imageObj,
                            imageSrc = (null == this.model.get('canvasId')) ? null : localStorage.getItem(this.model.get('canvasId')),
                            size;
                        if (null != imageSrc) {
                            imageObj = new Image();
                            size = this.model.get(this.displayMode + 'Size');
                            imageObj.onload = function () {
                                thisView.context.drawImage(this, 0, 0, size.width, size.height);
                            };
                            this.model.set('canvasDirty', true);
                            imageObj.src = imageSrc;
                            this.$el.css('background-image', '');
                            this.$el.removeClass('drawingObject');
                        } else {
                            this.model.set('canvasDirty', false);
                            this.$el.css('background-image', 'url(' + this.model.get("drawingIconURL") + ')');
                            this.$el.addClass('drawingObject');
                        }
                    },

                    draw: function (currentPosition) {

                        if (this.model.get('drawing')) {
                            //console.log("TUDOR DRAW");
                            if (!this.model.get('canvasDirty') && this.model.get('tool'))
                                this.model.set('canvasDirty', true);
                            this.context.beginPath();
                            this.context.moveTo(this.previousPosition.x, this.previousPosition.y);
                            this.context.lineTo(currentPosition.x, currentPosition.y);
                            this.context.closePath();
                            this.context.stroke();
                            this.previousPosition = currentPosition;
                        }
                    },

                    removeView: function () {
                        if (this.model.get("removed")) {
                            this.remove();
                            this.unbind();
                        }
                    },
                    activateObject: function () {
//                   // do nothing
                        //Needed here because is called from ElementBaseView
                    },
                    activateToggle: function () {
                        this.displayHideOnceOnClick();
                    },

                    updateCanvasElement: function () {
                        var size = this.model.get(this.displayMode + 'Size');
                        this.canvas.width = size.width;
                        this.canvas.height = size.height;
                    },
                    addMenu: function () {
                        this.menu = new DrawingElementMenu();
                        this.menu.init(this.model);
                        this.menu.getModel().set('visibility', false);
                        this.$el.append(this.menu.getView().render().el);
                    },
                    setBorder: function () {
                        this.$el.addClass(mappedBorderStyleCSSClasses[this.model.get('borderStyle')]);
                    },
                    /**
                     *  function used to clear the drawing element
                     */
                    shouldClear: function () {
                        if (this.model.get("shouldClear")) {
                            var size = this.model.get(this.displayMode + 'Size');
                            this.context.clearRect(0, 0, size.width, size.height);
                            this.model.set("canvasDirty", false);
                            this.model.set("drawing", false);
                            this.model.set("shouldClear", false);
                            this.$el.addClass('drawingObject');
                            this.$el.css('background-image', 'url(' + this.model.get("drawingIconURL") + ')');
                        }
                    },
                    /**
                     * function used to change between the text and drawing view of the note
                     */
                    showHide: function () {
                        if (this.model.get("displayed")) {
                            this.$el.css("display", "block");
                        }
                        else {
                            this.$el.css("display", "none");
                        }
                        console.log("DrawingView showHide");
                    },

                    render: function () {
                        if (this.shouldRenderObject()) {
                            this.$canvas = $(this.template(this.model.attributes));
                            this.canvas = this.$canvas[0];
                            this.context = this.canvas.getContext('2d');
                            this.setBorder();
                            this.updateCanvasElement();
                            this.activateTool();

                            this.setLeft(this.model.get("left"));
                            this.$el.html(this.canvas);

                            this.setWidth(this.model.get("width"));
                            this.$el.css('background-image', 'url(' + this.model.get("drawingIconURL") + ')');
                            this.setHeight(this.model.get("height"));
                            this.setTop(this.model.get("top"));
                            this.$el.css("z-index", this.model.get("ZIndex"));
                            this.$el.css('overflow', 'visible');
                            this.setInitialVisibility();

                            this.addMenu();
                            this.loadStoredDrawing();

//                            this.changeState();
                            return this;
                        }
                        return null;
                    }
                }
            )
            ;
        };
        this.instantiateObjects = function () {
            this.drawingElementModel = this.getDrawingElementModelModel();
            this.drawingElementView = new this.DrawingElementView({
                model: that.drawingElementModel
            });

        };

        this.getDrawingElementModelModel = function () {
            var model = new Backbone.Model(rawEmptyBoxModel.properties);
            model.set('canvasId', rawEmptyBoxModel.properties.id + '_canvas');
            model.set('drawingIconURL', App.drawingIconURL);
            model.set('displayed', rawEmptyBoxModel.properties.displayed);
            model.set('landscapeModeSize', this.getSize(rawEmptyBoxModel.properties.width, rawEmptyBoxModel.properties.height, App.settings.width, App.settings.height));
            model.set('portraitModeSize', this.getSize(rawEmptyBoxModel.properties.width, rawEmptyBoxModel.properties.height, App.settings.height, App.settings.width));
            model.set('landscapeModePosition', this.getFixedPosition(rawEmptyBoxModel.properties.left, rawEmptyBoxModel.properties.top, App.settings.width, App.settings.height));
            model.set('portraitModePosition', this.getFixedPosition(rawEmptyBoxModel.properties.left, rawEmptyBoxModel.properties.top, App.settings.height, App.settings.width));
            model.set('tool', 'marker');
            model.set('drawing', false);
            return model;
        };

        this.getSize = function (percentageWidth, percentageHeight, wrapperWidth, wrapperHeight) {
            var width, height;
            width = Math.round(parseFloat(percentageWidth) * wrapperWidth / 100);
            height = Math.round(parseFloat(percentageHeight) * wrapperHeight / 100);
            return {
                width: width,
                height: height
            }
        };
        this.getFixedPosition = function (percentageLeft, percentageTop, wrapperWidth, wrapperHeight) {
            var left, top;
            left = Math.round(parseFloat(percentageLeft) * wrapperWidth / 100);
            top = Math.round(parseFloat(percentageTop) * wrapperHeight / 100);
            return {
                left: left,
                top: top
            }
        };

        return {
            init: function () {
                that.defineClasses();
                that.instantiateObjects();
            },
            getView: function () {
                return that.drawingElementView;
            },
            getModel: function () {
                return that.drawingElementModel;
            }

        }
    }
})
;