define(function () {
    return function () {
        var that = this,
            defaultAppWidth = 1024,
            defaultAppHeight = 768,
            defaultBtnSize = 80,
            defaultMenuLeft = -82,
            mappedColors = {
                hexToString: {
                    '#FF0000': 'red',
                    '#FFC600': 'yellow',
                    '#4BA106': 'green',
                    '#0072FF': 'blue',
                    '#000000': 'black'},
                stringToHex: {
                    'red': '#FF0000',
                    'yellow': '#FFC600',
                    'green': '#4BA106',
                    'blue': '#0072FF',
                    'black': '#000000'
                }
            };

        this.defineClasses = function () {

            that.DrawingElementMenuView = BaseView.extend({
                    tagName: "div",
                    className: 'drawingElementMenu',
                    initialize: function (attr) {
                        this.listenTo(this.model, "change:visibility", this.changeVisibility);
                    },

                    events: function () {
                        var _events = {};
                        _events[AppUtils.EventTypes.mouseMove] = "mouseMoveHandler";
                        _events[AppUtils.EventTypes.mouseDown] = "mouseDownHandler";
                        _events[AppUtils.EventTypes.mouseUp] = "mouseUpHandler";
                        return _events;
                    },
                    mouseDownHandler: function (e) {
                        e.stopPropagation();
                        this.model.get('drawingElementModel').set('cancelInactiveDrawingModeTimer', true);
                    },
                    mouseUpHandler: function (e) {
                        this.model.get('drawingElementModel').set('cancelInactiveDrawingModeTimer', false);
                        e.stopPropagation(e);
                    },
                    appendColorButtonsIfNeeded: function () {
                        if (this.model.get('areColorsEnabled')) {
                            var idx, view, models = that.colorsCollection.models;
                            for (idx = 0; idx < models.length; idx++) {
                                view = new that.ColorBtnView({
                                    model: models[idx]
                                });
                                this.$el.append(view.render().el);
                            }

                        }
                    },
                    changeVisibility: function () {
                        if (this.model.get('visibility')) {
                            this.$el.show();
                        } else {
                            this.$el.hide();
                        }
                    },
                    render: function () {
                        this.setWidth(this.model.get('width'));
                        this.$el.append(that.clearBtnView.render().el)
                            .append(that.eraserBtnView.render().el)
                            .append(that.paintBtnView.render().el);
                        this.appendColorButtonsIfNeeded();
                        this.setLeft(this.model.get('left'));
                        return this;
                    }
                }
            );
            that.PaintBtnView = BaseView.extend({
                className: 'paintBrushBtnActive drawingElementMenuButton',
                position: {},
                initialize: function (attr) {
                    this.listenTo(this.model, "change:active", this.activateBtn);
                },
                events: function () {
                    var _events = {};
                    _events[AppUtils.EventTypes.mouseDown] = "mouseDownHandler";
                    _events[AppUtils.EventTypes.mouseUp] = "mouseUpHandler";
                    return _events;
                },
                mouseDownHandler: function (e) {
                    if (!this.model.get('active')) {
                        this.position = this.getPosition(e);
                        this.$el.toggleClass('paintBrushBtnActive');
                        this.$el.toggleClass('paintBrushBtnInactive');
                    }
                },
                mouseUpHandler: function (e) {
                    if (!this.model.get('active')) {
                        if (this.tapEvent(e)) {
                            this.model.set('active', true);
                            that.eraserBtnModel.set('active', false);
                        } else {
                            this.$el.toggleClass('paintBrushBtnInactive');
                            this.$el.toggleClass('paintBrushBtnActive');
                        }
                    }
                },
                activateBtn: function () {
                    if (this.model.get('active')) {
                        this.$el.removeClass('paintBrushBtnInactive');
                        this.$el.addClass('paintBrushBtnActive');
                        this.model.get('drawingElementModel').set('tool', 'marker');
                    } else {
                        this.$el.removeClass('paintBrushBtnActive');
                        this.$el.addClass('paintBrushBtnInactive')
                    }
                },
                render: function () {
                    this.setWidth(this.model.get('width'));
                    this.setHeight(this.model.get('height'));
                    return this;
                }
            });
            that.EraserBtnView = BaseView.extend({
                className: 'eraserBtnInactive drawingElementMenuButton',
                position: {},
                initialize: function (attr) {
                    this.listenTo(this.model, "change:active", this.activateBtn);
                },
                events: function () {
                    var _events = {};
                    _events[AppUtils.EventTypes.mouseDown] = "mouseDownHandler";
                    _events[AppUtils.EventTypes.mouseUp] = "mouseUpHandler";
                    return _events;
                },
                mouseDownHandler: function (e) {
                    if (!this.model.get('active')) {
                        this.position = this.getPosition(e);
                        this.$el.toggleClass('eraserBtnInactive');
                        this.$el.toggleClass('eraserBtnActive');
                    }
                },
                mouseUpHandler: function (e) {
                    if (!this.model.get('active')) {
                        if (this.tapEvent(e)) {
                            this.model.set('active', true);
                            that.paintBtnModel.set('active', false);
                        } else {
                            this.$el.toggleClass('eraserBtnActive');
                            this.$el.toggleClass('eraserBtnInactive');
                        }
                    }
                },
                activateBtn: function () {

                    if (this.model.get('active')) {
                        this.$el.removeClass('eraserBtnInactive');
                        this.$el.addClass('eraserBtnActive');
                        this.model.get('drawingElementModel').set('tool', 'eraser');
                    } else {
                        this.$el.removeClass('eraserBtnActive');
                        this.$el.addClass('eraserBtnInactive');
                    }
                },
                render: function () {
                    this.setWidth(this.model.get('width'));
                    this.setHeight(this.model.get('height'));
                    return this;
                }
            });
            that.ClearBtnView = BaseView.extend({

                position: {},
                className: 'clearDrawingElementBtnActive drawingElementMenuButton',
                events: function () {
                    var _events = {};
                    _events[AppUtils.EventTypes.mouseDown] = "mouseDownHandler";
                    _events[AppUtils.EventTypes.mouseUp] = "mouseUpHandler";
                    return _events;
                },
                mouseDownHandler: function (e) {
                    this.position = this.getPosition(e);
//                    this.$el.toggleClass('clearDrawingElementBtnActive');
//                    this.$el.toggleClass('clearDrawingElementBtnInactive');
                },
                mouseUpHandler: function (e) {
//                    this.$el.toggleClass('clearDrawingElementBtnActive');
//                    this.$el.toggleClass('clearDrawingElementBtnInactive');
                    if (this.tapEvent(e)) {
                        this.model.get('drawingElementModel').set('removeContent', true);
                        this.model.get('drawingElementModel').set('canvasDirty', false);
                        that.eraserBtnModel.set('active', false);
                        that.paintBtnModel.set('active', true);
                    }
                },
                render: function () {
                    this.setWidth(this.model.get('width'));
                    this.setHeight(this.model.get('height'));
                    return this;
                }
            });
            that.ColorBtnView = BaseView.extend({
                className: function () {
                    if (this.model.get('active')) {

                        return this.model.get('className').active;
                    } else {
                        return this.model.get('className').inactive;
                    }
                },
                initialize: function (attr) {
                    this.listenTo(this.model, "change:active", this.activateBtn);
                },
                events: function () {
                    var _events = {};
                    _events[AppUtils.EventTypes.click] = "clickHandler";
                    return _events;
                },
                clickHandler: function (e) {
                    // TODO: the logic here should respect the one in ElementsBaseView:
                    // first touch with menu open should close the menu.
                    // For now we can let activate the object with at the same click
                    if (!this.model.get('active')) {
                        var idx, models = that.colorsCollection.models;
                        for (idx = 0; idx < models.length; idx++) {
                            models[idx].set('active', false);
                        }
                        this.model.set('active', true);
                    }
                },
                activateBtn: function () {


                    if (this.model.get('active')) {
                        this.$el.removeClass(this.model.get('className').inactive);
                        this.$el.addClass(this.model.get('className').active);
                        this.model.get('drawingElementModel').set('lineColor', this.model.get('hexColor'));
                        that.paintBtnModel.set('active', true);
                        that.eraserBtnModel.set('active', false);
                    } else {
                        this.$el.removeClass(this.model.get('className').active);
                        this.$el.addClass(this.model.get('className').inactive);
                    }
                },
                render: function () {
                    this.setWidth(this.model.get('width'));
                    this.setHeight(this.model.get('height'));
                    return this;
                }
            });
        };
        this.instantiateObjects = function (drawingElementModel) {
            var btnSize = this.getButtonSize();
            this.drawingElementMenuModel = this.getDrawingElementMenuModel(drawingElementModel, btnSize);


            this.paintBtnModel = this.getDrawingElementMenuBtnModel(drawingElementModel, btnSize);
            this.paintBtnModel.set('active', true);

            this.eraserBtnModel = this.getDrawingElementMenuBtnModel(drawingElementModel, btnSize);
            this.eraserBtnModel.set('active', false);

            this.clearBtnModel = this.getDrawingElementMenuBtnModel(drawingElementModel, btnSize);

            this.setupColorButtons(drawingElementModel, btnSize);


            this.drawingElementMenuView = new this.DrawingElementMenuView({
                model: that.drawingElementMenuModel,
                id: that.drawingElementMenuModel.get('id')
            });
            this.paintBtnView = new this.PaintBtnView({
                model: that.paintBtnModel
            });
            this.eraserBtnView = new this.EraserBtnView({
                model: that.eraserBtnModel
            });
            this.clearBtnView = new this.ClearBtnView({
                model: that.clearBtnModel
            });

        };

        this.getDrawingElementMenuModel = function (drawingElementModel, btnSize) {
            var model = new Backbone.Model();
            model.set('areColorsEnabled', drawingElementModel.get('areColorsEnabled'));
            model.set('color', drawingElementModel.get('lineColor'));
            model.set('drawingElementModel', drawingElementModel);
            model.set('height', btnSize);
            model.set('width', btnSize);
            model.set('left', this.getMenuLeft());
            model.set('id', 'drawingMenu_' + drawingElementModel.get("id"));
            return model;
        };
        this.getDrawingElementMenuBtnModel = function (drawingElementModel, btnSize) {
            var model = new Backbone.Model();

            model.set('width', btnSize);
            model.set('height', btnSize);
            model.set('drawingElementModel', drawingElementModel);
            return model;
        };
        this.getButtonSize = function () {
            var size = defaultBtnSize;
            if (App.settings.width < defaultAppWidth || App.settings.height < defaultAppHeight) {
                var scaleFactor = Math.min(App.settings.width / defaultAppWidth, App.settings.height / defaultAppHeight);
                size = Math.round(defaultBtnSize * scaleFactor);
            }
            return size;
        };

        this.getMenuLeft = function () {
            var left = defaultMenuLeft;
            if (App.settings.width < defaultAppWidth || App.settings.height < defaultAppHeight) {
                var scaleFactor = Math.min(App.settings.width / defaultAppWidth, App.settings.height / defaultAppHeight);
                left = Math.round(defaultMenuLeft * scaleFactor);
            }
            return left;
        };

        this.getSize = function (percentageWidth, percentageHeight, wrapperWidth, wrapperHeight) {
            var width, height;
            width = Math.round(parseFloat(percentageWidth) * wrapperWidth / 100);
            height = Math.round(parseFloat(percentageHeight) * wrapperHeight / 100);
            return {
                width: width,
                height: height
            }
        };
        this.setupColorButtons = function (drawingElementModel, btnSize) {
            this.colorsCollection = new Backbone.Collection();
            this.colorsCollection.add(this.getColorBtnModel(drawingElementModel, 'red', btnSize));
            this.colorsCollection.add(this.getColorBtnModel(drawingElementModel, 'yellow', btnSize));
            this.colorsCollection.add(this.getColorBtnModel(drawingElementModel, 'green', btnSize));
            this.colorsCollection.add(this.getColorBtnModel(drawingElementModel, 'blue', btnSize));
            this.colorsCollection.add(this.getColorBtnModel(drawingElementModel, 'black', btnSize));
        };
        this.getColorBtnModel = function (drawingElementModel, btnColor, btnSize) {
            var hexColor = drawingElementModel.get('lineColor');
            var model = new Backbone.Model({
                className: that.getClassNameForColorButton(btnColor),
                active: btnColor == mappedColors.hexToString[hexColor],
                drawingElementModel: drawingElementModel,
                hexColor: mappedColors.stringToHex[btnColor],
                color: btnColor,
                width: btnSize,
                height: btnSize
            });
            return model;
        };

        this.getClassNameForColorButton = function (buttonColor) {
            var className = {};
            className.active = buttonColor + 'BtnActive drawingElementMenuButton';
            className.inactive = buttonColor + 'BtnInactive drawingElementMenuButton';
            return className;
        };
        return {
            init: function (drawingElementModel) {
                that.defineClasses(drawingElementModel);
                that.instantiateObjects(drawingElementModel);
            },
            getView: function () {
                return that.drawingElementMenuView;
            },
            getModel: function () {
                return that.drawingElementMenuModel;
            }

        }
    }
})
;