(function ($) {

    var methods = {

        init: function (options) {

            var $this = this;
            options.transitionInProgress = false;
            // here we could save all options to be used in other methods than init
            options.swipeWidth = $this.width();
            options.swipeHeight = $this.height();
            options.currentIndex = options.startIndex;
            var wrapper = $("<div class='swipeWrapper'></div>")
                .height(options.swipeHeight)
                .width(options.swipeWidth * options.questionSceneObjects.length)
                .css("left", -options.swipeWidth * options.startIndex + "px");
            options.wrapper = options.testObject.$el;

//            $this.append(wrapper);

            var leftIndex = methods._previousIndex(options);
            var centerIndex = options.currentIndex;
            var rightIndex = methods._nextIndex(options);

            for (var idx = 0; idx < options.questionSceneObjects.length; idx++) {
                var oneSlide = $("<div class='swipeSlide'></div>")
                    .css({
                        "width": options.swipeWidth,
                        "height": options.swipeHeight
                    });
                if (AppUtils.nativeApp) {
                    oneSlide.addClass("noFlicker");
                }
                wrapper.append(oneSlide);
                if (leftIndex != null && idx == leftIndex) { // only if starting test not from the first question
                    options.questionSceneObjects[leftIndex].printQuestion(oneSlide);
                } else if (idx == centerIndex) {
                    options.questionSceneObjects[centerIndex].printQuestion(oneSlide);
                } else if (rightIndex != null && idx == rightIndex) {
                    if (options.questionSceneObjects[rightIndex] instanceof SceneObjectQuizQuestion) { // a regular question
                        options.questionSceneObjects[rightIndex].printQuestion(oneSlide);
                    } else { // there is only one question in the test and test summary is on the second slide
                        options.testObject.printTestSummary(oneSlide);
                    }
                }
            }

            $this.options = options;

            var questionSceneObject = options.questionSceneObjects[centerIndex];
            options.testObject.updateCurrentQuestion(questionSceneObject);

            return this;
        },

        resetToFirstQuestion: function () {
            var $this = this;
            $this.options.currentIndex = 0;
            var swipeWrapper = $this.options.wrapper;
            swipeWrapper.css(navigator.transitionPropertyPrefix + "transform", "translate3d(0,0,0)");
            swipeWrapper.bind(navigator.transitionEndEventName, function () {
                var lastQuestionIndex = $this.options.questionSceneObjects.length - 1;
                if (lastQuestionIndex > 1) { // more than one question, than empty test summary slide
                    swipeWrapper.find(".swipeSlide:nth-child(" + (lastQuestionIndex + 1) + ")").empty();
                }
                if (lastQuestionIndex > 2) { // if only 2 questions avoid deleting the second one
                    swipeWrapper.find(".swipeSlide:nth-child(" + lastQuestionIndex + ")").empty();
                }
            });
        },

        bindRightNavigation: function (args) {
            var $this = this;
            var navArrowEl = args[0];
            var eventType = args[1];
            navArrowEl.bind(eventType, function (e) {
                e.preventDefault();
                e.stopPropagation();
                methods._navigateRightHandler($this, navArrowEl);
            });
            $this.bind("swipeleft", function (e) {
                e.preventDefault();
                e.stopPropagation();
                methods._navigateRightHandler($this, navArrowEl);
            });
        },

        bindLeftNavigation: function (args) {
            var $this = this;
            var navArrowEl = args[0];
            var eventType = args[1];
            navArrowEl.bind(eventType, function (e) {
                e.preventDefault();
                e.stopPropagation();
                methods._navigateLeftHandler($this, navArrowEl);
            });
            $this.bind("swiperight", function (e) {
                e.preventDefault();
                e.stopPropagation();
                methods._navigateLeftHandler($this, navArrowEl);
            });
        },

        _navigateRightHandler: function ($this, navArrowEl) {
            var options = $this.options;
            if (!options.transitionInProgress && !navArrowEl.hasClass("navigationDisabled")) {
                var wrapper = options.wrapper;
                options.transitionInProgress = true;
                wrapper.bind(navigator.transitionEndEventName, function () {
                    $(this).unbind(navigator.transitionEndEventName);
                    var leftIndex = methods._previousIndex(options);
                    var rightIndex = methods._nextIndex(options);

                    var previousLeft, nowRight;
                    if (leftIndex != null && leftIndex >= 1) {
                        previousLeft = $(this).find(".swipeSlide:nth-child(" + leftIndex + ")");
                        previousLeft.empty();
                    }
                    if (rightIndex != null) {
                        nowRight = $(this).find(".swipeSlide:nth-child(" + (rightIndex + 1) + ")");
                    }

                    // compute score for question just visited
                    if ((options.currentIndex > 0) && (options.currentIndex < options.questionSceneObjects.length)) {
                        var previousVisitedQuestionSceneObject = options.questionSceneObjects[options.currentIndex - 1];
                        previousVisitedQuestionSceneObject.computeQuestionScoreAndOptionLevelFeedback();
                    }

                    // now add new question / test summary content
                    if (rightIndex != null) {
                        var nextQuestionIndex;
                        if (options.questionSceneObjects[rightIndex] instanceof SceneObjectQuizQuestion) {
                            nextQuestionIndex = options.questionSceneObjects[rightIndex].getQuestionObject().questionNo;
                        } else {
                            nextQuestionIndex = options.questionSceneObjects[rightIndex].questionNo;
                        }
                        if (nextQuestionIndex == options.questionSceneObjects.length) { // show test summary
                            options.testObject.printTestSummary(nowRight);
                        } else { // print next question
                            options.questionSceneObjects[rightIndex].printQuestion(nowRight);
                        }
                    }
                    // update current visible question
                    if (options.currentIndex < options.questionSceneObjects.length - 1) {
                        options.testObject.updateCurrentQuestion(options.questionSceneObjects[options.currentIndex]);
                    } else {
                        options.testObject.updateTestSummaryScreenAfterTransition();
                    }
                    options.transitionInProgress = false;
                });
                if (options.currentIndex < options.questionSceneObjects.length - 1) {
                    var questionSlideEl = wrapper.find(".swipeSlide:nth-child(" + (options.currentIndex + 1) + ")");
                    options.testObject.stopPlayingMultimediaObjects(questionSlideEl);
                    options.questionSceneObjects[options.currentIndex].stopQuestionAnimation();
                }
                options.currentIndex = methods._nextIndex(options);
                // prepare test summary before it becomes visible
                if (options.currentIndex == options.questionSceneObjects.length - 1) {
                    options.testObject.prepareTestSummaryBeforeTransition();
                }
                wrapper.css(navigator.transitionPropertyPrefix + "transition", "all " + options.transitionDuration + "ms ease-in-out")
                    .css(navigator.transitionPropertyPrefix + "transform", "translate3d(-" + (options.swipeWidth * options.currentIndex) + "px,0,0)");
            }
        },

        _navigateLeftHandler: function ($this, navArrowEl) {
            var options = $this.options;
            if (!options.transitionInProgress && !navArrowEl.hasClass("navigationDisabled")) {
                var wrapper = options.wrapper;
                options.transitionInProgress = true;
                wrapper.bind(navigator.transitionEndEventName, function () {
                    $(this).unbind(navigator.transitionEndEventName);

                    var leftIndex = methods._previousIndex(options);
                    var rightIndex = methods._nextIndex(options);

                    var nowLeft, previousRight;
                    if (leftIndex != null) {
                        nowLeft = $(this).find(".swipeSlide:nth-child(" + (leftIndex + 1) + ")");
                    }
                    if (rightIndex != null && rightIndex <= options.questionSceneObjects.length) {
                        previousRight = $(this).find(".swipeSlide:nth-child(" + (rightIndex + 2) + ")");
                        previousRight.empty();
                    }

                    // compute score for question just visited
                    if ((options.currentIndex >= 0) && (options.currentIndex < options.questionSceneObjects.length - 1)) {
                        var previousVisitedQuestionSceneObject = options.questionSceneObjects[options.currentIndex + 1];
                        previousVisitedQuestionSceneObject.computeQuestionScoreAndOptionLevelFeedback();
                    }
                    // now add new question content
                    if (leftIndex != null) {
                        var previousQuestionIndex;
                        if (options.questionSceneObjects[leftIndex] instanceof SceneObjectQuizQuestion) {
                            previousQuestionIndex = options.questionSceneObjects[leftIndex].getQuestionObject().questionNo;
                        } else {
                            previousQuestionIndex = options.questionSceneObjects[leftIndex].questionNo;
                        }
                        if (previousQuestionIndex < options.questionSceneObjects.length) { // avoid printing test summary on the left of first slide
                            options.questionSceneObjects[leftIndex].printQuestion(nowLeft);
                        }
                    }
                    // update current visible question
                    options.testObject.updateCurrentQuestion(options.questionSceneObjects[options.currentIndex]);
                    options.transitionInProgress = false;
                });
                var questionSlideEl = wrapper.find(".swipeSlide:nth-child(" + (options.currentIndex + 1) + ")");
                options.testObject.stopPlayingMultimediaObjects(questionSlideEl);
                options.questionSceneObjects[options.currentIndex].stopQuestionAnimation();
                options.currentIndex = methods._previousIndex(options);
                wrapper.css(navigator.transitionPropertyPrefix + "transition", "all " + options.transitionDuration + "ms ease-in-out")
                    .css(navigator.transitionPropertyPrefix + "transform", "translate3d(-" + (options.swipeWidth * options.currentIndex) + "px,0,0)");
            }
        },

        refresh: function () {
            var $this = this;
            var options = $this.options;
            options.swipeWidth = $this.width();
            options.swipeHeight = $this.height();
            var wrapper = options.wrapper;
            wrapper.height(options.swipeHeight)
                .width(options.swipeWidth * 3)
                .css("left", -options.swipeWidth + "px");
            var swipeSlide = wrapper.find(".swipeSlide");
            swipeSlide.width(options.swipeWidth).height(options.swipeHeight);
            return this;
        },

        destroy: function () {
            var $this = this;
            $this.options = null;
            $this.empty();
        },

        _nextIndex: function (options) {
            if (options.currentIndex < options.questionSceneObjects.length - 1) {
                return options.currentIndex + 1;
            } else {
                return null;
            }
        },

        _previousIndex: function (options) {
            if (options.currentIndex >= 1) {
                return options.currentIndex - 1;
            } else {
                return null;
            }
        }
    };

    $.fn.quizNavigator = function (method) {

        // set up default options
        var defaults = {
            questionSceneObjects: [],
            testObject: null,
            startIndex: 0,
            transitionDuration: 500
        };

        // Method calling logic
        if (methods[method]) {
            return methods[ method ].call(this, Array.prototype.slice.call(arguments, 1));
        } else if (typeof method === 'object' || !method) {
            var options = $.extend({}, defaults, arguments[0]);
            return methods.init.call(this, options);
        } else {
            $.error('Method ' + method + ' does not exist on jQuery.quizNavigator');
            return null;
        }
    };

})(jQuery);
