define(['GenericLightBox', 'QuizQuestionElement'], function (GenericLightBox, QuizQuestionElement) {
    return function (rawQuizTestModel, pageModel) {
        var that = this,
            quizTestConstants = {
                QUIZ_TEST_PADDING: 20,
                QUIZ_TEST_TITLE_FONT_SIZE: 24,
                QUIZ_FOOTER_HEIGHT: 80,
                DEFAULT_QUIZ_HEIGHT: 750
            };


        this.defineClasses = function () {

            that.QuizTestView = ElementBaseView.extend({
                tagName: "div",
                className: "sceneObject",
                elementsViews: [],
                id: rawQuizTestModel.properties.id,
                parentElementModel: pageModel,
                quizContentWrapperModel: null,
                currentIndex: 0,
                isIcon: JSON.parse(rawQuizTestModel["properties"]["isIconQuiz"]),

                events: function () {
                    var _events = {};
                    if (this.isIcon) {
                        _events[AppUtils.EventTypes.mouseDown] = "touchstart";
                        _events[AppUtils.EventTypes.click] = "clickHandler";
                        _events[AppUtils.EventTypes.mouseUp] = "mouseUpHandler";
                    }
                    return _events;
                },
                customInitialize: function () {
                    this.model.set("allObjectTypesInPage", rawQuizTestModel.customProperties["allObjectTypesInPage"]);
                    this.listenTo(this.model, "change:numberOfQuestions", this.updateNumberOfQuestions);
                    this.listenTo(this.model, "change:direction", this.updateCurrentIndex);
                    this.listenTo(this.model, "change:restartTest", this.restartTest);
                    this.listenTo(this.model, "change:pageDeactivated", this.pageDeactivated);
                },
                pageDeactivated: function () {
                    var pageDeactivated = this.model.get('pageDeactivated');
                    if (pageDeactivated) {
                        this.model.set('pageDeactivated', false);
                        this.model.set('restartTest', true);
                    }
                },
                restartTest: function () {
                    if (this.model.get('restartTest') && this.quizContentWrapperModel) {
                        this.currentIndex = 0;
                        this.updateQuestions();
                        this.quizContentWrapperModel.set('restartTest', true);
                        this.model.set('restartTest', false);
                    }
                },
                updateQuestions: function () {
                    var models = that.quizQuestionsCollection.models,
                        index;
                    for (index = 0; index < models.length; index++) {
                        models[index].set('resetQuestion', true);
                    }
                },
                updateNumberOfQuestions: function () {
                    if (null != this.quizContentWrapperModel) {
                        this.quizContentWrapperModel.set('numberOfQuestions', this.model.get('numberOfQuestions'))
                    }
                },
                updateCurrentIndex: function () {
                    if (null != this.quizContentWrapperModel) {
                        var direction = this.model.get('direction');
                        if (direction == 'left' && this.currentIndex > 0) {
                            this.quizContentWrapperModel.set('currentIndex', --this.currentIndex);
                        } else if (direction == 'right' && this.currentIndex < this.model.get('numberOfQuestions')) {
                            this.quizContentWrapperModel.set('currentIndex', ++this.currentIndex);
                        }
                    }
                    this.model.set('direction', null);
                },
                removeView: function () {
                    if (this.model.get("removed")) {
                        this.remove();
                        this.unbind();
                    }
                },
                activateObject: function () {
                    if (this.model.get("activateObject")) {
                        this.model.set("objectActivated", true);
                        this.model.set("activateObject", false);
                    }
                },
                objectActivated: function () {
                    var thisView = this;
                    if (this.model.get("objectActivated")) {
                        this.model.set("objectActivated", false);

                        if (this.isIcon) {
                            var lightBoxWindowSettings = {
                                width: "90%",
                                height: "90%",
                                horizontalMargin: '5%',
                                verticalMargin: '5%',
                                color: '#FFFFFF'
                            };

                            this.lightBox = new GenericLightBox({
                                closeAction: function () {
                                    that.quizTestModel.set('numberOfQuestions', -1);
                                    thisView.model.set('restartTest', true);
                                    that.quizQuestionsCollection.reset([]);
                                }
                            }, lightBoxWindowSettings, {color: '#FFFFFF'});
                            this.lightBox.init();
                            App.viewPortView.$el.append(this.lightBox.getView().render().el);
                            this.appendQuizContentWrapper(this.lightBox.getContentView().$el, true);
                            this.restartTest();
                        }
                    }
                },
                activateToggle: function () {
                    this.displayHideOnceOnClick();
                },
                appendQuizContentWrapper: function (targetEl, iconQuiz) {
                    var thisView = this,
                        quizContentWrapperView,
                    //we use 80% since the lightBox window size is 90% from App viewport and the light box content is 90% from lightbox window
                    //I substracted twice 10% from each axis and resulted 80% width, 80% height.
                        contentSize, absoluteHeight, scaleFactor;

                    if (iconQuiz) {
                        contentSize = this.getQuizTestAbsoluteSize('80%', '80%');
                    } else {
                        contentSize = this.getQuizTestAbsoluteSize(null, null);
                    }

                    absoluteHeight = contentSize.height + 2 * quizTestConstants.QUIZ_TEST_PADDING;
                    scaleFactor = absoluteHeight / quizTestConstants.DEFAULT_QUIZ_HEIGHT;

                    this.quizContentWrapperModel = new Backbone.Model({
                        size: contentSize,
                        scaleFactor: scaleFactor,
                        scaledFooterHeight: Math.min((quizTestConstants.QUIZ_FOOTER_HEIGHT * scaleFactor ), quizTestConstants.QUIZ_FOOTER_HEIGHT)
                    });
                    quizContentWrapperView = new that.QuizTestContentWrapperView({
                        parentModel: thisView.model,
                        model: thisView.quizContentWrapperModel
                    });
                    targetEl.append(quizContentWrapperView.render().el);
                },
                getQuizTestAbsoluteSize: function (width, height) {
                    var size = {}, absoluteWidth, absoluteHeight,
                        percentageWidth = (width == null) ? this.model.get('width') : width,
                        percentageHeight = (height == null) ? this.model.get('height') : height;
                    absoluteWidth = Math.floor(parseFloat(percentageWidth) * App.settings.width / 100) - 2 * quizTestConstants.QUIZ_TEST_PADDING;
                    absoluteHeight = Math.floor(parseFloat(percentageHeight) * App.settings.height / 100) - 2 * quizTestConstants.QUIZ_TEST_PADDING;
                    size.width = absoluteWidth;
                    size.height = absoluteHeight;
                    return size;
                },
                render: function () {
                    if (this.shouldRenderObject()) {
                        this.renderStartHandler();
                        this.setZIndex(this.model.get("ZIndex"));
                        this.setLeft(this.model.get("left"));
                        this.setTop(this.model.get("top"));
                        this.setInitialVisibility();
                        if (this.isIcon) {
                            this.setupIconQuiz();
                        } else {
                            this.setupInlineQuiz();
                        }
                        return this;
                    }
                    return null;
                },
                setupIconQuiz: function () {
                    var quizIconInfo = Chapter.getSettings()["quizIconInfo"];
                    this.model.set("imageURL", App.quizIconURL);
                    this.template = templates.getTemplate("image");
                    this.$el.html(this.template(this.model.attributes));
                    this.setWidth(quizIconInfo['quizIconWidth']);
                    this.setHeight(quizIconInfo['quizIconHeight']);
                },
                setupInlineQuiz: function () {
                    this.setWidth(this.model.get("width"));
                    this.setHeight(this.model.get("height"));
                    this.setBackgroundColor("#F4F4F4", 1);
                    this.$el.css('pointer-events', 'none');
                    this.appendQuizContentWrapper(this.$el);
                }
            });
            that.QuizTestTitle = Backbone.View.extend({
                id: rawQuizTestModel.properties.id + '_quizTestTitle',
                template: templates.getTemplate("quizTestTitle"),
                defaultFontSize: 24,
                render: function () {
                    var scaleFactor = this.model.get('scaleFactor');
                    CSSHandler.setFontSize(this.$el, this.defaultFontSize * scaleFactor);
                    this.$el.append($(this.template(this.model.attributes)));
                    return this;
                }
            });
            that.QuizTestProgressBarWrapper = BaseView.extend({
                id: rawQuizTestModel.properties.id + '_quizTestProgressBar',
                template: templates.getTemplate("quizTestProgressBar"),
                className: 'testProgressWrapper testProgressBar',
                defaultProgressBarWrapperHeight: 24,
                progressBarModel: null,
                textProgressBarModel: null,
                initialize: function () {
                    this.listenTo(this.model, "change:progress", this.updateProgressBar);
                    this.listenTo(this.model, "change:progressTextBarContent", this.updateTextProgressBar);
                },
                appendQuizProgressBarText: function () {
                    var thisView = this;
                    this.textProgressBarModel = new Backbone.Model({
                        width: '100%',
                        progressTextBarContent: thisView.model.get('progressTextBarContent'),
                        scaleFactor: thisView.model.get('scaleFactor')
                    });
                    var quizTestProgressBarText = new that.QuizTestProgressBarText({
                        model: thisView.textProgressBarModel
                    });
                    this.$el.append(quizTestProgressBarText.render().el);
                },
                updateTextProgressBar: function () {
                    this.textProgressBarModel.set('progressTextBarContent', this.model.get('progressTextBarContent'));
                },
                updateProgressBar: function () {
                    this.progressBarModel.set('progress', this.model.get('progress'));
                },
                appendQuizProgressBar: function () {
                    var thisView = this;
                    this.progressBarModel = new Backbone.Model({
                        width: '0px',
                        parentWidth: thisView.model.get('width'),
                        scaleFactor: thisView.model.get('scaleFactor')
                    });
                    var quizTestProgressBar = new that.QuizTestGreenProgressBar({
                        model: thisView.progressBarModel
                    });
                    this.$el.append(quizTestProgressBar.render().el);

                },
                render: function () {
                    this.setWidth(this.model.get('width'));
                    this.setHeight(Math.ceil(this.model.get('scaleFactor') * this.defaultProgressBarWrapperHeight));
                    this.appendQuizProgressBar();
                    this.appendQuizProgressBarText();
                    return this;
                }
            });
            that.QuizTestGreenProgressBar = BaseView.extend({
                id: rawQuizTestModel.properties.id + '_quizTestProgressBar',
                className: 'testProgressGreenBar',
                defaultHeight: 20,
                defaultMargin: 2,
                defaultMarginBottom: 2,
                currentMargins: 0,
                initialize: function () {
                    this.listenTo(this.model, "change:progress", this.updateProgressBar);
                },
                updateProgressBar: function () {
                    var progress = this.model.get('progress'),
                        width;
                    if (progress <= 1) {
                        width = this.model.get('parentWidth') - 2 * this.currentMargins;
                        this.setWidth(this.model.get('progress') * width);
                    }

                },
                render: function () {
                    var scaleFactor = this.model.get('scaleFactor');
                    this.setWidth(this.model.get('width'));
                    this.setHeight(this.defaultHeight * scaleFactor);
                    this.currentMargins = this.defaultMargin * scaleFactor;
                    this.$el.css('margin', this.currentMargins);
                    this.$el.css('margin-bottom', this.defaultMarginBottom * scaleFactor);
                    return this;
                }
            });
            that.QuizTestProgressBarText = BaseView.extend({
                id: rawQuizTestModel.properties.id + '_quizTestProgressBarText',
                template: templates.getTemplate("quizTestProgressText"),
                defaultFontSize: 26,
                defaultMargins: 10,
                initialize: function () {
                    this.listenTo(this.model, "change:progressTextBarContent", this.updateTextProgressBar);
                },
                updateTextProgressBar: function () {
                    var progressTextBarContentInfo = this.model.get('progressTextBarContent');
                    if (progressTextBarContentInfo.currentIndex <= progressTextBarContentInfo.totalNumber)
                        this.$el.text(progressTextBarContentInfo.currentIndex + ' / ' + progressTextBarContentInfo.totalNumber);
                },
                render: function () {
                    var scaleFactor = this.model.get('scaleFactor');
                    this.setWidth(this.model.get('width'));
                    this.$el.css('margin', this.defaultMargins * scaleFactor);
                    CSSHandler.setFontSize(this.$el, this.defaultFontSize * scaleFactor);
                    this.$el.append($(this.template(this.model.attributes)));
                    return this;
                }
            });
            that.QuizTestButtonNavigation = BaseView.extend({
                defaultHeight: 50,
                defaultWidth: 80,
                className: function () {
                    return this.model.get('goTo') == 'left' ? 'questionPrevious navigationDisabled' : 'questionNext questionButtonGradient';
                },
                template: templates.getTemplate("quizTestNavigationButton"),
                initialize: function () {
                    this.listenTo(this.model, "change:disabled", this.toggleButtonAvailability);
                },
                toggleButtonAvailability: function () {
                    if (this.model.get('disabled')) {
                        this.$el.addClass('navigationDisabled');
                    } else {
                        this.$el.removeClass('navigationDisabled');
                    }
                },
                events: function () {
                    var _events = {};
                    _events[AppUtils.EventTypes.click] = "clickHandler";
                    return _events;
                },
                clickHandler: function () {
                    if (!this.model.get('disabled')) {
                        that.quizTestModel.set('direction', this.model.get('goTo'));
                    }
                },

                render: function () {
                    var scaleFactor = this.model.get('scaleFactor');
                    this.$el.append($(this.template(this.model.attributes)));
                    this.setWidth(this.defaultWidth * scaleFactor);
                    this.setHeight(this.defaultHeight * scaleFactor);
                    return this;
                }
            });
            that.QuizTestFooterView = BaseView.extend({
                id: rawQuizTestModel.properties.id + '_QuizTestFooterView',
                leftNavigationButtonModel: null,
                rightNavigationButtonModel: null,
                quizTestProgressBarWrapperModel: null,
                initialize: function () {
                    this.listenTo(this.model, "change:currentIndex", this.updateCurrentIndex);
                    this.listenTo(this.model, "change:numberOfQuestions", this.updateProgressBar);
                },
                updateProgressBar: function () {
                    var numberOfQuestions = this.model.get('numberOfQuestions') - 1,
                        percentageCompleted = Math.round(10000 / numberOfQuestions) / 10000,
                        textProgressBarInfo = {};
                    textProgressBarInfo.currentIndex = 1;
                    textProgressBarInfo.totalNumber = numberOfQuestions;
                    this.quizTestProgressBarWrapperModel.set('progress', percentageCompleted);
                    this.quizTestProgressBarWrapperModel.set('progressTextBarContent', textProgressBarInfo);
                },
                updateCurrentIndex: function () {
                    var currentIndex = this.model.get('currentIndex'),
                        numberOfQuestions = this.model.get('numberOfQuestions'),
                        percentageCompleted = Math.round(10000 * (currentIndex + 1) / (numberOfQuestions - 1)) / 10000,
                        textProgressBarInfo = {};
                    if (currentIndex == 0) {
                        this.leftNavigationButtonModel.set('disabled', true);
                    } else {
                        this.leftNavigationButtonModel.set('disabled', false);
                    }
                    if (currentIndex == numberOfQuestions - 1) {
                        this.rightNavigationButtonModel.set('disabled', true);
                        this.leftNavigationButtonModel.set('disabled', true);
                    } else {
                        this.rightNavigationButtonModel.set('disabled', false);
                    }
                    textProgressBarInfo.currentIndex = (currentIndex + 1);
                    textProgressBarInfo.totalNumber = (numberOfQuestions - 1);
                    this.quizTestProgressBarWrapperModel.set('progress', percentageCompleted);
                    this.quizTestProgressBarWrapperModel.set('progressTextBarContent', textProgressBarInfo);
                },
                appendNavigationButton: function (direction) {
                    var thisView = this,
                        imageUrl = direction == 'left' ? 'gfx/quiz/quiz_previous_arrow.png' : 'gfx/quiz/quiz_next_arrow.png',
                        model = new Backbone.Model({
                            goTo: direction,
                            imageURL: imageUrl,
                            scaleFactor: thisView.model.get('scaleFactor')
                        }),
                        navigationButtonView = new that.QuizTestButtonNavigation({
                            model: model,
                            parentModel: thisView.model
                        });
                    this.$el.append(navigationButtonView.render().el);
                    return model;
                },

                appendProgressBarWrapper: function () {
                    var thisView = this;
                    this.quizTestProgressBarWrapperModel = new Backbone.Model({
                        parentWidth: thisView.model.get('width'),
                        width: thisView.model.get('width') / 2,
                        height: thisView.model.get('height') / 2,
                        scaleFactor: thisView.model.get('scaleFactor')

                    });
                    var quizTestProgressBarWrapperView = new that.QuizTestProgressBarWrapper({
                        model: thisView.quizTestProgressBarWrapperModel
                    });
                    this.$el.append(quizTestProgressBarWrapperView.render().el);

                },
                render: function () {
                    this.setWidth(this.model.get('width'));
                    this.setHeight(this.model.get('height'));
                    this.appendProgressBarWrapper();
//                    this.setBottom(0);
                    this.setPositionType('relative');

                    this.rightNavigationButtonModel = this.appendNavigationButton('right');
                    this.leftNavigationButtonModel = this.appendNavigationButton('left');
                    return this;
                }
            });
            that.QuizTestQuestionsWrapperView = BaseView.extend({
                id: rawQuizTestModel.properties.id + '_quizTestQuestionsWrapperViewport',
                numberOfQuestions: 0,
                currentDisplayedQuestionIndex: 0,
                $questionsWrapper: null,
                defaultWidth: 1033,
                defaultHeight: 572,
                testResultModel: null,
                testRestarted: false,
                initialize: function () {
                    this.listenTo(this.model, "change:currentIndex", this.updateCurrentIndex);
                    this.listenTo(this.model, "change:restartTest", this.restartTest);
                },
                restartTest: function () {
                    if (this.model.get('restartTest')) {
                        this.testRestarted = true;
                        this.testResultModel.set('restartTest', true);
                        this.model.set('currentIndex', 0);
                        this.model.set('restartTest', false);
                    }
                },
                updateCurrentIndex: function () {
                    if (this.currentDisplayedQuestionIndex < that.quizQuestionsCollection.models.length) {
                        that.quizQuestionsCollection.models[this.currentDisplayedQuestionIndex].set('computeScore', true);
                    }
                    this.currentDisplayedQuestionIndex = this.model.get('currentIndex');
                    var xOffset = -this.currentDisplayedQuestionIndex * this.model.get('width');
                    if (this.testRestarted) {
                        CSSHandler.translate3D(this.$questionsWrapper, xOffset, 0, 0, 0);
                        this.testRestarted = false;
                    } else {
                        CSSHandler.translate3D(this.$questionsWrapper, xOffset, 0, 0, 700);
                    }
                },

                updateQuestionCollection: function () {
                    var index, thisView = this, questions = rawQuizTestModel.customProperties.questions,
                        questionProperties, question, width = this.model.get('width'),
                        testResultModel, testResultView, questionWrapperModel, questionWrapperView;
                    this.numberOfQuestions = questions.length;
                    that.quizTestModel.set('numberOfQuestions', this.numberOfQuestions + 1);

                    for (index = 0; index < questions.length; index++) {
                        questionProperties = questions[index];
                        questionProperties.properties = {}
                        questionProperties.properties.id = rawQuizTestModel.properties.id + '_question' + index;
                        questionProperties.properties.left = 0;
                        questionProperties.properties.top = 0;
                        questionProperties.properties.width = '100%';
                        questionProperties.properties.height = '100%';
                        questionProperties.properties.scaleFactor = this.model.get('scaleFactor');
                        questionProperties.properties.displayed = 'true';
                        questionProperties.properties.index = index;
                        questionWrapperModel = new Backbone.Model({
                            width: thisView.model.get('width'),
                            height: thisView.model.get('height'),
                            left: index * width
                        });

                        questionWrapperView = new that.QuizQuestionWrapperView({
                            model: questionWrapperModel
                        });
                        question = new QuizQuestionElement(questionProperties, questionWrapperModel);
                        question.init();
                        that.quizQuestionsCollection.add(question.getModel());
                        this.$questionsWrapper.append(questionWrapperView.render().el);
                        questionWrapperView.$questionScalableWrapper.append(question.getView().render().el);
                    }
                    this.testResultModel = new Backbone.Model({
                        width: width,
                        height: thisView.model.get('height'),
                        left: thisView.numberOfQuestions * width
                    });
                    testResultView = new that.QuizTestResultView({
                        model: thisView.testResultModel
                    });
                    this.$questionsWrapper.append(testResultView.render().el);
                },
                render: function () {
                    var thisView = this;
                    this.setTransitionEndHandler(function () {
                            if (thisView.numberOfQuestions == thisView.currentDisplayedQuestionIndex) {
                                thisView.testResultModel.set('updateScore', true);
                            }
                        }
                    )
                    ;
                    this.setWidth(this.model.get('width'));
                    this.setHeight(this.model.get('height'));
                    this.setLeft(this.model.get('left'));
                    this.setTop(this.model.get('top'));
                    this.setPositionType('relative');
                    this.$el.css('overflow', 'hidden');
                    this.$questionsWrapper = $('<DIV>')
                        .css('width', '100%')
                        .css('height', '100%');
                    this.$el.append(this.$questionsWrapper);
                    this.updateQuestionCollection();
                    return this;
                }
            });
            that.QuizQuestionWrapperView = BaseView.extend({
                defaultWidth: 1033,
                defaultHeight: 572,
                $questionScalableWrapper: null,
                initialize: function () {
                    this.listenTo(this.model, "change:contentHeight", this.updateQuestionHeight);
                    this.listenTo(this.model, "change:computeScore", this.computeScoreForQuestion);
                    this.$questionScalableWrapper = $('<div>');
                },

                updateQuestionHeight: function () {
                    var height = this.model.get('contentHeight');
                    var originalWidth = this.model.get('width'),
                        originalHeight = this.model.get('height'),
                        scaleFactor = originalHeight / height,
                        newWidth,
                        top = (scaleFactor > 1) ? (originalHeight - height) / 2 : 0;
                    newWidth = scaleFactor < 1 ? Math.floor(originalWidth / scaleFactor) : originalWidth;
                    this.$questionScalableWrapper.css('position', 'absolute');
                    this.$questionScalableWrapper.css('width', newWidth + 'px');
                    this.$questionScalableWrapper.css('height', height + 'px');
                    this.$questionScalableWrapper.css('top', top + 'px');
                    this.$questionScalableWrapper.css('left', '0px');
                    if (scaleFactor < 1) {
                        //CSSHandler.scale(this.$questionScalableWrapper, scaleFactor);
                        //this.$questionScalableWrapper.css('-webkit-transform-origin', '0 0');
                        this.$questionScalableWrapper.css("zoom", scaleFactor);
                    }
                },
                render: function () {
                    this.$questionScalableWrapper.css('width', 0);
                    this.$questionScalableWrapper.css('height', 0);
                    this.setWidth(this.model.get('width'));
                    this.setHeight(this.model.get('height'));
                    this.setLeft(this.model.get('left'));
                    this.setPositionType('absolute');
                    this.$el.css('overflow', 'hidden');
                    this.$el.append(this.$questionScalableWrapper);
                    return this;
                }
            });
            that.QuizTestContentWrapperView = BaseView.extend({
                    id: rawQuizTestModel.properties.id + 'quizTestContentWrapper',
                    footerModel: null,
                    contentQuizQuestionWrapperModel: null,
                    testRestarted: false,
                    initialize: function () {
                        this.listenTo(this.model, "change:numberOfQuestions", this.updateNumberOfQuestions);
                        this.listenTo(this.model, "change:currentIndex", this.updateCurrentIndex);
                        this.listenTo(this.model, "change:restartTest", this.restartTest);
                    },
                    restartTest: function () {
                        if (this.model.get('restartTest')) {
                            this.testRestarted = true;
                            this.footerModel.set('currentIndex', 0);
                            this.contentQuizQuestionWrapperModel.set('restartTest', true);
                            this.model.set('currentIndex', 0);
                            this.model.set('restartTest', false);
                        }

                    },
                    updateNumberOfQuestions: function () {
                        if (null != this.footerModel) {
                            this.footerModel.set('numberOfQuestions', this.model.get('numberOfQuestions'))
                        }
                        if (null != this.contentQuizQuestionWrapperModel) {
                            this.contentQuizQuestionWrapperModel.set('numberOfQuestions', this.model.get('numberOfQuestions'))
                        }
                    },
                    updateCurrentIndex: function () {
                        if (this.testRestarted) {
                            this.testRestarted = false;
                        } else {
                            if (null != this.footerModel) {
                                this.footerModel.set('currentIndex', this.model.get('currentIndex'));
                            }
                            if (null != this.contentQuizQuestionWrapperModel) {
                                this.contentQuizQuestionWrapperModel.set('currentIndex', this.model.get('currentIndex'));
                            }
                        }

                    },

                    appendQuizTitle: function () {
                        var thisView = this;
                        this.model.set('title', rawQuizTestModel.customProperties['title']);
                        var titleView = new that.QuizTestTitle({
                            model: thisView.model
                        });
                        this.$el.append(titleView.render().el);
                        return titleView;
                    },

                    getFooterView: function () {
                        var thisView = this;
                        this.footerModel = new Backbone.Model({
                            height: thisView.model.get('scaledFooterHeight'),
                            width: thisView.model.get('size').width,
                            scaleFactor: thisView.model.get('scaleFactor')
                        });

                        return new that.QuizTestFooterView({
                            model: thisView.footerModel
                        });
                    },

                    appendQuizQuestionsWrapper: function (titleHeight, footerHeight) {
                        var thisView = this, quizQuestionWrapperView;
                        this.contentQuizQuestionWrapperModel = new Backbone.Model({
                            height: thisView.model.get('size').height - 2 * quizTestConstants.QUIZ_TEST_PADDING - titleHeight - footerHeight,
                            width: thisView.model.get('size').width,
                            scaleFactor: thisView.model.get('scaleFactor'),
                            top: 0,
                            left: 0
                        });
                        quizQuestionWrapperView = new that.QuizTestQuestionsWrapperView({
                            model: thisView.contentQuizQuestionWrapperModel
                        });
                        this.$el.append(quizQuestionWrapperView.render().el);
                    },
                    render: function () {

                        var size = this.model.get('size'),
                            thisView = this,
                            titleView = this.appendQuizTitle();
                        this.setWidth(size.width);
                        this.setHeight(size.height);
                        this.setLeft(quizTestConstants.QUIZ_TEST_PADDING);
                        this.setTop(quizTestConstants.QUIZ_TEST_PADDING);
                        this.setPositionType('absolute');

                        setTimeout(function () {
                            thisView.appendQuizQuestionsWrapper(titleView.$el.outerHeight(), thisView.model.get('scaledFooterHeight'));
                            thisView.$el.append(thisView.getFooterView().render().el);
                            thisView.footerModel.set('numberOfQuestions', thisView.model.get('numberOfQuestions'))
                        }, 0);

                        return this;
                    }
                }
            );
            that.QuizTestResultView = BaseView.extend({
                id: rawQuizTestModel.properties.id + '_quizTestResultView',
                template: templates.getTemplate("quizTestResult"),
                defaultWidth: 1033,
                defaultHeight: 572,
                $resultWrapper: null,
                $percentageScore: null,
                $testSummaryNoPoints: null,
                $restartTestButton: null,
                initialize: function () {
                    this.listenTo(this.model, "change:updateScore", this.updateScore);
                    this.listenTo(this.model, "change:restartTest", this.resetScore);
                },

                updateScore: function () {
                    if (this.model.get('updateScore')) {
                        this.model.set('updateScore', false);
                        var score = this.getComputedScore(),
                            testScorePercentage = score.testScore * 100 / score.testMaxScore;
                        this.setDeceleratingTimeout(this.updateTestScorePercentage, this.updateTestPointsResult, 2, testScorePercentage / 2, testScorePercentage);
                        this.$testSummaryNoPoints.text(CAPTIONS.QUIZ_SCORE_TEXT_POINTS_START + score.testScore + CAPTIONS.QUIZ_SCORE_TEXT_POINTS_MIDDLE + score.testMaxScore);
                    }
                },
                resetScore: function () {
                    if (this.model.get('restartTest')) {
                        CSSHandler.setOpacity(this.$testSummaryNoPoints, 0);
                        CSSHandler.setOpacity(this.$restartTestButton, 0);
                        this.updateTestScorePercentage(this, 0);
                        this.$testSummaryNoPoints.text(CAPTIONS.QUIZ_SCORE_TEXT_POINTS_START + 0 + CAPTIONS.QUIZ_SCORE_TEXT_POINTS_MIDDLE + this.getComputedScore().testMaxScore);
                        this.model.set('restartTest', false);
                    }
                },
                getComputedScore: function () {
                    var ret = {},
                        models = that.quizQuestionsCollection.models,
                        index,
                        questionScore,
                        questionMaxScore;
                    ret.testScore = 0;
                    ret.testMaxScore = 0;

                    for (index = 0; index < models.length; index++) {
                        questionScore = models[index].get('questionScore');
                        questionMaxScore = models[index].get('questionMaxScore');
                        if (null != questionScore) {
                            ret.testScore += questionScore;
                        }
                        if (null != questionMaxScore) {
                            ret.testMaxScore += questionMaxScore;
                        }
                    }
                    if (ret.testMaxScore == 0) {
                        ret.testMaxScore = 1;
                    }
                    return ret;
                },
                setDeceleratingTimeout: function (callback, endCallback, factor, times, testScorePercentage) {
                    var that = this;
                    var startScore = 0;
                    var internalCallback = function (t, counter) {
                        return function () {
                            if (--t >= 0) {
                                counter += 2;
                                setTimeout(internalCallback, counter * factor);
                                startScore = Math.min(startScore + 2, testScorePercentage);
                                callback(that, startScore);
                            } else {
                                endCallback(that);
                            }
                        }
                    }(times, 0);
                    setTimeout(internalCallback, factor);
                },
                updateTestScorePercentage: function (that, currentScore) {
                    that.$percentageScore.text(currentScore + "%");
                },
                updateTestPointsResult: function (that) {
                    that.$testSummaryNoPoints.animate({opacity: 1}, 1000);
                    that.$restartTestButton.animate({opacity: 1}, 1000);
                },
                getScaleFactor: function (width, height) {
                    var hScaleFactor = width / this.defaultWidth,
                        vScaleFactor = height / this.defaultHeight;
                    return Math.min(hScaleFactor, vScaleFactor);

                },
                render: function () {
                    var originalWidth = this.model.get('width'),
                        originalHeight = this.model.get('height'),
                        scaleFactor = this.getScaleFactor(originalWidth, originalHeight),
                        newWidth = Math.floor(this.defaultWidth * scaleFactor),
                        newHeight = Math.floor(this.defaultHeight * scaleFactor),
                        left = (originalWidth - newWidth) / 2,
                        top = (originalHeight - newHeight) / 2,
                        thisView = this;
                    if (scaleFactor > 1) {
                        left = (originalWidth - this.defaultWidth) / 2;
                        top = (originalHeight - this.defaultHeight) / 2;
                    }

                    this.setWidth(originalWidth);
                    this.setHeight(originalHeight);
                    this.setLeft(this.model.get('left'));
                    this.setPositionType('absolute');
                    this.$el.append($(this.template(this.model.attributes)));
                    setTimeout(function () {
                        thisView.$resultWrapper = $("#quizTestResultWrapper", thisView.el);
                        thisView.$resultWrapper.css('position', 'absolute');
                        thisView.$resultWrapper.css('width', thisView.defaultWidth + 'px');
                        thisView.$resultWrapper.css('height', thisView.defaultHeight + 'px');
                        thisView.$resultWrapper.css('top', top + 'px');
                        thisView.$resultWrapper.css('left', left + 'px');
                        if (scaleFactor < 1) {
                            //CSSHandler.scale(thisView.$resultWrapper, scaleFactor);
                            //thisView.$resultWrapper.css('-webkit-transform-origin', '0 0');
                            thisView.$resultWrapper.css("zoom", scaleFactor);
                        }
                        thisView.$percentageScore = $(".testBigPercentage", thisView.el);
                        thisView.$testSummaryNoPoints = $(".testSummaryNoPoints", thisView.el);
                        CSSHandler.setOpacity(thisView.$testSummaryNoPoints, 0);
                        thisView.$restartTestButton = $(".testRetryButton", thisView.el);
                        CSSHandler.setOpacity(thisView.$restartTestButton, 0);
                        thisView.$restartTestButton.bind(AppUtils.EventTypes.click, function () {
                            CSSHandler.setOpacity(thisView.$testSummaryNoPoints, 0);
                            CSSHandler.setOpacity(thisView.$restartTestButton, 0);
                            that.quizTestModel.set('restartTest', true);
                        });
                    }, 0);
                    return this;
                }
            });
        };

        this.instantiateObjects = function () {
            this.quizQuestionsCollection = new Backbone.Collection();
            this.quizTestModel = this.getQuizTestModel();
            this.quizTestView = new this.QuizTestView({
                model: that.quizTestModel
            });

        };

        this.getQuizTestModel = function () {
            return new Backbone.Model(rawQuizTestModel.properties);

        };


        return {
            init: function () {
                that.defineClasses();
                that.instantiateObjects();
            },
            getView: function () {
                return that.quizTestView;
            },
            getModel: function () {
                return that.quizTestModel;
            }

        }
    }
})
;