(function (window) {

    /* Constructor, add object to window namespace */
    window.SceneObjectQuizQuestion = function (questionJSON, containerElView) {
        var that = this,
            multimediaObject;
        this.questionObject = JSON.parse(JSON.stringify(questionJSON)); // make sure we don't touch original question model
        this.containerEl = containerElView.$el;
        this.containerElModel = containerElView.model;
        multimediaObject = that.questionObject.multimediaObject;
        if (null != multimediaObject) {
            if (multimediaObject["type"] == 'animation') {
                require(['AnimationElement'], function (AnimationElement) {
                    that.printQuestion(AnimationElement);
                });
            } else {
                this.printQuestion(null);
            }
        } else {
            this.printQuestion(null);
        }
    };

    SceneObjectQuizQuestion.prototype = {

        submitButton: null,
        questionTouched: false,
        firstTimeScore: null,
        questionObject: null,
        animationObjectInstance: null,
        containerElModel: null,
        QUIZ_QUESTION_ANSWER_FONT_SIZE: 22,
        QUIZ_QUESTION_ANSWER_PADDING_LEFT: 20,
        QUIZ_QUESTION_ANSWER_MARGIN_LEFT: 40,
        QUIZ_QUESTION_ANSWER_MARGIN_RIGHT: 50,
        ANSWER_PADDING_TOP_BOTTOM: 15,
        ANSWER_LABEL_LINE_HEIGHT: 25,
        LABEL_RADIO_PADDING_LEFT: 34,
        LABEL_RADIO_BACKGROUND_SIZE: 25,
        QUESTION_SUBMIT_BUTTON_WIDTH: 222,
        QUESTION_SUBMIT_BUTTON_HEIGHT: 55,
        QUESTION_SUBMIT_BUTTON_MARGIN_TOP: 50,
        QUESTION_SUBMIT_BUTTON_MARGIN_LEFT: 40,
        QUESTION_SUBMIT_BUTTON_LINE_HEIGHT: 50,
        QUESTION_SUBMIT_BUTTON_FONT_SIZE: 28,

        resetQuestion: function (resetWithAnimation) {
            // 1) Reset all checked answers
            this.fieldsetEl.find("input").prop("checked", false);
            this.fieldsetEl.find(".r_on").removeClass("r_on");
            this.fieldsetEl.find(".c_on").removeClass("c_on");
            // 2) Revert to answers view if in feedback mode
            this._returnToAnswersView(resetWithAnimation);
            // mark question as not touched
            this.questionTouched = false;
            if (!resetWithAnimation) { // now the question is reset due to page change, reset one time score logic, see NKI-25
                this.firstTimeScore = null;
            }
            if (this.submitButton) {
                this.submitButton.css("opacity", "");
            }
        },

        printQuestion: function (MultimediaClass) {
            var that = this;
            var contentHeight = 0;
            var questionText = that.questionObject.questionText;
            var questionAnswers = that.questionObject.questionAnswers;
            var questionNo = that.questionObject.questionNo;
            var useNumbering = that.questionObject.useNumbering;
            var inputType = that.questionObject.inputType;
            var multimediaObject = that.questionObject.multimediaObject;
            var randomOrderOptions = that.questionObject.randomOrderOptions;
            var multimediaPlacement = that.questionObject.multimediaPlacement;

            var hasFeedback = (that.questionObject.questionFeedback.length > 0);
            var hasMultimedia = (multimediaObject != null);

            that.leftRightAlignment = (multimediaPlacement == MultimediaPlacement.PLACEMENT_RIGHT || multimediaPlacement == MultimediaPlacement.PLACEMENT_LEFT);
            that.multimediaPaddingValue = Math.round(20 * 1);

            questionText = that._getFormattedQuestionText(questionText, useNumbering, questionNo);

            // START printing answers
            var groupId = Math.random().toString().replace("0.", "");
            that.fieldsetEl = $("<fieldset>").addClass("optionFieldset");
            var labelClass = "label_check";
            if (inputType == "radio") {
                labelClass = "label_radio";
            }
            if (randomOrderOptions) {
                questionAnswers = that._shuffleArray(questionAnswers);
            }

            $.each(questionAnswers, function (idx, answerObject) {
                if (answerObject["answerFeedback"]) {
                    hasFeedback = true;
                }
                var answerStyles = "quizQuestionAnswer";
                var answerNo = idx + 1;
                if (answerNo % 2 == 1) {
                    answerStyles += " oddQuizQuestionAnswer";
                }
                var answerInputId = groupId + answerNo;
                var answerSecondWrapperEl = $("<div>").addClass(answerStyles)
                    .css("font-size", Math.round(that.QUIZ_QUESTION_ANSWER_FONT_SIZE * 1) + "px")
                    .css("padding-left", Math.round(that.QUIZ_QUESTION_ANSWER_PADDING_LEFT * 1) + "px")
                    .css("margin-left", Math.round(that.QUIZ_QUESTION_ANSWER_MARGIN_LEFT * 1) + "px")
                    .css("margin-right", Math.round(that.QUIZ_QUESTION_ANSWER_MARGIN_RIGHT * 1) + "px");
                var answerLabelEl = $("<label>").addClass(labelClass)
                    .attr("for", answerInputId)
                    .css("line-height", Math.round(that.ANSWER_LABEL_LINE_HEIGHT * 1) + "px")
                    .css("padding-left", Math.round(that.LABEL_RADIO_PADDING_LEFT * 1) + "px")
                    .css("background-size", Math.round(that.LABEL_RADIO_BACKGROUND_SIZE * 1) + "px");
                var userCheckedValue = (answerObject["userChecked"] != null) ? answerObject["userChecked"] : false;
                var answerInputEl = $("<input>").attr("id", answerInputId).attr("type", inputType).attr("name", groupId).prop("checked", userCheckedValue);
                var answerPaddingEl = $("<div>").addClass("answerPadding")
                    .css("padding-top", Math.round(that.ANSWER_PADDING_TOP_BOTTOM * 1) + "px")
                    .css("padding-bottom", Math.round(that.ANSWER_PADDING_TOP_BOTTOM * 1) + "px")
                    .append(answerObject["answerInput"]);
                answerLabelEl.append(answerInputEl).append(answerPaddingEl);
                answerObject["inputId"] = answerInputId;
                if (!AppUtils.DeviceType.iOS) { // do this because of problems when content displayed on lightbox on web browser
                    answerLabelEl.bind(AppUtils.EventTypes.click, function (e) {
                        e.preventDefault();
                        e.stopPropagation();
                        console.log("Mouse down on input : " + $(this).find("input").prop("checked"));
                    });
                }
                answerLabelEl.bind(AppUtils.EventTypes.mouseDown, function (e) { // only "click" event does the value toggle for input type checkbox
                    e.preventDefault();
                    e.stopPropagation();
                    if (!that.questionTouched) {
                        that.questionTouched = true;
                        if (that.submitButton) {
                            that.submitButton.css("opacity", 1);
                        }
                    }
                    var inputEl = $(this).find("input"),
                        currentValue = inputEl.prop("checked");
                    if (!currentValue || inputType == "checkbox") {
                        console.log("Toggle answer value to ON");
                        inputEl.prop("checked", !currentValue);
                        that.setupAnswers(inputType);
                    }
                });
                answerSecondWrapperEl.append(answerLabelEl);
                that.fieldsetEl.append(answerSecondWrapperEl);
            });
            that.setupAnswers(inputType);
            // END printing answers

            var quizQuestionPaddingEl = $("<div>").addClass("quizPadding");
            that.containerEl.append(quizQuestionPaddingEl);

            // will wrap question text, options, submit button and feedback
            var questionWrapper = that._getQuestionWrapperEl(hasMultimedia);

            // feedback section
            var questionFeedbackEl = $("<div>").addClass("questionFeedback");

            // setup the flip animation for feedback
            var flipAnimationHolderEl = $("<div>").addClass("flipAnimationHolder");
            var frontFlipSideEl = $("<div>").addClass("flipSide frontFlip");
            var backFlipSideEl = $("<div>").addClass("flipSide backFlip");

            that.flipAnimationHolderEl = flipAnimationHolderEl;

            frontFlipSideEl.append(that.fieldsetEl);
            // submit button if there is one
            if (hasFeedback) {
                that.submitButton = that._getSubmitButtonEl();
                frontFlipSideEl.append(that.submitButton);
            }
            backFlipSideEl.append(questionFeedbackEl);
            flipAnimationHolderEl.append(frontFlipSideEl).append(backFlipSideEl);

            if (hasMultimedia) { // print question and options on the wrapper
                that.fieldsetEl.find(".quizQuestionAnswer").css("margin-left", "0");
                questionWrapper.append(questionText).append(flipAnimationHolderEl);
                quizQuestionPaddingEl.append(questionWrapper);
            } else { // if no multimedia don't use any wrapper
                quizQuestionPaddingEl.append(questionText).append(flipAnimationHolderEl);
            }

            flipAnimationHolderEl.height(frontFlipSideEl.height() + 10);

            if (hasMultimedia) { // we have a multimedia attached

                var multimediaPaddingEl = that._getMultimediaPaddingEl();
                var multimediaWrapper = that._getMultimediaWrapperEl(multimediaPlacement);
                multimediaPaddingEl.append(multimediaWrapper);
                if (multimediaPlacement == MultimediaPlacement.PLACEMENT_RIGHT || multimediaPlacement == MultimediaPlacement.PLACEMENT_BELLOW) {
                    quizQuestionPaddingEl.append(multimediaPaddingEl);
                } else if (multimediaPlacement == MultimediaPlacement.PLACEMENT_LEFT || multimediaPlacement == MultimediaPlacement.PLACEMENT_ABOVE) {
                    quizQuestionPaddingEl.prepend(multimediaPaddingEl);
                } else {
                    quizQuestionPaddingEl.append(multimediaPaddingEl);
                }

                var multimediaWrapperWidth, multimediaWrapperHeight;
                if (!that.leftRightAlignment) {
                    multimediaWrapperHeight = quizQuestionPaddingEl.height() - questionWrapper.height() - that.multimediaPaddingValue;
                    multimediaWrapperWidth = quizQuestionPaddingEl.width();
                } else {
                    multimediaWrapperHeight = that.containerEl.height();
                    multimediaWrapperWidth = multimediaWrapper.width();
                }

                multimediaWrapper.append(that._getHTMLForMultimediaObject(MultimediaClass, multimediaObject, that.leftRightAlignment, multimediaWrapperWidth, multimediaWrapperHeight));
            }
            //75 are the margins for submit button and question title
            that.containerElModel.set('contentHeight', quizQuestionPaddingEl.height() + 75);
        },

        getQuestionObject: function () {
            return this.questionObject;
        },

        startQuestionAnimation: function () {
            var that = this;
            if (that.animationObjectInstance != null) {
                that.animationObjectInstance.startAnimation();
            }
        },

        stopQuestionAnimation: function () {
            var that = this;
            if (that.animationObjectInstance != null) {
                that.animationObjectInstance.stopAnimation();
            }
        },

        setupAnswers: function (inputType) {
            var that = this;
            if ((inputType == "checkbox" || inputType == null) && that.fieldsetEl.find('.label_check input').length) {
                that.fieldsetEl.find('.label_check').removeClass('c_on');
                var markedCheckBoxes = that.fieldsetEl.find('.label_check input:checked');
                console.log("Checked elements : " + markedCheckBoxes.length);
                markedCheckBoxes.each(function () {
                    $(this).parent('label').addClass('c_on');
                });
            } else if ((inputType == "radio" || inputType == null) && that.fieldsetEl.find('.label_radio input').length) {
                that.fieldsetEl.find('.label_radio').removeClass('r_on');
                var markedRadioButtons = that.fieldsetEl.find('.label_radio input:checked');
                console.log("Checked elements : " + markedRadioButtons.length);
                markedRadioButtons.each(function () {
                    $(this).parent('label').addClass('r_on');
                });
            }
        },

        _getMultimediaPaddingEl: function () {
            var multimediaPaddingEl = $("<div>").addClass("multimediaPadding");
            if (this.leftRightAlignment) {
                multimediaPaddingEl.addClass("multimediaWithOptions");
            }
            return multimediaPaddingEl;
        },

        _getMultimediaWrapperEl: function (multimediaPlacement) {
            var multimediaWrapper = $("<div>").addClass("multimediaWrapper");
            if (multimediaPlacement == MultimediaPlacement.PLACEMENT_LEFT) {
                multimediaWrapper.css("padding-right", this.multimediaPaddingValue + "px");
            } else if (multimediaPlacement == MultimediaPlacement.PLACEMENT_BELLOW) {
                multimediaWrapper.css("padding-top", this.multimediaPaddingValue + "px");
            } else if (multimediaPlacement == MultimediaPlacement.PLACEMENT_ABOVE) {
                multimediaWrapper.css("padding-bottom", this.multimediaPaddingValue + "px");
            } else if (multimediaPlacement == MultimediaPlacement.PLACEMENT_RIGHT) {
                multimediaWrapper.css("padding-left", this.multimediaPaddingValue + "px");
            }
            return multimediaWrapper;
        },

        _getQuestionWrapperEl: function (hasMultimedia) {
            var questionWrapper;
            if (hasMultimedia) { // we have a multimedia attached
                questionWrapper = $("<div>").addClass("questionWrapper");
                if (this.leftRightAlignment) {
                    questionWrapper.addClass("questionWithOptionsAndMultimedia");
                }
            }
            return questionWrapper;
        },

        _getFormattedQuestionText: function (questionText, useNumbering, questionNo) {
            if (questionNo > 0 && useNumbering) { // question in a test
                var questionNoText = questionNo + ".";
                if (questionNo < 10) {
                    questionNoText = "0" + questionNo + ".";
                }
                questionText = questionText.replace("<p>", "<div class='questionText'><span class='quizTestNumber'>" + questionNoText + "</span>");
                questionText = questionText + "</div>";
            } else { // stand alone question
                questionText = questionText.replace("<p>", "<div class='questionText'>");
                questionText = questionText + "</div>";
            }
            // now we need to size this and scale properly
            var questionTextEl = $(questionText);
//            questionTextEl.css("zoom", 1);
            if (!this.leftRightAlignment) {
                questionTextEl.addClass("topBottomMultimedia");
            }
            return questionTextEl;
        },

        _getWrapperId: function (sceneDomId, objectDomId) {
            var retId = null;

            if (typeof (objectDomId) === "undefined") {
                retId = sceneDomId;
            }
            else {
                if (objectDomId.length == 0)
                    retId = sceneDomId;
                else {
                    retId = sceneDomId + "-" + objectDomId;
                }
            }
            return retId;
        },

        _getHTMLForMultimediaObject: function (MultimediaClass, multimediaObject, questionWithOptions, multimediaWrapperWidth, multimediaWrapperHeight) {
            var that = this,
                retHTMLEl = "",
                multimediaWidth = multimediaObject["width"],
                multimediaHeight = multimediaObject["height"],
                setWidth, setHeight,
                multimediaAspectRatio = multimediaWidth / multimediaHeight,
                wrapperAspectRatio = multimediaWrapperWidth / multimediaWrapperHeight;

            if (wrapperAspectRatio > multimediaAspectRatio) { // keep height of wrapper
                setHeight = multimediaWrapperHeight;
                setWidth = Math.round(multimediaWidth * setHeight / multimediaHeight);
            } else { // keep width of container
                setWidth = multimediaWrapperWidth;
                setHeight = Math.round(multimediaHeight * setWidth / multimediaWidth);
            }
            var multimediaURL = ResourceManager.getPathForImage(multimediaObject["objectId"], multimediaObject["extension"]);
            if (multimediaObject["type"] == "image") {
                retHTMLEl = $("<img>").addClass("questionMultimedia").attr("src", multimediaURL).attr("width", setWidth).attr("height", setHeight);
            } else if (multimediaObject["type"] == "audio") {
                retHTMLEl = $("<audio>").addClass("questionSound questionMultimedia").prop("controls", true).prop("autoplay", false).attr("src", multimediaURL);
            } else if (multimediaObject["type"] == "video") {
                retHTMLEl = $("<video>").addClass("questionMultimedia").prop("controls", true).prop("autoplay", false).attr("src", multimediaURL).attr("width", setWidth).attr("height", setHeight);
            } else if (multimediaObject["type"] == "chart") {
                if (questionWithOptions) {
                    var minSize = Math.min(multimediaWrapperWidth, multimediaWrapperHeight);
                    retHTMLEl = $("<div>").css("position", "relative").width(minSize).height(minSize);
                } else {
                    retHTMLEl = $("<div>").css("position", "relative").width(multimediaWrapperWidth).height(multimediaWrapperHeight);
                }
                new SceneObjectChart(JSON.parse(multimediaObject["options"]), retHTMLEl, null, "inline");
            } else if (multimediaObject["type"] == "animation") {
                var animationElement = new MultimediaClass(multimediaObject, new Backbone.Model());
                animationElement.init();
                retHTMLEl = $("<div>").width(multimediaWrapperWidth).height(multimediaWrapperHeight);
                retHTMLEl.append(animationElement.getView().render().$el);
            }
            return retHTMLEl;
        },

        _getSubmitButtonEl: function () {
            var that = this,
                scaleFactor = that.containerElModel.get('scaleFactor');

            var submitButton = $("<div>").addClass("questionSubmitButton questionButtonGradient")
                .text(CAPTIONS.QUIZ_CHECK_ANSWER);
//                .css("width", Math.round(that.QUESTION_SUBMIT_BUTTON_WIDTH * 1) + "px")
//                .css("height", Math.round(that.QUESTION_SUBMIT_BUTTON_HEIGHT * 1) + "px")
//                .css("margin-top", Math.round(that.QUESTION_SUBMIT_BUTTON_MARGIN_TOP * 1) + "px")
//                .css("margin-left", Math.round(that.QUESTION_SUBMIT_BUTTON_MARGIN_LEFT * 1) + "px")
//                .css("line-height", Math.round(that.QUESTION_SUBMIT_BUTTON_LINE_HEIGHT * 1) + "px")
//                .css("font-size", Math.round(that.QUESTION_SUBMIT_BUTTON_FONT_SIZE * 1) + "px");
            if (null != scaleFactor) {
                submitButton.css('width', that.QUESTION_SUBMIT_BUTTON_WIDTH * scaleFactor + 'px')
                    .css('height', that.QUESTION_SUBMIT_BUTTON_HEIGHT * scaleFactor + 'px')
                    .css('font-size', that.QUESTION_SUBMIT_BUTTON_FONT_SIZE * scaleFactor + 'px');
            }
            if (!this.leftRightAlignment) {
                submitButton.addClass("topBottomMultimedia");
            }
            submitButton.bind(AppUtils.EventTypes.mouseDown, function (e) {
                if (e.button != 2 &&       // discard right click, that we use for debugging
                    that.questionTouched) { // when check answer button is disabled it means user has not touched the question (NKI-25)
                    var questionFeedback = that.questionObject.questionFeedback;
                    var questionScoreInfo = that.computeQuestionScoreAndOptionLevelFeedback(false);
                    var questionScore = questionScoreInfo.questionScore;

                    var allFeedback = questionScoreInfo.allCorrect ? CAPTIONS.QUIZ_CORRECT_ANSWER : "";

                    if (that.firstTimeScore == null) { // this is first time when question is answered
                        allFeedback += CAPTIONS.QUIZ_QUESTION_SCORE + questionScore;
                        that.firstTimeScore = questionScore;
                    }

                    allFeedback = "<p>" + allFeedback + "</p>";

                    allFeedback += questionScoreInfo.optionLevelFeedback;


                    for (var feedbackIdx in questionFeedback) {
                        if (questionFeedback.hasOwnProperty(feedbackIdx)) {
                            var feedbackObject = questionFeedback[feedbackIdx];
                            if (questionScore >= feedbackObject["rangeMinValue"] && questionScore <= feedbackObject["rangeMaxValue"]) {
                                allFeedback += "<p>" + feedbackObject["feedbackText"] + "</p>";
                            }
                        }
                    }

                    that._showFeedback(allFeedback);
                    if (!questionScoreInfo.allCorrect) {
                        that._showRetryButton();
                    }
                    that._doFlipQuestionShowFeedback();
                }
            });
            return submitButton;
        },

        computeQuestionScoreAndOptionLevelFeedback: function (useFirstTimeScore) {
            var that = this,
                ret = {allCorrect: true},
                questionMaxScore = 0,
                questionScore = 0,
                optionLevelFeedback = "",
                correctOptionScore = that.questionObject.correctOptionScore,
                wrongOptionScore = that.questionObject.wrongOptionScore,
                questionAnswers = that.questionObject.questionAnswers;
            for (var answerIdx in questionAnswers) {
                if (questionAnswers.hasOwnProperty(answerIdx)) {
                    var answerObject = questionAnswers[answerIdx];
                    var currentInputEl = $("#" + answerObject["inputId"]);
                    var isCorrect = answerObject["validAnswer"];
                    var checked = currentInputEl.prop("checked");
                    if (checked) {
                        var answerFeedback = answerObject["answerFeedback"];
                        if (answerFeedback != "") {
                            optionLevelFeedback += "<p>" + answerFeedback + "</p>";
                        }
                        answerObject["userChecked"] = true;
                        if (isCorrect) {
                            questionScore += correctOptionScore;
                        } else {
                            ret.allCorrect = false; // discovered something checked that is not correct
                            questionScore += wrongOptionScore;
                        }
                    }
                    if (isCorrect) {
                        if (!checked) {
                            ret.allCorrect = false; // discovered something correct that was not checked
                        }
                        questionMaxScore += that.questionObject.correctOptionScore;
                    }
                }
            }
            that.questionObject.questionMaxScore = questionMaxScore;
            that.questionObject.questionLastScore = questionScore;

            that.containerElModel.set('questionMaxScore', questionMaxScore);

            var nkiScore = questionScore;  // see NKI-25

            if (that.firstTimeScore != null) {
                nkiScore = useFirstTimeScore ? that.firstTimeScore : 0
            } else {
                nkiScore = questionScore;
            }

            that.containerElModel.set('questionScore', nkiScore);

            ret.questionScore = questionScore;
            ret.optionLevelFeedback = optionLevelFeedback;
            return ret;
        },

        _showRetryButton: function () {

            var that = this,
                retryButtonEl = $("<div>").addClass("questionButtonGradient questionRetryButton").text(CAPTIONS.QUIZ_RETRY_QUESTION),
                scaleFactor = that.containerElModel.get('scaleFactor');

            if (null != scaleFactor) {
                retryButtonEl.css('width', that.QUESTION_SUBMIT_BUTTON_WIDTH * scaleFactor + 'px')
                    .css('height', that.QUESTION_SUBMIT_BUTTON_HEIGHT * scaleFactor + 'px')
                    .css('font-size', that.QUESTION_SUBMIT_BUTTON_FONT_SIZE * scaleFactor + 'px');
            }
            that.flipAnimationHolderEl.find(".questionFeedback").append(retryButtonEl);
            retryButtonEl.bind(AppUtils.EventTypes.mouseDown, function (e) {
                    if (e.button != 2) { // discard right click, that we use for debugging)
                        that.resetQuestion(true);
                    }
                }
            );
        },

        _doFlipQuestionShowFeedback: function () {
            var that = this;
            if (AppUtils.DeviceType.W8) {
                that.flipAnimationHolderEl.find(".frontFlip").css("display", "none");
                that.flipAnimationHolderEl.find(".backFlip").css("display", "block");
            } else {
                that.flipAnimationHolderEl.find(".backFlip").css("display", "block");
                that.flipAnimationHolderEl.addClass("flipAnimation flipTransform");   // flip with animation
            }
        },

        _showFeedback: function (feedbackText) {
            this.flipAnimationHolderEl.find(".questionFeedback").html(feedbackText);
        },

        _returnToAnswersView: function (resetWithAnimation) {
            var that = this;
            if (resetWithAnimation) {
                this.flipAnimationHolderEl.removeClass("flipTransform");
            } else {
                if (AppUtils.DeviceType.W8) {
                    that.flipAnimationHolderEl.find(".frontFlip").css("display", "block");
                    that.flipAnimationHolderEl.find(".backFlip").css("display", "none");
                }
                else {
                    that.flipAnimationHolderEl.find(".backFlip").css("display", "block");
                    that.flipAnimationHolderEl.removeClass("flipTransform flipAnimation");
                }
            }
        },

        _shuffleArray: function (array) {
            var len = array.length;
            var i = len;
            while (i--) {
                var p = parseInt(Math.random() * len);
                var t = array[i];
                array[i] = array[p];
                array[p] = t;
            }
            return array;
        }

    };

})
(window);
