(function (window) {

    var fadeTimeout = 500,
        questionTransitionTimeout = 500,
        quizTestFooterHeight = 70,

        SceneObjectQuizTest = function (testTitle, includeFirst, sceneDomId, objectDomId, type, iconURL) {
            var that = this;
            console.log("Add Quiz Test called for sceneDomId " + sceneDomId + ", objectDomId " + objectDomId + " and title: " + testTitle);
            that.testTitle = testTitle;
            that.includeFirst = includeFirst;
            that.sceneQuestionArray = [];

            if (typeof(getHookHandlersForPage) != "undefined" && typeof (getUnloadHandlersForPage) != "undefined") {
                var wrapperId = that._getWrapperId(sceneDomId, objectDomId);
                var hookHandlersForPage = getHookHandlersForPage(sceneDomId);
//                var unloadHandlersForPage = getUnloadHandlersForPage(sceneDomId);

                if (type === "icon") {
                    hookHandlersForPage.push(function () {
                        that.addTestQuizSymbol(wrapperId, iconURL);
                    });
                } else {
                    hookHandlersForPage.push(function () {
                        that.printTestQuiz(wrapperId);
                    });
                }
            }
        };

    SceneObjectQuizTest.prototype = {
        sceneQuestionArray: null, // used in preview
        questionsNavigator: null,
        testTitle: "",

        addQuestion: function (questionText, questionAnswers, questionNo, useNumbering, inputType, multimediaObject, randomOrderOptions, questionFeedback, correctOptionScore, wrongOptionScore, multimediaPlacement) {
            var that = this;
            var sceneQuestionObject = new SceneObjectQuizQuestion(questionText, questionAnswers, questionNo, useNumbering, inputType, multimediaObject, randomOrderOptions, questionFeedback, correctOptionScore, wrongOptionScore, multimediaPlacement, null, null, null);
            that.sceneQuestionArray.push(sceneQuestionObject);
        },

        addTestSummaryPage: function () {
            var that = this;
            var testSummary = {};
            testSummary.questionNo = that.sceneQuestionArray.length + 1;
            that.sceneQuestionArray.push(testSummary);
        },

        printTestQuiz: function (wrapper) {
            var that = this;
            var wrapperEl;
            if (wrapper instanceof jQuery) {
                wrapperEl = wrapper;
            } else {
                wrapperEl = $("#" + wrapper);
            }
            // set the background for the wrapper
            wrapperEl.css("background-color", "rgb(244, 244, 244)");
            var testTitleEl = $("<p>").addClass('quizTestTitle').text(that.testTitle);
            var questionNavigatorViewPort = $("<div>").addClass("questionNavigatorViewPort");
            var quizTestPaddingEl = $("<div>").addClass("quizPadding");
            quizTestPaddingEl.append(testTitleEl).append(questionNavigatorViewPort).append(that._getTestFooterEl());
            wrapperEl.append(quizTestPaddingEl);
            questionNavigatorViewPort.height((wrapperEl.height() - 40) - testTitleEl.outerHeight(true) - quizTestFooterHeight);
            if (that.sceneQuestionArray.length > 1) {
                that.questionsNavigator = questionNavigatorViewPort.quizNavigator({questionSceneObjects: that.sceneQuestionArray, testObject: that, startIndex: 0, transitionDuration: questionTransitionTimeout});
                that.questionsNavigator.quizNavigator("bindLeftNavigation", that.questionPreviousEl, TAP_SIMPLE_EVENT);
                that.questionsNavigator.quizNavigator("bindRightNavigation", that.questionNextEl, TAP_SIMPLE_EVENT);
            }

        },

        addTestQuizSymbol: function (wrapperId, iconURL) {
            var that = this;
            var wrapperEl = $("#" + wrapperId);
            wrapperEl.css("background-image", "url(" + iconURL + ")").css("background-size", "100%").// this should be stretched to have a good WYSIWYG feeling
                css("background-repeat", "no-repeat").css("background-position", "center");
            wrapperEl.bind(TAP_EVENT, function (e) {
                e.preventDefault();
                e.stopPropagation();
                disableNavigationSystem();
                emptyLightBoxContent();
                setLightBoxDimensions("90%", "90%");
                setLightBoxPosition("5%", "5%");
                showLightBoxForQuiz(20, enableNavigationSystem);
                that.printTestQuiz("lightBoxHtmlContent");
            });
        },

        updateCurrentQuestion: function (questionSceneObject) {
            var that = this;
            var questionObject = questionSceneObject.getQuestionObject();
            that._updateNavigationProgress(questionObject.questionNo);
            questionSceneObject.startQuestionAnimation();
        },

        printTestSummary: function (container) {
            var that = this;
            var testSummaryContentEl = $("<div>").addClass("testSummaryContent");
            var testSummaryTitleEl = $("<div>").addClass("testSummaryTitle").text(CAPTIONS.QUIZ_SCORE_SUMMARY);
            var testScoreResultsSectionEl = $("<div>").addClass("testScoreResultsSection");
            that.testBigPercentageEl = $("<div>").addClass("testBigPercentage");
            that.testSummaryNoPointsEl = $("<div>").addClass("testSummaryNoPoints");
            testScoreResultsSectionEl.append(that.testBigPercentageEl).append(that.testSummaryNoPointsEl);
            that.testRetryButtonEl = that._getTestRetryButtonEl(container);
            testSummaryContentEl.append(testSummaryTitleEl).append(testScoreResultsSectionEl).append(that.testRetryButtonEl);
            container.append(testSummaryContentEl);
            that.testRetryButtonEl.css("left", ((testSummaryContentEl.width() - that.testRetryButtonEl.width()) / 2) + "px");
        },

        updateTestSummaryScreenAfterTransition: function () {
            var that = this;
            var testInfo = that._computeTestScore();
            that.testSummaryNoPointsEl.text(CAPTIONS.QUIZ_SCORE_TEXT_POINTS_START + testInfo.testScore + CAPTIONS.QUIZ_SCORE_TEXT_POINTS_MIDDLE + testInfo.testMaxScore);
            that.questionPreviousEl.addClass("navigationDisabled"); // last screen (feedback), disable navigation
            that.questionNextEl.addClass("navigationDisabled");
            var testScorePercentage = testInfo.testScore * 100 / testInfo.testMaxScore;
            that._setDeceleratingTimeout(that._updateTestScorePercentage, that._updateTestPointsResult, 2, testScorePercentage / 2, testScorePercentage);
        },

        prepareTestSummaryBeforeTransition: function () {
            var that = this;
            that.testBigPercentageEl.text("0%");
            that.testSummaryNoPointsEl.css("opacity", "0").text("You got 0 points out of 0");
            that.testRetryButtonEl.css("opacity", "0");
        },

        stopPlayingMultimediaObjects: function (questionSlideEl) {
            var audioVideoElements = questionSlideEl.find("audio, video");
            if (audioVideoElements.length > 0) {
                audioVideoElements.each(function () {
                    $(this)[0].pause();
                });
            }

        },

        _getWrapperId: function (sceneDomId, objectDomId) {
            var retId = null;
            if (typeof (objectDomId) === "undefined") {
                retId = sceneDomId;
            }
            else {
                if (objectDomId.length == 0)
                    retId = sceneDomId;
                else {
                    retId = sceneDomId + "-" + objectDomId;
                }
            }
            return retId;
        },

        _updateNavigationProgress: function (questionNo) {
            var that = this;
            var howManyQuestions = that.sceneQuestionArray.length - 1;
            if (that.includeFirst == false) {
                howManyQuestions--;
            }
            that.testTextProgressEl.text(questionNo + " / " + howManyQuestions);
            var testProgressPercentage = questionNo * that.testProgressBarEl.width() / howManyQuestions - 4;
            that.testProgressGreenBar.width(testProgressPercentage);
            if ((questionNo == 1 && that.includeFirst == true) || (questionNo == 0 && that.includeFirst == false)) { // first question, disable left navigation
                that.questionPreviousEl.addClass("navigationDisabled");
            } else {
                that.questionPreviousEl.removeClass("navigationDisabled");
            }
            that.questionNextEl.removeClass("navigationDisabled");
        },

        _getTestRetryButtonEl: function (container) {
            var that = this;
            var testRetryButtonEl = $("<div>").addClass("testRetryButton questionButtonGradient").text(CAPTIONS.QUIZ_RETRY_TEST);
            testRetryButtonEl.bind(TAP_EVENT, function () {
                // reset saved information in sceneQuestionArray
                for (var questionIdx = 0; questionIdx < that.sceneQuestionArray.length - 1; questionIdx++) {
                    var questionObject = that.sceneQuestionArray[questionIdx].getQuestionObject();
                    questionObject.feedback = null;
                    for (var answerIdx = 0; answerIdx < questionObject.questionAnswers.length; answerIdx++) {
                        questionObject.questionAnswers[answerIdx]["userChecked"] = false;
                    }
                }
                var swipeWrapper = container.parent();
                // first question that is on the right slide now is already printed
                // so we need to manually reset it's state
                var firstQuestionSlide = swipeWrapper.find(".swipeSlide:first-child");
                var secondQuestionSlide = firstQuestionSlide.next();
                firstQuestionSlide.empty();
                that.sceneQuestionArray[0].printQuestion(firstQuestionSlide);
                if (that.sceneQuestionArray.length > 2) { // test has at least 2 questions so we need to reset the second question also
                    secondQuestionSlide.empty();
                    that.sceneQuestionArray[1].printQuestion(secondQuestionSlide);
                }
                that.updateCurrentQuestion(that.sceneQuestionArray[0]);
                that.questionsNavigator.quizNavigator("resetToFirstQuestion");
            });
            return testRetryButtonEl;
        },

        _setDeceleratingTimeout: function (callback, endCallback, factor, times, testScorePercentage) {
            var that = this;
            var startScore = 0;
            var internalCallback = function (t, counter) {
                return function () {
                    if (--t >= 0) {
                        counter += 2;
                        setTimeout(internalCallback, counter * factor);
                        startScore = Math.min(startScore + 2, testScorePercentage);
                        callback(that, startScore);
                    } else {
                        endCallback(that);
                    }
                }
            }(times, 0);
            setTimeout(internalCallback, factor);
        },

        _computeTestScore: function () {
            var that = this;
            var ret = {};
            ret.testScore = 0;
            ret.testMaxScore = 0;
            for (var questionIdx = 0; questionIdx < that.sceneQuestionArray.length - 1; questionIdx++) {
                var questionObject = that.sceneQuestionArray[questionIdx].getQuestionObject();
                ret.testScore += questionObject.questionLastScore;
                ret.testMaxScore += questionObject.questionMaxScore;
            }
            return ret;
        },

        _updateTestScorePercentage: function (that, currentScore) {
            that.testBigPercentageEl.text(currentScore + "%");
        },

        _updateTestPointsResult: function (that) {
            that.testSummaryNoPointsEl.animate({opacity: 1}, fadeTimeout);
            that.testRetryButtonEl.animate({opacity: 1}, fadeTimeout);
        },

        _getTestFooterEl: function () {
            var that = this;
            var testFooterEl = $("<div>").addClass("quizTestFooter");
            if (isDevice) {
                testFooterEl.addClass("noFlicker");
            }
            testFooterEl.append(that._getTestProgressWrapperEl()).append(that._getQuestionNavigatorEl());
            return testFooterEl;
        },

        _getTestProgressWrapperEl: function () {
            var that = this;
            var testProgressWrapperEl = $("<div>").addClass("testProgressWrapper");
            that.testTextProgressEl = that._getTestProgressTextEl();
            that.testProgressBarEl = that._getTestProgressBarEl();
            testProgressWrapperEl.append(that.testProgressBarEl).append(that.testTextProgressEl);
            return testProgressWrapperEl;
        },

        _getTestProgressBarEl: function () {
            var that = this;
            var testProgressBarEl = $("<div>").addClass("testProgressBar");
            that.testProgressGreenBar = that._getTestProgressGreenBar();
            testProgressBarEl.append(that.testProgressGreenBar);
            return testProgressBarEl;
        },

        _getTestProgressGreenBar: function () {
            return $("<div>").addClass("testProgressGreenBar");
        },

        _getTestProgressTextEl: function () {
            return $("<div>").addClass("testProgressText");
        },

        _getQuestionNavigatorEl: function () {
            var that = this;
            that.questionPreviousEl = that._getGoToPreviousQuestionEl();
            that.questionNextEl = that._getGoToNextQuestionEl();
            return $("<div>").addClass("questionNavigator").append(that.questionPreviousEl).append(that.questionNextEl);
        },

        _getGoToNextQuestionEl: function () {
            var buttonImage = $("<img>").attr("src", "gfx/quiz/quiz_next_arrow.png").addClass("navArrow");
            return $("<div>").addClass("questionNext questionButtonGradient").append(buttonImage);
        },

        _getGoToPreviousQuestionEl: function () {
            var buttonImage = $("<img>").attr("src", "gfx/quiz/quiz_previous_arrow.png").addClass("navArrow");
            return $("<div>").addClass("questionPrevious").append(buttonImage);
        }

    };

    window.SceneObjectQuizTest = SceneObjectQuizTest;

})(window);
