var RecordingState = {
    COUNTING_DOWN: "COUNTING_DOWN",
    SAVING: "SAVING",
    RECORDING: "RECORDING",
    NO_RECORDING: "NO_RECORDING"
};

var SoundRecorderConstants = {
    RECORDING_COUNTDOWN_START_VALUE: 5000, // ms
    RECORDING_COUNTDOWN_STEP: 1000,
    RECORDING_BASE_PATH: DirectJSCalls.getAppPublicStorageDir() + "/files/"
};

var SoundRecorderOperationMode = {
    SINGLE_PAGE: "SINGLE_PAGE",
    MULTIPLE_PAGE: "MULTIPLE_PAGE",
    MULTIPLE_PAGE_FINISHED: "MULTIPLE_PAGE_FINISHED"
};

var SoundRecorder = {

    recordingState: RecordingState.NO_RECORDING,
    countDownValue: 0,
    recordingPage: 0,
    recordingMedia: null,
    playingMedia: {},
    playingWizardMedia: {},
    recordingCountDownInterval: null,
    recordingLightBoxModel: null,
    recordSoundModel: null,
    playSoundModel: null,
    operationMode: SoundRecorderOperationMode.SINGLE_PAGE,

    setRecordingLightBoxModel: function (model) {
        this.recordingLightBoxModel = model;
    },

    setRecordSoundModel: function (recordSoundModel) {
        this.recordSoundModel = recordSoundModel;
    },

    setOperationMode: function (operationMode) {
        this.operationMode = operationMode;
    },

    setPlaySoundModel: function (playSoundModel) {
        this.playSoundModel = playSoundModel;
    },

    setRecordingState: function (recordingState) {
        this.recordingState = recordingState;
    },

    getRecordingState: function () {
        return this.recordingState;
    },

    getRecordedSoundAvailableForPage: function (pageNumber) {
        return AppUtils.getLocalStorageBooleanItem(Chapter.getSettings()["rootProjectId"] + '-recordedSoundForPage' + pageNumber, false);
    },

    updatePlayIconStateForPage: function (pageNumber) {
        if (this.playSoundModel) {
            this.playSoundModel.set("disable", !this.getRecordedSoundAvailableForPage(pageNumber));
        }
    },

    toggleSoundRecordingForPage: function (pageNumber) {
        var recordingState = this.getRecordingState();
        if (recordingState == RecordingState.NO_RECORDING) {
            this.prepareRecordingSoundForPage(pageNumber);
        } else if (recordingState == RecordingState.RECORDING) {
            this.stopAndStoreRecording();
        } else if (recordingState == RecordingState.COUNTING_DOWN) {
            console.log("Should abort recording");
            this.abortRecordWhileCountDownInProgress();
        } else if (recordingState == RecordingState.SAVING) {
            console.log("Please wait while saving sound");
        }
    },

    abortRecordWhileCountDownInProgress: function () {
        this.updateUIForCountDownAborted();
        this.setRecordingState(RecordingState.NO_RECORDING);
        if (this.recordingCountDownInterval != null) {
            clearInterval(this.recordingCountDownInterval);
            this.recordingCountDownInterval = null;
        }
    },

    prepareRecordingSoundForPage: function (pageNumber) {
        console.log("SoundRecorder prepareRecordingSoundForPage : " + pageNumber);
        this.recordingLightBoxModel.set("RecordingCountDownViewClassName", "recordingCountdown");
        this.setRecordingState(RecordingState.COUNTING_DOWN);
        this.recordingPage = pageNumber;
        this.countDownValue = SoundRecorderConstants.RECORDING_COUNTDOWN_START_VALUE;
        this.doRecordingCountDown();
    },

    startRecording: function () {
        console.log("SoundRecorder startRecording");
        var audioFilePath = this.getFileNameForRecording();
        try {
            this.setRecordingState(RecordingState.RECORDING);
            SoundRecorder.preformAudioRecording(audioFilePath);
        } catch (e) {
            console.log(e.message);
        }
    },

    doRecordingCountDown: function () {
        this.showRecordingCountDown(SoundRecorderConstants.RECORDING_COUNTDOWN_START_VALUE / SoundRecorderConstants.RECORDING_COUNTDOWN_STEP);
        this.recordingCountDownInterval = setInterval(function () {
            SoundRecorder.countDownValue -= SoundRecorderConstants.RECORDING_COUNTDOWN_STEP;
            if (SoundRecorder.countDownValue == 0) {
                clearInterval(SoundRecorder.recordingCountDownInterval);
                SoundRecorder.recordingCountDownInterval = null;
                SoundRecorder.updateUIForEndOfCountDown();
                SoundRecorder.startRecording();
            } else {
                SoundRecorder.updateRecordingContDownForValue(SoundRecorder.countDownValue / SoundRecorderConstants.RECORDING_COUNTDOWN_STEP);
            }
        }, SoundRecorderConstants.RECORDING_COUNTDOWN_STEP);
    },

    getFileNameForRecording: function () {
        if (this.operationMode == SoundRecorderOperationMode.MULTIPLE_PAGE) {
            return "page" + this.recordingPage + "_" + Chapter.getSettings()["rootProjectId"] + "_" + "recorded_wizard.amr";
        }
        return "page" + this.recordingPage + "_" + Chapter.getSettings()["rootProjectId"] + "_" + "recorded.amr";
    },

    getFileNameForPlaying: function (pageNumber, mode) {
        if (mode == SoundRecorderOperationMode.MULTIPLE_PAGE) {
            return "/sdcard/" + SoundRecorderConstants.RECORDING_BASE_PATH + "page" + pageNumber + "_" + Chapter.getSettings()["rootProjectId"] + "_" + "recorded_wizard.amr";
        }
        return "/sdcard/" + SoundRecorderConstants.RECORDING_BASE_PATH + "page" + pageNumber + "_" + Chapter.getSettings()["rootProjectId"] + "_" + "recorded.amr";
    },

    preformAudioRecording: function (audioFilePath) {
        var recordingFilePath = SoundRecorderConstants.RECORDING_BASE_PATH + audioFilePath;
        console.log("preformAudioRecording in file " + recordingFilePath);
        this.recordingMedia = new Media(recordingFilePath, this.onRecordSuccess, this.onRecordError);
        this.recordingMedia.startRecord(); // start recording audio
    },

    onRecordSuccess: function () {
        console.log("onRecordSuccess");
    },

    onRecordError: function (error) {
        console.log('onRecordError code: ' + error.code + '\n' + 'message: ' + error.message + '\n');
    },

    stopAndStoreRecording: function () {
        console.log("stopAndStoreRecording");
        try {
            this.updateUIForSavingOperation();
            this.setRecordingState(RecordingState.SAVING);
            if (this.operationMode == SoundRecorderOperationMode.SINGLE_PAGE) {
                localStorage.setItem(Chapter.getSettings()["rootProjectId"] + '-recordedSoundForPage' + this.recordingPage, true);
            } else if (this.operationMode == SoundRecorderOperationMode.MULTIPLE_PAGE) {
                localStorage.setItem(Chapter.getSettings()["rootProjectId"] + '-recordedWizardSoundForPage' + this.recordingPage, true);
            }
            this.recordingMedia.stopRecord();
            SoundRecorder.doOnEndOfRecording();
        } catch (error) {
            console.log("stopAndStoreRecording ERROR: " + error.message);
            this.doOnEndOfRecording();
        }
    },

    togglePlayRecordedSoundForPage: function (pageNumber) {
        console.log("togglePlayRecordedSoundForPage" + pageNumber);
        if (this.getRecordedSoundAvailableForPage(pageNumber)) {
            if (!this.playingMedia[pageNumber]) { // now we play
                this.playRecordedSoundForPage(pageNumber);
            } else { // now we stop
                this.stopPlayingRecordedSound(pageNumber);
            }
        } else {
            console.log("Recorded sound not available for page");
        }
    },

    playRecordedSoundForPage: function (pageNumber) {
        console.log("playRecordedSoundForPage");
        var recordedSoundURL = this.getFileNameForPlaying(pageNumber, SoundRecorderOperationMode.SINGLE_PAGE);
        console.log("Will play file with URL " + recordedSoundURL);
        this.updateUIForStartPlayRecordedSound();
        this.playingMedia[pageNumber] = new Media(recordedSoundURL,
            function () {
                SoundRecorder.onPlayFinish(pageNumber);
            }, function (error) {
                SoundRecorder.onPlayError(error, pageNumber);
            });
        console.log("New player for recorded media was created for page " + pageNumber);
//        this.playingMedia[pageNumber].setVolume(1.0);
        this.playingMedia[pageNumber].play();
    },

    onPlayError: function (error, pageNumber) {
        console.log("There was an error playing recorded sound " + error.message);
        SoundRecorder.playingMedia[pageNumber].release();
        delete SoundRecorder.playingMedia[pageNumber];
    },

    onPlayFinish: function (pageNumber) {
        console.log("Finish playing recorded sound");
        SoundRecorder.playingMedia[pageNumber].release();
        delete SoundRecorder.playingMedia[pageNumber];
        SoundRecorder.updateUIForEndPlayingRecordedSound();
    },

    endAllRecordedSounds: function (pageNumber) {
        if (typeof SoundRecorder != "undefined") {
            SoundRecorder.stopPlayingRecordedSound(pageNumber);
            SoundRecorder.stopPlayRecordedWizardSoundForPage(pageNumber);
        }
    },

    stopPlayingRecordedSound: function (pageNumber) {
        var allKeys, idx;
        if (pageNumber != null) {
            if (this.playingMedia[pageNumber]) {
                console.log("stopPlayingRecordedSound");
                this.playingMedia[pageNumber].stop();
            }
        } else {
            allKeys = Object.keys(this.playingMedia);
            for (idx = 0; idx < allKeys.length; idx++) {
                this.playingMedia[allKeys[idx]].stop();
            }
        }
    },

    showRecordingCountDown: function (value) {
        console.log("SoundRecorder showRecordingCountDown with start value : " + value);
        this.recordingLightBoxModel.set("displayedMessage", value);
        if (this.operationMode == SoundRecorderOperationMode.SINGLE_PAGE) {
            this.recordSoundModel.set("toggleGlow", true);
        }
    },

    updateRecordingContDownForValue: function (countDownValue) {
        console.log("SoundRecorder updateRecordingContDownForValue : " + countDownValue);
        this.recordingLightBoxModel.set("displayedMessage", countDownValue);
    },

    updateUIForCountDownAborted: function () {
        console.log("SoundRecorder updateUIForCountDownAborted");
        this.recordingLightBoxModel.set("interruptCountDown", true);
        if (this.operationMode == SoundRecorderOperationMode.SINGLE_PAGE) {
            this.recordSoundModel.set("toggleGlow", false);
        }
    },

    updateUIForEndOfCountDown: function () {
        console.log("SoundRecorder updateUIForEndOfCountDown");
        this.setRecordingState(RecordingState.RECORDING);
        this.recordingLightBoxModel.set("displayedMessage", '');
    },

    updateUIForSavingOperation: function () {
        console.log("SoundRecorder updateUIForSavingOperation");
        this.recordingLightBoxModel.set("RecordingCountDownViewClassName", "savingSoundMessage");
        this.recordingLightBoxModel.set("displayedMessage", CAPTIONS.SAVING_SOUND);
    },

    doOnEndOfRecording: function () {
        console.log("SoundRecorder doOnEndOfRecording");
        this.setRecordingState(RecordingState.NO_RECORDING);
        this.recordingLightBoxModel.set("recordingFinished", true);
        if (this.operationMode == SoundRecorderOperationMode.SINGLE_PAGE) {
            this.recordSoundModel.set("toggleGlow", false);
            this.playSoundModel.set("disable", false);
        }
    },

    updateUIForStartPlayRecordedSound: function () {
        console.log("SoundRecorder updateUIForStartPlayRecordedSound");
        if (this.playSoundModel) {
            this.playSoundModel.set("toggleGlow", true);
        }
    },

    updateUIForEndPlayingRecordedSound: function () {
        console.log("SoundRecorder updateUIForEndPlayingRecordedSound");
        if (this.playSoundModel) {
            this.playSoundModel.set("toggleGlow", false);
        }
    },

    getRecordedWizardSoundAvailableForPage: function (pageNumber) {
        return AppUtils.getLocalStorageBooleanItem(Chapter.getSettings()["rootProjectId"] + '-recordedWizardSoundForPage' + pageNumber, false);
    },

    startPlayRecordedWizardSoundForPage: function (pageNumber) {
        console.log("startPlayRecordedWizardSoundForPage for pageNumber " + pageNumber);
        var recordedSoundURL = this.getFileNameForPlaying(pageNumber, SoundRecorderOperationMode.MULTIPLE_PAGE);
        console.log("Will play file with URL " + recordedSoundURL);
        this.playingWizardMedia[pageNumber] = new Media(recordedSoundURL,
            function () {
                SoundRecorder.onPlayWizardFinish(pageNumber);
            }, function (error) {
                SoundRecorder.onPlayWizardError(error, pageNumber);
            });
        console.log("New player for recorded wizard media was created for page " + pageNumber);
//        this.playingWizardMedia[pageNumber].setVolume(1.0);
        this.playingWizardMedia[pageNumber].play();
    },

    onPlayWizardError: function (error, pageNumber) {
        console.log("There was an error playing recorded wizard sound " + error.message + " for page " + pageNumber);
        SoundRecorder.playingWizardMedia[pageNumber].release();
        delete SoundRecorder.playingWizardMedia[pageNumber];
    },

    onPlayWizardFinish: function (pageNumber) {
        console.log("Finish playing recorded wizard sound for page " + pageNumber);
        SoundRecorder.playingWizardMedia[pageNumber].release();
        delete SoundRecorder.playingWizardMedia[pageNumber];
    },

    stopPlayRecordedWizardSoundForPage: function (pageNumber) {
        var allKeys, idx;
        if (pageNumber != null) {
            if (this.playingWizardMedia[pageNumber]) {
                console.log("stopPlayRecordedWizardSoundForPage for pageNumber " + pageNumber);
                this.playingWizardMedia[pageNumber].stop();
            }
        } else {
            console.log("stopPlayRecordedWizardSoundForPage stop wizard for all pages");
            allKeys = Object.keys(this.playingWizardMedia);
            for (idx = 0; idx < allKeys.length; idx++) {
                this.playingWizardMedia[allKeys[idx]].stop();
            }
        }
    }
};