function ScrollingElementsWrapper() {
    var that = this;
    this.defineClasses = function () {
        that.ScrollingElementWrapperView = BaseView.extend({
            pageInTransition: false,
            tagName: "div",
            id: "slidingScenesWrapper",
            scrollLeft: 0,
            touchStartTime: 0,
            initialScrollLeft: 0,
            position: {},
            minDistanceForChanging: App.settings.width / 8,
            currentPosition: 0,
            swipeMaxPeriod: 300,
            swipeMinDistance: 50,
            start: null,
            direction: 0,
            mousePressed: false,
            //will be used when scrolling from navigation arrows or bottom menu in order to force rendering
            defaultScrollStep: 20,

            initialize: function (attr) {
                this.listenTo(this.model, "change:scrollLeft", this.setScrollLeft);
                this.listenTo(this.model, "change:orientation", this.updateOrientation);
                if (!AppUtils.useNativeScroll) {
                    this.listenTo(this.model, "change:currentPosition", this.setCurrentPosition);
                }
                this.el.scrollLeft = this.model.get("scrollLeft");
            },
            events: function () {
                var _events = {};
                if (!AppUtils.useNativeScroll) {
                    _events[AppUtils.EventTypes.mouseDown] = "mouseDownHandler";
                    _events[AppUtils.EventTypes.mouseUp] = "mouseUpHandler";
                    _events["touchcancel"] = "touchCancelHandler";
                }
                return _events;
            },

            touchCancelHandler: function (e) {
                console.log("ScrollingElementsWrapper touchCancelHandler");
                this.mousePressed = false;
                this.revertToCurrentPage();
            },

            mouseDownHandler: function (e) {
                console.log("ScrollingElementsWrapper mouseDownHandler");
                if (e.button != 2) { // discard right click, that we use for debugging
                    this.stopAnimation();
                    this.initialScrollLeft = this.el.scrollLeft;
                    console.log("GYL-271:mouseDownHandler:initialScrollLeft " + this.initialScrollLeft);
                    this.position = this.getPosition(e);
                    this.touchStartTime = new Date();
                    this.mousePressed = true;
                    Chapter.closeBottomMenu();
                }
            },
            mouseUpHandler: function () {
                if (this.mousePressed) {
                    console.log("ScrollingElementsWrapper mouseUpHandler");
                    this.touchEndTime = new Date();
                    this.scrollLeft = this.el.scrollLeft;
                    if (this.isSwipeEvent() || this.shouldNavigateToNextPage()) {
                        this.navigateToNextPage();
                    } else if (this.scrollLeft != this.initialScrollLeft) {
                        this.revertToCurrentPage();
                    } else {
                        console.log("GYL-271:mouseUpHandler: WHAT WE DO HERE ???");
                    }

                    console.log("GYL-271:mouseUpHandler:initialScrollLeft " + this.initialScrollLeft);
                    console.log("GYL-271:mouseUpHandler:scrollLeft " + this.scrollLeft);
                    this.mousePressed = false;
                } else {
                    console.log("Ignoring mouse up on slidingScenesWrapper");
                }
            },

            isSwipeEvent: function () {
                console.log("distance: " + Math.abs(this.initialScrollLeft - this.scrollLeft));
                console.log("time: " + (this.touchEndTime.getTime() - this.touchStartTime.getTime()));

                return (Math.abs(this.initialScrollLeft - this.scrollLeft) > this.swipeMinDistance) && ((this.touchEndTime.getTime() - this.touchStartTime.getTime()) < this.swipeMaxPeriod);
            },
            shouldNavigateToNextPage: function () {
                console.log("should navigate distance: " + Math.abs(this.initialScrollLeft - this.scrollLeft));
                return Math.abs(this.initialScrollLeft - this.scrollLeft) > this.minDistanceForChanging;
            },
            navigateToNextPage: function () {
                var nextPos, remainingOffset, thisView = this;
                if (this.initialScrollLeft < this.scrollLeft) {
                    nextPos = this.currentPosition - this.model.get("width");
                } else {
                    nextPos = this.currentPosition + this.model.get("width");
                }
                remainingOffset = nextPos + this.scrollLeft;
                this.currentPosition = nextPos;
                console.log("start translating to" + this.currentPosition);
                console.log("start translating remainingOffset" + remainingOffset);
                setTimeout(function () {
                    thisView.pageInTransition = true;
                    Chapter.translateScenesTo(remainingOffset, function (event) {
                        console.log("GYL-271:navigateToNextPage: end transition " + thisView.el.scrollLeft);
                        event.stopPropagation();
                        Chapter.getScrollingElementsView().removeTransitionEndHandler();
                        Chapter.setActivePage(thisView.getActivePage(nextPos));
                        Chapter.restorePageTransitionDuration();
                        thisView.pageInTransition = false;
                    });
                }, 0);

            },

            initiateScrollingForCurrentPage: function () {
                var newScroll = (this.direction * this.defaultScrollStep) + this.scrollLeft;
                console.log("GYL-271:initiateScrollingForCurrentPage " + newScroll);
                this.el.scrollLeft = newScroll;
                this.scrollLeft = newScroll;
                that.scrollingElementsWrapperView.navigateToNextPage();
            },

            navigateToNeighborPage: function (direction) {
                var thisView = this,
                    currentScrollPosition, nextPos;
                ZoomUtils.resetZoom();
                if (AppUtils.useNativeScroll) {
                    console.log("navigateToNeighborPage: using native scroll");
                    currentScrollPosition = this.el.scrollLeft;
                    nextPos = direction * App.settings.width + currentScrollPosition;
                    Chapter.getScrollingElementsView().addTransitionEndHandler(function () {
                        Chapter.setActivePage(thisView.getActivePage(nextPos));
                    });
                    require(["PhoneGapUtils"], function () {
                        window.plugins.PhoneGapUtils.scrollToOffsetWithAnimation(nextPos);
                    });
                } else {
                    this.initialScrollLeft = this.el.scrollLeft;
                    this.scrollLeft = this.initialScrollLeft;
                    this.direction = direction;
                    this.initiateScrollingForCurrentPage();
                }
            },

            revertToCurrentPage: function () {
                var remainingOffset, thisView = this;
                if (this.initialScrollLeft < this.scrollLeft) {
                    remainingOffset = this.currentPosition + this.scrollLeft;
                } else {
                    remainingOffset = this.currentPosition + this.scrollLeft;
                }
                console.log("start translating to" + this.currentPosition);
                thisView.pageInTransition = true;
                Chapter.translateScenesTo(remainingOffset, function () {
                    Chapter.removePageTransitionDuration();
                    Chapter.getScrollingElementsModel().set("translate3DOffset", 0);
                    console.log("GYL-271:revertToCurrentPage " + (-thisView.currentPosition));
                    thisView.el.scrollLeft = -thisView.currentPosition;
                    Chapter.restorePageTransitionDuration();
                    thisView.pageInTransition = false;
                });
            },

            getActivePage: function (nextPosition) {
                var ret;
                if (App.dualLandscapeActive()) {
                    ret = Math.abs(Math.round(nextPosition * 2 / this.model.get("width")));
                } else {
                    ret = Math.abs(Math.round(nextPosition / this.model.get("width")));
                }
                return ret
            },

            removeView: function () {
                this.remove();
                this.unbind();
            },
            render: function () {
                // used here only to init native scrolling
                if (AppUtils.useNativeScroll) {
                    require(["PhoneGapUtils"], function () {
                        window.plugins.PhoneGapUtils.initNativeScrolling();
                    });
                    this.$el.css("-webkit-overflow-scrolling", "touch")
                        .css("overflow-x", "scroll");
                }
                return this;
            },
            setScrollLeft: function () {
                this.currentPosition = -this.model.get("scrollLeft");
                this.el.scrollLeft = -this.currentPosition;
                console.log("MOB-1718: setScrollLeft " + (-this.currentPosition));
//                if (this.currentPosition % App.settings.width != 0) {
//                    console.log("GYL-271:Damn bug with page swipe");
//                }
            },
            setCurrentPosition: function (currentPosition) {
                this.currentPosition = this.model.get("currentPosition");
            },
            updateOrientation: function () {
                this.model.set("width", App.settings.width);
                this.model.set("height", App.settings.height);
            }
        });
    };
    this.instantiateObjects = function () {
        that.scrollingElementsWrapperModel = new Backbone.Model({
            width: App.settings.width,
            height: App.settings.height,
            scrollLeft: 0,
            currentPosition: 0
        });
        that.scrollingElementsWrapperView = new that.ScrollingElementWrapperView({
            model: that.scrollingElementsWrapperModel
        });

    };

    return {
        init: function () {
            that.defineClasses();
            that.instantiateObjects();
        },
        getModel: function () {
            return that.scrollingElementsWrapperModel;
        },
        getView: function () {
            return that.scrollingElementsWrapperView;
        }
    }
}