define(['FileDownloader', 'DownloadUI', 'JqueryKnob', 'UserDialogs'], function () {
        return function () {
            var versionedIdsMappingObjectLocalStorageKey = 'versionedIdsMappingObject',
                downloadedIdsMappingObjectLocalStorageKey = 'downloadedIdsMappingObject',
                downloadStatuses = {
                    DOWNLOADED: 1,
                    DOWNLOADED_OLDER_VERSION: 2,
                    UN_DOWNLOADED: 3
                },
                downloadStatus;

            function downloadProjectArchiveIfNeeded(projectId, successCallback, errorCallback) {

                var activeProjectId = getActiveProjectId(projectId);

                if (downloadStatus == downloadStatuses.UN_DOWNLOADED) {
                    downloadProjectArchive(projectId, activeProjectId, successCallback, errorCallback);
                } else if (downloadStatus == downloadStatuses.DOWNLOADED_OLDER_VERSION) {
                    UserDialogs.showConfirmation(CAPTIONS.TITLE_UPDATE_MESSAGE, CAPTIONS.UPDATE_AVAILABLE, CAPTIONS.YES + "," + CAPTIONS.NO, function (buttonIndex) {
                        confirmationCallback(buttonIndex, projectId, activeProjectId, successCallback, errorCallback);
                    });
                } else {
                    if (null != successCallback) {
                        successCallback();
                    }
                }
            }

            function downloadProjectArchive(projectId, activeProjectId, successCallback, errorCallback) {
                var relativePathPrefix = 'versionedProjects/';
                DownloadUI.displayDownloadBoxWithMessage('Download started');
                setTimeout(function () {
                    window.plugins.FileDownloader.downloadFile(relativePathPrefix + activeProjectId, ProjectSettings.URLs.projectArchiveDownloadURL() + activeProjectId, null, function (message) {
                        var downloadedProjectId = getDownloadedProjectId(projectId);
                        if (downloadedProjectId) {
                            window.plugins.FileDownloader.removeFileFromStorageFolder(relativePathPrefix + downloadedProjectId, function () {
                                console.log("DownloadService: deleted old title after update downloaded");
                            }, function () {
                                console.log("DownloadService: failed to delete old title after update downloaded");
                            });
                        }
                        updateDownloadedProjectsMappingObject(projectId, activeProjectId);
                        DownloadUI.removeUnzipSpinner();
                        if (null != successCallback) {
                            successCallback(message);
                        }
                    }, errorCallback);
                }, 500);
            }

            function confirmationCallback(buttonIndex, projectId, activeProjectId, successCallback, errorCallback) {
                if (buttonIndex == 1) { // YES
                    downloadProjectArchive(projectId, activeProjectId, successCallback, errorCallback);
                } else if (buttonIndex == 2) { // NO
                    if (null != successCallback) {
                        successCallback();
                    }
                }
            }

            function getActiveProjectId(originalProjectId) {
                var currentVersionedIdsMappingObject = localStorage.getItem(versionedIdsMappingObjectLocalStorageKey),
                    activeProjectId, downloadedProjectId;
                if (null != currentVersionedIdsMappingObject) {
                    currentVersionedIdsMappingObject = JSON.parse(currentVersionedIdsMappingObject);
                    activeProjectId = currentVersionedIdsMappingObject[originalProjectId];
                }
                if (null == activeProjectId) {
                    activeProjectId = originalProjectId;
                }

                downloadedProjectId = getDownloadedProjectId(originalProjectId);

                if (null != downloadedProjectId) {
                    if (activeProjectId == downloadedProjectId) {
                        downloadStatus = downloadStatuses.DOWNLOADED;
                    } else {
                        downloadStatus = downloadStatuses.DOWNLOADED_OLDER_VERSION;
                    }
                } else {
                    downloadStatus = downloadStatuses.UN_DOWNLOADED;
                }
                return activeProjectId;
            }

            function getDownloadedProjectId(originalProjectId) {
                var downloadedProjectId = null,
                    currentDownloadedIdsMappingObject = localStorage.getItem(downloadedIdsMappingObjectLocalStorageKey);

                if (null != currentDownloadedIdsMappingObject) {
                    currentDownloadedIdsMappingObject = JSON.parse(currentDownloadedIdsMappingObject);
                    downloadedProjectId = currentDownloadedIdsMappingObject[originalProjectId];
                }

                return downloadedProjectId;
            }

            function updateDownloadedProjectsMappingObject(projectId, activeProjectId) {
                var currentDownloadedIdsMappingObject = localStorage.getItem(downloadedIdsMappingObjectLocalStorageKey);
                if (null == currentDownloadedIdsMappingObject) {
                    currentDownloadedIdsMappingObject = {};
                } else {
                    currentDownloadedIdsMappingObject = JSON.parse(currentDownloadedIdsMappingObject);
                }
                currentDownloadedIdsMappingObject[projectId] = activeProjectId;
                localStorage.setItem(downloadedIdsMappingObjectLocalStorageKey, JSON.stringify(currentDownloadedIdsMappingObject));
            }


            return {
                downloadProjectArchive: downloadProjectArchiveIfNeeded
            }
        }
    }
);