define(['InAppPurchaseManager', 'SpinnerPlugin'], function () {
    return function () {
        var acoTestProductIds = [
                'no.aschehoug.barnebokhylla.fremmed',
                'no.aschehoug.barnebokhylla.frida.doktor',
                'no.aschehoug.barnebokhylla.frida.pannekake',
                'no.aschehoug.barnebokhylla.frida.stjele',
                'no.aschehoug.barnebokhylla.morovers',
                'no.aschehoug.barnebokhylla.savnet',
                'no.aschehoug.barnebokhylla.uvenner',
                'no.aschehoug.barnebokhylla.venner'
            ],
            purchaseSuccessCallback = null,
            purchaseFailCallback = null,
            requestProductsDataTransactionCallback = null,
            restoreProductsCallback = null,
            purchasedIdsMappingObjectLocalStorageKey = 'purchasedIdsMappingObject',
            ownedProducts = [];

        if (AppUtils.nativeApp) {
            window.plugins.inAppPurchaseManager.onPurchased = function (productId) {
                console.log('JS purchased product: ' + productId);
                updatePurchasedProjectsMappingObject(productId);
                $("body").SpinnerProgressIndicator("onComplete");
                if (null != purchaseSuccessCallback) {
                    purchaseSuccessCallback(productId);
                }
                Chapter.isPurchaseActive = false;
            };
            window.plugins.inAppPurchaseManager.requestProductsDataTransactionFinished = function (validProducts, invalidProducts) {
                console.log("requestProductsDataTransactionFinished called on JS");
                if (null != requestProductsDataTransactionCallback) {
                    requestProductsDataTransactionCallback(validProducts, invalidProducts);
                }
            };

            window.plugins.inAppPurchaseManager.onRestored = function (productId) {
                console.log("onRestored called on JS");
                updatePurchasedProjectsMappingObject(productId);
                ownedProducts.push(productId);
            };

            window.plugins.inAppPurchaseManager.onRestoreCompletedTransactionsFinished = function () {
                console.log("onRestoreCompletedTransactionsFinished called on JS");
                $("body").SpinnerProgressIndicator("onComplete");
                console.log('JS onRestoreCompletedTransactionsFinished');
                if (restoreProductsCallback) {
                    restoreProductsCallback();
                }
            };

            window.plugins.inAppPurchaseManager.onFailed = function (errNo, errText) {
                (null == errText) ? console.log('JS IAP failed with error') : console.log('JS IAP failed with error: ' + errText);
                $("body").SpinnerProgressIndicator("onComplete");
                if (null != purchaseFailCallback) {
                    purchaseFailCallback(errNo);
                }
                Chapter.isPurchaseActive = false;
            };

            window.plugins.inAppPurchaseManager.onRestoreCompletedTransactionsFailed = function (errNo) {
                console.log('JS onRestoreCompletedTransactionsFailed failed with error:  ' + errNo);
                $("body").SpinnerProgressIndicator("onComplete");
            };
        }

        this.requestProductsData = function (projectsIds, successCallback) {
            requestProductsDataTransactionCallback = successCallback;
            window.plugins.inAppPurchaseManager.requestProductsData(projectsIds, window.plugins.inAppPurchaseManager.requestProductsDataTransactionFinished);

        };
        /**
         * Call the native side for making the purchase of the product with the product id specified by the productId parameter
         * and quantity equals with one unity
         * @param productId  The product id for the product that will be purchased
         * @param successCallback The function executed if the purchase succeeded
         * @param errorCallback The function executed if the purchase failed
         */
        this.makePurchase = function (productId, successCallback, errorCallback) {
            if (wasPurchased(productId)) {
                if (null != successCallback) {
                    successCallback(productId);
                }
            } else {
                $("body").SpinnerProgressIndicator({showOnlySpinner: true});
                Chapter.isPurchaseActive = true;
                purchaseSuccessCallback = successCallback;
                purchaseFailCallback = errorCallback;
                if (AppUtils.DeviceType.iOS) {
                    window.plugins.inAppPurchaseManager.makePurchase(productId, 1);
                } else if (AppUtils.DeviceType.ANDROID) {
                    window.plugins.inAppPurchaseManager.startAndroidIABForProduct(productId, null, null);
                }
            }
        };
        this.isPurchaseActive = function () {
            return Chapter.isPurchaseActive
        };
        this.getProductPrice = function (model) {
            return model.get('appleProductPrice');
        };
        this.getProductId = function (model) {
            return model.get('appleProductId');
        };
        this.getProductIdFromObject = function (object) {
            return  object['appleProductId'];
        };
        this.restoreProducts = function (callback) {
            ownedProducts = [];
            restoreProductsCallback = callback;
            if (AppUtils.nativeApp && window.plugins.inAppPurchaseManager) {
                window.plugins.inAppPurchaseManager.restoreCompletedTransactions();

            }
        };
        this.getOwnedProducts = function () {
            return ownedProducts;
        };
        this.setPurchasedProduct = function (productId) {
            updatePurchasedProjectsMappingObject(productId);
            ownedProducts.push(productId);
        };

        /**
         * Check if the product with specified product id was purchased
         * @param productId  The id of the product that will be verified
         * @returns {boolean} TRUE if the product was purchased. Otherwise it will return FALSE
         */
        function wasPurchased(productId) {
            var currentPurchasedIdsMappingObject, wasPurchased = false;
            if (productId == '') {
                wasPurchased = true;
            } else {
                currentPurchasedIdsMappingObject = localStorage.getItem(purchasedIdsMappingObjectLocalStorageKey);
                if (null != currentPurchasedIdsMappingObject) {
                    currentPurchasedIdsMappingObject = JSON.parse(currentPurchasedIdsMappingObject);
                    wasPurchased = currentPurchasedIdsMappingObject[productId] != null;
                }
            }
            return wasPurchased;
        }

        /**
         * Update the local storage with information about which projects were purchased
         * @param productId The id of newly purchased product
         */
        function updatePurchasedProjectsMappingObject(productId) {
            var currentPurchasedIdsMappingObject = localStorage.getItem(purchasedIdsMappingObjectLocalStorageKey);
            if (null == currentPurchasedIdsMappingObject) {
                currentPurchasedIdsMappingObject = {};
            } else {
                currentPurchasedIdsMappingObject = JSON.parse(currentPurchasedIdsMappingObject);
            }
            currentPurchasedIdsMappingObject[productId] = true;
            localStorage.setItem(purchasedIdsMappingObjectLocalStorageKey, JSON.stringify(currentPurchasedIdsMappingObject));
        }

        return {
            setPurchasedProduct: this.setPurchasedProduct,
            requestProductsData: this.requestProductsData,
            makePurchase: this.makePurchase,
            isPurchaseActive: this.isPurchaseActive,
            getProductId: this.getProductId,
            getProductPrice: this.getProductPrice,
            getProductIdFromObject: this.getProductIdFromObject,
            restoreProducts: this.restoreProducts,
            getOwnedProducts: this.getOwnedProducts
        }
    }
});