define([], function () {
    return function () {
        var separator = '/',
            versionedIdsMappingObjectLocalStorageKey = 'versionedIdsMappingObject',
            projectMetaFileName = '_projectMeta.json',
            projectMetaKey = 'projectMeta',
            defaultFileNameKey = 'Default',
            fallbackFileNameKey = 'index',
            relativePathPrefix = '/versionedProjects/',
            supportedOrientations = {
                landscape: 'landscapeOrientation',
                portrait: 'portraitOrientation',
                all: 'all'
            };


        return {
            goToProject: goToProject,
        };

        function getActiveProjectId(rootObjectId) {
            var currentVersionedIdsMappingObject = localStorage.getItem(versionedIdsMappingObjectLocalStorageKey),
                activeProjectId;
            if (null != currentVersionedIdsMappingObject) {
                currentVersionedIdsMappingObject = JSON.parse(currentVersionedIdsMappingObject);
                activeProjectId = currentVersionedIdsMappingObject[rootObjectId];
            }
            return activeProjectId;
        }

        function goToProject(rootObjectId) {
            var activeProjectId = getActiveProjectId(rootObjectId);
            if (AppUtils.nativeApp) {
                if (null != activeProjectId) {
                    getProjectMetaDataAndNavigateToProject(ProjectGlobals.DOWNLOAD_PATH + relativePathPrefix + activeProjectId + separator, setCurrentOrientationAndNavigateToProject);
                } else {
                    UserDialogs.showAlert('Project does not exist');
                }

            } else  if(AppUtils.isSCORM && null != rootObjectId) {
                getProjectMetaDataAndNavigateToProject(rootObjectId, setCurrentOrientationAndNavigateToProject);
            } else {
                var url = ProjectSettings.URLs.shareUrlRequestURL() + rootObjectId;
                HTTPRequestUtils.getInstance().getJSON(url, function (shareUrl) {
                    window.location.href = ProjectSettings.URLs.IAPProjectURL() + shareUrl.shareURL;
                }, function (e) {
                });
            }
        }

        function getProjectMetaDataAndNavigateToProject(relativePath, callback) {
            var url = relativePath + projectMetaFileName,
                metaKey = projectMetaKey;

            if(AppUtils.isSCORM)
                url = projectMetaFileName;

                $.getJSON(url, function (data) {
                console.log("_getAvailableOrientations: success");
                callback(relativePath, data[metaKey]);
            })
                .done(function (data) {
//                console.log("_getAvailableOrientations: second success");
                })
                .fail(function (error) {
                    console.log('Failed to navigateto specified project' + error);
                })
                .always(function () {
//                console.log("_getAvailableOrientations: complete");
                });
        }

        function setCurrentOrientationAndNavigateToProject(relativePath, projectMeta) {
            var availableFiles = projectMeta['availableFiles'],
                supportedOrientations = projectMeta['supportedOrientations'],
                deviceType, fileName = "", resolutionKey;

            if(AppUtils.isSCORM){
                fileName = availableFiles[defaultFileNameKey + "_" + relativePath];

                if(fileName != null && fileName != "")  {
//                    alert("Navigating to exported linked project.");
                    navigateToProject(fileName + '.html');
                }

                return;
            }


            localStorage.setItem('BookShelfLandscapeOrientation', true);
            localStorage.setItem('BookShelfPortraitOrientation', false);

            localStorage.setItem(supportedOrientations.landscape, false);
            localStorage.setItem(supportedOrientations.portrait, false);

            localStorage.removeItem('BookLandscapeOrientation');
            localStorage.removeItem('BookPortraitOrientation');

            deviceType = getDeviceType();

            if (null == availableFiles[deviceType]) {
                if (null == availableFiles[defaultFileNameKey]) {
                    if (null == availableFiles[fallbackFileNameKey]) {
                        // should never happen
                        alert("Current project does not exist");
                        return;
                    } else {
                        fileName += fallbackFileNameKey;
                        resolutionKey = fallbackFileNameKey;
                    }
                } else {
                    fileName += availableFiles[defaultFileNameKey];
                    resolutionKey = availableFiles[defaultFileNameKey];
                }
            } else {
                fileName += availableFiles[deviceType];
                resolutionKey = availableFiles[deviceType];
            }


//            if (null != supportedOrientations[resolutionKey]) {
//                setNextOrientation(supportedOrientations[resolutionKey]);
//            }

//            require(["PluginUtils"], function () {
//                window.plugins.PluginUtils.recheckScreenOrientation();
//
//            });
            navigateToProject(relativePath + fileName + '.html');

        }

        function getDeviceType() {
            var width, height;
            if (AppUtils.DeviceType.ANDROID) {
                width = window.DirectJSCalls.getWindowWidth();
                height = window.DirectJSCalls.getWindowHeight();
            } else if (AppUtils.DeviceType.iOS) {
                width = window.innerWidth;
                height = window.innerHeight;
            }
            width = Math.max(width, height);

            return width < 1000 ? 'iPhone' : 'iPad';
        }

        function navigateToProject(absoluteURL) {
            setTimeout(function () {
                window.location.href = absoluteURL;
            }, 100);
        }

        function setNextOrientation(orientations) {
            if (null != orientations) {
                if (orientations.indexOf('portrait') > -1) {
                    localStorage.setItem(supportedOrientations.portrait, true);
                    if (orientations.indexOf('landscape') > -1) {
                        localStorage.setItem(supportedOrientations.landscape, true);
                        if (AppUtils.DeviceType.ANDROID)
                            window.DirectJSCalls.setAvailableOrientation('ALL');
                    } else {
                        if (AppUtils.DeviceType.ANDROID)
                            window.DirectJSCalls.setAvailableOrientation('PORTRAIT');
                    }
                } else {
                    localStorage.setItem(supportedOrientations.landscape, true);
                    if (AppUtils.DeviceType.ANDROID)
                        window.DirectJSCalls.setAvailableOrientation('LANDSCAPE');
                }
            } else {
                localStorage.setItem(supportedOrientations.landscape, true);
                if (AppUtils.DeviceType.ANDROID)
                    window.DirectJSCalls.setAvailableOrientation('LANDSCAPE');
            }
        }
    }
});