define(['IAPService'], function (IAPService) {
        return function () {
            var versionedIdsMappingObjectLocalStorageKey = 'versionedIdsMappingObject';

            this.updateVersionedProjects = function (originalModels) {
                var originalProjectsIds = [], model, index, id;
                if (originalModels.length > 0) {
                    for (index = 0; index < originalModels.length; index++) {
                        model = originalModels[index];
                        id = model.get('goToObjectId');
                        if (null != id && id.length > 0 && model.get('goToObjectType') == GoToObjectType.GoToProject) {
                            originalProjectsIds.push(model.get('goToObjectId'));
                        }
                    }
                    if (originalProjectsIds.length > 0) {
                        HTTPRequestUtils.getInstance().doRequest(ProjectSettings.URLs.versionedProjectsUpdateUrl(), ProjectSettings.HTTPRequestSettings.requestType.POST, "objectList=" + JSON.stringify(originalProjectsIds),
                            ProjectSettings.HTTPRequestSettings.contentTypeHeader, ProjectSettings.HTTPRequestSettings.acceptHeader, function (response) {
                                saveVersionedChapterIds(response);
                                startAutoUpdateForVersionedProjects();
                            }, function (error) {
                                startAutoUpdateForVersionedProjects();
                                console.log("Error updating the versioned chapters" + error)
                            });
                    }
                }
            };


            this.updatePricesForIAPPricePlaceHolders = function (models) {
                var model, index, content,
                    regex = "{PRICE_WITH_CURRENCY}",
                    productId,
                    product,
                    productsIds = [],
                    iapService = new IAPService(),
                    modelsToBeUpdated = [];
                for (index = 0; index < models.length; index++) {
                    model = models[index];
                    if (model.get('type') == 'textBox') {
                        content = model.get('textContent');
                        if (content.indexOf(regex) > -1) {
                            productsIds.push(iapService.getProductPrice(model));
                            modelsToBeUpdated.push(model);
                        }
                    }
                }
                if (productsIds.length > 0) {
                    iapService.requestProductsData(productsIds, function (validProductIds, invalidProductIds) {
                        for (index = 0; index < modelsToBeUpdated.length; index++) {
                            model = modelsToBeUpdated[index];
                            productId = iapService.getProductPrice(model);
                            product = getProduct(productId, validProductIds, invalidProductIds);
                            if (null != product) {
                                content = model.get('textContent').replace(regex, product.price);
                                model.set('textContent', content);
                            }

                        }
                    });
                }
            };


            function getProduct(productId, validProductIds, invalidProductIds) {
                var product, index;
                if (null != validProductIds) {
                    for (index = 0; index < validProductIds.length; index++) {
                        if (validProductIds[index]['id'] == productId) {
                            product = validProductIds[index];
                        }
                    }
                }
                if (null == product) {
                    if (null != invalidProductIds) {
                        for (index = 0; index < invalidProductIds.length; index++) {
                            if (invalidProductIds[index]['id'] == productId) {
                                product = invalidProductIds[index];
                            }
                        }
                    }
                }
                return product;
            }

            function saveVersionedChapterIds(versionedIdsMapping) {
                var versionedIdsMappingArray = JSON.parse(versionedIdsMapping),
                    currentVersionedIdsMappingObject = localStorage.getItem(versionedIdsMappingObjectLocalStorageKey),
                    index, versionedIdMap;
                if (null == currentVersionedIdsMappingObject) {
                    currentVersionedIdsMappingObject = {};
                } else {
                    currentVersionedIdsMappingObject = JSON.parse(currentVersionedIdsMappingObject)
                }
                for (index = 0; index < versionedIdsMappingArray.length; index++) {
                    versionedIdMap = versionedIdsMappingArray[index];
                    currentVersionedIdsMappingObject[versionedIdMap['requestedId']] = versionedIdMap['activeId'];
                }

                localStorage.setItem(versionedIdsMappingObjectLocalStorageKey, JSON.stringify(currentVersionedIdsMappingObject));

            }

            function startAutoUpdateForVersionedProjects() {
                var POLL_PERIOD = 300000;// 5 minutes
                if (null == window.autoUpdateVersionedProjectsTimer) {
                    window.autoUpdateVersionedProjectsTimer = setInterval(function () {
                        updatedExistingVersionedProjects();
                    }, POLL_PERIOD);
                }
            }

            function updatedExistingVersionedProjects() {
                var currentVersionedIdsMappingObject = localStorage.getItem(versionedIdsMappingObjectLocalStorageKey),
                    projectsIdsThatNeedChecked = [], projectId;
                if (null != currentVersionedIdsMappingObject) {
                    currentVersionedIdsMappingObject = JSON.parse(currentVersionedIdsMappingObject);
                }
                for (projectId in currentVersionedIdsMappingObject) {
                    if (currentVersionedIdsMappingObject.hasOwnProperty(projectId)) {
                        projectsIdsThatNeedChecked.push(projectId);
                    }
                }
                if (projectsIdsThatNeedChecked.length > 0) {
                    HTTPRequestUtils.getInstance().doRequest(ProjectSettings.URLs.versionedProjectsUpdateUrl(), ProjectSettings.HTTPRequestSettings.requestType.POST, "objectList=" + JSON.stringify(projectsIdsThatNeedChecked),
                        ProjectSettings.HTTPRequestSettings.contentTypeHeader, ProjectSettings.HTTPRequestSettings.acceptHeader, function (response) {
                            saveVersionedChapterIds(response)
                        }, function (error) {
                            console.log("Error updating the versioned chapters" + error)
                        });
                }
            }

            return {
                updateVersionedProjects: this.updateVersionedProjects,
                updatePricesForIAPPricePlaceHolders: this.updatePricesForIAPPricePlaceHolders
            }
        }
    }
);
