if (!window["ViewportPadding"]) window["ViewportPadding"] = {};
(function (ViewportPadding) {

    ViewportPadding.constants = {
        "LANDSCAPE_TOGGLE_BUTTON": "gfx/browserToolbar/landscapeView.png",
        "PORTRAIT_TOGGLE_BUTTON": "gfx/browserToolbar/portraitView.png"
    };

    ViewportPadding.settings = {};

    ViewportPadding.defineClasses = function () {

        ViewportPadding.PaddingAreaView = BaseView.extend({
            tagName: "div",
            className: "paddingAreaClass",

            initialize: function () {
                this.addListeners();
            },

            addListeners: function () {
                this.stopListening();
                this.listenTo(this.model, "change:remove", this.removeView);
                this.listenTo(this.model, "change:orientation", this.updateOrientation);
                this.listenTo(this.model, "change:hide", this.updateVisibility);
            },

            removeView: function () {
                if (this.model.get("remove")) {
                    this.remove();
                    this.unbind();
                }
            },

            updateVisibility: function () {
                if (this.model.get('hide')) {
                    this.$el.hide();
                } else {
                    this.$el.show();
                }
            },
            updateOrientation: function () {
                if (this.model.get("isTop") || this.model.get("isBottom")) {
                    this.$el.height(ViewportPadding.settings.paddingAreaHeight / 2);
                } else { // left or right
                    this.setWidth(ViewportPadding.settings.paddingAreaWidth / 2);
                }
            },

            addOrientationButton: function (bottomStackContainer) {
                if ((Chapter.getSettings()["useTwoInOneLandscapeMode"] || ProjectGlobals.ORIENTATION != SupportedOrientation.ALL) && !AppUtils.DeviceType.MOBILE_DEVICE) {
                    var toggleOrientationButtonImageURL = (App.lastKnownOrientation == SupportedOrientation.PORTRAIT) ? ViewportPadding.constants["LANDSCAPE_TOGGLE_BUTTON"] : ViewportPadding.constants["PORTRAIT_TOGGLE_BUTTON"],
                        toggleOrientationButton;
                    ViewportPadding.toggleOrientationButtonModel = new Backbone.Model({imageURL: toggleOrientationButtonImageURL});
                    toggleOrientationButton = new ViewportPadding.ToggleOrientationButtonView({model: ViewportPadding.toggleOrientationButtonModel});
                    bottomStackContainer.prepend(toggleOrientationButton.render().el);
                }
            },

            addZoomControl: function (bottomStackContainer) {
                if (Chapter.getSettings()["zoomEnabled"]) {
                    require(['ZoomControls'], function (ZoomControls) {
                        ViewportPadding.zoomControls = new ZoomControls();
                        ViewportPadding.zoomControls.init();
                        bottomStackContainer.prepend(ViewportPadding.zoomControls.getView().render().el);
                    });
                }
            },

            addRightButtonsBar: function (bottomStackContainer) {
                this.addOrientationButton(bottomStackContainer);
            },

            addLeftButtonsBar: function (bottomStackContainer) {
                this.addZoomControl(bottomStackContainer);
            },

            renderHorizontalPaddingArea: function () {
                var leftPaddingAreaNavigationArrowView,
                    rightPaddingAreaNavigationArrowView,
                    chapterSettings = Chapter.getSettings(),
                    centerAlignContainer, bottomStackContainer;

                //TODO: Discuss with Sondre and see how will we handle it in long terms
//                if (ViewportPadding.isPaddingAreaPossible() && chapterSettings.useNavigationArrows) {
                if (ViewportPadding.isPaddingAreaPossible()) {
//                    $("body").css("background-color", "#404041");
                    centerAlignContainer = $("<div>").addClass("centerAlignContainer");
                    bottomStackContainer = $("<div>").addClass("bottomStackContainer");
                    centerAlignContainer.append(bottomStackContainer);
                    this.$el.append(centerAlignContainer);
                    this.setWidth(Math.round(ViewportPadding.settings.paddingAreaWidth / 2));
                    this.setHeight("100%");
                    if (this.model.get("isRight")) {
                        this.setRight(0);
                        ViewportPadding.rightPaddingAreaNavigationArrowModel = new Backbone.Model({
                            isRight: true,
                            imageURL: chapterSettings.rightPaddingNavigationArrow
                        });
                        rightPaddingAreaNavigationArrowView = new ViewportPadding.PaddingAreaNavigationArrowView({
                            model: ViewportPadding.rightPaddingAreaNavigationArrowModel
                        });
                        bottomStackContainer.append(rightPaddingAreaNavigationArrowView.render().el);
                        this.addRightButtonsBar(bottomStackContainer);
                    } else if (this.model.get("isLeft")) {
                        ViewportPadding.leftPaddingAreaNavigationArrowModel = new Backbone.Model({
                            isLeft: true,
                            imageURL: chapterSettings.leftPaddingNavigationArrow
                        });
                        leftPaddingAreaNavigationArrowView = new ViewportPadding.PaddingAreaNavigationArrowView({
                            model: ViewportPadding.leftPaddingAreaNavigationArrowModel
                        });
                        bottomStackContainer.append(leftPaddingAreaNavigationArrowView.render().el)
                        this.addLeftButtonsBar(bottomStackContainer);
                    }
                    return this;
                }
                return null;
            },

            renderVerticalPaddingArea: function () {
                console.log("Now add top and bottom padding zones " + ViewportPadding.settings.paddingAreaHeight);
                if (ViewportPadding.settings.paddingAreaHeight > 0) {
//                    $("body").css("background-color", "#404041");
                    this.$el.addClass("verticalPadding");
                    this.$el.height(Math.round(ViewportPadding.settings.paddingAreaHeight / 2));
                    if (this.model.get("isTop")) {
                        this.$el.css("top", 0);
                    } else if (this.model.get("isBottom")) {
                        this.$el.css("bottom", 0);
                    }
                    return this;
                }
                return null;
            },

            render: function () {
                if (this.model.get("isLeft") || this.model.get("isRight")) {
                    return this.renderHorizontalPaddingArea();
                } else if (this.model.get("isTop") || this.model.get("isBottom")) {
                    return this.renderVerticalPaddingArea();
                }
                this.$el.hide();

            }
        });

        ViewportPadding.ToggleOrientationButtonView = BaseView.extend({
            className: "toggleOrientationButton",
            tagName: "div",
            template: templates.getTemplate('simpleImage'),

            initialize: function () {
                this.listenTo(this.model, "change:orientation", this.updateOrientation);
            },

            removeView: function () {
                if (this.model.get("remove")) {
                    this.remove();
                    this.unbind();
                }
            },

            events: function () {
                var _events = {};
                _events[AppUtils.EventTypes.click + " img"] = "changeOrientationHandler";
                return _events;
            },

            changeOrientationHandler: function () {
                // first update the flag for lastKnownOrientation
                if (App.lastKnownOrientation == SupportedOrientation.PORTRAIT) {
                    App.lastKnownOrientation = SupportedOrientation.LANDSCAPE;
                } else {
                    App.lastKnownOrientation = SupportedOrientation.PORTRAIT;
                }
                App.updateForNewOrientation(App.lastKnownOrientation);
            },

            updateOrientation: function () {
                var newOrientation = this.model.get("orientation");
                if (newOrientation == SupportedOrientation.PORTRAIT) {
                    this.imgEl.attr("src", ViewportPadding.constants["LANDSCAPE_TOGGLE_BUTTON"]);
                } else {
                    this.imgEl.attr("src", ViewportPadding.constants["PORTRAIT_TOGGLE_BUTTON"]);
                }
            },

            render: function () {
                this.$el.html(this.template(this.model.attributes));
                this.imgEl = this.$el.find("img");
                return this;
            }
        });

        ViewportPadding.PaddingAreaNavigationArrowView = BaseView.extend({
            tagName: "div",
            className: "paddingAreaNavigationAreaClass",
            template: templates.getTemplate('retinaImage'),
            canUpdateState: true,

            initialize: function () {
                this.listenTo(this.model, "change:enable", this.toggleEnableHandler);
                this.listenTo(this.model, "change:pageNumber", this.updateForPage);
                this.listenTo(this.model, "change:orientation", this.updateOrientation);
                this.listenTo(this.model, "change:hide", this.updateVisibility);
            },

            events: function () {
                var _events = {};
//                _events[AppUtils.EventTypes.click + " img"] = "navigationArrowPressed";
                _events[AppUtils.EventTypes.click] = "navigationArrowPressed";
                return _events;
            },
            updateVisibility: function () {
                if (this.model.get('hide')) {
                    this.$el.hide();
                } else {
                    this.$el.show();
                }
            },
            updateOrientation: function () {
                this.model.set("pageNumber", Chapter.getActivePage());
                this.updateForPage();
            },

            toggleEnableHandler: function () {
                if (this.model.get("enable")) {
                    this.canUpdateState = true;
                    this.$el.removeClass("disabled");
                } else {
                    this.canUpdateState = false;
                    this.$el.addClass("disabled");
                }
            },

            updateForPage: function () {
                console.log("PaddingAreaNavigationArrowView: updateForPage " + this.model.get("pageNumber"));
                if (this.canUpdateState) {
                    if (this.model.get("isRight")) { // right arrow
                        if (this.model.get("pageNumber") == App.maxActivePageNumberForDualLandscape()) {
                            this.$el.addClass("disabled");
                        } else {
                            this.$el.removeClass("disabled");
                        }
                    } else if (this.model.get("isLeft")) { // left arrow
                        if (this.model.get("pageNumber") == 0) {
                            this.$el.addClass("disabled");
                        } else {
                            this.$el.removeClass("disabled");
                        }
                    }
                } else {
                    console.log("Update padding arrow state disabled");
                }
            },

            navigationArrowPressed: function () {
                var scrollingElementsWrapperView = Chapter.getScrollingElementsWrapper().getView(),
                    willNavigate = false,
                    canNavigate = true;

                if (typeof SoundRecorder != "undefined" && SoundRecorder.operationMode == SoundRecorderOperationMode.MULTIPLE_PAGE) {
                    canNavigate = false;
                }
                if (this.$el.hasClass("disabled")) {
                    canNavigate = false;
                }
                if (scrollingElementsWrapperView.pageInTransition) {
                    canNavigate = false;
                }

                if (AppUtils.DeviceType.IE9 == true)
                    canNavigate = true;

                if (canNavigate) {
                    if (this.model.get("isRight")) {
                        willNavigate = true;
                        scrollingElementsWrapperView.navigateToNeighborPage(1);
                    } else if (this.model.get("isLeft")) {
                        willNavigate = true;
                        scrollingElementsWrapperView.navigateToNeighborPage(-1);
                    }
                }
                console.log("Will navigate:" + willNavigate + " can navigate:" + canNavigate);
            },

            render: function () {
                this.$el.html(this.template(this.model.attributes));
                this.$el.hide();
                return this;
            }
        });
    };

    ViewportPadding.updateSettings = function () {
        if (!AppUtils.DeviceType.MOBILE_DEVICE) { // desktop browsers
            ViewportPadding.settings.paddingAreaWidth = Math.max(App.settings.windowWidth - App.settings.width, App.minimumSidePaddingForDesktopBrowsers);
        } else {
            ViewportPadding.settings.paddingAreaWidth = App.settings.windowWidth - App.settings.width;
        }
        ViewportPadding.settings.paddingAreaHeight = App.settings.windowHeight - App.settings.height;
    };

    ViewportPadding.init = function () {
        ViewportPadding.updateSettings();
        ViewportPadding.defineClasses();
        ViewportPadding.instantiateObjects();
    };

    ViewportPadding.instantiateObjects = function () {
        ViewportPadding.leftPaddingAreaModel = new Backbone.Model({
            isLeft: true,
            remove: true
        });
        ViewportPadding.rightPaddingAreaModel = new Backbone.Model({
            isRight: true,
            remove: true
        });
        ViewportPadding.topPaddingAreaModel = new Backbone.Model({
            isTop: true,
            remove: true
        });
        ViewportPadding.bottomPaddingAreaModel = new Backbone.Model({
            isBottom: true,
            remove: true
        });

        ViewportPadding.leftPaddingAreaView = new ViewportPadding.PaddingAreaView({
            model: ViewportPadding.leftPaddingAreaModel
        });

        ViewportPadding.rightPaddingAreaView = new ViewportPadding.PaddingAreaView({
            model: ViewportPadding.rightPaddingAreaModel
        });

        ViewportPadding.topPaddingAreaView = new ViewportPadding.PaddingAreaView({
            model: ViewportPadding.topPaddingAreaModel
        });

        ViewportPadding.bottomPaddingAreaView = new ViewportPadding.PaddingAreaView({
            model: ViewportPadding.bottomPaddingAreaModel
        });
    };

    ViewportPadding.isPaddingAreaPossible = function () {
        var customValue = AppUtils.runCustomMenuMethod("isPaddingAreaPossible");
        if (customValue != null) {
            return customValue;
        }
        return ((ViewportPadding.settings.paddingAreaWidth > 2 * (39 * App.settings.menuScaleFactor + 3)) && !ProjectGlobals.IS_TIP_OF_THE_DAY && !ProjectGlobals.IS_SCENE_PREVIEW);
    };

    ViewportPadding.addPaddingAreas = function () {
        var leftPaddingAreaRenderResult,
            rightPaddingAreaRenderResult ,
            topPaddingAreaRenderResult ,
            bottomPaddingAreaRenderResult;

        if (ViewportPadding.leftPaddingAreaModel.get("remove")) {
            leftPaddingAreaRenderResult = ViewportPadding.leftPaddingAreaView.render();
            if (leftPaddingAreaRenderResult) {
                ViewportPadding.leftPaddingAreaModel.set("remove", false);
                ViewportPadding.leftPaddingAreaView.addListeners();
                $("body").append(leftPaddingAreaRenderResult.el);
            }
        }
        if (ViewportPadding.rightPaddingAreaModel.get("remove")) {
            rightPaddingAreaRenderResult = ViewportPadding.rightPaddingAreaView.render();
            if (rightPaddingAreaRenderResult) {
                ViewportPadding.rightPaddingAreaModel.set("remove", false);
                ViewportPadding.rightPaddingAreaView.addListeners();
                $("body").append(rightPaddingAreaRenderResult.el);
            }
        }
        if (ViewportPadding.topPaddingAreaModel.get("remove")) {
            topPaddingAreaRenderResult = ViewportPadding.topPaddingAreaView.render();
            if (topPaddingAreaRenderResult) {
                ViewportPadding.topPaddingAreaModel.set("remove", false);
                ViewportPadding.topPaddingAreaView.addListeners();
                $("body").append(topPaddingAreaRenderResult.el);
            }
        }
        if (ViewportPadding.bottomPaddingAreaModel.get("remove")) {
            bottomPaddingAreaRenderResult = ViewportPadding.bottomPaddingAreaView.render();
            if (bottomPaddingAreaRenderResult) {
                ViewportPadding.bottomPaddingAreaModel.set("remove", false);
                ViewportPadding.bottomPaddingAreaView.addListeners();
                $("body").append(bottomPaddingAreaRenderResult.el);
            }
        }
    };


    ViewportPadding.removeAllPaddings = function () {
        ViewportPadding.leftPaddingAreaModel.set("remove", true);
        ViewportPadding.rightPaddingAreaModel.set("remove", true);
        ViewportPadding.topPaddingAreaModel.set("remove", true);
        ViewportPadding.bottomPaddingAreaModel.set("remove", true);
    };
    ViewportPadding.hideAllPaddings = function () {
        ViewportPadding.leftPaddingAreaModel.set("hide", true);
        ViewportPadding.rightPaddingAreaModel.set("hide", true);
        ViewportPadding.topPaddingAreaModel.set("hide", true);
        ViewportPadding.bottomPaddingAreaModel.set("hide", true);
        if (null != ViewportPadding.rightPaddingAreaNavigationArrowModel) {
            ViewportPadding.rightPaddingAreaNavigationArrowModel.set("hide", true);
        }
        if (null != ViewportPadding.leftPaddingAreaNavigationArrowModel) {
            ViewportPadding.leftPaddingAreaNavigationArrowModel.set("hide", true);
        }
    };
    ViewportPadding.hideLeftPadding = function () {
        ViewportPadding.leftPaddingAreaModel.set("hide", true);
        if (null != ViewportPadding.leftPaddingAreaNavigationArrowModel) {
            ViewportPadding.leftPaddingAreaNavigationArrowModel.set("hide", true);
        }
    };
    ViewportPadding.hideRightPadding = function () {
        ViewportPadding.rightPaddingAreaModel.set("hide", true);
        if (null != ViewportPadding.rightPaddingAreaNavigationArrowModel) {
            ViewportPadding.rightPaddingAreaNavigationArrowModel.set("hide", true);
        }
    };
    ViewportPadding.showLeftPadding = function () {
        ViewportPadding.leftPaddingAreaModel.set("hide", false);
        if (null != ViewportPadding.leftPaddingAreaNavigationArrowModel) {
            ViewportPadding.leftPaddingAreaNavigationArrowModel.set("hide", false);
        }
    };
    ViewportPadding.showRightPadding = function () {
        ViewportPadding.rightPaddingAreaModel.set("hide", false);
        if (null != ViewportPadding.rightPaddingAreaNavigationArrowModel) {
            ViewportPadding.rightPaddingAreaNavigationArrowModel.set("hide", false);
        }
    };
    ViewportPadding.showAllPaddings = function () {
        ViewportPadding.leftPaddingAreaModel.set("hide", false);
        ViewportPadding.rightPaddingAreaModel.set("hide", false);
        ViewportPadding.topPaddingAreaModel.set("hide", false);
        ViewportPadding.bottomPaddingAreaModel.set("hide", false);
        if (null != ViewportPadding.rightPaddingAreaNavigationArrowModel) {
            ViewportPadding.rightPaddingAreaNavigationArrowModel.set("hide", false);
        }
        if (null != ViewportPadding.leftPaddingAreaNavigationArrowModel) {
            ViewportPadding.leftPaddingAreaNavigationArrowModel.set("hide", false);
        }
    };

    ViewportPadding.handleOrientationChanged = function () {
        var thumbnailWrapperModel = Chapter.getGlobalModels().thumbnailWrapperModel;
        Chapter.getScrollingElementsWrapper().getView().pageInTransition = false;
        App.updateSettings(App.lastKnownOrientation == SupportedOrientation.LANDSCAPE);
        console.log("MOB-1718: updated app height to " + App.settings.height);
        ViewportPadding.updateSettings();
        console.log("Now should change orientation to " + App.lastKnownOrientation);
        App.viewPortModel.set("orientation", App.lastKnownOrientation);
        ViewportPadding.leftPaddingAreaModel.set("orientation", App.lastKnownOrientation);
        ViewportPadding.rightPaddingAreaModel.set("orientation", App.lastKnownOrientation);
        ViewportPadding.topPaddingAreaModel.set("orientation", App.lastKnownOrientation);
        ViewportPadding.bottomPaddingAreaModel.set("orientation", App.lastKnownOrientation);
        Chapter.getScrollingElementsWrapper().getModel().set("orientation", App.lastKnownOrientation);
        Chapter.getScrollingElementsModel().set("orientation", App.lastKnownOrientation);
        thumbnailWrapperModel.set("activateThumbs", "-1"); // hack: first change value to a dummy that is ignored in handler
        thumbnailWrapperModel.set("activateThumbs", Chapter.getActivePage());
        if (App.lastKnownOrientation == SupportedOrientation.PORTRAIT) {
            Chapter.stopPlayingMultimediaForPage(Chapter.getActivePage() + 1);
        }
        if (ViewportPadding.isPaddingAreaPossible()) {
            ViewportPadding.rightPaddingAreaNavigationArrowModel.set("orientation", App.lastKnownOrientation);
        }
        ViewportPadding.updateTranslate();
        if (ViewportPadding.toggleOrientationButtonModel != undefined) {
            ViewportPadding.toggleOrientationButtonModel.set("orientation", App.lastKnownOrientation);
        }
        ZoomUtils.updateZoomOnOrientationChange();
    };

    ViewportPadding.updateTranslate = function () {
        var activePage = Chapter.getActivePage(),
            translateOffset;
        if (activePage % 2 == 1) {
            translateOffset = Chapter.computeTranslateOffset(activePage, activePage - 1);
            console.log("MOB-1718: computeTranslateOffset going to previous page with offset " + translateOffset);
            Chapter.setActivePage(activePage - 1); // here we want to activate page on the left side, see MOB-1674
        } else { // compute the translate offset based on differences of App.settings.pageWidth
            translateOffset = Chapter.getTranslateOffsetForSamePageOnOrientationChange(activePage);
            console.log("MOB-1718: getTranslateOffsetForSamePageOnOrientationChange " + translateOffset);
        }
        if (Chapter.getScrollingElementsModel().get("translate3DOffset") != translateOffset || AppUtils.useNativeScroll) {
            // make sure this gets executed only if a translate will happen
            Chapter.removePageTransitionDuration();
            Chapter.translateScenesTo(translateOffset);
            if (activePage % 2 == 1) { // change active page in this case
                Chapter.setActivePage(activePage - 1);
            } else {
                ZoomUtils.updateDummyDivForActivePageOnOrientationChange(activePage);
                Chapter.getScrollingElementsWrapper().getModel().set("scrollLeft", activePage * App.settings.pageWidth);
            }
            Chapter.restorePageTransitionDuration();
        } else {
            console.log("MOB-1718: updateTranslate skip translate for " + translateOffset);
        }
    };

    ViewportPadding.shouldSnapMenuToBottomOfScreen = function () {
        var ret = AppUtils.DeviceType.MOBILE_DEVICE; // first we need to be on a mobile device
        ret = ret && !ViewportPadding.isPaddingAreaPossible && ( App.settings.height == App.settings.windowHeight || App.settings.width == App.settings.windowWidth);
        return ret;
    };

})(window["ViewportPadding"]);