define(function () {
    return function () {
        var that = this,
            logTag = "MOB-1601: ",
            maxZoom = 3,
            minZoom = 1,
            zoomStep = 0.2;

        this.defineClasses = function () {

            that.ZoomControlView = BaseView.extend({
                tagName: "div",
                className: "zoomControl",

                removeView: function () {
                    if (this.model.get("removed")) {
                        this.remove();
                        this.unbind();
                    }
                },

                render: function () {
                    var zoomInButtonModel = new Backbone.Model({imageURL: "gfx/browserToolbar/zoom_in.png", zoomIn: true}),
                        zoomInButtonView = new that.ZoomButtonView({model: zoomInButtonModel}),
                        zoomOutButtonModel = new Backbone.Model({imageURL: "gfx/browserToolbar/zoom_out.png", zoomOut: true}),
                        zoomOutButtonView = new that.ZoomButtonView({model: zoomOutButtonModel}),
                        zoomSliderView;

                    that.zoomSliderModel = new Backbone.Model({
                        initValue: minZoom,
                        minValue: minZoom,
                        maxValue: maxZoom,
                        stepValue: zoomStep
                    });
                    zoomSliderView = new that.ZoomSliderView({model: that.zoomSliderModel});
                    this.$el.append(zoomInButtonView.render().el);
                    this.$el.append(zoomSliderView.render().el);
                    this.$el.append(zoomOutButtonView.render().el);
                    return this;
                }
            });

            that.ZoomSliderView = BaseView.extend({
                tag: "div",
                className: "zoomSliderContainer",
                template: templates.getTemplate("zoomSlider"),

                wheelOffset: 0,

                initialize: function () {
                    this.listenTo(this.model, "change:zoomValue", this.updateSliderValue);
                    this.listenTo(this.model, "change:incrementValue", this.incrementSliderValue);
                },

                events: {
                    "input input": "handleZoomAction",  // bind to input event for input element
                    "wheel input": "sliderWheelAction"  // bind to input event for input element
                },

                handleZoomAction: function (event) {
                    console.log(logTag + "Zoom action on slider " + event.target.value);
                    // TODO: make sure new zoom value is executed
                    Chapter.getScrollingElementsView().activePageModel.set("zoomValue", event.target.value);
                },

                sliderWheelAction: function (event) {
                    var zoomInstance;
                    event.preventDefault();
//                    event.stopPropagation();
                    this.wheelOffset += event.originalEvent.deltaY;
                    console.log("sliderWheelAction: " + event.originalEvent.deltaY + " total " + this.wheelOffset);
                    if (this.wheelOffset > 50) {
                        zoomInstance = Chapter.getScrollingElementsView().getZoomElementForPage(Chapter.getActivePage()).getZoomInstance();
                        zoomInstance.updateZoomWithIncrement(-zoomStep);
                        this.model.set("incrementValue", -zoomStep);
                        this.wheelOffset = 0;
                    } else if (this.wheelOffset < -50) {
                        zoomInstance = Chapter.getScrollingElementsView().getZoomElementForPage(Chapter.getActivePage()).getZoomInstance();
                        zoomInstance.updateZoomWithIncrement(zoomStep);
                        this.model.set("incrementValue", zoomStep);
                        this.wheelOffset = 0;
                    }
                },

                updateSliderValue: function () {
                    this.inputElement.val(this.model.get("zoomValue"));
                },

                incrementSliderValue: function () {
                    var currentValue,
                        incrementValue = this.model.get("incrementValue");
                    if (incrementValue != 0) {
                        currentValue = parseFloat(this.inputElement.val());
                        this.model.set("zoomValue", currentValue + this.model.get("incrementValue"));
                        this.model.set("incrementValue", 0);
                    }
                },

                removeView: function () {
                    if (this.model.get("removed")) {
                        this.remove();
                        this.unbind();
                    }
                },

                render: function () {
                    this.$el.html(this.template(this.model.attributes));
                    this.inputElement = this.$el.find("input");
                    if (AppUtils.DeviceType.Firefox) { // apply some different styles to slider
                        this.inputElement.addClass("FFSlider");
                    }
                    return this;
                }
            });

            that.ZoomButtonView = BaseView.extend({
                template: templates.getTemplate("simpleImage"),
                tagName: "div",
                className: "zoomButton",

                events: function () {
                    var _events = {};
                    _events[AppUtils.EventTypes.click] = "handleZoomAction";
                    return _events;
                },

                handleZoomAction: function () {
                    var zoomInstance = Chapter.getScrollingElementsView().getZoomElementForPage(Chapter.getActivePage()).getZoomInstance();
                    if (this.model.get("zoomIn")) {
                        console.log(logTag + "handleZoomAction zoom in");
                        zoomInstance.updateZoomWithIncrement(zoomStep);
                        that.zoomSliderModel.set("incrementValue", zoomStep);
                    } else if (this.model.get("zoomOut")) {
                        console.log(logTag + "handleZoomAction zoom out");
                        that.zoomSliderModel.set("incrementValue", -zoomStep);
                        zoomInstance.updateZoomWithIncrement(-zoomStep);
                    }
                },

                removeView: function () {
                    if (this.model.get("removed")) {
                        this.remove();
                        this.unbind();
                    }
                },

                render: function () {
                    this.$el.html(this.template(this.model.attributes));
                    return this;
                }
            });
        };

        this.instantiateObjects = function () {
            this.zoomControlView = new this.ZoomControlView();
        };

        return {
            init: function () {
                that.defineClasses();
                that.instantiateObjects();
            },
            getView: function () {
                return that.zoomControlView;
            },
            getZoomSliderModel: function () {
                return that.zoomSliderModel;
            }
        }
    }
});