var elementsTypeMapping = {
        emptyBox: 'EmptyBoxElement',
        externalURL: 'ExternalURLElement',
        downloadLink: 'DownloadLinkElement',
        image: 'ImageElement',
        imageWithGradient: 'ImageElement',
        textBox: 'TextBoxElement',
        sound: 'SoundElement',
        timedEvent: 'TimedEventElement',
        event: 'EventElement',
        animation: 'AnimationElement',
        video: 'VideoElement',
        drawing: 'DrawingElement',
        textInput: 'TextInputElement',
        cameraPicture: 'CameraElement',
        pdfPackage: 'PdfPackageElement',
        embeddedScene: 'EmbeddedSceneElement',
        moveZoomAnchor: 'AnchorElement',
        quizQuestion: 'QuizQuestionElement',
        quizTest: 'QuizTestElement',
        zipPackage: 'PackageElement'
    },
    domExcludedElements = {
        timedEvent: true,
        event: true
    },
    additionalLibrariesForObjectType = {
        sound: function () {
            return ['SoundPlayer', 'AudioTextSync'];
        },
        video: function () {
            return ['VideoPlayer'];
        },
        image: function () {
            return ['vendor/jquery.simulate', 'utils/MathHelpers', 'vendor/gifffer'];
        },
        quizQuestion: function () {
            return ['SceneObjectQuizQuestion'];
        }
    },
    templates = {
        getTemplate: function (type) {
            if (null == this[type]) {
                this[type] = _.template(htmlTemplates.getTemplate(type));
            }
            return this[type];
        }
    },
    htmlTemplates = {
        getTemplate: function (type) {
            return this[type]();
        },

        zoomSlider: function () {
            return [
                "<div class='zoomSliderBackground'></div>",
                "<input type='range' class='zoomSlider' " +
                "min=<%- minValue %> max=<%- maxValue %> value=<%- initValue %> step=<%- stepValue %>>"
            ].join("\n");
        },

        simpleImage: function () {
            return [
                "<img class='images'",
                "onload='AppUtils.scaleNonRetinaImageOutsideMenu(this);this.onload=null;'",
                "src='<%- imageURL %>'>"
            ].join("\n");
        },

        image: function () {
            return [
                "<img class='images'",
                "src='<%- imageURL %>'",
                "width='100%'",
                "height='100%'>"
            ].join("\n");
        },

        gifImage: function () {
            return [
                "<img class='images'",
                "data-gifffer='<%- imageURL %>'",
                //"src='<%- imageURL %>'",
                "width='100%'",
                "height='100%'>"
            ].join("\n");
        },

        downloadLink: function () {
            return [
                "<img class='images'",
                "src='<%- imageURL %>'",
                "width='100%'",
                "height='100%'>"
            ].join("\n");
        },

        imageWithGradient: function () {
            return [
                "<img class='images'",
                "src='<%- imageURL %>'",
                "width='100%'",
                "height='100%'>",
                "<div style='position:absolute;width:100%;height:100%;top:0;left:0;",
                "background: -moz-linear-gradient(top, rgba(<%- gradientColor %>, 1) 0%, rgba(<%- gradientColor %>, 0) <%- value1 %>%, rgba(<%- gradientColor %>, 0) <%- value2 %>%, rgba(<%- gradientColor %>, 1) 100%);", //Firefox 3.6+
                "background: -webkit-linear-gradient(top, rgba(<%- gradientColor %>, 1) 0%, rgba(<%- gradientColor %>, 0) <%- value1 %>%, rgba(<%- gradientColor %>, 0) <%- value2 %>%, rgba(<%- gradientColor %>, 1) 100%);", //Chrome 10+ and Safari 5.1+
                "background: -o-linear-gradient(top, rgba(<%- gradientColor %>, 1) 0%, rgba(<%- gradientColor %>, 0) <%- value1 %>%, rgba(<%- gradientColor %>, 0) <%- value2 %>%, rgba(<%- gradientColor %>, 1) 100%);", //Opera 11.10+
                "background: -ms-linear-gradient(top, rgba(<%- gradientColor %>, 1) 0%, rgba(<%- gradientColor %>, 0) <%- value1 %>%, rgba(<%- gradientColor %>, 0) <%- value2 %>%, rgba(<%- gradientColor %>, 1) 100%);", //IE 10+
                "'></div>"
            ].join("\n");
        },

        externalURL: function () {
            return [
                "<iframe",
                "scrolling='no'",
                "src='<%- externalURL %>'",
                "frameborder='0'",
                "width='100%'",
                "height='100%'>"
            ].join("\n");
        },

        pdfPackage: function () {
            return [
                "<div",
                "width='100%'",
                "height='100%'>",
                "</div>"
            ].join("\n");
        },

        retinaImage: function () {
            return [
                "<img class='images'",
                "src='<%- imageURL %>'",
                "onload='AppUtils.scaleRetinaImage(this);this.onload=null;'>"
            ].join("\n");
        },

        video: function () {
            return [
                "<video class='videos'",
                "poster='<%- posterURL %>'",
                "src='<%- videoURL %>'",
                "preload='metadata' controls",
                "height='100%' width='100%'>"
            ].join("\n");
        },
        externalVideo: function () {
            return [
                "<video class='videos'",
                "preload='metadata' controls",
                "width='100%' height='100%'>",
                "<p>Your user agent does not support the HTML5 Video element.</p>",
                "</video>"
            ].join("\n");
        },
        youtubeVideo: function () {
            return [
                "<div",
                "width='100%'",
                "height='100%'>",
                "</div>"
            ].join("\n");
        },
        thumbnail: function () {
            return [
                "<div class='thumbLoading'>",
                "<img src='gfx/thumbSpinner.gif' class='spinnerImage'>",
                "</div>",
                "<div class='thumbShadow'></div>"
            ].join("\n");
        },
        thumbnailImage: function () {
            return [
                "<img class='images'",
                "src='<%- thumbURL %>'",
                "width='100%'",
                "height='100%'>"
            ].join("\n");
        },
        activeTab: function () {
            return [
                "<div class='activeTabHighlightImageWrapper centerAlign'>",
                "<img class='menuIconHighlight'",
                "src='<%- highlightURL %>'>",
                "</div>",
                "<div class='activeTabWrapper centerAlign'>",
                "<img class='images'",
                "src='<%- imageURL %>'>",
                "</div>"
            ].join("\n");
        },
        goToPreviousChapter: function () {
            return [
                "<div class='goToNeighbourChapterButtonsWrapper'>",
                "<div class='menuNavigationExternalSpacer'>",
                "</div>",
                "<div class='goToNeighbourChapterArrowButtonWrapper centerAlign'>",
                "<img class='goToNeighbourChapterArrowButton'",
                "src='<%- leftImgSrc %>'>",
                "</div>",
                "<div class='menuNavigationMiddleSpacer'>",
                "</div>",
                "<div class='goToNeighbourChapterTextButtonWrapper centerAlign'>",
                "<img class='goToNeighbourChapterTextButton'",
                "src='<%- rightImgSrc %>'>",
                "</div>",
                "<div class='menuNavigationInternalSpacer'>",
                "</div>",
                "</div>"
            ].join("\n");
        },
        goToNextChapter: function () {
            return [
                "<div class='goToNeighbourChapterButtonsWrapper'>",
                "<div class='menuNavigationInternalSpacer'>",
                "</div>",
                "<div class='goToNeighbourChapterTextButtonWrapper centerAlign'>",
                "<img class='goToNeighbourChapterTextButton'",
                "src='<%- leftImgSrc %>'>",
                "</div>",
                "<div class='menuNavigationMiddleSpacer'>",
                "</div>",
                "<div class='goToNeighbourChapterArrowButtonWrapper centerAlign'>",
                "<img class='goToNeighbourChapterArrowButton'",
                "src='<%- rightImgSrc %>'>",
                "</div>",
                "<div class='menuNavigationExternalSpacer'>",
                "</div>",
                "</div>"
            ].join("\n");
        },
        bottomMenuSoundEffects: function () {
            return [
                "<label class='menuSettingsItemTextLabel'>",
                CAPTIONS.SOUND_EFFECTS_ON_OFF,
                "</label>",
                "<div class='soundEffect settingsRadioButtonGroup'>",
                "<div class='soundEffectEnabled roundedLeftButton activeSettingButton'>" + CAPTIONS.YES + "</div>",
                "<div class='soundEffectDisabled roundedRightButton inactiveSettingButton settingsRadioButtonBorder'>" + CAPTIONS.NO + "</div>",
                "</div>"
            ].join("\n");
        },
        bottomMenuGlobalSounds: function () {
            return [
                "<label class='menuSettingsItemTextLabel'>",
                CAPTIONS.GLOBAL_SOUNDS_ON_OFF,
                "</label>",
                "<div class='globalSounds settingsRadioButtonGroup'>",
                "<div class='globalSoundsEnabled roundedLeftButton activeSettingButton'>" + CAPTIONS.YES + "</div>",
                "<div class='globalSoundsDisabled roundedRightButton inactiveSettingButton settingsRadioButtonBorder'>" + CAPTIONS.NO + "</div>",
                "</div>"
            ].join("\n");
        },
        bottomMenuShowText: function () {
            return [
                "<label class='menuSettingsItemTextLabel'>",
                CAPTIONS.SHOW_TEXT,
                "</label>",
                "<div class='showText settingsRadioButtonGroup'>",
                "<div class='showTextEnabled roundedLeftButton activeSettingButton'>" + CAPTIONS.YES + "</div>",
                "<div class='showTextDisabled roundedRightButton inactiveSettingButton settingsRadioButtonBorder'>" + CAPTIONS.NO + "</div>",
                "</div>"
            ].join("\n");
        },
        bottomMenuReadText: function () {
            return [
                "<label class='menuSettingsItemTextLabel'>",
                CAPTIONS.SOUND_READING,
                "</label>",
                "<div class='showText settingsRadioButtonGroup'>",
                "<div class='readTextEnabled roundedLeftButton'>" + CAPTIONS.YES + "</div>",
                "<div class='readTextOwnVoice settingsRadioButtonBorder'>" + CAPTIONS.OWN_VOICE + "</div>",
                "<div class='readTextDisabled roundedRightButton settingsRadioButtonBorder'>" + CAPTIONS.NO + "</div>",
                "</div>"
            ].join("\n");
        },
        // TODO: we should have this delivered from model server side to be fully customizable

        soundsInfo: function () {
            return [
                "<div class='menuRecordingTextLabel'>",
                "<p>",
                "<span>" + CAPTIONS.RECORDING_INFO_PART_1 + "</span>",
                "<img src='gfx/little_play_icon_green.png'>",
                "<span>" + CAPTIONS.RECORDING_INFO_PLAY_PART_2 + "</span>",
                "</p>",

                "<p>",
                "<span>" + CAPTIONS.RECORDING_INFO_PART_1 + "</span>",
                "<img src='gfx/record_icon_little.png'>",
                "<span>" + CAPTIONS.RECORDING_INFO_RECORD_PART_2 + "</span>",
                "</p>",
                "</div>"
            ].join("\n");
        },
        recordSound: function () {
            return [
                "<img class='images recordingGlow' src='gfx/record_glow.png' style='display:none'",
                "onload='AppUtils.adjustRecIconGlow(this);this.onload=null;'>",
                "<img class='images'",
                "src='gfx/record_icon.png'",
                "onload='AppUtils.scaleNonRetinaImage(this);this.onload=null;'>"
            ].join("\n");
        },
        playSound: function () {
            return [
                "<img class='images playGlow' src='gfx/play_glow_green.png' style='display:none'",
                "onload='AppUtils.adjustRecIconGlow(this);this.onload=null;'>",
                "<img class='images'",
                "src='gfx/play_button_green.png'",
                "onload='AppUtils.scaleNonRetinaImage(this);this.onload=null;'>"
            ].join("\n");
        },
        bottomMenuTabContentArea: function () {
            return [
                "<div id = 'mainMenuBackground' ></div>",
                "<div id = 'mainMenuRepeatBackground'></div>"
            ].join("\n");
        },
        textBoxElement: function () {
            var defaultTextContent = "<%= textContent %>";
            if (AppUtils.DeviceType.ANDROID) {
                defaultTextContent = "<div class='androidScroll'><%= textContent %></div>"
            }
            return [
                "<div class='textBoxBackground'></div>",
                "<div class='textBoxContent' style='width:<%- originalWidth %>px;height:<%- originalHeight %>px'>" + defaultTextContent + "</div>"
            ].join("\n");
        },
        textBoxElementWithGradient: function () {
            var defaultTextContent = "<%= textContent %>";
            if (AppUtils.DeviceType.ANDROID) {
                defaultTextContent = "<div class='androidScroll'><%= textContent %></div>"
            }
            return [
                "<div class='textBoxBackground'></div>",
                "<div class='textBoxContent' style='width:<%- originalWidth %>px;height:<%- originalHeight %>px'>" + defaultTextContent + "</div>",
                "<div style='position:absolute;width:100%;height:100%;top:0;left:0;",
                "background: -moz-linear-gradient(top, rgba(<%- gradientColor %>, 1) 0%, rgba(<%- gradientColor %>, 0) <%- value1 %>%, rgba(<%- gradientColor %>, 0) <%- value2 %>%, rgba(<%- gradientColor %>, 1) 100%);", //Firefox 3.6+
                "background: -webkit-linear-gradient(top, rgba(<%- gradientColor %>, 1) 0%, rgba(<%- gradientColor %>, 0) <%- value1 %>%, rgba(<%- gradientColor %>, 0) <%- value2 %>%, rgba(<%- gradientColor %>, 1) 100%);", //Chrome 10+ and Safari 5.1+
                "background: -o-linear-gradient(top, rgba(<%- gradientColor %>, 1) 0%, rgba(<%- gradientColor %>, 0) <%- value1 %>%, rgba(<%- gradientColor %>, 0) <%- value2 %>%, rgba(<%- gradientColor %>, 1) 100%);", //Opera 11.10+
                "background: -ms-linear-gradient(top, rgba(<%- gradientColor %>, 1) 0%, rgba(<%- gradientColor %>, 0) <%- value1 %>%, rgba(<%- gradientColor %>, 0) <%- value2 %>%, rgba(<%- gradientColor %>, 1) 100%);", //IE 10+
                "'></div>"
            ].join("\n");
        },
        soundElement: function () {
            return [
                "<audio controls>",
                "<source src='<%- soundURL %>' type='audio/mpeg'>",
                CAPTIONS.AUDIO_NOT_SUPPORTED,
                "</audio>"].join("\n");
        },
        recordingLightBoxAbortRecording: function () {
            return [
                "<img class='images'",
                "src='gfx/closeLink.png'",
                "onload='AppUtils.scaleRecordingWizardImage(this);this.onload=null;'>",
                "<span>" + CAPTIONS.RECORDING_WIZARD_ABORT_MESSAGE + "</span>"
            ].join("\n");
        },
        recordingLightBoxMicrophoneImage: function () {
            return [
                "<img class='recordingLightBoxMicrophoneImage'",
                "src='gfx/icon_recording_red.png'",
                "onload='AppUtils.scaleRecordingWizardImage(this);this.onload=null;'>"
            ].join("\n");
        },
        recordingLightBoxWizardBottomMenuMessage: function () {
            return [
                "<span id='recordingLightBoxWizardBottomMenuMessageWrapper'>" + CAPTIONS.RECORDING_WIZARD_MESSAGE + "</span>"
            ].join("\n");
        },
        recordingLightBoxWizardMessage: function () {
            return [
                "<span id='recordingLightBoxWizardMessageWrapper' class='recordingCountdown'>" + CAPTIONS.RECORDING_WIZARD_MESSAGE + "</span>"
            ].join("\n");
        },
        animationContainer: function () {
            return [
                "<div class='animationContainer'></div>"
            ].join("\n");
        }, drawingElementCanvas: function () {
            return [
                "<canvas id='<%- canvasId %>'></canvas>"
            ].join("\n");
        }, textInputSingleLine: function () {
            return [
                "<input id='<%- textInputId %>' class='textInputClass'>"
            ].join("\n");
        }, textInputMultipleLines: function () {
            return [
                "<textarea id='<%- textInputId %>' class='textInputClass'></textarea>"
            ].join("\n");
        }, textInputSingleLineWithGradient: function () {
            return [
                "<input id='<%- textInputId %>' class='textInputClass'>",
                "<div style='position:absolute;width:100%;height:100%;top:0;left:0;",
                "background: -moz-linear-gradient(top, rgba(<%- gradientColor %>, 1) 0%, rgba(<%- gradientColor %>, 0) <%- value1 %>%, rgba(<%- gradientColor %>, 0) <%- value2 %>%, rgba(<%- gradientColor %>, 1) 100%);", //Firefox 3.6+
                "background: -webkit-linear-gradient(top, rgba(<%- gradientColor %>, 1) 0%, rgba(<%- gradientColor %>, 0) <%- value1 %>%, rgba(<%- gradientColor %>, 0) <%- value2 %>%, rgba(<%- gradientColor %>, 1) 100%);", //Chrome 10+ and Safari 5.1+
                "background: -o-linear-gradient(top, rgba(<%- gradientColor %>, 1) 0%, rgba(<%- gradientColor %>, 0) <%- value1 %>%, rgba(<%- gradientColor %>, 0) <%- value2 %>%, rgba(<%- gradientColor %>, 1) 100%);", //Opera 11.10+
                "background: -ms-linear-gradient(top, rgba(<%- gradientColor %>, 1) 0%, rgba(<%- gradientColor %>, 0) <%- value1 %>%, rgba(<%- gradientColor %>, 0) <%- value2 %>%, rgba(<%- gradientColor %>, 1) 100%);", //IE 10+
                "'></div>"
            ].join("\n");
        }, textInputMultipleLinesWithGradient: function () {
            return [
                "<textarea id='<%- textInputId %>' class='textInputClass'></textarea>",
                "<div style='position:absolute;width:100%;height:100%;top:0;left:0;",
                "background: -moz-linear-gradient(top, rgba(<%- gradientColor %>, 1) 0%, rgba(<%- gradientColor %>, 0) <%- value1 %>%, rgba(<%- gradientColor %>, 0) <%- value2 %>%, rgba(<%- gradientColor %>, 1) 100%);", //Firefox 3.6+
                "background: -webkit-linear-gradient(top, rgba(<%- gradientColor %>, 1) 0%, rgba(<%- gradientColor %>, 0) <%- value1 %>%, rgba(<%- gradientColor %>, 0) <%- value2 %>%, rgba(<%- gradientColor %>, 1) 100%);", //Chrome 10+ and Safari 5.1+
                "background: -o-linear-gradient(top, rgba(<%- gradientColor %>, 1) 0%, rgba(<%- gradientColor %>, 0) <%- value1 %>%, rgba(<%- gradientColor %>, 0) <%- value2 %>%, rgba(<%- gradientColor %>, 1) 100%);", //Opera 11.10+
                "background: -ms-linear-gradient(top, rgba(<%- gradientColor %>, 1) 0%, rgba(<%- gradientColor %>, 0) <%- value1 %>%, rgba(<%- gradientColor %>, 0) <%- value2 %>%, rgba(<%- gradientColor %>, 1) 100%);", //IE 10+
                "'></div>"
            ].join("\n");
        },


        bottomMenuToggleOC: function () {
            return [
                "<label class='menuSettingsItemTextLabel'>",
                "<%- caption %>",
                "</label>",
                "<div class='buttonsWrapper settingsRadioButtonGroup'>",
                "</div>"
            ].join("\n");
        },
        bottomMenuToggleOCButton: function () {
            return [
                "<div class='readTextEnabled roundedLeftButton'>",
                "<%- caption %>",
                "</div>",
                "</div>"
            ].join("\n");
        },
        toggleOCMenuButton: function () {
            return [
                "<img class='activeImage'",
                "src='<%- iconActive %>'>",
                "<img class='inactiveImage'",
                "src='<%- iconInactive %>'>"
            ].join("\n");
        },
        cameraButtonImage: function () {
            return [
                "<div class='cameraButtonWrapper'>",
                "<img src='<%- imageURL %>'>",
                "</div>"
            ].join("\n");
        },
        embeddedScene: function () {
            return [
                "<img class='embeddedSceneIconImage'",
                "src='<%- imageURL %>'>"
            ].join("\n");
        },
        animationDot: function () {
            return [
                "<img src='<%- imageURL %>'>"
            ].join("\n");
        },
        animationItem: function () {
            return [
                "<img class='animationItemImage'",
                "src='<%- imageURL %>'>",
                "<div class='caption'>",
                "</div>"
            ].join("\n");
        },
        quizTestTitle: function () {
            return '<p class="quizTestTitle"><%- title %></p>';
        },
        quizTestProgressBar: function () {
            return [
                "<div class='testProgressGreenBar'>",
                "</div>"
            ].join("\n");
        },
        quizTestProgressBarNKI: function () {
            return [
                "<div class='testProgressText'>",
                "</div>",
                "<div class='testProgressGreenBar'>",
                "</div>"
            ].join("\n");
        },
        quizTestProgressText: function () {
            return [
                "<div class='testProgressText'></div>"
            ].join("\n");
        },
        quizTestNavigationButton: function () {
            return [
                "<img class='navArrow'",
                "src='<%- imageURL %>'>"
            ].join("\n");
        },
        quizTestResult: function () {
            return [
                "<div id='quizTestResultWrapper'>",
                "<div class='testSummaryTitle'>" + CAPTIONS.QUIZ_SCORE_SUMMARY + "</div>",
                "<div class='testScoreResultsSection'>",
                "<div class='testBigPercentage'>0%</div>",
                "<div class='testSummaryNoPoints' style='opacity: 1;'>You got 0 points out of 2</div>",
                "</div>",
                "<div class='testRetryButton questionButtonGradient' style='left: 406.5px; opacity: 1;'>" + CAPTIONS.QUIZ_RETRY_TEST + "</div>",
                "</div>"
            ].join("\n");
        },
        inputPromoCodeLightBox: function () {
            return ["<div class='promoCodeInputView'>" +
            "<div class='promoCodeLightBoxMessage'>" + CAPTIONS.PROMO_CODE_LIGHTBOX_MESSAGE + "</div>" +
            "<div class='promoCodeLightBoxInputSection'>" +
            "<input class='promoCodeLightBoxInput'>" +
            "</div>" +
            "<div class='promoCodeLightBoxUnlockButtonSection'>" +
            "<div class='promoCodeLightBoxUnlockButton'>" + CAPTIONS.PROMO_CODE_LIGHTBOX_UNLOCK_BUTTON_TEXT + "</div>" +
            "</div>" +
            "</div>"
            ].join("\n");
        },
        promoCodeLightBoxWrongCode: function () {
            return ["<div class='promoCodeIconSection'>" +
            "<img class='promoCodeIcon' src='gfx/contentUnlock/error.png'>" +
            "</div>" +
            "<div class='promoCodeIconScreenMessage redText'>" + CAPTIONS.PROMO_CODE_WRONG_CODE_MESSAGE + "</div>"
            ].join("\n");
        },
        promoCodeLightBoxContentUnlocked: function () {
            return ["<div class='promoCodeIconSection'>" +
            "<img class='promoCodeIcon' src='gfx/contentUnlock/success.png'>" +
            "</div>" +
            "<div class='promoCodeIconScreenMessage greenText'>" + CAPTIONS.PROMO_CODE_CONTENT_UNLOCKED_MESSAGE + "</div>"
            ].join("\n");
        },
        compactPlayer: function () {
            return [
                "<div class='compactPlayerButtons playButton'></div>" +
                "<div class='compactPlayerButtons pauseButton'></div>" +
                "<div class='compactPlayer timeElapsed'>" +
                    //"<div class='pie spinner'></div>" +
                    //"<div class='pie filler'></div>" +
                    //"<div class='mask'></div>" +
                "</div>"
            ].join("\n");
        }

    };

function getAdditionalLibrariesForObjectType(objectType) {
    if (typeof additionalLibrariesForObjectType[objectType] == "function") {
        return additionalLibrariesForObjectType[objectType]();
    }
    return [];
}

function getIncludePathsProject(settings) {
    var ret = {};
    ret.Captions = getCaptionsPath(settings.language);
    if (AppUtils.DeviceType.iOS && AppUtils.nativeApp) {
        ret.SoundRecorder = "platforms/iOS/recording";
    } else if (AppUtils.DeviceType.ANDROID && AppUtils.nativeApp) {
        ret.SoundRecorder = "platforms/android/recording";
    }

    if (null != settings['customQuizStyle']) {
        ret.QuizQuestionElement = '../customQuizzes/' + settings['customQuizStyle'] + '/QuizQuestionElement';
        ret.QuizTestElement = '../customQuizzes/' + settings['customQuizStyle'] + '/QuizTestElement';
    }

    if (null != settings['isSCORMExport'] && settings['isSCORMExport'] == true) {
        ret.TextInputElement = '../scormElements/TextInputElement';
        ret.TextBoxElement = '../scormElements/TextBoxElement';
        AppUtils.isSCORM = true;
    }
    return ret;
}

function getCaptionsPath(language) {
    if (language == 'ENGLISH_GB') {
        return 'captions/english'
    } else if (language == 'NORSK_BOKMAL') {
        return 'captions/bokmal'
    } else if (language == 'NORSK_NYNORSK') {
        return 'captions/nynorsk'
    }
}

