if (null == window["DownloadUI"]) {
    window["DownloadUI"] = {};
    (function (DownloadUI) {
        var InternalDownloadUI = {};
        InternalDownloadUI.DownloadUIView = BaseView.extend({
            tagName: "div",
            id: 'DownloadUI',
            transparentOverlayModel: null,
            contentWrapperModel: null,
            initialize: function (attr) {
                this.listenTo(this.model, "change:removed", this.removeView);
                this.listenTo(this.model, "change:progress", this.updateProgress);
                this.listenTo(this.model, "change:downloadMessage", this.updateDownloadMessage);
            },
            events: function () {
                var _events = {};
                _events[AppUtils.EventTypes.mouseDown] = "mouseDownHandler";
                _events[AppUtils.EventTypes.mouseUp] = "mouseUpHandler";
                return _events;
            },
            mouseDownHandler: function (e) {
                e.preventDefault();
                e.stopPropagation();
                event.stopImmediatePropagation()
            },
            mouseUpHandler: function (e) {
                e.preventDefault();
                e.stopPropagation();
                event.stopImmediatePropagation()
            },
            updateProgress: function () {
                this.contentWrapperModel.set("progress", this.model.get('progress'));
            },
            updateDownloadMessage: function () {
                this.contentWrapperModel.set("downloadMessage", this.model.get('downloadMessage'));
            },
            removeView: function () {
                if (this.model.get("removed")) {
                    this.transparentOverlayModel.set("removed", true);
                    this.contentWrapperModel.set("removed", true);
                    this.remove();
                    this.unbind();
                }
            },

            render: function () {
                var thisView = this, transparentOverlayView , contentWrapperView;

                this.transparentOverlayModel = new Backbone.Model();
                this.contentWrapperModel = new Backbone.Model();

                transparentOverlayView = new InternalDownloadUI.TransparentOverlayView({
                    model: thisView.transparentOverlayModel
                });
                contentWrapperView = new InternalDownloadUI.DownloadUIContentWrapperView({
                    model: thisView.contentWrapperModel
                });
                this.$el.append(transparentOverlayView.render().el);
//                setTimeout(function(){
                thisView.$el.append(contentWrapperView.render().el);
//                },5);

                return this;
            }
        });
        InternalDownloadUI.TransparentOverlayView = BaseView.extend({
            tagName: "div",
            id: 'DownloadUITransparentOverlay',

            initialize: function (attr) {
                this.listenTo(this.model, "change:removed", this.removeView);
            },
            events: function () {
                var _events = {};
                _events[AppUtils.EventTypes.mouseDown] = "touchstart";
                _events[AppUtils.EventTypes.click] = "clickHandler";
                _events["touchcancel"] = "touchcancelHandler";
                _events["touchcend"] = "touchcendHandler";
                return _events;
            },
            touchcendHandler: function () {
                console.log("touchcendHandler on " + this.$el.attr("id"));
            },
            touchcancelHandler: function () {
                console.log("touchcancelHandler on " + this.$el.attr("id"));
            },
            removeView: function () {
                if (this.model.get("removed")) {
                    this.remove();
                    this.unbind();
                }
            },

            render: function () {
                return this;
            }
        });

        InternalDownloadUI.DownloadUIContentWrapperTempView = BaseView.extend({
            tagName: "div",
            id: 'DownloadUIContentTempWrapper',
            initialize: function (attr) {
                this.listenTo(this.model, "change:removed", this.removeView);
                this.listenTo(this.model, "change:downloadMessage", this.updateDownloadMessage);
            },
            removeView: function () {
                if (this.model.get("removed")) {
                    this.remove();
                    this.unbind();
                }
            },
            render: function () {
                this.$el.html('Downloading. Please wait.');
                return this;
            }
        });
        InternalDownloadUI.DownloadUIContentWrapperView = BaseView.extend({
            tagName: "div",
            id: 'DownloadUIContentWrapper',
            knobBackgroundModel: null,
            knobModel: null,
            knobDownloadMessageModel: null,
            knobPercentageDownloadProgressModel: null,
            knobCompleteMessageModel: null,
            knobOverlayModel: null,
            initialize: function (attr) {
                this.listenTo(this.model, "change:removed", this.removeView);
                this.listenTo(this.model, "change:progress", this.updateProgress);
                this.listenTo(this.model, "change:downloadMessage", this.updateDownloadMessage);
            },
            removeView: function () {
                if (this.model.get("removed")) {
                    this.knobBackgroundModel.set("removed", true);
                    this.knobModel.set("removed", true);
                    this.knobDownloadMessageModel.set("removed", true);
                    this.knobPercentageDownloadProgressModel.set("removed", true);
                    this.knobCompleteMessageModel.set("removed", true);
                    this.knobOverlayModel.set("removed", true);
                    this.remove();
                    this.unbind();
                }
            },
            updateProgress: function () {
                this.knobModel.set("progress", this.model.get('progress'));
                this.knobPercentageDownloadProgressModel.set("textContent", this.model.get('progress'));
            },
            updateDownloadMessage: function () {
                this.knobDownloadMessageModel.set("textContent", this.model.get('downloadMessage'));
            },
            render: function () {
                var thisView = this, knobView, knobBackgroundView, knobDownloadMessageView, knobPercentageDownloadProgressView, knobCompleteMessageView, knobOverlayView;
                this.knobModel = new Backbone.Model();
                this.knobBackgroundModel = new Backbone.Model();
                this.knobDownloadMessageModel = new Backbone.Model();
                this.knobPercentageDownloadProgressModel = new Backbone.Model();
                this.knobCompleteMessageModel = new Backbone.Model();
                this.knobOverlayModel = new Backbone.Model();
                knobView = new InternalDownloadUI.KnobView({
                    model: thisView.knobModel
                });
                knobBackgroundView = new InternalDownloadUI.KnobBackgroundView({
                    model: thisView.knobBackgroundModel
                });
                knobDownloadMessageView = new InternalDownloadUI.KnobDownloadMessageView({
                    model: thisView.knobDownloadMessageModel
                });
                knobPercentageDownloadProgressView = new InternalDownloadUI.KnobPercentageDownloadProgressView({
                    model: thisView.knobPercentageDownloadProgressModel
                });
                knobCompleteMessageView = new InternalDownloadUI.KnobCompleteMessageView({
                    model: thisView.knobCompleteMessageModel
                });
                knobOverlayView = new InternalDownloadUI.KnobOverlayView({
                    model: thisView.knobOverlayModel
                });
                this.$el.append(knobBackgroundView.render().el)
                    .append(knobView.render().el)
                    .append(knobDownloadMessageView.render().el)
                    .append(knobPercentageDownloadProgressView.render().el)
                    .append(knobCompleteMessageView.render().el)
                    .append(knobOverlayView.render().el);
                this.knobModel.set('lazyInitialization', true);

                return this;
            }
        });

        InternalDownloadUI.KnobBackgroundView = BaseView.extend({
            tagName: "div",
            id: 'DownloadUIKnobContainer',

            initialize: function (attr) {
                this.listenTo(this.model, "change:removed", this.removeView);
            },

            removeView: function () {
                if (this.model.get("removed")) {
                    this.remove();
                    this.unbind();
                }
            },
            render: function () {
                return this;
            }
        });
        InternalDownloadUI.KnobView = BaseView.extend({
            tagName: "div",
            id: 'KnobView',

            initialize: function (attr) {
                this.listenTo(this.model, "change:removed", this.removeView);
                this.listenTo(this.model, "change:progress", this.updateProgress);
                this.listenTo(this.model, "change:lazyInitialization", this.initializeKnob);
            },
            updateProgress: function () {
                this.$el.val(this.model.get('progress')).trigger('change');
            },
            initializeKnob: function () {
                this.$el.knob({
                    "readOnly": true,
                    "min": 0,
                    "max": 100,
                    "width": 220,
                    "height": 220,
                    "thickness": 0.25,
                    "top": 15,
                    "left": 65,
                    "position": "absolute"
                });
            },
            removeView: function () {
                if (this.model.get("removed")) {
                    this.remove();
                    this.unbind();
                }
            },
            render: function () {
                return this;
            }
        });
        InternalDownloadUI.KnobDownloadMessageView = BaseView.extend({
            tagName: "div",
            id: 'KnobDownloadMessage',
            initialize: function (attr) {
                this.listenTo(this.model, "change:removed", this.removeView);
                this.listenTo(this.model, "change:textContent", this.updateTextContent);
            },
            updateTextContent: function () {
                this.$el.html(this.model.get('textContent'));
            },
            removeView: function () {
                if (this.model.get("removed")) {
                    this.remove();
                    this.unbind();
                }
            },
            render: function () {
                return this;
            }
        });
        InternalDownloadUI.KnobCompleteMessageView = BaseView.extend({
            tagName: "div",
            id: 'KnobCompleteMessage',
            initialize: function (attr) {
                this.listenTo(this.model, "change:removed", this.removeView);
                this.listenTo(this.model, "change:textContent", this.updateTextContent);
            },
            updateTextContent: function () {
                this.$el.html(this.model.get('textContent'));
            },
            removeView: function () {
                if (this.model.get("removed")) {
                    this.remove();
                    this.unbind();
                }
            },
            render: function () {
                this.$el.html(CAPTIONS.COMPLETE);
                return this;
            }
        });
        InternalDownloadUI.KnobPercentageDownloadProgressView = BaseView.extend({
            tagName: "div",
            id: 'KnobDownloadProgress',
            initialize: function (attr) {
                this.listenTo(this.model, "change:removed", this.removeView);
                this.listenTo(this.model, "change:textContent", this.updateTextContent);
            },
            updateTextContent: function () {
                this.$el.html(this.model.get('textContent') + '%');
            },
            removeView: function () {
                if (this.model.get("removed")) {
                    this.remove();
                    this.unbind();
                }
            },
            render: function () {
                this.$el.html('0%');
                return this;
            }
        });
        InternalDownloadUI.KnobOverlayView = BaseView.extend({
            tagName: "div",
            id: 'KnobOverlay',
            initialize: function (attr) {
                this.listenTo(this.model, "change:removed", this.removeView);
            },
            removeView: function () {
                if (this.model.get("removed")) {
                    this.remove();
                    this.unbind();
                }
            },
            render: function () {
                return this;
            }
        });


        DownloadUI.displayDownloadBoxWithMessage = function (message) {
            InternalDownloadUI.downloadUIModel = new Backbone.Model();
            var downloadUIView = new InternalDownloadUI.DownloadUIView({
                model: InternalDownloadUI.downloadUIModel
            });
            App.viewPortView.$el.append(downloadUIView.render().el);
            InternalDownloadUI.downloadUIModel.set('downloadMessage', message);
        };
        DownloadUI.updateDownloadBoxProgress = function (currentStep) {
            console.log("Progress updated to: " + currentStep);
            InternalDownloadUI.downloadUIModel.set('progress', currentStep);
        };
        DownloadUI.removeDownloadBoxProgress = function () {
            console.log("Progress updated to: " + currentStep);
            InternalDownloadUI.downloadUIModel.set('progress', currentStep);
        };
        DownloadUI.removeDownloadBoxProgress = function () {
            // we want to see 100% progress before closing download UI
            InternalDownloadUI.downloadUIModel.set('removed', true);
            InternalDownloadUI.downloadUIModel.set('removed', false);
        };
        DownloadUI.showUnzipSpinner = function () {
            $("body").SpinnerProgressIndicator({displayMessage: CAPTIONS["WAIT_FOR_UNZIP"]});
        };

        DownloadUI.removeUnzipSpinner = function () {
            $("body").SpinnerProgressIndicator("onComplete");
        };

    })(DownloadUI);
}

