if (!window["ResourceManager"]) window["ResourceManager"] = {};

(function (ResourceManager) {

    ResourceManager.getPathForImage = function (objectId, extension) {
        var imageURL;
        if (AppUtils.nativeApp) {
            var currentLocation = window.location.href;
            if (currentLocation.indexOf("Library/Caches") != -1) {
                // TODO: this is not a very safe approach.
                // Here we don't cover the case when HTML files
                // are all bundled and resources will be downloaded !!!
                imageURL = "files/" + objectId + "." + extension;
            } else if (AppUtils.getLocalStorageBooleanItem("bookChapter", false)) {
                var bookId = Chapter.getSettings()["bookId"];
                if (localStorage.getItem("allResourcesAreDownloaded." + bookId) == "true") {
                    // book with all resources embedded
                    imageURL = "files/" + objectId + "." + extension;
                } else {
                    imageURL = AppUtils.storageFolder + "files/" + objectId + "." + extension;
                }
            } else {
                imageURL = "files/" + objectId + "." + extension;
            }
        } else if (AppUtils.offlineExport) { // an offline export with local resources
            imageURL = "files/" + objectId + "." + extension;
        } else {
            imageURL = "/file?objectId=" + objectId + "&extension=" + extension;
        }
        return imageURL;
    };

    ResourceManager.getPathForVideo = function (objectId, extension) {
        return ResourceManager.getPathForImage(objectId, extension);
    };

    ResourceManager.getPathForSound = function (objectId, extension) {
        var soundURL, prefix = '', currentLocation;
        if (AppUtils.nativeApp) {
            currentLocation = window.location.href;
            if (AppUtils.DeviceType.iOS) {
                if (AppUtils.getLocalStorageBooleanItem("bookChapter", false)) {
                    var bookId = Chapter.getSettings()["bookId"];
                    if (localStorage.getItem("allResourcesAreDownloaded." + bookId) != "true") {
                        // keep this hardcoded like it is, see INT-287,
                        // we handle the replace in iOS native side CDVSoundCustom
                        prefix = "../../Library/Caches/";
                    } else {
                        prefix = "chapters/"
                    }
                } else if (currentLocation.indexOf("Library/Caches") != -1) { // downloaded resources
                    var regex = "versionedProjects";
                    prefix = currentLocation.substring(currentLocation.indexOf(regex), currentLocation.lastIndexOf("/") + 1);
                    prefix = ProjectGlobals.DOWNLOAD_PATH + prefix;
                    localStorage.setItem('absolutePath', prefix + "files/" + objectId + "." + extension);

                    // keep this hardcoded like it is, see INT-287,
                    // we handle the replace in iOS native side CDVSoundCustom
                    prefix = "../../" + currentLocation.substring(currentLocation.indexOf("Library/Caches"), currentLocation.lastIndexOf("/") + 1);
                }
                soundURL = prefix + "files/" + objectId + "." + extension;
            } else if (AppUtils.DeviceType.ANDROID) {
                if (currentLocation.indexOf("Android/data") != -1) { // downloaded resources
                    prefix = currentLocation.substring("file://".length, currentLocation.lastIndexOf("/") + 1);
                } else { // bundled resources
                    prefix = "/android_asset/www/";
                }
                soundURL = prefix + "files/" + objectId + "." + extension;
            }
        } else if (AppUtils.offlineExport) { // an offline export with local resources
            soundURL = "files/" + objectId + "." + extension;
        } else {
            soundURL = "/file?objectId=" + objectId + "&extension=" + extension;
        }
        return soundURL;
    };

    ResourceManager.getPathForPDFPackage = function (packageId, startFileName) {
        var pdfPackageURL;
        if (AppUtils.nativeApp) {
            var currentLocation = window.location.href;
            if (currentLocation.indexOf("Library/Caches") != -1) {
                pdfPackageURL = "files/" + packageId + "/" + startFileName;
            } else if (AppUtils.getLocalStorageBooleanItem("bookChapter", false)) {
                var bookId = Chapter.getSettings()["bookId"];
                if (localStorage.getItem("allResourcesAreDownloaded." + bookId) == "true") {
                    // book with all resources embedded
                    pdfPackageURL = "files/" + packageId + "/" + startFileName;
                } else {
                    pdfPackageURL = AppUtils.storageFolder + "files/" + packageId + "/" + startFileName;
                }
            } else {
                pdfPackageURL = "files/" + packageId + "/" + startFileName;
            }
        } else if (AppUtils.offlineExport) { // an offline export with local resources
            pdfPackageURL = "files/" + packageId + "/" + startFileName;
        } else {
            pdfPackageURL = "/zip/" + packageId + "/" + startFileName + "?alterHTMLPathsFromPDF=1";
        }
        return pdfPackageURL;
    };

    ResourceManager.getPathForIcon = function (defaultIconUrl) {
        var imageURL,
            objectInfo = ResourceManager.getObjectInfo(defaultIconUrl),
            objectId, extension;
        if (null == objectInfo) {
            return defaultIconUrl;
        } else if (AppUtils.nativeApp) {
            objectId = objectInfo.objectId;
            extension = objectInfo.extension;
            var currentLocation = window.location.href;
            if (currentLocation.indexOf("Library/Caches") != -1) {
                // TODO: this is not a very safe approach.
                // Here we don't cover the case when HTML files
                // are all bundled and resources will be downloaded !!!
                imageURL = "files/" + objectId + "." + extension;
            } else if (AppUtils.getLocalStorageBooleanItem("bookChapter", false)) {
                var bookId = Chapter.getSettings()["bookId"];
                if (localStorage.getItem("allResourcesAreDownloaded." + bookId) == "true") {
                    // book with all resources embedded
                    imageURL = "files/" + objectId + "." + extension;
                } else {
                    imageURL = AppUtils.storageFolder + "files/" + objectId + "." + extension;
                }
            } else {
                imageURL = "files/" + objectId + "." + extension;
            }
        } else if (AppUtils.offlineExport) { // an offline export with local resources
            imageURL = "files/" + objectId + "." + extension;
        } else {
            imageURL = "/file?objectId=" + objectId + "&extension=" + extension;
        }
        return imageURL;
    };
    ResourceManager.getObjectInfo = function (url) {
        var idRegex = 'objectId=', extensionRegex = 'extension=', ret = null;
        if (url.indexOf(idRegex) > -1) {
            if (url.indexOf(extensionRegex) > -1) {
                ret = {};
                var tmp = url.substring(url.indexOf(idRegex) + idRegex.length);
                ret.objectId = parseInt(tmp);
                ret.extension = url.substring(url.indexOf(extensionRegex) + extensionRegex.length);
            }
        }
        return ret;
    }


})(window["ResourceManager"]);

// http://paulirish.com/2011/requestanimationframe-for-smart-animating/
// http://my.opera.com/emoller/blog/2011/12/20/requestanimationframe-for-smart-er-animating

// requestAnimationFrame polyfill by Erik Möller
// fixes from Paul Irish and Tino Zijdel

(function () {
    var lastTime = 0;
    var vendors = ['ms', 'moz', 'webkit', 'o'];
    for (var x = 0; x < vendors.length && !window.requestAnimationFrame; ++x) {
        window.requestAnimationFrame = window[vendors[x] + 'RequestAnimationFrame'];
        window.cancelAnimationFrame = window[vendors[x] + 'CancelAnimationFrame']
            || window[vendors[x] + 'CancelRequestAnimationFrame'];
    }

    if (!window.requestAnimationFrame)
        window.requestAnimationFrame = function (callback, element) {
            var currTime = new Date().getTime();
            var timeToCall = Math.max(0, 16 - (currTime - lastTime));
            var id = window.setTimeout(function () {
                    callback(currTime + timeToCall);
                },
                timeToCall);
            lastTime = currTime + timeToCall;
            return id;
        };

    if (!window.cancelAnimationFrame)
        window.cancelAnimationFrame = function (id) {
            clearTimeout(id);
        };
}());