var SoundState = {
    PLAYING: "PLAYING",
    SUSPENDED: "SUSPENDED",
    ENDED: "ENDED",
    PAUSED: "PAUSED",
    LOADED: "LOADED",
    NOT_LOADED: "NOT_LOADED"
};

var SoundPlayer = {
    readyToPlayAudio: AppUtils.deviceReady,
    queueForAutoStartSounds: [],
    playingSounds: [],
    createdSounds: [],

    playSoundWithURL: function (soundInfo) {

        if (this.readyToPlayAudio) {
            if (AppUtils.nativeApp) {
                this.playSoundWithMediaAPI(soundInfo);
            } else {
                this.playSoundWithSM2(soundInfo);
            }
        } else {
            console.log("Not ready to play sound with id " + soundInfo.soundId + " and url " + soundInfo.soundURL);
            this.queueForAutoStartSounds.push(soundInfo);
        }
    },

    playSoundWithSM2: function (soundInfo) {
        var soundObject = soundManager.getSoundById(soundInfo.soundId),
            soundObjectModel;
        if (soundObject == null) {
            soundObject = this.getNewSoundObject(soundInfo);
            soundObjectModel = {
                soundInstance: soundObject,
                soundInfo: soundInfo
            };
            this.getPlayingSoundsForPage(soundInfo.pageNumber)[soundInfo.soundId] = soundObjectModel;
        }
        if (soundInfo.autoPlay) {
            console.log("Will play sound with id " + soundInfo.soundId + " and url " + soundInfo.soundURL);
            // using SM2 this will start playing sound from the beginning
            soundObject.play({
                volume: 50,
                onfinish: function () {
                    // using SM2 this is only fired is sound has played till the end, and was not stopped
                    SoundPlayer.runSoundFinishedHandler(soundInfo);
                    SoundPlayer.runSoundEndPlayHandler(soundInfo);
                }
            })
        }
    },

    playSoundWithMediaAPI: function (soundInfo) {
        var soundObject;
        if (typeof this.createdSounds[soundInfo.soundId] == "undefined") {
            console.log("playSoundWithMediaAPI: Will create a new MediaPlaySoundObject for id " + soundInfo.soundId);
            this.createdSounds[soundInfo.soundId] = this.getNewMediaPlaySoundObject(soundInfo);
            var newSoundObject = {
                soundInstance: SoundPlayer.createdSounds[soundInfo.soundId],
                soundInfo: soundInfo
            };
            this.getPlayingSoundsForPage(soundInfo.pageNumber)[soundInfo.soundId] = newSoundObject;
        }
        if (soundInfo.autoPlay) {
            soundObject = this.createdSounds[soundInfo.soundId];
            if (soundInfo.stopBeforePlay) { // on Android sound won't play from the beginning if already playing
                soundObject.getCurrentPosition(function (position) {
                        console.log("playSoundWithMediaAPI: currentPosition is " + position);
                        if (position > 0) {
                            console.log("Now seekTo 0");
                            soundObject.seekTo(0);
                        }
                        console.log("CDB-179: playSoundWithMediaAPI: Will play sound with id " + soundInfo.soundId + " for page " + soundInfo.pageNumber);
                        if (Chapter.getActivePage() == soundInfo.pageNumber) {
                            soundObject.play();
                        } else {
                            console.log("CDB-179: skip playing sound because page has changed");
                        }
                    },
                    function (e) {
                        console.log("Error getting currentPosition: " + e);
                        SoundPlayer.runSoundEndPlayHandler(soundInfo);
                    });
            } else {
                soundObject.play();
            }
        }
    },

    getPlayingSoundsForPage: function (pageNumber) {
        if (typeof this.playingSounds[pageNumber] == "undefined") {
            this.playingSounds[pageNumber] = {};
        }
        return this.playingSounds[pageNumber];
    },

    playSoundsFromQueue: function () {
        for (var sIdx in this.queueForAutoStartSounds) {
            console.log("Will play sound from queue " + this.queueForAutoStartSounds[sIdx].soundURL);
            this.playSoundWithURL(this.queueForAutoStartSounds[sIdx]);
        }
    },

    endAllPlayingSounds: function () {
        var i, j, soundsForPage, soundKeys;
        console.log("endAllPlayingSounds");
        for (j = 0; j < this.playingSounds.length; j++) {
            soundsForPage = this.getPlayingSoundsForPage(j);
            soundKeys = Object.keys(soundsForPage);
            for (i = 0; i < soundKeys.length; i++) {
                this.endSound(soundsForPage[soundKeys[i]].soundInfo, true);
            }
        }
        this.playingSounds = [];
        // now also stop recorded sound play if the case
        this.endAllRecordedSounds(null);
        console.log("endAllPlayingSounds end");
    },


    endAllPlayingSoundsForPage: function (pageNumber, destroy) {
        var i, soundsForPage, soundKeys;
        console.log("endAllPlayingSoundsForPage " + pageNumber + " start and destroy " + destroy);
        soundsForPage = this.getPlayingSoundsForPage(pageNumber);
        soundKeys = Object.keys(soundsForPage);
        for (i = 0; i < soundKeys.length; i++) {
            this.endSound(soundsForPage[soundKeys[i]].soundInfo, destroy);
        }
        if (destroy) {

//            for (i = 0; i < soundsForPage.length; i++) {
//                PersistentSceneObjectsUtils.removePersistentObject(soundsForPage[i].soundInfo.soundId);
//            }
//            this.playingSounds[pageNumber] = []; // this can be empty now;
        }
        this.endAllRecordedSounds(pageNumber);
        console.log("endAllPlayingSoundsForPage " + pageNumber + " end");
    },

    endAllRecordedSounds: function (pageNumber) {
        if (typeof SoundRecorder != "undefined") {
            SoundRecorder.stopPlayingRecordedSound(pageNumber);
            SoundRecorder.stopPlayRecordedWizardSoundForPage(pageNumber);
        }
    },

    endSound: function (playingSound, destroy) {
        var soundObject;
        console.log("endSound called");
        if (AppUtils.nativeApp) {
            soundObject = this.createdSounds[playingSound.soundId];
            if (typeof soundObject != "undefined") {
                console.log("endSound with ID " + playingSound.soundId);
                if (!(playingSound.availableInAllOrientations && App.changedOrientation)) {
                    soundObject.stop(); // will also run runSoundEndPlayHandler
                    if (destroy) {
                        //remove from playing sounds specific sound
                        delete this.createdSounds[playingSound.soundId];
                        delete this.playingSounds[playingSound.pageNumber][playingSound.soundId];
                        PersistentSceneObjectsUtils.removePersistentObject(playingSound.soundId);
                        soundObject.release();
                        console.log("CDB-179: Stopped and released resources for sound with ID " + playingSound.soundId + " for page " + playingSound.pageNumber);
                    } else {
                        console.log("CDB-179: Stopped sound with ID " + playingSound.soundId + " for page " + playingSound.pageNumber);
                    }
                }
            }
        } else if (typeof soundManager != "undefined") {
            console.log("endSound with ID " + playingSound.soundId);

            if (!(playingSound.availableInAllOrientations && App.changedOrientation)) {
                soundManager.stop(playingSound.soundId);
                this.runSoundEndPlayHandler(playingSound);
                if (destroy) {
                    //remove from playing sounds specific sound
                    delete this.playingSounds[playingSound.pageNumber][playingSound.soundId];
                    PersistentSceneObjectsUtils.removePersistentObject(playingSound.soundId);
                    soundManager.destroySound(playingSound.soundId);
                    console.log("CDB-179: Resources released for sound with ID " + playingSound.soundId + " for page " + playingSound.pageNumber);
                }
            }
        }
    },

    pauseSound: function (playingSound) {
        if (typeof soundManager != "undefined") {
            console.log(LOG_TAGS.MOB_2206, "pauseSound with ID " + playingSound.soundId);
            soundManager.pause(playingSound.soundId);
        }
    },

    resumeSound: function (playingSound) {
        if (typeof soundManager != "undefined") {
            console.log(LOG_TAGS.MOB_2206, "resumeSound with ID " + playingSound.soundId);
            soundManager.resume(playingSound.soundId);
        }
    },

    setPosition: function (playingSound, position) {
        if (typeof soundManager != "undefined") {
            console.log(LOG_TAGS.MOB_2206, "setPosition for sound with ID " + playingSound.soundId);
            soundManager.setPosition(playingSound.soundId, position);
        }
    },

    getNewSoundObject: function (soundInfo) {
        console.log("CDB-179: Create a new sound with id " + soundInfo.soundId + " for page " + soundInfo.pageNumber);
        return soundManager.createSound({
            id: soundInfo.soundId,
            url: soundInfo.soundURL,
            stream: true,
            autoLoad: true,
            autoPlay: soundInfo.autoPlay,
            volume: 50,
            loops: 1,
            multiShot: false,
            multiShotEvents: false,
            pan: 0,
            usePolicyFile: false,
            onsuspend: function () {
                console.log(LOG_TAGS.MOB_2206, "Got suspended " + this.id + " at position " + this.position);
                //if (this.position) {
                //    SoundPlayer.runSoundOnSuspendHandler(soundInfo);
                //}
            },
            onpause: function () {
                SoundPlayer.runSoundOnPauseHandler(soundInfo);
            },
            onload: function (success) {
                SoundPlayer.runSoundOnLoadHandler(soundInfo);
            },
            onplay: function () {
                console.log(LOG_TAGS.MOB_2206, "Duration on start is " + this.duration);
                SoundPlayer.runSoundPlayStartHandler(soundInfo, this.duration);
            },
            whileplaying: function () {
                SoundPlayer.runSoundWhilePlayingHandler(soundInfo, this);
                //console.log(LOG_TAGS.MOB_2206, "Duration while playing is "+this.duration);
            },
            onresume: function () {
                SoundPlayer.runSoundOnResumeHandler(soundInfo);
            },
            onfinish: function () {
                SoundPlayer.runSoundEndPlayHandler(soundInfo);
            },
            whileloading: function () {
                console.log(this.id + ' is loading');
            }
        });
    },

    getNewMediaPlaySoundObject: function (soundInfo) {
        console.log("CDB-179: Create a new sound with id " + soundInfo.soundId + " for page " + soundInfo.pageNumber);
        var ret = new Media(soundInfo.soundURL,
            function () {
                if (this.status == Media.MEDIA_FINISHED) { // custom status for when sound is finished because has reached end
                    SoundPlayer.runSoundFinishedHandler(soundInfo);
                }
                SoundPlayer.runSoundEndPlayHandler(soundInfo);
                console.log("onPlay Finish " + soundInfo.soundId + " with status " + this.status);
            },
            function () {
                SoundPlayer.runSoundEndPlayHandler(soundInfo);
                console.log("onPlay Error " + soundInfo.soundId);
            },
            function (status) {
                this.status = status; // save the status to be used in success callback (10 lines above)
                console.log("onPlay StatusChange " + Media.MEDIA_MSG[status] + " for " + soundInfo.soundId);
                if (status == Media.MEDIA_RUNNING || status == Media.MEDIA_STARTING) {
                    SoundPlayer.runSoundPlayStartHandler(soundInfo);
                }
            });
        SoundPlayer.runSoundOnLoadHandler(soundInfo);
        return ret;
    },

    runSoundFinishedHandler: function (soundInfo) {
        console.log("runSoundFinishedHandler: for sound with id " + soundInfo.soundId + " and url " + soundInfo.soundURL);
        if (soundInfo.onPlayFinish != null && typeof soundInfo.onPlayFinish == "function") {
            soundInfo.onPlayFinish();
        }
    },

    runSoundEndPlayHandler: function (soundInfo) {
        console.log("runSoundEndPlayHandler: for sound with id " + soundInfo.soundId + " and url " + soundInfo.soundURL);
        if (soundInfo.onPlayEnd != null && typeof soundInfo.onPlayEnd == "function") {
            soundInfo.onPlayEnd();
        }
    },

    runSoundOnLoadHandler: function (soundInfo) {
        console.log("runSoundOnLoadHandler: for sound with id " + soundInfo.soundId + " and url " + soundInfo.soundURL);
        if (soundInfo.onLoad != null && typeof soundInfo.onLoad == "function") {
            soundInfo.onLoad();
        }
    },

    runSoundPlayStartHandler: function (soundInfo, duration) {
        console.log("runSoundPlayStartHandler: for sound with id " + soundInfo.soundId + " and url " + soundInfo.soundURL);
        if (soundInfo.onPlayStart != null && typeof soundInfo.onPlayStart == "function") {
            soundInfo.onPlayStart(duration);
        }
    },

    runSoundOnPauseHandler: function (soundInfo) {
        console.log("runSoundOnPauseHandler: for sound with id " + soundInfo.soundId + " and url " + soundInfo.soundURL);
        if (soundInfo.onPause != null && typeof soundInfo.onPause == "function") {
            soundInfo.onPause();
        }
    },

    runSoundOnSuspendHandler: function (soundInfo) {
        console.log("runSoundOnSuspendHandler: for sound with id " + soundInfo.soundId + " and url " + soundInfo.soundURL);
        if (soundInfo.onSuspend != null && typeof soundInfo.onSuspend == "function") {
            soundInfo.onSuspend();
        }
    },

    runSoundOnResumeHandler: function (soundInfo) {
        console.log("runSoundOnResumeHandler: for sound with id " + soundInfo.soundId + " and url " + soundInfo.soundURL);
        if (soundInfo.onResume != null && typeof soundInfo.onResume == "function") {
            soundInfo.onResume();
        }
    },

    runSoundWhilePlayingHandler: function (soundInfo, soundObject) {
        if (soundInfo.whilePlaying != null && typeof soundInfo.whilePlaying == "function") {
            soundInfo.whilePlaying(soundObject.position, soundObject.duration);
        }
    }
};

console.log("typeof soundManager = " + (typeof soundManager));
// TODO: check why this is not loading correct
if (!AppUtils.nativeApp) {
    soundManager.setup({
        // where to find flash audio SWFs, as needed
        url: 'js/vendor/soundmanager/swf',
        allowScriptAccess: 'always',
        debugMode: false,
        debugFlash: false,
        flashVersion: 9,
        flashPollingInterval: null,
        html5PollingInterval: null,
//        html5Test: /^(probably|maybe)$/i,
        html5Test: /^(probably)$/i,
        flashLoadTimeout: 10000,
        noSWFCache: false,
        preferFlash: false,
        useHTML5Audio: true,
        onready: function () {
            // SM2 is ready to play audio!
            console.log("Soundmanager : ready to play audio");
            SoundPlayer.readyToPlayAudio = true;
            if (SoundPlayer.queueForAutoStartSounds.length != 0) {
                SoundPlayer.playSoundsFromQueue();
            }
        }
    });
}
