var TransitionPlayer = function (model, $el, transitionType) {
    var TAG = "GYL-414",
        savedTransform,
        initialStartTime,
        totalElapsedTime = 0,
        remainingTime,
        currentTransformValueOnPause,
        privateMethods = {
            getUpdatedTransformMatrix: function (transformMatrix) {
                var transformMatrixValues = transformMatrix.match(/-?[0-9\.]+/g);
                if (transitionType == TransitionPlayerType.MOVE) {
                    // TODO: the following 4 values are set to default values due to a bug on iOS 8 !!!
                    transformMatrixValues[0] = 1;
                    transformMatrixValues[1] = 0;
                    transformMatrixValues[2] = 0;
                    transformMatrixValues[3] = 1;
                } else if (transitionType == TransitionPlayerType.ZOOM) {
                    // TODO: the following 4 values are set to default values due to a bug on iOS 8 !!!
                    transformMatrixValues[1] = 0;
                    transformMatrixValues[2] = 0;
                    transformMatrixValues[4] = 0;
                    transformMatrixValues[5] = 0;
                    transformMatrixValues[0] = transformMatrixValues[3]; // yet another iOS 8 bug here !!!
                }
                return "matrix(" + parseFloat(transformMatrixValues[0]).toFixed(3) +
                    ", " + parseFloat(transformMatrixValues[1]).toFixed(3) +
                    ", " + parseFloat(transformMatrixValues[2]).toFixed(3) +
                    ", " + parseFloat(transformMatrixValues[3]).toFixed(3) +
                    ", " + parseFloat(transformMatrixValues[4]).toFixed(3) +
                    ", " + parseFloat(transformMatrixValues[5]).toFixed(3) + ")";
            },
            getTransformMatrixForCSSTransform: function (cssTransform) {
                var ret = cssTransform,
                    translateValues = cssTransform.match(/-?[0-9\.]+/g),
                    transformMatrixValues = currentTransformValueOnPause.match(/-?[0-9\.]+/g);
                if (cssTransform.indexOf("translate") > -1) {
                    transformMatrixValues[4] = translateValues[0];
                    transformMatrixValues[5] = translateValues[1];
                    ret = "matrix(" + transformMatrixValues[0] +
                    ", " + transformMatrixValues[1] +
                    ", " + transformMatrixValues[2] +
                    ", " + transformMatrixValues[3] +
                    ", " + transformMatrixValues[4] +
                    ", " + transformMatrixValues[5] + ")";
                } else if (cssTransform.indexOf("scale") > -1) {
                    transformMatrixValues[0] = translateValues[0];
                    transformMatrixValues[3] = translateValues[0];
                    ret = "matrix(" + parseFloat(transformMatrixValues[0]).toFixed(3) +
                    ", " + parseFloat(transformMatrixValues[1]).toFixed(3) +
                    ", " + parseFloat(transformMatrixValues[2]).toFixed(3) +
                    ", " + parseFloat(transformMatrixValues[3]).toFixed(3) +
                    ", " + parseFloat(transformMatrixValues[4]).toFixed(3) +
                    ", " + parseFloat(transformMatrixValues[5]).toFixed(3) + ")";
                }
                console.log(TAG, "getTransformMatrixForCSSTransform ret value " + ret);
                return ret;
            }
        };

    return {
        handleCSSTransformPause: function () {
            var startTime = model.get("transformStartTime"),
                elapsedTimeInLastRound;

            currentTransformValueOnPause = window.getComputedStyle($el[0]).getPropertyValue(CSSHandler.getCurrentBrowserPrefix().transform);

            if (currentTransformValueOnPause != "none") {
                currentTransformValueOnPause = privateMethods.getUpdatedTransformMatrix(currentTransformValueOnPause);
                console.log(TAG, "Transform matrix on pause " + currentTransformValueOnPause);
                if (!initialStartTime) {
                    initialStartTime = startTime;
                    remainingTime = parseInt($el.css("transition-duration")) * 1000;
                    console.log(TAG, "Initial transition duration : " + remainingTime);
                }

                elapsedTimeInLastRound = new Date().getTime() - startTime;
                totalElapsedTime += elapsedTimeInLastRound;
                remainingTime -= elapsedTimeInLastRound;

                console.log(TAG, "Elapsed time " + totalElapsedTime);
                console.log(TAG, "Remaining time " + remainingTime);

                savedTransform = {};
                savedTransform["transform"] = $el[0].style['webkitTransform'];
                savedTransform["transition-property"] = $el.css("transition-property");
                savedTransform["transition-timing-function"] = $el.css("transition-timing-function");

                console.log(TAG, "Saved transform: " + savedTransform["transform"]);

                $el.css(CSSHandler.getCurrentBrowserPrefix().transform, currentTransformValueOnPause);
                CSSHandler.removeBrowserSpecificProperty($el, "transition");
            }
        },
        handleCSSTransformResume: function () {
            if (savedTransform) {
                $el.css(CSSHandler.getCurrentBrowserPrefix().transition + "-property", savedTransform["transition-property"]);
                $el.css(CSSHandler.getCurrentBrowserPrefix().transition + "-timing-function", savedTransform["transition-timing-function"]);
                $el.css(CSSHandler.getCurrentBrowserPrefix().transition + "-duration", remainingTime + "ms");
                setTimeout(function () {
                    $el.css(CSSHandler.getCurrentBrowserPrefix().transform, privateMethods.getTransformMatrixForCSSTransform(savedTransform["transform"]));
                    //$el.css(CSSHandler.getCurrentBrowserPrefix().transform, "translate(256px,192px)");
                    model.set("transformStartTime", new Date().getTime());
                    console.log(TAG, "Now resume transform at " + new Date().toISOString());
                    savedTransform = null;
                }, 100);
            }
        }
    }
};