define(function () {
    return function (zoomOptions) {
        var that = this;

        this.scrollInstance = null;
        this.zoomDOMElement = null;

        this.initScroll = function (zoomDOMElement) {
            this.zoomDOMElement = zoomDOMElement;
            this.scrollInstance = new IScroll(zoomDOMElement, {
                scrollX: true,
                scrollY: true,
                directionLockThreshold: 1000, // set this to a big value so we have no direction lock !!!
                deceleration: 0.1, // not sure what this does
                momentum: true,
                bounce: false,
                bounceTime: 0,
                zoom: true,
                useOriginalVersion: false,
                zoomMin: zoomOptions.zoomMin,
                zoomMax: zoomOptions.zoomMax,
                doubleTapZoom: zoomOptions.zoomMax, // ??? No longer supported ?
                mouseWheel: true,
                wheelAction: 'zoom',
                preventDefault: false,
                translateOffset: zoomOptions.translateOffset,
                bindToWrapper: true,
                useTransition: true, // maybe change this to false ?
                HWCompositing: false // avoid using HWCompositing in order to get good quality in zoomed mode
            });
            this.scrollInstance.on("zoomStart", this.onZoomStart);
            this.scrollInstance.on("zoomEnd", this.onZoomEnd);
            this.scrollInstance.on("zoomWheel", this.onZoomWheel);
        };

        this.onZoomStart = function () {
            console.log("onZoomStart");
            ZoomUtils.updateZoomSlider(this.scale);
            this.options.preventDefault = true; // no page change allowed
        };

        this.onZoomWheel = function () {
            // on web when zooming with wheel this will be the event sent, not zoom start
            console.log("onZoomWheel");
            ZoomUtils.updateZoomSlider(this.scale);
            if (this.scale != zoomOptions.zoomMin) {
                this.options.preventDefault = true; // no page change allowed
            }
        };

        this.onZoomEnd = function () {
            // called also when zooming with mouse wheel, but not called for a manual zoom
            console.log("onZoomEnd");
            if (this.scale <= zoomOptions.zoomMin) { // back to non zoomed state, allow page change
                this.options.preventDefault = false;
            }
            ZoomUtils.updateZoomSlider(this.scale);
        };

        this.updateZoomWithIncrement = function (incrementValue) {
            var newScale = this.scrollInstance.scale + incrementValue;
            if (newScale > zoomOptions.zoomMax) {
                newScale = zoomOptions.zoomMax;
            }
            if (newScale < zoomOptions.zoomMin) {
                newScale = zoomOptions.zoomMin;
            }
            this.scrollInstance.zoom(newScale, undefined, undefined, 0, true);
        };

        this.doZoom = function (scale, x, y, time) {
            this.scrollInstance.zoom(parseFloat(scale), x, y, time);
        };

        this.resetZoom = function () {
            this.scrollInstance.options.preventDefault = false; // allow page change next time when visiting this page
            this.scrollInstance.zoom(zoomOptions.zoomMin, undefined, undefined, 0);
        };

        this.destroy = function () {
            console.log("Now destroy zoom for previous orientation");
            this.scrollInstance.destroy();
            $(this.zoomDOMElement).unbind(AppUtils.EventTypes.mouseDown); // make sure we remove double tap handler !!!
        };

        this.handleDoubleTapAction = function (x, y) {
            var newScaleFactor;
            if (this.scrollInstance.scale < zoomOptions.zoomMax) { // handle a zoom in
                newScaleFactor = zoomOptions.zoomMax;
                this.scrollInstance.options.preventDefault = true; // no page change allowed
            } else {    // handle a zoom out
                newScaleFactor = zoomOptions.zoomMin;
                this.scrollInstance.options.preventDefault = false; // back to non zoomed state, allow page change
            }
            this.scrollInstance.zoom(newScaleFactor, x, y, 100);
            ZoomUtils.updateZoomSlider(newScaleFactor);
        };

        this.initDoubleTapZoom = function () {
            var doubleTapFlag = false,
                that = this,
                touch;
            $(this.zoomDOMElement).unbind(AppUtils.EventTypes.mouseDown).bind(AppUtils.EventTypes.mouseDown, function (e) {
                if (doubleTapFlag) {
                    doubleTapFlag = false;
                    touch = AppUtils.getMouseEventCoordinates(e, event);
                    that.handleDoubleTapAction(touch.x, touch.y);
                } else {
                    doubleTapFlag = true;
                    setTimeout(function () {
                        doubleTapFlag = false;
                    }, 250);
                }
            });
        };

        return {
            initScroll: function (zoomDOMElement) {
                that.initScroll(zoomDOMElement);
            },
            initDoubleTapZoom: function ($el) {
                that.initDoubleTapZoom($el);
            },
            doZoom: function (scale, x, y, time) {
                that.doZoom(scale, x, y, time);
            },
            resetZoom: function () {
                that.resetZoom();
            },
            updateZoomWithIncrement: function (incrementValue) {
                that.updateZoomWithIncrement(incrementValue);
            },
            destroy: function () {
                that.destroy();
            }
        }
    }
});