define(function () {
    return function (rawTextBoxModel, pageModel) {
        var that = this,
            mappedBorderStyleCSSClasses = {
                'Solid': 'divSolidBorder',
                'No border': 'divNoBorder',
                'Dotted': 'divDottedBorder',
                'Dashed': 'divDashedBorder',
                'Double': 'divDoubleBorder',
                'Groove': 'divGrooveBorder',
                'Ridge': 'divRidgeBorder'
            },
            runWithIScroll = AppUtils.DeviceType.ANDROID && !AppUtils.DeviceType.XWalk;

        this.defineClasses = function () {

            that.TextBoxView = ElementBaseView.extend({
                tagName: "div",
                className: "sceneObject textBoxObject",
                elementsViews: [],
                id: rawTextBoxModel.properties.id,
                template: rawTextBoxModel.properties.fadeGradient == "1" ? templates.getTemplate("textBoxElementWithGradient") : templates.getTemplate("textBoxElement"),
                parentElementModel: pageModel,
                hiddenLeftSidePosition: -10000,
                hiddenTopSidePosition: -10000,
                isFocused: false,

                customInitialize: function () {
                    this.model.set("allObjectTypesInPage", rawTextBoxModel.customProperties["allObjectTypesInPage"]);
                    this.apiHandle = this.getSCORMAPI();
                },

                getSCORMAPI: function() {
//                    alert("getAPI called");
                    var theAPI = this.findAPI(window);

                    if ((theAPI == null) && (window.opener != null) && (typeof(window.opener) != "undefined"))
                        theAPI = this.findAPI(window.opener);

//                    if (theAPI == null)
//                        alert("Unable to find an API adapter");
//                    if (theAPI != null)
//                        alert("Sucsessfully found API adapter");
//                    alert("return theAPI");

                    return theAPI;
                },
                findAPI: function (win) {
                    var findAPITries = 0;

                    while ((win.API == null) && (win.parent != null) && (win.parent != win))    {
                        findAPITries++;
                        // Note: 7 is an arbitrary number, but should be more than sufficient
                        if (findAPITries > 7)
                        {
                            message("Error finding API -- too deeply nested.");
                            return null;
                        }
                        win = win.parent;
                    }

                    return win.API;

                },

                events: function () {
                    var _events = {};
                    _events[AppUtils.EventTypes.mouseDown] = "touchstart";
                    _events[AppUtils.EventTypes.mouseUp] = "touchend";
//                    if (this.model.get("scrollable") && !runWithIScroll) {
                        _events[AppUtils.EventTypes.mouseMove] = "mouseMoveHandler";
//                    }
                    _events[AppUtils.EventTypes.click] = "clickHandler";
                    return _events;
                },

                customTouchStartHandler: function (e) {
                    if(this.model.get("print") == true)
                        window.print();

                    this.isFocused = true;
                },

                touchend: function (e) {
                    this.isFocused = false;
                },

                mouseMoveHandler: function (event) {
                    event.stopPropagation();
                    console.log("Stopped propagation for mouse move on TextBox element");

                    if (this.isFocused)
                        event.preventDefault();
                },

                addCustomListeners: function () {
                    this.listenTo(this.model, "change:textContent", this.updateTextContent);
                    this.listenTo(this.model, "change:hiddenDueToTextSettings", this.hideShowBasedOnTexSettings);
                },
                updateTextContent: function () {
                    this.updateContent();
                    var content = this.model.get('textContent');
                    if (null != content) {
                        $(".textBoxContent", this.el).html(content);
                        this.$el.css("visibility", "visible");
                    }
                },
                removeView: function () {
                    if (this.model.get("removed")) {
                        this.remove();
                        this.unbind();
                    }
                },
                activateToggle: function () {
                    this.displayHideOnceOnClick();
                },
                hideShowBasedOnTexSettings: function () {
                    var hiddenDueToTextSettings = this.model.get('hiddenDueToTextSettings');
                    if (null != hiddenDueToTextSettings) {
                        if (this.model.get('hiddenDueToTextSettings')) {
                            this.setLeft(this.hiddenLeftSidePosition);
                            this.setTop(this.hiddenTopSidePosition);
                        } else {
                            this.setLeft(this.model.get('left'));
                            this.setTop(this.model.get('top'));
                        }
                        this.model.set('hiddenDueToTextSettings', null);
                    }
                },
                updateObjectPosition: function () {
                    var partOfAnActiveCategory = this.currentObjectIsPartOfAnActiveCategory(),
                        textVisibleSetting = Chapter.getTextVisible();
                    if (!textVisibleSetting || !partOfAnActiveCategory) {
                        this.model.set("hiddenDueToTextSettings", true);
                    } else {
                        this.model.set("hiddenDueToTextSettings", false);
                    }
                },
                setInitialVisibility: function () {
                    var opacity = this.model.get('displayed') ? this.model.get("opacity") : 0;
                    CSSHandler.setOpacity(this.$el, opacity);
                    if (this.isMoveToLinkTarget() || !(this.model.get('displayed') || ModelAccessFunctions.shouldNotHideBecauseOfEffects(this.model)) || this.isIAPPricePlaceHolder()) {
                        this.$el.css("visibility", "hidden");
                    }
                },
                isIAPPricePlaceHolder: function () {
                    var regex = "{PRICE_WITH_CURRENCY}";
                    return this.model.get('textContent').indexOf(regex) > -1;
                },
                setBorder: function () {
                    this.$el.addClass(mappedBorderStyleCSSClasses[this.model.get('borderStyle')]);
                },
                updateContent: function () {
                    if (!this.isIAPPricePlaceHolder()) {
                            var pages = Chapter.getPagesModelCollection(),
                                i,
                                j,
                                currentPageObjects,
                                textInputIds = [],
                                availableInputs = {};
							this.apiHandle.LMSInitialize("");
                            for(i = 0; i < pages.models.length; i++) {
                                currentPageObjects = pages.models[i].get('objects');

                                if(currentPageObjects != null){
                                    for(j = 0; j < currentPageObjects.length; j++) {
                                        if(currentPageObjects[j].properties.type == "textInput") {
                                            textInputIds.push(currentPageObjects[j].properties.textInputIdForSummary);


                                            if (this.apiHandle != null ){
                                                

                                                if(this.apiHandle.LMSGetValue("cmi.suspend_data") != null && this.apiHandle.LMSGetValue("cmi.suspend_data") != "") {
                                                    var jsonString = this.apiHandle.LMSGetValue("cmi.suspend_data");
                                                    var obj = JSON.parse(jsonString);
                                                    var k;

                                                    for(k = 0; k < obj.textInputElements.length; k++)
                                                        if(obj.textInputElements[k].id == currentPageObjects[j].properties.textInputIdForSummary)
                                                            availableInputs[currentPageObjects[j].properties.textInputIdForSummary] = obj.textInputElements[k].value
                                                }

                                                if(availableInputs[currentPageObjects[j].properties.textInputIdForSummary] == undefined)
                                                    availableInputs[currentPageObjects[j].properties.textInputIdForSummary] = currentPageObjects[j].properties.inputAsHtml;

                                                
                                            }
                                            else
                                                availableInputs[currentPageObjects[j].properties.textInputIdForSummary] = currentPageObjects[j].properties.inputAsHtml;
                                        }
                                    }
                                }
                            }
							this.apiHandle.LMSFinish("");
                            var content = rawTextBoxModel.customProperties.textContent;

                            for(i = 0; i < textInputIds.length; i++)
                                if(content.indexOf("[INPUT:"+textInputIds[i]+"]") != -1)
                                    content = content.replace("[INPUT:"+textInputIds[i]+"]", availableInputs[textInputIds[i]]);

                            this.model.set("textContent", content);
                            that.textBoxModel.set("textContent", content);
                    }
                },
                render: function () {
//                    if (this.shouldRenderObject()) {
//                        if (this.model.get("hiddenDueToTextSettings") && Chapter.getActivePage() == pageModel.get("index")) {
//                            this.model.set("displayed", true); // this should have been displayed but it was not because text was hidden
//                            CommonActions.show(this.$el, this.model.get("opacity"));
//                        }
                    this.updateObjectPosition();
                    this.renderStartHandler();
                    var $textBoxBackground, $textBoxContent;
                    // this is only needed when in combination with a YT video on same page when text won't be visible else
                    // taking this out now because with this on and missing on images will not show images on Android (XWalk)
                    // TODO: more testing here is required to figure out the best setup
//                        this.setBackfaceVisibility('hidden');
                    this.model.set("originalWidth", rawTextBoxModel.customProperties.originalWidth);
                    this.model.set("originalHeight", rawTextBoxModel.customProperties.originalHeight);
                    this.$el.html(this.template(this.model.attributes));
                    $textBoxBackground = $(".textBoxBackground", this.el);
                    $textBoxContent = $(".textBoxContent", this.el);
                    if (this.model.get("karaokeModeEnabled")) {
                        $textBoxContent.addClass("useInAudioSync");
                    }
                    if (this.model.get("scrollable")) {
                        if (AppUtils.DeviceType.ANDROID && !AppUtils.DeviceType.XWalk) {
                            require(['iScroll'], function () {
                                new IScroll($textBoxContent[0],
                                    {
                                        disableMouse: true,
                                        disablePointer: true,
                                        preventDefault: true,
                                        stopPropagation: true,
                                        bindToWrapper: true
                                    });
                            });
                        } else {
                            $textBoxContent.addClass("overflowScroll");
                        }
                    }
//                    if (!rawTextBoxModel.customProperties.userInteractionRequired) {
//                        this.$el.css("pointer-events", "none");
//                    }
                    CSSHandler.setBackgroundColor($textBoxBackground, this.model.get('bgColor'), this.model.get('bgOpacity'));
                    this.setWidth(this.model.get("width"));
                    this.setHeight(this.model.get("height"));
                    this.$el.css("z-index", this.model.get("ZIndex"));
                    CSSHandler.scale($textBoxContent, App.settings.scaleFactor);
                    this.setBorder();
                    this.setInitialVisibility();
//                    }
                    this.updateContent();
                    return this;
                },
                activateObject: function () {
                    if (this.model.get("activateObject")) {
                        this.model.set("objectActivated", true);
                        this.model.set("activateObject", false);
                    }
                }
            });
        };
        this.instantiateObjects = function () {
            this.textBoxModel = this.getTextBoxModel();
            this.textBoxView = new this.TextBoxView({
                model: that.textBoxModel
            });
        };

        this.getTextBoxModel = function () {
            var model = new Backbone.Model(rawTextBoxModel.properties);
            model.set("textContent", rawTextBoxModel.customProperties.textContent);
            model.set("gradientColor", rawTextBoxModel.properties.gradientColorRGB);
            model.set("value1", rawTextBoxModel.properties.gradientValue1);
            model.set("value2", rawTextBoxModel.properties.gradientValue2);
            return model;
        };

        return {
            init: function () {
                that.defineClasses();
                that.instantiateObjects();
            },
            getView: function () {
                return that.textBoxView;
            },
            getModel: function () {
                return that.textBoxModel;
            }

        }
    }
});